# The global sync table is copied to the user layer every time the main and sim threads are
# synchronized on the sim beat (which is like a tick but happens even when the game is paused)
Sync = {}

function ResetSyncTable()
    Sync = {
        # A list of camera control operations that we'd like the user layer to perform.
        CameraRequests = {},
        Sounds = {},

		# a movie to play in the UI MFD area
        PlayMFDMovie = nil,

		# the latest objectives table, see ui/game/objectives.lua for a description of the table format
        ObjectivesTable = nil,
        ObjectivesUpdateTable = nil,
        ObjectiveTimer = nil,
        MissionText = nil,
        VideoText = nil,
        HelpPrompt = nil,
        NISVideo = nil,

		# contains the current score for each army
        Score = {},

		# Table of army indices set to "victory" or "defeat".
		# It's the user layer's job to determine if any UI needs to be shown
		# for the focus army.
		GameResult = {},

        # Player to player queries that can affect the Sim
        PlayerQueries = {},
        QueryResults = {},

		UserUnitEnhancements = nil,

		# Contain operation data when op is complete
		OperationComplete = nil,
    }
end

SimUnitEnhancements = {}

function AddUnitEnhancement(unit, enhancement, slot)
    if not slot then return end
    local id = unit:GetEntityId()
    local unitEnh = SimUnitEnhancements[id]
    if unitEnh then
        SimUnitEnhancements[id][slot] = enhancement
    else
        SimUnitEnhancements[id] = {}
        SimUnitEnhancements[id][slot] = enhancement
    end
    SyncUnitEnhancements()
end

function RemoveUnitEnhancement(unit, enhancement)
    if not unit or unit:IsDead() then return end
    local id = unit:GetEntityId()
    local slots = SimUnitEnhancements[id]
    if not slots then return end
    local key = nil
    for k, v in slots do
        if v == enhancement then
            key = k
        end
    end
    if SimUnitEnhancements[id][key] then
        SimUnitEnhancements[id][key] = nil
    end
    if table.getsize(slots) == 0 then
        SimUnitEnhancements[id] = nil
    end
    SyncUnitEnhancements()
end

function RemoveAllUnitEnhancements(unit)
    local id = unit:GetEntityId()
    SimUnitEnhancements[id] = nil
    SyncUnitEnhancements()
end

function SyncUnitEnhancements()
    Sync.UserUnitEnhancements = SimUnitEnhancements
end

function DebugMoveCamera(x0,y0,x1,y1)
    local Camera = import('/lua/modules/SimCamera.lua').SimCamera
    local cam = Camera("WorldCamera")
#    cam:ScaleMoveVelocity(0.02)
    cam:MoveTo(Rect(x0,y0,x1,y1),5.0)
end

function SyncPlayableRect(x0,y0,x1,y1)
    #LOG('syncing playable rect change to user side')
    local Camera = import('/lua/modules/SimCamera.lua').SimCamera
    local cam = Camera("WorldCamera")
    cam:SyncPlayableRect(Rect(x0,y0,x1,y1))
end
