# The global sync table is copied from the sim layer every time the main and sim threads are
# synchronized on the sim beat (which is like a tick but happens even when the game is paused)
Sync = {}

# The PreviousSync table holds just what you'd expect it to, the sync table from the previous
# beat.
PreviousSync = {}

# Here's an opportunity for user side script to examine the Sync table for the new tick
function OnSync()
    import('/lua/modules/UserCamera.lua').ProcessCameraRequests(Sync.CameraRequests)

    #Play Sounds
    for k, v in Sync.Sounds do
        PlaySound(Sound{ Bank=v.Bank, Cue=v.Cue })
    end
    if Sync.RequestingExit then
        #LOG("Got it")
        ExitGame()
    end
    
    if Sync.CampaignMode then
        import('/lua/modules/ui/uiutil.lua').SetCampaignMode(Sync.CampaignMode)
    end
    
    if Sync.PlayMFDMovie then
        import('/lua/modules/ui/game/missiontext.lua').PlayMFDMovie(Sync.PlayMFDMovie, Sync.VideoText)
    end
    if Sync.UserUnitEnhancements then
        import('/lua/modules/ui/game/enhancements.lua').AddEnhancementEntry(Sync.UserUnitEnhancements)
    end

    if Sync.ObjectivesTable and next(Sync.ObjectivesTable) then
        import('/lua/modules/ui/game/objectives2.lua').AddObjectives(Sync.ObjectivesTable)
    end

    if Sync.ObjectivesUpdateTable and next(Sync.ObjectivesUpdateTable) then
        import('/lua/modules/ui/game/objectives2.lua').UpdateObjectivesTable(Sync.ObjectivesUpdateTable)
    end

    if Sync.ObjectiveTimer then
        if Sync.ObjectiveTimer != false then
            import('/lua/modules/ui/game/timer.lua').SetTimer(Sync.ObjectiveTimer)
        else
            import('/lua/modules/ui/game/timer.lua').ResetTimer()
        end
    end

    if Sync.EnhanceRestrict then
        import('/lua/modules/ui/game/enhancements.lua').RestrictList( Sync.EnhanceRestrict )
    end

    if Sync.NISVideo then
        import('/lua/modules/ui/game/missiontext.lua').PlayNIS(Sync.NISVideo)
    end

    if Sync.EndGameMovie then
        import('/lua/modules/ui/game/missiontext.lua').PlayEndGameMovie(Sync.EndGameMovie)
    end

    if Sync.ToggleGamePanels then
        ConExecute('UI_ToggleGamePanels')
    end

    if Sync.ToggleLifeBarsOff then
        ConExecute('UI_RenderUnitBars false')
    end

    if Sync.ToggleLifeBarsOn then
        ConExecute('UI_RenderUnitBars true')
    end

    if Sync.HelpPrompt then
        import('/lua/modules/ui/game/helptext.lua').AddHelpTextPrompt(Sync.HelpPrompt)
    end

    if Sync.MPTaunt then
        OnlineProvider.SendMessage("TAUNT:"..string.format("%02d", Sync.MPTaunt[2])..":"..Sync.MPTaunt[1])
        GameSendChatMessage("TAUNT:"..string.format("%02d", Sync.MPTaunt[2])..":"..Sync.MPTaunt[1])
    end

    if Sync.UserConRequests then
        for num, execRequest in Sync.UserConRequests do
            ConExecute(execRequest)
        end
    end

    if table.getn(Sync.Score) > 0 then
        import('/lua/modules/ui/game/score.lua').currentScores = Sync.Score
        import('/lua/modules/ui/game/scoreaccum.lua').UpdateScoreData(Sync.Score)
    end

	for k,gameResult in Sync.GameResult do
		local armyIndex, result = unpack(gameResult)
		LOG('army ',armyIndex,' ends in ', result)
		if (StdOutMessage) then
		    StdOutMessage('/GAMERESULT '..armyIndex..' '..result)
		end
		LOG('focus == ', GetFocusArmy())
		if armyIndex == GetFocusArmy() then
			if result == "victory" then
				import('/lua/modules/ui/game/gameresult.lua').ShowVictoryDialog()
			elseif result == "defeat" then
				import('/lua/modules/ui/game/gameresult.lua').ShowDefeatDialog()
			else
				WARN('Unknown game result received: ', result)
			end
		else
			LOG('not focus')
		end
	end

    if Sync.PausedBy then
        if not PreviousSync.PausedBy then
            import('/lua/modules/ui/game/gamemain.lua').OnPause(Sync.PausedBy, Sync.TimeoutsRemaining)
        end
    else
        if PreviousSync.PausedBy then
            import('/lua/modules/ui/game/gamemain.lua').OnResume()
        end
    end

    if Sync.Paused != PreviousSync.Paused then
        import("/lua/modules/ui/game/gamemain.lua").OnPause(Sync.Paused);
    end

    if Sync.PlayerQueries then
        import('/lua/UserPlayerQuery.lua').ProcessQueries(Sync.PlayerQueries)
    end

    if Sync.QueryResults then
        import('/lua/UserPlayerQuery.lua').ProcessQueryResults(Sync.QueryResults)
    end

    if Sync.OperationComplete then
        import('/lua/modules/ui/campaign/campaignmanager.lua').OperationVictory(Sync.OperationComplete)
    end

    if Sync.LockTeams then
        import('/lua/modules/ui/game/diplomacy.lua').LockTeams()
    end

    if Sync.Cheaters then
        # Ted, this is where you would hook in better cheater reporting.
        local names = ''
        local isare = 'is'
        local srcs = SessionGetCommandSourceNames()
        for k,v in ipairs(Sync.Cheaters) do
            if names != '' then
                names = names .. ', '
                isare = 'are'
            end
            names = names .. (srcs[v] or '???')
        end
        local msg = names .. ' ' .. isare
        if Sync.Cheaters.CheatsEnabled then
            msg = msg .. ' cheating!'
        else
            msg = msg .. ' trying to cheat!'
        end
        print(msg)
    end
end
