LAND = 0x01
SEABED = 0x02
SUB = 0x04
WATER = 0x08
AIR = 0x10
ORBIT = 0x20

None = 0x00
IgnoreStructures = 0x01

#
# Each table must contain 7 or fewer entries, strictly ordered by decreasing mobility. (I.e. later
# entries in a table must have the same or larger footprint as earlier entries, the same or a subset
# of caps and flags, and the same or smaller MaxSlope.)
#
# Large footprint sizes are *extremely* expensive, so only add entries up to the sizes we actually
# need for units. Additional tables are also expensive, so try to keep the number down to a
# minimum.
#
# The script "data/lua/tests/dump_footprints.lua" can be used to figure out what footprint shapes
# the blueprints are currently expecting.
#
SpecFootprintGroup {
   DisplayName = 'Vehicle Group',
    {  Name = 'Vehicle1x1',   SizeX=1,  SizeZ=1,  Caps=LAND, MaxWaterDepth=0.05, MaxSlope=0.75, Flags=None },
    {  Name = 'Vehicle2x2',   SizeX=2,  SizeZ=2,  Caps=LAND, MaxWaterDepth=0.05, MaxSlope=0.75, Flags=None },
}

SpecFootprintGroup {
   DisplayName = 'Amphibious Group',
    {  Name = 'Amphibious1x1',   SizeX=1,  SizeZ=1,  Caps=LAND|SEABED, MaxWaterDepth=15, MaxSlope=0.75, Flags=None },
    {  Name = 'Amphibious3x3',   SizeX=3,  SizeZ=3,  Caps=LAND|SEABED, MaxWaterDepth=15, MaxSlope=0.75, Flags=IgnoreStructures },
    {  Name = 'Amphibious6x6',   SizeX=6,  SizeZ=6,  Caps=LAND|SEABED, MaxWaterDepth=15, MaxSlope=0.75, Flags=IgnoreStructures },
}

SpecFootprintGroup {
   DisplayName = 'Water Land Group',
    {  Name = 'WaterLand1x1',   SizeX=1,  SizeZ=1,  Caps=LAND|WATER, MaxWaterDepth=5, MaxSlope=0.75, Flags=None },
    {  Name = 'WaterLand2x2',   SizeX=2,  SizeZ=2,  Caps=LAND|WATER, MaxWaterDepth=5, MaxSlope=0.75, Flags=None },
    {  Name = 'WaterLand3x3',   SizeX=3,  SizeZ=3,  Caps=LAND|WATER, MaxWaterDepth=5, MaxSlope=0.75, Flags=IgnoreStructures },
    {  Name = 'WaterLand5x5',   SizeX=5,  SizeZ=5,  Caps=LAND|WATER, MaxWaterDepth=5, MaxSlope=0.75, Flags=IgnoreStructures },
}

SpecFootprintGroup {
   DisplayName = 'GiantHexaped Group',
    {  Name = 'GiantHexaped6x6',   SizeX=6,  SizeZ=6,  Caps=LAND|SEABED, MaxWaterDepth=15, MaxSlope=0.75, Flags=IgnoreStructures },
}

SpecFootprintGroup {
   DisplayName = 'SurfacingSub Group',
    {  Name = 'SurfacingSub2x2',   SizeX=2,  SizeZ=2,  Caps=SUB|WATER, MinWaterDepth=1.5, Flags=None },
    {  Name = 'SurfacingSub3x3',   SizeX=3,  SizeZ=3,  Caps=SUB|WATER, MinWaterDepth=1.5, Flags=None },
    {  Name = 'SurfacingSub4x4',   SizeX=4,  SizeZ=4,  Caps=SUB|WATER, MinWaterDepth=1.5, Flags=None },
    {  Name = 'SurfacingSub12x12', SizeX=12, SizeZ=12, Caps=SUB|WATER, MinWaterDepth=1.5, Flags=None },
}

SpecFootprintGroup {
   DisplayName = 'Water Group',
    {  Name = 'Water1x1',   SizeX=1,  SizeZ=1,  Caps=WATER, MinWaterDepth=1.5, Flags=None },
    {  Name = 'Water3x3',   SizeX=3,  SizeZ=3,  Caps=WATER, MinWaterDepth=0.25, Flags=None },
    {  Name = 'Water4x4',   SizeX=4,  SizeZ=4,  Caps=WATER, MinWaterDepth=1.5, Flags=None },
    {  Name = 'Water6x6',   SizeX=6,  SizeZ=6,  Caps=WATER, MinWaterDepth=1.5, Flags=None },
    {  Name = 'Water8x8',   SizeX=8,  SizeZ=8,  Caps=WATER, MinWaterDepth=1.5, Flags=None },
    {  Name = 'Water11x11', SizeX=11, SizeZ=11, Caps=WATER, MinWaterDepth=1.5, Flags=None },
}
