#****************************************************************************
#**
#**  File     :  /cdimage/lua/modules/platoon.lua
#**  Author(s):
#**
#**  Summary  :
#**
#**  Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************
#########################################
# Platoon Lua Module                    #
#########################################
local AIUtils = import('/lua/modules/ai/aiutilities.lua')
local Utilities = import('/lua/modules/utilities.lua')
local AIBuildStructures = import('/lua/modules/ai/aibuildstructures.lua')
local UnitUpgradeTemplates = import('/lua/modules/upgradetemplates.lua').UnitUpgradeTemplates
local StructureUpgradeTemplates = import('/lua/modules/upgradetemplates.lua').StructureUpgradeTemplates
local ScenarioFramework = import('/lua/modules/ScenarioFramework.lua')
local AIAttackUtils = import('/lua/modules/ai/aiattackutilities.lua')



# ===== CDR ADD BEHAVIORS ===== #
function CDROverchargeBehavior(platoon)
    local cdr = platoon:GetPlatoonUnits()[1]
    if cdr.OverchargeThread then
        return
    else
        cdr.OverchargeThread = cdr:ForkThread( CDROverChargeThread )
    end
end

function CDROverChargeThread( cdr )
    local aiBrain = cdr:GetAIBrain()
    local weapBPs = cdr:GetBlueprint().Weapon
    local weapon
    for k,v in weapBPs do
        if v.Label == 'OverCharge' then
            weapon = v
            break
        end
    end
    local distressRange = 40
    local beingBuilt = false
    local plat
    local startX, startZ = aiBrain:GetArmyStartPos()
    cdr.UnitBeingBuiltBehavior = false
    while not cdr:IsDead() do
        if not cdr.Running and not cdr.GivingUp and not cdr.Leashing then
            local cdrPos = cdr:GetPosition()
            local numUnits = aiBrain:GetNumUnitsAroundPoint( 'Enemy', categories.LAND - categories.SCOUT, cdrPos, ( weapon.MaxRadius * 3 ) )
            local distressLoc = aiBrain:BaseMonitorDistressLocation(cdrPos)
            local overCharging = false
            if numUnits > 0 or ( not cdr.DistressCall and distressLoc and Utilities.XZDistanceTwoVectors( distressLoc, cdrPos ) < distressRange ) then
                cdr.Fighting = true
                CDRRevertPriorityChange( cdr )
                if cdr:GetUnitBeingBuilt() then
                    #LOG('*AI DEBUG: ARMY ' .. aiBrain:GetArmyIndex() .. ': CDR was building something')
                    cdr.UnitBeingBuiltBehavior = cdr:GetUnitBeingBuilt()
                end
                plat = aiBrain:MakePlatoon( '', '' )
                aiBrain:AssignUnitsToPlatoon( plat, {cdr}, 'support', 'None' )
                plat:Stop()
                local priList = { categories.COMMAND, categories.EXPERIMENTAL, categories.TECH3 * categories.INDIRECTFIRE, 
                    categories.TECH3 * categories.MOBILE, categories.TECH2 * categories.INDIRECTFIRE, categories.MOBILE * categories.TECH2, 
                    categories.TECH1 * categories.INDIRECTFIRE, categories.TECH1 * categories.MOBILE, categories.ALLUNITS }
                #LOG('*AI DEBUG: ARMY ' , repr(aiBrain:GetArmyIndex()),  ': CDR AI ACTIVATE - Commander go fight stuff! -- ' .. numUnits)
                local target
                local continueFighting = true
                local counter = 0
                repeat
                    overCharging = false
                    if counter >= 5 or not target or target:IsDead() or Utilities.GetDistanceBetweenTwoEntities(cdr, target) > ( weapon.MaxRadius * 3 ) then
                        for k,v in priList do
                            target = plat:FindClosestUnit( 'Support', 'Enemy', true, v )
                            if target and Utilities.GetDistanceBetweenTwoEntities(cdr, target) < (weapon.MaxRadius * 3 ) then
                                break
                            end
                            target = false
                        end
                        if target then
                            if aiBrain:GetEconomyStored('ENERGY') >= weapon.EnergyRequired and target and not target:IsDead() then
                                overCharging = true
                                IssueClearCommands({cdr})
                                IssueOverCharge( {cdr}, target )
                            elseif not target:IsDead() then
                                local tarPos = target:GetPosition()
                                IssueClearCommands( {cdr} )
                                IssueMove( {cdr}, tarPos )
                                IssueMove( {cdr}, {startX, 0, startZ} )
                            end
                        else
                            if distressLoc and ( Utilities.XZDistanceTwoVectors( distressLoc, cdrPos ) < distressRange ) then
                                #LOG('*AI DEBUG: ARMY ' .. aiBrain:GetArmyIndex() .. ': CDR Responding to distress call')
                                IssueClearCommands( {cdr} )
                                IssueMove( {cdr}, distressLoc )
                                IssueMove( {cdr}, {startX, 0, startZ} )
                            end
                        end
                    end
                    if overCharging then
                        while target and not target:IsDead() and not cdr:IsDead() do
                            WaitSeconds(.5)
                        counter = counter + .5
                        end
                    else
                        WaitSeconds(5)
                        counter = counter + 5
                    end
                    cdrPos = cdr:GetPosition()
                    distressLoc = aiBrain:BaseMonitorDistressLocation(cdrPos)
                    if cdr:IsDead() then
                        return
                    end
                    if ( aiBrain:GetNumUnitsAroundPoint( 'Enemy', categories.LAND - categories.SCOUT, cdrPos, ( weapon.MaxRadius * 3 ) ) <= 1 ) 
                        and ( not distressLoc or ( Utilities.XZDistanceTwoVectors( distressLoc, cdrPos ) < distressRange ) ) then
                        continueFighting = false
                    end
                until not continueFighting or not aiBrain:PlatoonExists(plat)
                if not cdr:IsDead() then
                    cdr.Fighting = false
                    if overCharging then
                        IssueMove( {cdr}, {startX, 0, startZ} )
                    end
                end
                #LOG('*AI DEBUG: ARMY ', repr(aiBrain:GetArmyIndex()),  ': CDR AI DEACTIVATE - Nothing to see here!')
                if aiBrain:PlatoonExists(plat) then
                    cdr:ForkThread( CDRRepairBuildingUnit, plat )
                end
            end
        end
        WaitSeconds(3)
    end
end

function CDRRepairBuildingUnit( cdr, plat )
    local aiBrain = cdr:GetAIBrain()
    if cdr.UnitBeingBuiltBehavior and not cdr.UnitBeingBuiltBehavior:BeenDestroyed() then
        IssueClearCommands( {cdr} )
        IssueRepair( {cdr}, cdr.UnitBeingBuiltBehavior )
        repeat
            WaitSeconds(1)
            if cdr.Fighting or cdr.Running or cdr.GivingUp or cdr.Leashing or cdr.Cornered then
                return
            end
        until cdr:IsIdleState()
        cdr.UnitBeingBuiltBehavior = false
    end
    if not cdr:IsDead() and not ( cdr.Fighting or cdr.Running or cdr.GivingUp or cdr.Leashing or cdr.Cornered ) then
        local pool = aiBrain:GetPlatoonUniquelyNamed('ArmyPool')
        aiBrain:AssignUnitsToPlatoon( pool, {cdr}, 'Unassigned', 'None' )
    else
        local pool = aiBrain:GetPlatoonUniquelyNamed('ArmyPool')
        aiBrain:AssignUnitsToPlatoon( pool, {cdr}, 'Unassigned', 'None' )
        if cdr.Fighting then
            error( '*AI ERROR: Commander in some state its not supposed to be in at this point - Fighting' )
        elseif cdr.Running then
            error( '*AI ERROR: Commander in some state its not supposed to be in at this point - Running' )
        elseif cdr.GivingUp then
            error( '*AI ERROR: Commander in some state its not supposed to be in at this point - GivingUp' )
        elseif cdr.Leashing then
            error( '*AI ERROR: Commander in some state its not supposed to be in at this point - Leashing' )
        elseif cdr.Cornered then
            error( '*AI ERROR: Commander in some state its not supposed to be in at this point - Cornered' )
        end
    end
end

function CDRRevertPriorityChange( cdr )
    local aiBrain = cdr:GetAIBrain()
    if cdr.PreviousPriority and cdr.Platoon and aiBrain:PlatoonExists(cdr.Platoon) then
        #LOG('*AI DEBUG: ARMY ' .. aiBrain:GetArmyIndex() .. ': Reverting priority of builder')
        aiBrain:PBMSetPriority( cdr.Platoon, cdr.PreviousPriority )
    end
end


function CDRCallForHelp(platoon)
    local cdr = platoon:GetPlatoonUnits()[1]
    if cdr.CDRCallForHelpThread then
        return
    else
        cdr.CDRCallForHelpThread = cdr:ForkThread( CDRCallForHelpThread )
    end
end

function CDRCallForHelpThread( cdr )
    local aiBrain = cdr:GetAIBrain()
    while not cdr:IsDead() do
        if not cdr.DistressCall then
            if cdr:GetHealthPercent() < .6 and ( aiBrain:GetNumUnitsAroundPoint( 'Enemy', categories.MOBILE, cdr:GetPosition(), 32 ) > 0 ) then
                cdr.DistressCall = true
                aiBrain.BaseMonitor.CDRDistress = true
                #LOG('*AI DEBUG: ARMY ' .. aiBrain:GetArmyIndex() .. ': CDR AI ACTIVATE --- Calling for help ---')
                # While not dead check health of CDR and count nearby enemies
                repeat
                    WaitSeconds(3)
                    if cdr:IsDead() then
                        return
                    end
                until cdr:GetHealthPercent() >= .6 or ( aiBrain:GetNumUnitsAroundPoint( 'Enemy', categories.MOBILE, cdr:GetPosition(), 32 ) == 0 )
                #LOG('*AI DEBUG: ARMY ' .. aiBrain:GetArmyIndex() .. ': CDR AI DEACTIVATE --- Disable distress call ---')
                cdr.DistressCall = false
                aiBrain.BaseMonitor.CDRDistress = false
            end 
        end
        WaitSeconds(5)
    end
end


function CDRLeash(platoon)
    local cdr = platoon:GetPlatoonUnits()[1]
    if cdr.LeashThread then
        return
    else
        cdr.LeashThread = cdr:ForkThread( CDRLeashThread )
    end
end

function CDRLeashThread(cdr)
    local aiBrain = cdr:GetAIBrain()
    local loc, rad
    for k,v in aiBrain.PBM.Locations do
        if v.LocationType == 'MAIN' then
            loc = v.Location
            rad = v.Radius
            break
        end
    end
    while not cdr:IsDead() do
        if not cdr.GivingUp and not cdr.Running then
            local cdrPos = cdr:GetPosition()
            if VDist2( cdrPos[1], cdrPos[3], loc[1], loc[3] ) > 65 then
                CDRRevertPriorityChange( cdr )
                local plat = aiBrain:MakePlatoon( '', '' )
                aiBrain:AssignUnitsToPlatoon( plat, {cdr}, 'support', 'none' )
                plat:Stop()
                plat:MoveToLocation( loc, false )
                cdr.Leashing = true
                #LOG('*AI DEBUG: ARMY ' .. aiBrain:GetArmyIndex() .. ': CDR AI ACTIVATE - Commander leashing to MAIN' )
                WaitSeconds( 15 )
                if not cdr:IsDead() then
                    cdr.Leashing = false
                    if aiBrain:PlatoonExists(plat) then
                        cdr:ForkThread( CDRRepairBuildingUnit, plat )
                    end
                end
            end
        end
        WaitSeconds(5)
    end
end

function CDRGiveUpBehavior(platoon)
    local cdr = platoon:GetPlatoonUnits()[1]
    if cdr.GiveUpThread then
        return
    else
        cdr.GiveUpThread = cdr:ForkThread( CDRGiveUpThread )
    end
end

function CDRGiveUpThread(cdr)
    local aiBrain = cdr:GetAIBrain()
    local maxNum
    local plat
    local startPos = { aiBrain:PBMGetLocationCoords('MAIN') }
    while not cdr:IsDead() do
        local numUnits = aiBrain:GetCurrentUnits( categories.FACTORY )
        if not maxNum or numUnits > maxNum then
            maxNum = numUnits
        end
        if numUnits < ( maxNum / 5 ) or cdr.GivingUp or ( cdr.Cornered and numUnits < ( maxNum /3 ) ) then
            if numUnis < ( maxNum / 5 ) then
                cdr.TotalLoss = true
            end
            #LOG( '*AI DEBUG: ARMY ' .. aiBrain:GetArmyIndex() .. ': CDR AI ACTIVATE - COMMANDER GIVING UP  '.. aiBrain.Name )
            CDRRevertPriorityChange( cdr )
            local plat = aiBrain:MakePlatoon( '', '' )
            aiBrain:AssignUnitsToPlatoon( plat, {cdr}, 'Support', 'None' )
            cdr.GivingUp = true
            local canTeleport = cdr:HasEnhancement( 'Teleporter' )
            local closeEnemyCDR = AIAttackUtils.GetClosestUnitToPosition( aiBrain, categories.COMMAND, cdr:GetPosition(), 1000, 'Enemy' )
            if closeEnemyCDR then
                #LOG( '*AI DEBUG: ARMY ' .. aiBrain:GetArmyIndex() .. ': CDR AI ACTIVATE - CDR Moving to enemy commander' )
                if canTeleport then
                    IssueTeleport( {cdr}, closeEnemyCDR:GetPosition() )
                else
                    IssueMove( {cdr}, closeEnemyCDR:GetPosition() )
                end
            else
                #LOG( '*AI DEBUG: ARMY ' .. aiBrain:GetArmyIndex() .. ': CDR AI ACTIVATE - CDR Moving to highest threat' )
                if canTeleport then
                    IssueTeleport( {cdr}, ( aiBrain:GetHighestThreatPosition(0, false) ) )
                else
                    IssueMove( {cdr}, ( aiBrain:GetHighestThreatPosition(0, false) ) )
                end
            end
            while not cdr:IsDead() and ( cdr.TotalLoss or cdr:GetHealthPercent() < .4 or aiBrain:GetNumUnitsAroundPoint( 'Enemy', categories.LAND, cdr:GetPosition(), 25) > 5 ) do
                local nearUnits = aiBrain:GetUnitsAroundPoint( 'Enemy', categories.COMMAND, cdr:GetPosition(), 25 )
                if table.getn(nearUnits) > 0 then
                    IssueOverCharge( {cdr}, nearUnits[1] )
                end
                WaitSeconds(3)
            end
            if not cdr:IsDead() then
                cdr.GivingUp = false
                cdr.Cornered = false
            end
            if aiBrain:PlatoonExists(plat) then
                cdr:ForkThread( CDRRepairBuildingUnit, plat )
            end
            #LOG('*AI DEBUG: ARMY ' .. aiBrain:GetArmyIndex() .. ': CDR AI DEACTIVATE - No longer giving up, was overreacting slightly')
        end
        WaitSeconds(3)
    end
end

function CDRRunAwayBehavior(platoon)
    local cdr = platoon:GetPlatoonUnits()[1]
    if cdr.RunAwayThread then
        return
    else
        cdr.RunAwayThread = cdr:ForkThread( CDRRunAwayThread )
    end
end

function CDRRunAwayThread( cdr )
    local aiBrain = cdr:GetAIBrain()
    AIUtils.AIFindDefensiveArea( aiBrain, cdr, categories.DEFENSE * categories.ANTIAIR, 10000 )
    while not cdr:IsDead() do
        if cdr:GetHealthPercent() < .4 and not cdr.GivingUp then
            #LOG('*AI DEBUG: ARMY ' .. aiBrain:GetArmyIndex() .. ': CDR AI ACTIVATE - CDR RUNNING AWAY' )
            local cdrPos = cdr:GetPosition()
            local nmeAir = aiBrain:GetUnitsAroundPoint( 'Enemy', categories.AIR, cdrPos, 25 )
            local nmeLand = aiBrain:GetUnitsAroundPoint( 'Enemy', categories.LAND, cdrPos, 25 )
            local nmeHardcore = aiBrain:GetUnitsAroundPoint( 'Enemy', categories.EXPERIMENTAL, cdrPos, 25 )
            if nmeAir > 3 or nmeLand > 3 or nmeHardcore > 0 then
                if cdr:IsUnitState('Building') then
                    cdr.UnitBeingBuiltBehavior = cdr:GetUnitBeingBuilt()
                end
                local canTeleport = cdr:HasEnhancement( 'Teleporter' )
                cdr.Running = true
                CDRRevertPriorityChange( cdr )
                local plat = aiBrain:MakePlatoon( '', '' )
                aiBrain:AssignUnitsToPlatoon( plat, {cdr}, 'support', 'None' )
                local category
                if nmeAir > 3 then
                    category = categories.DEFENSE * categories.ANTIAIR
                else
                    category = categories.DEFENSE * categories.DIRECTFIRE
                end
                local runSpot, prevSpot
                repeat
                    if canTeleport then
                        runSpot = AIUtils.AIFindDefensiveArea( aiBrain, cdr, category, 10000 )
                    else
                        runSpot = AIUtils.AIFindDefensiveArea( aiBrain, cdr, category, 50 )
                    end
                    if not prevSpot or runSpot[1] ~= prevSpot[1] or runSpot[3] ~= prevSpot[3] then
                        plat:Stop()
                        if VDist2( cdrPos[1], cdrPos[3], runSpot[1], runSpot[3] ) < 10 then
                            # Commander already at the safest spot
                            cdr.Cornered = true
                        else
                            cdr.Cornered = false
                            if canTeleport then
                                IssueTeleport( {cdr}, runSpot )
                            else
                                cmd = plat:MoveToLocation( runSpot, false )
                            end
                        end
                    end
                    WaitSeconds(3)
                    if not cdr:IsDead() then
                        cdrPos = cdr:GetPosition()
                        nmeAir = aiBrain:GetUnitsAroundPoint( 'Enemy', categories.AIR, cdrPos, 25 )
                        nmeLand = aiBrain:GetUnitsAroundPoint( 'Enemy', categories.LAND, cdrPos, 25 )
                        nmeHardcore = aiBrain:GetUnitsAroundPoint( 'Enemy', categories.EXPERIMENTAL, cdrPos, 25 )
                    end
                until cdr:IsDead() or cdr.GivingUp or ( nmeAir < 2 and nmeLand < 2 and nmeHardcore == 0 ) or cdr:GetHealthPercent() > .4
                #LOG('*AI DEBUG: ARMY ' .. aiBrain:GetArmyIndex() .. ': CDR AI DEACTIVATE - Run away no more!')
                cdr.Running = false
                cdr.Cornered = false
                if not cdr:IsDead() and not cdr.GivingUp and aiBrain:PlatoonExists(plat) then
                    cdr:ForkThread( CDRRepairBuildingUnit, plat )
                end
            end
        end
        WaitSeconds(3)
    end
end




# ==== Generic Unit Behaviors === #
function AirUnitRefit(self)
    for k,v in self:GetPlatoonUnits() do
        if not v:IsDead() and not v.RefitThread then
            v.RefitThreat = v:ForkThread( AirUnitRefitThread, self:GetPlan(), self.PlatoonData )
        end
    end
end

function AirUnitRefitThread(unit, plan, data)
    unit.PlanName = plan
    if data then
        unit.PlatoonData = data
    end
    local aiBrain = unit:GetAIBrain()
    while not unit:IsDead() do
        local fuel = unit:GetFuelRatio()
        local health = unit:GetHealthPercent()
        if not unit.Loading and ( fuel < .2 or health < .4 ) then
            # Find air stage
            if aiBrain:GetCurrentUnits( categories.AIRSTAGINGPLATFORM ) > 0 then
                local unitPos = unit:GetPosition()
                local plats = AIUtils.GetOwnUnitsAroundPoint( aiBrain, categories.AIRSTAGINGPLATFORM, unitPos, 400 )
                if table.getn( plats ) > 0 then
                    local closest, distance
                    for k,v in plats do
                        if not v:IsDead() then
                            local roomAvailable = false
                            if EntityCategoryContains( categories.CARRIER, v ) then
                                #roomAvailable = v:TransportHasAvailableStorage( unit )
                            else
                                roomAvailable = v:TransportHasSpaceFor( unit )
                            end
                            if roomAvailable then
                                local platPos = v:GetPosition()
                                local tempDist = VDist2( unitPos[1], unitPos[3], platPos[1], platPos[3] )
                                if ( not closest or tempDist < distance ) then
                                    closest = v
                                    distance = tempDist
                                end
                            end
                        end
                    end
                    if closest then
                        local plat = aiBrain:MakePlatoon( '', '' )
                        aiBrain:AssignUnitsToPlatoon( plat, {unit}, 'Attack', 'None' )
                        IssueStop( {unit} )
                        IssueClearCommands( {unit} )
                        IssueTransportLoad( {unit}, closest )
                        if EntityCategoryContains( categories.AIRSTAGINGPLATFORM, closest ) and not closest.AirStaging then
                            closest.AirStaging = closest:ForkThread( AirStagingThread )
                            closest.Refueling = {}
                        elseif EntityCategoryContains( categories.CARRIER, closest) and not closest.CarrierStaging then
                            closest.CarrierStaging = closest:ForkThread( CarrierStagingThread )
                            closest.Refueling = {}
                        end
                        table.insert( closest.Refueling, unit )
                        unit.Loading = true
                    end
                end
            end
        end
        WaitSeconds(1)
    end
end

function AirStagingThread(unit)
    local aiBrain = unit:GetAIBrain()
    while not unit:IsDead() do
        local ready = true
        local numUnits = 0
        for k,v in unit.Refueling do
            if not v:IsDead() and ( v:GetFuelRatio() < .9 or v:GetHealthPercent() < .9 ) then
                ready = false
            elseif not v:IsDead() then
                numUnits = numUnits + 1
            end
        end
        if ready and numUnits > 0 then
            #LOG('*AI DEBUG: Release the doves')
            local pos = unit:GetPosition()
            IssueClearCommands( {unit} )
            IssueTransportUnload( {unit}, { pos[1] + 5, pos[2], pos[3] + 5 } )
            WaitSeconds(2)
            for k,v in unit.Refueling do
                if not v:IsDead() then
                    v.Loading = false
                    local plat
                    if not v.PlanName then
                        plat = aiBrain:MakePlatoon( '', 'HuntAI' )
                    else    
                        plat = aiBrain:MakePlatoon( '', v.PlanName )
                    end
                    if v.PlatoonData then
                        plat.PlatoonData = {}
                        plat.PlatoonData = v.PlatoonData
                    end
                    aiBrain:AssignUnitsToPlatoon( plat, {v}, 'Attack', 'GrowthFormation' )
                end
            end
        end
        WaitSeconds( 10 )
    end
end

function AirLandToggle(platoon)
    for k,v in platoon:GetPlatoonUnits() do
        if not v:IsDead() and not v.AirLandToggleThread then
            v.AirLandToggleThread = v:ForkThread( AirLandToggleThread )
        end
    end
end

function AirLandToggleThread(unit)
    local aiBrain = unit:GetAIBrain()
    local bp = unit:GetBlueprint()
    local weapons = bp.Weapon
    local antiAirRange
    local landRange
    local toggleWeapons = {}
    local unitCat = ParseEntityCategory( unit:GetUnitId() )
    for k,v in weapons do
        if v.ToggleWeapon then
            local weaponType = 'Land'
            for n,wType in v.FireTargetLayerCapsTable do
                if string.find( wType, 'Air' ) then
                    weaponType = 'Air'
                    break
                end
            end
            if weaponType == 'Land' then
                landRange = v.MaxRadius
            else
                antiAirRange = v.MaxRadius
            end
        end
    end
    if not landRange or not antiAirRange then
        return
    end
    while not unit:IsDead() and unit:IsUnitState('Busy') do
        WaitSeconds(2)
    end
    local toggleState = 'Air'
    while not unit:IsDead() do
        local position = unit:GetPosition()
        local numAir = aiBrain:GetNumUnitsAroundPoint( 'Enemy', ( categories.MOBILE * categories.AIR ) - unitCat , position, antiAirRange )
        local numGround = aiBrain:GetNumUnitsAroundPoint( 'Enemy', ( categories.LAND + categories.NAVAL + categories.STRUCTURE ) - unitCat, position, landRange )
        local frndAir = aiBrain:GetNumUnitsAroundPoint( 'Ally', ( categories.MOBILE * categories.AIR ) - unitCat, position, antiAirRange )
        local frndGround = aiBrain:GetNumUnitsAroundPoint( 'Ally', ( categories.LAND + categories.NAVAL + categories.STRUCTURE ) - unitCat, position, landRange )
        if numAir > 5 and frndAir < 3 then
            if toggleState ~= 'Air' then
                # Make sure its air
                toggleState = 'Air'
                unit:ToggleScriptBit('RULEUCC_WeaponToggle')
            end
        elseif numGround > ( numAir * 1.5 ) then
            if toggleState ~= 'Land' then
                # Switch to ground
                toggleState = 'Land'
                unit:ToggleScriptBit('RULEUCC_WeaponToggle')
            end
        elseif frndAir > frndGround then
            if toggleState ~= 'Land' then
                # Switch to ground
                toggleState = 'Land'
                unit:ToggleScriptBit('RULEUCC_WeaponToggle')
            end
        else
            if toggleState ~= 'Air' then
                # Make sure its air
                toggleState = 'Air'
                unit:ToggleScriptBit('RULEUCC_WeaponToggle')
            end
        end
        WaitSeconds(10)
    end
end






# ==== Indivitual Unit Behaviors ==== #
function FatBoyBehavior(self)
    local aiBrain = self:GetBrain()
    local unit
    for k,v in self:GetPlatoonUnits() do
        if not v:IsDead() then
            unit = v
            break
        end
    end
    if not unit then
        return
    end
    if not EntityCategoryContains( categories.uel0401, unit ) then
        return
    end
    local otherPlat
    unit.BuiltUnitCount = 0
    while aiBrain:PlatoonExists(self) do
        if unit:GetCurrentLayer() == 'Land' then
            local position = unit:GetPosition()
            local numStrucs = aiBrain:GetNumUnitsAroundPoint( 'Enemy', categories.STRUCTURE - ( categories.MASSEXTRACTION + categories.WALL ), position, 50 )
            local numLand = aiBrain:GetNumUnitsAroundPoint( 'Enemy', ( categories.MOBILE * categories.LAND ) * ( categories.EXPERIMENTAL + categories.TECH3 ), position, 50 )
            while unit.BuiltUnitCount < 15 and ( numStrucs > 5 or numLand > 10 ) do
                self.BreakOff = true
                numStrucs = aiBrain:GetNumUnitsAroundPoint( 'Enemy', categories.STRUCTURE - ( categories.MASSEXTRACTION + categories.WALL ), position, 50 )
                numLand = aiBrain:GetNumUnitsAroundPoint( 'Enemy', ( categories.MOBILE * categories.LAND ) * ( categories.EXPERIMENTAL + categories.TECH3 ), position, 50 )
                local numAir = aiBrain:GetNumUnitsAroundPoint( 'Enemy', ( categories.MOBILE * categories.AIR ) * ( categories.EXPERIMENTAL + categories.TECH3 ), position, 50 )
                local numDef = aiBrain:GetNumUnitsAroundPoint( 'Enemy', categories.STRUCTURE * ( categories.DEFENSE + categories.STRATEGIC ), position, 60 )
                local unitToBuild = false
                if numDef > numLand and numDef > numAir then
                    if Random(1,2) == 1 then
                        unitToBuild = 'uel0307'
                    else
                        unitToBuild = 'uel0309'
                    end
                    # Build fatboy protectin
                elseif numLand > numStrucs and numLand > numAir then
                    unitToBuild = 'uel0202'
                    # Build Land killin
                elseif numStrucs > numAir and numStrucs > numLand then
                    unitToBuild = 'uel0111'
                    # Build struc killin
                else
                    unitToBuild = 'uel0205'
                    # build air killin
                end
                if unitToBuild then
                    IssueStop( {unit} )
                    IssueClearCommands( {unit} )
                    aiBrain:BuildUnit( unit, unitToBuild, 1 )
                end
                local unitBeingBuilt = false
                local building
                repeat
                    WaitSeconds(5)
                    unitBeingBuilt = unit:GetUnitBeingBuilt()
                    building = false
                    for k,v in self:GetPlatoonUnits() do
                        if not v:IsDead() and v:IsUnitState('Building') then
                            building = true
                            break
                        end
                    end
                until not building
                if unitBeingBuilt and ( not otherPlat or not aiBrain:PlatoonExists( otherPlat ) ) then
                    otherPlat = aiBrain:MakePlatoon( '', '' )
                end
                if unitBeingBuilt and not unitBeingBuilt:BeenDestroyed() then
                    unit.BuiltUnitCount = unit.BuiltUnitCount + 1
                    aiBrain:AssignUnitsToPlatoon( otherPlat, {unitBeingBuilt}, 'Attack', 'NoFormation' )
                    unitBeingBuilt:ForkThread( FatBoyBehaviors['FatBoy_'..unitToBuild..'AIThread'], unit )
                end
                self.BreakOff = false
            end
        end
        WaitSeconds( 5 )
    end
end

TempestBehavior = function(self)
    local aiBrain = self:GetBrain()
    local unit
    for k,v in self:GetPlatoonUnits() do
        if not v:IsDead() then
            unit = v
            break
        end
    end
    if not unit then
        return
    end
    if not EntityCategoryContains( categories.uas0401, unit ) then
        return
    end
    local otherPlat
    unit.BuiltUnitCount = 0
    self.Patrolling = false
    while aiBrain:PlatoonExists(self) and not unit:IsDead() do
        local position = unit:GetPosition()
        local numStrucs = aiBrain:GetNumUnitsAroundPoint( 'Enemy', categories.STRUCTURE - ( categories.MASSEXTRACTION + categories.WALL ), position, 65 )
        local numNaval = aiBrain:GetNumUnitsAroundPoint( 'Enemy', ( categories.MOBILE * categories.NAVAL ) * ( categories.EXPERIMENTAL + categories.TECH3 + categories.TECH2 ), position, 65 )
        while unit.BuiltUnitCount < 8 and ( numStrucs > 5 or numNaval > 10 ) do
            self.Patrolling = false
            self.BreakOff = true
            local numAir = aiBrain:GetNumUnitsAroundPoint( 'Enemy', ( categories.MOBILE * categories.AIR ) * ( categories.EXPERIMENTAL + categories.TECH3 ), position, 65 )
            local numDef = aiBrain:GetNumUnitsAroundPoint( 'Enemy', categories.STRUCTURE * ( categories.DEFENSE + categories.STRATEGIC ), position, 75 )
            local unitToBuild = false
            if numDef > numNaval and numDef > numAir then
                if Random(1,2) == 1 then
                    unitToBuild = 'uas0201'
                else
                    unitToBuild = 'uas0201'
                    #unitToBuild = 'ual0309'
                end
                # Build fatboy protectin
            elseif numNaval > numStrucs and numNaval > numAir then
                unitToBuild = 'uas0203'
                # Build Land killin
            elseif numStrucs > numAir and numStrucs > numLand then
                unitToBuild = 'uas0201'
                # Build struc killin
            else
                unitToBuild = 'uas0202'
                # build air killin
            end
            if unitToBuild then
                IssueStop( {unit} )
                IssueClearCommands( {unit} )
                aiBrain:BuildUnit( unit, unitToBuild, 1 )
            end
            local unitBeingBuilt = false
            local building
            repeat
                WaitSeconds(5)
                unitBeingBuilt = unit:GetUnitBeingBuilt()
                building = false
                for k,v in self:GetPlatoonUnits() do
                    if not v:IsDead() and v:IsUnitState('Building') then
                        building = true
                        break
                    end
                end
            until not building
            if unitBeingBuilt and not unitBeingBuilt:BeenDestroyed() then
                local testHeading = false
                testHeading = unit:GetHeading()
                unit.BuiltUnitCount = unit.BuiltUnitCount + 1
                ScenarioFramework.CreateUnitDestroyedTrigger( TempestUnitDeath, unitBeingBuilt )
                aiBrain:AssignUnitsToPlatoon( self, {unitBeingBuilt}, 'Attack', 'GrowthFormation' )
                IssueClearCommands( {unitBeingBuilt} )
                unitBeingBuilt:ForkThread( TempestBuiltUnitMoveOut, position, testHeading )
            end
            self.BreakOff = false
            numStrucs = aiBrain:GetNumUnitsAroundPoint( 'Enemy', categories.STRUCTURE - ( categories.MASSEXTRACTION + categories.WALL ), position, 65 )
            numNaval = aiBrain:GetNumUnitsAroundPoint( 'Enemy', ( categories.MOBILE * categories.NAVAL ) * ( categories.EXPERIMENTAL + categories.TECH3 + categories.TECH2 ), position, 65 )
        end
        if aiBrain:PlatoonExists(self) and self:IsOpponentAIRunning() and not self.Patrolling then
            self:Stop()
            self.Patrolling = true
            scoutPath = AIUtils.AIGetSortedNavalLocations(self:GetBrain())
            for k, v in scoutPath do
                self:Patrol(v)
            end
        end
        WaitSeconds( 5 )
    end
end

function TempestUnitDeath(unit)
    if unit.Tempest and not unit.Tempest:IsDead() and unit.Tempest.BuiltUnitCount then
        unit.Tempest.BuildUnitCount = unit.Tempest.BuildUnitCount - 1
    end
end

function TempestBuiltUnitMoveOut(unit, platoon, position, heading)
    if heading >= 270 or heading <= 90 then
        position =  { position[1], position[2], position[3] + 20 }
    else
        position = { position[1], position[2], position[3] - 20 }
    end
    local counter = 0
    repeat
        IssueClearCommands( {unit} )
        IssueMove( {unit}, position )
        WaitSeconds(5)
        if unit:IsDead() then
            return
        end
        counter = counter + 1
    until counter == 4 or platoon.Patrolling
end

    



# ==== FAT BOY BUILT UNIT BEHAVIORS ==== #
function FatBoyUnitDeath(unit)
    if unit.FatBoy and not unit.FatBoy:IsDead() and unit.FatBoy.BuiltUnitCount then
        unit.FatBoy.BuiltUnitCount = unit.FatBoy.BuiltUnitCount - 1
    end
end

FatBoyBehaviors = {
    
    FatBoy_uel0307AIThread = function(unit, fatboy)
        unit.FatBoy = fatboy
        ScenarioFramework.CreateUnitDestroyedTrigger( FatBoyUnitDeath, unit )
        IssueGuard( {unit}, fatboy )
    end,
    
    FatBoy_uel0202AIThread = function(unit, fatboy)
        unit.FatBoy = fatboy
        ScenarioFramework.CreateUnitDestroyedTrigger( FatBoyUnitDeath, unit )
        IssueGuard( {unit}, fatboy )
    end,
    
    FatBoy_uel0111AIThread = function(unit, fatboy)
        unit.FatBoy = fatboy
        ScenarioFramework.CreateUnitDestroyedTrigger( FatBoyUnitDeath, unit )
        IssueGuard( {unit}, fatboy )
    end,
    
    FatBoy_uel0205AIThread = function(unit, fatboy)
        unit.FatBoy = fatboy
        ScenarioFramework.CreateUnitDestroyedTrigger( FatBoyUnitDeath, unit )
        IssueGuard( {unit}, fatboy )
    end,
    
    FatBoy_uel0309AIThread = function(unit, fatboy)
        unit.FatBoy = fatboy
        ScenarioFramework.CreateUnitDestroyedTrigger( FatBoyUnitDeath, unit )
        local aiBrain = unit:GetAIBrain()
        local factionIndex = aiBrain:GetFactionIndex()
        repeat
            WaitSeconds(1)
        until unit:IsIdleState()
        IssueClearCommands({unit})
        while not unit:IsDead() do
            if not fatboy:IsDead() and fatboy:GetHealthPercent() < .6 then
                IssueRepair( {unit}, fatboy )
            else
                buildingTmplFile = import('/lua/modules/basetemplates.lua')
                baseTmplFile = import('/lua/modules/basetemplates.lua')
                buildingTmpl = buildingTmplFile['BuildingTemplates'][factionIndex]
                baseTmpl = baseTmplFile['ExpansionBaseTemplates'][factionIndex]
                relative = false
                baseTmpl = AIBuildStructures.AIBuildBaseTemplateFromLocation( baseTmpl, unit:GetPosition() )
                IssueStop({unit})
                IssueClearCommands({unit})
                local retVal, pos = AIBuildStructures.AIExecuteBuildStructure(aiBrain, unit, 'T2Artillery', nil, relative, buildingTmpl, baseTmpl, unit:GetPosition())
                repeat
                    WaitSeconds(3)
                    if unit:IsDead() then
                        return
                    end
                until unit:IsDead() or unit:IsIdleState()
                if pos then
                    # Check if unit at location
                    local checkUnits = GetUnitsInRect( Rect( pos[1] - 7, pos[3] - 7, pos[1] + 7, pos[3] + 7 ) )
                    if checkUnits then
                        for k,v in checkUnits do
                            if IsEnemy( v:GetAIBrain():GetArmyIndex(), armyIndex ) then
                                IssueCapture( {unit}, v )
                            end
                        end
                        repeat
                            WaitSeconds(3)
                            if unit:IsDead() then
                                return
                            end
                        until unit:IsDead() or unit:IsIdleState()
                    end
                end
            end
            WaitSeconds(2)
        end
    end,
}