#****************************************************************************
#**
#**  File     :  /cdimage/lua/modules/AI/aiarchetype-medium.lua
#**  Author(s): John Comes
#**
#**  Summary  : Medium AI
#**
#**  Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************

local AIBuildUnits = import('/lua/modules/ai/aibuildunits.lua')
local AIUtils = import('/lua/modules/ai/aiutilities.lua')
local PlatoonTemplates = import('/lua/modules/platoontemplates.lua').PlatoonTemplates

local BBTmplFile = '/lua/modules/basetemplates.lua'
local BuildingTmpl = 'BuildingTemplates'
local BaseTmpl = 'BaseTemplates'
local ExBaseTmpl = 'ExpansionBaseTemplates'
local Adj2x2Tmpl = 'Adjacency2x2'
local UCBC = '/lua/modules/editor/UnitCountBuildConditions.lua'
local MIBC = '/lua/modules/editor/MiscBuildConditions.lua'
local MABC = '/lua/modules/editor/MarkerBuildConditions.lua'
local OAUBC = '/lua/modules/editor/OtherArmyUnitCountBuildConditions.lua'
local EBC = '/lua/modules/editor/EconomyBuildConditions.lua'
local PCBC = '/lua/modules/editor/PlatoonCountBuildConditions.lua'
local SAI = '/lua/modules/ScenarioPlatoonAI.lua'
local PlatoonFile = '/lua/modules/platoon.lua'


function EvaluatePlan( aiBrain )
    local per = ScenarioInfo.ArmySetup[aiBrain.Name].AIPersonality
    if not per then return 1 end
    if per == 'medium' then 
        return 150
    end
    return 1
end


local PlatoonList = {
    
    
    ###################################################################
    ##  Commander
    ###################################################################    
        ###################
        ##  Initial
        ###################
    {
        BuilderName = 'CDR Initial',
        PlatoonAddBehaviors = { 'CDRLeash', },
        PlatoonTemplate = 'CommanderBuilder',
        Priority = 1000,
        BuildConditions = {
                { UCBC, 'HaveEqualToUnitsWithCategory', { 1, categories.COMMAND}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 1, categories.FACTORY * categories.LAND}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
        ExpansionExclude = {'Sea'},
        PlatoonData = {
            Construction = {
                BuildStructures = {
                    'T1Resource',
                    'T1EnergyProduction',
                    'T1EnergyProduction',
                    'T1LandFactory',
                    'T1Resource',
                    'T1Resource',
                    'T1Resource',
                }
            }
        }
    },
    {
        BuilderName = 'CDR Mass Fab CDR',
        PlatoonTemplate = 'CommanderBuilder',
        Priority = 900,
        BuildConditions = {
                { UCBC, 'HaveEqualToUnitsWithCategory', { 1, categories.COMMAND}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 1, categories.MASSFABRICATION * categories.TECH3}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 1, categories.MASSEXTRACTION * categories.TECH1}},
                { EBC, 'LessThanEconStorageRatio', { 0.1, 1.1}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
        ExpansionExclude = {'Sea'},
        PlatoonData = {
            Construction = {
                BuildClose = true,
                BuildStructures = {
                    'T1MassCreation',
                    'T1EnergyProduction',
                    'T1EnergyProduction',
                },
                Location = 'MAIN',
            }
        }
    },
    {
        BuilderName = 'CDR T1 Energy',
        PlatoonTemplate = 'CommanderBuilder',
        Priority = 900,
        BuildConditions = {
                { UCBC, 'HaveEqualToUnitsWithCategory', { 1, categories.COMMAND}},
                { EBC, 'LessThanEconStorageRatio', { 1.1, 0.1}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
        ExpansionExclude = {'Sea'},
        PlatoonData = {
            Construction = {
                AdjacencyCategory = 'FACTORY',
                AdjacencyDistance = 100,
                BuildClose = true,
                BuildStructures = {
                    'T1EnergyProduction',
                },
                Location = 'MAIN',
            }
        }
    },
    {
        BuilderName = 'CDR Assist Factory',
        PlatoonTemplate = 'CommanderAssist',
        Priority = 800,
        BuildConditions = {
                { UCBC, 'HaveEqualToUnitsWithCategory', { 1, categories.COMMAND}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
        ExpansionExclude = {'Sea'},
        PlatoonData = {
            Assist = {
                AssistRange = 80,
                BeingBuiltCategories = {'FACTORY', 'ENGINEER'}, 
                Time = 60,
            },
        }
    },
    {
        BuilderName = 'CDR Assist Mass Extractor Upgrade',
        PlatoonTemplate = 'CommanderAssist',
        Priority = 900,
        BuildConditions = {
                { UCBC, 'HaveEqualToUnitsWithCategory', { 1, categories.COMMAND}},
                { UCBC, 'HaveGreaterThanUnitsInCategoryBeingBuilt', { 0, categories.TECH2 * categories.MASSEXTRACTION}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
        ExpansionExclude = {'Sea'},
        PlatoonData = {
            Assist = {
                AssistRange = 80,
                BeingBuiltCategories = {'MASSEXTRACTION'}, 
                Time = 30,
            },
        }
    },
    {
        BuilderName = 'CDR T1 Land Factory',
        PlatoonTemplate = 'CommanderBuilder',
        Priority = 900,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.COMMAND}},
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.LAND}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 10, categories.FACTORY * categories.LAND}},
                
                { EBC, 'GreaterThanEconStorageRatio', { 0.7, 0.7}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
        ExpansionExclude = {'Sea'},
        PlatoonData = {
            Construction = {
                BuildClose = true,
                BuildStructures = {
                    'T1LandFactory',
                },
                Location = 'MAIN',
            }
        }
    },
    {
        BuilderName = 'CDR T1 AirFactory',
        PlatoonTemplate = 'CommanderBuilder',
        Priority = 750,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.COMMAND}},
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.AIR}},
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.TECH2 * categories.MASSEXTRACTION}},
                { UCBC, 'HaveLessThanUnitsInCategoryBeingBuilt', { 1, categories.TECH2 * categories.MASSEXTRACTION}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 4, categories.FACTORY * categories.AIR}},
                { EBC, 'GreaterThanEconTrend', { 0.7, 0.7}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
        ExpansionExclude = {'Sea'},
        PlatoonData = {
            Construction = {
                BuildClose = true,
                BuildStructures = {
                    'T1AirFactory',
                },
                Location = 'MAIN',
            }
        }
    },
    {
        BuilderName = 'CDR Base D',
        PlatoonTemplate = 'CommanderBuilder',
        PlatoonAIPlan = 'EngineerBuildAI',
        Priority = 950,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.COMMAND}},
                { MABC, 'MarkerLessThanDistance',  { 'Rally Point', 50, -5, 5, 0}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.7, 0.7}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
        ExpansionExclude = {'Sea'},
        PlatoonData = {
            Construction = {
                BaseTemplate = ExBaseTmpl,
                BuildClose = false,
                NearMarkerType = 'Rally Point',
                ThreatMin = -5,
                ThreatMax = 5,
                ThreatRings = 0,
                BuildStructures = {
                    'T1GroundDefense',
                    'T1AADefense',
                }
            }
        }
    },
   
    ###################################################################
    ##  Engineers
    ###################################################################   
        ##########
        ##  Tech 1
        ##########
    {
        BuilderName = 'T1 Engineer Disband - Init',
        PlatoonTemplate = 'EngineerOnlyBuild',
        Priority = 900,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.TECH1}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 5, categories.ENGINEER * categories.TECH1}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        RequiresConstruction = true,
        ExpansionExclude = {'Sea'},
    },
    { 
        BuilderName = 'T1 Engineer Disband - Filler',
        PlatoonTemplate = 'EngineerOnlyBuild',
        Priority = 500,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.TECH1}},
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 20, categories.MOBILE}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 12, categories.ENGINEER * categories.TECH1}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        RequiresConstruction = true,
        ExpansionExclude = {'Sea'},
    },
    {
        BuilderName = 'T2 Engineer Disband - Init',
        PlatoonTemplate = 'T2EngineerOnlyBuild',
        Priority = 925,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY - categories.TECH1}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 9, categories.ENGINEER * categories.TECH2}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        RequiresConstruction = true,
        ExpansionExclude = {'Sea'},
    },
    {
        BuilderName = 'T2 Engineer Disband - Filler',
        PlatoonTemplate = 'T2EngineerOnlyBuild',
        Priority = 925,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY - categories.TECH1}},
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 40, categories.MOBILE}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 18, categories.ENGINEER * categories.TECH2}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        ExpansionExclude = {'Sea'},
        RequiresConstruction = true,
    },
    {
        BuilderName = 'T3 Engineer Disband - Init',
        PlatoonTemplate = 'T3EngineerOnlyBuild',
        Priority = 950,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.TECH3}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 12, categories.ENGINEER * categories.TECH3}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        ExpansionExclude = {'Sea'},
        RequiresConstruction = true,
    },
    {
        BuilderName = 'T3 Engineer Disband - Filler',
        PlatoonTemplate = 'T3EngineerOnlyBuild',
        Priority = 950,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.TECH3}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 24, categories.ENGINEER * categories.TECH3}},
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 60, categories.MOBILE}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        ExpansionExclude = {'Sea'},
        RequiresConstruction = true,
    },
    {
        BuilderName = 'T3 Engineer Disband - Filler 2',
        PlatoonTemplate = 'T3EngineerOnlyBuild',
        Priority = 950,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.TECH3}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 36, categories.ENGINEER * categories.TECH3}},
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 60, categories.MOBILE * categories.TECH3}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        ExpansionExclude = {'Sea'},
        RequiresConstruction = true,
    },
    ########################################
    ##  ECONOMY CONSTRUCTION
    ########################################
    {
        BuilderName = 'T1 T1Resource Engineer',
        PlatoonTemplate = 'EngineerGenericSingle',
        Priority = 1000,
        InstanceCount = 1,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH1}},
                { MABC, 'MarkerLessThanDistance',  { 'Mass', 10000, 0, 0, 0}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildStructures = {
                    'T1Resource',
                    'T1Resource',
                    'T1Resource',
                    'T1Resource',
                    'T1Resource',
                    'T1Resource',
                    'T1Resource',
                }
            }
        }
    },
    {
        BuilderName = 'T1 Power Engineer',
        PlatoonTemplate = 'EngineerGenericSingle',
        Priority = 950,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH1}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 30, categories.TECH1 * categories.ENERGYPRODUCTION}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                AdjacencyCategory = 'FACTORY',
                AdjacencyDistance = 100,
                BuildClose = true,
                BuildStructures = {
                    'T1EnergyProduction',
                    'T1EnergyProduction',
                    'T1EnergyProduction',
                    'T1EnergyProduction',
                },
                Location = 'MAIN',
            }
        }
    },
    {
        BuilderName = 'T2 Power Engineer 2',
        PlatoonTemplate = 'T2EngineerBuilder',
        Priority = 900,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH2}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 6, categories.TECH2 * categories.ENERGYPRODUCTION}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                AdjacencyCategory = 'FACTORY',
                AdjacencyDistance = 100,
                BuildClose = true,
                BuildStructures = {
                    'T2EnergyProduction',
                    'T2EnergyProduction',
                    'T2EnergyProduction',
                },
                Location = 'MAIN',
            }
        }
    },
    {
        BuilderName = 'T2 Engineer Patrol',
        PlatoonTemplate = 'T2EngineerGenericSingle',
        PlatoonAIPlan = 'PatrolBaseVectorsAI',
        Priority = 950,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH1}},
                { EBC, 'LessThanEconStorageRatio', { 0.5, 1.1}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            NotPartOfAttackForce = true,
        },
    },
    {
        BuilderName = 'T3 Power Engineer',
        PlatoonTemplate = 'T3EngineerBuilder',
        Priority = 900,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH3}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 10, categories.TECH3 * categories.ENERGYPRODUCTION}},
                { EBC, 'LessThanEconTrend', { 1000, 2000}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                AdjacencyCategory = 'FACTORY',
                AdjacencyDistance = 100,
                BuildClose = true,
                BuildStructures = {
                    'T3EnergyProduction',
                    'T3EnergyProduction',
                    'T3EnergyProduction',
                },
                Location = 'MAIN',
            }
        }
    },
    {
        BuilderName = 'T3 Mass Ext Engineer',
        PlatoonTemplate = 'T3EngineerBuilder',
        Priority = 900,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH3}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 5, categories.TECH3 * categories.MASSEXTRACTION}},
                { EBC, 'LessThanEconTrend', { 40, 100000}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.1, 0.1}},
                { EBC, 'LessThanEconStorageRatio', { 0.8, 1.1}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildStructures = {
                    'T3MassExtraction',
                    'T3MassExtraction',
                }
            }
        }
    },
    {
        BuilderName = 'T3 Mass Fab Engineer',
        PlatoonTemplate = 'T3EngineerBuilder',
        Priority = 900,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH3}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 5, categories.TECH3 * categories.MASSFABRICATION}},
                { EBC, 'LessThanEconTrend', { 40, 100000}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.1, 0.1}},
                { EBC, 'LessThanEconStorageRatio', { 0.8, 1.1}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildClose = true,
                BuildStructures = {
                    'T3MassCreation',
                    'T3MassCreation',
                },
                Location = 'MAIN',
            }
        }
    },
    
    ########################################
    ##  BASE CONSTRUCTION
    ########################################
    {
        BuilderName = 'T1 Radar Engineer',
        PlatoonTemplate = 'EngineerBuilder',
        Priority = 950,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH1}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 1, categories.RADAR}},
                { EBC, 'GreaterThanEconIncome',  { 1, 10}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildClose = true,
                BuildStructures = {
                    'T1Radar',
                },
                Location = 'MAIN',
            }
        }
    },
    {
        BuilderName = 'T2 Air Staging Engineer',
        PlatoonTemplate = 'T2EngineerBuilder',
        Priority = 800,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH2}},
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 25, categories.AIR}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 2, categories.AIRSTAGINGPLATFORM}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.1, 0.1}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildClose = true,
                BuildStructures = {
                    'T2AirStagingPlatform',
                },
                Location = 'MAIN',
            }
        }
    },
    {
        BuilderName = 'T2 Artillery Engineer',
        PlatoonTemplate = 'T2EngineerBuilder',
        Priority = 800,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH2}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 5, categories.ARTILLERY * categories.STRUCTURE * categories.TECH2}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildClose = true,
                BuildStructures = {
                    'T2Artillery',
                    'T2StrategicMissile',
                },
                Location = 'MAIN',
            }
        }
    },
    {
        BuilderName = 'T3 Gate Engineer',
        PlatoonTemplate = 'T3EngineerBuilder',
        Priority = 900,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH3}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 1, categories.GATE * categories.TECH3 * categories.STRUCTURE}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.7, 0.7}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildClose = true,
                BuildStructures = {
                    'T3QuantumGate',
                },
                Location = 'MAIN',
            }
        }
    },
    {
        BuilderName = 'T3 Artillery Engineer',
        PlatoonTemplate = 'T3EngineerBuilder',
        Priority = 800,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH3}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.75, 0.75}},
                { EBC, 'GreaterThanEconIncome', {12, 100}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildClose = true,
                BuildStructures = {
                    'T3Artillery', 
                },
                Location = 'MAIN',
            }
        }
    },
    {
        BuilderName = 'T3 Nuke Engineer',
        PlatoonTemplate = 'T3EngineerBuilder',
        Priority = 825,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH3}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.75, 0.75}},
                { EBC, 'GreaterThanEconIncome', {15, 100}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildClose = true,
                BuildStructures = {
                    'T3StrategicMissile', 
                },
                Location = 'MAIN',
            }
        }
    },
    {
        BuilderName = 'T3 Land Exp1 Engineer',
        PlatoonTemplate = 'T3EngineerBuilder',
        Priority = 800,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH3}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.75, 0.75}},
                { EBC, 'GreaterThanEconIncome', {12, 100}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildClose = false,
                BaseTemplate = ExBaseTmpl,
                NearMarkerType = 'Rally Point',
                BuildStructures = {
                    'T4LandExperimental1', 
                },
                Location = 'MAIN',
            }
        }
    },
    {
        BuilderName = 'T3 Land Exp2 Engineer',
        PlatoonTemplate = 'T3EngineerBuilder',
        Priority = 800,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH3}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.75, 0.75}},
                { EBC, 'GreaterThanEconIncome', {12, 100}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildClose = false,
                BaseTemplate = ExBaseTmpl,
                NearMarkerType = 'Rally Point',
                BuildStructures = {
                    'T4LandExperimental2', 
                },
                Location = 'MAIN',
            }
        }
    },
    {
        BuilderName = 'T3 Air Exp1 Engineer',
        PlatoonTemplate = 'T3EngineerBuilder',
        Priority = 800,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH3}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.75, 0.75}},
                { EBC, 'GreaterThanEconIncome', {7, 100}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildClose = false,
                BaseTemplate = ExBaseTmpl,
                NearMarkerType = 'Protected Experimental Construction',
                BuildStructures = {
                    'T4AirExperimental1', 
                },
                Location = 'MAIN',
            }
        }
    },
    ########################################
    ##  BASE DEFENSE CONSTRUCTION
    ########################################
    {
        BuilderName = 'T1 Base D Engineer',
        PlatoonTemplate = 'EngineerGenericSingle',
        PlatoonAIPlan = 'EngineerBuildAI',
        Priority = 700,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH1}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 10, categories.DEFENSE * categories.TECH1}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildClose = true,
                BuildStructures = {
                    'T1GroundDefense', 
                },
                Location = 'MAIN',
            }
        }
    },
    {
        BuilderName = 'T1 Base D AA Engineer - Response',
        PlatoonTemplate = 'EngineerGenericSingle',
        PlatoonAIPlan = 'EngineerBuildAI',
        Priority = 900,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH1}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 8, categories.DEFENSE * categories.TECH1 * categories.ANTIAIR}},
                { UCBC, 'HaveUnitsWithCategoryAndAlliance', { true, 5, categories.MOBILE * categories.AIR, 'Enemy'}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildClose = true,
                BuildStructures = {
                    'T1AADefense', 
                },
                Location = 'MAIN',
            }
        }
    },
    {
        BuilderName = 'T1 Base D AA Engineer',
        PlatoonTemplate = 'EngineerGenericSingle',
        PlatoonAIPlan = 'EngineerBuildAI',
        Priority = 900,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH1}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 5, categories.DEFENSE * categories.TECH1 * categories.ANTIAIR}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildClose = true,
                BuildStructures = {
                    'T1AADefense', 
                },
                Location = 'MAIN',
            }
        }
    },
    {
        BuilderName = 'T2 Base D Engineer',
        PlatoonTemplate = 'T2EngineerGenericSingle',
        PlatoonAIPlan = 'EngineerBuildAI',
        Priority = 900,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH2}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 10, categories.DEFENSE * categories.TECH2}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildClose = true,
                BuildStructures = {
                    'T2AADefense',
                    'T2GroundDefense',
                },
                Location = 'MAIN',
            }
        }
    },
    {
        BuilderName = 'T2 Base D AA Engineer - Response',
        PlatoonTemplate = 'T2EngineerGenericSingle',
        PlatoonAIPlan = 'EngineerBuildAI',
        Priority = 900,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH2}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 10, categories.DEFENSE * categories.TECH2 * categories.ANTIAIR}},
                { UCBC, 'HaveUnitsWithCategoryAndAlliance', { true, 10, categories.MOBILE * categories.AIR, 'Enemy'}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildClose = true,
                BuildStructures = {
                    'T2AADefense', 
                },
                Location = 'MAIN',
            }
        }
    },
    {
        BuilderName = 'T2 Shield D Engineer Energy Production',
        PlatoonTemplate = 'T2EngineerBuilder',
        Priority = 850,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH2}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 5, categories.SHIELD * categories.TECH2}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                AdjacencyCategory = 'ENERGYPRODUCTION',
                AdjacencyDistance = 100,
                BuildClose = false,
                BuildStructures = {
                    'T2ShieldDefense', 
                },
                Location = 'MAIN',
            }
        }
    },
    {
        BuilderName = 'T2 Shield D Engineer',
        PlatoonTemplate = 'T2EngineerBuilder',
        Priority = 850,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH2}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 5, categories.SHIELD * categories.TECH2}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                AdjacencyCategory = 'DEFENSE DIRECTFIRE',
                AdjacencyDistance = 100,
                BuildClose = false,
                BuildStructures = {
                    'T2ShieldDefense', 
                },
                Location = 'MAIN',
            }
        }
    },
    {
        BuilderName = 'T3 Anti-Nuke Engineer',
        PlatoonTemplate = 'T3EngineerBuilder',
        Priority = 925,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 2, categories.ENGINEER * categories.TECH3}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 2, categories.ANTIMISSILE * categories.TECH3}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
                { EBC, 'GreaterThanEconIncome', { 5, 100}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildClose = false,
                BuildStructures = {
                    'T3StrategicMissileDefense', 
                },
                Location = 'MAIN',
            }
        }
    },
    {
        BuilderName = 'T3 Base D Engineer',
        PlatoonTemplate = 'T3EngineerBuilder',
        Priority = 875,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH3}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 20, categories.DEFENSE * categories.TECH3}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildClose = true,
                BuildStructures = {
                    'T3AADefense',
                },
                Location = 'MAIN',
            }
        }
    },
    {
        BuilderName = 'T3 Shield D Engineer',
        PlatoonTemplate = 'T3EngineerBuilder',
        Priority = 875,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH3}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 3, categories.SHIELD * categories.TECH3}},
                { MIBC, 'FactionIndex', {1, 2}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildClose = false,
                BuildStructures = {
                    'T3ShieldDefense', 
                },
                Location = 'MAIN',
            }
        }
    },


    ########################################
    ## EMERGENCY BUILDING TECH 1
    ########################################
    
    {
        BuilderName = 'T1 Emergency Mass Extraction Engineer',
        PlatoonTemplate = 'EngineerBuilder',
        Priority = 700,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH1}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
                { EBC, 'LessThanEconTrend', { 0, 100000}},
                { MABC, 'MarkerLessThanDistance',  { 'Mass', 1000}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildStructures = {
                    'T1Resource',
                }
            }
        }
    },
    {
        BuilderName = 'T1 Emergency Mass Creation Engineer',
        PlatoonTemplate = 'EngineerBuilder',
        Priority = 850,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH1}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 1, categories.MASSFABRICATION * categories.TECH3}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 1, categories.MASSEXTRACTION * categories.TECH1}},
                { EBC, 'LessThanEconStorageRatio', { 0.1, 1.1}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildClose = true,
                BuildStructures = {
                    'T1MassCreation',
                    'T1EnergyProduction',
                    'T1EnergyProduction',
                },
                Location = 'MAIN',
            }
        }
    },
    {
        BuilderName = 'T1 Emergency Power Engineer',
        PlatoonTemplate = 'EngineerBuilder',
        Priority = 850,
        InstanceCount = 3,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH1}},
                { EBC, 'LessThanEconStorageRatio', { 1.1, 0.1}},
                { EBC, 'LessThanEconTrend', { 100000, 0}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                AdjacencyCategory = 'FACTORY',
                AdjacencyDistance = 100,
                BuildClose = true,
                BuildStructures = {
                    'T1EnergyProduction',
                    'T1EnergyProduction',
                    'T1EnergyProduction',
                },
                Location = 'MAIN',
            }
        }
    },
    {
        BuilderName = 'T1 Emergency Hydrocarbon Engineer',
        PlatoonTemplate = 'EngineerBuilder',
        Priority = 900,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH1}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 1, categories.HYDROCARBON}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
                { MABC, 'MarkerLessThanDistance',  { 'Hydrocarbon', 200}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildStructures = {
                    'T1HydroCarbon',
                }
            }
        }
    },
    {
        BuilderName = 'T1 Energy Storage Engineer',
        PlatoonTemplate = 'EngineerBuilder',
        Priority = 750,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH1}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.9}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                AdjacencyCategory = 'ENERGYPRODUCTION',
                AdjacencyDistance = 100,
                BuildClose = false,
                BuildStructures = {
                    'EnergyStorage',
                },
                Location = 'MAIN',
            }
        }
    },
    
    ########################################
    ## EMERGENCY BUILDING TECH 2
    ########################################
    {
        BuilderName = 'T2 Power Engineer',
        PlatoonTemplate = 'T2EngineerBuilder',
        Priority = 975,
        InstanceCount = 1,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH2}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 15, categories.ENERGYPRODUCTION * categories.STRUCTURE * categories.TECH2}},
                { EBC, 'LessThanEconStorageRatio', { 1.1, 0.3}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                AdjacencyCategory = 'FACTORY',
                AdjacencyDistance = 100,
                BuildClose = true,
                BuildStructures = {
                    'T2EnergyProduction',
                    'T2EnergyProduction',
                },
                Location = 'MAIN',
            }
        }
    },
    
    ########################################
    ## EMERGENCY BUILDING TECH 3
    ########################################
    {
        BuilderName = 'T3 Emergency Power Engineer',
        PlatoonTemplate = 'T3EngineerBuilder',
        Priority = 975,
        InstanceCount = 1,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH3}},
                { EBC, 'LessThanEconStorageRatio', { 1.1, 0.5}},
                { EBC, 'LessThanEconTrend', { 100000, 0}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                AdjacencyCategory = 'FACTORY',
                AdjacencyDistance = 100,
                BuildClose = true,
                BuildStructures = {
                    'T3EnergyProduction',
                    'T3EnergyProduction',
                },
                Location = 'MAIN',
            }
        }
    },
    {
        BuilderName = 'T3 Emergency Mass Fab Engineer',
        PlatoonTemplate = 'T3EngineerBuilder',
        Priority = 900,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH3}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
                { EBC, 'LessThanEconTrend', { 0, 100000}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildClose = true,
                BuildStructures = {
                    'T3MassCreation',
                },
                Location = 'MAIN',
            }
        }
    },
    {
        BuilderName = 'T3 Extra Power Engineer',
        PlatoonTemplate = 'T3EngineerBuilder',
        Priority = 900,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH3}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
                { EBC, 'LessThanEconTrend', { 1000, 30}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                AdjacencyCategory = 'FACTORY',
                AdjacencyDistance = 100,
                BuildClose = true,
                BuildStructures = {
                    'T3EnergyProduction',
                    'T3EnergyProduction',
                },
                Location = 'MAIN',
            }
        }
    },
        
    ########################################
    ## FORWARD BASE BUILDING
    ########################################
    {
        BuilderName = 'T1 Defensive Point Engineer',
        PlatoonTemplate = 'EngineerGenericSingle',
        PlatoonAIPlan = 'EngineerBuildAI',
        Priority = 900,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH1}},
                

                # Most paramaters freaking ever Build Condition -- All the threat ones are optional
                ####                                                 MarkerType   MarkerRadius  LocationType  
                { UCBC, 'HaveLessThanUnitsAroundMarkerCategory', { 'Defensive Point', 20,         'MAIN', 
                # LocationRadius  UnitCount    UnitCategory 
                       200,           5,     'DEFENSE TECH1'} },
                       
                       
                       
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
        },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                Location = 'MAIN',
                BuildClose = false,
                BaseTemplate = ExBaseTmpl,
                NearMarkerType = 'Defensive Point', 
                MarkerRadius = 20, 
                LocationType = 'MAIN',
                MarkerUnitCount = 5, 
                MarkerUnitCategory = 'DEFENSE TECH1',
                BuildStructures = {
                    'T1GroundDefense', 
                    'T1AADefense', 
                },
            },
        },
    },
    {
        BuilderName = 'T1 Defensive Point Engineer 2',
        PlatoonTemplate = 'EngineerGenericSingle',
        PlatoonAIPlan = 'EngineerBuildAI',
        Priority = 900,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH1}},
                { UCBC, 'HaveUnitsWithCategoryAndAlliance', { true, 5, categories.MOBILE * categories.AIR, 'Enemy'}},
                { UCBC, 'HaveLessThanUnitsAroundMarkerCategory', { 'Defensive Point', 20, 'MAIN', 200, 5, 'DEFENSE TECH1'} },
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
        },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                Location = 'MAIN',
                BuildClose = false,
                BaseTemplate = ExBaseTmpl,
                NearMarkerType = 'Defensive Point', 
                MarkerRadius = 20, 
                LocationType = 'MAIN',
                MarkerUnitCount = 5, 
                MarkerUnitCategory = 'DEFENSE TECH1',
                BuildStructures = {
                    'T1AADefense', 
                },
            },
        },
    },
    {
        BuilderName = 'T1 Expansion Area Engineer',
        PlatoonTemplate = 'EngineerBuilder',
        Priority = 900,
        InstanceCount = 1,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH1}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildClose = false,
                BaseTemplate = ExBaseTmpl,
                ExpansionBase = true,
                ExpansionRadius = 50,
                ExpansionTypes = { 'Land', 'Air' },
                NearMarkerType = 'Expansion Area',
                ThreatMin = -100,
                ThreatMax = 10,
                ThreatRings = 0,
                BuildStructures = {
                    'T1LandFactory',
                    'T1GroundDefense',
                }
            }
        }
    },
    {
        BuilderName = 'T2 Expansion Area Engineer',
        PlatoonTemplate = 'T2EngineerBuilder',
        Priority = 900,
        InstanceCount = 1,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH2}},
                { EBC, 'GreaterThanEconTrend',  { 0.1, 0.5}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildClose = false,
                BaseTemplate = ExBaseTmpl,
                NearMarkerType = 'Expansion Area',
                BuildStructures = {
                    'T2GroundDefense',
                    'T2AADefense',
                }
            }
        }
    },
    ###################################################################
    ##  ENGINEER ASSIST
    ################################################################### 
    {
        BuilderName = 'T1 Engineer Assist',
        PlatoonTemplate = 'EngineerAssist',
        Priority = 900,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 1, categories.ENGINEER * categories.TECH1}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Assist = {
                AssistRange = 80,
                BuilderCategories = {'FACTORY', 'ENGINEER'},
                Time = 60,
            },
        }
    },
    {
        BuilderName = 'T1 Engineer Assist Mass Upgrade',
        PlatoonTemplate = 'EngineerAssist',
        Priority = 950,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH1}},
                { UCBC, 'HaveGreaterThanUnitsInCategoryBeingBuilt', { 0, categories.TECH2 * categories.MASSEXTRACTION}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Assist = {
                AssistRange = 80,
                BeingBuiltCategories = {'MASSEXTRACTION'}, 
                Time = 60,
            },
        }
    },
    {
        BuilderName = 'T2 Engineer Assist',
        PlatoonTemplate = 'T2EngineerAssist',
        Priority = 900,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 1, categories.ENGINEER * categories.TECH2}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Assist = {
                AssistRange = 80,
                BuilderCategories = {'FACTORY', 'ENGINEER'},
                Time = 60,
            },
        }
    },
    {
        BuilderName = 'T3 Engineer Assist Factory/Engineer',
        PlatoonTemplate = 'T3EngineerAssist',
        Priority = 900,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 1, categories.ENGINEER * categories.TECH3}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        ExpansionExclude = {'Sea'},
        RequiresConstruction = false,
        PlatoonData = {
            Assist = {
                AssistRange = 80,
                BuilderCategories = {'FACTORY', 'ENGINEER'},
                Time = 60,
            },
        }
    },
    ###################################################################
    ##  STRUCTURES
    ################################################################### 
    {
        BuilderName = 'T1 Mass Extractor Upgrade',
        PlatoonTemplate = 'T1MassExtractorUpgrade',
        InstanceCount = 1,
        Priority = 200,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.MASSEXTRACTION * categories.TECH1}},
                { EBC, 'GreaterThanEconIncome',  { 1.5, 10}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
    },
    {
        BuilderName = 'T1 Land Factory Upgrade',
        PlatoonTemplate = 'T1LandFactoryUpgrade',
        Priority = 200,
        InstanceCount = 1,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.TECH1 * categories.LAND}},
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.MASSEXTRACTION * categories.TECH2}},
                { EBC, 'GreaterThanEconIncome',  { 4, 10}},
                { EBC, 'GreaterThanEconStorageRatio',  { 0.5, 0.5}},
            },
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
    },
    {
        BuilderName = 'T1 Air Factory Upgrade',
        PlatoonTemplate = 'T1AirFactoryUpgrade',
        Priority = 200,
        InstanceCount = 1,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.TECH1 * categories.AIR}},
                { EBC, 'GreaterThanEconIncome',  { 3, 10}},
                { EBC, 'GreaterThanEconStorageRatio',  { 0.5, 0.5}},
            },
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
    },
    {
        BuilderName = 'T2 Mass Extractor Upgrade',
        PlatoonTemplate = 'T2MassExtractorUpgrade',
        InstanceCount = 1,
        Priority = 200,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.MASSEXTRACTION * categories.TECH2}},
                { EBC, 'GreaterThanEconIncome',  { 4, 15}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
    },
    {
        BuilderName = 'T2 Land Factory Upgrade',
        PlatoonTemplate = 'T2LandFactoryUpgrade',
        Priority = 300,
        InstanceCount = 2,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.TECH2 * categories.LAND}},
                { EBC, 'GreaterThanEconIncome',  { 9, 15}},
                { EBC, 'GreaterThanEconStorageRatio',  { 0.5, 0.5}},
            },
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
    },
    {
        BuilderName = 'T2 Air Factory Upgrade',
        PlatoonTemplate = 'T2AirFactoryUpgrade',
        Priority = 300,
        InstanceCount = 2,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.TECH2 * categories.AIR}},
                { EBC, 'GreaterThanEconIncome',  { 9, 15}},
                { EBC, 'GreaterThanEconStorageRatio',  { 0.5, 0.5}},
            },
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
    },
    {
        BuilderName = 'T1 Radar Upgrade',
        PlatoonTemplate = 'T1RadarUpgrade',
        Priority = 200,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.RADAR * categories.TECH1}},
                { EBC, 'GreaterThanEconIncome',  { 5, 15}},
                { EBC, 'GreaterThanEconStorageRatio',  { 0.5, 0.5}},
            },
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
    },
    {
        BuilderName = 'T2 Radar Upgrade',
        PlatoonTemplate = 'T2RadarUpgrade',
        Priority = 300,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.RADAR * categories.TECH2}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 1, categories.RADAR * categories.TECH3}},
                { EBC, 'GreaterThanEconIncome',  { 10, 600}},
                { EBC, 'GreaterThanEconStorageRatio',  { 0.5, 0.5}},
            },
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
    },
    {
        BuilderName = 'T2 TML Silo',
        PlatoonTemplate = 'T2TacticalLauncher',
        Priority = 5,
        InstanceCount = 5,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.STRUCTURE * categories.TECH2 * categories.TACTICALMISSILEPLATFORM}},
            },
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
    },
    {
        BuilderName = 'T2 Shield Cybran',
        PlatoonTemplate = 'T2Shield',
        Priority = 5,
        InstanceCount = 5,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.SHIELD * categories.TECH2 * categories.STRUCTURE}},
                { EBC, 'GreaterThanEconIncome',  { 5, 100}},
                { MIBC, 'FactionIndex', {3, 3}},
            },
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
        PlatoonData = {
            Enhancement = { 
                    'Shield1',
                    'Shield2',
                    'Shield3',
                    'Shield4',
                },
            TimeBetweenEnhancements = 120,
            DoNotDisband = true,
        },
    },
    {
        BuilderName = 'T2 Shield UEF and Aeon',
        PlatoonTemplate = 'T2Shield',
        Priority = 5,
        InstanceCount = 1,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.SHIELD * categories.TECH2 * categories.STRUCTURE}},
                { EBC, 'GreaterThanEconIncome',  { 7, 100}},
                { MIBC, 'FactionIndex', {1, 1}},
            },
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
    },
    {
        BuilderName = 'T3 Nuke Silo',
        PlatoonTemplate = 'T3Nuke',
        Priority = 5,
        InstanceCount = 5,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.STRUCTURE * categories.TECH3 * categories.NUKE}},
            },
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
    },
    {
        BuilderName = 'T3 Anti Nuke Silo',
        PlatoonTemplate = 'T3AntiNuke',
        Priority = 5,
        InstanceCount = 5,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.STRUCTURE * categories.TECH3 * categories.ANTIMISSILE}},
            },
        BuildTimeOut = 120,
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
    },
    {
        BuilderName = 'T1 Mass Fabricator Pause',
        PlatoonTemplate = 'T1MassFabricator',
        Priority = 300,
        InstanceCount = 3,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.MASSFABRICATION * categories.TECH1}},
                { EBC, 'LessThanEconStorageRatio',  { 1.1, 0.05}},
            },
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
    },
    {
        BuilderName = 'T3 Mass Fabricator Pause',
        PlatoonTemplate = 'T3MassFabricator',
        Priority = 300,
        InstanceCount = 3,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.MASSFABRICATION * categories.TECH3}},
                { EBC, 'LessThanEconStorageRatio',  { 1.1, 0.05}},
            },
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
    },

    ###################################################################
    ##  LAND ATTACK
    ################################################################### 
        ##########
        ##  Tech 1
        ##########      
    {
        BuilderName = 'T1 Land Scout',
        PlatoonTemplate = 'T1LandScout1',
        Priority = 750,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.LAND}},
                { UCBC, 'HaveEqualToUnitsWithCategory', { 0, categories.LAND * categories.SCOUT * categories.TECH1}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Land',
        PlatoonAddPlans = {'DistressResponseAI', 'PlatoonCallForHelpAI'},
        RequiresConstruction = true,
    },
    {
        BuilderName = 'T1 Light Assault Bot',
        PlatoonTemplate = 'T1LandDFBot1',
        Priority = 700,
        InstanceCount = 1,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.LAND}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Land',
        PlatoonAddPlans = {'DistressResponseAI', 'PlatoonCallForHelpAI'},
        RequiresConstruction = true,
        PlatoonData = {
            LocationType = 'MAIN',
        },
    },
    {
        BuilderName = 'T1 Light Tank',
        PlatoonTemplate = 'T1LandDFTank1',
        Priority = 500,
        InstanceCount = 1,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.LAND}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Land',
        PlatoonAddPlans = {'DistressResponseAI', 'PlatoonCallForHelpAI'},
        RequiresConstruction = true,
        PlatoonData = {
            LocationType = 'MAIN',
        },
    },
    {
        BuilderName = 'T1 Mobile AA',
        PlatoonTemplate = 'T1LandAA1',
        PlatoonAddBehaviors = { 'AirLandToggle' },
        Priority = 525,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.LAND}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}}         
            },
        LocationType = 'MAIN',
        PlatoonType = 'Land',
        PlatoonAddPlans = {'DistressResponseAI', 'PlatoonCallForHelpAI'},
        RequiresConstruction = true,
        PlatoonData = {
            LocationType = 'MAIN',
        },
    },
    {
        BuilderName = 'T1 Mortar',
        PlatoonTemplate = 'T1LandArtillery1',
        Priority = 500,
        InstanceCount = 1,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.LAND}},
                { UCBC, 'HaveUnitsWithCategoryAndAlliance', { true, 5, categories.DEFENSE, 'Enemy'}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Land',
        PlatoonAddPlans = {'DistressResponseAI', 'PlatoonCallForHelpAI'},
        RequiresConstruction = true,
        PlatoonData = {
            LocationType = 'MAIN',
        },
    },
        ##########
        ##  Tech 2
        ########## 
    {
        BuilderName = 'T2 Tank',
        PlatoonTemplate = 'T2LandDFTank1',
        Priority = 600,
        InstanceCount = 2,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.LAND - categories.TECH1}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Land',
        PlatoonAddPlans = {'DistressResponseAI', 'PlatoonCallForHelpAI'},
        RequiresConstruction = true,
        PlatoonData = {
            LocationType = 'MAIN',
        },
    },
    {
        BuilderName = 'T2 MML',
        PlatoonTemplate = 'T2LandArtillery1',
        Priority = 600,
        InstanceCount = 1,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.LAND - categories.TECH1}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Land',
        PlatoonAddPlans = {'DistressResponseAI', 'PlatoonCallForHelpAI'},
        RequiresConstruction = true,
        PlatoonData = {
            LocationType = 'MAIN',
        },
    },
    {
        BuilderName = 'T2 Mobile Flak  Response',
        PlatoonTemplate = 'T2LandAA1',
        Priority = 650,
        InstanceCount = 1,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.LAND - categories.TECH1}},
                { UCBC, 'HaveUnitsWithCategoryAndAlliance', { true, 10, categories.MOBILE * categories.AIR, 'Enemy'}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Land',
        PlatoonAddPlans = {'DistressResponseAI', 'PlatoonCallForHelpAI'},
        RequiresConstruction = true,
        PlatoonData = {
            LocationType = 'MAIN',
        },
    },
    {
        BuilderName = 'T2 Mobile Flak 2',
        PlatoonTemplate = 'T2LandAA1',
        Priority = 600,
        InstanceCount = 1,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.LAND - categories.TECH1}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Land',
        PlatoonAddPlans = {'DistressResponseAI', 'PlatoonCallForHelpAI'},
        RequiresConstruction = true,
        PlatoonData = {
            LocationType = 'MAIN',
        },
    },
    {
        BuilderName = 'T2 Amphibious Tank',
        PlatoonTemplate = 'T2LandAmphibious1',
        Priority = 600,
        InstanceCount = 1,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.LAND - categories.TECH1}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Land',
        PlatoonAddPlans = {'DistressResponseAI', 'PlatoonCallForHelpAI'},
        RequiresConstruction = true,
        PlatoonData = {
            LocationType = 'MAIN',
        },
    },
        ##########
        ##  Tech 3
        ########## 
    {
        BuilderName = 'T3 Siege Assault Bot',
        PlatoonTemplate = 'T3LandBot1',
        Priority = 700,
        InstanceCount = 1,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.LAND * categories.TECH3}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}}, 
            },
        LocationType = 'MAIN',
        PlatoonType = 'Land',
        PlatoonAddPlans = {'DistressResponseAI', 'PlatoonCallForHelpAI'},
        RequiresConstruction = true,
        PlatoonData = {
            LocationType = 'MAIN',
        },
    },
    {
        BuilderName = 'T3 Mobile Heavy Artillery',
        PlatoonTemplate = 'T3LandArtillery1',
        Priority = 700,
        InstanceCount = 1,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.LAND * categories.TECH3}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Land',
        PlatoonAddPlans = {'DistressResponseAI', 'PlatoonCallForHelpAI'},
        RequiresConstruction = true,
        PlatoonData = {
            LocationType = 'MAIN',
        },
    },
    {
        BuilderName = 'T3 Sub Commander',
        PlatoonTemplate = 'T3LandSubCommander1',
        Priority = 700,
        InstanceCount = 1,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.GATE * categories.TECH3}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Gate',
        RequiresConstruction = true,
    },
    {
        BuilderName = 'T4 Exp Land 1',
        PlatoonAddBehaviors = { 'FatBoyBehavior', },
        PlatoonTemplate = 'T4ExperimentalLand1',
        Priority = 800,
        InstanceCount = 1,
        BuildTimeOut = 1200,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.LAND * categories.EXPERIMENTAL}},
            },
        PlatoonType = 'Land',
        PlatoonAddPlans = {'DistressResponseAI', 'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
    },
    {
        BuilderName = 'T4 Exp Land 2',
        PlatoonAddBehaviors = { 'FatBoyBehavior', },
        PlatoonTemplate = 'T4ExperimentalLand2',
        Priority = 800,
        InstanceCount = 1,
        BuildTimeOut = 1200,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.LAND * categories.EXPERIMENTAL}},
            },
        PlatoonType = 'Land',
        PlatoonAddPlans = {'DistressResponseAI', 'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
    },
    
    
    ###################################################################
    ##  AIR ATTACK
    ###################################################################   
        ##########
        ##  Tech 1
        ##########
    
    {
        BuilderName = 'T1 Air Scout',
        PlatoonTemplate = 'T1AirScout1',
        Priority = 650,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.AIR}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Air',
        RequiresConstruction = true,
    },
    {
        BuilderName = 'T1 Air Bomber',
        PlatoonTemplate = 'T1AirBomber1',
        PlatoonAddBehaviors = { 'AirUnitRefit' },
        Priority = 500,
        InstanceCount = 1,
        BuildTimeOut = 2400,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.AIR}},
                { EBC, 'GreaterThanEconTrend', { -0.5, -1}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Air',
        RequiresConstruction = true,
    },
    {
        BuilderName = 'T1 Air Fighter',
        PlatoonTemplate = 'T1AirFighter1',
        PlatoonAddBehaviors = { 'AirUnitRefit' },
        Priority = 550,
        InstanceCount = 1,
        BuildTimeOut = 2400,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.AIR}},
                { UCBC, 'HaveUnitsWithCategoryAndAlliance', { true, 5, categories.MOBILE * categories.AIR, 'Enemy'}},
                { EBC, 'GreaterThanEconTrend', { -0.5, -1}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Air',
        RequiresConstruction = true,
    },
        ##########
        ##  Tech 2
        ##########
    {
        BuilderName = 'T2 Air Gunship',
        PlatoonTemplate = 'T2AirGunship1',
        PlatoonAddBehaviors = { 'AirUnitRefit' },
        Priority = 600,
        InstanceCount = 1,
        BuildTimeOut = 2400,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.AIR - categories.TECH1}},
                { EBC, 'GreaterThanEconTrend', { -0.8, -1}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Air',
        RequiresConstruction = true,
    },
  
        ##########
        ##  Tech 3
        ##########
    {
        BuilderName = 'T3 Air Gunship',
        PlatoonTemplate = 'T3AirGunship1',
        PlatoonAddBehaviors = { 'AirUnitRefit' },
        Priority = 700,
        InstanceCount = 1,
        BuildTimeOut = 2400,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.AIR * categories.TECH3}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
                { EBC, 'GreaterThanEconTrend', { -1, -2}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Air',
        RequiresConstruction = true,
    },
    {
        BuilderName = 'T3 Air Scout',
        PlatoonTemplate = 'T3AirScout1',
        PlatoonAddBehaviors = { 'AirUnitRefit' },
        Priority = 750,
        InstanceCount = 1,
        BuildTimeOut = 2400,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.AIR * categories.TECH3}},
                { EBC, 'GreaterThanEconTrend', { -1, -2}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Air',
        RequiresConstruction = true,
    },
    {
        BuilderName = 'T3 Air Fighter',
        PlatoonTemplate = 'T3AirFighter1',
        Priority = 725,
        PlatoonAddBehaviors = { 'AirUnitRefit' },
        InstanceCount = 1,
        BuildTimeOut = 2400,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.AIR * categories.TECH3}},
                { UCBC, 'HaveUnitsWithCategoryAndAlliance', { true, 10, categories.MOBILE * categories.AIR, 'Enemy'}},
                { EBC, 'GreaterThanEconTrend', { -1, -2}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Air',
        RequiresConstruction = true,
    },
    {
        BuilderName = 'T3 Air Bomber',
        PlatoonTemplate = 'T3AirBomber1',
        Priority = 700,
        PlatoonAddBehaviors = { 'AirUnitRefit' },
        InstanceCount = 1,
        BuildTimeOut = 2400,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.AIR * categories.TECH3}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
                { EBC, 'GreaterThanEconTrend', { -1, -2}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Air',
        RequiresConstruction = true,
    },
    {
        BuilderName = 'T4 Exp Air',
        PlatoonTemplate = 'T4ExperimentalAir',
        Priority = 800,
        InstanceCount = 1,
        BuildTimeOut = 2400,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.AIR * categories.EXPERIMENTAL}},
                { MIBC, 'FactionIndex', {2, 3}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        PlatoonType = 'Air',
        RequiresConstruction = false,
    },


    
    ########################################
    ## NAVAL BASE BUILDING
    ########################################
    {
        BuilderName = 'T1 Naval Builder',
        PlatoonTemplate = 'EngineerGenericSingle',
        Priority = 975,
        InstanceCount = 1,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH1}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 2, categories.FACTORY * categories.NAVAL}},
                { MABC, 'MarkerLessThanDistance',  { 'Naval Area', 200}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildClose = false,
                BaseTemplate = ExBaseTmpl,
                ExpansionBase = true,
                ExpansionRadius = 50,
                ExpansionTypes = { 'Sea' },
                NearMarkerType = 'Naval Area',
                BuildStructures = {
                    'T1SeaFactory',
                    'T1NavalDefense',
                    'T1Sonar',
                }
            }
        }
    },
    {
        BuilderName = 'T1 Naval D Engineer',
        PlatoonTemplate = 'EngineerGenericSingle',
        PlatoonAIPlan = 'EngineerBuildAI',
        Priority = 700,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH1}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 4, categories.DEFENSE * categories.TECH1}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Sea',
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildClose = true,
                BuildStructures = {
                    'T1NavalDefense', 
                },
            }
        }
    },
    {
        BuilderName = 'T1 Base D Naval AA Engineer',
        PlatoonTemplate = 'EngineerGenericSingle',
        PlatoonAIPlan = 'EngineerBuildAI',
        Priority = 750,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH1}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 5, categories.DEFENSE * categories.TECH1 * categories.ANTIAIR}},
                { UCBC, 'HaveUnitsWithCategoryAndAlliance', { true, 5, categories.MOBILE * categories.AIR, 'Enemy'}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Sea',
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildClose = true,
                BuildStructures = {
                    'T1AADefense', 
                },
            }
        }
    },
    {
        BuilderName = 'T2 Naval D Engineer',
        PlatoonTemplate = 'EngineerGenericSingle',
        PlatoonAIPlan = 'EngineerBuildAI',
        Priority = 750,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.ENGINEER * categories.TECH2}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 5, categories.DEFENSE * categories.TECH2 * categories.ANTINAVY}},
                { UCBC, 'HaveUnitsWithCategoryAndAlliance', { true, 5, categories.MOBILE * categories.NAVAL, 'Enemy'}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Sea',
        RequiresConstruction = false,
        PlatoonData = {
            Construction = {
                BuildClose = true,
                BuildStructures = {
                    'T2NavalDefense', 
                },
            }
        }
    },
    {
        BuilderName = 'T1 Sonar Upgrade',
        PlatoonTemplate = 'T1SonarUpgrade',
        Priority = 200,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.SONAR * categories.TECH1}},
                { EBC, 'GreaterThanEconIncome',  { 5, 15}},
                { EBC, 'GreaterThanEconStorageRatio',  { 0.5, 0.5}},
            },
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
    },
    {
        BuilderName = 'T2 Sonar Upgrade',
        PlatoonTemplate = 'T2SonarUpgrade',
        Priority = 300,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.SONAR * categories.TECH2}},
                { UCBC, 'HaveLessThanUnitsWithCategory', { 1, categories.SONAR * categories.TECH3}},
                { EBC, 'GreaterThanEconIncome',  { 10, 600}},
                { EBC, 'GreaterThanEconStorageRatio',  { 0.5, 0.5}},
            },
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
    },
     {
        BuilderName = 'T1 Sea Factory Upgrade',
        PlatoonTemplate = 'T1SeaFactoryUpgrade',
        Priority = 200,
        InstanceCount = 2,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.TECH1 * categories.NAVAL}},
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 3, categories.MOBILE * categories.NAVAL}},
                { EBC, 'GreaterThanEconIncome',  { 6, 10}},
                { EBC, 'GreaterThanEconStorageRatio',  { 0.5, 0.5}},
            },
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
    },
    {
        BuilderName = 'T2 Sea Factory Upgrade',
        PlatoonTemplate = 'T2SeaFactoryUpgrade',
        Priority = 300,
        InstanceCount = 2,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.TECH2 * categories.NAVAL}},
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 6, categories.MOBILE * categories.NAVAL}},
                { EBC, 'GreaterThanEconIncome',  { 10, 15}},
                { EBC, 'GreaterThanEconStorageRatio',  { 0.5, 0.5}},
            },
        PlatoonType = 'Any',
        PlatoonAddPlans = {'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
    },
    
    ###################################################################
    ##  SEA ATTACK
    ###################################################################   
        ##########
        ##  Tech 1
        ##########
    {
        BuilderName = 'T1 Naval Sub',
        PlatoonTemplate = 'T1SeaSub1',
        Priority = 500,
        InstanceCount = 1,
        BuildTimeOut = 2400,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.NAVAL}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
                { EBC, 'GreaterThanEconTrend', { -1, -2}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Sea',
        PlatoonAddPlans = {'DistressResponseAI', 'PlatoonCallForHelpAI'},
        RequiresConstruction = true,
    },
    {
        BuilderName = 'T1 Naval Frigate',
        PlatoonTemplate = 'T1SeaFrigate1',
        Priority = 500,
        InstanceCount = 1,
        BuildTimeOut = 2400,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.NAVAL}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
                { EBC, 'GreaterThanEconTrend', { -1, -2}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Sea',
        PlatoonAddPlans = {'DistressResponseAI', 'PlatoonCallForHelpAI'},
        RequiresConstruction = true,
    },
    {
        BuilderName = 'T1 Naval Anti-Air',
        PlatoonTemplate = 'T1SeaFrigate2',
        Priority = 525,
        InstanceCount = 1,
        BuildTimeOut = 2400,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.NAVAL}},
                { UCBC, 'HaveUnitsWithCategoryAndAlliance', { true, 10, categories.MOBILE * categories.AIR, 'Enemy'}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
                { EBC, 'GreaterThanEconTrend', { -1, -2}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Sea',
        PlatoonAddPlans = {'DistressResponseAI', 'PlatoonCallForHelpAI'},
        RequiresConstruction = true,
    },
        ##########
        ##  Tech 2
        ##########
    {
        BuilderName = 'T2 Naval Destroyer',
        PlatoonTemplate = 'T2SeaDestroyer1',
        Priority = 600,
        InstanceCount = 1,
        BuildTimeOut = 2400,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.NAVAL - categories.TECH1}},
                { EBC, 'GreaterThanEconTrend', { -1, -2}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Sea',
        PlatoonAddPlans = {'DistressResponseAI', 'PlatoonCallForHelpAI'},
        RequiresConstruction = true,
    },
    {
        BuilderName = 'T2 Naval Cruiser',
        PlatoonTemplate = 'T2SeaCruiser1',
        PlatoonAddBehaviors = { 'AirLandToggle' },
        Priority = 600,
        InstanceCount = 1,
        BuildTimeOut = 2400,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.NAVAL - categories.TECH1}},
                { EBC, 'GreaterThanEconTrend', { -1, -2}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Sea',
        PlatoonAddPlans = {'DistressResponseAI', 'PlatoonCallForHelpAI'},
        RequiresConstruction = true,
    },
        ##########
        ##  Tech 3
        ##########
    {
        BuilderName = 'T3 Naval Battleship',
        PlatoonTemplate = 'T3SeaBattleship1',
        Priority = 700,
        InstanceCount = 1,
        BuildTimeOut = 2400,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.NAVAL * categories.TECH3}},
                { EBC, 'GreaterThanEconTrend', { -1, -2}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Sea',
        PlatoonAddPlans = {'DistressResponseAI', 'PlatoonCallForHelpAI'},
        RequiresConstruction = true,
    },
    {
        BuilderName = 'T3 Naval Nuke Sub',
        PlatoonTemplate = 'T3SeaNukeSub1',
        Priority = 700,
        InstanceCount = 1,
        BuildTimeOut = 2400,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.NAVAL * categories.TECH3}},
                { EBC, 'GreaterThanEconTrend', { 10, 20}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Sea',
        PlatoonAddPlans = {'DistressResponseAI', 'PlatoonCallForHelpAI'},
        RequiresConstruction = true,
    },
    {
        BuilderName = 'T3 Naval Carrier',
        PlatoonTemplate = 'T3SeaNukeSub1',
        Priority = 700,
        InstanceCount = 1,
        BuildTimeOut = 2400,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.FACTORY * categories.NAVAL * categories.TECH3}},
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 20, categories.MOBILE * categories.AIR}},
                { EBC, 'GreaterThanEconTrend', { -1, -2}},
                { EBC, 'GreaterThanEconStorageRatio', { 0.5, 0.5}},
            },
        LocationType = 'MAIN',
        PlatoonType = 'Sea',
        PlatoonAddPlans = {'DistressResponseAI', 'PlatoonCallForHelpAI'},
        RequiresConstruction = true,
    },
    {
        BuilderName = 'T4 Exp Sea',
        PlatoonTemplate = 'T4ExperimentalSea',
        PlatoonAddBehaviors = { 'TempestBehavior' },
        Priority = 800,
        InstanceCount = 1,
        BuildTimeOut = 2400,
        BuildConditions = {
                { UCBC, 'HaveGreaterThanUnitsWithCategory', { 0, categories.AIR * categories.EXPERIMENTAL}},
            },
        PlatoonType = 'Sea',
        PlatoonAddPlans = {'DistressResponseAI', 'PlatoonCallForHelpAI'},
        RequiresConstruction = false,
    },
    {
        BuilderName = 'T1 Wall Builder',
        PlatoonTemplate = 'EngineerGenericSingle',
        PlatoonAIPlan = 'EngineerBuildAI',
        Priority = 800,
        InstanceCount = 2,
        RequiresConstruction = false,
        PlatoonType = 'Any',
        BuildConditions = {
            { UCBC, 'HaveAreaWithUnitsFewWalls', { 'MAIN', 100, 5, 'STRUCTURE - WALL', false, false, false } },
        },
        PlatoonData = {
            Construction = {
                BuildStructures = { 'Wall' },
                LocationType = 'MAIN',
                Wall = true,
                MarkerRadius = 100,
                MarkerUnitCount = 5,
                MarkerUnitCategory = 'STRUCTURE - WALL',
            },
        },
    },

    
}

function AIAC(aiBrain, num)
    if aiBrain:AMGetNumberAttackForcePlatoons() > num then 
        return true
    end
    return false
end

#Shared area
function ExecutePlan( aiBrain )
    aiBrain:SetConstantEvaluate(false)
    if not aiBrain:PBMHasPlatoonList() then
        LOG('*AI DEBUG: ARMY ', repr(aiBrain:GetArmyIndex()), ': Initiating Archetype Medium')
        aiBrain:SetResourceSharing(true)
        aiBrain:PBMEnableRandomSamePriority()
        aiBrain:PBMSetCheckInterval(90)
        AIBuildUnits.AIExecutePlanUnitList(aiBrain, PlatoonList, PlatoonTemplates[aiBrain:GetFactionIndex()])
    end
    if not aiBrain:AMIsAttackManagerActive() then
        local spec = {
            AttackCheckInterval = 300,
            AttackConditions = {
                    {AIAC, {2}}
                },
        }
        aiBrain:InitializeAttackManager(spec)
    end
    if not aiBrain.BaseMonitor then
        aiBrain:BaseMonitorInitialization()
        local plat = aiBrain:GetPlatoonUniquelyNamed('ArmyPool')
        plat:ForkThread( plat.PoolDistressAI )
    end
    if not aiBrain.EnemyPickerThread then
        aiBrain.EnemyPickerThread = aiBrain:ForkThread( aiBrain.PickEnemy )
    end
#    local econ = AIUtils.AIGetEconomyNumbers(aiBrain)
#    aiBrain:GiveT1Resource('Energy', econ.EnergyMaxStored * 0.2)
#    aiBrain:GiveT1Resource('Mass', econ.MassMaxStored * 0.2)
end
