#****************************************************************************
#**
#**  File     :  /cdimage/lua/modules/AI/aiattackutilities.lua
#**  Author(s): John Comes, Dru Staltman
#**
#**  Summary  :
#**
#**  Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************
local BuildingTemplates = import('/lua/modules/basetemplates.lua').BuildingTemplates
local UnitTemplates = import('/lua/modules/unittemplates.lua').UnitTemplates
local ScenarioUtils = import('/lua/ScenarioUtilities.lua')
local AIUtils = import('/lua/modules/ai/aiutilities.lua')

function AIUseGates( platoon, gates, units, location )
    #Unit:HasValidTeleportDest()
    local aiBrain = platoon:GetBrain()
    if aiBrain:GetThreatAtPosition( location, 2, false ) < 0 then
        IssueClearCommands( gates )
        local numGates = table.getn(gates)
        local gatesSplit = {}
        for k,v in gates do
            table.insert( gatesSplit, {} )
        end
        if table.getn( gatesSplit ) == 0 then
            return false
        end
        local which = 1
        for k,v in units do
            table.insert( gatesSplit[which], v )
            which = which + 1
            if which > numGates then
                which = 1
            end
        end
        IssueTransportUnload( gates, location )
        for k,v in gates do
            IssueTransportLoad( gatesSplit[k], gates[k] )
        end
        local teleporting = true
        repeat
            WaitSeconds(2)
            if not platoon:GetBrain():PlatoonExists( platoon ) then
                return
            end
            teleporting = false
            for k,v in gates do
                if not v:IsDead() and v:IsUnitState('TransportLoading') then
                    teleporting = true
                    break
                end
            end
        until not teleporting
        return true
    else
        return false
    end
end

function AISquadFollowSquad( aiBrain, platoon, followers, targets, formation )
    local cmd = false
    followerPos = platoon:GetSquadPosition( followers )
    targetPos = platoon:GetSquadPosition( targets )
    if VDist2( followerPos[1], followerPos[3], targetPos[1], targetPos[3] ) > 5 then
        cmd = platoon:MoveToLocation( targetPos, false, followers )
        if targetPos[1] <= 0 or targetPos[3] <= 0 or targetPos[1] >= ScenarioInfo.size[1] or targetPos[3] >= ScenarioInfo.size[2] then
            LOG('*AI DEBUG: AISQUADFOLLOWSQUAD SENDING UNITS TO WRONG LOCATIONS')
        end
    end
    return cmd
end

function CheckSquads( platoon )
    local types = { 'Attack', 'Support', 'Artillery', 'Guard', 'Scout', 'Unassigned' }
    local retTable = {}
    for k,v in types do
        local units = platoon:GetSquadUnits(v)
        if not units then
            retTable[v]= false
        else
            local found = false
            for num, unit in units do
                if not unit:IsDead() then
                    found = true
                    break
                end
            end
            retTable[v] = found                
        end
    end
    return retTable
end

function AIFindStrongestInRoute( aiBrain, maxRange, atkPri, route, enemy )
    local radius = 16
    local targetPoint = false
    local pointStrength = false
    if route and enemy then
        for num, cat in atkPri do
            local catCheck = ParseEntityCategory(cat)
            for k,v in route do
                local friendNum = table.getn(AIUtils.GetOwnUnitsAroundPoint( aiBrain, categories.MOBILE, v, radius ))
                if friendNum < 2 then
                    local units = AIUtils.GetBrainUnitsAroundPoint( aiBrain, 'Enemy', catCheck, v, radius, enemy )
                    if units and ( not targetPoint or table.getn(units) < pointStrength ) then
                        for uNum, unit in units do
                            if not unit:IsDead() then
                                targetPoint = unit
                                pointStrength = table.getn(units)
                                break
                            end
                        end
                    end
                end
            end
            if targetPoint then
                return targetPoint
            end
        end
    end
    return false
end

function AIFindUnitRadiusThreat( aiBrain, alliance, priTable, position, radius, tMin, tMax, tRing )
    local catTable = {}
    local unitTable = {}
    for k,v in priTable do
        table.insert( catTable, ParseEntityCategory(v) )
        table.insert( unitTable, {} )
    end
    
    local units = aiBrain:GetUnitsAroundPoint( alliance, categories.ALLUNITS, position, radius )
    for num, unit in units do
        for tNum, catType in catTable do
            if EntityCategoryContains( catType, unit ) then
                table.insert( unitTable[tNum], unit )
                break
            end
        end
    end
    
    local checkThreat = false
    if tMin and tMax and tRing then
        checkThreat = true
    end
    
    local distance = false
    local retUnit = false
    for tNum, catList in unitTable do
        for num, unit in catList do
            if not unit:IsDead() then
                local unitPos = unit:GetPosition()
                local useUnit = true
                if checkThreat then
                    local threat = aiBrain:GetThreatAtPosition( unitPos, tRing, false )
                    if not ( threat >= tMin and threat <= tMax ) then
                        useUnit = false
                    end
                end
                if useUnit then
                    local tempDist = VDist2( unitPos[1], unitPos[3], position[1], position[3] )
                    if tempDist < radius and ( not distance or tempDist < distance ) then
                        distance = tempDist
                        retUnit = unit
                    end
                end
            end
        end
        if retUnit then
            return retUnit
        end
    end
end

function AIGetAttackVecs( aiBrain, platoon, squad )
    local vecs = {}
    if aiBrain:GetCurrentEnemy() then
        aiBrain:SetUpAttackVectorsToArmy(categories.STRUCTURE - ( categories.MASSEXTRACTION ) )
        vecs.WeakestVector = BuildAttackVectors(aiBrain:PickBestAttackVector( platoon, squad, 'Enemy', 'LeastDefended', categories.STRUCTURE - ( categories.WALL + categories.DEFENSE ) ) )
        vecs.ClosestVector = BuildAttackVectors(aiBrain:PickBestAttackVector( platoon, squad, 'Enemy', 'Closest', categories.STRUCTURE - ( categories.WALL + categories.DEFENSE ) ) )
        vecs.FurthestVector = BuildAttackVectors(aiBrain:PickBestAttackVector( platoon, squad, 'Enemy', 'Furthest', categories.STRUCTURE - ( categories.WALL + categories.DEFENSE ) ) )
        vecs.HighestVector = BuildAttackVectors(aiBrain:PickBestAttackVector( platoon, squad, 'Enemy', 'HighestValue', categories.STRUCTURE - ( categories.WALL + categories.DEFENSE ) ) )
    else
        local tPos = (aiBrain:GetHighestThreatPosition( 1, false ))
        vecs.WeakestVector = { tPos, tPos }
        vecs.ClosestVector = { tPos, tPos }
        vecs.FurthestVector = { tPos, tPos }
        vecs.HighestVector = { tPos, tPos }
    end
    if not vecs.WeakestVector then
        aiBrain:ForceNewEnemy()
        local tPos = (aiBrain:GetHighestThreatPosition( 1, false ))
        vecs.WeakestVector = { tPos, tPos }
        vecs.ClosestVector = { tPos, tPos }
        vecs.FurthestVector = { tPos, tPos }
        vecs.HighestVector = { tPos, tPos }
    end
    return vecs
end

function AIPlatoonSquadAttackVector( aiBrain, platoon, squad, vecType, formation, gates )
    local cmd = false
    local position
    local squadName
    local units
    if squad == 'All' then
        position = platoon:GetPlatoonPosition()
        if platoon:GetSquadUnits('Attack') then
            squadName = 'Attack'
        elseif platoon:GetSquadUnits('Artillery') then
            squadName = 'Attack'
        elseif platoon:GetSquadUnits('Guard') then
            squadName = 'Guard'
        end
        squad = nil
        units = platoon:GetPlatoonUnits()
    else
        position = platoon:GetSquadPosition(squad)
        squadName = squad
        units = platoon:GetSquadUnits(squadName)
    end
    local vecs = AIGetAttackVecs( aiBrain, platoon, squadName )
    
    ### Check if transports are needed to cross obstacles
    local haveTransports = false
    local unit = false
    local unitType = 'Air'
    for k,v in units do
        if not v:IsDead() then
            local mType = v:GetBlueprint().Physics.MotionType
            if ( mType == 'RULEUMT_AmphibiousFloating' or mType == 'RULEUMT_Hover' or mType == 'RULEUMT_Amphibious' 
                or mType == 'RULEUMT_GiantHexaped' ) and unitType == 'Air' then
                unitType = 'Amphibious'
                unit = v
            elseif mType == 'RULEUMT_Water' and ( unitType == 'Air' or unitType == 'Amphibious' ) then
                unitType = 'Water'
                unit = v
            elseif mType == 'RULEUMT_Air' and unitType == 'Air' then
                unit = v
            elseif ( mType == 'RULEUMT_Biped' or mType == 'RULEUMT_Land' ) and unitType ~= 'Land' then
                unitType = 'Land'
                unit = v
            end
        end
    end
    if unit then
        local needed = false
        if not AIUtils.CheckUnitPathing( vecs[vecType][2], unit:GetPosition(), unit ) then
            needed = true
            local iTable = { 0, 1, -1, 2, -2, 3, -3 }
            local jTable = { 0, 1, -1, 2, -2, 3, -3 }
            for num,i in iTable do
                for jNum, j in jTable do
                    local checkSpot = { vecs[vecType][2][1] + ( i * 8.5 ), 0, vecs[vecType][2][3] + ( j * 8.5 ) }
                    if AIUtils.CheckUnitPathing(checkSpot, unit:GetPosition(), unit ) then
                        needed = false
                        vecs[vecType][2] = checkSpot
                        break
                    end
                end
                if not needed then
                    break
                end
            end
        end
        if needed then
            local counter = 0
            while not AIUtils.GetTransports(platoon) do
                counter = counter + 1
                #LOG('*AI DEBUG: ARMY ' .. aiBrain:GetArmyIndex() .. ': Attack force needs transport - waiting to be built: counter - ' .. (counter * 10) )
                if not aiBrain.NeedTransports then
                    aiBrain.NeedTransports = true
                    #LOG('*AI DEBUG: ARMY ' .. aiBrain:GetArmyIndex() .. ': transports needed - Num waiting= ' .. table.getn(units) )
                end
                WaitSeconds(10)
                if not aiBrain:PlatoonExists(platoon) then
                    return false
                end
            end
            #LOG('*AI DEBUG: ARMY ' .. aiBrain:GetArmyIndex() .. ': Attack force requiring has transports')
            aiBrain.NeedTransports = false
            haveTransports = true
        end
    end
    
    # Grab transports if we want them because its far away
    if not haveTransports and VDist2( position[1], position[3], vecs[vecType][2][1], vecs[vecType][2][3] ) > 350 then
        haveTransports = AIUtils.GetTransports(platoon)
    end
    
    # Generate path to base
    local path
    if haveTransports then
        path = AIGeneratePathToVector( aiBrain, position, 'Land', vecs[vecType][2], units, nil, 2 )
    else
        path = AIGeneratePathToVector( aiBrain, position, 'Land', vecs[vecType][2], units )
    end
    
    # ATTACK !!!!
    if path then
        aiBrain.LastAttackForceRoute = path
        local transportsUsed = false
        for i=table.getn(path),1,-1 do
            if i == table.getn(path) and haveTransports then
                if AIUtils.UseTransports( units, platoon:GetSquadUnits( 'Scout' ), path[i] ) then
                    transportsUsed = true
                else
                    if not aiBrain:PlatoonExists(platoon) then
                        return false
                    end
                    cmd = platoon:MoveToLocation( path[i], false )
                    while aiBrain:PlatoonExists(platoon) and platoon:IsCommandsActive(cmd) do
                        WaitSeconds(5)
                    end
                end 
            elseif squad then
                if squad == 'Artillery' then
                    cmd = platoon:AggressiveMoveToLocation( path[i], squad )
                else
                    cmd = platoon:MoveToLocation( path[i], false, squad )
                end
            else
                cmd = platoon:MoveToLocation( path[i], false )
            end
            if haveTransports and not transportsUsed then
                # Return to pool
                AIUtils.ReturnTransportsToPool( platoon:GetSquadUnits( 'Scout' ), false )
                haveTransports = false
            end
            if path[i][1] <= 0 or path[i][3] <= 0 or path[i][1] >= ScenarioInfo.size[1] or path[i][2] >= ScenarioInfo.size[2] then
                LOG('*AI DEBUG: AIPLATOONSQUADATTACKVECTOR SENDING UNITS TO WRONG LOCATION')
            end
        end
        if squad == 'Attack' then
            local lastSpot = path[1]
            cmd = platoon:MoveToLocation( { lastSpot[1] - 16, lastSpot[2], lastSpot[3] - 16}, false, squad )
            cmd = platoon:MoveToLocation( { lastSpot[1] + 16, lastSpot[2], lastSpot[3] - 16}, false, squad )
            cmd = platoon:MoveToLocation( { lastSpot[1] + 16, lastSpot[2], lastSpot[3] + 16}, false, squad )
            cmd = platoon:MoveToLocation( { lastSpot[1] - 16, lastSpot[2], lastSpot[3] + 16}, false, squad )
        end
    #else
        #LOG('*AI DEBUG: ERROR ERROR ERROR - NO PATH GENERATED - ERROR ERROR ERROR' )
    end
    return cmd
end

function SetGroupAttackPri( platoon, group, t1Table, t2Table, t3Table )
    local t1Pri = {}
    for k,v in t1Table do
        table.insert( t1Pri, ParseEntityCategory( v ) )
    end
    local t2Pri = {}
    for k,v in t2Table do
        table.insert( t2Pri, ParseEntityCategory( v ) )
    end
    local t3Pri = {}
    for k,v in t3Table do
        table.insert( t3Pri, ParseEntityCategory( v ) )
    end
    
    platoon:SetPrioritizedTargetList( group, t3Pri )

    local t1Str = 0
    local t2Str = 0
    local t3Str = 0
    local units = platoon:GetSquadUnits( group )
    for k,v in units do
        if EntityCategoryContains( categories.TECH3 + categories.EXPERIMENTAL, v ) then
            t3Str = t3Str + 1
        elseif EntityCategoryContains( categories.TECH2, v ) then
            t2Str = t2Str + 1
        else
            t1Str = t1Str + 1
        end
    end
    local t1Use = t1Pri
    local t2Use = t2Pri
    if t1Str > 50 then
        t1Use = t3Pri
    elseif t1Str > 20 then
        t1Use = t2Pri
    end
    if t2Str > 20 then
        t2Use = t3Pri
    end
    for k,v in units do
        if EntityCategoryContains( categories.TECH3 + categories.EXPERIMENTAL, v ) then
            v:SetTargetPriorities( t3Pri )
        elseif EntityCategoryContains( categories.TECH2, v ) then
            v:SetTargetPriorities( t2Use )
        else
            v:SetTargetPriorities( t1Use )
        end
    end
end

function BuildAttackVectors( vector )
    if vector then
        local height = GetTerrainHeight( vector.px + ( vector.vx ), vector.pz + ( vector.vz ) )
        if GetSurfaceHeight( vector.px + ( vector.vx ), vector.pz + ( vector.vz  )) > height then
            height = GetSurfaceHeight( vector.px + ( vector.vx ), vector.pz + ( vector.vz ) )
        end
        return { { vector.px, vector.py, vector.pz }, { vector.px + ( vector.vx ), height, vector.pz + ( vector.vz ) } }
    else
        return false
    end
#    if vector then
#        local height = GetTerrainHeight( vector.px + ( vector.vx * 2 ), vector.pz + ( vector.vz * 2 ) )
#        if GetSurfaceHeight( vector.px + ( vector.vx * 2 ), vector.pz + ( vector.vz * 2 )) > height then
#            height = GetSurfaceHeight( vector.px + ( vector.vx * 2 ), vector.pz + ( vector.vz * 2 ) )
#        end
#        return { { vector.px, vector.py, vector.pz }, { vector.px + ( vector.vx * 2 ), height, vector.pz + ( vector.vz * 2 ) } }
#    else
#        return false
#    end
end

function AIGeneratePathToVector( aiBrain, platPos, layer, vector, units, path, depth )
    if not vector or table.getn(units) <= 0 then
        return false
    end
    if not path then
        depth = 1
        path = {}
        table.insert( path, vector )
    end
    if not vector or depth > 6 then
        return path
    end
    local threatTable = {}
    local unit = false
    local unitType = 'Air'
    for k,v in units do
        if not v:IsDead() then
            local mType = v:GetBlueprint().Physics.MotionType
            if ( mType == 'RULEUMT_AmphibiousFloating' or mType == 'RULEUMT_Hover' or mType == 'RULEUMT_Amphibious' 
                or mType == 'RULEUMT_GiantHexaped' ) and unitType == 'Air' then
                unitType = 'Amphibious'
                unit = v
            elseif mType == 'RULEUMT_Water' and ( unitType == 'Air' or unitType == 'Amphibious' ) then
                unitType = 'Water'
                unit = v
            elseif mType == 'RULEUMT_Air' and unitType == 'Air' then
                unit = v
            elseif ( mType == 'RULEUMT_Biped' or mType == 'RULEUMT_Land' ) and unitType ~= 'Land' then
                unitType = 'Land'
                unit = v
            end
        end
    end
    if not unit then
        return path
    end
    for i=-1,1 do
        for j=-1,1 do
            if i ~= 0 or j~= 0 then
                local pos = { vector[1] + ( i * 32 ), 0, vector[3] + ( j * 32 ) }
                local height = GetTerrainHeight( pos[1], pos[3] )
                if GetSurfaceHeight( pos[1], pos[3] ) > height then
                    height = GetSurfaceHeight( pos[1], pos[3] )
                end
                pos = { pos[1], height, pos[3] }
                if AIUtils.CheckUnitPathing( pos, unit:GetPosition(), unit ) then
                    table.insert( threatTable, 
                        { 
                            Position = pos, 
                            Threat = aiBrain:GetThreatAtPosition( pos, 1, false )
                        }
                    )
                end
            end
        end
    end
    local lowest = false
    local pos = false
    local lastPos = path[ table.getn(path) ]
    for num, data in threatTable do
        if not ( data.Position[1] == lastPos[1] and data.Position[3] == lastPos[3] ) then
            if not lowest or data.Threat < lowest then
                lowest = data.Threat
                pos = data.Position
            elseif data.Threat == lowest then
                if VDist2( platPos[1], platPos[3], pos[1], pos[3] ) > VDist2( platPos[1], platPos[3], data.Position[1], data.Position[3] ) then
                    pos = data.Position
                end
            end
        end
    end
    if pos then
        table.insert( path, pos )
    end
    if not lowest or lowest < 0 then
        return path
    else
        return AIGeneratePathToVector( aiBrain, platPos, layer, pos, units, path, depth + 1 )
    end
end

function GetClosestUnitToPosition( aiBrain, category, position, radius, alliance )
    local units = aiBrain:GetUnitsAroundPoint( alliance, category, position, radius )
    local closest, distance
    if units then
        for k,v in units do
            local unitPos = v:GetPosition()
            local testDist = VDist2( position[1], position[3], unitPos[1], unitPos[3] )
            if not closest or testDist < distance then
                closest = v
                distance = testDist
            end
        end
    end
    return closest
end

