#****************************************************************************
#**
#**  File     :  /cdimage/lua/modules/AI/aibuildstructures.lua
#**  Author(s): John Comes
#**
#**  Summary  : Foundation script for all structure building in the AI.
#**
#**  Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************
local BaseTemplates = import('/lua/modules/basetemplates.lua').BaseTemplates
local BuildingTemplates = import('/lua/modules/basetemplates.lua').BuildingTemplates
local Utils = import('/lua/modules/utilities.lua')
local AIUtils = import('/lua/modules/ai/aiutilities.lua')
local StructureUpgradeTemplates = import('/lua/modules/upgradeTemplates.lua').StructureUpgradeTemplates
local UnitUpgradeTemplates = import('/lua/modules/upgradeTemplates.lua').UnitUpgradeTemplates
local RebuildStructuresTemplate = import('/lua/modules/basetemplates.lua').RebuildStructuresTemplate
local ScenarioUtils = import('/lua/ScenarioUtilities.lua')
local aiEconomy
local allowedEnergyStorageRatio = 0.7
local allowedMassStorageRatio = 0.6

function AISetEconomyNumbers(aiBrain)
    #LOG('*AI DEBUG: SETTING ECONOMY NUMBERS FROM AIBRAIN ', repr(aiBrain))
    local econ = AIUtils.AIGetEconomyNumbers(aiBrain)
    aiEconomy = econ
end

function AIModEconomyNumbers(aiBrain, unitBP)
    #LOG('*AI DEBUG: MODDING ECON NUMBERS, BRAIN = ', repr(aiBrain), ' UNITBP = ', repr(unitBP))
    #LOG('*AI DEBUG: MODDING ECON NUMBERS, ENERGYTREND BEFORE = ', repr(aiEconomy.EnergyTrend))
    #LOG('*AI DEBUG: MODDING ECON NUMBERS, ENERGY USE OF UNIT = ', repr(aiBrain:GetUnitBlueprint(unitBP).Economy.ActiveConsumptionPerSecondEnergy * 0.1))
    aiEconomy.MassTrend = aiEconomy.MassTrend - (aiBrain:GetUnitBlueprint(unitBP).Economy.ActiveConsumptionPerSecondMass or 0) * 0.1
    aiEconomy.MassIncome = aiEconomy.MassIncome - (aiBrain:GetUnitBlueprint(unitBP).Economy.ActiveConsumptionPerSecondMass or 0) * 0.1
    aiEconomy.EnergyTrend = aiEconomy.EnergyTrend - (aiBrain:GetUnitBlueprint(unitBP).Economy.ActiveConsumptionPerSecondEnergy or 0) * 0.1
    aiEconomy.EnergyIncome = aiEconomy.EnergyIncome - (aiBrain:GetUnitBlueprint(unitBP).Economy.ActiveConsumptionPerSecondEnergy or 0) * 0.1
    #LOG('*AI DEBUG: MODDING ECON NUMBERS, ENERGYTREND AFTER = ', repr(aiEconomy.EnergyTrend * 0.1))
end

function AIGetBuilder(aiBrain, techLevel)
    local builderTechLevel = categories.TECH1 * categories.ENGINEER
    if techLevel == 2 then
        builderTechLevel = categories.TECH2 * categories.ENGINEER
    elseif techLevel == 3 then
        builderTechLevel = categories.TECH3 * categories.ENGINEER
    elseif techLevel == 0 then
        builderTechLevel = categories.COMMAND
    end
    #LOG('*AI DEBUG: AIGETBUILDER FINDING TECH LEVEL ', repr(techLevel))
    local builder = aiBrain:FindUnit(builderTechLevel, true)
    #LOG('*AI DEBUG: AIGETBUILDER RETURNING ', repr(builder))
    return builder
end

function AIGetAnyBuilder(aiBrain, techLevel, inclCDR)
    local builder = AIGetBuilder(aiBrain, techLevel)
    if not builder then
       builder = AIGetBuilder(aiBrain, 3)
    end
    if not builder then
       builder = AIGetBuilder(aiBrain, 2)
    end
    if not builder then
       builder = AIGetBuilder(aiBrain, 1)
    end
    if not builder and inclCDR then
        builder = AIGetBuilder(aiBrain, 0)
    end
    return builder
end

#THIS IS THE FUNCTION THAT ACTUALLY TELLS THE BUILDER TO BUILD A BUILDING.
function AIExecuteBuildStructure( aiBrain, builder, buildingType , closeToBuilder, relative, buildingTemplate, baseTemplate, reference)
    local factionIndex = aiBrain:GetFactionIndex()
    local whatToBuild = aiBrain:DecideWhatToBuild( builder, buildingType, buildingTemplate)
    if whatToBuild then
        if buildingType != 'Resource' and buildingType != 'T1HydroCarbon' and
            buildingType != 'T1Resource' and buildingType != 'T2Resource' and buildingType != 'T3Resource' then
            local location = aiBrain:FindPlaceToBuild(buildingType, whatToBuild, baseTemplate, relative, closeToBuilder)
            if location then
                aiBrain:BuildStructure( builder, whatToBuild, location, relative )
                if not relative then
                    return true, location
                else
                    return true, false
                end
            end
        else
            local markerType = 'Mass'
            if buildingType == 'T1HydroCarbon' then
                markerType = 'Hydrocarbon'
            end
            local markers 
            if markerType ~= 'Hydrocarbon' then
                markers = AIUtils.AIGetSortedMassLocations(aiBrain, 100, false, false, false, false, builder)
            else
                markers = AIUtils.AIGetSortedHydrocarbonLocation(aiBrain, 100, false, false, false, false, builder)
            end
            local location = false
            for k,v in markers do
                if aiBrain:GetNumUnitsAroundPoint( 'Ally', categories.MASSEXTRACTION + categories.HYDROCARBON, v, 2 ) == 0 then
                    if aiBrain:CanBuildStructureAt( whatToBuild, v ) then
                        location = v
                        break
                    end
                end
            end
            if location then
                aiBrain:BuildStructure( builder, whatToBuild, { location[1], location[3], 0 }, false )
                return true, location
            else
                return false, location
            end
        end
    end
    return false, false
end

function AIBuildBaseTemplateOrdered( aiBrain, builder, buildingType , closeToBuilder, relative, buildingTemplate, baseTemplate, reference)
    local factionIndex = aiBrain:GetFactionIndex()
    local whatToBuild = aiBrain:DecideWhatToBuild( builder, buildingType, buildingTemplate)
    if whatToBuild then
        if buildingType == 'Resource' or buildingType == 'T1Resource' or buildingType == 'T2Resource' or buildingType == 'T3Resource' or buildingType == 'T1HydroCarbon' then
            return AIExecuteBuildStructure( aiBrain, builder, buildingType , closeToBuilder, relative, buildingTemplate, baseTemplate, reference)
        else
            for l,bType in baseTemplate do
                for m,bString in bType[1] do
                    if bString == buildingType then
                        for n,position in bType do
                            if n > 1 then
                                if aiBrain:CanBuildStructureAt( whatToBuild, { position[1], 0, position[2] } ) then
                                    aiBrain:BuildStructure( builder, whatToBuild, position, false )
                                    return true, { position[1], 0, position[2] }
                                end
                            end
                        end
                        break
                    end
                end
            end
        end
    end
    return false, false
end

function AIBuildBaseTemplateFromLocation( baseTemplate, location )
    local baseT = {}
    if location and baseTemplate then
        for templateNum, template in baseTemplate do
            baseT[templateNum] = {}
            for rowNum,rowData in template do # rowNum, rowData in template do
                if type(rowData[1]) == 'number' then
                    baseT[templateNum][rowNum] = {}
                    baseT[templateNum][rowNum][1] = math.floor( rowData[1] + location[1] ) + 0.5
                    baseT[templateNum][rowNum][2] = math.floor( rowData[2] + location[3] ) + 0.5
                    baseT[templateNum][rowNum][3] = 0
                else
                    baseT[templateNum][rowNum] = template[rowNum]
                end
            end
        end
    end
    return baseT
end

function AIBuildAdjacency( aiBrain, builder, buildingType , closeToBuilder, relative, buildingTemplate, baseTemplate, reference)
    local factionIndex = aiBrain:GetFactionIndex()
    local whatToBuild = aiBrain:DecideWhatToBuild( builder, buildingType, buildingTemplate )
    if whatToBuild then
        local upperString = ParseEntityCategory( string.upper(whatToBuild) )
        local unitSize = aiBrain:GetUnitBlueprint( whatToBuild ).Physics
        local template = {}
        table.insert( template, {} )
        table.insert( template[1], { buildingType } )
        for k,v in reference do
            if not v:BeenDestroyed() then
                local targetSize = v:GetBlueprint().Physics
                local targetPos = v:GetPosition()
                targetPos[1] = targetPos[1] - (targetSize.SkirtSizeX/2)
                targetPos[3] = targetPos[3] - (targetSize.SkirtSizeZ/2)
                # Top/bottom of unit
                for i=0,((targetSize.SkirtSizeX/2)-1) do
                    local testPos = { targetPos[1] + 1 + (i * 2), targetPos[3]-(unitSize.SkirtSizeZ/2), 0 }
                    local testPos2 = { targetPos[1] + 1 + (i * 2), targetPos[3]+targetSize.SkirtSizeZ+(unitSize.SkirtSizeZ/2), 0 }
                    table.insert( template[1], testPos )
                    table.insert( template[1], testPos2 )
                end
                # Sides of unit
                for i=0,((targetSize.SkirtSizeZ/2)-1) do
                    local testPos = { targetPos[1]+targetSize.SkirtSizeX + (unitSize.SkirtSizeX/2), targetPos[3] + 1 + (i * 2), 0 }
                    local testPos2 = { targetPos[1]-(unitSize.SkirtSizeX/2), targetPos[3] + 1 + (i*2), 0 }
                    table.insert( template[1], testPos )
                    table.insert( template[1], testPos2 )
                end
            end
        end
        local location = aiBrain:FindPlaceToBuild(buildingType, whatToBuild, template, false, builder)
        if location then
            aiBrain:BuildStructure(builder, whatToBuild, location, false)
            return true, { location[1], 0, location[2] }
            ## Build in a regular spot if adjacency not found
        else
            local retVal, pos = AIExecuteBuildStructure( aiBrain, builder, buildingType, builder, true,  buildingTemplate, baseTemplate )
            return retVal, pos
        end
    end
    return false, false
end

function AINewExpansionBase( aiBrain, baseName, position, radius, expansionTypes )
    # Check if it already exists
    for k,v in aiBrain.PBM.Locations do
        if v.LocationType == baseName then
            return
        end
    end
    # Figure out what type of builders to import
    if not expansionTypes then
        expansionTypes = { 'Air', 'Land', 'Sea', 'Gate' }
    end

    aiBrain:PBMAddBuildLocation( position, radius, baseName, true )
    
    for num, typeString in expansionTypes do
        for bNum, builder in aiBrain.PBM.Platoons[typeString] do
            if builder.LocationType == 'MAIN' and CheckExpansionType( typeString, ScenarioInfo.BuilderTable[typeString][builder.BuilderName].ExpansionExclude )  then
                local pltnTable = {}
                for dField, data in builder do
                    if dField == 'LocationType' then
                        pltnTable[dField] = baseName
                    elseif dField == 'PlatoonHandle' then
                        pltnTable[dField] = false
                    elseif dField == 'PlatoonTimeOutThread' then
                        pltnTable[dField] = nil
                    else
                        pltnTable[dField] = data
                    end
                end
                table.insert( aiBrain.PBM.Platoons[typeString], pltnTable )
                aiBrain.PBM.NeedSort[typeString] = true
            end
        end
    end
end

function CheckExpansionType( typeString, typeTable )
    if not typeTable then
        return true
    elseif type(typeTable) == 'table' then
        for k,v in typeTable do
            if v == typeString then
                return false
            end
        end
    end
    return true
end

function FindNearestIntegers( pos )
    local x = math.floor( pos[1] )
    local z = math.floor( pos[3] )
    return { x, z }
end
    

function WallBuilder( aiBrain, builder, buildingType , closeToBuilder, relative, buildingTemplate, baseTemplate, reference )
    if not reference then
        return false
    end
    local points = BuildWallsAtLocation( aiBrain, reference )
    if not points then
        return false
    end
    local i = 2
    while i <= table.getn( points ) do
        local point1 = FindNearestIntegers( points[i-1] )
        local point2 = FindNearestIntegers( points[i] )
        #### Horizontal line
        local buildTable = {}
        if point1[2] == point2[2] then
            local xDir = -1
            if point1[1] < point2[1] then
                xDir = 1
            end
            for j = 1, math.abs( point1[1] - point2[1] ) do
                table.insert( buildTable, { point1[1] + ( j * xDir ) + .5, point1[2] + .5, 0 } )
            end
        #### Vertical line
        elseif point1[1] == point2[1] then
            local yDir = -1
            if point1[2] < point2[2] then
                yDir = 1
            end
            for j = 1, math.abs( point1[2] - point2[2] ) do
                table.insert( buildTable, { point1[1] + .5, point1[2] + ( j * yDir ) + .5, 0 } )
            end
        #### Angled line
        else
            local angle = ( point1[1] - point2[1] ) / ( point1[2] - point2[2] )
            if angle == 0 then
                angle = 1
            end
            
            local xDir = -1
            if point1[1] < point2[1] then
                xDir = 1
            end
            for j=1,math.abs( point1[1] - point2[1] ) do
                table.insert( buildTable, { point1[1] + ( j * xDir ) - .5, ( point1[2] + math.floor( ( angle * xDir ) * (j-1) ) + .5 ), 0 } )
            end
        end
        local faction = aiBrain:GetFactionIndex()
        local whatToBuild = aiBrain:DecideWhatToBuild( builder, buildingType, buildingTemplate )
        for k,v in buildTable do
            if aiBrain:CanBuildStructureAt( whatToBuild, { v[1], 0, v[2] } ) then
                aiBrain:BuildStructure( builder, whatToBuild, v, false )
            end
        end
        i = i + 1
    end
    return true, false
end

function GetBuildingDirection( main, loc )
    local distance = Utils.XZDistanceTwoVectors( main, loc )
    local cutoff = distance / 2
    local direction = {}
    if math.abs(loc[1] - main[1]) > cutoff then
        if loc[1] > main[1] then
            direction[1] = 1
        else
            direction[1] = -1
        end
    else
        direction[1] = 0
    end
    if math.abs(loc[3] - main[3]) > cutoff then
        if loc[3] > main[3] then
            direction[2] = 1
        else
            direction[2] = -1
        end
    else
        direction[2] = 0
    end
    local points = {}
    if direction[1] ~= 0 and direction[2] ~= 0 then
        table.insert( points, { loc[1], loc[2], loc[3] + ( direction[2] * 16 ) } )
        table.insert( points, { loc[1] + ( direction[1] * 16 ), loc[2], loc[3] + ( direction[2] * 16 ) } )
        table.insert( points, { loc[1] + ( direction[1] * 16 ), loc[2], loc[3] } )
    elseif direction[1] ~= 0 then
        table.insert( points, { loc[1], loc[2], loc[3] + 16 } )
        table.insert( points, { loc[1] + ( direction[1] * 16 ), loc[2], loc[3] } )
        table.insert( points, { loc[1], loc[2], loc[3] - 16 } )
    else
        table.insert( points, { loc[1] + 16, loc[2], loc[3] } )
        table.insert( points, { loc[1], loc[2], loc[3] + ( direction[1] * 16 ) } )
        table.insert( points, { loc[1] - 16, loc[2], loc[3] } )
    end
    return points
end

function BuildWallsAtLocation( aiBrain, location )
    local mainPos = aiBrain:PBMGetLocationCoords( 'MAIN' )    
    return GetBuildingDirection( mainPos, location )    
end





### OPERATION STUFF BELOW ###

### Takes a group of <name> from <army> in the save file.
### Populates a base template from this save file.
### Stores the template naming it <name> in the brain.
function CreateBuildingTemplate( brain, army, name )
    local list = {}
    local template = {}
    local tblUnit = ScenarioUtils.AssembleArmyGroup( army, name )
    local factionIndex = brain:GetFactionIndex()
    if not tblUnit then
        LOG('*ERROR AIBUILDSTRUCTURES - Group: ', repr(name), ' not found for Army: ', repr(army))
    else
        for i,unit in tblUnit do
            for k, unitId in RebuildStructuresTemplate[factionIndex] do
                if unit.type == unitId[1] then
                    unit.buildtype = unitId[2]
                    break
                end
            end
            if not unit.buildtype then
                unit.buildtype = unit.type
            end
        end
        for i, unit in tblUnit do
            for j,buildList in BuildingTemplates[factionIndex] do
                local unitPos = { unit.Position[1], unit.Position[3], 0 }
                if unit.buildtype == buildList[2] and buildList[1] ~= 'T3Sonar' then
                    local inserted = false
                    for k,section in template do
                        if section[1][1] == buildList[1] then
                            table.insert( section, unitPos )
                            list[unit.buildtype].AmountWanted = list[unit.buildtype].AmountWanted + 1
                            inserted = true
                            break
                        end
                    end
                    if not inserted then
                        table.insert( template, { {buildList[1]}, unitPos } )
                        list[unit.buildtype] =  { StructureType = buildList[1], StructureCategory = unit.buildtype, AmountNeeded = 0, AmountWanted = 1, CloseToBuilder = nil }
                    end
                    break
                end
            end
        end
        brain.BaseTemplates[name] = { Template=template, List=list }
    end
end

### Takes a group of <name> from <army> in the save file.
### Populates a base template from this save file.
### Appends the template named <templateName> in the brain with new data
function AppendBuildingTemplate( brain, army, name, templateName )
    local tblUnit = ScenarioUtils.AssembleArmyGroup( army, name )
    local factionIndex = brain:GetFactionIndex()
    if not brain.BaseTemplates[templateName] then
        error('*AI BUILD STRUCTURES: Invalid template name to append- ' .. templateName, 2)
    end
    local template = brain.BaseTemplates[templateName].Template
    local list = brain.BaseTemplates[templateName].List
    if not tblUnit then
        LOG('*ERROR AIBUILDSTRUCTURES - Group: ', repr(name), ' not found for Army: ', repr(army))
    else
        # Convert building to the proper type to be built if needed (ex: T2 and T3 factories to T1)
        for i,unit in tblUnit do
            for k, unitId in RebuildStructuresTemplate[factionIndex] do
                if unit.type == unitId[1] then
                    unit.buildtype = unitId[2]
                    break
                end
            end
            if not unit.buildtype then
                unit.buildtype = unit.type
            end
        end
        for i, unit in tblUnit do
            for j,buildList in BuildingTemplates[factionIndex] do # buildList[1] is type ("T1LandFactory"); buildList[2] is unitId (ueb0101)
                local unitPos = { unit.Position[1], unit.Position[3], 0 }
                if unit.buildtype == buildList[2] and buildList[1] ~= 'T3Sonar' then # if unit to be built is the same id as the buildList unit it needs to be added
                    local inserted = false
                    for k,section in template do # check each section of the template for the right type
                        if section[1][1] == buildList[1] then
                            table.insert( section, unitPos ) # add position of new unit if found
                            list[unit.buildtype].AmountWanted = list[unit.buildtype].AmountWanted + 1 # increment num wanted if found
                            inserted = true
                            break
                        end
                    end
                    if not inserted then # if section doesn't exist create new one
                        table.insert( template, { {buildList[1]}, unitPos } ) # add new build type to list with new unit
                        list[unit.buildtype] =  { StructureType = buildList[1], StructureCategory = unit.buildtype, AmountNeeded = 0, AmountWanted = 1, CloseToBuilder = nil } # add new section of build list with new unit type information
                    end
                    break
                end
            end
        end
    end
end

function StructureCheck( unitName )
    local bp = ArmyBrains[1]:GetUnitBlueprint(unitName)
    for k,v in bp.Categories do
        if v == 'BUILTBYTIER1ENGINEER' then
            return true
        elseif v == 'BUILTBYTIER2ENGINEER' then
            return true
        elseif v == 'BUILTBYTIER3ENGINEER' then
            return true
        end
    end
    return false
end

function AIMaintainBuildList(aiBrain, builder, buildingTemplate, brainBaseTemplate)
    if not buildingTemplate then
        buildingTemplate = BuildingTemplates[aiBrain:GetFactionIndex()]
    end
    for k,v in brainBaseTemplate.List do
        if builder:CanBuild( v.StructureCategory ) then
            if v.StructureType == 'Resource' or v.StructureType == 'T1HydroCarbon' or v.StructureType == 'T1Resource' 
                or v.StructureType == 'T2Resource' or v.StructureType == 'T3Resource' then
                for l,type in brainBaseTemplate.Template do
                    if type[1][1] == v.StructureType then
                        for m,location in type do
                            if m > 1 then
                                if aiBrain:CanBuildStructureAt(v.StructureCategory, {location[1], 0, location[2]}) then
                                    IssueStop({builder})
                                    IssueClearCommands({builder})
                                    aiBrain:BuildStructure(builder, v.StructureCategory, location, false)
                                    return true
                                end
                            end
                        end
                    end
                end
            elseif aiBrain:FindPlaceToBuild( v.StructureType, v.StructureCategory,  brainBaseTemplate.Template, false, v.CloseToBuilder ) then
                IssueStop({builder})
                IssueClearCommands({builder})
                if AIExecuteBuildStructure(aiBrain, builder, v.StructureType , v.CloseToBuilder, false, buildingTemplate, brainBaseTemplate.Template) then
                    return true
                end
            end
        end
    end
    return false
end
