#****************************************************************************
#**
#**  File     :  /cdimage/lua/modules/AI/aiutilities.lua
#**  Author(s): John Comes, Dru Staltman
#**
#**  Summary  :
#**
#**  Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************
local BuildingTemplates = import('/lua/modules/basetemplates.lua').BuildingTemplates
local UnitTemplates = import('/lua/modules/unittemplates.lua').UnitTemplates
local ScenarioUtils = import('/lua/ScenarioUtilities.lua')
local Utils = import('/lua/modules/utilities.lua')
local AIAttackUtils = import('/lua/modules/AI/aiattackutilities.lua')

function AIGetEconomyNumbers(aiBrain)
    #LOG('*AI DEBUG: RETURNING ECONOMY NUMBERS FROM AIBRAIN ', repr(aiBrain))
    local econ = {}
    econ.MassTrend = aiBrain:GetEconomyTrend('MASS')
    econ.EnergyTrend = aiBrain:GetEconomyTrend('ENERGY')
    econ.MassStorageRatio = aiBrain:GetEconomyStoredRatio('MASS')
    econ.EnergyStorageRatio = aiBrain:GetEconomyStoredRatio('ENERGY')
    econ.EnergyIncome = aiBrain:GetEconomyIncome('ENERGY')
    econ.MassIncome = aiBrain:GetEconomyIncome('MASS')
    econ.EnergyStorage = aiBrain:GetEconomyStored('ENERGY')
    econ.MassStorage = aiBrain:GetEconomyStored('MASS')
    if econ.MassStorageRatio != 0 then
        econ.MassMaxStored = econ.MassStorage / econ.MassStorageRatio
    else
        econ.MassMaxStored = econ.MassStorage
    end
    if econ.EnergyStorageRatio != 0 then
        econ.EnergyMaxStored = econ.EnergyStorage / econ.EnergyStorageRatio
    else
        econ.EnergyMaxStored = econ.EnergyStorage
    end
    return econ
end

function AIGetStructureUnitId(aiBrain, structureType)
    local unitId
    for k, v in BuildingTemplates[aiBrain:GetFactionIndex()] do
        if v[1] == structureType then
            unitId = v[2]
            break
        end
    end
    return unitId
end

function AIGetMobileUnitId(aiBrain, unitType)
    local unitId
    for k, v in UnitTemplates[aiBrain:GetFactionIndex()] do
        if v[1] == unitType then
            unitId = v[2]
            break
        end
    end
    return unitId
end

function AIGetSortedScoutingLocations(aiBrain, maxNum)
    local markerList = AIGetMarkerLocations(aiBrain, 'Combat Zone')
    for i = 1, table.getn(ArmyBrains) do
        local tmpLoc = ScenarioUtils.GetMarker('ARMY_' .. i)
        if tmpLoc then
            table.insert(markerList, { Position = tmpLoc.position, Name = 'ARMY_' .. i } )
        end
    end
    local markers = AISortMarkersFromStartPos(aiBrain, markerList, maxNum or 1000)
    local retMarkers = {}
    local numMarkers = table.getn( markers )
    for i = 1, numMarkers do
        rand = Random(1, numMarkers + 1 - i)
        table.insert(retMarkers, markers[rand])
        table.remove(markers, rand)
    end
    return retMarkers
end


function AIGetSortedDefensiveLocations(aiBrain, maxNum)
    local markerList = AIGetMarkerLocations(aiBrain, 'Defensive Point')
    return AISortMarkersFromStartPos(aiBrain, markerList, maxNum or 1000)
end

function AIGetSortedMassLocations(aiBrain, maxNum, tMin, tMax, tRings, platoon, unit)
    local markerList = AIGetMarkerLocations(aiBrain, 'Mass')
    return AISortMarkersFromLastPos(aiBrain, markerList, maxNum, tMin, tMax, tRings, platoon, unit)
end

function AIGetSortedHydrocarbonLocation(aiBrain, maxNum, tMin, tMax, tRings, platoon, unit)
    local markerList = AIGetMarkerLocations(aiBrain, 'Hydrocarbon')
    return AISortMarkersFromLastPos(aiBrain, markerList, maxNum, tMin, tMax, tRings, platoon, unit)
end

function AIGetSortedNavalLocations(aiBrain, maxNum)
    local markerList = AIGetMarkerLocations(aiBrain, 'Naval Area')
    return AISortMarkersFromLastPos(aiBrain, markerList, maxNum or 1000)
end

function AISortMarkersFromStartPos(aiBrain, markerList, maxNumber, tMin, tMax, tRings, platoon)
    local threatCheck = false
    if tMin and tMax and tRings then
        threatCheck = true
    end
    local startPosX, startPosZ = aiBrain:GetArmyStartPos()
    if platoon then
        startPosX = platoon:GetPlatoonPosition()[1]
        startPosZ = platoon:GetPlatoonPosition()[3]
    end
    #Simple selection sort, this can be made faster later if we decide we need it.
    local sortedMarkerList = {}
    if table.getn(markerList) == 0 then return {} end
    local num = table.getsize(markerList)
    if maxNumber < num then
        num = maxNumber
    end
    #LOG('*AI DEBUG: ARMY 2: LINE 112 of aiutilities.lua, NUM =', repr(num))
    for i = 1, num do
        local lowest = nil
        local czX, czZ, distance, key, pos
        for k, v in markerList do
            if v.Position then
                local x = v.Position[1]
                local z = v.Position[3]
                distance = VDist2(startPosX, startPosZ, x, z)
                local threat
                if threatCheck then
                    threat = aiBrain:GetThreatAtPosition( v.Position, tRings, false )
                end
                if not lowest or distance < lowest and ( not threatCheck or ( threat >= tMin and threat <= tMax ) ) then
                    pos = v.Position
                    lowest = distance
                    key = k
                end
            else
                LOG('*DEBUG INVALID MARKER:')
            end
        end
        if pos then
            sortedMarkerList[i] = pos
            table.remove(markerList, key)
        end
    end
    return sortedMarkerList
end


function AISortMarkersFromLastPos(aiBrain, markerList, maxNumber, tMin, tMax, tRings, platoon, unit)
    local threatCheck = false
    if tMin and tMax and tRings then
        threatCheck = true
    end
    local startPosX, startPosZ = aiBrain:GetArmyStartPos()
    if platoon then
        startPosX = platoon:GetPlatoonPosition()[1]
        startPosZ = platoon:GetPlatoonPosition()[3]
    end
    if unit then
        startPosX = unit:GetPosition()[1]
        startPosZ = unit:GetPosition()[3]
    end
    #Simple selection sort, this can be made faster later if we decide we need it.
    local sortedMarkerList = {}
    local lastX = startPosX
    local lastZ = startPosZ
    if table.getsize(markerList) == 0 then return {} end
    local num = table.getsize(markerList)
    if maxNumber < num then
        num = maxNumber
    end
    for i = 1, num do
        local threat
        local lowest
        local czX, czZ, pos, distance, key
        for k, v in markerList do
            local x = v.Position[1]
            local z = v.Position[3]
            distance = VDist2(lastX, lastZ, x, z)
            if threatCheck then
                threat = aiBrain:GetThreatAtPosition( v.Position, tRings, false )
            end
            if ( not lowest or distance < lowest ) and ( not threatCheck or ( threat >= tMin and threat <= tMax ) ) then
                pos = v.Position
                lowest = distance
                key = k
            end
        end
        if pos then
            sortedMarkerList[i] = pos
            lastX = pos[1]
            lastZ = pos[3]
            table.remove(markerList, key)
        end
    end
    return sortedMarkerList
end


function AIGetMarkerLocations(aiBrain, markerType)
    #LOG('*AI DEBUG: ARMY 2: Getting Marker Locations of Type ', markerType)
    local markers = ScenarioUtils.GetMarkers()
    local markerList = {}
    #Make a list of all the markers in the scenario that are of the markerType
    if markers then
        for k, v in markers do
            if v.type == markerType then
                table.insert(markerList, { Position = v.position, Name = k } )
            end
        end
    end
    return markerList
end

function AIGetMarkerPositionsAroundLocation( aiBrain, markerType, pos, radius, threatMin, threatMax, threatRings )
    local markers = AIGetMarkersAroundLocation( aiBrain, markerType, pos, radius, threatMin, threatMax, threatRings )
    local retMarkers = {}
    for k,v in markers do
        table.insert( markers, v.Position )
    end
    return retMarkers
end

function AIGetMarkersAroundLocation( aiBrain, markerType, pos, radius, threatMin, threatMax, threatRings )
    local markers = AIGetMarkerLocations( aiBrain, markerType )
    local returnMarkers = {}
    for k,v in markers do
        local dist = VDist2( pos[1], pos[3], v.Position[1], v.Position[3] )
        if dist < radius then
            if not threatMin then
                table.insert( returnMarkers, v )
            else
                local threat = aiBrain:GetThreatAtPosition( v.Position, threatRings, false )
                if threat >= threatMin and threat <= threatMax then
                    table.insert( returnMarkers, v )
                end
            end
        end
    end
    return returnMarkers
end

function AIGetMarkerLeastUnits( aiBrain, markerType, markerRadius, pos, posRad, unitCount, unitCat, tMin, tMax, tRings)
    local markers = AIGetMarkersAroundLocation( aiBrain, markerType, pos, posRad, tMin, tMax, tRings )
    local lowest
    local retPos = false
    local retName = false
    for k,v in markers do
        local numUnits = table.getn( GetOwnUnitsAroundPoint( aiBrain, unitCat, v.Position, markerRadius, tMin, tMax, tRings ) )
        if not retPos or numUnits < lowest then
            lowest = numUnits
            retPos = v.Position
            retName = v.Name
        end
    end
    return retPos, retName
end

function AIGetMarkerMostUnits( aiBrain, markerType, markerRadius, pos, posRad, unitCount, unitCat, tMin, tMax, tRings)
    local markers = AIGetMarkersAroundLocation( aiBrain, markerType, pos, posRad, tMin, tMax, tRings )
    local lowest
    local retPos = false
    local retName = false
    for k,v in markers do
        local numUnits = table.getn( GetOwnUnitsAroundPoint( aiBrain, unitCat, v.Position, markerRadius, tMin, tMax, tRings ) )
        if not retPos or numUnits > lowest then
            lowest = numUnits
            retPos = v.Position
            retName = v.Name
        end
    end
    return retPos, retName
end

function AIGetClosestMarkerLocation(aiBrain, markerType, startX, startZ, extraTypes)
    local markerList = AIGetMarkerLocations(aiBrain, markerType)
    if extraTypes then
        for num, pType in extraTypes do
            local moreMarkers = AIGetMarkerLocations(aiBrain, pType)
            if table.getn(moreMarkers) > 0 then
                for k,v in moreMarkers do
                    table.insert(markerList, { Position = v.Position, Name = v.Name } )
                end
            end
        end
    end
    local loc, distance, lowest, name = nil
    for k, v in markerList do
        local x = v.Position[1]
        local y = v.Position[2]
        local z = v.Position[3]
        distance = VDist2(startX, startZ, x, z)
        if not lowest or distance < lowest then
            loc = v.Position
            name = v.Name
            lowest = distance
        end
    end
    return loc, name
end


function AIGetClosestThreatMarkerLoc(aiBrain, markerType, startX, startZ, threatMin, threatMax, rings)
    local markerList = AIGetMarkerLocations(aiBrain, markerType)
    local loc, name, distance, lowest = nil
    #local threatAtLoc
    for k, v in markerList do
        local x = v.Position[1]
        local z = v.Position[3]
        distance = VDist2(startX, startZ, x, z)
        local threat = aiBrain:GetThreatAtPosition({x, 0, z}, rings, false)
        if (not lowest or distance < lowest) and threat >= threatMin and threat <= threatMax then
            loc = v.Position
            name = v.Name
            lowest = distance
            #threatAtLoc = threat
        end
    end
    #LOG('*AI DEBUG: MARKER: Threat Marker, TYPE = ', markerType, ' Threat = ', threatAtLoc, ' LOC = ', repr(loc))
    return loc, name
end

function AIFindDefensiveArea( aiBrain, unit, category, range )
    if not unit:IsDead() then
        # Build a grid to find units near
        local gridSize = range / 5
        if gridSize > 150 then
            gridSize = 150
        end
        local grid = {}
        local highPoint = false
        local highNum = false
        local unitPos = unit:GetPosition()
        local distance
        local startPosX, startPosZ = aiBrain:GetArmyStartPos()
        for i=-5,5 do
            for j=-5,5 do
                local height = GetTerrainHeight( unitPos[1] - ( gridSize * i ), unitPos[3] - ( gridSize * j ) )
                if GetSurfaceHeight( unitPos[1] - ( gridSize * i ), unitPos[3] - ( gridSize * j ) ) > height then
                    height = GetSurfaceHeight( unitPos[1] - ( gridSize * i ), unitPos[3] - ( gridSize * j ) )
                end
                local checkPos = { unitPos[1] - ( gridSize * i ), height, unitPos[3] - ( gridSize * j ) }
                local units = aiBrain:GetUnitsAroundPoint( 'Ally', category, checkPos, gridSize )
                local tempNum = 0
                for k,v in units do
                    if EntityCategoryContains( categories.TECH3, v ) then
                        tempNum = tempNum + 10
                    elseif EntityCategoryContains( categories.TECH2, v ) then
                        tempNum = tempNum + 5
                    else
                        tempNum = tempNum + 1
                    end
                end
                local units = aiBrain:GetUnitsAroundPoint( 'Enemy', categories.MOBILE, checkPos, gridSize )
                for k,v in units do
                    if EntityCategoryContains( categories.TECH3, v ) then
                        tempNum = tempNum - 10
                    elseif EntityCategoryContains( categories.TECH2, v ) then
                        tempNum = tempNum - 5
                    else
                        tempNum = tempNum - 1
                    end
                end
                if not highNum or tempNum > highNum then
                    highNum = tempNum
                    distance = VDist2( startPosX, startPosZ, checkPos[1], checkPos[3] )
                    highPoint = checkPos
                elseif tempNum == highNum then
                    local tempDist = VDist2( startPosX, startPosZ, checkPos[1], checkPos[3] )
                    if tempDist < distance then
                        highNum = tempNum
                        highPoint = checkPos
                    end
                end                        
            end
        end
        return highPoint
    else
        return { 0, 0, 0 }
    end
end

function GetLocationNeedingWalls( aiBrain, radius, count, unitCategory, tMin, tMax, tRings )
    local positions = {}
    for k,v in aiBrain.PBM.Locations do
        if v.LocationType ~= 'MAIN' then
            table.insert( positions, v.Location )
        end
    end
    local bestFit
    local mostUnits
    local mainPos = aiBrain:PBMGetLocationCoords( 'MAIN' )
    local otherPos = AIGetMarkersAroundLocation( aiBrain, 'Defensive Point', mainPos, radius, tMin, tMax, tRings )
    for k,v in otherPos do
        table.insert( positions, v.Position )
    end
    for k,v in positions do
        if Utils.XZDistanceTwoVectors( v, mainPos ) < radius then
            local tempUnits = table.getn(GetOwnUnitsAroundPoint( aiBrain, ParseEntityCategory(unitCategory), v, 30))
            local numWalls = table.getn(GetOwnUnitsAroundPoint( aiBrain, categories.WALL, v, 40))
            if tempUnits > count and numWalls < 10 and ( not bestFit or tempUnits > mostUnits ) then
                bestFit = v
                mostUnits = tempUnits
            end
        end
    end
    if bestFit then
        return bestFit
    else
        return false
    end
end
                                

# Assist factories based on what factories have less units helping
function AIEngineersAssistFactories( aiBrain, engineers, factories )
    local facsAssisting = 0
    local factoryData = {}
    
    local lowNum, key, value, active, tempNum, tempActive, setVal

    # Sort Factories based on number of guards
    for i = 1,table.getn(factories) do
        lowNum = false
        key = -1
        value = false
        active = false
        tempActive = false
        
        for j,v in factories do
            # We only want factories that are actively doin stuff and aren't like dead
            tempNum = table.getn(v:GetGuards())
            if not v:IsDead() then
                setVal = false
                tempActive = v:IsUnitState('Building') or ( v:GetNumBuildOrders(categories.ALLUNITS) > 0 )
                if not active and tempActive then
                    active = true
                    setVal = true
                elseif active and tempActive and ( not lowNum or tempNum < lowNum ) then
                    setVal = true
                elseif not active and not tempActive and ( not lowNum or tempNum < lowNum ) then
                    setVal = true
                end
                if setVal then
                    lowNum = table.getn(v:GetGuards())
                    value = v
                    key = j
                end                    
            end
        end
        if key > 0 then
            factoryData[i] = { Factory = value, NumGuards = lowNum }
            table.remove( factories, key )
        end
    end

    # Find a factory for each engineer and update number of guards
    for unitNum, unit in engineers do
        lowNum = false
        key = 0
        for k,v in factoryData do
            if not lowNum or v.NumGuards < lowNum then
                lowNum = v.NumGuards
                key = k
            end
        end
        if lowNum then
            IssueGuard( {unit}, factoryData[key].Factory )
            factoryData[key].NumGuards = factoryData[key].NumGuards + 1
        else
            aiBrain:AssignUnitsToPlatoon( 'ArmyPool', {unit}, 'Unassigned', 'NoFormation' )
        end 
    end
    return true
end

# Find all units working together on a builder and return all factories
function AIReturnAssistingFactories( factory )
    local guards = factory:GetGuards()
    local guardFacs = {}
    table.insert( guardFacs, factory )
    for k,v in guards do
        if not v:IsDead() and EntityCategoryContains( categories.FACTORY, v ) then
            table.insert( guardFacs, v )
        end
    end
    return guardFacs
end

function GetBasePatrolPoints( aiBrain, location, radius, layer )
    if type(location) == 'string' then
        for k,v in aiBrain.PBM.Locations do
            if v.LocationType == location then
                radius = v.Radius
                location = v.Location
                break
            end
        end
        if not radius then
            error('*AI ERROR: Invalid locationType- '..location..' for army- '..aiBrain.Name, 2)
        end
    end
    if not location or not radius then
        error('*AI ERROR: Need location and radius or locationType for AIUtilities.GetBasePatrolPoints', 2)
    end
    local vecs = aiBrain:GetBaseVectors()
    local locList = {}
    if not layer then
        layer = 'Land'
    end
    for k,v in vecs do
        if LayerCheckPosition( v, layer) and VDist2( v[1], v[3], location[1], location[3] ) < radius then
            table.insert(locList, v)
        end
    end
    local sortedList = {}
    local lastX = location[1]
    local lastZ = location[3]
    if table.getsize(locList) == 0 then return {} end
    local num = table.getsize(locList)
    #Sort the locations from point to closest point, that way it  makes a nice patrol path
    for i = 1, num do
        local lowest
        local czX, czZ, pos, distance, key
        for k, v in locList do
            local x = v[1]
            local z = v[3]
            distance = VDist2(lastX, lastZ, x, z)
            if not lowest or distance < lowest then
                pos = v
                lowest = distance
                key = k
            end
        end
        if not pos then return {} end
        sortedList[i] = pos
        lastX = pos[1]
        lastZ = pos[3]
        table.remove(locList, key)
    end
    return sortedList
end

function GetUnitBaseStructureVector( unit )
    if not unit:IsDead() then
        local pos = unit:GetPosition()
        pos[1] = pos[1] + 16
        pos[3] = pos[3] + 16
        local x = math.floor( pos[1] / 32 ) - 1
        local z = math.floor( pos[3] / 32 ) - 1
        local height = GetTerrainHeight( x, z )
        if GetSurfaceHeight( x, z ) > height then
            height = GetSurfaceHeight( x, z )
        end
        return { ( x * 32 ) + 16 , height,  ( z * 32 ) + 16 }
    else
        return false
    end    
end

function GetOwnUnitsAroundPoint( aiBrain, category, location, radius, min, max, rings )
    local units = aiBrain:GetUnitsAroundPoint( 'Ally', category, location, radius )
    local index = aiBrain:GetArmyIndex()
    local retUnits = {}
    local checkThreat = false
    if min and max and rings then
        checkThreat = true
    end
    for k,v in units do
        if not v:IsDead() and v:GetAIBrain():GetArmyIndex() == index then
            if checkThreat then 
                local threat = aiBrain:GetThreatAtPosition( v:GetPosition(), rings, false )
                if threat >= min and threat <= max then
                    table.insert(retUnits, v)
                end
            else
                table.insert(retUnits, v)
            end
        end
    end
    return retUnits
end

function GetBrainUnitsAroundPoint( aiBrain, alliance, category, location, radius, tBrain )
    local units = aiBrain:GetUnitsAroundPoint( alliance, category, location, radius )
    local tIndex = tBrain:GetArmyIndex()
    local retTable = {}
    for k,v in units do
        if not v:IsDead() and v:GetAIBrain():GetArmyIndex() == tIndex then
            table.insert( retTable, v )
        end
    end
    return retTable
end

function LayerCheckPosition( pos, layer )
    if pos[1] > 0 and pos[1] < ScenarioInfo.size[1] and pos[3] > 0 and pos[3] < ScenarioInfo.size[2] then
        local surf = GetSurfaceHeight( pos[1], pos[3] )
        local terr = GetTerrainHeight( pos[1], pos[3] )
        if surf > terr and layer == 'Sea' then
            return true
        elseif terr >= surf and layer == 'Land' then
            return true
        else
            return false
        end
    else
        return false
    end
end

function GetNearestPathingPoint( position )
    local x = math.floor( position[1] / 8 )
    local z = math.floor( position[3] / 8 )
    retPos = { ( x * 8 ) , 0,  ( z * 8 )  } 
    if retPos[1] == 0 then
        retPos[1] = 1
    elseif retPos[1] == ScenarioInfo.size[1] then
        retPos[1] = retPos[1] - 1
    end
    if retPos[3] == 0 then
        retPos[3] = 1
    elseif retPos[3] == ScenarioInfo.size[2] then
        retPos[3] = retPos[3] - 1
    end
    return retPos
end

function CheckUnitPathing( destination, location, unit )
    if unit:IsDead() then
        return false
    end
    local pathingType = 'Land'
    local mType = unit:GetBlueprint().Physics.MotionType
    if mType == 'RULEUMT_AmphibiousFloating' or mType == 'RULEUMT_Hover' or mType == 'RULEUMT_Amphibious' or mType == 'RULEUMT_GiantHexaped' then
        pathingType = 'Amphibious'
    elseif mType == 'RULEUMT_Water' then
        pathingType = 'Water'
    elseif mType == 'RULEUMT_Air' then
        return true
    end
    local unitPos = GetNearestPathingPoint(unit:GetPosition())
    local locPos = GetNearestPathingPoint(location)
    local getResult = false
    if locPos[1] == unitPos[1] and locPos[3] == unitPos[3] then
        getResult = true
    end
    local destPos = GetNearestPathingPoint(destination)
    if not ScenarioInfo.MapData.PathingTable[pathingType][unitPos[1]] then
        ScenarioInfo.MapData.PathingTable[pathingType][unitPos[1]] = {}
    end
    if not ScenarioInfo.MapData.PathingTable[pathingType][unitPos[1]][unitPos[3]] then
        ScenarioInfo.MapData.PathingTable[pathingType][unitPos[1]][unitPos[3]] = {}
    end
    if not ScenarioInfo.MapData.PathingTable[pathingType][unitPos[1]][unitPos[3]][destPos[1]] then
        ScenarioInfo.MapData.PathingTable[pathingType][unitPos[1]][unitPos[3]][destPos[1]] = {} 
    end
    if not ScenarioInfo.MapData.PathingTable[pathingType][unitPos[1]][unitPos[3]][destPos[1]][destPos[3]] then
        if getResult then
            local surf = GetSurfaceHeight( destPos[1], destPos[3] )
            local terr = GetTerrainHeight( destPos[1], destPos[3] )
            local result = terr >= surf 
            if result then
                local finalPos = {destPos[1], terr, destPos[3] }
                result = unit:CanPathTo( finalPos )
            end
            ScenarioInfo.MapData.PathingTable[pathingType][unitPos[1]][unitPos[3]][destPos[1]][destPos[3]] = result
        end
    end
    if ScenarioInfo.MapData.PathingTable[pathingType][unitPos[1]][unitPos[3]][destPos[1]][destPos[3]] then
        return true
    else
        return false
    end
end

function FindPointInTable( point, posTable )
    for k,v in posTable do
        if point[1] == v[1] and point[2] == v[2] and point[3] == v[3] then
            return true
        end
    end
    return false
end

function AIFindBrainTargetInRange( aiBrain, platoon, squad, maxRange, atkPri, enemyBrain )
    local position = platoon:GetPlatoonPosition()
    if enemyBrain then
        local enemyIndex = enemyBrain:GetArmyIndex()
        local targetUnits = aiBrain:GetUnitsAroundPoint( 'Enemy', categories.ALLUNITS, position, maxRange )
        for k,v in atkPri do
            local category = ParseEntityCategory( v )
            local retUnit = false
            local distance = false
            for num, unit in targetUnits do
                if not unit:IsDead() and EntityCategoryContains( category, unit ) and unit:GetAIBrain():GetArmyIndex() == enemyIndex and platoon:CanAttackTarget( squad, unit ) then
                    local unitPos = unit:GetPosition()
                    if not retUnit or Utils.XZDistanceTwoVectors( position, unitPos ) < distance then
                        retUnit = unit
                        distance = Utils.XZDistanceTwoVectors( position, unitPos )
                    end
                end
            end
            if retUnit then
                return retUnit
            end
        end
    end
    return false
end

function RandomLocation(x,z)
    local finalX = x + Random(-30, 30)
    while finalX <= 0 or finalX >= ScenarioInfo.size[1] do
        finalX = x + Random(-30, 30)
    end
    local finalZ = z + Random(-30, 30)
    while finalZ <= 0 or finalZ >= ScenarioInfo.size[2] do
        finalZ = z + Random(-30, 30)
    end
    local movePos = { finalX, 0, finalZ }
#        until AIUtils.CheckUnitPathing( movePos, eng:GetPosition(), eng )
    local height = GetTerrainHeight( movePos[1], movePos[3] )
    if GetSurfaceHeight( movePos[1], movePos[3] ) > height then
        height = GetSurfaceHeight( movePos[1], movePos[3] )
    end
    movePos[2] = height
    return movePos
end

function FindIdleGates(aiBrain)
    local gates = aiBrain:GetListOfUnits( categories.GATE, true )
    if gates and table.getn(gates) > 0 then
        local retGates = {}
        for k,v in gates do
            if not v:IsUnitState('Building') and not v:IsUnitState('TransportLoading') then
                table.insert(retGates, v)
            end
        end
        return retGates
    end
    return false
end



# -------------------------------------------------------
# Utility Function
# Returns the number of slots the transport has available
# -------------------------------------------------------
function GetNumTransportSlots(unit)
    local bones = {
        Large = 0,
        Medium = 0,
        Small = 0,
    }
    for i=1,unit:GetBoneCount() do
        if unit:GetBoneName(i) ~= nil then
            if string.find(unit:GetBoneName(i), 'Attachpoint_Lrg') then
                bones.Large = bones.Large + 1
            elseif string.find(unit:GetBoneName(i), 'Attachpoint_Med') then
                bones.Medium = bones.Medium + 1
            elseif string.find(unit:GetBoneName(i), 'Attachpoint') then
                bones.Small = bones.Small + 1
            end
        end
    end
    return bones
end

# -------------------------------------------------------------
# Utility Function
# Returns the number of transports required to move the platoon
# -------------------------------------------------------------
function GetNumTransports(units)
    local transportNeeded = {
        Small = 0,
        Medium = 0,
        Large = 0,
    }
    local transportClass
    for k, v in units do
        if not v:IsDead() then
            transportClass = v:GetBlueprint().Transport.TransportClass
            if(transportClass == 1) then
                transportNeeded.Small = transportNeeded.Small + 1
            elseif(transportClass == 2) then
                transportNeeded.Medium = transportNeeded.Medium + 1
            elseif(transportClass == 3) then
                transportNeeded.Large = transportNeeded.Large + 1
            else
                transportNeeded.Small = transportNeeded.Small + 1
            end
        end
    end

    return transportNeeded
end

# -----------------------------------------------------------------
# Utility Function
# Function that gets the correct number of transports for a platoon
# -----------------------------------------------------------------
function GetTransports(platoon, units)
    local data = platoon.PlatoonData
    local aiBrain = platoon:GetBrain()

    if not units then
        units = platoon:GetPlatoonUnits()
    end

    local neededTable = GetNumTransports(units)
    local numTransports = 0
    local transportsNeeded = false
    if neededTable.Small > 0 or neededTable.Medium > 0 or neededTable.Large > 0 then
        transportsNeeded = true
    end
    local transSlotTable = {}

    local pool = aiBrain:GetPlatoonUniquelyNamed( 'ArmyPool' )

    # make sure more are needed
    local tempNeeded = {}
    tempNeeded.Small = neededTable.Small
    tempNeeded.Medium = neededTable.Medium
    tempNeeded.Large = neededTable.Large

    local location = platoon:GetPlatoonPosition()
    local transports = {}
    # Determine distance of transports from platoon
    for k,unit in pool:GetPlatoonUnits() do
        if EntityCategoryContains( categories.TRANSPORTATION - categories.uea0203, unit ) and not unit:IsUnitState('Busy') then
            local unitPos = unit:GetPosition()
            local curr = { Unit=unit, Distance=VDist2( unitPos[1], unitPos[3], location[1], location[3] ),
                           Id = unit:GetUnitId() }
            table.insert( transports, curr )
        end
    end
    if table.getn(transports) > 0 then
        local sortedList = {}
        # Sort distances
        for k = 1,table.getn(transports) do
            local lowest = -1
            local key, value
            for j,u in transports do
                if lowest == -1 or u.Distance < lowest then
                    lowest = u.Distance
                    value = u
                    key = j
                end
            end
            sortedList[k] = value
            # remove from unsorted table
            table.remove( transports, key )
        end


        # Take transports as needed
        for i=table.getn(sortedList),1,-1 do
            if transportsNeeded then
                local id = sortedList[i].Id
                aiBrain:AssignUnitsToPlatoon(platoon, {sortedList[i].Unit}, 'Scout', 'GrowthFormation')
                numTransports = numTransports + 1
                if not transSlotTable[id] then
                    transSlotTable[id] = GetNumTransportSlots(sortedList[i].Unit)
                end
                local tempSlots = {}
                tempSlots.Small = transSlotTable[id].Small
                tempSlots.Medium = transSlotTable[id].Medium
                tempSlots.Large = transSlotTable[id].Large
                # update number of slots needed
                while tempNeeded.Large > 0 and tempSlots.Large > 0 do
                    tempNeeded.Large = tempNeeded.Large - 1
                    tempSlots.Large = tempSlots.Large - 1
                    tempSlots.Medium = tempSlots.Medium - 2
                    tempSlots.Small = tempSlots.Small - 4
                end
                while tempNeeded.Medium > 0 and tempSlots.Medium > 0 do
                    tempNeeded.Medium = tempNeeded.Medium - 1
                    tempSlots.Medium = tempSlots.Medium - 1
                    tempSlots.Small = tempSlots.Small - 2
                end
                while tempNeeded.Small > 0 and tempSlots.Small > 0 do
                    tempNeeded.Small = tempNeeded.Small - 1
                    tempSlots.Small = tempSlots.Small - 1
                end
                if tempNeeded.Small <= 0 and tempNeeded.Medium <= 0 and tempNeeded.Large <= 0 then
                    transportsNeeded = false
                end
            end
        end
    end

    if transportsNeeded then
#        if platoon:GetSquadUnits('Scout') then
#            LOG('*AI DEBUG: Attack force attempting to get enough transport failed: TransTotal - ' .. repr(table.getn(platoon:GetSquadUnits('Scout'))) .. ' Large - ' .. repr(tempNeeded.Large) .. ' Medium - ' .. repr(tempNeeded.Medium) .. ' Small - ' .. repr(tempNeeded.Small) )
#        end
        ReturnTransportsToPool( platoon:GetSquadUnits('Scout'), false )
        return false
    else
        return numTransports
    end
end


# ------------------------------------------
# Utility Function
# Get and load transports with platoon units
# ------------------------------------------
function UseTransports(units, transports, location)
    local aiBrain
    for k,v in units do
        if not v:IsDead() then
            aiBrain = v:GetAIBrain()
            break
        end
    end
        # Load transports
    local transportTable = {}
    local transSlotTable = {}
    if not transports then
        return false
    end
    LOG('*AI DEBUG: Attack force using Transports')
    for num,unit in transports do
        local id = unit:GetUnitId()
        if not transSlotTable[id] then
            transSlotTable[id] = GetNumTransportSlots(unit)
        end
        table.insert( transportTable,
            {
                Transport = unit,
                LargeSlots = transSlotTable[id].Large,
                MediumSlots = transSlotTable[id].Medium,
                SmallSlots = transSlotTable[id].Small,
                Units = {}
            }
        )
    end

    local shields = {}
    local remainingSize3 = {}
    local remainingSize2 = {}
    local remainingSize1 = {}
    local pool = aiBrain:GetPlatoonUniquelyNamed('ArmyPool')
    for num, unit in units do
        if not unit:IsDead() then
            if unit:IsUnitState( 'Attached' ) then
                aiBrain:AssignUnitsToPlatoon( pool, {unit}, 'Unassigned', 'None' )
                LOG('*AI DEBUG: ARMY ' .. aiBrain:GetArmyIndex() .. ': Already Attached units adding to pool' )
            elseif EntityCategoryContains( categories.url0306 + categories.DEFENSE, unit ) then
                table.insert( shields, unit )
            elseif unit:GetBlueprint().Transport.TransportClass == 3 then
                table.insert( remainingSize3, unit )
            elseif unit:GetBlueprint().Transport.TransportClass == 2 then
                table.insert( remainingSize2, unit )
            elseif unit:GetBlueprint().Transport.TransportClass == 1 then
                table.insert( remainingSize1, unit )
            else
                table.insert( remainingSize1, unit )
            end
        end
    end
#    LOG( '*AI DEBUG: NUM SHIELDS= ', table.getn( shields ) )
#    LOG( '*AI DEBUG: NUM LARGE = ', table.getn( remainingSize3 ) )
#    LOG( '*AI DEBUG: NUM MEDIUM = ', table.getn( remainingSize2 ) )
#    LOG( '*AI DEBUG: NUM SMALL = ', table.getn( remainingSize1 ) )

    local needed = GetNumTransports(units)
    local largeHave = 0
    for num, data in transportTable do
        largeHave = largeHave + data.LargeSlots
    end
    local leftoverUnits = {}
    local currLeftovers = {}
    local leftoverShields = {}
    transportTable, leftoverShields = SortUnitsOnTransports( transportTable, shields, largeHave - needed.Large )

    transportTable, leftoverUnits = SortUnitsOnTransports( transportTable, remainingSize3, -1 )

    transportTable, currLeftovers = SortUnitsOnTransports( transportTable, leftoverShields, -1 )

    for k,v in currLeftovers do table.insert(leftoverUnits, v) end
    transportTable, currLeftovers = SortUnitsOnTransports( transportTable, remainingSize2, -1 )

    for k,v in currLeftovers do table.insert(leftoverUnits, v) end
    transportTable, currLeftovers = SortUnitsOnTransports( transportTable, remainingSize1, -1 )

    for k,v in currLeftovers do table.insert(leftoverUnits, v) end
    transportTable, currLeftovers = SortUnitsOnTransports( transportTable, currLeftovers, -1 )


    LOG('*AI DEBUG: ARMY ' .. aiBrain:GetArmyIndex() .. ': Loading Attack force on transports - Faction = ' .. aiBrain:GetFactionIndex() )
    local monitorUnits = {}
    for num, data in transportTable do
        if table.getn( data.Units ) > 0 then
            IssueClearCommands( data.Units )
            IssueTransportLoad( data.Units, data.Transport )
            for k,v in data.Units do table.insert( monitorUnits, v) end
        end
    end
    
    local attached = true
    repeat
        WaitSeconds(2)
        local allDead = true
        for k,v in units do
            if not v:IsDead() then
                allDead = false
                break
            end
        end
        if allDead then return false end
        attached = true
        for k,v in monitorUnits do
            if not v:IsDead() and v:IsUnitState('WaitingForTransport') then
                attached = false
            end
        end
    until attached
    # Any units that aren't transports and aren't attached send back to pool
    for k,unit in units do
        if not unit:IsDead() and not EntityCategoryContains( categories.TRANSPORTATION, unit ) then
            if not unit:IsUnitState('Attached') then
                aiBrain:AssignUnitsToPlatoon( pool, {unit}, 'Unassigned', 'None' )
            end
        end
    end
    
    IssueTransportUnload( transports, location )
    local attached = true
    while attached do
        WaitSeconds(3)
        local allDead = true
        for k,v in transports do
            if not v:IsDead() then
                allDead = false
                break
            end
        end
        if allDead then
            return false
        end
        attached = false
        for num, unit in units do
            if not unit:IsDead() and unit:IsUnitState('Attached') then
                attached = true
                break
            end
        end
    end
    ReturnTransportsToPool( transports, true )
    
    return true
end

# ------------------------------------------------
# Utility function
# Sorts units onto transports distributing equally
# ------------------------------------------------
function SortUnitsOnTransports( transportTable, unitTable, numSlots )
    local leftoverUnits = {}
    numSlots = numSlots or -1
    for num, unit in unitTable do
        if numSlots == -1 or num <= numSlots then
            local transSlotNum = 0
            local remainingLarge = 0
            local remainingMed = 0
            local remainingSml = 0
            for tNum, tData in transportTable do
                if tData.LargeSlots > remainingLarge then
                    transSlotNum = tNum
                    remainingLarge = tData.LargeSlots
                    remainingMed = tData.MediumSlots
                    remainingSml = tData.SmallSlots
                elseif tData.LargeSlots == remainingLarge and tData.MediumSlots > remainingMed then
                    transSlotNum = tNum
                    remainingLarge = tData.LargeSlots
                    remainingMed = tData.MediumSlots
                    remainingSml = tData.SmallSlots
                elseif tData.LargeSlots == remainingLarge and tData.MediumSlots == remainingMed and tData.SmallSlots > remainingSml then
                    transSlotNum = tNum
                    remainingLarge = tData.LargeSlots
                    remainingMed = tData.MediumSlots
                    remainingSml = tData.SmallSlots
                end
            end
            if transSlotNum > 0 then
                table.insert( transportTable[transSlotNum].Units, unit )
                if unit:GetBlueprint().Transport.TransportClass == 3 and remainingLarge >= 1 then
                    transportTable[transSlotNum].LargeSlots = transportTable[transSlotNum].LargeSlots - 1
                    transportTable[transSlotNum].MediumSlots = transportTable[transSlotNum].MediumSlots - 2
                    transportTable[transSlotNum].SmallSlots = transportTable[transSlotNum].SmallSlots - 4
                elseif unit:GetBlueprint().Transport.TransportClass == 2 and remainingMed > 0 then
                    if transportTable[transSlotNum].LargeSlots > 0 then
                        transportTable[transSlotNum].LargeSlots = transportTable[transSlotNum].LargeSlots - .5
                    end
                    transportTable[transSlotNum].MediumSlots = transportTable[transSlotNum].MediumSlots - 1
                    transportTable[transSlotNum].SmallSlots = transportTable[transSlotNum].SmallSlots - 2
                elseif unit:GetBlueprint().Transport.TransportClass == 1 and remainingSml > 0 then
                    transportTable[transSlotNum].SmallSlots = transportTable[transSlotNum].SmallSlots - 1
                elseif remainingSml > 0 then
                    transportTable[transSlotNum].SmallSlots = transportTable[transSlotNum].SmallSlots - 1
                else
                    #LOG('*AI DEBUG: FOUND TRANSPORT NOT ENOUGH SLOTS')
                    table.insert(leftoverUnits, unit)
                end
            else
                #LOG('*AI DEBUG: NO TRANSPORT FOUND')
                table.insert(leftoverUnits, unit)
            end
        end
    end
    return transportTable, leftoverUnits
end

# ------------------------------------------------------------------------------------
# Utility Function
# Takes transports in platoon, returns them to pool, flys them back to return location
# ------------------------------------------------------------------------------------
function ReturnTransportsToPool(units, move)
    # Put transports back in TPool
    local unit
    if not units then
        return false
    end
    for k,v in units do
        if not v:IsDead() then
            unit = v
            break
        end
    end
    if not unit then
        return false
    end
    local aiBrain = unit:GetAIBrain()
    local x,z = aiBrain:GetArmyStartPos()
    local position = RandomLocation(x,z)
    for k,unit in units do
        if not unit:IsDead() and EntityCategoryContains( categories.TRANSPORTATION, unit ) then
            aiBrain:AssignUnitsToPlatoon( 'ArmyPool', {unit}, 'Scout', 'None' )
            if move then
                IssueMove( {unit}, position )
            end
        end
    end
end

