# Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#
# General Sim scripts

#==============================================================================
# Diplomacy
#==============================================================================

function BreakAlliance( data )

    # You can't change alliances in a team game
    if ScenarioInfo.TeamGame then
        return
    end

    if OkayToMessWithArmy(data.From) then
        SetAlliance(data.From,data.To,"Enemy")
    end
end

function OnAllianceResult( resultData )
    # You can't change alliances in a team game
    if ScenarioInfo.TeamGame then
        return
    end

    if OkayToMessWithArmy(data.From) then
        if resultData.ResultValue == "accept" then
            SetAlliance(resultData.From,resultData.To,"Ally")
        end
    end
end

function GiveUnitsToPlayer( data, units )
    if units then
        for k,v in units do
            local owner = v:GetArmy()
            if OkayToMessWithArmy(owner) and IsAlly(owner,data.To) then

                # Only allow units not attached to be given.
                # This is because units will give all of it's children
                # over as well, so we only want the top level units to be given.

                if v:GetParent() == v then
                    ChangeUnitArmy(v,data.To)
                end
            end
        end
    end
end

function GiveResourcesToPlayer( data )
    if not OkayToMessWithArmy(data.From) then return end
    local fromBrain = GetArmyBrain(data.From)
    local toBrain = GetArmyBrain(data.To)
    local massTaken = fromBrain:TakeResource('Mass',data.Mass)
    local energyTaken = fromBrain:TakeResource('Energy',data.Energy)
    toBrain:GiveResource('Mass',massTaken)
    toBrain:GiveResource('Energy',energyTaken)
end

function SetResourceSharing( data )
    if not OkayToMessWithArmy(data.Army) then return end
    local brain = GetArmyBrain(data.Army)
    brain:SetResourceSharing(data.Value)
end

function RequestAlliedVictory( data )
    # You can't change this in a team game
    if ScenarioInfo.TeamGame then
        return
    end

    if not OkayToMessWithArmy(data.Army) then return end

    local brain = GetArmyBrain(data.Army)
    brain.RequestingAlliedVictory = data.Value
end

import('/lua/SimPlayerQuery.lua').AddResultListener( "OfferAlliance", OnAllianceResult )

#==============================================================================
# UNIT CAP
#==============================================================================
function UpdateUnitCap()
    # If we are asked to share out unit cap for the defeated army, do the following...
    if not ScenarioInfo.Options.DoNotShareUnitCap then
        local aliveCount = 0
        for k,brain in ArmyBrains do
            if not brain:IsDefeated() then
                aliveCount = aliveCount + 1
            end
        end
        if aliveCount > 0 then
            local initialCap = tonumber(ScenarioInfo.Options.UnitCap)
            local totalCap = aliveCount * initialCap
            for k,brain in ArmyBrains do
                if not brain:IsDefeated() then
                    SetArmyUnitCap(brain:GetArmyIndex(),math.floor(totalCap / aliveCount))
                end
            end
        end
    end
end

