#****************************************************************************
#**
#**  File     :  /cdimage/lua/modules/aibrain.lua
#**  Author(s):
#**
#**  Summary  :
#**
#**  Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************
#########################################
# AIBrain Lua Module                    #
#########################################

local AIDefaultPlansList = import('/lua/modules/aibrainplans.lua').AIPlansList
local AIUtils = import('/lua/modules/ai/aiutilities.lua')
local PCBC = import('/lua/modules/editor/platooncountbuildconditions.lua')
local Utilities = import('/lua/modules/utilities.lua')
local ScenarioUtils = import('/lua/ScenarioUtilities.lua')
local Behaviors = import('/lua/modules/ai/aibehaviors.lua')
local AIBuildUnits = import('/lua/modules/ai/aibuildunits.lua')
#LOG('aibrain_methods.__index = ',moho.aibrain_methods.__index,' ',moho.aibrain_methods)


#############################################
###### VO Timeout and Replay Durations ######
#############################################
local VOReplayTime = {
    ExperimentalDetected = 60,
    ExperimentalUnitDestroyed = 5,
    FerryPointSet = 5,
    CoordinatedAttackInitiated = 5,
    BaseUnderAttack = 30,
    UnderAttack = 60,
    EnemyForcesDetected = 120,
    NukeArmed = 1,
    NuclearLaunchInitiated = 1,
    NuclearLaunchDetected = 1,
    BattleshipDestroyed = 5,
    EnemyNavalForcesDetected = 60,
}

#############################################
###### Runtime score update sync loop  ######
#############################################
function SyncCurrentScores()
    while true do
        for index, brain in ArmyBrains do
           Sync.Score[index] = {}

           ####################
           ## General scores ##
           ####################
           Sync.Score[index].general = {}
           Sync.Score[index].general.score = brain:CalculateScore()
           Sync.Score[index].general.mass = brain:GetArmyStat("Economy_TotalProduced_Mass", 0.0).Value
           Sync.Score[index].general.energy = brain:GetArmyStat("Economy_TotalProduced_Energy", 0.0).Value
           Sync.Score[index].general.kills = {}
           Sync.Score[index].general.kills.count = brain:GetArmyStat("Enemies_Killed", 0.0).Value
           Sync.Score[index].general.kills.mass = brain:GetArmyStat("Enemies_MassValue_Destroyed", 0.0).Value
           Sync.Score[index].general.kills.energy = brain:GetArmyStat("Enemies_EnergyValue_Destroyed", 0.0).Value
           Sync.Score[index].general.built = {}
           Sync.Score[index].general.built.count = brain:GetArmyStat("Units_History", 0.0).Value
           Sync.Score[index].general.built.mass = brain:GetArmyStat("Units_MassValue_Built", 0.0).Value
           Sync.Score[index].general.built.energy = brain:GetArmyStat("Units_EnergyValue_Built", 0.0).Value
           Sync.Score[index].general.lost = {}
           Sync.Score[index].general.lost.count = brain:GetArmyStat("Units_Killed", 0.0).Value
           Sync.Score[index].general.lost.mass = brain:GetArmyStat("Units_MassValue_Lost", 0.0).Value
           Sync.Score[index].general.lost.energy = brain:GetArmyStat("Units_EnergyValue_Lost", 0.0).Value
           Sync.Score[index].general.currentunits = {}
           Sync.Score[index].general.currentunits.count = brain:GetArmyStat("UnitCap_Current", 0.0).Value
           Sync.Score[index].general.currentcap = {}
           Sync.Score[index].general.currentcap.count = brain:GetArmyStat("UnitCap_MaxCap", 0.0).Value


           #################
           ## unit scores ##
           #################
           Sync.Score[index].units = {}
           Sync.Score[index].units.cdr = {}
           Sync.Score[index].units.cdr.kills = brain:GetBlueprintStat("Enemies_Killed", categories.COMMAND)
           Sync.Score[index].units.cdr.built = brain:GetBlueprintStat("Units_History", categories.COMMAND)
           Sync.Score[index].units.cdr.lost = brain:GetBlueprintStat("Units_Killed", categories.COMMAND)
           Sync.Score[index].units.land = {}
           Sync.Score[index].units.land.kills = brain:GetBlueprintStat("Enemies_Killed", categories.LAND)
           Sync.Score[index].units.land.built = brain:GetBlueprintStat("Units_History", categories.LAND)
           Sync.Score[index].units.land.lost = brain:GetBlueprintStat("Units_Killed", categories.LAND)
           Sync.Score[index].units.air = {}
           Sync.Score[index].units.air.kills = brain:GetBlueprintStat("Enemies_Killed", categories.AIR)
           Sync.Score[index].units.air.built = brain:GetBlueprintStat("Units_History", categories.AIR)
           Sync.Score[index].units.air.lost = brain:GetBlueprintStat("Units_Killed", categories.AIR)
           Sync.Score[index].units.naval = {}
           Sync.Score[index].units.naval.kills = brain:GetBlueprintStat("Enemies_Killed", categories.NAVAL)
           Sync.Score[index].units.naval.built = brain:GetBlueprintStat("Units_History", categories.NAVAL)
           Sync.Score[index].units.naval.lost = brain:GetBlueprintStat("Units_Killed", categories.NAVAL)
           Sync.Score[index].units.structures = {}
           Sync.Score[index].units.structures.kills = brain:GetBlueprintStat("Enemies_Killed", categories.STRUCTURE)
           Sync.Score[index].units.structures.built = brain:GetBlueprintStat("Units_History", categories.STRUCTURE)
           Sync.Score[index].units.structures.lost = brain:GetBlueprintStat("Units_Killed", categories.STRUCTURE)
           Sync.Score[index].units.experimental = {}
           Sync.Score[index].units.experimental.kills = brain:GetBlueprintStat("Enemies_Killed", categories.EXPERIMENTAL)
           Sync.Score[index].units.experimental.built = brain:GetBlueprintStat("Units_History", categories.EXPERIMENTAL)
           Sync.Score[index].units.experimental.lost = brain:GetBlueprintStat("Units_Killed", categories.EXPERIMENTAL)


           #####################
           ## resource scores ##
           #####################
           Sync.Score[index].resources = {}
           Sync.Score[index].resources.massin = {}
           Sync.Score[index].resources.massin.total = brain:GetArmyStat("Economy_TotalProduced_Mass", 0.0).Value
           Sync.Score[index].resources.massin.rate = brain:GetArmyStat("Economy_Income_Mass", 0.0).Value
           Sync.Score[index].resources.massout = {}
           Sync.Score[index].resources.massout.total = brain:GetArmyStat("Economy_TotalConsumed_Mass", 0.0).Value
           Sync.Score[index].resources.massout.rate = brain:GetArmyStat("Economy_Output_Mass", 0.0).Value
           Sync.Score[index].resources.massover = brain:GetArmyStat("Economy_AccumExcess_Mass", 0.0).Value
           Sync.Score[index].resources.energyin = {}
           Sync.Score[index].resources.energyin.total = brain:GetArmyStat("Economy_TotalProduced_Energy", 0.0).Value
           Sync.Score[index].resources.energyin.rate = brain:GetArmyStat("Economy_Income_Energy", 0.0).Value
           Sync.Score[index].resources.energyout = {}
           Sync.Score[index].resources.energyout.total = brain:GetArmyStat("Economy_TotalConsumed_Energy", 0.0).Value
           Sync.Score[index].resources.energyout.rate = brain:GetArmyStat("Economy_Output_Energy", 0.0).Value
           Sync.Score[index].resources.energyover = brain:GetArmyStat("Economy_AccumExcess_Energy", 0.0).Value
        end
        WaitSeconds(1)  -- update scores every second
    end
end

AIBrain = Class(moho.aibrain_methods) {

    defeated = false,

    #####################################################################
    ## ------------- HUMAN BRAIN FUNCTIONS HANDLED HERE  ------------- ##
    #####################################################################
    OnCreateHuman = function(self, planName)
        self.Trash = TrashBag()
        #LOG('*AI DEBUG: Human planName = ', repr(planName))
        #LOG('===== AI DEBUG: Human Brain Fork Theads =====')
        local aiScenarioPlans = self:ImportScenarioArmyPlans(planName)
        #LOG('*AI DEBUG: HUMAN SCENARIO AI PLAN LIST = ', repr(aiScenarioPlans))

        if aiScenarioPlans then
            self.AIPlansList = aiScenarioPlans
            self.EvaluateThread = self:ForkThread(self.EvaluateAIThread)
            self.ExecuteThread = self:ForkThread(self.ExecuteAIThread)
        else
            self.AIPlansList = nil
        end
        #LOG('*AI DEBUG: HUMAN AI PLAN LIST = ', repr(self.AIPlansList))
        self.AttackData = {
            NeedSort = false,
            AMPlatoonCount = { DefaultGroupAir = 0, DefaultGroupLand = 0, DefaultGroupSea = 0, },
        }
        self.TriggerList = {}
        self.IntelTriggerList = {}
        self.VeterancyTriggerList = {}
        self.UnitBuiltTriggerList = {}
        self.RepeatExecution = false
        self:InitializeEconomyState()
        self:InitializeVO()
        self.BrainType = 'Human'
        self.IgnoreArmyCaps = false
        self.ConstantEval = true
    end,


    #####################################################################
    ## ------------- GLOBAL AI BRAIN ARMY FEATURES ------------------- ##
    #####################################################################

    InitializeEconomyState = function(self)
        # This is called very early, so ensure stats exist
        self:SetArmyStat('Economy_Ratio_Mass',0.0)
        self:SetArmyStat('Economy_Ratio_Energy',0.0)

        if not self.EconStateUnits then
            self.EconStateUnits = {
                MassStorage = {},
                EnergyStorage = {},
            }
        end
        self.EconMassStorageState = nil
        self.EconEnergyStorageState = nil
        self.EconStorageTrigs = {}

        self:SetArmyStatsTrigger('Economy_Ratio_Mass','EconLowMassStore','LessThan',0.1)
        self:SetArmyStatsTrigger('Economy_Ratio_Energy','EconLowEnergyStore','LessThan',0.1)
    end,

    ESRegisterUnitMassStorage = function(self, unit)
        if not self.EconStateUnits then
            self.EconStateUnits = {
                MassStorage = {},
                EnergyStorage = {},
            }
        end
        table.insert(self.EconStateUnits.MassStorage, unit)
    end,

    ESRegisterUnitEnergyStorage = function(self, unit)
        if not self.EconStateUnits then
            self.EconStateUnits = {
                MassStorage = {},
                EnergyStorage = {},
            }
        end
        table.insert(self.EconStateUnits.EnergyStorage, unit)
    end,

    ESUnregisterUnit = function(self, unit)
        for k, v in self.EconStateUnits do
            for i, j in v do
                if j == unit then
                    table.remove(self.EconStateUnits[k], i)
                end
            end
        end
    end,

    ESMassStorageUpdate = function(self, newState)
        if self.EconMassStorageState != newState then
            for k, v in self.EconStateUnits.MassStorage do
                if not v:IsDead() then
                    v:OnMassStorageStateChange(newState)
                else
                    table.remove(self.EconStateUnits.MassStorage, k)
                end
            end
            if newState == 'EconLowMassStore' then
                if not self.EconStorageTrigs['EconMidMassStore'] then
                    self:SetArmyStatsTrigger('Economy_Ratio_Mass','EconMidMassStore','GreaterThanOrEqual',0.11)
                    self.EconStorageTrigs['EconMidMassStore'] = true
                end
            elseif newState == 'EconMidMassStore' then
                if not self.EconStorageTrigs['EconLowMassStore'] then
                    self:SetArmyStatsTrigger('Economy_Ratio_Mass','EconLowMassStore','LessThan',0.1)
                    self.EconStorageTrigs['EconLowMassStore'] = true
                end
                if not self.EconStorageTrigs['EconFullMassStore'] then
                    self:SetArmyStatsTrigger('Economy_Ratio_Mass','EconFullMassStore','GreaterThanOrEqual',0.9)
                    self.EconStorageTrigs['EconFullMassStore'] = true
                end
            elseif newState == 'EconFullMassStore' then
                if not self.EconStorageTrigs['EconMidMassStore'] then
                    self:SetArmyStatsTrigger('Economy_Ratio_Mass','EconMidMassStore','LessThan',0.89)
                    self.EconStorageTrigs['EconMidMassStore'] = true
                end
            end
            self.EconMassStorageState = newState
            return true
        end
        return false
    end,

    ESEnergyStorageUpdate = function(self, newState)
        if self.EconEnergyStorageState != newState then
            for k, v in self.EconStateUnits.EnergyStorage do
                if not v:IsDead() then
                    v:OnEnergyStorageStateChange(newState)
                else
                    table.remove(self.EconStateUnits.EnergyStorage, k)
                end
            end
            if newState == 'EconLowEnergyStore' then
                if not self.EconStorageTrigs['EconMidEnergyStore'] then
                    self:SetArmyStatsTrigger('Economy_Ratio_Energy','EconMidEnergyStore','GreaterThanOrEqual',0.11)
                    self.EconStorageTrigs['EconMidEnergyStore'] = true
                end
            elseif newState == 'EconMidEnergyStore' then
                if not self.EconStorageTrigs['EconLowEnergyStore'] then
                    self:SetArmyStatsTrigger('Economy_Ratio_Energy','EconLowEnergyStore','LessThan',0.1)
                    self.EconStorageTrigs['EconLowEnergyStore'] = true
                end
                if not self.EconStorageTrigs['EconFullEnergyStore'] then
                    self:SetArmyStatsTrigger('Economy_Ratio_Energy','EconFullEnergyStore','GreaterThanOrEqual',0.9)
                    self.EconStorageTrigs['EconFullEnergyStore'] = true
                end
            elseif newState == 'EconFullEnergyStore' then
                if not self.EconStorageTrigs['EconMidEnergyStore'] then
                    self:SetArmyStatsTrigger('Economy_Ratio_Energy','EconMidEnergyStore','LessThan',0.89)
                    self.EconStorageTrigs['EconMidEnergyStore'] = true
                end
            end
            self.EconMassStorageState = newState
            return true
        end
        return false
    end,

    CalculateScore = function(self)
        local massSpent = self:GetArmyStat("Economy_TotalConsumed_Mass",0.0).Value
        local massProduced = self:GetArmyStat("Economy_TotalProduced_Mass",0.0).Value -- not currently being used
        local energySpent = self:GetArmyStat("Economy_TotalConsumed_Energy",0.0).Value
        local energyProduced = self:GetArmyStat("Economy_TotalProduced_Energy",0.0).Value -- not currently being used
        local massValueDestroyed = self:GetArmyStat("Enemies_MassValue_Destroyed",0.0).Value
        local massValueLost = self:GetArmyStat("Units_MassValue_Lost",0.0).Value
        local energyValueDestroyed = self:GetArmyStat("Enemies_EnergyValue_Destroyed",0.0).Value
        local energyValueLost = self:GetArmyStat("Units_EnergyValue_Lost",0.0).Value
        local commanderKills = self:GetArmyStat("Enemies_Commanders_Destroyed",0).Value

        -- helper variables to make equation more clear
        local excessMassProduced = massProduced - massSpent -- not currently being used
        local excessEnergyProduced = energyProduced - energySpent -- not currently being used
        local energyValueCoefficient = 20
        local commanderKillBonus = commanderKills + 1 -- not currently being used

        -- score components calculated
        local resourceProduction = ((massSpent) + (energySpent / energyValueCoefficient)) / 2
        local battleResults = (((massValueDestroyed - massValueLost) + ((energyValueDestroyed - energyValueLost) / energyValueCoefficient)) / 2)
        if battleResults < 0 then
            battleResults = 0
        end

        -- score calculated
        local score = math.floor(resourceProduction + battleResults)

        return score
    end,

    #####################################################################
    ## ------------- TRIGGERS BASED ON AN AI BRAIN       ------------- ##
    #####################################################################

    OnStatsTrigger = function(self, triggerName)
        #LOG('*AI DEBUG: ON STATS TRIGGER, TRIGGERNAME = ', repr(triggerName),' Triggers = ', repr(self.TriggerList))
        for k, v in self.TriggerList do
            if v.Name == triggerName then
                if v.CallingObject then
                    if not v.CallingObject:BeenDestroyed() then
                        v.CallbackFunction(v.CallingObject)
                    end
                else
                    v.CallbackFunction()
                end
                table.remove(self.TriggerList, k)
            end
        end
        if triggerName == 'EconLowEnergyStore' or triggerName == 'EconMidEnergyStore' or triggerName == 'EconFullEnergyStore' then
            self.EconStorageTrigs[triggerName] = false
            self:ESEnergyStorageUpdate(triggerName)
        elseif triggerName == 'EconLowMassStore' or triggerName == 'EconMidMassStore' or triggerName == 'EconFullMassStore' then
            self.EconStorageTrigs[triggerName] = false
            self:ESMassStorageUpdate(triggerName)
        end
    end,

    RemoveEconomyTrigger = function(self, triggerName)
        for k, v in self.TriggerList do
            if v.Name == triggerName then
                table.remove(self.TriggerList, k)
            end
        end
    end,

#    INTEL TRIGGER SPEC
#    {
#        CallbackFunction = <function>,
#        Type = 'LOS'/'Radar'/'Sonar'/'Omni',
#        Blip = true/false,
#        Value = true/false,
#        Category: blip category to match
#        OnceOnly: fire onceonly
#        TargetAIBrain: AI Brain of the army you want it to trigger off of.
#    },

    SetupArmyIntelTrigger = function(self, triggerSpec)
        table.insert(self.IntelTriggerList, triggerSpec)
    end,


    # Called when recon data changes for enemy units (e.g. A unit comes into line of sight)
    # Params
    #   blip: the unit (could be fake) in question
    #   type: 'LOSNow', 'Radar', 'Sonar', or 'Omni'
    #   val: true or false
    # calls callback function with blip it saw.
    OnIntelChange = function(self, blip, reconType, val)
        #LOG('*AI DEBUG: ONINTELCHANGED: Blip = ', repr(blip), ' ReconType = ', repr(reconType), ' Value = ', repr(val))
        #LOG('*AI DEBUG: IntelTriggerList = ', repr(self.IntelTriggerList))
        #LOG('*AI DEBUG: BlipID = ', repr(blip:GetBlueprint().BlueprintId))
        if self.IntelTriggerList then
            for k, v in self.IntelTriggerList do
                if EntityCategoryContains(v.Category, blip:GetBlueprint().BlueprintId)
                    and v.Type == reconType and (not v.Blip or v.Blip == blip:GetSource())
                    and v.Value == val and v.TargetAIBrain == blip:GetAIBrain() then
                    v.CallbackFunction(blip)
                    if v.OnceOnly then
                        self.IntelTriggerList[k] = nil
                    end
                end
            end
        end
    end,

    AddUnitBuiltPercentageCallback = function(self, callback, category, percent)
        if not callback or not category or not percent then
            error('*ERROR: Attempt to add UnitBuiltPercentageCallback but invalid data given',2)
        end
        table.insert(self.UnitBuiltTriggerList, { Callback=callback, Category=category, Percent=percent })
    end,

    SetupBrainVeterancyTrigger = function(self, triggerSpec)
        if not triggerSpec.CallCount then
            triggerSpec.CallCount = 1
        end
        table.insert(self.VeterancyTriggerList, triggerSpec)
    end,

    OnBrainUnitVeterancyLevel = function(self, unit, level)
        for k,v in self.VeterancyTriggerList do
            if EntityCategoryContains( v.Category, unit ) and level == v.Level and v.CallCount > 0 then
                v.CallCount = v.CallCount - 1
                v.CallbackFunction(unit)
            end
        end
    end,

    ##################################################################
    ## ------------- AI BRAIN FUNCTIONS HANDLED HERE  ------------- ##
    ##################################################################
    OnCreateAI = function(self, planName)
        #LOG('*AI DEBUG: AI planName = ', repr(planName))

        local aiScenarioPlans = self:ImportScenarioArmyPlans(planName)

        #LOG('*AI DEBUG: SCENARIO AI PLAN LIST = ', repr(aiScenarioPlans))
        if aiScenarioPlans then
            self.AIPlansList = aiScenarioPlans
        else
            self.DefaultPlan = true
            self.AIPlansList = AIDefaultPlansList
        end
        self.Trash = TrashBag()

        self.CurrentPlan = self.AIPlansList[self:GetFactionIndex()][1]

        #LOG('*AI DEBUG: AI PLAN LIST = ', repr(self.AIPlansList))
        #LOG('===== AI DEBUG: AI Brain Fork Theads =====')
        self.EvaluateThread = self:ForkThread(self.EvaluateAIThread)
        self.ExecuteThread = self:ForkThread(self.ExecuteAIThread)

        self.AttackData = {
            NeedSort = false,
            AMPlatoonCount = { DefaultGroupAir = 0, DefaultGroupLand = 0, DefaultGroupSea = 0, },
        }
        self.PlatoonNameCounter = {}
        self.PlatoonNameCounter['AttackForce'] = 0
        self.TriggerList = {}
        self.IntelTriggerList = {}
        self.VeterancyTriggerList = {}
        self.UnitBuiltTriggerList = {}
        self.FactoryAssistList = {}
        self.BaseTemplates = {}
        self.RepeatExecution = true
        self:InitializeEconomyState()
        self.BrainType = 'AI'
        self.IgnoreArmyCaps = false
        self.ConstantEval = true
    end,

    ImportScenarioArmyPlans = function(self, planName)
        if planName and planName != '' then
            #LOG('*AI DEBUG: IMPORTING PLAN NAME = ', repr(planName))
            return import(planName).AIPlansList
        else
            return nil
        end
    end,

    ForkThread = function(self, fn, ...)
        if fn then
            local thread = ForkThread(fn, self, unpack(arg))
            self.Trash:Add(thread)
            return thread
        else
            return nil
        end
    end,

    OnDestroy = function(self)
        if self.Trash then
            self.Trash:Destroy()
        end
        #LOG('===== AI DEBUG: Brain Evaluate Thead killed =====')
    end,

    OnDefeat = function(self)
        #LOG('I have been defeated.')
        self.defeated = true
        local units = self:GetListOfUnits(categories.ALLUNITS - categories.WALL, false)
        for index,unit in units do
            unit:Kill()
        end
        table.insert( Sync.GameResult, { self:GetArmyIndex(), "defeat" } )

        import('/lua/modules/SimUtils.lua').UpdateUnitCap()
    end,

    OnVictory = function(self)
        #LOG('I am teh winnr!')
        table.insert( Sync.GameResult, { self:GetArmyIndex(), "victory" } )
    end,

    IsDefeated = function(self)
        return self.defeated
    end,

    SetCurrentPlan = function(self, bestPlan)
        if not bestPlan then
            self.CurrentPlan = self.AIPlansList[self:GetFactionIndex()][1]
        else
            self.CurrentPlan = bestPlan
        end
        if not self.CurrentPlan then
            error( '*AI ERROR: Invalid plan list for army - '..self.Name, 2 )
        end
    end,

    SetConstantEvaluate = function(self, eval)
        if eval == true and self.ConstantEval == false then
            self.ConstantEval = eval
            self:ForkThread(self.EvaluateAIThread)
        end
        self.ConstantEval = eval
    end,

    EvaluateAIThread = function(self)
        local personality = self:GetPersonality()
        local factionIndex = self:GetFactionIndex()

        if not self.LayerPref then
            self:CalculateLayerPreference()
        end

        while self.ConstantEval do
            self:EvaluateAIPlanList()
            local delay = personality:AdjustDelay(100, 2)
            WaitTicks(delay)
        end
    end,

    EvaluateAIPlanList = function(self)
        local factionIndex = self:GetFactionIndex()
        #Forget faction index 4 or greater.
        if factionIndex > 3 then return end
        if self:IsOpponentAIRunning() then

            local bestPlan = nil
            local bestValue = 0
            for i,u in self.AIPlansList[factionIndex] do
                local value = self:EvaluatePlan(u)
                #LOG('*AI DEBUG: EVALUATED PLAN = ', repr(u), ' , VALUE = ', repr(value))
                if value > bestValue then
                    bestPlan = u
                    bestValue = value
                end
            end
            #LOG('*AI DEBUG: SETTING CURRENT PLAN = ', repr(bestPlan))
            if bestPlan then
                self:SetCurrentPlan( bestPlan )
                local bPlan = import(bestPlan)
                if bPlan != self.CurrentPlanScript then
                    self.CurrentPlanScript = import(bestPlan)
                    self:SetRepeatExecution(true)
                    self:ExecutePlan(self.CurrentPlan)
                end
            end
        end
    end,

    ExecuteAIThread = function(self)
        local personality = self:GetPersonality()

        while true do
            if self:IsOpponentAIRunning() and self.CurrentPlan and self.RepeatExecution then
                self:ExecutePlan(self.CurrentPlan)
            end

            local delay = personality:AdjustDelay(20, 4)
            WaitTicks(delay)

        end
    end,

    EvaluatePlan = function(self,  planName )
        #LOG('*AI DEBUG: EVALUATE PLAN IN AIBRAIN, PLANNAME = ', repr(planName))
        local plan = import(planName)
        if plan then
            return plan.EvaluatePlan(self)
        else
            LOG('*WARNING: TRIED TO IMPORT PLAN NAME ', repr(planName), ' BUT IT ERRORED OUT IN THE AI BRAIN.')
            return 0
        end
    end,

    ExecutePlan = function(self)
        self.CurrentPlanScript.ExecutePlan(self)
    end,

    SetRepeatExecution = function(self, repeatEx)
        self.RepeatExecution = repeatEx
    end,

    GetCurrentPlanScript = function(self)
        return self.CurrentPlanScript
    end,

    IgnoreArmyUnitCap = function(self, val)
        self.IgnoreArmyCaps = val
        SetIgnoreArmyCap(self, val)
    end,


    ##################################################################
    ## ---------- System for playing VOs to the Player ------------ ##
    ##################################################################
    InitializeVO = function(self)
        if not self.VOTable then
            self.VOTable = {
            }
        end
    end,

    PlayVOSound = function(self, sound, string)
        local cue,bank = GetCueBank(sound)
        table.insert(Sync.Sounds, {Cue=cue, Bank=bank} )
        local time = VOReplayTime[string]
        WaitSeconds(time)
        self.VOTable[string] = nil
    end,

    ExperimentalDetected = function(self, sound)
        if self.VOTable and not self.VOTable['ExperimentalDetected'] then
            self.VOTable['ExperimentalDetected'] = ForkThread(self.PlayVOSound, self, sound, 'ExperimentalDetected')
        end
    end,

    EnemyForcesDetected = function(self, sound)
        if self.VOTable and not self.VOTable['EnemyForcesDetected'] then
            self.VOTable['EnemyForcesDetected'] = ForkThread(self.PlayVOSound, self, sound, 'EnemyForcesDetected')
        end
    end,

    EnemyNavalForcesDetected = function(self, sound)
        if self.VOTable and not self.VOTable['EnemyNavalForcesDetected'] then
            self.VOTable['EnemyNavalForcesDetected'] = ForkThread(self.PlayVOSound, self, sound, 'EnemyNavalForcesDetected')
        end
    end,

    FerryPointSet = function(self,sound)
        if self.VOTable and not self.VOTable['FerryPointSet'] then
            self.VOTable['FerryPointSet'] = ForkThread(self.PlayVOSound, self, sound, 'FerryPointSet')
        end
    end,

    UnderAttack = function(self,sound)
        if self.VOTable and not self.VOTable['UnderAttack'] then
            self.VOTable['UnderAttack'] = ForkThread(self.PlayVOSound, self, sound, 'UnderAttack')
        end
    end,

    BaseUnderAttack = function(self,sound)
        if self.VOTable and not self.VOTable['BaseUnderAttack'] then
            self.VOTable['BaseUnderAttack'] = ForkThread(self.PlayVOSound, self, sound, 'BaseUnderAttack')
        end
    end,

    NukeArmed = function(self,sound)
        if self.VOTable and not self.VOTable['NukeArmed'] then
            self.VOTable['NukeArmed'] = ForkThread(self.PlayVOSound, self, sound, 'NukeArmed')
        end
    end,

    NuclearLaunchInitiated = function(self,sound)
        if self.VOTable and not self.VOTable['NuclearLaunchInitiated'] then
            self.VOTable['NuclearLaunchInitiated'] = ForkThread(self.PlayVOSound, self, sound, 'NuclearLaunchInitiated')
        end
    end,

    NuclearLaunchDetected = function(self,sound)
        if self.VOTable and not self.VOTable['NuclearLaunchDetected'] then
            self.VOTable['NuclearLaunchDetected'] = ForkThread(self.PlayVOSound, self, sound, 'NuclearLaunchDetected')
        end
    end,

    ExperimentalUnitDestroyed = function(self,sound)
        if self.VOTable and not self.VOTable['ExperimentalUnitDestroyed'] then
            self.VOTable['ExperimentalUnitDestroyed'] = ForkThread(self.PlayVOSound, self, sound, 'ExperimentalUnitDestroyed')
        end
    end,

    BattleshipDestroyed = function(self,sound)
        if self.VOTable and not self.VOTable['BattleshipDestroyed'] then
            self.VOTable['BattleshipDestroyed'] = ForkThread(self.PlayVOSound, self, sound, 'BattleshipDestroyed')
        end
    end,



    ##################################################################
    ## --------------- SKIRMISH AI HELPER SYSTEMS  ---------------- ##
    ##################################################################

    CalculateLayerPreference = function(self)
        local personality = self:GetPersonality()
        local factionIndex = self:GetFactionIndex()
        #SET WHAT THE AI'S LAYER PREFERENCE IS.
        #LOG('*AI DEBUG: PERSONALITY = ', repr(personality))
        local airpref = personality:GetAirUnitsEmphasis() * 100
        local tankpref = personality:GetTankUnitsEmphasis() * 100
        local botpref = personality:GetBotUnitsEmphasis() * 100
        local seapref = personality:GetSeaUnitsEmphasis() * 100
        local landpref = tankpref
        if tankpref < botpref then
            landpref = botpref
        end
        #SEA PREF COMMENTED OUT FOR NOW
        local totalpref = landpref + airpref  + seapref
        totalpref = totalpref
        local random = Random(0, totalpref)
#            LOG('*AI DEBUG: LANDPREF LAYER PREF = ', repr(landpref))
#            LOG('*AI DEBUG: AIRPREF FOR LAYER PREF = ', repr(airpref))
#            LOG('*AI DEBUG: SEAPREF FOR LAYER PREF = ', repr(seapref))
#            LOG('*AI DEBUG: TOTAL FOR LAYER PREF = ', repr(totalpref))
#            LOG('*AI DEBUG: RANDOM NUMBER FOR LAYER PREF = ', repr(random))
        if random < landpref then
            self.LayerPref = 'LAND'
        elseif random < (landpref + airpref) then
            self.LayerPref = 'AIR'
        else
            self.LayerPref = 'LAND'
            #COMMENTING OUT SEA FOR NOW
            #self.LayerPref = 'SEA'
        end
        #LOG('*AI DEBUG: LAYER PREFERENCE = ', repr(self.LayerPref))
    end,

    AIGetLayerPreference = function(self)
        return self.LayerPref
    end,

    ##################################################################
    ## --------------- AI ATTACK MANAGEMENT  ---------------------- ##
    ##################################################################

# ATTACK MANAGER SPEC
#{
#    AttackCheckInterval = interval,
#    Platoons = {
#        {
#            PlatoonName = string,
#            AttackConditions = { function, {args} },
#            AIThread = function, -- If AMPlatoon needs a specific function
#            AIName = string, -- AIs from platoon.lua
#            Priority = num,
#            PlatoonData = table,
#            OverrideFormation = string, -- formation to use for the attack platoon
#            FormCallbacks = table, -- table of functions called when an AM Platoon forms
#            DestroyCallbacks = table, -- table of functions called when the platoon is destroyed
#            LocationType = string, -- location from PBM -- used if you want to get units from pool
#            PlatoonType = string, -- 'Air', 'Sea', 'Land' -- MUST BE SET IF UsePool IS TRUE
#            UsePool = bool, -- bool to use pool or not
#        },
#    },
#}
#
# Spec for Platoons - within PlatoonData
# PlatoonData = {
#     AMPlatoons = { AMPlatoonName, AMPlatoonName, etc },
# },

    InitializeAttackManager = function(self, attackDataTable)
        if not self.AttackData then
            self.AttackData = {
                NeedSort = false,
            }
        end
        self:AMAddDefaultPlatoons(attackDataTable.AttackConditions)
        if attackDataTable then
            self.AttackData.AttackCheckInterval = attackDataTable.AttackCheckInterval or 13
            if attackDataTable.Platoons then
                self:AMAddPlatoonsTable(attackDataTable.Platoons)
            end
        elseif not self.AttackData.AttackCheckInterval then
            self.AttackData.AttackCheckInterval = 13
        end
        self.AttackData['AttackManagerState'] = 'ACTIVE'
        self.AttackData['AttackManagerThread'] = self:ForkThread(self.AttackManagerThread)
    end,

    AttackManagerThread = function(self)
        local ad = self.AttackData
        local alertLocation, alertLevel, tempLevel
        while true do
            WaitSeconds(ad.AttackCheckInterval)
            if ad.AttackManagerState == 'ACTIVE' and self.AttackData.Platoons then
                self:AMAttackManageAttackVectors()
                self:AMFormAttackPlatoon()
            end
        end
    end,

    AMAddDefaultPlatoons = function(self, AttackConds)
        local atckCond = {}
        if not AttackConds then
            AttackConds = {
                    { '/lua/modules/editor/platooncountbuildconditions.lua', 'NumGreaterOrEqualAMPlatoons', {'default_brain', 'DefaultGroupAir', 3} },
            }
        end
        local platoons = {
            {
                PlatoonName = 'DefaultGroupAir',
                AttackConditions = AttackConds,
                AIName = 'HuntAI',
                Priority = 1,
                PlatoonType = 'Air',
                UsePool = true,
            },
            {
                PlatoonName = 'DefaultGroupLand',
                AttackConditions = AttackConds,
                AIName = 'AttackForceAI',
                Priority = 1,
                PlatoonType = 'Land',
                UsePool = true,
            },
            {
                PlatoonName = 'DefaultGroupSea',
                AttackConditions = AttackConds,
                AIName = 'HuntAI',
                Priority = 1,
                PlatoonType = 'Sea',
                UsePool = true,
            },
        }
        self:AMAddPlatoonsTable(platoons)
    end,

    AMAddPlatoonsTable = function(self, platoons)
        for k,v in platoons do
            self:AMAddPlatoon(v)
        end
    end,

    AMAddPlatoon = function(self, pltnTable)
        if not pltnTable.AttackConditions then
            error('*AI WARNING: INVALID ATTACK MANAGER PLATOON LIST - Missing AttackConditions', 2)
            return
        end
        if not pltnTable.AIThread and not pltnTable.AIName then
            error('*AI WARNING: INVALID ATTACK MANAGER PLATOON LIST - Mission either AIName or AIThread', 2)
            return
        end
        if not pltnTable.Priority then
            error('*AI WARNING: INVALID ATTACK MANAGER PLATOON LIST - Missing Priority', 2)
            return
        end
        if not pltnTable.UsePool then
            pltnTable.UsePool = false
        end
        if not self.AttackData then
            self.AttackData = {}
        end
        if not self.AttackData.Platoons then
            self.AttackData.Platoons = {}
        end
        self.AttackData.NeedSort = true
        table.insert( self.AttackData.Platoons, pltnTable )
    end,

    AMClearPlatoonList = function(self)
        self.AttackData.Platoons = {}
        self.AttackData.NeedSort = false
    end,

    AMSetAttackCheckInterval = function(self, interval)
        self.AttackData.AttackCheckInterval = interval
    end,

    AMCheckAttackConditions = function(self, pltnInfo)
        for k, v in pltnInfo.AttackConditions do
            if v[3][1] == "default_brain" then
                table.remove(v[3], 1)
            end
            if iscallable(v[1]) then
                if not v[1](self, unpack(v[2])) then
                    return false
                end
            else
                if not import(v[1])[v[2]](self, unpack(v[3])) then
                    return false
                end
            end
        end
        return true
    end,

    AMSortPlatoonsViaPriority = function(self)
        local sortedList = {}
        #Simple selection sort, this can be made faster later if we decide we need it.
        if self.AttackData.Platoons then
            for i = 1, table.getn(self.AttackData.Platoons) do
                local highest = 0
                local key, value
                for k, v in self.AttackData.Platoons do
                    if v.Priority > highest then
                        highest = v.Priority
                        value = v
                        key = k
                    end
                end
                sortedList[i] = value
                table.remove(self.AttackData.Platoons, key)
            end
            self.AttackData.Platoons = sortedList
        end
        self.AttackData.NeedSort = false
        return sortedList
    end,

    AMFormAttackPlatoon = function(self)
        local attackForcePL = {}
        local namedPlatoonList = {}
        local poolPlatoon = self:GetPlatoonUniquelyNamed('ArmyPool')
        if poolPlatoon then
            table.insert(attackForcePL, poolPlatoon)
        end
        if self.AttackData.NeedSort then
            self:AMSortPlatoonsViaPriority()
        end
        for k,v in self.AttackData.Platoons do
            if self:AMCheckAttackConditions(v) then
                local combineList = {}
                local platoonList = self:GetPlatoonsList()
                for j, platoon in platoonList do
                    if platoon:IsPartOfAttackForce() then
                        for i, name in platoon.PlatoonData.AMPlatoons do
                            if name == v.PlatoonName then
                                table.insert( combineList, platoon )
                            end
                        end
                    end
                end
                if table.getn(combineList) > 0 then
                    local tempPlatoon
                    if self.AttackData.Platoons[k].AIName then
                        tempPlatoon = self:CombinePlatoons(combineList, v.AIName)
                    else
                        tempPlatoon = self:CombinePlatoons(combineList)
                    end
                    local formation = 'GrowthFormation'
                    if v.OverrideFormation then
                        tempPlatoon:SetPlatoonFormationOverride(v.OverrideFormation)
                    end
                    if v.UsePool then
                        local checkCategory
                        if v.PlatoonType == 'Air' then
                            checkCategory = categories.AIR * categories.MOBILE
                        elseif v.PlatoonType == 'Land' then
                            checkCategory = categories.LAND * categories.MOBILE - categories.ENGINEER - categories.EXPERIMENTAL
                        elseif v.PlatoonType == 'Sea' then
                            checkCategory = categories.NAVAL * categories.MOBILE
                        elseif v.PlatoonType == 'Any' then
                            checkCategory = categories.MOBILE - categories.ENGINEER
                        else
                            error('*AI WARNING: Invalid Platoon Type - ' .. v.PlatoonType, 2)
                            break
                        end
                        local poolPlatoon = self:GetPlatoonUniquelyNamed('ArmyPool')
                        local poolUnits = poolPlatoon:GetPlatoonUnits()
                        local addUnits = {}
                        if v.LocationType then
                            local location = false
                            for locNum, locData in self.PBM.Locations do
                                if v.LocationType == locData.LocationType then
                                    location = locData
                                    break
                                end
                            end
                            if not location then
                                error('*AI WARNING: Invalid location type - ' .. v.LocationType, 2)
                                break
                            end
                            for i,unit in poolUnits do
                                if Utilities.GetDistanceBetweenTwoVectors(unit:GetPosition(), location.Location) <= location.Radius
                                    and EntityCategoryContains(checkCategory, unit) then
                                        table.insert(addUnits, unit)
                                end
                            end
                        else
                            for i,unit in poolUnits do
                                if EntityCategoryContains(checkCategory, unit) then
                                    table.insert(addUnits, unit)
                                end
                            end
                        end
                        self:AssignUnitsToPlatoon(tempPlatoon, addUnits, 'Attack', formation)
                    end
                    if v.PlatoonData then
                        tempPlatoon:SetPlatoonData(v.PlatoonData)
                    else
                        tempPlatoon.PlatoonData = {}
                    end
                    tempPlatoon.PlatoonData.PlatoonName = v.PlatoonName
                    #LOG('*AM DEBUG: AM Master Platoon Formed, Builder Named: ', repr(v.BuilderName))
                    if v.AIThread then
                        tempPlatoon:ForkAIThread(import(v.AIThread[1])[v.AIThread[2]])
                        #LOG('*AM DEBUG: AM Master Platoon using AI Thread: ', repr(v.AIThread[2]), ' Builder named: ', repr(v.BuilderName))
                    end
                    if v.DestroyCallbacks then
                        for dcbNum, destroyCallback in v.DestroyCallbacks do
                            tempPlatoon:AddDestroyCallback(import(destroyCallback[1])[destroyCallback[2]])
                            #LOG('*AM DEBUG: AM Master Platoon adding destroy callback: ', destroyCallback[2], ' Builder named: ', repr(v.BuilderName))
                        end
                    end
                    if v.FormCallbacks then
                        for cbNum, callback in v.FormCallbacks do
                            ForkThread(import(callback[1])[callback[2]], tempPlatoon)
                            #LOG('*AM DEBUG: AM Master Platoon Form callback: ', repr(callback[2]), ' Builder Named: ', repr(v.BuilderName))
                        end
                    end
                end
            end
        end
    end,

    AMDestroyAttackManager = function(self)
        if self.AttackData.AttackManagerThread then
            self.AttackData.AttackManagerThread:Destroy()
            self.AttackData.AttackManagerThread = nil
        end
    end,

    AMPauseAttackManager = function(self)
        self.AttackData.AttackManagerState = 'PAUSED'
    end,

    AMUnPauseAttackManager = function(self)
        self.AttackData.AttackManagerState = 'ACTIVE'
    end,

    AMIsAttackManagerActive = function(self)
        if self.AttackData and self.AttackData.AttackManagerThread and self.AttackData.AttackManagerState == 'ACTIVE' then
            return true
        end
        return false
    end,

    AMGetNumberAttackForcePlatoons = function(self)
        local platoonList = self:GetPlatoonsList()
        local result = 0
        for k, v in platoonList do
            if v:IsPartOfAttackForce() then
                result = result + 1
            end
        end
        #Add in pool platoon, pool platoon is always used.
        result = result + 1
        return result
    end,

    AMAttackManageAttackVectors = function(self)
        local enemyBrain = self:GetCurrentEnemy()
        if enemyBrain then
            self:SetUpAttackVectorsToArmy()
        end
    end,

    AMDecrementCount = function(self, platoon)
        local data = platoon.PlatoonData
        for k,v in data.AMPlatoons do
            self.AttackData.AMPlatoonCount[v] = self.AttackData.AMPlatoonCount[v] - 1
        end
    end,

    ##################################################################
    ## ------------- AI PLATOON MANAGEMENT  ----------------------- ##
    ##################################################################
    #New PlatoonBuildManager
    #This system is meant to be able to give some data about the platoon you want and have them
    #built and formed into platoons at will.


    InitializePlatoonBuildManager = function(self)
        if not self.PBM then
            self.PBM = {
                Platoons = {
                    Air = {},
                    Land = {},
                    Sea = {},
                    Gate = {},
                },
                Locations = {
                    # {
                    #   Location,
                    #   Radius,
                    #   LocType, ('MAIN', 'EXPANSION')
                    #   PrimaryFactories = { Air = X, Land = Y, Sea = Z}
                    #   UseCenterPoint, - Bool
                    # }
                },
                PlatoonTypes = { 'Air', 'Land', 'Sea', 'Gate'},
                NeedSort = {
                    ['Air'] = false,
                    ['Land'] = false,
                    ['Sea'] = false,
                    ['Gate'] = false,
                },
                RandomSamePriority = false,
            }
            #Create basic starting area
            local strtX, strtZ = self:GetArmyStartPos()
            self:PBMAddBuildLocation({strtX, 20, strtZ}, 100, 'MAIN')
            #TURNING OFF AI POOL PLATOON, I MAY JUST REMOVE THAT PLATOON FUNCTIONALITY LATER
            local poolPlatoon = self:GetPlatoonUniquelyNamed('ArmyPool')
            if poolPlatoon then
                poolPlatoon:TurnOffPoolAI()
            end
            self.HasPlatoonList = false
            self:ForkThread(self.PlatoonBuildManagerThread)
        end
    end,


#Platoon Spec
#{
#        PlatoonTemplate = platoon template,
#        InstanceCount = number of duplicates to place in the platoon list
#        Priority = integer,
#        BuildConditions = list of functions that return true/false, list of args,  { < function>, {<args>}}
#        LocationType = string for type of location, setup via addnewlocation function,
#        BuildTimeOut = how long it'll try to form this platoon after it's been told to build.,
#        PlatoonType = 'Air'/'Land'/'Sea' basic type of unit, used for finding what type of factory to build from,
#        RequiresConstruction = true/false do I need to build this from a factory or should I just try to form it?,
#        PlatoonBuildCallbacks = {FunctionsToCallBack when the platoon starts to build}
#        PlatoonAIFunction = if nil uses function in platoon.lua, function for the main AI thread
#        PlatoonAddFunctions = {<other threads to be forked on this platoon>}
#        PlatoonData = {
#            Construction = {
#                BaseTemplate = basetemplates, must contain templates for all 3 factions it will be viewed by faction index,
#                BuildingTemplate = building templates, contain templates for all 3 factions it will be viewed by faction index,
#                BuildClose = true/false do I follow the table order or do build the best spot near me?
#                BuildRelative = true/false are the build coordinates relative to the starting location or absolute coords?,
#                BuildStructures = { List of structure types and the order to build them.}
#            }
#        }
#    },

    PBMAddPlatoon = function(self, pltnTable)
        if not pltnTable.PlatoonTemplate then
            local stng = '*AI ERROR: INVALID PLATOON LIST IN '.. self.CurrentPlan.. ' - MISSING TEMPLATE.  '
            error(stng, 1)
            return
        end
        if pltnTable.RequiresConstruction == nil then
            error('*AI ERROR: INVALID PLATOON LIST IN ' .. self.CurrentPlan .. ' - MISSING RequiresConstruction', 1)
            return
        end
        if not pltnTable.Priority then
            error('*AI ERROR: INVALID PLATOON LIST IN ' .. self.CurrentPlan .. ' - MISSING PRIORITY', 1)
            return
        end
        if not pltnTable.BuildConditions then
            pltnTable.BuildConditions = {}
        end
        if not pltnTable.BuildTimeOut or pltnTable.BuildTimeOut == 0 then
            pltnTable.GenerateTimeOut = true
        end

        local num = 1
        if pltnTable.InstanceCount and pltnTable.InstanceCount > 1 then
            num = pltnTable.InstanceCount
        end

        if not ScenarioInfo.BuilderTable[self.CurrentPlan] then
            ScenarioInfo.BuilderTable[self.CurrentPlan] = { Air = {}, Sea = {}, Land = {}, Gate = {} }
        end

        if pltnTable.PlatoonType ~= 'Any' then
            if not ScenarioInfo.BuilderTable[self.CurrentPlan][pltnTable.PlatoonType][pltnTable.BuilderName] then
                ScenarioInfo.BuilderTable[self.CurrentPlan][pltnTable.PlatoonType][pltnTable.BuilderName] = pltnTable
            elseif not pltnTable.Inserted then
                error('AI DEBUG: BUILDER DUPLICATE NAME FOUND - ' .. pltnTable.BuilderName, 2)
            end
            for i=1,num do
                table.insert( self.PBM.Platoons[pltnTable.PlatoonType], { BuilderName = pltnTable.BuilderName, PlatoonHandle = false, Priority = pltnTable.Priority, LocationType = pltnTable.LocationType, PlatoonTemplate = pltnTable.PlatoonTemplate } )
                self.PBM.NeedSort[pltnTable.PlatoonType] = true
            end
        else
            local brainBuilderTable = {}
            for i=1,num do
                table.insert( brainBuilderTable, { BuilderName = pltnTable.BuilderName, PlatoonHandle = false, Priority = pltnTable.Priority, LocationType = pltnTable.LocationType, PlatoonTemplate = pltnTable.PlatoonTemplate } )
            end
            local types = { 'Air', 'Land', 'Sea' }
            for num, pType in types do
                if not ScenarioInfo.BuilderTable[self.CurrentPlan][pType][pltnTable.BuilderName] then
                    ScenarioInfo.BuilderTable[self.CurrentPlan][pType][pltnTable.BuilderName] = pltnTable
                elseif not pltnTable.Inserted then
                    error('AI DEBUG: BUILDER DUPLICATE NAME FOUND - ' .. pltnTable.BuilderName, 2)
                end
                for k,v in brainBuilderTable do
                    table.insert( self.PBM.Platoons[pType], v )
                    self.PBM.NeedSort[pType] = true
                end
            end
        end

        self.HasPlatoonList = true
    end,

    #Function to clear all the platoon lists so you can feed it a bunch more.
    #formPlatoons - Gives you the option to form all the platoons in the list before its cleaned up so that
    #you don't have units hanging around.
    PBMClearPlatoonList = function(self, formPlatoons)
        if formPlatoons then
            for k, v in self.PBM.PlatoonTypes do
                self:PBMFormPlatoons(false, v)
            end
        end
        self.PBM.NeedSort['Air'] = false
        self.PBM.NeedSort['Land'] = false
        self.PBM.NeedSort['Sea'] = false
        self.PBM.NeedSort['Gate'] = false
        self.HasPlatoonList = false
        self.PBM.Platoons = {
            Air = {},
            Land = {},
            Sea = {},
            Gate = {},
        }
    end,

    PBMHasPlatoonList = function(self)
        return self.HasPlatoonList
    end,

    PBMResetPrimaryFactories = function(self)
        for k, v in self.PBM.Locations do
            v.PrimaryFactories.Air = nil
            v.PrimaryFactories.Land = nil
            v.PrimaryFactories.Sea = nil
            v.PrimaryFactories.Gate = nil
        end
    end,

    #Goes through the location areas, finds the factories, sets a primary then tells all the others to guard.
    PBMSetPrimaryFactories = function(self)
        for k, v in self.PBM.Locations do
            local factories = self:GetAvailableFactories(v.Location, v.Radius)
            local airFactories = {}
            local landFactories = {}
            local seaFactories = {}
            local gates = {}
            for ek, ev in factories do
                if EntityCategoryContains(categories.FACTORY * categories.AIR, ev) and self:PBMFactoryLocationCheck( ev, v ) then
                    table.insert(airFactories, ev)
                elseif EntityCategoryContains(categories.FACTORY * categories.LAND, ev) and self:PBMFactoryLocationCheck( ev, v ) then
                    table.insert(landFactories, ev)
                elseif EntityCategoryContains(categories.FACTORY * categories.NAVAL, ev) and self:PBMFactoryLocationCheck( ev, v ) then
                    table.insert(seaFactories, ev)
                elseif EntityCategoryContains(categories.FACTORY * categories.GATE, ev) and self:PBMFactoryLocationCheck( ev, v ) then
                    table.insert(gates, ev)
                end
            end
            local afac, lfac, sfac, gatefac
            if table.getn(airFactories) > 0 then
                if not v.PrimaryFactories.Air or v.PrimaryFactories.Air:IsDead()
                    or v.PrimaryFactories.Air:IsUnitState('Upgrading')
                    or self:PBMCheckHighestTechFactory(airFactories, v.PrimaryFactories.Air) then
                        fac = self:PBMGetPrimaryFactory(airFactories)
                        v.PrimaryFactories.Air = fac
                end
                self:PBMAssistGivenFactory( airFactories, v.PrimaryFactories.Air )
            end
            if table.getn(landFactories) > 0 then
                if not v.PrimaryFactories.Land or v.PrimaryFactories.Land:IsDead()
                    or v.PrimaryFactories.Land:IsUnitState('Upgrading')
                    or self:PBMCheckHighestTechFactory(landFactories, v.PrimaryFactories.Land) then
                        lfac = self:PBMGetPrimaryFactory(landFactories)
                        v.PrimaryFactories.Land = lfac
                end
                self:PBMAssistGivenFactory( landFactories, v.PrimaryFactories.Land )
            end
            if table.getn(seaFactories) > 0 then
                if not v.PrimaryFactories.Sea or v.PrimaryFactories.Sea:IsDead()
                    or v.PrimaryFactories.Sea:IsUnitState('Upgrading')
                    or self:PBMCheckHighestTechFactory(seaFactories, v.PrimaryFactories.Sea) then
                        sfac = self:PBMGetPrimaryFactory(seaFactories)
                        v.PrimaryFactories.Sea = sfac
                end
                self:PBMAssistGivenFactory( seaFactories, v.PrimaryFactories.Sea )
            end
            if table.getn(gates) > 0 then
                if not v.PrimaryFactories.Gate or v.PrimaryFactories.Gate:IsDead() then
                    gatefac = self:PBMGetPrimaryFactory(gates)
                    v.PrimaryFactories.Gate = gatefac
                end
                self:PBMAssistGivenFactory( gates, v.PrimaryFactories.Gate )
            end
            if not v.RallyPoint or table.getn(v.RallyPoint) == 0 then
                self:PBMSetRallyPoint(airFactories, v, nil)
                self:PBMSetRallyPoint(landFactories, v, nil)
                self:PBMSetRallyPoint(seaFactories, v, nil)
                self:PBMSetRallyPoint(gates, v, nil)
            end
        end
    end,

    PBMAssistGivenFactory = function( self, factories, primary )
        for k,v in factories do
            if not v:IsDead() and not ( v:IsUnitState('Building') or v:IsUnitState('Upgrading') ) then
                local guarded = v:GetGuardedUnit()
                if not guarded or guarded:GetEntityId() ~= primary:GetEntityId() then
                    IssueClearCommands( {v} )
                    IssueFactoryAssist( {v}, primary )
                end
            end
        end
    end,

    PBMSetRallyPoint = function(self, factories, location, rallyLoc, markerType)
        if table.getn(factories) > 0 then
            local rally
            local position = factories[1]:GetPosition()
            for facNum, facData in factories do
                if facNum > 1 then
                    position[1] = position[1] + facData:GetPosition()[1]
                    position[3] = position[3] + facData:GetPosition()[3]
                end
            end
            position[1] = position[1] / table.getn(factories)
            position[3] = position[3] / table.getn(factories)
            if not rallyLoc and not location.UseCenterPoint then
                local pnt
                if not markerType then
                    pnt = AIUtils.AIGetClosestMarkerLocation( self, 'Rally Point', position[1], position[3] )
                else
                    pnt = AIUtils.AIGetClosestMarkerLocation( self, markerType, position[1], position[3] )
                end
                if(pnt and table.getn(pnt) == 3) then
                    rally = Vector(pnt[1], pnt[2], pnt[3])
                end
            elseif not rallyLoc and location.UseCenterPoint then
                rally = location.Location
            elseif rallyLoc then
                rally = rallyLoc
            else
                error('*ERROR: PBMSetRallyPoint - Missing Rally Location and Marker Type', 2)
                return false
            end
            if(rally) then
                for k,v in factories do
                    IssueClearFactoryCommands({v})
                    IssueFactoryRallyPoint({v}, rally)
                end
            end
        end
        return true
    end,

    PBMFactoryLocationCheck = function(self, factory, location)
        if not factory.PBMData then
            factory.PBMData = {}
        end
        if not factory.PBMData[location.LocationType] then
            local pos = factory:GetPosition()
            factory.PBMData[location.LocationType] = VDist2( location.Location[1], location.Location[3], pos[1], pos[3] )
        end
        local closest, distance
        for k,v in factory.PBMData do
            if not distance or v < distance then
                distance = v
                closest = k
            end
        end
        if closest and closest == location.LocationType then
            return true
        else
            return false
        end
    end,

    PBMCheckHighestTechFactory = function(self, factories, primary)
        local catTable = { categories.TECH1, categories.TECH2, categories.TECH3 }
        local catLevel = 1
        if EntityCategoryContains( categories.TECH3, primary ) then
            catLevel = 3
        elseif EntityCategoryContains( categories.TECH2, primary ) then
            catLevel = 2
        end
        for catNum, cat in catTable do
            if catNum > catLevel then
                for unitNum, unit in factories do
                    if not unit:IsDead() and EntityCategoryContains( cat, unit ) and not unit:IsUnitState('Upgrading') then
                        return true
                    end
                end
            end
        end
        return false
    end,

    #Picks the first tech 3, tech 2 or tech 1 factory to make primary
    PBMGetPrimaryFactory = function(self, factories)
        local categoryTable = {categories.TECH3, categories.TECH2, categories.TECH1}
        for kc, vc in categoryTable do
            for k, v in factories do
                if EntityCategoryContains(vc, v) and not v:IsUnitState('Upgrading') then
                    return v
                end
            end
        end
    end,
    
    PBMGetPriority = function(self, platoon)
        for typek, typev in self.PBM.PlatoonTypes do
            for k,v in self.PBM.Platoons[typev] do
                if v.PlatoonHandle and v.PlatoonHandle == platoon then
                    return v.Priority
                end
            end
        end
    end,

    PBMGetPriority = function(self, platoon)
        for typek, typev in self.PBM.PlatoonTypes do
            for k,v in self.PBM.Platoons[typev] do
                if v.PlatoonHandle and v.PlatoonHandle == platoon then
                    return v.Priority
                end
            end
        end
    end,

    PBMAdjustPriority = function(self, platoon, amount)
        for typek, typev in self.PBM.PlatoonTypes do
            for k, v in self.PBM.Platoons[typev] do
                if v.PlatoonHandle and v.PlatoonHandle == platoon then
                    if typev == 'Any' then
                        self.PBM.NeedSort['Air'] = true
                        self.PBM.NeedSort['Sea'] = true
                        self.PBM.NeedSort['Land'] = true
                    else
                        self.PBM.NeedSort[typev] = true
                    end
                    v.Priority = v.Priority + amount
                end
            end
        end
    end,

    PBMSetPriority = function(self, platoon, amount)
        for typek, typev in self.PBM.PlatoonTypes do
            for k, v in self.PBM.Platoons[typev] do
                if v.PlatoonHandle and v.PlatoonHandle == platoon then
                    if typev == 'Any' then
                        self.PBM.NeedSort['Air'] = true
                        self.PBM.NeedSort['Sea'] = true
                        self.PBM.NeedSort['Land'] = true
                    else
                        self.PBM.NeedSort[typev] = true
                    end
                    v.Priority = amount
                end
            end
        end
    end,

    #Adds a new build location
    PBMAddBuildLocation = function(self, loc, radius, locType, useCenterPoint)
        if not radius or not loc or not locType then
            error('*AI ERROR: INVALID BUILD LOCATION FOR PBM', 2)
            return false
        end
        if type(loc) == 'string' then
            loc = ScenarioUtils.MarkerToPosition(loc)
        end
        useCenterPoint = useCenterPoint or false
        local spec = {
            Location = loc,
            Radius = radius,
            LocationType = locType,
            PrimaryFactories = { Air = nil, Land = nil, Sea = nil, Gate = nil},
            UseCenterPoint = useCenterPoint,
        }
        local found = false
        for num, loc in self.PBM.Locations do
            if loc.LocationType == spec.LocationType then
                found = true
                break
            end
        end
        if not found then
            table.insert(self.PBM.Locations, spec)
        else
            error('*AI  ERROR: Attempting to add a build location with a duplicate name: '..spec.LocationType, 2)
            return
        end
    end,

    PBMGetLocationCoords = function(self, loc)
        if not loc then
            return false
        end
        for k, v in self.PBM.Locations do
            if v.LocationType == loc then
                local height = GetTerrainHeight( v.Location[1], v.Location[3] )
                if GetSurfaceHeight( v.Location[1], v.Location[3] ) > height then
                    height = GetSurfaceHeight( v.Location[1], v.Location[3] )
                end
                return { v.Location[1], height, v.Location[3] }
            end
        end
        return false
    end,

    PBMGetLocationRadius = function( self, loc )
        if not loc then
            return false
        end
        for k, v in self.PBM.Locations do
            if v.LocationType == loc then
               return v.Radius
            end
        end
        return false
    end,
    
    PBMGetLocationFactories = function(self, location)
        if not location then
            return false
        end
        for k,v in self.PBM.Locations do
            if v.LocationType == location then
                return v.PrimaryFactories
            end
        end
        return false
    end,

    PBMGetLocationFactories = function(self, location)
        if not location then
            return false
        end
        for k,v in self.PBM.Locations do
            if v.LocationType == location then
                return v.PrimaryFactories
            end
        end
        return false
    end,

    #Removes a build location based on it area
    #IF either is nil, then it will do the other.
    #This way you can remove all of one type or all of one rectangle
    PBMRemoveBuildLocation = function(self, loc, locType)
        for k, v in self.PBM.Locations do
            if (loc and v.Location == loc) or (locType and v.LocationType == locType) then
                table.remove(self.PBM.Locations, k)
            end
        end
    end,

    #Sort platoon list
    #PlatoonType = 'Air', 'Land' or 'Sea'
    PBMSortPlatoonsViaPriority = function(self, platoonType)
         if platoonType != 'Air' and platoonType != 'Land' and platoonType != 'Sea' and platoonType != 'Gate' then
            local strng = '*AI ERROR: TRYING TO SORT PLATOONS VIA PRIORITY BUT AN INVALID TYPE (', repr(platoonType),') WAS PASSED IN.'
            error(strng, 2)
            return false
        end
        local sortedList = {}
        #Simple selection sort, this can be made faster later if we decide we need it.
        for i = 1, table.getn(self.PBM.Platoons[platoonType]) do
            local highest = 0
            local key, value
            for k, v in self.PBM.Platoons[platoonType] do
                if v.Priority > highest then
                    highest = v.Priority
                    value = v
                    key = k
                end
            end
            sortedList[i] = value
            table.remove(self.PBM.Platoons[platoonType], key)
        end
        self.PBM.Platoons[platoonType] = sortedList
        self.PBM.NeedSort[platoonType] = false
    end,

    PBMSetCheckInterval = function(self, interval)
        self.PBM.BuildCheckInterval = interval
    end,

    PBMEnableRandomSamePriority = function(self)
        self.PBM.RandomSamePriority = true
    end,

    PBMDisableRandomSamePriority = function(self)
        self.PBM.RandomSamePriority = false
    end,

    PBMCheckBusyFactories = function(self)
        local busyPlat = self:GetPlatoonUniquelyNamed('BusyFactories')
        if not busyPlat then
            busyPlat = self:MakePlatoon('', '')
            busyPlat:UniquelyNamePlatoon( 'BusyFactories' )
        end
        local poolPlat = self:GetPlatoonUniquelyNamed('ArmyPool')

        local poolTransfer = {}
        for k,v in poolPlat:GetPlatoonUnits() do
            if not v:IsDead() and EntityCategoryContains( categories.FACTORY - categories.MOBILE, v ) then
                if v:IsUnitState( 'Building' ) or v:IsUnitState( 'Upgrading' ) then
                    table.insert( poolTransfer, v )
                end
            end
        end

        local busyTransfer = {}
        for k,v in busyPlat:GetPlatoonUnits() do
            if not v:IsDead() and not v:IsUnitState( 'Building' ) and not v:IsUnitState( 'Upgrading' ) then
                table.insert( busyTransfer, v )
            end
        end

        self:AssignUnitsToPlatoon( poolPlat, busyTransfer, 'Unassigned', 'None' )
        self:AssignUnitsToPlatoon( busyPlat, poolTransfer, 'Unassigned', 'None' )
    end,

    #Main building and forming platoon thread for the Platoon Build Manager
    PlatoonBuildManagerThread = function(self)
        local personality = self:GetPersonality()
        while self:IsOpponentAIRunning() do
            self:PBMCheckBusyFactories()
            if self.BrainType == 'AI' then
                self:PBMSetPrimaryFactories()
            end
            local platoonList = self.PBM.Platoons
            #First go through the list of locations and see if we can build stuff there.
            for k, v in self.PBM.Locations do
                #Go through the different types of platoons
                for typek, typev in self.PBM.PlatoonTypes do
                    #See if we have platoons to build in that type
                    if table.getn(platoonList[typev]) > 0 then
                        #Sort the list of platoons via priority
                        if self.PBM.NeedSort[typev] then
                            self:PBMSortPlatoonsViaPriority(typev)
                        end
                        ########################################
                        # FORM PLATOONS
                        ########################################
                        self:PBMFormPlatoons(true, typev, v)
                        ########################################
                        # BUILD PLATOONS
                        ########################################
                        #See if our primary factory is busy.
                        if v.PrimaryFactories[typev] then
                            local priFac = v.PrimaryFactories[typev]
                            local numBuildOrders = nil
                            if priFac and not priFac:IsDead() then
                                numBuildOrders = priFac:GetNumBuildOrders(categories.ALLUNITS)
                                if numBuildOrders == 0 then
                                    local guards = priFac:GetGuards()
                                    if guards and table.getn(guards) > 0 then
                                        for kg, vg in guards do
                                            numBuildOrders = numBuildOrders + vg:GetNumBuildOrders(categories.ALLUNITS)
                                            if numBuildOrders == 0 and vg:IsUnitState('Building') then
                                                numBuildOrders = 1
                                            end
                                            if numBuildOrders > 0 then
                                                break
                                            end
                                        end
                                    end
                                end
                            end
                            if numBuildOrders and numBuildOrders == 0 then
                                local possibleTemplates = {}
                                local priorityLevel = false
                                #Now go through the platoon templates and see which ones we can build.
                                for kp, vp in platoonList[typev] do
                                    #Don't try to build things that are higher pri than 0
                                    #This platoon requires construction and isn't just a form-only platoon.
                                    local globalBuilder = ScenarioInfo.BuilderTable[self.CurrentPlan][typev][vp.BuilderName]
                                    if priorityLevel and ( vp.Priority ~= priorityLevel or not self.PBM.RandomSamePriority ) then
                                            break
                                    elseif (not priorityLevel or priorityLevel == vp.Priority)
                                            and vp.Priority > 0 and globalBuilder.RequiresConstruction and
                                            #The location we're looking at is an allowed location
                                            ( vp.LocationType == v.LocationType or not vp.LocationType ) and
                                            #The platoon has no handle so it hasn't been formed yet
                                            (not vp.PlatoonHandle and vp.PlatoonHandle != 'BUILDING') then
                                        #Fix up the primary factories to fit the proper table required by CanBuildPlatoon
                                        local suggestedFactories = {v.PrimaryFactories[typev]}
                                        local factories = self:CanBuildPlatoon( vp.PlatoonTemplate, suggestedFactories )

                                        if factories and self:PBMCheckBuildConditions(globalBuilder.BuildConditions) then
                                            priorityLevel = vp.Priority
                                            table.insert( possibleTemplates, { Builder = vp, Index = kp, Global = globalBuilder } )
                                        end
                                    end
                                end
                                if priorityLevel then
#                                    if table.getn(possibleTemplates) > 1 then
#                                        LOG('*DEBUG: RANDOMING OFF MORE THAN 1')
#                                    end
                                    local builderData = possibleTemplates[ Random(1,table.getn(possibleTemplates) ) ]
                                    local vp = builderData.Builder
                                    local kp = builderData.Index
                                    local globalBuilder = builderData.Global
                                    local suggestedFactories = {v.PrimaryFactories[typev]}
                                    local factories = self:CanBuildPlatoon( vp.PlatoonTemplate, suggestedFactories )
                                    vp.BuildTemplate = table.deepcopy(self:PBMBuildNumFactories(vp.PlatoonTemplate, v, typev, factories))
                                    local template = vp.BuildTemplate
                                    local factionIndex = self:GetFactionIndex()
                                    #Check all the requirements to build the platoon
                                    #The Primary Factory can actually build this platoon
                                    #The platoon build condition has been met
                                    local ptnSize = personality:GetPlatoonSize()
                                    #Finally, build the platoon.
                                    self:BuildPlatoon( template, factories, ptnSize)
                                    #LOG('*AI DEBUG: ARMY ', repr(self:GetArmyIndex()),': PBM Start building platoon named - ',repr(vp.BuilderName or vp.PlatoonTemplate),' LocationType: ',repr(vp.LocationType))
                                    #LOG('*PBM DEBUG: Size of first squad: ', repr(template[3][3]))
                                    self.PBM.Platoons[typev][kp].PlatoonHandle = 'BUILDING'
                                    if globalBuilder.GenerateTimeOut then
                                        vp.BuildTimeOut = self:PBMGenerateTimeOut(globalBuilder, factories, v, typev)
                                    else
                                        vp.BuildTimeOut = globalBuilder.BuildTimeOut
                                    end
                                    vp.PlatoonTimeOutThread = self:ForkThread(self.PBMPlatoonTimeOutThread, vp)
                                    if globalBuilder.PlatoonBuildCallbacks then
                                        for cbk, cbv in globalBuilder.PlatoonBuildCallbacks do
                                            import(cbv[1])[cbv[2]](self, globalBuilder.PlatoonData)
                                        end
                                    end
                                end
                            end
                        end
                    end
                end
            end
            #Do it all over again in 13 seconds.
            WaitSeconds(self.PBM.BuildCheckInterval or 13)
        end
    end,

    #Form platoons
    #Extracted as it's own function so you can call this to try and form platoons to clean up the pool
    #requireBuilding: true = platoon must have 'BUILDING' has its handle, false = it'll form any platoon it can
    #Platoontype is just 'Air'/'Land'/'Sea', those are found in the platoon build manager table template.
    #Location/Radius are where to do this.  If they aren't specified they will grab from anywhere.
    PBMFormPlatoons = function(self, requireBuilding, platoonType, location)
        local platoonList = self.PBM.Platoons
        local personality = self:GetPersonality()
        local numBuildOrders = nil
        if location.PrimaryFactories[platoonType] and not location.PrimaryFactories[platoonType]:IsDead() then
            numBuildOrders = location.PrimaryFactories[platoonType]:GetNumBuildOrders(categories.ALLUNITS)
            if numBuildOrders == 0 then
                local guards = location.PrimaryFactories[platoonType]:GetGuards()
                if guards and table.getn(guards) > 0 then
                    for kg, vg in guards do
                        numBuildOrders = numBuildOrders + vg:GetNumBuildOrders(categories.ALLUNITS)
                        if numBuildOrders == 0 and vg:IsUnitState('Building') then
                            numBuildOrders = 1
                        end
                        if numBuildOrders > 0 then
                            break
                        end
                    end
                end
            end
        end
        #Go through the platoon list to form a platoon
        for kp, vp in platoonList[platoonType] do
            local globalBuilder = ScenarioInfo.BuilderTable[self.CurrentPlan][platoonType][vp.BuilderName]
            #To build we need to accept the following:
            #The platoon is required to be in the building state and it is
            #or The platoon doesn't have a handle and either doesn't require to be building state or doesn't require construction
            #all that and passes it's build condition function.
            if vp.Priority > 0 and ( requireBuilding and vp.PlatoonHandle == 'BUILDING'
                and numBuildOrders and numBuildOrders == 0
                and (not vp.LocationType or vp.LocationType == location.LocationType) )
                or (((not vp.PlatoonHandle) and (not requireBuilding or not globalBuilder.RequiresConstruction))
                and (not vp.LocationType or vp.LocationType == location.LocationType)
                and self:PBMCheckBuildConditions(globalBuilder.BuildConditions) ) then

                    local poolPlatoon = self:GetPlatoonUniquelyNamed('ArmyPool')
                    local formIt = false
                    local template = vp.BuildTemplate
                    if not template then
                        template = vp.PlatoonTemplate
                    end
                    local flipTable = {}
                    local squadNum = 3
                    while squadNum <= table.getn(template) do
                        if template[squadNum][2] < 0 then
                            table.insert( flipTable, { Squad = squadNum, Value = template[squadNum][2] } )
                            template[squadNum][2] = 1
                        end
                        squadNum = squadNum + 1
                    end
                    if location.Location and location.Radius and vp.LocationType then
                        formIt = poolPlatoon:CanFormPlatoon(template, personality:GetPlatoonSize(), location.Location, location.Radius)
                    elseif not vp.LocationType then
                        formIt = poolPlatoon:CanFormPlatoon(template, personality:GetPlatoonSize())
                    end
                    local ptnSize = personality:GetPlatoonSize()
                    if formIt then
                        local hndl
                        if location.Location and location.Radius and vp.LocationType then
                            hndl = poolPlatoon:FormPlatoon(template, personality:GetPlatoonSize(), location.Location, location.Radius)
                            self.PBM.Platoons[platoonType][kp].PlatoonHandle = hndl
                            if vp.PlatoonTimeOutThread then
                                vp.PlatoonTimeOutThread:Destroy()
                            end
                        elseif not vp.LocationType then
                            hndl = poolPlatoon:FormPlatoon(template, personality:GetPlatoonSize())
                            self.PBM.Platoons[platoonType][kp].PlatoonHandle = hndl
                            if vp.PlatoonTimeOutThread then
                                vp.PlatoonTimeOutThread:Destroy()
                            end
                        end
                        #LOG('*AI DEBUG: ARMY ', repr(self:GetArmyIndex()),': PBM Forming - ',repr(vp.BuilderName or vp.PlatoonTemplate))
                        #LOG('*PBM DEBUG: Platoon formed with: ', repr(table.getn(hndl:GetPlatoonUnits())), ' Builder Named: ', repr(vp.BuilderName))
                        hndl.PlanName = template[2]
                        #If we have specific AI, fork that AI thread
                        local pltn = self.PBM.Platoons[platoonType][kp]
                        if globalBuilder.PlatoonAIFunction then
                            hndl:StopAI()
                            hndl:ForkAIThread(import(globalBuilder.PlatoonAIFunction[1])[globalBuilder.PlatoonAIFunction[2]])
                            #LOG('*PBM DEBUG: Platoon using:', repr(globalBuilder.PlatoonAIFunction[2]), ' Builder Named: ', repr(vp.BuilderName))
                        end
                        if globalBuilder.PlatoonAIPlan then
                            hndl:SetAIPlan(globalBuilder.PlatoonAIPlan)
                        end
                        #If we have additional threads to fork on the platoon, do that as well.
                        if globalBuilder.PlatoonAddPlans then
                            for papk, papv in globalBuilder.PlatoonAddPlans do
                                hndl:ForkThread( hndl[papv] )
                            end
                        end
                        if globalBuilder.PlatoonAddFunctions then
                            for pafk, pafv in globalBuilder.PlatoonAddFunctions do
                                hndl:ForkThread(import(pafv[1])[pafv[2]])
                                #LOG('*PBM DEBUG: Platoon Add Function: ', repr(pafv[2]), ' Builder Named: ', repr(vp.BuilderName))
                            end
                        end
                        if globalBuilder.PlatoonAddBehaviors then
                            for pafk, pafv in globalBuilder.PlatoonAddBehaviors do
                                hndl:ForkThread( Behaviors[pafv] )
                            end
                        end
                        if vp.BuilderName then
                            if self.PlatoonNameCounter[vp.BuilderName] then
                                self.PlatoonNameCounter[vp.BuilderName] = self.PlatoonNameCounter[vp.BuilderName] + 1
                            else
                                self.PlatoonNameCounter[vp.BuilderName] = 1
                            end
                        end
                        vp.PlatoonHandle:AddDestroyCallback(self.PBMPlatoonDestroyed)
                        hndl.BuilderName = vp.BuilderName
                        if globalBuilder.PlatoonData then
                            vp.PlatoonHandle:SetPlatoonData(globalBuilder.PlatoonData)
                            if globalBuilder.PlatoonData.AMPlatoons then
                                for k,v in globalBuilder.PlatoonData.AMPlatoons do
                                    hndl:SetPartOfAttackForce()
                                    break
                                end
                            end
                        end
                    end
                    for k,v in flipTable do
                        template[v.Squad][2] = v.Value
                    end
            end
        end
    end,

    #Get the primary factory with the lowest order count
    #This is used for the 'Any' platoon type so we can find any primary factory to build from.
    GetLowestOrderPrimaryFactory = function(self, location)
        local num
        local fac
        for k, v in self.PBM.PlatoonTypes do
            local priFac = location.PrimaryFactories[v]
            if priFac then
                local ord = priFac:GetNumBuildOrders(categories.ALLUNITS)
                if not num or num > ord then
                    num = ord
                    fac = location.PrimaryFactories[v]
                end
            end
        end
        return fac
    end,

    # Set number of units to be built as the number of factories in a location
    PBMBuildNumFactories = function (self, template, location, pType, factory)
        local assistFacs = factory[1]:GetGuards()
        table.insert( assistFacs, factory[1] )
        local facs = { T1 = 0, T2 = 0, T3 = 0 }
        for k,v in assistFacs do
            if EntityCategoryContains( categories.TECH3 * categories.FACTORY, v ) then
                facs.T3 = facs.T3 + 1
            elseif EntityCategoryContains( categories.TECH2 * categories.FACTORY, v ) then
                facs.T2 = facs.T2 + 1
            elseif EntityCategoryContains( categories.FACTORY, v ) then
                facs.T1 = facs.T1 + 1
            end
        end

        # handle any squads with a specified build quantity
        local squad = 3
        while squad <= table.getn(template) do
            if template[squad][2] > 0 then
                local bp = self:GetUnitBlueprint(template[squad][1])
                local buildLevel = AIBuildUnits.UnitBuildCheck(bp)
                local remaining = template[squad][3]
                while buildLevel <= 3 do
                    if facs['T'..buildLevel] > 0 then
                        if facs['T'..buildLevel] < remaining then
                            remaining = remaining - facs['T'..buildLevel]
                            facs['T'..buildLevel] = 0
                            buildLevel = buildLevel + 1
                        else
                            facs['T'..buildLevel] = facs['T'..buildLevel] - remaining
                            buildLevel = 10
                        end
                    else
                        buildLevel = buildLevel + 1
                    end
                end
            end
            squad = squad + 1
        end

        # Handle squads with programatic build quantity
        squad = 3
        local remainingIds = { T1 = {}, T2 = {}, T3 = {}, }
        while squad <= table.getn(template) do
            if template[squad][2] < 0 then
                table.insert( remainingIds['T'..AIBuildUnits.UnitBuildCheck(self:GetUnitBlueprint(template[squad][1]) ) ], template[squad][1] )
            end
            squad = squad + 1
        end
        local rTechLevel = 3
        while rTechLevel >=1 do
            for num,unitId in remainingIds['T'..rTechLevel] do
                for tempRow=3,table.getn(template) do
                    if template[tempRow][1] == unitId and template[tempRow][2] < 0 then
                        template[tempRow][3] = 0
                        for fTechLevel = rTechLevel,3 do
                            template[tempRow][3] = template[tempRow][3] + ( facs['T'..fTechLevel] * math.abs(template[tempRow][2]) )
                            facs['T'..fTechLevel] = 0
                        end
                    end
                end
            end
            rTechLevel = rTechLevel - 1
        end
        return template
    end,

    PBMGenerateTimeOut = function(self, platoon, factories, location, pType)
        local retBuildTime = 0
        local i = 3
        local numFactories = table.getn(factories[1]:GetGuards()) + 1
        if numFactories == 0 then
            numFactories = 1
        end
        local template = platoon.PlatoonTemplate
        while i <= table.getn(template) do
            local unitBuildTime, factoryBuildRate
            local bp = self:GetUnitBlueprint(template[i][1])
            if bp then
                unitBuildTime = self:GetUnitBlueprint(template[i][1]).Economy.BuildTime
            end
            if not unitBuildTime then
                unitBuildTime = 1000
            end
            if not factoryBuildRate then
                factoryBuildRate = 10
            end
            retBuildTime = retBuildTime + (math.ceil(template[i][3] / numFactories) * ((unitBuildTime/factoryBuildRate)*1.5))
            i = i + 1
        end
        local buildCheck = self.PBM.BuildCheckInterval or 13
        if retBuildTime > 0 then
            return (math.floor(retBuildTime / buildCheck) + 2) * buildCheck + 1
        else
            return 0
        end
    end,

    PBMGetNumFactoriesAtLocation = function(self, location, pType)
        local airFactories = {}
        local landFactories = {}
        local seaFactories = {}
        local gates = {}
        local factories = self:GetAvailableFactories(location.Location, location.Radius)
        local numFactories = 0
        for ek, ev in factories do
            if EntityCategoryContains(categories.FACTORY * categories.AIR, ev) then
                table.insert(airFactories, ev)
            elseif EntityCategoryContains(categories.FACTORY * categories.LAND, ev) then
                table.insert(landFactories, ev)
            elseif EntityCategoryContains(categories.FACTORY * categories.NAVAL, ev) then
                table.insert(seaFactories, ev)
            elseif EntityCategoryContains(categories.FACTORY * categories.GATE, ev) then
                table.insert(gates, ev)
            end
        end
        local retFacs = {}
        if pType == 'Air' then
            numFactories = table.getn(airFactories)
        elseif pType == 'Land' then
            numFactories = table.getn(landFactories)
        elseif pType == 'Sea' then
            numFactories = table.getn(seaFactories)
        elseif pType == 'Gate' then
            numFactories = table.getn(gates)
        end
        return numFactories
    end,

    PBMPlatoonTimeOutThread = function(self, platoon)
        local minWait = 240
        if platoon.BuildTimeOut and platoon.BuildTimeOut < minWait then
            #LOG('*AI DEBUG: ARMY ' .. self:GetArmyIndex() .. ' Builder - ' .. platoon.BuilderName .. ' --- PlatoonTimeout = ' .. minWait)
            WaitSeconds( minWait )
        else
            #LOG('*AI DEBUG: ARMY ' .. self:GetArmyIndex() .. ' Builder - ' .. platoon.BuilderName .. ' --- PlatoonTimeout = ' .. platoon.BuildTimeOut)
            WaitSeconds(platoon.BuildTimeOut or 600)
        end
        #LOG('*PBM DEBUG: ARMY ' .. self:GetArmyIndex() .. ' Platoon Builder timeout: Builder name- ', repr(platoon.BuilderName))
        if platoon.PlatoonHandle == 'BUILDING' then
            platoon.PlatoonHandle = nil
        end
    end,

    PBMFactoryCanBuildPlatoon = function(self, platoonTemplate, factory)
        for i = 3, table.getn(platoonTemplate) do
            if not factory:CanBuild(platoonTemplate[i][1]) then
                return false
            end
        end
        return true
    end,

    PBMPlatoonDestroyed = function(self, platoon)
        for tk, tv in self.PBM.Platoons do
            for k, v in tv do
                if v.PlatoonHandle == platoon then
                    v.PlatoonHandle = nil
                end
            end
        end
        if platoon.PlatoonData.BuilderName then
            self.PlatoonNameCounter[platoon.PlatoonData.BuilderName] = self.PlatoonNameCounter[platoon.PlatoonData.BuilderName] - 1
        end
    end,

    PBMCheckBuildConditions = function(self, bCs)
        for k, v in bCs do
            if v[3][1] == "default_brain" then
                table.remove(v[3], 1)
            end
            if not import(v[1])[v[2]](self, unpack(v[3])) then
                return false
            end
        end
        return true
    end,

    CombinePlatoons = function(self, platoonList, ai)
        local squadTypes = { 'Unassigned', 'Attack', 'Artillery', 'Support', 'Scout', 'Guard' }
        local returnPlatoon
        if not ai then
            returnPlatoon = self:MakePlatoon( ' ', 'None' )
        else
            returnPlatoon = self:MakePlatoon( ' ', ai)
        end
        for k,platoon in platoonList do
            for j,type in squadTypes do
                local squadUnits = platoon:GetSquadUnits(type)
                local formation = 'GrowthFormation'
                if squadUnits then
                    for k,v in squadUnits do
                        if not v:IsDead() then
                            if EntityCategoryContains( categories.AIR, v ) then
                                formation = 'GrowthFormation'
                            end
                            break
                        end
                    end
                    self:AssignUnitsToPlatoon(returnPlatoon, squadUnits, type, formation)
                end
            end
            self:DisbandPlatoon(platoon)
        end
        return returnPlatoon
    end,

    ###########################################
    ###      BASE MONITORING SYSTEM         ###
    ###########################################
    BaseMonitorInitialization = function(self, spec)
        self.BaseMonitor = {}
        self.BaseMonitor.BaseMonitorStatus = 'ACTIVE'
        self.BaseMonitor.BaseMonitorPoints = {}
        self.BaseMonitor.AlertSounded = false
        self.BaseMonitor.AlertsTable = {}
        self.BaseMonitor.AlertLocation = false
        self.BaseMonitor.AlertSoundedThreat = 0
        self.BaseMonitor.ActiveAlerts = 0

        self.BaseMonitor.PoolDistressRange = 75
        self.BaseMonitor.PoolReactionTime = 7

        # Variables for checking a radius for enemy units
        self.BaseMonitor.UnitRadiusThreshold = spec.UnitRadiusThreshold or 3
        self.BaseMonitor.UnitCategoryCheck = spec.UnitCategoryCheck or ( categories.MOBILE - ( categories.SCOUT + categories.ENGINEER ) )
        self.BaseMonitor.UnitCheckRadius = spec.UnitCheckRadius or 40

        # Threat level must be greater than this number to sound a base alert
        self.BaseMonitor.AlertLevel = spec.AlertLevel or 9
        # Delay time for checking base
        self.BaseMonitor.BaseMonitorTime = spec.BaseMonitorTime or 11
        # Default distance a platoon will travel to help around the base
        self.BaseMonitor.DefaultDistressRange = spec.DefaultDistressRange or 75
        # Default how often platoons will check if the base is under duress
        self.BaseMonitor.PlatoonDefaultReactionTime = spec.PlatoonDefaultReactionTime or 5
        # Default duration for an alert to time out
        self.BaseMonitor.DefaultAlertTimeout = spec.DefaultAlertTimeout or 10


        ## Monitor platoons for help
        self.BaseMonitor.PlatoonDistressTable = {}
        self.BaseMonitor.PlatoonDistressThread = false
        self.BaseMonitor.PlatoonAlertSounded = false

        self:ForkThread( self.BaseMonitorThread )
    end,

    BaseMonitorPlatoonDistress = function(self, platoon, threat)
        local found = false
        for k,v in self.BaseMonitor.PlatoonDistressTable do
            if v.Platoon == platoon then
                found = true
                break
            end
        end
        if not found then
            table.insert( self.BaseMonitor.PlatoonDistressTable, { Platoon = platoon, Threat = threat } )
            #LOG('*AI DEBUG: ARMY ' .. self:GetArmyIndex() .. ': --- PLATOON DISTRESS CALL ---')
        end
        if not self.BaseMonitor.PlatoonDistressThread then
            self.BaseMonitor.PlatoonDistressThread = self:ForkThread(self.BaseMonitorPlatoonDistressThread)
        end
    end,

    BaseMonitorPlatoonDistressThread = function(self)
        self.BaseMonitor.PlatoonAlertSounded = true
        while true do
            local numPlatoons = 0
            for k,v in self.BaseMonitor.PlatoonDistressTable do
                if self:PlatoonExists(v.Platoon) then
                    local threat = self:GetThreatAtPosition( v.Platoon:GetPlatoonPosition(), 1, false)
                    if threat > 0 then
                        v.Threat = threat
                    else
                        self.BaseMonitor.PlatoonDistressTable[k] = nil
                        v.Platoon.DistressCall = false
                    end
                else
                    self.BaseMonitor.PlatoonDistressTable[k] = nil
                end
                numPlatoons = k
            end
            if numPlatoons > 0 then
                self.BaseMonitor.PlatoonAlertSounded = true
            else
                self.BaseMonitor.PlatoonAlertSounded = false
            end
            WaitSeconds(self.BaseMonitor.BaseMonitorTime)
        end
    end,

    BaseMonitorDistressLocation = function(self, position)
        local returnPos = false
        local highThreat = false
        local distance
        if position then
            if self.BaseMonitor.CDRDistress then
                local cdrUnits = self:GetListOfUnits( categories.COMMAND, false )
                local cdr
                if cdrUnits then
                    for k,v in cdrUnits do
                        if not v:IsDead() then
                            cdr = v
                        end
                    end
                end
                if cdr then
                    distance = Utilities.XZDistanceTwoVectors( cdr:GetPosition(), position )
                    returnPos = cdr:GetPosition()
                end
            end
            if self.BaseMonitor.AlertSounded then
                for k,v in self.BaseMonitor.AlertsTable do
                    local tempDist = VDist2( position[1], position[3], v.Position[1], v.Position[3] )
                    if not returnPos or tempDist < distance then
                        local height = GetTerrainHeight( v.Position[1], v.Position[3] )
                        local surfHeight = GetSurfaceHeight( v.Position[1], v.Position[3] )
                        if surfHeight > height then
                            height = surfHeight
                        end
                        returnPos = { v.Position[1], height, v.Position[3] }
                        distance = tempDist
                    end
                end
            end
            if self.BaseMonitor.PlatoonAlertSounded then
                for k,v in self.BaseMonitor.PlatoonDistressTable do
                    if self:PlatoonExists( v.Platoon ) then
                        local platPos = v.Platoon:GetPlatoonPosition()
                        local tempDist = VDist2( position[1], position[3], platPos[1], platPos[3] )
                        if not returnPos or tempDist < distance then
                            returnPos = platPos
                            distance = tempDist
                        end
                    end
                end
            end
            if returnPos then
                return returnPos
            end
        else
            if self.BaseMonitor.CDRDistress then
                local cdrUnits = self:GetListOfUnits( categories.COMMAND, false )
                local cdr
                if cdrUnits then
                    if not cdrUnits[1]:IsDead() then
                        cdr = cdrUnits[1]
                    end
                end
                if cdr then
                    return cdr:GetPosition()
                end
            end
            if self.BaseMonitor.AlertSounded then
                for k,v in self.BaseMonitor.AlertsTable do
                    if not highThreat or v.Threat > highThreat then
                        local height = GetTerrainHeight( v.Position[1], v.Position[3] )
                        local surfHeight = GetSurfaceHeight( v.Position[1], v.Position[3] )
                        if surfHeight > height then
                            height = surfHeight
                        end
                        returnPos = { v.Position[1], height, v.Position[3] }
                        highThreat = v.Threat
                    end
                end
            end
            if returnPos then
                return returnPos
            end
            if self.BaseMonitor.PlatoonAlertSounded then
                for k,v in self.BaseMonitor.PlatoonDistressTable do
                    if self:PlatoonExists(v.Platoon) and ( not highThreat or v.Threat > highThreat ) then
                        returnPos = v.Platoon:GetPlatoonPosition()
                        highThreat = v.Threat
                    end
                end
            end
        end
        return returnPos
    end,

    BaseMonitorThread = function(self)
        while true do
            if self.BaseMonitor.BaseMonitorStatus == 'ACTIVE' then
                self:BaseMonitorCheck()
            end
            WaitSeconds( self.BaseMonitor.BaseMonitorTime )
        end
    end,

    BaseMonitorAlertTimeout = function(self, pos)
        local timeout = self.BaseMonitor.DefaultAlertTimeout
        local allies
        local enemies
        repeat
            WaitSeconds(timeout)
            allies = self:GetNumUnitsAroundPoint( 'Ally', categories.STRUCTURE, pos, 32 )
            enemies = self:GetNumUnitsAroundPoint( 'Enemy', self.BaseMonitor.UnitCategoryCheck, pos, self.BaseMonitor.UnitCheckRadius)
        until enemies == 0 or allies < 1
        for k,v in self.BaseMonitor.AlertsTable do
            if pos[1] == v.Position[1] and pos[3] == v.Position[3] then
                self.BaseMonitor.AlertsTable[k] = nil
                break
            end
        end
        for k,v in self.BaseMonitor.BaseMonitorPoints do
            if pos[1] == v.Position[1] and pos[3] == v.Position[3] then
                v.Alert = false
                break
            end
        end
        self.BaseMonitor.ActiveAlerts = self.BaseMonitor.ActiveAlerts - 1
        if self.BaseMonitor.ActiveAlerts == 0 then
            self.BaseMonitor.AlertSounded = false
            #LOG('*AI DEBUG: ARMY ' .. self:GetArmyIndex() .. ': --- ALERTS DEACTIVATED ---')
        end
    end,

    BaseMonitorCheck = function(self)
        local vecs = self:GetStructureVectors()
        if table.getn(vecs) > 0 then
            # Find new points to monitor
            for k,v in vecs do
                local found = false
                for subk, subv in self.BaseMonitor.BaseMonitorPoints do
                    if v[1] == subv.Position[1] and v[3] == subv.Position[3] then
                        found = true
                        break
                    end
                end
                if not found then
                    table.insert( self.BaseMonitor.BaseMonitorPoints,
                                 { 
                                    Position = v, 
                                    Threat = self:GetNumUnitsAroundPoint( 'Enemy', self.BaseMonitor.UnitCategoryCheck, v, self.BaseMonitor.UnitCheckRadius), 
                                    Alert = false
                                 }
                             )
                end
            end
            # Remove any points that we dont monitor anymore
            for k,v in self.BaseMonitor.BaseMonitorPoints do
                local found = false
                for subk, subv in vecs do
                    if v.Position[1] == subv[1] and v.Position[3] == subv[3] then
                        found = true
                        break
                    end
                end
                # If point not in list and the num units around the point is small
                if not found and not self:GetNumUnitsAroundPoint( 'Ally', categories.STRUCTURE, v.Position, 16 ) > 1 then
                    self.BaseMonitor.BaseMonitorPoints[k] = nil
                end
            end
            # Check monitor points for change
            local alertThreat = self.BaseMonitor.AlertLevel
            local alertPosition
            for k,v in self.BaseMonitor.BaseMonitorPoints do
                if not v.Alert then
                    local allies = self:GetNumUnitsAroundPoint( 'Ally', categories.STRUCTURE, v.Position, 32 )
                    local enemies = self:GetNumUnitsAroundPoint( 'Enemy', self.BaseMonitor.UnitCategoryCheck, v.Position, self.BaseMonitor.UnitCheckRadius)
                    if ( allies > 0 ) and enemies >= self.BaseMonitor.UnitRadiusThreshold then
                        #LOG('*AI DEBUG: ARMY ' .. self:GetArmyIndex() .. ': #### AI ALERT ####')
                        # Add to alert table
                        v.Alert = true
                        table.insert( self.BaseMonitor.AlertsTable, 
                            { 
                                Position = v.Position, 
                                Threat = self:GetNumUnitsAroundPoint( 'Enemy', self.BaseMonitor.UnitCategoryCheck, v.Position, self.BaseMonitor.UnitCheckRadius)
                            }
                        )
                        self.BaseMonitor.AlertSounded = true
                        self:ForkThread(self.BaseMonitorAlertTimeout, v.Position)
                        self.BaseMonitor.ActiveAlerts = self.BaseMonitor.ActiveAlerts + 1
                    end
                end
            end
        end
    end,

    GetBaseVectors = function(self)
        local enemy = self:GetCurrentEnemy()
        local index = self:GetArmyIndex()
        self:SetCurrentEnemy( self )
        self:SetUpAttackVectorsToArmy(categories.STRUCTURE - ( categories.MASSEXTRACTION ))
        local vecs = self:GetAttackVectors()
        self:SetCurrentEnemy(enemy)
        if enemy then
            self:SetUpAttackVectorsToArmy()
        end
        local returnPoints = {}
        if vecs then
            for k,v in vecs do
                local loc = { v.px, v.py, v.pz }
                local found = false
                for subk, subv in returnPoints do
                    if subv[1] == loc[1] and subv[3] == loc[3] then
                        found = true
                        break
                    end
                end
                if not found then
                    table.insert( returnPoints, loc )
                end
            end
        end
        return returnPoints
    end,

    GetStructureVectors = function(self)
        local structures = self:GetListOfUnits( categories.STRUCTURE - categories.WALL, false )
        # Add all points around location
        local tempGridPoints = {}
        local indexChecker = {}

        for k,v in structures do
            if not v:IsDead() then
                local pos = AIUtils.GetUnitBaseStructureVector( v )
                if pos then
                    if not indexChecker[pos[1]] then
                        indexChecker[pos[1]] = {}
                    end
                    if not indexChecker[pos[1]][pos[3]] then
                        indexChecker[pos[1]][pos[3]] = true
                        table.insert( tempGridPoints, pos )
                    end
                end
            end
        end

        return tempGridPoints
    end,



    #########################################################################
    ###                     ENEMY PICKER AI                               ###
    #########################################################################

    EvaluateArmies = function(self, evalCats)
        local brainStr = self:GetBrainStrengthTable( evalCats )
        local evalOrder = {}
        local brainVals = {}
        for num,brain in ArmyBrains do
            table.insert( brainVals, {} )
        end
        for catName, catData in brainStr do
            evalOrder[catName] = {}
            for i = 1, table.getn(catData) do
                local high = -1
                local key
                for num,val in catData do
                    if val > high then
                        high = val
                        key = num
                    end
                end
                table.insert( evalOrder[catName], { BrainIndex = key, Strength = high } )
                brainVals[key][catName] = high
                catData[key] = -1
            end
        end
        return evalOrder, brainVals
    end,

    GetBrainStrengthTable = function(self, categoryList)
        # Setup table of brains
        local brainStrengthTable = {}
        for k,catName in categoryList do
            brainStrengthTable[catName] = {}
            for num,data in ArmyBrains do
                brainStrengthTable[catName][num] = 0
            end
        end

        local catList = {}
        for k,v in categoryList do
             catList[v] =  ParseEntityCategory( v )
        end

        local alliances = { 'ENEMY', 'ALLY', 'NEUTRAL' }
        for aType in alliances do
            local units = self:GetUnitsAroundPoint( aType, categories.ALLUNITS, {0,0,0}, 100000 )
            for k,currUnit in units do
                local index = currUnit:GetAIBrain():GetArmyIndex()
                for catName,cat in catList do
                    if EntityCategoryContains( cat, currUnit ) then
                        brainStrengthTable[catName][index] = brainStrengthTable[catName][index] + 1
                    end
                end
            end
        end

        for cName,cData in brainStrengthTable do
            for bNum, bVal in brainStrengthTable[cName] do
                if ScenarioInfo.ArmySetup[ArmyBrains[bNum].Name].Civilian then
                    brainStrengthTable[cName][bNum] = 0
                end
            end
        end

        return brainStrengthTable
    end,

    PickEnemy = function(self)
        while true do
            self:PickEnemyLogic()
            WaitSeconds(300)
        end
    end,

    ForceNewEnemy = function(self)
        local evalCats = { 'MOBILE', 'CONSTRUCTION', 'DEFENSE', 'ECONOMIC' }
        local evalOrder, brainVals = self:EvaluateArmies(evalCats)

        local currEnemy = self:GetCurrentEnemy()
        local currIndex
        if currEnemy then
            currIndex = currEnemy:GetArmyIndex()
        else
            currIndex = 0
        end
        local closest = self:FindClosestArmyWithBase('Enemy')
        local weakestArmy = self:FindWeakestEnemyFromCat( evalOrder, 'MOBILE', brainVals, currIndex )
        local weakestBase = self:FindWeakestEnemyFromCat( evalOrder, 'DEFENSE', brainVals, currIndex )
        if closest and weakestArmy and weakestBase then
            if closest:GetArmyIndex() == weakestBase and closest:GetArmyIndex() == weakestArmy then
                #LOG('*AI DEBUG: ARMY ' .. self:GetArmyIndex() .. ' - Pick enemy number: ' .. closest:GetArmyIndex() )
                self:SetCurrentEnemy( closest )
            elseif weakestArmy == closest:GetArmyIndex() then
                #LOG('*AI DEBUG: ARMY ' .. self:GetArmyIndex() .. ' - Pick enemy number: ' .. weakestArmy )
                self:SetCurrentEnemy( ArmyBrains[weakestArmy] )
            else
                #LOG('*AI DEBUG: ARMY ' .. self:GetArmyIndex() .. ' - Pick enemy number: ' .. weakestBase)
                self:SetCurrentEnemy( ArmyBrains[weakestBase] )
            end
        elseif weakestBase then
            #LOG('*AI DEBUG: ARMY ' .. self:GetArmyIndex() .. ' - Pick enemy number: ' .. weakestBase)
            self:SetCurrentEnemy( ArmyBrains[weakestBase] )
        end
    end,

    FindWeakestEnemyFromCat = function(self, evalOrder, category, brainVals, excludeNum)
        local excludeTable = {}
        for k,v in brainVals do
            if excludeNum == k or v['CONSTRUCTION'] == 0 or v['ECONOMIC'] == 0 then
                excludeTable[k] = true
            else
                excludeTable[k] = false
            end
        end
        for catName,catData in evalOrder do
            if catName == category then
                local weakestValue
                local weakestKey
                for key, bData in catData do
                    if IsEnemy( bData.BrainIndex, self:GetArmyIndex() ) then
                        if ( not weakestValue or bData.Strength < weakestValue ) and bData.Strength > 0 and not excludeTable[bData.BrainIndex] then
                            weakestValue = bData.Strength
                            weakestKey = bData.BrainIndex
                        end
                    end
                end
                if weakestKey then
                    return weakestKey
                else
                    return false
                end
            end
        end
    end,

    PickEnemyLogic = function(self)
        # Evaluating categories
        local evalCats = { 'MOBILE', 'CONSTRUCTION', 'DEFENSE', 'ECONOMIC' }
        local evalOrder, brainVals = self:EvaluateArmies(evalCats)

        local strongestAlly = false
        local allyEnemy
        local selfStrength = 0

        #BrainIndex = key, Strength = high
        for catName, catData in evalOrder do
            if catName == 'MOBILE' then
                for key, bData in catData do
                    if self:GetArmyIndex() == bData.BrainIndex then
                        selfStrength = bData.Strength
                    elseif IsAlly( bData.BrainIndex, self:GetArmyIndex() ) then
                        self:SetResourceSharing(true)
                        if ( not strongestAlly or bData.Strength > strongestAlly ) and ArmyBrains[bData.BrainIndex]:GetCurrentEnemy() then
                            allyEnemy = ArmyBrains[bData.BrainIndex]:GetCurrentEnemy()
                            strongestAlly = bData.Strength
                        end
                    end
                end
            end
        end

        if allyEnemy and not allyEnemy:IsDefeated()  and strongestAlly > selfStrength then
            if not self:GetCurrentEnemy() or allyEnemy:GetArmyIndex() ~= self:GetCurrentEnemy():GetArmyIndex() then
                #LOG('*AI DEBUG: ARMY ' .. self:GetArmyIndex() .. ' - Picking allied enemy number: ' .. allyEnemy:GetArmyIndex() )
            end
            self:SetCurrentEnemy( allyEnemy )
        else
            local findEnemy = false
            if not self:GetCurrentEnemy() then
                findEnemy = true
            else
                local cIndex = self:GetCurrentEnemy():GetArmyIndex()
                if self:GetCurrentEnemy():IsDefeated() or brainVals[cIndex]['CONSTRUCTION'] == 0 or brainVals[cIndex]['ECONOMIC'] == 0 then
                    findEnemy = true
                end
            end
            if findEnemy then
                local closest = self:FindClosestArmyWithBase('Enemy')
                local weakestArmy = self:FindWeakestEnemyFromCat( evalOrder, 'MOBILE', brainVals)
                local weakestBase = self:FindWeakestEnemyFromCat( evalOrder, 'DEFENSE', brainVals )
                if closest and weakestArmy and weakestBase then
                    if closest:GetArmyIndex() == weakestBase and closest:GetArmyIndex() == weakestArmy then
                        #LOG('*AI DEBUG: ARMY ' .. self:GetArmyIndex() .. ' - Pick enemy number: ' .. closest:GetArmyIndex() )
                        self:SetCurrentEnemy( closest )
                    elseif weakestArmy == closest:GetArmyIndex() then
                        #LOG('*AI DEBUG: ARMY ' .. self:GetArmyIndex() .. ' - Pick enemy number: ' .. weakestArmy )
                        self:SetCurrentEnemy( ArmyBrains[weakestArmy] )
                    else
                        #LOG('*AI DEBUG: ARMY ' .. self:GetArmyIndex() .. ' - Pick enemy number: ' .. weakestBase )
                        self:SetCurrentEnemy( ArmyBrains[weakestBase] )
                    end
                elseif weakestBase then
                    #LOG('*AI DEBUG: ARMY ' .. self:GetArmyIndex() .. ' - Pick enemy number: ' .. weakestBase )
                    self:SetCurrentEnemy( ArmyBrains[weakestBase] )
                end
            end
        end
    end,


    #################################
    ## Skirmish expansion help     ##
    #################################

    ExpansionHelp = function( self, eng, reference )
        self:ForkThread( self.ExpansionHelpThread, eng, reference )
    end,

    ExpansionHelpThread = function( self, eng, reference )
        #LOG( '*AI DEBUG: ARMY ' ..self:GetArmyIndex() .. ' Sending units to help expand')
        local pool = self:GetPlatoonUniquelyNamed( 'ArmyPool' )
        local landHelp = {}
        local val = 0
        for k,v in pool:GetPlatoonUnits() do
            if val == 0 and EntityCategoryContains( ( categories.LAND * categories.MOBILE ) - categories.CONSTRUCTION, v ) then
                table.insert( landHelp, v )
            end
            val = val + 1
            if val == 4 then
                val = 0
            end
        end
        self:ForkThread( self.GroupHelpThread, landHelp, reference )
    end,

    GroupHelpThread = function( self, units, reference )
        local plat = self:MakePlatoon('', '')
        self:AssignUnitsToPlatoon( plat, units, 'Attack', 'GrowthFormation' )
        local cmd = plat:MoveToLocation( reference, false )
        while self:PlatoonExists(plat) and plat:IsCommandsActive(cmd) do
            WaitSeconds(5)
        end
        WaitSeconds(30)
        if self:PlatoonExists(plat) then
            local x,z = self:GetArmyStartPos()
            plat:MoveToLocation( { x, 0, z }, false )
            self:DisbandPlatoon(plat)
            #LOG('*AI DEBUG: ARMY ' .. self:GetArmyIndex() .. ' Returning expansion helpers to pool' )
        end
    end,

    AbandonedByPlayer = function(self)
        #LOG('Army abandoned by player, AI taking over.')
        self:OnCreateAI()
    end,
    
}
