#****************************************************************************
#**
#**  File     :  /cdimage/lua/modules/cinematics.lua
#**  Author(s):  David Tomandl
#**
#**  Summary  :  Helper functions for cinematics
#**
#**  Copyright  2006 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************

SimCamera = import('/lua/modules/simcamera.lua').SimCamera

# To be used when starting a cinematic / NIS
function EnterNISMode()
    ScenarioInfo.Camera = SimCamera('WorldCamera')
    ScenarioInfo.Camera:Lock()
    ScenarioInfo.Camera:UseGameClock()
    #Take Away UI
    #Set Game Speed to normal
end

# Used at the end of a cinematic / NIS
function ExitNISMode()
    #Set Game Speed to user value
    #Restore UI
    CameraRevertRotation()
    ScenarioInfo.Camera:UseSystemClock()
    ScenarioInfo.Camera:Release()
    ScenarioInfo.Camera = nil
end

# This will move the camera to the position of a marker
function CameraMoveToMarker( marker, seconds )

    # Adding this in case we just want to start the camera
    # somewhere at the beginning of an operation without
    # playing a full NIS
    if not ScenarioInfo.Camera then
        ScenarioInfo.Camera = SimCamera('WorldCamera')
    end

    # Move the camera
    ScenarioInfo.Camera:MoveToMarker( marker, seconds )

    if ( seconds ) and ( seconds != 0 ) then
        # Wait for it to be done
        WaitForCamera()
    end
end

# This will move the camera to a rectangle
function CameraMoveToRectangle( rectangle, seconds )

    # Adding this in case we just want to start the camera
    # somewhere at the beginning of an operation without
    # playing a full NIS
    if not ScenarioInfo.Camera then
        ScenarioInfo.Camera = SimCamera('WorldCamera')
    end

    # Move the camera
    ScenarioInfo.Camera:MoveTo( rectangle, seconds )

    if ( seconds ) and ( seconds != 0 ) then
        # Wait for it to be done
        WaitForCamera()
    end
end

# This will make the camera track an entity.
# The "seconds" field is how long it will take to get in place.
# After that the camera will follow that unit until told to do something else.
function CameraTrackEntity( entity, zoom, seconds )
    # Watch the entity
    ScenarioInfo.Camera:TrackEntity( entity, zoom, seconds )

    if ( seconds ) and ( seconds != 0 ) then
        # Wait for it to be done
        WaitForCamera()
    end
end

# This will make the camera track a group of entities.
# The "seconds" field is how long it will take to get in place.
# After that the camera will follow that unit until told to do something else.
function CameraTrackEntities( units, zoom, seconds )
    # Watch the entities
    ScenarioInfo.Camera:TrackEntities( units, zoom, seconds )

    if ( seconds ) and ( seconds != 0 ) then
        # Wait for it to be done
        WaitForCamera()
    end
end

# Similar to CameraTrackEntity, but this gives more control with the
# pitchAdjust parameter.
function CameraThirdPerson( entity, pitchAdjust, zoom, seconds )
    ScenarioInfo.Camera:NoseCam( entity, pitchAdjust, zoom, seconds )

    if ( seconds ) and ( seconds != 0 ) then
        # Wait for it to be done
        WaitForCamera()
    end
end

# This will modify the current zoom of the camera without
# adjusting its position.
function CameraSetZoom( zoom, seconds )
    # Move the camera
    ScenarioInfo.Camera:SetZoom( zoom, seconds )

    if ( seconds ) and ( seconds != 0 ) then
        # Wait for it to be done
        WaitForCamera()
    end
end

# This will spin/zoom the camera for the specified
# number of seconds, then stop the camera again.
# Using it with 0 seconds will keep the camera
# spinning/zooming until the next command.
function CameraSpinAndZoom( spin, zoom, seconds )
    # Move the camera
    ScenarioInfo.Camera:Spin( spin, zoom )

    if ( seconds ) and ( seconds != 0 ) then
        # Wait for it to be done
        WaitForCamera()
        # Stop the camera
        ScenarioInfo.Camera:Spin( 0, 0 )
    end
end

# This will bring the camera to the highest zoomed-out level.
function CameraReset()
    ScenarioInfo.Camera:Reset()
end

# This will reset the azimuth back to default and change
# the rotation back to default as well
function CameraRevertRotation()
    ScenarioInfo.Camera:RevertRotation()
end

# Used by other functions to make sure that they don't return
# before the camera is done moving.
function WaitForCamera()
    # Wait for it to be done
    ScenarioInfo.Camera:WaitFor()

    # Reset the event tracker, so that the next
    # camera action can be waited for
    ScenarioInfo.Camera:EventReset()
end
