#****************************************************************************
#**
#**  File     :  /cdimage/lua/modules/editor/EconomyBuildConditions.lua
#**  Author(s): Dru Staltman, John Comes
#**
#**  Summary  : Generic AI Platoon Build Conditions
#**             Build conditions always return true or false
#**
#**  Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************
local AIUtils = import('/lua/modules/ai/aiutilities.lua')
local ScenarioFramework = import('/lua/modules/scenarioframework.lua')
local ScenarioUtils = import('/lua/ScenarioUtilities.lua')

##############################################################################################################
# function: GreaterThanEconStorageRatio = BuildCondition	doc = "Please work function docs."
# 
# parameter 0: string	aiBrain		= "default_brain"		
# parameter 1: float	mStorageRatio	= 0.0				doc = "docs for param1"
# parameter 2: float	eStorageRatio	= 0.0				doc = "param2 docs"
#
##############################################################################################################
function GreaterThanEconStorageRatio(aiBrain, mStorageRatio, eStorageRatio)
    local econ = AIUtils.AIGetEconomyNumbers(aiBrain)
    if (econ.MassStorageRatio > mStorageRatio and econ.EnergyStorageRatio > eStorageRatio) then
        return true
    end
    return false
end

##############################################################################################################
# function: GreaterThanEconStorageMax = BuildCondition	doc = "Please work function docs."
# 
# parameter 0: string	aiBrain         = "default_brain"		
# parameter 1: int	mStorage        = 0				doc = "docs for param1"
# parameter 2: int	eStorage	= 0				doc = "param2 docs"
#
##############################################################################################################
function GreaterThanEconStorageMax(aiBrain, mStorage, eStorage)
    local econ = AIUtils.AIGetEconomyNumbers(aiBrain)
    if (econ.MassMaxStored > mStorage and econ.EnergyMaxStored > eStorage) then
        return true
    end
    return false
end

##############################################################################################################
# function: GreaterThanEconStorageCurrent = BuildCondition	doc = "Please work function docs."
# 
# parameter 0: string	aiBrain		= "default_brain"		doc = "docs for param1"
# parameter 1: integer	mStorage	= 0					doc = "docs for param1"
# parameter 2: integer	eStorage	= 0					doc = "param2 docs"
#
##############################################################################################################
function GreaterThanEconStorageCurrent(aiBrain, mStorage, eStorage)
    local econ = AIUtils.AIGetEconomyNumbers(aiBrain)
    if (econ.MassStorage > mStorage and econ.EnergyStorage > eStorage) then
        return true
    end
    return false
end

##############################################################################################################
# function: LessThanEconTrend = BuildCondition	doc = "Please work function docs."
# 
# parameter 0: string	aiBrain		= "default_brain"			doc = "docs for param1"	
# parameter 1: integer	mTrend	        = 0				doc = "docs for param1"
# parameter 2: integer	eTrend	        = 0      			doc = "param2 docs"
#
##############################################################################################################
function LessThanEconTrend(aiBrain, mTrend, eTrend)
    local econ = AIUtils.AIGetEconomyNumbers(aiBrain)
    if (econ.MassTrend < mTrend and econ.EnergyTrend < eTrend) then
        return true
    else
        return false
    end
end

##############################################################################################################
# function: LessThanEconStorageRatio = BuildCondition	doc = "Please work function docs."
# 
# parameter 0: string	aiBrain		     = "default_brain"				doc = "docs for param1"
# parameter 1: integer	mStorageRatio        = 0					doc = "docs for param1"
# parameter 2: integer	eStorageRatio	     = 0					doc = "param2 docs"
#
##############################################################################################################
function LessThanEconStorageRatio(aiBrain, mStorageRatio, eStorageRatio)
    local econ = AIUtils.AIGetEconomyNumbers(aiBrain)
    if (econ.MassStorageRatio < mStorageRatio and econ.EnergyStorageRatio < eStorageRatio) then
        return true
    end
    return false
end

##############################################################################################################
# function: LessEconStorageMax = BuildCondition	doc = "Please work function docs."
# 
# parameter 0: string       aiBrain	    = "default_brain"				doc = "docs for param1"
# parameter 1: integer      mStorage    = 0					doc = "docs for param1"
# parameter 2: integer      eStorage    = 0					doc = "param2 docs"
#
##############################################################################################################
function LessEconStorageMax(aiBrain, mStorage, eStorage)
    local econ = AIUtils.AIGetEconomyNumbers(aiBrain)
    if (econ.MassMaxStored < mStorage and econ.EnergyMaxStored < eStorage) then
        return true
    end
    return false
end

##############################################################################################################
# function: LessEconStorageCurrent = BuildCondition	doc = "Please work function docs."
# 
# parameter 0: string       aiBrain         = "default_brain"				doc = "docs for param1"
# parameter 1: integer      mStorage	= 0					doc = "docs for param1"
# parameter 2: integer      eStorage	= 0					doc = "param2 docs"
#
##############################################################################################################
function LessEconStorageCurrent(aiBrain, mStorage, eStorage)
    local econ = AIUtils.AIGetEconomyNumbers(aiBrain)
    if (econ.MassStorage < mStorage and econ.EnergyStorage < eStorage) then
        return true
    end
    return false
end


##############################################################################################################
# function: GreaterThanEconTrend = BuildCondition	doc = "Please work function docs."
# 
# parameter 0: string	aiBrain		= "default_brain"				doc = "docs for param1"
# parameter 1: int	MassTrend	= 1             doc = "docs for param1"
# parameter 2: int	EnergyTrend	= 1             doc = "param2 docs"
#
##############################################################################################################
function GreaterThanEconTrend(aiBrain, MassTrend, EnergyTrend)
    local econ = AIUtils.AIGetEconomyNumbers(aiBrain)
    if (econ.MassTrend > MassTrend and econ.EnergyTrend > EnergyTrend) then
        return true
    end
    return false
end

##############################################################################################################
# function: GreaterThanEconIncome = BuildCondition	doc = "Please work function docs."
# 
# parameter 0: string	aiBrain		= "default_brain"				doc = "docs for param1"
# parameter 1: int	MassIncome	= 0.1             doc = "docs for param1"
# parameter 2: int	EnergyIncome	= 1             doc = "param2 docs"
#
##############################################################################################################
function GreaterThanEconIncome(aiBrain, MassIncome, EnergyIncome)
    local econ = AIUtils.AIGetEconomyNumbers(aiBrain)
    if (econ.MassIncome > MassIncome and econ.EnergyIncome > EnergyIncome) then
        return true
    end
    return false
end


##############################################################################################################
# function: LessThanEconIncome = BuildCondition	doc = "Please work function docs."
# 
# parameter 0: string	aiBrain		= "default_brain"				doc = "docs for param1"
# parameter 1: int	MassIncome	= 0.1             doc = "docs for param1"
# parameter 2: int	EnergyIncome	= 1             doc = "param2 docs"
#
##############################################################################################################
function LessThanEconIncome(aiBrain, MassIncome, EnergyIncome)
    local econ = AIUtils.AIGetEconomyNumbers(aiBrain)
    if (econ.MassIncome < MassIncome and econ.EnergyIncome < EnergyIncome) then
        return true
    end
    return false
end
