#****************************************************************************
#**
#**  File     :  /cdimage/lua/modules/MiscBuildConditions.lua
#**  Author(s): Dru Staltman, John Comes
#**
#**  Summary  : Generic AI Platoon Build Conditions
#**             Build conditions always return true or false
#**
#**  Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************
local AIUtils = import('/lua/modules/ai/aiutilities.lua')
local ScenarioFramework = import('/lua/modules/scenarioframework.lua')
local ScenarioUtils = import('/lua/ScenarioUtilities.lua')
local Utils = import('/lua/modules/utilities.lua')

##############################################################################################################
# function: True = BuildCondition   doc = "Please work function docs."
# 
# parameter 0: string   aiBrain     = "default_brain"       
# 
##############################################################################################################
function True(aiBrain)
    return true
end

##############################################################################################################
# function: False = BuildCondition  doc = "Please work function docs."
# 
# parameter 0: string   aiBrain     = "default_brain"       
# 
##############################################################################################################
function False(aiBrain)
    return false
end

##############################################################################################################
# function: RandomNumber = BuildCondition   doc = "Please work function docs."
# 
# parameter 0: string   aiBrain         = "default_brain"       
# parameter 1: int      higherThan      = 0                 doc = "docs for param1"
# parameter 2: int      lowerThan       = 0                 doc = "param2 docs"
# parameter 3: int      minNumber       = 0                 doc = "param2 docs"
# parameter 4: int      maxNumber       = 0                 doc = "param2 docs"
#
##############################################################################################################
function RandomNumber(aiBrain, higherThan, lowerThan, minNumber, maxNumber)
    local num = Random( minNumber, maxNumber)
    if higherThan < num and lowerThan > num then
        return true
    end
    return false
end

##############################################################################################################
# function: IsAIBrainLayerPref = BuildCondition doc = "Please work function docs."
# 
# parameter 0: string   aiBrain         = "default_brain"       
# parameter 1: string   layerPref       = "Land"            doc = "docs for param1"
#
##############################################################################################################
function IsAIBrainLayerPref(aiBrain, layerPref)
    if layerPref == aiBrain:AIGetLayerPreference() then
        return true
    end
    return false
end

##############################################################################################################
# function: MissionNumber = BuildCondition  doc = "Please work function docs."
# 
# parameter 0: string   aiBrain         = "default_brain"       
# parameter 1: int  num             = 1         doc = "docs for param1"
#
##############################################################################################################
function MissionNumber(aiBrain, num)
    if ScenarioInfo.MissionNumber and num == ScenarioInfo.MissionNumber then
        return true
    else
        return false
    end
end

##############################################################################################################
# function: MissionNumberGreaterOrEqual = BuildCondition  doc = "Please work function docs."
# 
# parameter 0: string   aiBrain         = "default_brain"       
# parameter 1: int  num             = 1         doc = "docs for param1"
#
##############################################################################################################
function MissionNumberGreaterOrEqual(aiBrain, num)
    if ScenarioInfo.MissionNumber and num <= ScenarioInfo.MissionNumber then
        return true
    else
        return false
    end
end

##############################################################################################################
# function: MissionNumberLessOrEqual = BuildCondition  doc = "Please work function docs."
# 
# parameter 0: string   aiBrain         = "default_brain"       
# parameter 1: int  num             = 1         doc = "docs for param1"
#
##############################################################################################################
function MissionNumberLessOrEqual(aiBrain, num)
    if ScenarioInfo.MissionNumber and num >= ScenarioInfo.MissionNumber then
        return true
    else
        return false
    end
end

##############################################################################################################
# function: CheckScenarioInfoVarTable = BuildCondition  doc = "Please work function docs."
# 
# parameter 0: string   aiBrain         = "default_brain"       
# parameter 1: string   varName         = "VarName"         doc = "docs for param1"
#
##############################################################################################################
function CheckScenarioInfoVarTable(aiBrain, varName)
    if ScenarioInfo.VarTable then
        local i = 1
        if not ScenarioInfo.VarTable[varName] then
            return false
        end
        return true
    end
end

##############################################################################################################
# function: CheckScenarioInfoVarTableFalse = BuildCondition doc = "Please work function docs."
# 
# parameter 0: string   aiBrain         = "default_brain"       
# parameter 1: string   varName         = "VarName"         doc = "docs for param1"
#
##############################################################################################################
function CheckScenarioInfoVarTableFalse(aiBrain, varName)
    if ScenarioInfo.VarTable then
        local i = 1
        if ScenarioInfo.VarTable[varName] then
            return false
        end
        return true
    end
end

##############################################################################################################
# function: DifficultyEqual = BuildCondition    doc = "Please work function docs."
# 
# parameter 0: string   aiBrain         = "default_brain"       
# parameter 1: int  diffLevel         = "1"         doc = "docs for param1"
#
##############################################################################################################
function DifficultyEqual(aiBrain, diffLevel)
    if not ScenarioInfo.Options.Difficulty then
        return false
    end 
    if ScenarioInfo.Options.Difficulty == diffLevel then
        return true
    else
        return false
    end
end

##############################################################################################################
# function: DifficultyGreaterOrEqual = BuildCondition   doc = "Please work function docs."
# 
# parameter 0: string   aiBrain         = "default_brain"       
# parameter 1: int  diffLevel         = "1"         doc = "docs for param1"
#
##############################################################################################################
function DifficultyGreaterOrEqual(aiBrain, diffLevel)
    if not ScenarioInfo.Options.Difficulty then
        return false
    end 
    if ScenarioInfo.Options.Difficulty >= diffLevel then
        return true
    else
        return false
    end
end

##############################################################################################################
# function: DifficultyLessOrEqual = BuildCondition  doc = "Please work function docs."
# 
# parameter 0: string   aiBrain         = "default_brain"       
# parameter 1: int  diffLevel         = "1"         doc = "docs for param1"
#
##############################################################################################################
function DifficultyLessOrEqual(aiBrain, diffLevel)
    if not ScenarioInfo.Options.Difficulty then
        return false
    end 
    if ScenarioInfo.Options.Difficulty <= diffLevel then
        return true
    else
        return false
    end
end

##############################################################################################################
# function: MarkerChainExists = BuildCondition  doc = "Please work function docs."
# 
# parameter 0: string  aiBrain         = "default_brain"       
# parameter 1: string  chainName       = "CHAIN_NAME"         doc = "docs for param1"
#
##############################################################################################################
function MarkerChainExists(aiBrain, chainName)
    local chain = Scenario.Chains[chainName]
    if not chain then
        return false
    else
        return true
    end
end

##############################################################################################################
# function: FactionIndex = BuildCondition   doc = "Please work function docs."
# 
# parameter 0: string   aiBrain     = "default_brain"       
# parameter 1: int factionNum = "1"
# parameter 2: int otherFactionNum = "0"
# 
##############################################################################################################
function FactionIndex(aiBrain, factionNum, otherFactionNum)
    local facIndex = aiBrain:GetFactionIndex()
    #LOG('*AI DEBUG: ARMY 2: Getting Faction Index = ', facIndex, ' while factionNum = ', factionNum, ' and otherFactionNum = ', otherFactionNum)
    if otherFactionNum ~= 0 then
        if facIndex == factionNum or otherFactionNum == facIndex then
            return true
        else
            return false
        end
    elseif facIndex == factionNum then
        return true
    else
        return false
    end
end

##############################################################################################################
# function: ReclaimablesInArea = BuildCondition   doc = "Please work function docs."
# 
# parameter 0: string   aiBrain     = "default_brain"       
# parameter 1: string   locType     = "MAIN"
# 
##############################################################################################################
function ReclaimablesInArea(aiBrain, locType)
    local position, radius
    for k,v in aiBrain.PBM.Locations do
        if v.LocationType == locType then
            position = v.Location
            radius = v.Radius
            break
        end
    end
    if not position then
        return false
    end
    local x1 = position[1] - radius
    local x2 = position[1] + radius
    local z1 = position[3] - radius
    local z2 = position[3] + radius
    local rect = Rect( x1, z1, x2, z2 )
    local ents = GetEntitiesInRect( rect )
    local massNeed = false
    local energyNeed = false
    if aiBrain:GetEconomyStoredRatio('MASS') < .9 then
        massNeed = true
    end
    if aiBrain:GetEconomyStoredRatio('ENERGY') < .9 then
        energyNeed = true
    end
    for k,v in ents do
        if ( ( v.MassReclaim and massNeed ) or ( v.EnergyReclaim and energyNeed ) ) then
            return true
        end
    end
    return false
end

##############################################################################################################
# function: CheckAvailableGates = BuildCondition   doc = "Please work function docs."
# 
# parameter 0: string   aiBrain     = "default_brain"       
# parameter 1: string   locType     = "MAIN"
# 
##############################################################################################################
function CheckAvailableGates( aiBrain, locType )
    local pos, rad
    for k,v in aiBrain.PBM.Locations do
        if v.LocationType == locType then
            pos = v.Location
            rad = v.Radius
            break
        end
    end
    if not pos then
        return false
    end
    local gates = GetOwnUnitsAroundPoint( aiBrain, categories.GATE, pos, rad )
    if not gates then
        return false
    else
        for k,v in gates do
            if not v:IsUnitState('TransportLoading') then
                return true
            end
        end
    end
    return false
end



##############################################################################################################
# function: GreaterThanMapWaterRatio = BuildCondition  doc = "Please work function docs."
# 
# parameter 0: string   aiBrain         = "default_brain"       
# parameter 1: int  num             = 1         doc = "docs for param1"
#
##############################################################################################################
function GreaterThanMapWaterRatio(aiBrain, num)
    local ratio = aiBrain:GetMapWaterRatio()
    if ratio > num then
        return true
    end
    return false
end



##############################################################################################################
# function: ArmyNeedsTransports = BuildCondition  doc = "Please work function docs."
# 
# parameter 0: string   aiBrain         = "default_brain"       
#
##############################################################################################################
function ArmyNeedsTransports(aiBrain)
    if not aiBrain.NeedTransports then
        return false
    else
        return true
    end
end


##############################################################################################################
# function: CDRRunningAway = BuildCondition  doc = "Please work function docs."
# 
# parameter 0: string   aiBrain         = "default_brain"       
#
##############################################################################################################
function CDRRunningAway(aiBrain)
    local units = aiBrain:GetListOfUnits( categories.COMMAND, false )
    for k,v in units do
        if not v:IsDead() and v.Running then
            return true
        end
    end
    return false
end

