#****************************************************************************
#**
#**  File     :  /cdimage/lua/modules/editor/UnitCountBuildConditions.lua
#**  Author(s): Dru Staltman, John Comes
#**
#**  Summary  : Generic AI Platoon Build Conditions
#**             Build conditions always return true or false
#**
#**  Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************
local AIUtils = import('/lua/modules/ai/aiutilities.lua')
local ScenarioFramework = import('/lua/modules/scenarioframework.lua')
local ScenarioUtils = import('/lua/ScenarioUtilities.lua')
local Utils = import('/lua/modules/utilities.lua')

##############################################################################################################
# function: HaveEqualToUnitsWithCategory = BuildCondition	doc = "Please work function docs."
#
# parameter 0: string	aiBrain		= "default_brain"
# parameter 1: int      numReq     	= 0					doc = "docs for param1"
# parameter 2: expr   category        = categories.ALLUNITS			doc = "param2 docs"
# parameter 3: bool   idleReq       = false         doc = "docs for param3"
#
##############################################################################################################
function HaveEqualToUnitsWithCategory(aiBrain, numReq, category, idleReq)
    local numUnits
    if not idleReq then
        numUnits = aiBrain:GetCurrentUnits(category)
    else
        numUnits = table.getn(aiBrain:GetListOfUnits(category, true))
    end
    if numUnits == numReq then
        return true
    end
    return false
end

##############################################################################################################
# function: HaveGreaterThanUnitsWithCategory = BuildCondition	doc = "Please work function docs."
#
# parameter 0: string   aiBrain		    = "default_brain"
# parameter 1: int      numReq     = 0					doc = "docs for param1"
# parameter 2: expr   category        = categories.ALLUNITS		doc = "param2 docs"
# parameter 3: expr   idleReq       = false         doc = "docs for param3"
#
##############################################################################################################
function HaveGreaterThanUnitsWithCategory(aiBrain, numReq, category, idleReq)
    local numUnits
    if not idleReq then
        numUnits = aiBrain:GetCurrentUnits(category)
    else
        numUnits = table.getn(aiBrain:GetListOfUnits(category, true))
    end
    if numUnits > numReq then
        return true
    end
    return false
end


##############################################################################################################
# function: HaveLessThanUnitsWithCategory = BuildCondition	doc = "Please work function docs."
#
# parameter 0: string	aiBrain		= "default_brain"
# parameter 1: int	numReq          = 0				doc = "docs for param1"
# parameter 2: expr   category        = categories.ALLUNITS		doc = "param2 docs"
# parameter 3: expr   idleReq       = false         doc = "docs for param3"
#
##############################################################################################################
function HaveLessThanUnitsWithCategory(aiBrain, numReq, category, idleReq)
    local numUnits
    if not idleReq then
        numUnits = aiBrain:GetCurrentUnits(category)
    else
        numUnits = table.getn(aiBrain:GetListOfUnits(category, true))
    end
    if numUnits < numReq then
        return true
    end
    return false
end

##############################################################################################################
# function: HaveLessThanUnitsWithCategoryInArea = BuildCondition	doc = "Please work function docs."
#
# parameter 0: string	aiBrain	        = "default_brain"
# parameter 1: int      numReq          = 0				doc = "docs for param1"
# parameter 2: expr   category        = categories.ALLUNITS		doc = "param2 docs"
# parameter 3: string   area            = "Area_1"
#
##############################################################################################################
function HaveLessThanUnitsWithCategoryInArea(aiBrain, numReq, category, area)
    local numUnits = ScenarioFramework.NumCatUnitsInArea(category, ScenarioUtils.AreaToRect(area), aiBrain )
    if numUnits < numReq then
        return true
    end
    return false
end

##############################################################################################################
# function: NumUnitsLessNearBase = BuildCondition	doc = "Please work function docs."
#
# parameter 0: string	aiBrain         = "default_brain"
# parameter 1: string	baseName        = "MAIN"			doc = "docs for param1"
# parameter 2: expr   category        = categories.ALLUNITS
# parameter 3: int      num             = 1
#
##############################################################################################################
function NumUnitsLessNearBase( aiBrain, baseName, category, num )
    if brain.BaseTemplates[baseName].Location == nil then
        return false
    else
        local unitList = brain:GetUnitsAroundPoint('Ally', category,
                                                   brain.BaseTemplates[baseName].Location,
                                                   brain.BaseTemplates[baseName].Radius )
        local count = 0
        for i,unit in unitList do
            if unit:GetAIBrain() == brain then
                count = count + 1
            end
        end
        if count < num then
            return true
        end
        return false
    end
end


##############################################################################################################
# function: HaveLessThanUnitComparison = BuildCondition	doc = "Check to see if the number of units in category 1 is less than the number of units in category 2"
#
# parameter 0: string	aiBrain		= "default_brain"
# parameter 1: expr	category1    = categories.ALLUNITS       doc = "Category of units to compare"
# parameter 2: expr     category2    = categories.ALLUNITS       doc = "Category of units to compare against"
#
##############################################################################################################
function HaveLessThanUnitComparison(aiBrain, category1, category2)
    local numUnits1 = aiBrain:GetCurrentUnits(category1)
    local numUnits2 = aiBrain:GetCurrentUnits(category2)
    if numUnits1 < numUnits2 then
        return true
    end
    return false
end

##############################################################################################################
# function: HaveGreaterThanUnitComparison = BuildCondition	doc = "Check to see if the number of units in category 1 is greater than the number of units in category 2"
#
# parameter 0: string	aiBrain		= "default_brain"
# parameter 1: expr	category1    = categories.ALLUNITS       doc = "Category of units to compare"
# parameter 2: expr     category2    = categories.ALLUNITS       doc = "Category of units to compare against"
#
##############################################################################################################
function HaveGreaterThanUnitComparison(aiBrain, category1, category2)
    local numUnits1 = aiBrain:GetCurrentUnits(category1)
    local numUnits2 = aiBrain:GetCurrentUnits(category2)
    if numUnits1 > numUnits2 then
        return true
    end
    return false
end

##############################################################################################################
# function: HaveLessThanVarTableUnitsWithCategory = BuildCondition	doc = "Please work function docs."
#
# parameter 0: string	aiBrain		= "default_brain"
# parameter 1: string	varName     = "VarName"     			doc = "VarTable Name"
# parameter 2: expr     category    = categories.ALLUNITS		doc = "param2 docs"
#
##############################################################################################################
function HaveLessThanVarTableUnitsWithCategory(aiBrain, varName, category)
    local numReq
    local numUnits = aiBrain:GetCurrentUnits(category)
    if ScenarioInfo.VarTable[varName] then
        if numUnits < ScenarioInfo.VarTable[varName] then
            return true
        end
    end
    return false
end

##############################################################################################################
# function: HaveGreaterThanVarTableUnitsWithCategory = BuildCondition	doc = "Please work function docs."
#
# parameter 0: string	aiBrain		= "default_brain"
# parameter 1: string	varName     = "VarName"     			doc = "VarTable Name"
# parameter 2: expr     category    = categories.ALLUNITS		doc = "param2 docs"
#
##############################################################################################################
function HaveGreaterThanVarTableUnitsWithCategory(aiBrain, varName, category)
    local numReq
    local numUnits = aiBrain:GetCurrentUnits(category)
    if ScenarioInfo.VarTable[varName] then
        if numUnits > ScenarioInfo.VarTable[varName] then
            return true
        end
    end
    return false
end
##############################################################################################################
# function: HaveLessThanVarTableUnitsWithCategoryInArea = BuildCondition	doc = "Please work function docs."
#
# parameter 0: string	aiBrain		= "default_brain"
# parameter 1: string	varName     = "VarName"     			doc = "VarTable Name"
# parameter 2: expr     category    = categories.ALLUNITS		doc = "param2 docs"
# parameter 3: string   area            = "Area_1"
#
##############################################################################################################
function HaveLessThanVarTableUnitsWithCategoryInArea(aiBrain, varName, category, area)
    local numReq
    local numUnits = ScenarioFramework.NumCatUnitsInArea(category, ScenarioUtils.AreaToRect(area), aiBrain )
    if ScenarioInfo.VarTable[varName] then
        if numUnits < ScenarioInfo.VarTable[varName] then
            return true
        end
    end
    return false
end
##############################################################################################################
# function: HaveGreaterThanVarTableUnitsWithCategoryInArea = BuildCondition	doc = "Please work function docs."
#
# parameter 0: string	aiBrain		= "default_brain"
# parameter 1: string	varName     = "VarName"     			doc = "VarTable Name"
# parameter 2: expr     category    = categories.ALLUNITS		doc = "param2 docs"
# parameter 3: string   area            = "Area_1"
#
##############################################################################################################
function HaveGreaterThanVarTableUnitsWithCategoryInArea(aiBrain, varName, category, area)
    local numReq
    local numUnits = ScenarioFramework.NumCatUnitsInArea(category, ScenarioUtils.AreaToRect(area), aiBrain )
    if ScenarioInfo.VarTable[varName] then
        if numUnits > ScenarioInfo.VarTable[varName] then
            return true
        end
    end
    return false
end

##############################################################################################################
# function: HaveGreaterThanUnitsInCategoryBeingBuilt = BuildCondition	doc = "Please work function docs."
#
# parameter 0: string	aiBrain		= "default_brain"
# parameter 1: int      numReq     	= 0					doc = "docs for param1"
# parameter 2: expr   category        = categories.ALLUNITS			doc = "param2 docs"
#
##############################################################################################################
function HaveGreaterThanUnitsInCategoryBeingBuilt(aiBrain, numReq, category)
    local numUnits = aiBrain:NumCurrentlyBuilding(category)
    if numUnits > numReq then
        return true
    end
    return false
end


##############################################################################################################
# function: HaveLessThanUnitsInCategoryBeingBuilt = BuildCondition	doc = "Please work function docs."
#
# parameter 0: string	aiBrain		= "default_brain"
# parameter 1: int      numReq     	= 0					doc = "docs for param1"
# parameter 2: expr   category        = categories.ALLUNITS			doc = "param2 docs"
#
##############################################################################################################
function HaveLessThanUnitsInCategoryBeingBuilt(aiBrain, numReq, category)
    local numUnits = aiBrain:NumCurrentlyBuilding(category)
    if numUnits < numReq then
        return true
    end
    return false
end

##############################################################################################################
# function: HaveLessThanUnitsAroundMarkerCategory = BuildCondition	doc = "Please work function docs."
#
# parameter 0: string	aiBrain		= "default_brain"
# parameter 1: string   markerType  = "Mass"	
# parameter 2: int      markerRadius = 50
# parameter 3: string   locationType = "MAIN"
# parameter 4: int      locationRadius = 50
# parameter 5: int      unitCount    = 1
# parameter 6: string   unitCategory = "ALLUNITS"
# parameter 7: expr     threatMin = false
# parameter 8: expr     threatMax = false
# parameter 9: expr     threatRings = false
#
##############################################################################################################
function HaveLessThanUnitsAroundMarkerCategory( aiBrain, markerType, markerRadius, locationType, locationRadius, 
    unitCount, unitCategory, threatMin, threatMax, threatRings )
    local pos = aiBrain:PBMGetLocationCoords( locationType )
    if not pos then
        return false
    end
    local positions = AIUtils.AIGetMarkersAroundLocation( aiBrain, markerType, pos, locationType, threatMin, threatMax, threatRings)
    for k,v in positions do
        local unitTotal = table.getn( AIUtils.GetOwnUnitsAroundPoint( aiBrain, ParseEntityCategory(unitCategory), v.Position, markerRadius, threatMin, 
            threatMax, threatRings) )
        if unitTotal < unitCount then
            return true
        end
    end
    return false            
end

function HaveAreaWithUnitsFewWalls( aiBrain, locationType, locationRadius, unitCount, unitCategory, threatMin, threatMax, threatRings )
    local pos = aiBrain:PBMGetLocationCoords( locationType )
    if not pos then
        return false
    end
    local positions = {}
    for k,v in aiBrain.PBM.Locations do
        if v.LocationType ~= locationType and Utils.XZDistanceTwoVectors( pos, v.Location ) <= locationRadius then
            table.insert( positions, v.Location )
        end
    end
    local otherPos = AIUtils.AIGetMarkersAroundLocation( aiBrain, 'Defensive Point', pos, locationRadius, threatMin, threatMax, threatRings)
    for k,v in otherPos do
        table.insert( positions, v.Position )
    end
    for k,v in positions do
        local unitTotal = table.getn( AIUtils.GetOwnUnitsAroundPoint( aiBrain, ParseEntityCategory('DEFENSE'), v, 30, threatMin, 
            threatMax, threatRings) )
        if unitTotal > unitCount then
            if aiBrain:GetNumUnitsAroundPoint( 'Ally', categories.WALL, v, 30 ) < 15 then
                return true
            end
        end
    end
    return false                
end



##############################################################################################################
# function: HaveUnitsWithCategoryAndAlliance = BuildCondition	doc = "Please work function docs."
#
# parameter 0: string   aiBrain		    = "default_brain"
# parameter 1: bool   greater           = true          doc = "true = greater, false = less"
# parameter 2: int    numReq     = 0					doc = "docs for param1"
# parameter 3: expr   category        = categories.ALLUNITS		doc = "param2 docs"
# parameter 4: expr   alliance       = false         doc = "docs for param3"
#
##############################################################################################################
function HaveUnitsWithCategoryAndAlliance(aiBrain, greater, numReq, category, alliance)
    local numUnits = aiBrain:GetNumUnitsAroundPoint(alliance, category, Vector(0,0,0), 100000)
    if numUnits > numReq and greater then
        return true
    elseif numUnits < numReq and not greater then
        return true
    end
    return false
end
