#****************************************************************************
#**
#**  File     :  /cdimage/lua/modules/formations.lua
#**  Author(s):
#**
#**  Summary  :
#**
#**  Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************
#
# Basic create formation scripts
#

SurfaceFormations = {
    'AttackFormation',
    'GrowthFormation',
    'SixWideFormation',
    'TravellingFormation',
}

AirFormations = {
    'StaggeredChevronFormation',
    'ChevronFormation',
}

ComboFormations = {
    'AttackFormation',
    'GrowthFormation',
    'SixWideFormation',
    'TravellingFormation',
    'ChevronFormation',
}

local FormationPos = {} # list to be returned

#=========================================#
#================ LAND DATA ==============#
#=========================================#
local RemainingCategory = { 'RemainingCategory', }
   
#=== LAND CATEGORIES ===#
local DirectFire = (( categories.DIRECTFIRE - categories.CONSTRUCTION ) ) * categories.LAND
local Construction = ( categories.COMMAND + categories.CONSTRUCTION + categories.ENGINEER ) * categories.LAND
local Artillery = (( categories.ARTILLERY + categories.INDIRECTFIRE ) - categories.ANTIAIR ) * categories.LAND
local AntiAir = ( categories.ANTIAIR - ( categories.EXPERIMENTAL + categories.DIRECTFIRE ) ) * categories.LAND
local UtilityCat = ( ( categories.RADAR + categories.COUNTERINTELLIGENCE ) - categories.DIRECTFIRE ) * categories.LAND
local DFHeavy = DirectFire * ( categories.TECH3 + categories.EXPERIMENTAL )
local ShieldCat = categories.uel0307 + categories.ual0307

#=== TECH LEVEL LAND CATEGORIES ===#
local LandCategories = {
    Tank1 = DirectFire * categories.TECH1,
    Tank2 = DirectFire * categories.TECH2,
    Tank3 = DirectFire * categories.TECH3,

    Art1 = Artillery * categories.TECH1,
    Art2 = Artillery * categories.TECH2,
    Art3 = Artillery * categories.TECH3,

    AA1 = AntiAir * categories.TECH1,
    AA2 = AntiAir * categories.TECH2,
    AA3 = AntiAir * categories.TECH3,

    Com1 = Construction * categories.TECH1,
    Com2 = Construction * categories.TECH2,
    Com3 = Construction - (categories.TECH1 + categories.TECH2),

    Util1 = (UtilityCat * categories.TECH1) + categories.OPERATION,
    Util2 = UtilityCat * categories.TECH2,
    Util3 = UtilityCat * categories.TECH3,

    Shields = ShieldCat,
    
    Experimentals = DFHeavy,
    
    RemainingCategory = categories.ALLUNITS - ( DirectFire + Construction + Artillery + AntiAir + UtilityCat + DFHeavy + ShieldCat )
}

#=== SUB GROUP ORDERING ===#
local Tanks = { 'Tank3', 'Tank2', 'Tank1', }
local Art = { 'Art3', 'Art2', 'Art1', }
local AA = { 'AA3', 'AA2', 'AA1', }
local Util = { 'Util3', 'Util2', 'Util1', }
local Com = { 'Com3', 'Com2', 'Com1', }
local Shield = { 'Shields', }
local Experimental = { 'Experimentals', }


#=== LAND BLOCK TYPES =#
local TankFirst = { Experimental, Tanks, Art, AA, Shield, Com, Util, RemainingCategory }
local ShieldFirst = { Shield, Art, Tanks, AA, Com, Util, RemainingCategory }
local AAFirst = { AA, Tanks, Art, Shield, Com, Util, RemainingCategory }
local ArtFirst = { Art, Tanks, AA, Shield, Com, Util, RemainingCategory }


#=== LAND BLOCKS ===#

#=== Attack Formation Block ==#
local AttackFormationBlock = {
    ## first row
    { TankFirst, TankFirst, TankFirst, TankFirst, TankFirst, TankFirst, TankFirst, TankFirst, TankFirst, TankFirst, },
    ## Second Row
    { TankFirst, AAFirst, ShieldFirst, TankFirst, TankFirst, TankFirst, TankFirst, ShieldFirst, AAFirst, TankFirst, },
    #### Third Row
    { TankFirst, AAFirst, ArtFirst, ArtFirst, ArtFirst, ArtFirst, ArtFirst, ArtFirst, AAFirst, TankFirst, },
    ## Fourth Row
    { TankFirst, AAFirst, ArtFirst, ArtFirst, ArtFirst, ArtFirst, ArtFirst, ArtFirst, AAFirst, TankFirst, },
    ## Fifth Row
    { TankFirst, AAFirst, AAFirst, TankFirst, AAFirst, AAFirst, TankFirst, AAFirst, AAFirst, TankFirst, },
}

#=== 9 Wide Attack Block ===#
local NineWideAttackFormationBlock = {
    ## first row
    { TankFirst, TankFirst, TankFirst, TankFirst, TankFirst, TankFirst, TankFirst, TankFirst, TankFirst, },
    ## Second Row
    { TankFirst, AAFirst, ShieldFirst, TankFirst, TankFirst, TankFirst, ShieldFirst, AAFirst, TankFirst, },
    #### Third Row
    { TankFirst, AAFirst, ArtFirst, ArtFirst, ArtFirst, ArtFirst, ArtFirst, AAFirst, TankFirst, },
    ## Fourth Row
    { TankFirst, AAFirst, ArtFirst, ArtFirst, ArtFirst, ArtFirst, ArtFirst, AAFirst, TankFirst, },
    ## Fifth Row
    { TankFirst, AAFirst, AAFirst, TankFirst, AAFirst, TankFirst, AAFirst, AAFirst, TankFirst, },
}

#=== 8 Wide Attack Block ===#
local EightWideAttackFormationBlock = {
    ## first row
    { TankFirst, TankFirst, TankFirst, TankFirst, TankFirst, TankFirst, TankFirst, TankFirst, },
    ## Second Row
    { TankFirst, AAFirst, ShieldFirst, TankFirst, TankFirst, ShieldFirst, AAFirst, TankFirst, },
    #### Third Row
    { TankFirst, AAFirst, ArtFirst, ArtFirst, ArtFirst, ArtFirst, AAFirst, TankFirst, },
    ## Fourth Row
    { TankFirst, AAFirst, ArtFirst, ArtFirst, ArtFirst, ArtFirst, AAFirst, TankFirst, },
    ## Fifth Row
    { TankFirst, AAFirst, AAFirst, TankFirst, TankFirst, AAFirst, AAFirst, TankFirst, },
}

#=== 7 Wide Attack Block ===#
local SevenWideAttackFormationBlock = {
    ## first row
    { TankFirst, TankFirst, TankFirst, TankFirst, TankFirst, TankFirst, TankFirst, },
    ## Second Row
    { TankFirst, AAFirst, ShieldFirst, TankFirst, AAFirst,  ShieldFirst, TankFirst, },
    #### Third Row
    { TankFirst, AAFirst, ArtFirst, ArtFirst, ArtFirst, AAFirst, TankFirst, },
    ## Fourth Row
    { TankFirst, AAFirst, ArtFirst, ArtFirst, ArtFirst, AAFirst, TankFirst, },
    ## Fifth Row
    { TankFirst, AAFirst, AAFirst, TankFirst, AAFirst, AAFirst, TankFirst, },
}

#=== 6 Wide Attack Block ===#
local SixWideAttackFormationBlock = {
    ## first row
    { TankFirst, TankFirst, TankFirst, TankFirst, TankFirst, TankFirst, },
    ## Second Row
    { TankFirst, ShieldFirst, AAFirst, AAFirst, ShieldFirst,  TankFirst, },
    #### Third Row
    { TankFirst, AAFirst, ArtFirst, ArtFirst, AAFirst, TankFirst, },
    ## Fourth Row
    { TankFirst, AAFirst, ArtFirst, ArtFirst, AAFirst, TankFirst, },
}

#=== 5 Wide Attack Block ===#
local FiveWideAttackFormationBlock = {
    ## first row
    { TankFirst, TankFirst, TankFirst, TankFirst, TankFirst, },
    ## Second Row
    { TankFirst, ShieldFirst, AAFirst, ShieldFirst,  TankFirst, },
    #### Third Row
    { AAFirst, ArtFirst, ArtFirst, ArtFirst, AAFirst, },
}

#=== 4 Wide Attack Block ===#
local FourWideAttackFormationBlock = {
    ## first row
    { TankFirst, TankFirst, TankFirst, TankFirst, },
    ## Second Row
    { ShieldFirst, AAFirst, AAFirst, ShieldFirst,  },
}

#=== 3 Wide Attack Block ===#
local ThreeWideAttackFormationBlock = {
    ## first row
    { TankFirst, TankFirst, TankFirst, },
}

#=== Travelling Block ===#
local TravelSlot = { Experimental, Tanks, AA, Art, Shield, Util, Com }
local TravelFormationBlock = {
    HomogenousRows = true,
    UtilBlocks = true,
    RowBreak = 0.5,
    { TravelSlot, TravelSlot, },
    { TravelSlot, TravelSlot, },
    { TravelSlot, TravelSlot, },
    { TravelSlot, TravelSlot, },
    { TravelSlot, TravelSlot, },
}




#=========================================#
#================ AIR DATA ===============#
#=========================================#

#=== AIR CATEGORIES ===#
local GroundAttackAir = categories.AIR * categories.GROUNDATTACK
local TransportationAir = categories.AIR * categories.TRANSPORTATION - categories.GROUNDATTACK
local BomberAir = categories.AIR * categories.BOMBER
local AAAir = categories.AIR * categories.ANTIAIR
local AntiNavyAir = categories.AIR * categories.ANTINAVY
local IntelAir = categories.AIR * ( categories.SCOUT + categories.RADAR )
local ExperimentalAir = categories.AIR * categories.EXPERIMENTAL

#=== TECH LEVEL AIR CATEGORIES ===#
local AirCategories = {
    Ground1 = GroundAttackAir * categories.TECH1,
    Ground2 = GroundAttackAir * categories.TECH2,
    Ground3 = GroundAttackAir * categories.TECH3,

    Trans1 = TransportationAir * categories.TECH1,
    Trans2 = TransportationAir * categories.TECH2,
    Trans3 = TransportationAir* categories.TECH3,

    Bomb1 = BomberAir * categories.TECH1,
    Bomb2 = BomberAir * categories.TECH2,
    Bomb3 = BomberAir * categories.TECH3,

    AA1 = AAAir * categories.TECH1,
    AA2 = AAAir * categories.TECH2,
    AA3 = AAAir * categories.TECH3,

    AN1 = AntiNavyAir * categories.TECH1,
    AN2 = AntiNavyAir * categories.TECH2,
    AN3 = AntiNavyAir * categories.TECH3,

    AIntel1 = IntelAir * categories.TECH1,
    AIntel2 = IntelAir * categories.TECH2,
    AIntel3 = IntelAir * categories.TECH3,

    AExper = ExperimentalAir,
    
    RemainingCategory = categories.ALLUNITS - ( GroundAttackAir + TransportationAir + BomberAir + AAAir + AntiNavyAir + IntelAir + ExperimentalAir )
}

#=== SUB GROUP ORDERING ===#
local GroundAttack = { 'Ground3', 'Ground2', 'Ground1', }
local Transports = { 'Trans3', 'Trans2', 'Trans1', }
local Bombers = { 'Bomb3', 'Bomb2', 'Bomb1', }
local AntiAir = { 'AA3', 'AA2', 'AA1', }
local AntiNavy = { 'AN3', 'AN2', 'AN1', }
local Intel = { 'AIntel3', 'AIntel2', 'AIntel1', }
local ExperAir = { 'AExper', }

#=== Air Block Arrangement ===#
local ChevronSlot = { AntiAir, ExperAir, AntiNavy, GroundAttack, Bombers, Intel, Transports, RemainingCategory }
local InitialChevronBlock = {
    RepeatAllRows = false,
    HomogenousBlocks = true,
    ChevronSize = 3,
    { ChevronSlot },
    { ChevronSlot, ChevronSlot },
}

local StaggeredChevronBlock = {
    RepeatAllRows = true,
    HomogenousBlocks = true,
    { ChevronSlot, ChevronSlot, ChevronSlot, },
    { ChevronSlot, ChevronSlot, },
}



#=========================================#
#============== NAVAL DATA ===============#
#=========================================#

##### NAVAL CATEGORIES #####
local LightAttackNaval = categories.uas0102
local FrigateNaval = categories.uas0103 + categories.ues0103 + categories.urs0103
local SubNaval = categories.uas0203 + categories.ues0203 + categories.urs0203
local DestroyerNaval = categories.uas0201 + categories.urs0201 + categories.ues0201
local CruiserNaval = categories.uas0202 + categories.ues0202 + categories.urs0202
local BattleshipNaval = categories.uas0302 + categories.ues0302 + categories.urs0302
local CarrierNaval = categories.uas0303 + categories.urs0303
local NukeSubNaval = categories.uas0304 + categories.ues0304 + categories.urs0304
local UnderwaterCarrierNaval = categories.ues0401 + categories.uas0401

#### Naval formation blocks #####
local NavalSpacing = 1.2
local StandardNavalBlock = {
    { { {0, 0}, }, { 'Carriers', 'Battleships', 'Cruisers', 'Destroyers', 'Frigates', 'Submarines' }, },
    { { {-1, 1.5}, {1, 1.5}, }, { 'Destroyers', 'Cruisers', 'Frigates', 'Submarines'}, },
    { { {-2.5, 0}, {2.5, 0}, }, { 'Cruisers', 'Battleships', 'Destroyers', 'Frigates', 'Submarines' }, },
    { { {-1, -1.5}, {1, -1.5}, }, { 'Frigates', 'Battleships', 'Submarines' }, },
    { { {-3, 2}, {3, 2}, {-3, 0}, {3, 0}, }, { 'Submarines', }, },
}

#=== TECH LEVEL LAND CATEGORIES ===#
local NavalCategories = {
    LightCount = LightAttackNaval,
    FrigateCount = FrigateNaval,

    CruiserCount = CruiserNaval,
    DestroyerCount = DestroyerNaval,
    
    BattleshipCount = BattleshipNaval,
    CarrierCount = CarrierNaval + UnderwaterCarrierNaval,
    
    NukeSubCount = NukeSubNaval,
    
    RemainingCategory = categories.ALLUNITS - ( LightAttackNaval + FrigateNaval + SubNaval + DestroyerNaval + CruiserNaval + BattleshipNaval 
                        + CarrierNaval + NukeSubNaval + UnderwaterCarrierNaval )
}

local SubCategories = {
    SubCount = SubNaval,
}

#=== SUB GROUP ORDERING ===#
local Frigates = { 'FrigateCount', 'LightCount', }
local Destroyers = { 'DestroyerCount', }
local Cruisers = { 'CruiserCount', }
local Battleships = { 'BattleshipCount', }
local Subs = { 'SubCount', }
local NukeSubs = { 'NukeSubCount', }
local Carriers = { 'CarrierCount', }


#=== LAND BLOCK TYPES =#
local FrigatesFirst = { Frigates, Destroyers, Cruisers, Battleships, Carriers, NukeSubs, RemainingCategory }
local DestroyersFirst = { Destroyers, Frigates, Battleships, Cruisers, Carriers, NukeSubs, RemainingCategory }
local CruisersFirst = { Cruisers, Battleships, Frigates, Destroyers, Carriers, NukeSubs, RemainingCategory }
local BattleshipsFirst = { Battleships, Destroyers, Frigates, Cruisers, Carriers, NukeSubs, RemainingCategory }
local CarriersFirst = { Carriers, NukeSubs, Cruisers, Battleships, Destroyers, Frigates, RemainingCategory }
local Subs = { Subs, RemainingCategory }


#=== LAND BLOCKS ===#

#=== Three Naval Growth Formation Block ==#
local ThreeNavalGrowthFormation = {
    LineBreak = 0.5,
    ## first row
    { FrigatesFirst, FrigatesFirst, FrigatesFirst, },
}

#=== Three Naval Growth Formation Block ==#
local FourNavalGrowthFormation = {
    LineBreak = 0.5,
    ## first row
    { FrigatesFirst, FrigatesFirst, FrigatesFirst, FrigatesFirst, },
    ## second row
    { DestroyersFirst, CruisersFirst, CruisersFirst, DestroyersFirst, },
}

#=== Three Naval Growth Formation Block ==#
local FiveNavalGrowthFormation = {
    LineBreak = 0.5,
    ## first row
    { FrigatesFirst, FrigatesFirst, FrigatesFirst, FrigatesFirst, FrigatesFirst },
    ## second row
    { DestroyersFirst, CruisersFirst, BattleshipsFirst, CruisersFirst, DestroyersFirst },
    ## third row
    { DestroyersFirst, CruisersFirst, CarriersFirst, CruisersFirst, DestroyersFirst },
}

#=== Three Naval Growth Formation Block ==#
local SixNavalGrowthFormation = {
    LineBreak = 0.5,
    ## first row
    { FrigatesFirst, FrigatesFirst, FrigatesFirst, FrigatesFirst, FrigatesFirst, FrigatesFirst },
    ## second row
    { DestroyersFirst, CruisersFirst, BattleshipsFirst, BattleshipsFirst, FrigatesFirst, DestroyersFirst },
    ## third row
    { DestroyersFirst, BattleshipsFirst, CruisersFirst, CruisersFirst, BattleshipsFirst, DestroyersFirst },
    ## fourth row
    { DestroyersFirst, CruisersFirst, CarriersFirst, CarriersFirst, CruisersFirst, DestroyersFirst },
}


#=== Three Wide Subs Formation ===#
local ThreeWideSubGrowthFormation = {
    LineBreak = 0.5,
    { Subs, Subs },
}

#=== Four Wide Subs Formation ===#
local FourWideSubGrowthFormation = {
    LineBreak = 0.5,
    { Subs, Subs, Subs },
}

#=== Five Wide Subs Formation ===#
local FiveWideSubGrowthFormation = {
    LineBreak = 0.5,
    { Subs, Subs, Subs, Subs },
}

#=== Six Wide Subs Formation ===#
local SixWideSubGrowthFormation = {
    LineBreak = 0.5,
    { Subs, Subs, Subs, Subs, Subs },
}





#============ Formation Pickers ============#
function PickBestTravelFormationIndex( typeName, distance )
    if typeName == 'AirFormations' then
        return 0;
    else
        return 1;
    end
end

function PickBestFinalFormationIndex( typeName, distance )
    return 0;
end








#================ THE GUTS ====================#
#============ Formation Functions =============#
#==============================================#
function TravellingFormation(formationUnits)
    FormationPos = {}

    local landUnitsList = CategorizeLandUnits( formationUnits )
    BlockBuilderLand(landUnitsList, TravelFormationBlock, LandCategories)

    local seaUnitsList = CategorizeSeaUnits( formationUnits )
    NavalBlocks( seaUnitsList, StandardNavalBlock )

    local airUnitsList = CategorizeAirUnits( formationUnits )
    BlockBuilderAir(airUnitsList, StaggeredChevronBlock)

    return FormationPos
end

function AttackFormation( formationUnits )
    FormationPos = {}

    local landUnitsList = CategorizeLandUnits( formationUnits )
    BlockBuilderLand(landUnitsList, AttackFormationBlock, LandCategories)

    local seaUnitsList = CategorizeSeaUnits( formationUnits )
    NavalBlocks( seaUnitsList, StandardNavalBlock )

    local airUnitsList = CategorizeAirUnits( formationUnits )
    BlockBuilderAir(airUnitsList, StaggeredChevronBlock)

    return FormationPos
end

function GrowthFormation( formationUnits )
    FormationPos = {}

    local landUnitsList = CategorizeLandUnits( formationUnits )
    local landBlock
    if landUnitsList.UnitTotal <= 3 then
        landBlock = ThreeWideAttackFormationBlock
    elseif landUnitsList.UnitTotal <= 8 then        
        landBlock = FourWideAttackFormationBlock
    elseif landUnitsList.UnitTotal <= 15 then        
        landBlock = FiveWideAttackFormationBlock
    elseif landUnitsList.UnitTotal <= 24 then        
        landBlock = SixWideAttackFormationBlock
    elseif landUnitsList.UnitTotal <= 35 then        
        landBlock = SevenWideAttackFormationBlock
    elseif landUnitsList.UnitTotal <= 48 then        
        landBlock = EightWideAttackFormationBlock
    elseif landUnitsList.UnitTotal <= 63 then        
        landBlock = NineWideAttackFormationBlock
    else
        landBlock = AttackFormationBlock
    end
    BlockBuilderLand(landUnitsList, landBlock, LandCategories)

    local seaUnitsList = CategorizeSeaUnits( formationUnits )
    seaUnitsList.UnitTotal = seaUnitsList.UnitTotal - seaUnitsList.SubCount
    local seaBlock
    local subBlock
    local subUnitsList = {}
    subUnitsList.UnitTotal = seaUnitsList.SubCount
    subUnitsList.SubCount = seaUnitsList.SubCount
    local seaCounter = seaUnitsList.UnitTotal
    if seaUnitsList.UnitTotal < subUnitsList.UnitTotal then
        seaCounter = subUnitsList.UnitTotal
    end
    if seaCounter <= 3 then
        seaBlock = ThreeNavalGrowthFormation
        subBlock = ThreeWideSubGrowthFormation
    elseif seaCounter <= 8 then
        seaBlock = FourNavalGrowthFormation
        subBlock = FourWideSubGrowthFormation
    elseif seaCounter <= 15 then
        seaBlock = FiveNavalGrowthFormation
        subBlock = FiveWideSubGrowthFormation
    else
        seaBlock = SixNavalGrowthFormation
        subBlock = SixWideSubGrowthFormation
    end
    BlockBuilderLand( seaUnitsList, seaBlock, NavalCategories, 1.5 )
    BlockBuilderLand( subUnitsList, subBlock, SubCategories, 1.5 )

    local airUnitsList = CategorizeAirUnits( formationUnits )
    BlockBuilderAir(airUnitsList, StaggeredChevronBlock)

    return FormationPos
end

function SixWideFormation( formationUnits )
    FormationPos = {}

    local landUnitsList = CategorizeLandUnits( formationUnits )
    BlockBuilderLand(landUnitsList, SixWideAttackFormationBlock, LandCategories)

    local seaUnitsList = CategorizeSeaUnits( formationUnits )
    NavalBlocks( seaUnitsList, StandardNavalBlock )

    local airUnitsList = CategorizeAirUnits( formationUnits )
    BlockBuilderAir(airUnitsList, StaggeredChevronBlock)

    return FormationPos
end

function ChevronFormation( formationUnits )
    FormationPos = {}
    local airUnitsList = CategorizeAirUnits(formationUnits)
    local seaUnitsList = CategorizeSeaUnits(formationUnits)
    local landUnitsList = CategorizeLandUnits(formationUnits)
    BlockBuilderLand(landUnitsList, AttackFormationBlock, LandCategories)
    BlockBuilderAir( airUnitsList, InitialChevronBlock )
    NavalBlocks( seaUnitsList, StandardNavalBlock )
    return FormationPos
end

function StaggeredChevronFormation(formationUnits)
    FormationPos = {}
    local airUnitsList = CategorizeAirUnits(formationUnits)
    local seaUnitsList = CategorizeSeaUnits(formationUnits)
    local landUnitsList = CategorizeLandUnits( formationUnits )
    BlockBuilderLand(landUnitsList, AttackFormationBlock, LandCategories)
    BlockBuilderAir( airUnitsList, StaggeredChevronBlock )
    NavalBlocks( seaUnitsList, StandardNavalBlock )
    return FormationPos
end

function BlockFormation( formationUnits )
    local rotate = true
    local smallUnitsList = {}
    local largeUnitsList = {}
    local smallUnits = 0
    local largeUnits = 0

    for i,u in formationUnits do
        local footPrintSize = u:GetFootPrintSize()
        if footPrintSize > 3  then
            largeUnitsList[largeUnits] = { u }
            largeUnits = largeUnits + 1
        else
            smallUnitsList[smallUnits] = { u }
            smallUnits = smallUnits + 1
        end
    end

    local FormationPos = {}
    local n = smallUnits + largeUnits
    local width = math.ceil(math.sqrt(n))
    local length = n / width

    # Put small units (Size 1 through 3) in front of the formation
    for i in smallUnitsList do
        local offsetX = (( math.mod(i,width)  - math.floor(width/2) ) * 2) + 1
        local offsetY = ( math.floor(i/width) - math.floor(length/2) ) * 2
        local delay = 0.1 + (math.floor(i/width) * 3)
        table.insert(FormationPos, { offsetX, -offsetY, categories.ALLUNITS, delay, rotate })
    end

    # Put large units (Size >= 4) in the back of the formation
    for i in largeUnitsList do
        local adjIndex = smallUnits + i
        local offsetX = (( math.mod(adjIndex,width)  - math.floor(width/2) ) * 2) + 1
        local offsetY = ( math.floor(adjIndex/width) - math.floor(length/2) ) * 2
        local delay = 0.1 + (math.floor(adjIndex/width) * 3)
        table.insert(FormationPos, { offsetX, -offsetY, categories.ALLUNITS, delay, rotate })
    end

    return FormationPos
end

# local function for performing lerp
local function lerp( alpha, a, b )
    return a + ((b-a) * alpha)
end

function CircleFormation( formationUnits )
    local rotate = false
    local FormationPos = {}
    local numUnits = table.getn(formationUnits)
    local sizeMult = 2.0 + math.max(1.0, numUnits / 3.0)

    # make circle around center point
    for i in formationUnits do
        offsetX = sizeMult * math.sin( lerp( i/numUnits, 0.0, math.pi * 2.0 ) )
        offsetY = sizeMult * math.cos( lerp( i/numUnits, 0.0, math.pi * 2.0 ) )
        table.insert(FormationPos, { offsetX, offsetY, categories.ALLUNITS, 0, rotate })
    end

    return FormationPos
end



#=========== LAND BLOCK BUILDING =================#
function BlockBuilderLand(unitsList, formationBlock, categoryTable, spacing)
    spacing = spacing or 1
    local numRows = table.getn(formationBlock)
    local i = 1
    local whichRow = 1
    local whichCol = 1
    local currRowLen = table.getn(formationBlock[whichRow])
    local rowType = false
    local formationLength = 0
    local inserted = false
    while unitsList.UnitTotal >= i do
        if whichCol > currRowLen then
            if whichRow == numRows then
                whichRow = 1
                if formationBlock.RowBreak then
                    formationLength = formationLength + 1 + formationBlock.RowBreak
                else
                    formationLength = formationLength + 1
                end
            else
                whichRow = whichRow + 1
                if formationBlock.LineBreak then
                    formationLength = formationLength + 1 + formationBlock.LineBreak
                else
                    formationLength = formationLength + 1
                end
                rowType = false
            end
            whichCol = 1
            currRowLen = table.getn(formationBlock[whichRow])
        end
        local currColSpot = GetColSpot(currRowLen, whichCol) # Translate whichCol to correct spot in row
        local currSlot = formationBlock[whichRow][currColSpot]
        for numType, type in currSlot do
            if inserted then
                break
            end
            for numGroup, group in type do
                if not formationBlock.HomogenousRows or (rowType == false or rowType == type) then
                    if unitsList[group] > 0 then
                        #local xPos = (math.ceil(whichCol/2)/2) - 0.25
                        local xPos
                        if math.mod( currRowLen, 2 ) == 0 then
                            xPos = math.ceil(whichCol/2) - .5
                            if not (math.mod(whichCol, 2) == 0) then
                                xPos = xPos * -1
                            end
                        else
                            if whichCol == 1 then
                                xPos = 0
                            else
                                xPos = math.ceil( ( (whichCol-1) /2 ) )
                                if not (math.mod(whichCol, 2) == 0) then
                                    xPos = xPos * -1
                                end
                            end
                        end
                        if formationBlock.HomogenousRows and not rowType then
                            rowType = type
                        end
                        table.insert(FormationPos, {xPos*spacing, -formationLength*spacing, categoryTable[group], formationLength, true})
                        inserted = true
                        unitsList[group] = unitsList[group] - 1
                        break
                    end
                end
            end
        end
        if inserted then
            i = i + 1
            inserted = false
        end
        whichCol = whichCol + 1
    end
    return FormationPos
end

function GetColSpot(rowLen, col)
    local len = rowLen
    if math.mod(rowLen,2) == 1 then
        len = rowLen + 1
    end
    local colType = 'left'
    if math.mod(col, 2) == 0 then
        colType = 'right'
    end
    local colSpot = math.floor(col / 2)
    local halfSpot = len/2
    if colType == 'left' then
        return halfSpot - colSpot
    else
        return halfSpot + colSpot
    end
end





#============ AIR BLOCK BUILDING =============#
function BlockBuilderAir(unitsList, airBlock)
    local numRows = table.getn(airBlock)
    local i = 1
    local whichRow = 1
    local whichCol = 1
    local chevronPos = 1
    local currRowLen = table.getn(airBlock[whichRow])
    local longestRow = 1
    local longestLength = 0
    local chevronSize = airBlock.ChevronSize or 5
    while i < numRows do
        if table.getn(airBlock[i]) > longestLength then
            longestLength = table.getn(airBlock[i])
            longestRow = i
        end
        i = i + 1
    end
    local chevronType = false
    local formationLength = 0
    i = 1
    while unitsList.UnitTotal >= i do
        if chevronPos > chevronSize then
            chevronPos = 1
            chevronType = false
            if whichCol == currRowLen then
                if whichRow == numRows then
                    if airBlock.RepeatAllRows then
                        whichRow = 1
                        currRowLen = table.getn(airBlock[whichRow])
                    end
                else
                    whichRow = whichRow + 1
                    currRowLen = table.getn(airBlock[whichRow])
                end
                formationLength = formationLength + 1
                whichCol = 1
            else
                whichCol = whichCol + 1
            end
        end
        local currSlot = airBlock[whichRow][whichCol]
        local inserted = false
        for numType, type in currSlot do
            if inserted then
                break
            end
            for numGroup, group in type do
                if not airBlock.HomogenousBlocks or chevronType == false or chevronType == type then
                    if unitsList[group] > 0 then
                        local xPos, yPos = GetChevronPosition(chevronPos, whichCol, currRowLen, formationLength)
                        if airBlock.HomogenousBlocks and not chevronType then
                            chevronType = type
                        end
                        table.insert(FormationPos, {xPos, yPos, AirCategories[group], yPos, true})
                        unitsList[group] = unitsList[group] - 1
                        inserted = true
                        break
                    end
                end
            end
        end
        if inserted then
            i = i + 1
        end
        chevronPos = chevronPos + 1
    end
    return FormationPos
end

function GetChevronPosition(chevronPos, currCol, currRowLen, formationLen)
    local offset = math.floor(chevronPos/2) * .375
    local xPos = offset
    if math.mod(chevronPos,2) == 0 then
        xPos = -1 * offset
    end
    local yPos = -offset
    yPos = yPos + ( formationLen * -1.5 )
    local firstBlockOffset = -2
    if math.mod(currRowLen,2) == 1 then
        firstBlockOffset = -1
    end
    local blockOff = math.floor(currCol/2) * 2
    if math.mod(currCol,2) == 1 then
        blockOff = -blockOff
    end
    xPos = xPos + blockOff + firstBlockOffset
    return xPos, yPos
end





#=========== NAVAL UNIT BLOCKS ============#
function NavalBlocks( unitsList, navyType )
    local Carriers = true
    local Battleships = true
    local Cruisers = true
    local Destroyers = true
    local unitNum = 1
    for i,v in navyType do
        for k,u in v[2] do
            if u == 'Carriers' and Carriers and unitsList.CarrierCount > 0 then
                for j, coord in v[1] do
                    if unitsList.CarrierCount ~= 0 then
                        table.insert(FormationPos, { coord[1]*NavalSpacing, coord[2]*NavalSpacing, categories.NAVAL * categories.AIRSTAGINGPLATFORM * ( categories.TECH3 + categories.EXPERIMENTAL ), 0, true })
                        unitsList.CarrierCount = unitsList.CarrierCount - 1
                        unitNum = unitNum + 1
                    end
                end
                Carriers = false
                break
            elseif u == 'Battleships' and Battleships and unitsList.BattleshipCount > 0 then
                for j, coord in v[1] do
                    if unitsList.BattleshipCount ~= 0 then
                        table.insert(FormationPos, { coord[1]*NavalSpacing, coord[2]*NavalSpacing, BattleshipNaval, 0, true })
                        unitsList.BattleshipCount = unitsList.BattleshipCount - 1
                        unitNum = unitNum + 1
                    end
                end
                Battleships = false
                break
            elseif u == 'Cruisers' and Cruisers and unitsList.CruiserCount > 0 then
                for j, coord in v[1] do
                    if unitsList.CruiserCount ~= 0 then
                        table.insert(FormationPos, { coord[1]*NavalSpacing, coord[2]*NavalSpacing, CruiserNaval, 0, true })
                        unitNum = unitNum + 1
                        unitsList.CruiserCount = unitsList.CruiserCount - 1
                    end
                end
                Cruisers = false
                break
            elseif u == 'Destroyers' and Destroyers and unitsList.DestroyerCount > 0 then
                for j, coord in v[1] do
                    if unitsList.DestroyerCount > 0 then
                        table.insert(FormationPos, { coord[1]*NavalSpacing, coord[2]*NavalSpacing, DestroyerNaval, 0, true })
                        unitNum = unitNum + 1
                        unitsList.DestroyerCount = unitsList.DestroyerCount - 1
                    end
                end
                Destroyers = false
                break
            elseif u == 'Frigates' and unitsList.FrigateCount > 0 then
                for j, coord in v[1] do
                    if unitsList.FrigateCount > 0 then
                        table.insert(FormationPos, { coord[1]*NavalSpacing, coord[2]*NavalSpacing, FrigateNaval, 0, true })
                        unitNum = unitNum + 1
                        unitsList.FrigateCount = unitsList.FrigateCount - 1
                    end
                end
                break
            elseif u == 'Frigates' and unitsList.LightCount > 0 then
                for j,coord in v[1] do
                    if unitsList.LightCount > 0 then
                        table.insert(FormationPos, { coord[1]*NavalSpacing, coord[2]*NavalSpacing, LightAttackNaval, 0, true })
                        unitNum = unitNum + 1
                        unitsList.LightCount = unitsList.LightCount - 1
                    end
                end
                break
            elseif u == 'Submarines' and unitsList.SubCount > 0 then
                for j,coord in v[1] do
                    if ( unitsList.SubCount + unitsList.NukeSubCount ) > 0 then
                        table.insert(FormationPos, { coord[1]*NavalSpacing, coord[2]*NavalSpacing, SubNaval + NukeSubNaval, 0, true })
                        unitNum = unitNum + 1
                        unitsList.SubCount = unitsList.SubCount - 1
                    end
                end
                break
            end
        end
    end

    local sideTable = { 0, -2, 2 }
    local sideIndex = 1
    local length = -3
    
    i = unitNum
    
    # Figure out how many left we have to assign
    local numLeft = unitsList.UnitTotal - i + 1
    if numLeft == 2 then
        sideIndex = 2
    end
        
    while i <= unitsList.UnitTotal do
        if unitsList.CarrierCount > 0 then
            table.insert(FormationPos, { sideTable[sideIndex]*NavalSpacing, length*NavalSpacing, categories.NAVAL * categories.AIRSTAGINGPLATFORM * ( categories.TECH3 + categories.EXPERIMENTAL ), 0, true  })
            unitNum = unitNum + 1
            unitsList.CarrierCount = unitsList.CarrierCount - 1
        elseif unitsList.BattleshipCount > 0 then
            table.insert(FormationPos, { sideTable[sideIndex]*NavalSpacing, length*NavalSpacing, BattleshipNaval, 0, true })
            unitNum = unitNum + 1
            unitsList.BattleshipCount = unitsList.BattleshipCount - 1
        elseif unitsList.CruiserCount > 0 then
            table.insert(FormationPos, { sideTable[sideIndex]*NavalSpacing, length*NavalSpacing, CruiserNaval, 0, true })
            unitNum = unitNum + 1
            unitsList.CruiserCount = unitsList.CruiserCount - 1
        elseif unitsList.DestroyerCount > 0 then
            table.insert(FormationPos, { sideTable[sideIndex]*NavalSpacing, length*NavalSpacing, DestroyerNaval, 0, true })
            unitNum = unitNum + 1
            unitsList.DestroyerCount = unitsList.DestroyerCount - 1
        elseif unitsList.FrigateCount > 0 then
            table.insert(FormationPos, { sideTable[sideIndex]*NavalSpacing, length*NavalSpacing, FrigateNaval, 0, true })
            unitNum = unitNum + 1
            unitsList.FrigateCount = unitsList.FrigateCount - 1
        elseif unitsList.LightCount > 0 then
            table.insert(FormationPos, { sideTable[sideIndex]*NavalSpacing, length*NavalSpacing, LightAttackNaval, 0, true })
            unitNum = unitNum + 1
            unitsList.LightCount = unitsList.LightCount - 1
        elseif ( unitsList.SubCount + unitsList.NukeSubCount ) > 0 then
            table.insert(FormationPos, { sideTable[sideIndex]*NavalSpacing, length*NavalSpacing, SubNaval + NukeSubNaval, 0, true })
            unitNum = unitNum + 1
            unitsList.SubCount = unitsList.SubCount - 1
        end

        # Figure out the next viable location for the naval unit
        numLeft = numLeft - 1
        sideIndex = sideIndex + 1
        if sideIndex == 4 then
            length = length - 2
            if numLeft == 2 then
                sideIndex = 2
            else
                sideIndex = 1
            end
        end

        i = i + 1
    end
    return FormationPos
end



#========= UNIT SORTING ==========#
function CategorizeAirUnits( formationUnits )
    local unitsList = {
        #### Air Lists
        Ground1 = 0, Ground2 = 0, Ground3 = 0,
        Trans1 = 0, Trans2 = 0, Trans3 = 0,
        Bomb1 = 0, Bomb2 = 0, Bomb3 = 0,
        AA1 = 0, AA2 = 0, AA3 = 0,
        AN1 = 0, AN2 = 0, AN3 = 0,
        AIntel1 = 0, AIntel2 = 0, AIntel3 = 0,
        AExper = 0,
        RemainingAirUnits = 0,
        UnitTotal = 0,
    }
    for i,u in formationUnits do
        #== AIR LISTS
        if EntityCategoryContains( AirCategories.Ground1, u ) then
            unitsList.Ground1 = unitsList.Ground1 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( AirCategories.Ground2, u ) then
            unitsList.Ground2 = unitsList.Ground2 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( AirCategories.Ground3, u ) then
            unitsList.Ground3 = unitsList.Ground3 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( AirCategories.Trans1, u ) then
            unitsList.Trans1 = unitsList.Trans1 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( AirCategories.Trans2, u ) then
            unitsList.Trans2 = unitsList.Trans2 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( AirCategories.Trans3, u ) then
            unitsList.Trans3 = unitsList.Trans3 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( AirCategories.Bomb1, u ) then
            unitsList.Bomb1 = unitsList.Bomb1 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( AirCategories.Bomb2, u ) then
            unitsList.Bomb2 = unitsList.Bomb2 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( AirCategories.Bomb3, u ) then
            unitsList.Bomb3 = unitsList.Bomb3 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( AirCategories.AA1, u ) then
            unitsList.AA1 = unitsList.AA1 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( AirCategories.AA2, u ) then
            unitsList.AA2 = unitsList.AA2 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( AirCategories.AA3, u ) then
            unitsList.AA3 = unitsList.AA3 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( AirCategories.AN1, u ) then
            unitsList.AN1 = unitsList.AN1 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( AirCategories.AN2, u ) then
            unitsList.AN2 = unitsList.AN2 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( AirCategories.AN3, u ) then
            unitsList.AN3 = unitsList.AN3 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( AirCategories.AIntel1, u ) then
            unitsList.AIntel1 = unitsList.AIntel1 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( AirCategories.AIntel2, u ) then
            unitsList.AIntel2 = unitsList.AIntel2 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( AirCategories.AIntel3, u ) then
            unitsList.AIntel3 = unitsList.AIntel3 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( AirCategories.AExper, u ) then
            unitsList.AExper = unitsList.AExper + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1            
            #== Extra units ==#
        elseif EntityCategoryContains( categories.AIR, u ) then
            unitsList.RemainingAirUnits = unitsList.RemainingAirUnits + 1
            LOG('*FORMATION DEBUG: Missed unit: ' .. u:GetUnitId())
        end
    end
    #LOG('UnitsList=', repr(unitsList))
    return unitsList
end

function CategorizeSeaUnits( formationUnits)
    local unitsList = {
        #### Naval Lists
        UnitTotal = 0,
        CarrierCount = 0,
        BattleshipCount = 0,
        DestroyerCount = 0,
        CruiserCount = 0,
        FrigateCount = 0,
        LightCount = 0,
        SubCount = 0,
        NukeSubCount = 0,
        RemainingNavalUnits = 0,
    }
    #== NAVAL UNITS ==#
    for i,u in formationUnits do
        if EntityCategoryContains( LightAttackNaval, u ) then
            unitsList.LightCount = unitsList.LightCount + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( FrigateNaval, u ) then
            unitsList.FrigateCount = unitsList.FrigateCount + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( DestroyerNaval, u ) then
            unitsList.DestroyerCount = unitsList.DestroyerCount + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( CruiserNaval, u ) then
            unitsList.CruiserCount = unitsList.CruiserCount + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( SubNaval, u ) then
            unitsList.SubCount = unitsList.SubCount + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( BattleshipNaval, u ) then
            unitsList.BattleshipCount = unitsList.BattleshipCount + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( CarrierNaval, u ) then
            unitsList.CarrierCount = unitsList.CarrierCount + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( NukeSubNaval, u ) then
            unitsList.NukeSubCount = unitsList.NukeSubCount + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( UnderwaterCarrierNaval, u ) then
            unitsList.CarrierCount = unitsList.CarrierCount + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( categories.NAVAL, u) then
            unitsList.RemainingNavalUnits = unitsList.RemainingNavalUnits + 1
            LOG('*FORMATION DEBUG: Missed unit: ' .. u:GetUnitId())
        end
    end
    return unitsList
end

function CategorizeLandUnits( formationUnits )
    local unitsList = {
        #### Land Numbers
        Tank1 = 0, Tank2 = 0, Tank3 = 0,
        Art1 = 0, Art2 = 0, Art3 = 0,
        AA1 = 0, AA2 = 0, AA3 = 0,
        Com1 = 0, Com2 = 0, Com3 = 0,
        Util1 = 0, Util2 = 0, Util3 = 0,
        Shields = 0,
        Experimentals = 0,
        UnitTotal = 0,
        RemainingLandUnits = 0,
    }
    for i,u in formationUnits do
        if EntityCategoryContains(LandCategories['Tank1'], u) then
            unitsList.Tank1 = unitsList.Tank1 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains(LandCategories['Tank2'], u) then
            unitsList.Tank2 = unitsList.Tank2 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains(LandCategories['Tank3'], u) then
            unitsList.Tank3 = unitsList.Tank3 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains(LandCategories['AA1'], u) then
            unitsList.AA1 = unitsList.AA1 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains(LandCategories['AA2'], u) then
            unitsList.AA2 = unitsList.AA2 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( LandCategories['AA3'], u) then
            unitsList.AA3 = unitsList.AA3 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( LandCategories['Art1'], u) then
            unitsList.Art1 = unitsList.Art1 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( LandCategories['Art2'], u) then
            unitsList.Art2 = unitsList.Art2 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( LandCategories['Art3'], u) then
            unitsList.Art3 = unitsList.Art3 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( LandCategories['Com1'], u) then
            unitsList.Com1 = unitsList.Com1 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( LandCategories['Com2'], u) then
            unitsList.Com2 = unitsList.Com2 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( LandCategories['Com3'], u) then
            unitsList.Com3 = unitsList.Com3 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( LandCategories['Util1'], u) then
            unitsList.Util1 = unitsList.Util1 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( LandCategories['Util2'], u) then
            unitsList.Util2 = unitsList.Util2 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( LandCategories['Util3'], u) then
            unitsList.Util3 = unitsList.Util3 + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( LandCategories['Shields'], u) then
            unitsList.Shields = unitsList.Shields + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( LandCategories['Experimentals'], u) then
            unitsList.Experimentals = unitsList.Experimentals + 1
            unitsList.UnitTotal = unitsList.UnitTotal + 1
        elseif EntityCategoryContains( categories.LAND, u) then
            unitsList.RemainingLandUnits = unitsList.RemainingLandUnits + 1
            LOG('*FORMATION DEBUG: Missed unit: ' .. u:GetUnitId())
        end
    end
    return unitsList
end
