-- Class methods:
-- SetTexture(filename(s), border=1)
-- SetSolidColor(color)
-- SetUV(float u0, float v0, float u1, float v1)
-- SetTiled(bool)
-- UseAlphaHitTest(bool)
-- Loop(bool)
-- Play()
-- Stop()
-- SetFrame(int)
-- int GetFrame()
-- int GetNumFrames()
-- SetFrameRate(float fps)
-- SetFramePattern(pattern)
    -- pattern is an array of integers reflecting texture indicies
-- SetForwardPattern()
-- SetBackwardPattern()
-- SetPingPongPattern()
-- SetLoopPingPongPattern()
-- ShareTextures(bitmap) -- allows two bitmaps to share the same textures

-- Frame patterns are arrays that indicate what texture to play at a particular frame index
-- Textures are indexed by the order you pass them in to SetTexture. Note that frames are
-- 0 based, not 1 based.

-- related global function (returns nil if file not found)
-- width, height GetTextureDimensions(filename)


local Control = import('control.lua').Control

Bitmap = Class(moho.bitmap_methods, Control) {

    __init = function(self, parent, filename, debugname)
        InternalCreateBitmap(self, parent)
        if debugname then
            self:SetName(debugname)
        end

        local LazyVar = import('/lua/modules/lazyvar.lua')
        self._filename = {_texture = LazyVar.Create(), _border = 1}
        self._filename._texture.OnDirty = function(var)
            self:SetNewTexture(self._filename._texture(), self._filename._border)
        end

        if filename then
            self:SetTexture(filename)
        end
    end,

    SetTexture = function(self, texture, border)
        if self._filename then
            border = border or 1
            self._filename._border = border
            self._filename._texture:Set(texture)
        end
    end,
    
    ResetLayout = function(self)
        Control.ResetLayout(self)
        self.Width:SetFunction(function() return self.BitmapWidth() end)
        self.Height:SetFunction(function() return self.BitmapHeight() end)
    end,

    OnInit = function(self)
        Control.OnInit(self)
    end,
    
    OnDestroy = function(self)
        self._filename._texture:Destroy()
        self._filename = nil
    end,
    
    -- callback scripts
    OnAnimationFinished = function(self) end,
    OnAnimationStopped = function(self) end,
    OnAnimationFrame = function(self, frameNumber) end,
}

