-- Class methods:
-- SetNewFont(family, pointsize)
-- SetNewColors(foreground, background, selected_foreground, selected_background)
-- item GetItem(index)
-- AddItem('newitem')
-- ModifyItem(index, string)
-- DeleteItem(index)
-- DeleteAllItems()
-- index GetSelection()
-- SetSelection(index)
-- int GetItemCount()
-- bool Empty()
-- ScrollToTop()
-- ScrollToBottom()
-- ShowItem(index)
-- int GetRowHeight()

local Control = import('control.lua').Control
local Dragger = import('dragger.lua').Dragger


ItemList = Class(moho.item_list_methods, Control) {

    __init = function(self, parent, debugname)
        InternalCreateItemList(self, parent)
        if debugname then
            self:SetName(debugname)
        end

        local LazyVar = import('/lua/modules/lazyvar.lua')
        self._font = {_family = LazyVar.Create(), _pointsize = 12}
        self._font._family.OnDirty = function(var)
            self:SetNewFont(self._font._family(), self._font._pointsize)
        end

        self._fg = LazyVar.Create()
        self._fg.OnDirty = function(var)
            self:SetNewColors(var(), nil, nil, nil)
        end

        self._bg = LazyVar.Create()
        self._bg.OnDirty = function(var)
            self:SetNewColors(nil, var(), nil, nil)
        end

        self._sfg = LazyVar.Create()
        self._sfg.OnDirty = function(var)
            self:SetNewColors(nil, nil, var(), nil)
        end

        self._sbg = LazyVar.Create()
        self._sbg.OnDirty = function(var)
            self:SetNewColors(nil, nil, nil, var())
        end
    end,

    -- lazy var support
    SetFont = function(self, family, pointsize)
        if self._font then
            self._font._pointsize = pointsize
            self._font._family:Set(family)
        end
    end,

    SetColors = function(self, foreground, background, selected_foreground, selected_background)
        if foreground and self._fg then self._fg:Set(foreground) end
        if background and self._bg then self._bg:Set(background) end
        if selected_foreground and self._sfg then self._sfg:Set(selected_foreground) end
        if selected_background and self._sbg then self._sbg:Set(selected_background) end
    end,

    OnDestroy = function(self)
        self._font._family:Destroy()
        self._font = nil
        self._fg:Destroy()
        self._fg = nil
        self._bg:Destroy()
        self._bg = nil
        self._sfg:Destroy()
        self._sfg = nil
        self._sbg:Destroy()
        self._sbg = nil
    end,

    -- default override methods
    OnClick = function(self, row)
        self:SetSelection(row)
    end,

    OnDoubleClick = function(self, row)
        self:OnClick(row)
    end,

}

