--*****************************************************************************
--* File: lua/modules/multiplayer/onlineprovider.lua
--* Author: Sam Demulling
--* Summary: 3rd party integration options.  This will receive and send commands
--* by hooking into stdin/out in the supremecommander app.
--*
--* Copyright  2006 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

--local Lobby = import('/lua/modules/ui/lobby/lobby.lua')
--local Chat = import('/lua/modules/ui/game/chat.lua')

local HasStdInOut = false
local ProviderName = "GPGNET"
local chatDisplay = false
local autolaunched = false

function SetOnline()
    HasStdInOut = true
end

function CheckOnline()
    return HasStdInOut
end

function RegisterChatDisplay(display)
    chatDisplay = display
end

function UnregisterChat()
    chatDisplay = false
end

function ProcessStdInCommand(command)
	--LOG("STDIN CMD: "..command)
    local startindex = 0
    local endindex = 0
    startindex, endindex = string.find(command, " ")
    if startindex then
        local commandtype = string.sub(command, 1, startindex)
        local commanddata = string.sub(command, startindex + 1, string.len(command))
        if CheckCommand(commandtype, "/GAMESTATE") then
            SetOnline()
        end
        if CheckCommand(commandtype, "/CHAT") then
			if CheckCommand(commanddata, "VERSION") then
				SendVersionMessage()
            elseif not CheckCommand(commanddata, "//") then
                SendSupcomChatMessage(commanddata)
            end
        end
    end
end
 
function SendVersionMessage()
	local version = GetVersion()
	if CheckOnline() then
		StdOutMessage("/GAMEVERSION "..version)
	end
end

function SendMessage(message)
    if CheckOnline() then
        StdOutMessage("/CHAT "..message)
    end
end

function ChangeMap(message)
    if CheckOnline() then
        StdOutMessage("/MAP "..message)
    end
end

function ChangeState(message)
    if CheckOnline() then
        StdOutMessage("/GAMESTATE "..message)
    end
end

function ChangeOption(message)
	if CheckOnline() then
		StdOutMessage("/GAMEOPTION "..message)
	end
end

function Launching()
    ChangeState("Launching")
end

function CheckCommand(checkcmd, actualcmd)
    local beginindex = 0
    local throwawayindex = 0
    beginindex, throwawayindex = string.find(checkcmd, actualcmd)
    if beginindex then
        return true
    else
        return false
    end
end

function SendSupcomChatMessage(message)
    if CheckOnline() then
        if chatDisplay then
            chatDisplay:PushTextBottom(message)
        else
			Chat.UpdateChat(message)
        end
    end
end