local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Text = import('/lua/modules/maui/text.lua').Text
local ItemList = import('/lua/modules/maui/itemlist.lua').ItemList
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Edit = import('/lua/modules/maui/edit.lua').Edit

FilePicker = Class(Group) {
    __init = function(self, parent, fileType, selectAction, debugName)
        Group.__init(self, parent)
        self:SetName(debugName or "FilePicker")            
        
        self._fileType = fileType
        
--[[---------------------------------------------------------------------------
        LAYOUT
--]]----------------------------------------------------------------------------

        self._filenameEdit = Edit(self)
        self._filenameEdit:SetForegroundColor(UIUtil.fontColor)
        self._filenameEdit:SetBackgroundColor(UIUtil.panelColor)
        self._filenameEdit:ShowBackground(false)
        self._filenameEdit.Width:Set(self.Width)
        self._filenameEdit.Height:Set(function() return self._filenameEdit:GetFontHeight() end)
        LayoutHelpers.AtBottomIn(self._filenameEdit, self)
        LayoutHelpers.AtLeftIn(self._filenameEdit, self)
        self._filenameEdit:AcquireFocus()

        self._filenameLabel = UIUtil.CreateText(self, "<LOC uifilepicker_0000>Filename", 16, UIUtil.titleFont)
        LayoutHelpers.Above(self._filenameLabel, self._filenameEdit, 5)
        
        local fileListLabel = UIUtil.CreateText(self, "<LOC uifilepicker_0003>Files", 16, UIUtil.titleFont)
        LayoutHelpers.AtLeftTopIn(fileListLabel, self)

        self._fileList = ItemList(self)
        self._fileList:SetFont(UIUtil.bodyFont, 16)
        self._fileList:SetColors(UIUtil.fontColor, "00000000", UIUtil.fontColor,  UIUtil.highlightColor)
        LayoutHelpers.Below(self._fileList, fileListLabel)
        self._fileList.Bottom:Set(self._filenameLabel.Top)
        self._fileList.Width:Set(function() return self.Width() - 32 end)
        
        UIUtil.CreateVertScrollbarFor(self._fileList)
        
        ForkThread(function() self._filenameEdit:SetFont(UIUtil.bodyFont, 14) end)

--[[---------------------------------------------------------------------------
        LOGIC
--]]----------------------------------------------------------------------------
        self:RepopulateList()

        self._fileList.OnClick = function(control, row)
            self._fileList:SetSelection(row)
            self._filenameEdit:SetText(self._fileList:GetItem(row))
        end
        
        self._fileList.OnDoubleClick = function(control, row)
            self._fileList.OnClick(control, row)
            if selectAction then
                selectAction(self:GetFilename())
            end                    
        end
        
        self._filenameEdit.OnEnterPressed = function(control, text)
            if selectAction then
                selectAction(self:GetFilename())
            end
        end
        
    end,

    RepopulateList = function(self)
        self._fileList:DeleteAllItems()
        
        local filesData = GetSpecialFiles(self._fileType)
        
        self._currentDir = filesData.directory
        self._currentExt = filesData.extension
        
        for index, file in filesData.files do
            self._fileList:AddItem(file)
        end
    end,
    
    SetFilename = function(self, newText)
        self._filenameEdit:SetText(newText)
    end,
    
    GetFilename = function(self)
        return self._currentDir .. self._filenameEdit:GetText() .. "." .. self._currentExt
    end,
}