--*****************************************************************************
--* File: lua/modules/ui/dialogs/exitdialog.lua
--* Author: Chris Blackwell
--* Summary: Contains the context sensitive exit dialog
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Text = import('/lua/modules/maui/text.lua').Text
local Button = import('/lua/modules/maui/button.lua').Button
local Tooltip = import('/lua/modules/ui/game/tooltip.lua')
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local OnlineProvider = import('/lua/modules/multiplayer/onlineprovider.lua')
local Options = import('/lua/modules/ui/game/options.lua')
local GameMain = import('/lua/modules/ui/game/gamemain.lua')

local parentGroup = false
local background = false
local backgroundGlow = false
local backgroundGlowbars = false
local backgroundBrackets = false
local resumeBtn = false
local quitBtn = false
local restartBtn = false
local optionsBtn = false
local exitBtn = false
local saveBtn = false
local quickDialog = false

function CreateDialog()
    local function CreateYesNoDialog()
        if quickDialog then
            return
        end
        quickDialog = UIUtil.QuickDialog(GetFrame(0), "<LOC EXITDLG_0000>Are you sure you'd like to quit?", 
            "<LOC _Yes>", function() ExitApplication() end, "<LOC _No>", nil, nil, nil, true)
    end

    local function CreateInGameDialog()
        local sound = Sound({Cue = "UI_Main_Window_Open", Bank = "Interface",})
        PlaySound(sound)
        local mapGroup = import('/lua/modules/ui/game/borders.lua').GetMapGroup()
        if not parentGroup then
            parentGroup = Group(GetFrame(0), "exitDialogParentGroup")
        end
        if not background then
            background = Bitmap(parentGroup, UIUtil.UIFile("/dialogs/exit-dialog/panel03_bmp.dds"))
        end
        parentGroup.Width:Set(background.Width)
        parentGroup.Height:Set(background.Height)
        LayoutHelpers.AtCenterIn(parentGroup, mapGroup, 20)
        parentGroup.Depth:Set(GetFrame(0):GetTopmostDepth() + 1)
        LayoutHelpers.FillParent(background, parentGroup)
        
        if not backgroundGlow then
            backgroundGlow = Bitmap(parentGroup, UIUtil.UIFile("/dialogs/exit-dialog/panel-white_bmp.dds"))
        end
        backgroundGlow.Depth:Set(function() return background.Depth() + 1 end)
        LayoutHelpers.AtCenterIn(backgroundGlow, parentGroup)
        backgroundGlow:SetAlpha(0)
        
        if not backgroundGlowbars then
            backgroundGlowbars = Bitmap(parentGroup, UIUtil.UIFile("/dialogs/exit-dialog/glow-bar_bmp.dds"))
        end
        backgroundGlowbars.Depth:Set(function() return background.Depth() + 1 end)
        LayoutHelpers.AtCenterIn(backgroundGlowbars, parentGroup)
        
        if not backgroundBrackets then
            backgroundBrackets = Bitmap(parentGroup, UIUtil.UIFile("/dialogs/exit-dialog/brackets_bmp.dds"))
        end
        backgroundBrackets.Depth:Set(function() return background.Depth() + 1 end)
        LayoutHelpers.AtCenterIn(backgroundBrackets, parentGroup)
                
        background:DisableHitTest()
        backgroundGlow:DisableHitTest()
        backgroundGlowbars:DisableHitTest()
        backgroundBrackets:DisableHitTest()
        parentGroup:DisableHitTest()
        
        if not resumeBtn then
            resumeBtn = UIUtil.CreateButtonStd(background, "/medium-uef-btn/medium02", "<LOC _Resume>", 20, 2, 0, "UI_Main_Window_Close")
        end
        LayoutHelpers.AtHorizontalCenterIn(resumeBtn, parentGroup)
        LayoutHelpers.AtTopIn(resumeBtn, parentGroup, 33)

        if not restartBtn then
            if UIUtil.GetNetworkBool() and not GameMain.GetReplayState() then
                restartBtn = UIUtil.CreateButtonStd(background, "/medium-uef-btn/medium02", "<LOC _Connectivity>", 20, 2)
            else
                restartBtn = UIUtil.CreateButtonStd(background, "/medium-uef-btn/medium02", "<LOC _Restart>", 20, 2)
            end
        end
        restartBtn.Top:Set(function() return resumeBtn.Bottom() - 10 end)
        LayoutHelpers.AtHorizontalCenterIn(restartBtn, parentGroup)
        if UIUtil.BETA_FLAG then
            restartBtn:Disable()
        end
        
        if not optionsBtn then
            optionsBtn = UIUtil.CreateButtonStd(background, "/medium-uef-btn/medium02", "<LOC _Options>", 20, 2)
        end
        optionsBtn.Top:Set(function() return restartBtn.Bottom() - 10 end)
        LayoutHelpers.AtHorizontalCenterIn(optionsBtn, parentGroup)

        if not saveBtn then
            saveBtn = UIUtil.CreateButtonStd(background, "/medium-uef-btn/medium02", "<LOC _Save>", 20, 2)
        end
        saveBtn.Top:Set(function() return optionsBtn.Bottom() - 10 end)
        LayoutHelpers.AtHorizontalCenterIn(saveBtn, parentGroup)
        if UIUtil.BETA_FLAG then
            saveBtn:Disable()
        end
        
        if not quitBtn then
            quitBtn = UIUtil.CreateButtonStd(background, "/medium-uef-btn/medium02", "<LOC _End_Game>End Game", 20, 2)
        end
        quitBtn.Top:Set(function() return saveBtn.Bottom() - 10 end)
        LayoutHelpers.AtHorizontalCenterIn(quitBtn, parentGroup)
        
        if not exitBtn then
            exitBtn = UIUtil.CreateButtonStd(background, "/medium-uef-btn/medium02", "<LOC _Exit_to_Windows>Exit to Windows", 20, 2)
        end
        exitBtn.Top:Set(function() return quitBtn.Bottom() - 10 end)
        LayoutHelpers.AtHorizontalCenterIn(exitBtn, parentGroup)

        
        local function DoClickBehavior(clickFunc)
            if UIUtil.GetAnimationPrefs() then
                CloseWindow(clickFunc)
            else
                DestroyParents()
                if clickFunc then clickFunc() end
            end
        end

        saveBtn.OnClick = function(self, modifiers)
            DoClickBehavior(function()
                import('/lua/modules/ui/dialogs/saveload.lua').CreateSaveDialog(mapGroup)
            end)            
        end
        saveBtn.HandleEvent = function(self, event)
            if event.Type == 'MouseEnter' then
                Tooltip.CreateMouseoverDisplay(self, "esc_save", nil, true)
            elseif event.Type == 'MouseExit' then
                Tooltip.DestroyMouseoverDisplay()
            end
            Button.HandleEvent(self, event)
        end

        resumeBtn.OnClick = function(self)
            DoClickBehavior()
            if not UIUtil.GetNetworkBool() then
                Options.PauseGame(false)
            end
        end
        resumeBtn.HandleEvent = function(self, event)
            if event.Type == 'MouseEnter' then
                Tooltip.CreateMouseoverDisplay(self, "esc_resume", nil, true)
            elseif event.Type == 'MouseExit' then
                Tooltip.DestroyMouseoverDisplay()
            end
            Button.HandleEvent(self, event)
        end

        quitBtn.OnClick = function(self)
            DoClickBehavior(function()
                UIUtil.QuickDialog(mapGroup, "<LOC EXITDLG_0001>Are you sure you'd like to quit?", 
                    "<LOC _Yes>", function() import('/lua/modules/ui/dialogs/score.lua').CreateDialog() end, "<LOC _No>", nil, nil, nil, true)
            end)
        end
        quitBtn.HandleEvent = function(self, event)
            if event.Type == 'MouseEnter' then
                Tooltip.CreateMouseoverDisplay(self, "esc_quit", nil, true)
            elseif event.Type == 'MouseExit' then
                Tooltip.DestroyMouseoverDisplay()
            end
            Button.HandleEvent(self, event)
        end

        restartBtn.OnClick = function(self)
            if UIUtil.GetNetworkBool() and not GameMain.GetReplayState() then
                DoClickBehavior(function()
                    import('/lua/modules/ui/dialogs/disconnect.lua').CreateDialog()
                end)
            else
                DoClickBehavior(function()
                    UIUtil.QuickDialog(mapGroup, "<LOC EXITDLG_0002>Are you sure you'd like to restart?", 
                        "<LOC _Yes>", function() RestartSession() end, "<LOC _No>")
                end)
            end
        end
        restartBtn.HandleEvent = function(self, event)
            if event.Type == 'MouseEnter' then
                if UIUtil.GetNetworkBool() and not GameMain.GetReplayState() then
                    Tooltip.CreateMouseoverDisplay(self, "esc_conn", nil, true)
                else
                    Tooltip.CreateMouseoverDisplay(self, "esc_restart", nil, true)
                end
            elseif event.Type == 'MouseExit' then
                Tooltip.DestroyMouseoverDisplay()
            end
            Button.HandleEvent(self, event)
        end

        exitBtn.OnClick = function(self)
            DoClickBehavior(function()
                UIUtil.QuickDialog(mapGroup, "<LOC EXITDLG_0003>Are you sure you'd like to exit?", 
                    "<LOC _Yes>", function()
                        if not UIUtil.BETA_FLAG then
                            import('/lua/modules/ui/campaign/campaignmanager.lua').ClearAutoContinueOp()
                        end
                    ExitApplication()
                    end, "<LOC _No>")
            end)
        end
        exitBtn.HandleEvent = function(self, event)
            if event.Type == 'MouseEnter' then
                Tooltip.CreateMouseoverDisplay(self, "esc_exit", nil, true)
            elseif event.Type == 'MouseExit' then
                Tooltip.DestroyMouseoverDisplay()
            end
            Button.HandleEvent(self, event)
        end

        optionsBtn.OnClick = function(self)
            DoClickBehavior(function()
                import('/lua/modules/ui/dialogs/options.lua').CreateDialog(mapGroup)
            end)
        end
        optionsBtn.HandleEvent = function(self, event)
            if event.Type == 'MouseEnter' then
                Tooltip.CreateMouseoverDisplay(self, "esc_options", nil, true)
            elseif event.Type == 'MouseExit' then
                Tooltip.DestroyMouseoverDisplay()
            end
            Button.HandleEvent(self, event)
        end
        
        if not UIUtil.GetNetworkBool() then
            Options.PauseGame(true)
        end
        
        if UIUtil.GetAnimationPrefs() then
             ExitDialogOpen()
        end
    end

    -- if dialog is already up, then bring it down
    if parentGroup then
        if parentGroup:IsHidden() then
            if UIUtil.GetAnimationPrefs() then
                parentGroup:Show()
                ExitDialogOpen()
            else
                ResetWindow()
                parentGroup:Show()
            end
            local sound = Sound({Cue = "UI_Main_Window_Open", Bank = "Interface",})
            PlaySound(sound)
            if not UIUtil.GetNetworkBool() then
                Options.PauseGame(true)
            end
        else
            local sound = Sound({Cue = "UI_Main_Window_Close", Bank = "Interface",})
            PlaySound(sound)
            if UIUtil.GetAnimationPrefs() then
                CloseWindow(false)
            else
                ResetWindow()
                DestroyParents()
            end
            if not UIUtil.GetNetworkBool() then
                Options.PauseGame(false)
            end
        end
    else
        currentUIState = GetCurrentUIState()
        if  currentUIState == "splash" then
            CreateYesNoDialog()
        elseif currentUIState == "frontend" then
            CreateYesNoDialog()
        elseif currentUIState == "game" then
            CreateInGameDialog()
        end
    end
end

function CloseWindow(func)
    ExitDialogClose(func)
end

function ResetWindow()
    backgroundBrackets.Height:Set(backgroundBrackets.BitmapHeight)
    background:SetAlpha(1, true)
    backgroundGlow:SetAlpha(0)
end

function DestroyParents()
    parentGroup:Hide()
end


function ExitDialogOpen()
    background:SetNeedsFrameUpdate(true)
    background:SetAlpha(0, true)
      
    local totalTime = 0
    local braceGoal = backgroundBrackets.BitmapHeight()
    backgroundBrackets:SetAlpha(1)
    backgroundBrackets.Height:Set(1)
    local AnimationCues = {
        stage1 = {
            started = true,
            running = true,
        },
        stage2 = {
            started = false,
            running = false,
        },
        stage3 = {
            started = false,
            running = false,
        },
        stage4 = {
            started = false,
            running = false,
        },
        stage5 = {
            started = false,
            running = false,
        },
        stage6 = {
            started = false,
            running = false,
        },
        stage7 = {
            started = false,
            running = false,
        },
        stage8 = {
            started = false,
            running = false,
        },
        stage9 = {
            started = false,
            running = false,
        },
    }
    
    local function FadeIn(control, elapsedTime, speed, cue)
        if control:GetAlpha() < 1 then
            local curAlpha = math.min(control:GetAlpha() + (elapsedTime * speed), 1)
            control:SetAlpha(curAlpha, true)
        else
            cue = false
        end
    end
    
    local PulseAscending = true
    local function Pulse(control, elapsedTime, speed)
        local curAlpha
        if PulseAscending then
            curAlpha = math.min(control:GetAlpha() + (elapsedTime * speed), 1)
            control:SetAlpha(curAlpha, true)
        else
            curAlpha = math.max(control:GetAlpha() - (elapsedTime * speed), 0)
        end
        if curAlpha >= 1 then
            curAlpha = 1
            PulseAscending = false
        end
        if curAlpha <= 0 then
            curAlpha = 0
            PulseAscending = true
        end
        control:SetAlpha(curAlpha, true)
    end
            
    background.OnFrame = function(self, deltaTime)
        totalTime = totalTime + deltaTime
        if totalTime > .3 and not AnimationCues.stage2.started then
            AnimationCues.stage2.started = true
            AnimationCues.stage2.running = true
            background:SetAlpha(1)
            backgroundGlow:SetAlpha(1)
        end
        if totalTime > .6 and not AnimationCues.stage3.started then
            backgroundBrackets.Height:Set(braceGoal)
            AnimationCues.stage3.started = true
            AnimationCues.stage3.running = true
        end
        if totalTime > .65 and not AnimationCues.stage4.started then
            AnimationCues.stage4.started = true
            AnimationCues.stage4.running = true
        end
        if totalTime > .7 and not AnimationCues.stage5.started then
            AnimationCues.stage5.started = true
            AnimationCues.stage5.running = true
        end
        if totalTime > .75 and not AnimationCues.stage6.started then
            AnimationCues.stage6.started = true
            AnimationCues.stage6.running = true
        end
        if totalTime > .8 and not AnimationCues.stage7.started then
            AnimationCues.stage7.started = true
            AnimationCues.stage7.running = true
        end
        if totalTime > .85 and not AnimationCues.stage8.started then
            AnimationCues.stage8.started = true
            AnimationCues.stage8.running = true
        end
        if totalTime > .9 and not AnimationCues.stage9.started then
            AnimationCues.stage9.started = true
            AnimationCues.stage9.running = true
        end
        if AnimationCues.stage1.running then
            if  backgroundBrackets.Height() > braceGoal then
                 backgroundBrackets.Height:Set(braceGoal)
                 AnimationCues.stage1.running = false
            else
                local curHeight = backgroundBrackets.Height()
                if curHeight + (deltaTime * 1500) > braceGoal + 50 then
                    backgroundBrackets.Height:Set(braceGoal)
                    AnimationCues.stage1.running = false
                else
                    backgroundBrackets.Height:Set(function() return curHeight + (deltaTime * 1500) end)
                end
            end
        end
        if AnimationCues.stage2.running then
            if backgroundGlow:GetAlpha() >= 0 then
                local curAlpha = math.max(backgroundGlow:GetAlpha() - (deltaTime * 4), 0)
                backgroundGlow:SetAlpha(curAlpha)
            else
                AnimationCues.stage2.running = false
            end
        end
        if AnimationCues.stage3.running then
            FadeIn(resumeBtn, deltaTime, 2, AnimationCues.stage3.running)
        end
        if AnimationCues.stage4.running then
            FadeIn(restartBtn, deltaTime, 2, AnimationCues.stage4.running)
        end
        if AnimationCues.stage5.running then
            FadeIn(optionsBtn, deltaTime, 2, AnimationCues.stage5.running)
        end
        if AnimationCues.stage6.running then
            FadeIn(saveBtn, deltaTime, 2, AnimationCues.stage6.running)
        end
        if AnimationCues.stage7.running then
            FadeIn(quitBtn, deltaTime, 2, AnimationCues.stage7.running)
        end
        if AnimationCues.stage8.running then
            FadeIn(exitBtn, deltaTime, 2, AnimationCues.stage8.running)
        end
        if AnimationCues.stage9.running then
            Pulse(backgroundGlowbars, deltaTime, 2)
        end
    end
end

function ExitDialogClose(func)
    background:SetNeedsFrameUpdate(true)
    local braceGoal = 1
    local totalTime = 0
    local AnimationCues = {
        stage1 = {
            started = true,
            running = true,
        },
        stage2 = {
            started = false,
            running = false,
        },
    }
    local function FadeOut(control, elapsedTime, speed, cue)
        if control:GetAlpha() > 0 then
            local curAlpha = math.max(control:GetAlpha() - (elapsedTime * speed), 0)
            control:SetAlpha(curAlpha, true)
        else
            if cue then
                cue = false
            end
        end
    end
    background.OnFrame = function(self, elapsedTime)
        totalTime = totalTime + elapsedTime
        if totalTime > .3 and not AnimationCues.stage2.started then
            AnimationCues.stage2.started = true
            AnimationCues.stage2.running = true
        end
        if totalTime > .6 then
            background:SetNeedsFrameUpdate(false)
            if func then
                func()
            end
            DestroyParents()
        end
        if AnimationCues.stage1.running then
            FadeOut(resumeBtn, elapsedTime, 3, false)
            FadeOut(quitBtn, elapsedTime, 3, false)
            FadeOut(restartBtn, elapsedTime, 3, false)
            FadeOut(optionsBtn, elapsedTime, 3, false)
            FadeOut(exitBtn, elapsedTime, 3, false)
            FadeOut(backgroundGlowbars, elapsedTime, 3, false)
            FadeOut(background, elapsedTime, 3, AnimationCues.stage1.running)
        end
        if AnimationCues.stage2.running then
            if  backgroundBrackets.Height() <= braceGoal then
                 backgroundBrackets.Height:Set(braceGoal)
                 AnimationCues.stage2.running = false
            else
                local curHeight = backgroundBrackets.Height()
                if curHeight - (elapsedTime * 1500) < 0 then
                    backgroundBrackets.Height:Set(braceGoal)
                    AnimationCues.stage2.running = false
                else
                    backgroundBrackets.Height:Set(function() return curHeight - (elapsedTime * 1500) end)
                end
            end
        end
    end
end