--*****************************************************************************
--* File: lua/modules/ui/dialogs/replay.lua
--* Author: Chris Blackwell
--* Summary: Allows you to choose mods
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Edit = import('/lua/modules/maui/edit.lua').Edit
local Prefs = import('/lua/modules/ui/prefs.lua')
local MenuCommon = import('/lua/modules/ui/menus/menucommon.lua')
local FilePicker = import('/lua/modules/ui/controls/filepicker.lua').FilePicker

function CreateDialog(over, exitBehavior)
    ---------------------------------------------------------------------------
    -- basic layout and operation of dialog
    ---------------------------------------------------------------------------
    local parent = nil
    local background = nil
    if over then
        parent = over
    else
        parent = UIUtil.CreateScreenGroup(GetFrame(0), "Map Select ScreenGroup")
        background = MenuCommon.SetupBackground(GetFrame(0))
    end

    -- don't parent background to screen group so it doesn't get destroyed until we leave the menus
    local exitButton = nil
    if background then
        exitButton = MenuCommon.CreateExitMenuButton(parent, background, "<LOC _BACK>")
    end

    local panel = Bitmap(parent, UIUtil.UIFile('/dialogs/options/panel-ingame_bmp.dds'))
    LayoutHelpers.AtCenterIn(panel, parent)
    
    if over then
        panel.Depth:Set(GetFrame(over:GetRootFrame():GetTargetHead()):GetTopmostDepth() + 1)
    end

    local dlgHead = UIUtil.CreateText(panel, "<LOC uireplay_0001>Replay", 16, UIUtil.titleFont)
    LayoutHelpers.AtLeftTopIn(dlgHead, panel, 40, 25)

    local dlgLabel = UIUtil.CreateText(panel, "<LOC uireplay_0002>Enter or select a name", 16, UIUtil.titleFont)
    LayoutHelpers.AtLeftTopIn(dlgLabel, panel, 40, 55)

    
    ---------------------------------------------------------------------------
    -- OK and cancel button behaviors
    ---------------------------------------------------------------------------
    local function KillDialog()
        if over then
            panel:Destroy()
        else
            parent:Destroy()
        end
        if exitBehavior then
            exitBehavior()
        end
    end

    function DoReplayLaunch(filename)
        if LaunchReplaySession(filename) == false then
            UIUtil.ShowInfoDialog(parent, LOCF("<LOC REPLAY_0000>Unable to launch replay: %s", Basename(filename, true)), "<LOC _Ok>")    
        else
            if not over then
                parent:Destroy()
            end
        end
    end

    local filePicker = FilePicker(panel, "Replay", function(filename)
        DoReplayLaunch(filename)
    end)
    LayoutHelpers.AtLeftTopIn(filePicker, panel, 36, 91)
    filePicker.Width:Set(530)
    filePicker.Height:Set(307)

    local okBtn = UIUtil.CreateButtonStd(panel, "/dialogs/standard_btn/standard", "<LOC _Ok>", 14, 2)
    LayoutHelpers.AtLeftTopIn(okBtn, panel, 44, 417)
    okBtn.OnClick = function(self, modifiers)
        DoReplayLaunch(filePicker:GetFilename())
    end

    local cancelBtn = UIUtil.CreateButtonStd(panel, "/dialogs/standard_btn/standard", "<LOC _Cancel>", 14, 2)
    LayoutHelpers.RightOf(cancelBtn, okBtn, 10)
    cancelBtn.OnClick = function(self, modifiers)
        KillDialog()
    end

    if exitButton then
        exitButton.OnClick = function(self, modifiers)
            KillDialog()
        end
    end

end
