--*****************************************************************************
--* File: lua/modules/ui/dialogs/saveload.lua
--* Author: Chris Blackwell
--* Summary: Load and save game UI
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Prefs = import('/lua/modules/ui/prefs.lua')
local MenuCommon = import('/lua/modules/ui/menus/menucommon.lua')
local FilePicker = import('/lua/modules/ui/controls/filepicker.lua').FilePicker

local function CreateDialog(over, isLoad, callback, exitBehavior)
    ---------------------------------------------------------------------------
    -- basic layout and operation of dialog
    ---------------------------------------------------------------------------
    local parent = nil
    local background = nil
    if over then
        parent = over
    else
        parent = UIUtil.CreateScreenGroup(GetFrame(0), "Load Game ScreenGroup")
        background = MenuCommon.SetupBackground(GetFrame(0))
    end

    -- don't parent background to screen group so it doesn't get destroyed until we leave the menus
    local exitButton = nil
    if background then
        exitButton = MenuCommon.CreateExitMenuButton(parent, background, "<LOC _Back>")
    end

    local panel = Bitmap(parent, UIUtil.UIFile('/dialogs/options/panel-ingame_bmp.dds'))
    LayoutHelpers.AtCenterIn(panel, parent)
    
    if over then
        panel.Depth:Set(GetFrame(over:GetRootFrame():GetTargetHead()):GetTopmostDepth() + 1)
    end
    
    local worldCover = UIUtil.CreateWorldCover(panel)

    local dlgHead = UIUtil.CreateText(panel, "", 16, UIUtil.titleFont)
    LayoutHelpers.AtLeftTopIn(dlgHead, panel, 40, 23)
    if isLoad then
        dlgHead:SetText(LOC("<LOC uisaveload_0001>Load"))
    else
        dlgHead:SetText(LOC("<LOC uisaveload_0002>Save"))
    end
    
    local dlgLabel = UIUtil.CreateText(panel, "<LOC uisaveload_0003>Enter or select filename", 16, UIUtil.titleFont)
    LayoutHelpers.AtLeftTopIn(dlgLabel, panel, 40, 55)

    ---------------------------------------------------------------------------
    -- OK and cancel button behaviors
    ---------------------------------------------------------------------------
    local function KillDialog()
        if over then
            panel:Destroy()
        else
            parent:Destroy()
        end
        if exitBehavior then
            exitBehavior()
        end
    end

    local okButton = UIUtil.CreateButtonStd(panel, '/dialogs/standard_btn/standard', "<LOC _Ok>", 10)
    LayoutHelpers.AtLeftTopIn(okButton, panel, 44, 417)

    local cancelButton = UIUtil.CreateButtonStd(panel, '/dialogs/standard_btn/standard', "<LOC _Cancel>", 10)
    LayoutHelpers.RightOf(cancelButton, okButton, 10)
    cancelButton.OnClick = function(self, modifiers)
        KillDialog()
    end
    
    if exitButton then
        exitButton.OnClick = function(self, modifiers)
            KillDialog()
        end
    end

    local filePicker = FilePicker(panel, "SaveGame", function(filename)
        ForkThread(function()
            callback(filename, panel, KillDialog)
        end)
    end)
    LayoutHelpers.AtLeftTopIn(filePicker, panel, 36, 91)
    filePicker.Width:Set(530)
    filePicker.Height:Set(307)

    local lastStr
    if isLoad then
        lastStr = Prefs.GetFromCurrentProfile("LastLoadFile")
    else
        lastStr = Prefs.GetFromCurrentProfile("LastSaveFile")
    end
    if lastStr then
        filePicker:SetFilename(lastStr)
    end
    
    okButton.OnClick = function(self, modifiers)
        ForkThread(function ()
                       local text = filePicker:GetFilename()
                       callback(text, panel, KillDialog)
                   end)
    end

end

function CreateSaveDialog(parent, exitBehavior)
    local function DoSave(text, lparent, killBehavior)
        local prettyName = Basename(text, true)
        Prefs.SetToCurrentProfile("LastSaveFile", prettyName)
        local statusStr = "<LOC saveload_0001>Saving game..."
        local status = UIUtil.ShowInfoDialog(lparent, statusStr)
        InternalSaveGame(text, prettyName, function(worked, errmsg)
            status:Destroy()
            local infoStr
            if not worked then
                infoStr = "Save failed! " .. errmsg
                UIUtil.ShowInfoDialog(lparent, infoStr, "<LOC _Ok>")
            else
                killBehavior()
            end
        end)
    end
    CreateDialog(parent, false, DoSave, exitBehavior)
end

function CreateLoadDialog(parent, exitBehavior)
    local function DoLoad(filename, lparent, killBehavior)
        Prefs.SetToCurrentProfile("LastLoadFile", Basename(filename, true))
        if not LoadSavedGame(filename) then
            UIUtil.ShowInfoDialog(lparent, LOCF("<LOC uisaveload_0004>Couldn't open savegame %s", Basename(filename, true)), "<LOC _Ok>")
        end
    end
    CreateDialog(parent, true, DoLoad, exitBehavior)
end
