--*****************************************************************************
--* File: lua/modules/ui/game/avatars.lua
--* Author: Ted Snook
--* Summary: In Game Avatar Icons
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Button = import('/lua/modules/maui/button.lua').Button
local Group = import('/lua/modules/maui/group.lua').Group
local StatusBar = import('/lua/modules/maui/statusbar.lua').StatusBar
local GameCommon = import('/lua/modules/ui/game/gamecommon.lua')
local GameMain = import('/lua/modules/ui/game/gamemain.lua')
local ToolTip = import('/lua/modules/ui/game/tooltip.lua')

local avatarIcons = {}
local avatarContainer = false
local avatarThread = false
local currentT3Index = 1
local currentT2Index = 1
local currentT1Index = 1
local renewTable = false
local avatarSortedTable = {}
local permi = 0
local previousT1Engineers = {}
local previousT2Engineers = {}
local previousT3Engineers = {}
local showBool = true
local dirtyFlag = false
local avatarHealths = {}
local alertControls = {}
local tooltip = false
local notificationCue = {}

function Contract()
    showBool = false
    for i=1, table.getn(avatarIcons) do
        if avatarIcons[i] then
            avatarIcons[i]:Destroy()
            avatarIcons[i] = false
        end
    end
end

function Expand()
    showBool = true
    dirtyFlag = true
end

function CheckFaction()
    local armies = GetArmiesTable()
    local faction = armies.armiesTable[armies.focusArmy].faction
    if faction != notificationCue.lastFaction then
        if faction == 0 then
            name = 'UEF'
        elseif faction == 1 then
            name = 'Cybran'
        elseif faction == 2 then
            name = 'Aeon'
        end

        notificationCue.CommanderUnderAttack = Sound {
            Bank = 'Computer',
            Cue = string.format("Computer_%sComputer_Commanders_01314",name),
        }

    end

    notificationCue.lastFaction = faction
end

function AvatarUpdate()
    if showBool then
        CheckFaction()

        avatarSortedTable, renewTable = CheckForAvatarChange()

        if renewTable then
            for i=1, table.getn(avatarIcons) do
                if avatarIcons[i] then
                    avatarIcons[i]:Destroy()
                    avatarIcons[i] = false
                end
            end
            avatarIcons = {}
            for index, avatarObject in avatarSortedTable do
                CreateAvatarIcon(avatarContainer, avatarObject, index, false)
                permi = index
            end
        else
            for i=1, table.getn(avatarIcons) do
                local avatarObject = avatarSortedTable[i]
                if avatarObject then
                    if avatarObject:GetHealth() / avatarObject:GetMaxHealth() > .75 then
                        avatarIcons[i].Healthbar._bar:SetTexture(UIUtil.UIFile('/game/unit_bmp/bar-02_bmp.dds'))
                    elseif avatarObject:GetHealth() / avatarObject:GetMaxHealth() > .25 then
                        avatarIcons[i].Healthbar._bar:SetTexture(UIUtil.UIFile('/game/unit_bmp/bar-01_bmp.dds'))
                    elseif avatarObject:GetHealth() / avatarObject:GetMaxHealth() > 0 then
                        avatarIcons[i].Healthbar._bar:SetTexture(UIUtil.UIFile('/game/unit_bmp/bar-03_bmp.dds'))
                    end
                    avatarIcons[i].Healthbar:SetValue(avatarObject:GetHealth())
                end
            end
        end
        permi = permi or 0
        AddIdleBuilders(permi, renewTable)
    end
end

function UpdateAlerts(deltaTime)
    for i,v in alertControls do
        if v.lasthit > 3 then
            v.control:SetAlpha(0.0)
        else
            v.control:SetAlpha( math.abs(math.cos( v.lasthit * 4.0 )) )
        end

        # Play commander under attack
        if v.commander and (not v.lastWarning or v.total - v.lastWarning > 12) then
            PlaySound( notificationCue.CommanderUnderAttack )
            v.lastWarning = v.total
        end

        v.lasthit = v.lasthit + deltaTime
        v.total = v.total + deltaTime
    end

    # Remove stale alerts
    for _,k in table.keys(alertControls) do
        if alertControls[k].lasthit > 6 then
            table.remove(alertControls,k)
        end
    end
end

function CheckForAvatarChange()
    local newSortedTable = {}
    local tempAvatars = GetArmyAvatars()
    local temptable = {}

    if tempAvatars != nil then
        for i, v in tempAvatars do
            local blueprintID = v:GetBlueprint().BlueprintId
            local newentry = true
            for h, x in temptable do
                if x[blueprintID] then
                    if x[blueprintID] < 3 then
                        table.insert(newSortedTable, v)
                    end
                    x[blueprintID] = x[blueprintID] + 1
                    newentry = false
                    break
                end
            end
            if newentry then
                table.insert(newSortedTable, v)
                table.insert(temptable, {[blueprintID] = 1})
            end
        end
    end

    local newTable = false

    if table.getn(newSortedTable) == table.getn(avatarSortedTable) then
        for i, v in newSortedTable do
            if v != avatarSortedTable[i] then
                newTable = true
            end
        end
    else
        newTable = true
    end

    if dirtyFlag or newTable then
        dirtyFlag = false
        newTable = true
        alertControls = {}
    else
        if table.getn(avatarSortedTable) > 0 then
            for i, v in avatarSortedTable do
                if avatarHealths[i] then
                    if avatarHealths[i] > newSortedTable[i]:GetHealth() then
                        local found = false
                        for k, alert in alertControls do
                            if alert.control == avatarIcons[i].Glow then
                                alertControls[k].lasthit = 0
                                found = true
                            end
                        end
                        if not found then
                            local tempTable = {
                                control = avatarIcons[i].Glow,
                                total = 0,
                                lasthit = 0,
                                blueprint = avatarIcons[i].Blueprint
                            }

                            if EntityCategoryContains( categories.COMMAND, avatarIcons[i].Blueprint.BlueprintId ) then
                                tempTable.commander = true
                            end

                            table.insert(alertControls, tempTable)
                        end
                    end
                end
            end
            for i, v in avatarSortedTable do
                avatarHealths[i] = v:GetHealth()
            end
        else
            alertControls = {}
            permi = 0
            return {}, true
        end
    end
    return newSortedTable, newTable
end

function CreateAvatarIcons(parent)

    avatarContainer = Group(parent, "avatarContainer")
    avatarContainer.Top:Set(parent.Top)
    avatarContainer.Left:Set(parent.Left)
    avatarContainer.Right:Set(parent.Right)
    avatarContainer.Bottom:Set(parent.Bottom)
    avatarContainer:DisableHitTest()
    avatarContainer:SetNeedsFrameUpdate(true)
    avatarContainer.OnFrame = function(self, delta)
        if avatarIcons[1] then
            UpdateAlerts(delta)
        end
    end

    GameMain.AddBeatFunction(AvatarUpdate)
end

function AddIdleBuilders(ini, renewedTable)
    local idleEngineers = GetIdleEngineers()
    local t1Engineers = {}
    local t2Engineers = {}
    local t3Engineers = {}
    local index = ini + 1
    if idleEngineers then
        t3Engineers = EntityCategoryFilterDown(categories.TECH3, idleEngineers)
        t2Engineers = EntityCategoryFilterDown(categories.TECH2, idleEngineers)
        t1Engineers = EntityCategoryFilterDown(categories.TECH1, idleEngineers)
    end
    if table.getn(t3Engineers) > 0 then
        local buildT3Bool = false
        if table.getn(t3Engineers) != table.getn(previousT3Engineers) then
            buildT3Bool = true

            for i, v in avatarIcons do
                if v.TechLevel then
                    if v.TechLevel == '3' then
                        buildT3Bool = false
                        v.Label:SetText(table.getn(t3Engineers))
                    end
                end
            end
        end
        if renewedTable or buildT3Bool then
            CreateAvatarIcon(avatarContainer, t3Engineers[1], index, true, table.getn(t3Engineers), "3")
        end
        index = index + 1
    else
        local dirtyTable = {}
        for i, v in avatarIcons do
            if v.TechLevel then
                if v.TechLevel == '3' then
                    table.insert(dirtyTable, i)
                end
            end
        end
        for i, v in dirtyTable do
            if avatarIcons[v] then
                avatarIcons[v]:Destroy()
                avatarIcons[v] = false
            end
        end
    end

    if table.getn(t2Engineers) > 0 then
        local buildT2Bool = false
        if table.getn(t2Engineers) != table.getn(previousT2Engineers) then
            buildT2Bool = true

            for i, v in avatarIcons do
                if v.TechLevel then
                    if v.TechLevel == '2' then
                        buildT2Bool = false
                        v.Label:SetText(table.getn(t2Engineers))
                    end
                end
            end
        end

        if renewedTable or buildT2Bool then
            CreateAvatarIcon(avatarContainer, t2Engineers[1], index, true, table.getn(t2Engineers), "2")
        end
        index = index + 1
    else
        local dirtyTable = {}
        for i, v in avatarIcons do
            if v.TechLevel then
                if v.TechLevel == '2' then
                    table.insert(dirtyTable, i)
                end
            end
        end
        for i, v in dirtyTable do
            if avatarIcons[v] then
                avatarIcons[v]:Destroy()
                avatarIcons[v] = false
            end
        end
    end

    if table.getn(t1Engineers) > 0 then
        local buildT1Bool = false
        if table.getn(t1Engineers) != table.getn(previousT1Engineers) then
            buildT1Bool = true

            for i, v in avatarIcons do
                if v.TechLevel then
                    if v.TechLevel == '1' then
                        buildT1Bool = false
                        v.Label:SetText(table.getn(t1Engineers))
                    end
                end
            end
        end

        if renewedTable or buildT1Bool then
            CreateAvatarIcon(avatarContainer, t1Engineers[1], index, true, table.getn(t1Engineers), "1")
        end
    else
        local dirtyTable = {}
        for i, v in avatarIcons do
            if v.TechLevel then
                if v.TechLevel == '1' then
                    table.insert(dirtyTable, i)
                end
            end
        end
        for i, v in dirtyTable do
            if avatarIcons[v] then
                avatarIcons[v]:Destroy()
                avatarIcons[v] = false
            end
        end
    end
end

function GetEngineer(techLevel)
    local idleEngineers = GetIdleEngineers()
    local curIndex = 1
    local selEngineer = false
    if idleEngineers then
        if techLevel == '3' then
            idleEngineers = EntityCategoryFilterDown(categories.TECH3, idleEngineers)
            curIndex = currentT3Index
        elseif techLevel == '2' then
            idleEngineers = EntityCategoryFilterDown(categories.TECH2, idleEngineers)
            curIndex = currentT2Index
        elseif techLevel == '1' then
            idleEngineers = EntityCategoryFilterDown(categories.TECH1, idleEngineers)
            curIndex = currentT1Index
        end
    end

    selEngineer = idleEngineers[curIndex] or idleEngineers[1]

    curIndex = curIndex + 1

    if idleEngineers[curIndex] then
        if techLevel == '3' then
            currentT3Index = curIndex
        elseif techLevel == '2' then
            currentT2Index = curIndex
        elseif techLevel == '1' then
            currentT1Index = curIndex
        end
    else
        if techLevel == '3' then
            currentT3Index = 1
        elseif techLevel == '2' then
            currentT2Index = 1
        elseif techLevel == '1' then
            currentT1Index = 1
        end
    end
    return selEngineer
end

function CreateAvatarIcon(parent, avatarUserObject, index, builderBool, label, techLevel)
    local blueprintID = avatarUserObject:GetBlueprint().BlueprintId
    local blueprint = __blueprints[blueprintID]
    local iconName = GameCommon.GetCachedUnitIconFileNames(blueprint)
    local previousIcon = false
    if index != 1 then
        if avatarIcons[index-1] then
            previousIcon = avatarIcons[index-1]
        else
            index = 1
        end
    end
    if avatarIcons[index] then avatarIcons[index]:Destroy() end
    avatarIcons[index] = {}
    avatarIcons[index] = Button(parent,
    iconName,
    iconName,
    iconName,
    iconName)
    avatarIcons[index].Height:Set(50)
    avatarIcons[index].Width:Set(50)
    avatarIcons[index].Depth:Set(50)
    avatarIcons[index].Blueprint = blueprint
    if index == 1 then
        avatarIcons[index].Top:Set(function() return parent.Top() + 60 end)
        avatarIcons[index].Right:Set(function() return parent.Right() - 20 end)
    elseif previousIcon.Bottom() + avatarIcons[index].Height() > parent.Bottom() - 140 then
        local tempNum = previousIcon.Button.Left() - 5
        avatarIcons[index].Top:Set(function() return parent.Top() + 60 end)
        avatarIcons[index].Right:Set(tempNum)
    else
        local tempNum = previousIcon.Button.Bottom() + 5
        avatarIcons[index].Top:Set(tempNum)
        avatarIcons[index].Right:Set(previousIcon.Right)
    end
    if builderBool then
        if avatarIcons[index].Label then avatarIcons[index].Label:Destroy() end
        avatarIcons[index].Label = UIUtil.CreateText(avatarIcons[index], label, 18, UIUtil.bodyFont)
        avatarIcons[index].Label.Right:Set(avatarIcons[index].Right)
        avatarIcons[index].Label.Bottom:Set(avatarIcons[index].Bottom)
        avatarIcons[index].Label:SetDropShadow(true)
    else
        if avatarIcons[index].Glow then avatarIcons[index].Glow:Destroy() end
        avatarIcons[index].Glow = Bitmap(avatarIcons[index], UIUtil.UIFile('/game/avitar/pulse-bars_bmp.dds'))
        LayoutHelpers.AtCenterIn(avatarIcons[index].Glow, avatarIcons[index])
        avatarIcons[index].Glow.Depth:Set(function() return avatarIcons[index].Depth() - 1 end)
        avatarIcons[index].Glow:SetAlpha(0)

        if avatarIcons[index].Healthbar then avatarIcons[index].Healthbar:Destroy() end
        avatarIcons[index].Healthbar = StatusBar(avatarIcons[index], 0, 1, false, false,
            UIUtil.UIFile('/game/unit_bmp/bar-back_bmp.dds'),
            UIUtil.UIFile('/game/unit_bmp/bar-01_bmp.dds'),
            true, "avatar RO Health Status Bar")
        avatarIcons[index].Healthbar.Left:Set(function() return avatarIcons[index].Left() + 5 end)
        avatarIcons[index].Healthbar.Right:Set(function() return avatarIcons[index].Right() - 5 end)
        avatarIcons[index].Healthbar.Bottom:Set(function() return avatarIcons[index].Bottom() + 5 end)
        avatarIcons[index].Healthbar.Top:Set(function() return avatarIcons[index].Bottom() - 2 end)
        avatarIcons[index].Healthbar.Height:Set(function() return avatarIcons[index].Healthbar.Bottom() - avatarIcons[index].Healthbar.Top() end)
        avatarIcons[index].Healthbar.Width:Set(function() return avatarIcons[index].Healthbar.Right() - avatarIcons[index].Healthbar.Left() end)
        if avatarIcons[index].Healthbar._rangeMax != avatarUserObject:GetMaxHealth() then
            avatarIcons[index].Healthbar:SetRange(0, avatarUserObject:GetMaxHealth())
        end
        if avatarUserObject:GetHealth() / avatarUserObject:GetMaxHealth() > .75 then
            avatarIcons[index].Healthbar._bar:SetTexture(UIUtil.UIFile('/game/unit_bmp/bar-02_bmp.dds'))
        elseif avatarUserObject:GetHealth() / avatarUserObject:GetMaxHealth() > .25 then
            avatarIcons[index].Healthbar._bar:SetTexture(UIUtil.UIFile('/game/unit_bmp/bar-01_bmp.dds'))
        elseif avatarUserObject:GetHealth() / avatarUserObject:GetMaxHealth() > 0 then
            avatarIcons[index].Healthbar._bar:SetTexture(UIUtil.UIFile('/game/unit_bmp/bar-03_bmp.dds'))
        end
        avatarIcons[index].Healthbar:SetValue(avatarUserObject:GetHealth())
        avatarIcons[index].Healthbar:DisableHitTest(true)
    end

    if avatarIcons[index].Button then avatarIcons[index].Button:Destroy() end
    avatarIcons[index].Button = Button(avatarIcons[index],
        UIUtil.UIFile('/game/avitar/avitar_bmp.dds'),
        UIUtil.UIFile('/game/avitar/avitar_bmp.dds'),
        UIUtil.UIFile('/game/avitar/avitar_bmp.dds'),
        UIUtil.UIFile('/game/avitar/avitar_bmp.dds'))
    avatarIcons[index].Button:DisableHitTest()
    LayoutHelpers.AtCenterIn(avatarIcons[index].Button, avatarIcons[index])
    avatarIcons[index].Button.Depth:Set(function() return avatarIcons[index].Depth() - 1 end)

    if builderBool then
        avatarIcons[index].TechLevel = techLevel
        avatarIcons[index].OnClick = function(self, modifiers)
            if modifiers.Left then
                SelectUnits({GetEngineer(self.TechLevel)})
            elseif modifiers.Right then
                UISelectAndZoomTo(GetEngineer(self.TechLevel))
            end
        end
        avatarIcons[index].HandleEvent = function(self, event)
            if event.Type == 'MouseEnter' then
                if tooltip then
                    tooltip:Destroy()
                    tooltip = false
                end
                tooltip = ToolTip.CreateToolTip(self, "Idle T"..self.TechLevel.." Engineers")
                tooltip.Right:Set(self.Left)
                LayoutHelpers.AtVerticalCenterIn(tooltip, self)
            elseif event.Type == 'MouseExit' then
                tooltip:Destroy()
                tooltip = false
            elseif event.Type == 'ButtonPress' or event.Type == 'ButtonDClick' then
                if event.Modifiers.Left then
                    SelectUnits({GetEngineer(self.TechLevel)})
                elseif event.Modifiers.Right then
                    UISelectAndZoomTo(GetEngineer(self.TechLevel))
                end
            end
            #Button.HandleEvent(self, event)
        end
    else
        avatarIcons[index].OnClick = function(self, modifiers)
            if modifiers.Left then
                SelectUnits({avatarUserObject})
            elseif modifiers.Right then
                UISelectAndZoomTo(avatarUserObject)
            end
        end
        avatarIcons[index].HandleEvent = function(self, event)
            if event.Type == 'MouseEnter' then
                if tooltip then
                    tooltip:Destroy()
                    tooltip = false
                end
                tooltip = ToolTip.CreateToolTip(self, blueprint.General.UnitName or blueprint.Description)
                tooltip.Right:Set(self.Left)
                LayoutHelpers.AtVerticalCenterIn(tooltip, self)
            elseif event.Type == 'MouseExit' then
                tooltip:Destroy()
                tooltip = false
            end
            Button.HandleEvent(self, event)
        end
    end
end

local genIndex = 1

function GetEngineerGeneric()
    local idleEngineers = GetIdleEngineers()
    local selEngineer = idleEngineers[genIndex] or idleEngineers[1]
    genIndex = genIndex + 1
    if genIndex > table.getn(idleEngineers) then
        genIndex = 1
    end
    UISelectAndZoomTo(selEngineer)
end