local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap


local gameParent = false

controls = {
-- general controls
    borderGroup = false,
    
-- controls returned to outline the size of certain areas
    statusClusterGroup = false,
    controlClusterGroup = false,
    mapGroup = false,
    mapGroupLeft = false,
    mapGroupRight = false,
    
-- these bits make up the control regions
    bottom = false,
    left = false,
    right = false,
    top = false,
    
-- these bits are on top of the map
    bottomOS = false,
    bottomLeftOS = false,
    bottomRightOS = false,
    leftOS = false,
    rightOS = false,
    topOS = false,
    topLeftOS = false,
    topRightOS = false
}

function HideBorder(hide)
    if controls.borderGroup then controls.borderGroup:SetHidden(hide) end
end

local function DoBottomLayout()
    if not controls.bottom then
        controls.bottom = Bitmap(controls.borderGroup)
    end
    controls.bottom:SetTexture(UIUtil.SkinnableFile('/game/border_bottom/back_brd_b.dds'))
    controls.bottom.Bottom:Set(gameParent.Bottom)
    controls.bottom.Left:Set(gameParent.Left)
    controls.bottom.Right:Set(gameParent.Right)
    controls.bottom.Width:Set(function() return bottom.Right() - bottom.Left() end)
    controls.bottom:SetRenderPass(UIUtil.UIRP_UnderWorld)
    
    if not controls.controlClusterGroup then
        controls.controlClusterGroup = Group(controls.bottom, "controlClusterGroup")
    end
    LayoutHelpers.FillParent(controls.controlClusterGroup, controls.bottom)
    controls.controlClusterGroup:SetRenderPass(UIUtil.UIRP_PostGlow)    
    
    if not controls.top then
        controls.top = Bitmap(controls.borderGroup)
    end
    controls.top:SetTexture(UIUtil.SkinnableFile('/game/border_bottom/back_brd_t.dds'))
    controls.top.Top:Set(gameParent.Top)
    controls.top.Left:Set(gameParent.Left)
    controls.top.Right:Set(gameParent.Right)
    controls.top.Width:Set(function() return controls.top.Right() - controls.top.Left() end)
    
    if not controls.statusClusterGroup then
        controls.statusClusterGroup = Group(controls.top, "statusClusterGroup")
    end
    LayoutHelpers.FillParent(controls.statusClusterGroup, controls.top)
    
    if not controls.left then
        controls.left = Bitmap(controls.borderGroup)
    end
    controls.left:SetTexture(UIUtil.SkinnableFile('/game/border_bottom/back_brd_l.dds'))
    controls.left.Top:Set(controls.top.Bottom)
    controls.left.Bottom:Set(controls.bottom.Top)
    controls.left.Left:Set(function() return gameParent.Left() - 1 end)
    controls.left.Height:Set(function() return controls.left.Bottom() - controls.left.Top() end)
    
    if not controls.right then
        controls.right = Bitmap(controls.borderGroup)
    end
    controls.right:SetTexture(UIUtil.SkinnableFile('/game/border_bottom/back_brd_r.dds'))
    controls.right.Top:Set(controls.top.Bottom)
    controls.right.Bottom:Set(controls.bottom.Top)
    controls.right.Right:Set(gameParent.Right)
    controls.right.Height:Set(function() return controls.right.Bottom() - controls.right.Top() end)
    
    if not controls.mapGroup then
        controls.mapGroup = Group(gameParent, "mapGroup")
    end
    controls.mapGroup.Left:Set(controls.left.Right)
    controls.mapGroup.Top:Set(controls.top.Bottom)
    controls.mapGroup.Right:Set(controls.right.Left)
    controls.mapGroup.Bottom:Set(controls.bottom.Top)
    
    if not controls.bottomLeftOS then
        controls.bottomLeftOS = Bitmap(controls.borderGroup)
    end
    controls.bottomLeftOS:SetTexture(UIUtil.SkinnableFile('/game/border_bottom/back_brd_bl.dds'))
    controls.bottomLeftOS.Left:Set(controls.left.Right)
    controls.bottomLeftOS.Bottom:Set(controls.bottom.Top)

    if not controls.bottomRightOS then
        controls.bottomRightOS = Bitmap(controls.borderGroup)
    end
    controls.bottomRightOS:SetTexture(UIUtil.SkinnableFile('/game/border_bottom/back_brd_br.dds'))
    controls.bottomRightOS.Right:Set(function() return controls.right.Left() end)
    controls.bottomRightOS.Bottom:Set(controls.bottom.Top)
    
    if not controls.topLeftOS then
        controls.topLeftOS = Bitmap(controls.borderGroup)
    end
    controls.topLeftOS:SetTexture(UIUtil.SkinnableFile('/game/border_bottom/back_brd_tl.dds'))
    controls.topLeftOS.Left:Set(function() return controls.left.Right() end)
    controls.topLeftOS.Top:Set(function() return controls.top.Bottom() end)

    if not controls.topRightOS then
        controls.topRightOS = Bitmap(controls.borderGroup)
    end
    controls.topRightOS:SetTexture(UIUtil.SkinnableFile('/game/border_bottom/back_brd_tr.dds'))
    controls.topRightOS.Right:Set(function() return controls.right.Left() end)
    controls.topRightOS.Top:Set(function() return controls.top.Bottom() end)
    
    if not controls.bottomOS then
        controls.bottomOS = Bitmap(controls.borderGroup)
    end
    controls.bottomOS:SetTexture(UIUtil.SkinnableFile('/game/border_bottom/back_brd_b-os.dds'))
    controls.bottomOS.Left:Set(controls.bottomLeftOS.Right)
    controls.bottomOS.Right:Set(controls.bottomRightOS.Left)
    controls.bottomOS.Bottom:Set(function() return controls.bottom.Top() + 2 end)
    controls.bottomOS.Width:Set(function() return controls.bottomOS.Right() - controls.bottomOS.Left() end)

    if not controls.topOS then
        controls.topOS = Bitmap(controls.borderGroup)
    end
    controls.topOS:SetTexture(UIUtil.SkinnableFile('/game/border_bottom/back_brd_t-os.dds'))
    controls.topOS.Left:Set(controls.topLeftOS.Right)
    controls.topOS.Right:Set(controls.topRightOS.Left)
    controls.topOS.Top:Set(function() return controls.top.Bottom()  end)
    controls.topOS.Width:Set(function() return controls.topOS.Right() - controls.topOS.Left() end)

    if not controls.leftOS then
        controls.leftOS = Bitmap(controls.borderGroup)
    end
    controls.leftOS:SetTexture(UIUtil.SkinnableFile('/game/border_bottom/back_brd_l-os.dds'))
    controls.leftOS.Left:Set(function() return controls.left.Right() end)
    controls.leftOS.Top:Set(controls.topLeftOS.Bottom)
    controls.leftOS.Bottom:Set(controls.bottomLeftOS.Top)
    controls.leftOS.Height:Set(function() return controls.leftOS.Top() - controls.leftOS.Bottom() end)

    if not controls.rightOS then
        controls.rightOS = Bitmap(controls.borderGroup)
    end
    controls.rightOS:SetTexture(UIUtil.SkinnableFile('/game/border_bottom/back_brd_r-os.dds'))
    controls.rightOS.Right:Set(function() return controls.right.Left() end)
    controls.rightOS.Top:Set(controls.topRightOS.Bottom)
    controls.rightOS.Bottom:Set(controls.bottomRightOS.Top)
    controls.rightOS.Height:Set(function() return controls.rightOS.Top() - controls.rightOS.Bottom() end)
end

local function DoLeftLayout()
    if not controls.top then
        controls.top = Bitmap(controls.borderGroup)
    end
    controls.top:SetTexture(UIUtil.SkinnableFile('/game/border_left/back_brd_t.dds'))
    controls.top.Top:Set(gameParent.Top)
    controls.top.Left:Set(gameParent.Left)
    controls.top.Right:Set(gameParent.Right)
    controls.top.Width:Set(function() return controls.top.Right() - controls.top.Left() end)
    
    if not controls.statusClusterGroup then
        controls.statusClusterGroup = Group(controls.top, "statusClusterGroup")
    end
    LayoutHelpers.FillParent(controls.statusClusterGroup, controls.top)
    
    if not controls.left then
        controls.left = Bitmap(controls.borderGroup)
    end
    controls.left:SetTexture(UIUtil.SkinnableFile('/game/border_left/back_brd_l.dds'))
    controls.left.Top:Set(controls.top.Bottom)
    controls.left.Bottom:Set(gameParent.Bottom)
    controls.left.Left:Set(gameParent.Left)
    controls.left.Height:Set(function() return controls.left.Bottom() - controls.left.Top() end)
    controls.left:SetRenderPass(UIUtil.UIRP_UnderWorld)
    
    if not controls.controlClusterGroup then
        controls.controlClusterGroup = Group(controls.left, "controlClusterGroup")
    end
    LayoutHelpers.FillParent(controls.controlClusterGroup, controls.left)
    controls.controlClusterGroup:SetRenderPass(UIUtil.UIRP_PostGlow)
    
    if not controls.bottom then
        controls.bottom = Bitmap(controls.borderGroup)
    end
    controls.bottom:SetTexture(UIUtil.SkinnableFile('/game/border_left/back_brd_b.dds'))
    controls.bottom.Bottom:Set(gameParent.Bottom)
    controls.bottom.Left:Set(controls.left.Right)
    controls.bottom.Right:Set(gameParent.Right)
    controls.bottom.Width:Set(function() return controls.bottom.Right() - controls.bottom.Left() end)

    if not controls.right then
        controls.right = Bitmap(controls.borderGroup)
    end
    controls.right:SetTexture(UIUtil.SkinnableFile('/game/border_left/back_brd_r.dds'))
    controls.right.Top:Set(controls.top.Bottom)
    controls.right.Bottom:Set(controls.bottom.Top)
    controls.right.Right:Set(gameParent.Right)
    controls.right.Height:Set(function() return controls.right.Bottom() - controls.right.Top() end)
    
    if not controls.mapGroup then
        controls.mapGroup = Group(gameParent, "mapGroup")
    end
    controls.mapGroup.Left:Set(controls.left.Right)
    controls.mapGroup.Top:Set(controls.top.Bottom)
    controls.mapGroup.Right:Set(controls.right.Left)
    controls.mapGroup.Bottom:Set(controls.bottom.Top)
    
    if not controls.bottomLeftOS then
        controls.bottomLeftOS = Bitmap(controls.borderGroup)
    end
    controls.bottomLeftOS:SetTexture(UIUtil.SkinnableFile('/game/border_bottom/back_brd_bl.dds'))
    controls.bottomLeftOS.Left:Set(function() return controls.left.Right()  end)
    controls.bottomLeftOS.Bottom:Set(function() return controls.bottom.Top() + 1 end)

    if not controls.bottomRightOS then
        controls.bottomRightOS = Bitmap(controls.borderGroup)
    end
    controls.bottomRightOS:SetTexture(UIUtil.SkinnableFile('/game/border_left/back_brd_br.dds'))
    controls.bottomRightOS.Right:Set(function() return controls.right.Left() end)
    controls.bottomRightOS.Bottom:Set(function() return controls.bottom.Top() end)
    
    if not controls.topLeftOS then
        controls.topLeftOS = Bitmap(controls.borderGroup)
    end
    controls.topLeftOS:SetTexture(UIUtil.SkinnableFile('/game/border_left/back_brd_tl.dds'))
    controls.topLeftOS.Left:Set(function() return controls.left.Right() - 2 end)
    controls.topLeftOS.Top:Set(function() return controls.top.Bottom() end)

    if not controls.topRightOS then
        controls.topRightOS = Bitmap(controls.borderGroup)
    end
    controls.topRightOS:SetTexture(UIUtil.SkinnableFile('/game/border_left/back_brd_tr.dds'))
    controls.topRightOS.Right:Set(function() return controls.right.Left() end)
    controls.topRightOS.Top:Set(function() return controls.top.Bottom()end)
    
    if not controls.bottomOS then
        controls.bottomOS = Bitmap(controls.borderGroup)
    end
    controls.bottomOS:SetTexture(UIUtil.SkinnableFile('/game/border_left/back_brd_b-os.dds'))
    controls.bottomOS.Left:Set(controls.bottomLeftOS.Right)
    controls.bottomOS.Right:Set(controls.bottomRightOS.Left)
    controls.bottomOS.Bottom:Set(function() return controls.bottom.Top() end)
    controls.bottomOS.Width:Set(function() return controls.bottomOS.Right() - controls.bottomOS.Left() end)

    if not controls.topOS then
        controls.topOS = Bitmap(controls.borderGroup)
    end
    controls.topOS:SetTexture(UIUtil.SkinnableFile('/game/border_left/back_brd_t-os.dds'))
    controls.topOS.Left:Set(controls.topLeftOS.Right)
    controls.topOS.Right:Set(controls.topRightOS.Left)
    controls.topOS.Top:Set(function() return controls.top.Bottom() end)
    controls.topOS.Width:Set(function() return controls.topOS.Right() - controls.topOS.Left() end)

    if not controls.leftOS then
        controls.leftOS = Bitmap(controls.borderGroup)
    end
    controls.leftOS:SetTexture(UIUtil.SkinnableFile('/game/border_left/back_brd_l-os.dds'))
    controls.leftOS.Left:Set(function() return controls.left.Right() - 1 end)
    controls.leftOS.Top:Set(controls.topLeftOS.Bottom)
    controls.leftOS.Bottom:Set(controls.bottomLeftOS.Top)
    controls.leftOS.Height:Set(function() return controls.leftOS.Top() - controls.leftOS.Bottom() end)

    if not controls.rightOS then
        controls.rightOS = Bitmap(controls.borderGroup)
    end
    controls.rightOS:SetTexture(UIUtil.SkinnableFile('/game/border_left/back_brd_r-os.dds'))
    controls.rightOS.Right:Set(function() return controls.right.Left() end)
    controls.rightOS.Top:Set(controls.topRightOS.Bottom)
    controls.rightOS.Bottom:Set(controls.bottomRightOS.Top)
    controls.rightOS.Height:Set(function() return controls.rightOS.Top() - controls.rightOS.Bottom() end)
end

local function DoRightLayout()
    if not controls.top then
        controls.top = Bitmap(controls.borderGroup)
    end
    controls.top:SetTexture(UIUtil.SkinnableFile('/game/border_right/back_brd_t.dds'))
    controls.top.Top:Set(gameParent.Top)
    controls.top.Left:Set(gameParent.Left)
    controls.top.Right:Set(gameParent.Right)
    controls.top.Width:Set(function() return controls.top.Right() - controls.top.Left() end)
    
    if not controls.statusClusterGroup then
        controls.statusClusterGroup = Group(controls.top, "statusClusterGroup")
    end
    LayoutHelpers.FillParent(controls.statusClusterGroup, controls.top)
    
    if not controls.left then
        controls.left = Bitmap(controls.borderGroup)
    end
    controls.left:SetTexture(UIUtil.SkinnableFile('/game/border_right/back_brd_l.dds'))
    controls.left.Top:Set(controls.top.Bottom)
    controls.left.Bottom:Set(gameParent.Bottom)
    controls.left.Left:Set(gameParent.Left)
    controls.left.Height:Set(function() return controls.left.Bottom() - controls.left.Top() end)
    controls.left:SetRenderPass(UIUtil.UIRP_UnderWorld)
    

    if not controls.right then
        controls.right = Bitmap(controls.borderGroup)
    end
    controls.right:SetTexture(UIUtil.SkinnableFile('/game/border_right/back_brd_r.dds'))
    controls.right.Top:Set(controls.top.Bottom)
    controls.right.Bottom:Set(gameParent.Bottom)
    controls.right.Right:Set(gameParent.Right)
    controls.right.Height:Set(function() return controls.right.Bottom() - controls.right.Top() end)
    controls.right:SetRenderPass(UIUtil.UIRP_UnderWorld)
    
    if not controls.bottom then
        controls.bottom = Bitmap(controls.borderGroup)
    end
    controls.bottom:SetTexture(UIUtil.SkinnableFile('/game/border_right/back_brd_b.dds'))
    controls.bottom.Bottom:Set(gameParent.Bottom)
    controls.bottom.Left:Set(controls.left.Right)
    controls.bottom.Right:Set(controls.right.Left)
    controls.bottom.Width:Set(function() return controls.bottom.Right() - controls.bottom.Left() end)
    
    if not controls.controlClusterGroup then
        controls.controlClusterGroup = Group(controls.right, "controlClusterGroup")
    end
    LayoutHelpers.FillParent(controls.controlClusterGroup, controls.right)
    controls.controlClusterGroup:SetRenderPass(UIUtil.UIRP_PostGlow)
    
    if not controls.mapGroup then
        controls.mapGroup = Group(gameParent, "mapGroup")
    end
    controls.mapGroup.Left:Set(controls.left.Right)
    controls.mapGroup.Top:Set(controls.top.Bottom)
    controls.mapGroup.Right:Set(controls.right.Left)
    controls.mapGroup.Bottom:Set(controls.bottom.Top)
    
    if not controls.bottomLeftOS then
        controls.bottomLeftOS = Bitmap(controls.borderGroup)
    end
    controls.bottomLeftOS:SetTexture(UIUtil.SkinnableFile('/game/border_bottom/back_brd_bl.dds'))
    controls.bottomLeftOS.Left:Set(controls.left.Right)
    controls.bottomLeftOS.Bottom:Set(function() return controls.bottom.Top() + 1 end)

    if not controls.bottomRightOS then
        controls.bottomRightOS = Bitmap(controls.borderGroup)
    end
    controls.bottomRightOS:SetTexture(UIUtil.SkinnableFile('/game/border_right/back_brd_br.dds'))
    controls.bottomRightOS.Right:Set(function() return controls.right.Left() end)
    controls.bottomRightOS.Bottom:Set(function() return controls.bottom.Top()  end)
    
    if not controls.topLeftOS then
        controls.topLeftOS = Bitmap(controls.borderGroup)
    end
    controls.topLeftOS:SetTexture(UIUtil.SkinnableFile('/game/border_right/back_brd_tl.dds'))
    controls.topLeftOS.Left:Set(controls.left.Right)
    controls.topLeftOS.Top:Set(function() return controls.top.Bottom() end)

    if not controls.topRightOS then
        controls.topRightOS = Bitmap(controls.borderGroup)
    end
    controls.topRightOS:SetTexture(UIUtil.SkinnableFile('/game/border_right/back_brd_tr.dds'))
    controls.topRightOS.Right:Set(function() return controls.right.Left() + 1 end)
    controls.topRightOS.Top:Set(function() return controls.top.Bottom()end)
    
    if not controls.bottomOS then
        controls.bottomOS = Bitmap(controls.borderGroup)
    end
    controls.bottomOS:SetTexture(UIUtil.SkinnableFile('/game/border_right/back_brd_b-os.dds'))
    controls.bottomOS.Left:Set(controls.bottomLeftOS.Right)
    controls.bottomOS.Right:Set(controls.bottomRightOS.Left)
    controls.bottomOS.Bottom:Set(function() return controls.bottom.Top() end)
    controls.bottomOS.Width:Set(function() return controls.bottomOS.Right() - controls.bottomOS.Left() end)

    if not controls.topOS then
        controls.topOS = Bitmap(controls.borderGroup)
    end
    controls.topOS:SetTexture(UIUtil.SkinnableFile('/game/border_right/back_brd_t-os.dds'))
    controls.topOS.Left:Set(controls.topLeftOS.Right)
    controls.topOS.Right:Set(controls.topRightOS.Left)
    controls.topOS.Top:Set(function() return controls.top.Bottom() end)
    controls.topOS.Width:Set(function() return controls.topOS.Right() - controls.topOS.Left() end)

    if not controls.leftOS then
        controls.leftOS = Bitmap(controls.borderGroup)
    end
    controls.leftOS:SetTexture(UIUtil.SkinnableFile('/game/border_right/back_brd_l-os.dds'))
    controls.leftOS.Left:Set(controls.left.Right)
    controls.leftOS.Top:Set(controls.topLeftOS.Bottom)
    controls.leftOS.Bottom:Set(controls.bottomLeftOS.Top)
    controls.leftOS.Height:Set(function() return controls.leftOS.Top() - controls.leftOS.Bottom() end)

    if not controls.rightOS then
        controls.rightOS = Bitmap(controls.borderGroup)
    end
    controls.rightOS:SetTexture(UIUtil.SkinnableFile('/game/border_right/back_brd_r-os.dds'))
    controls.rightOS.Right:Set(function() return controls.right.Left() end)
    controls.rightOS.Top:Set(controls.topRightOS.Bottom)
    controls.rightOS.Bottom:Set(controls.bottomRightOS.Top)
    controls.rightOS.Height:Set(function() return controls.rightOS.Top() - controls.rightOS.Bottom() end)
end

function SetLayout(layout)
    if not gameParent then 
        WARN("gameParent not set in borders:SetLayout") 
    end

    if not controls.borderGroup then
        controls.borderGroup = Group(gameParent, "borderGroup")
    end
    controls.borderGroup.Left:Set(0)
    controls.borderGroup.Right:Set(0)
    controls.borderGroup.Width:Set(0)
    controls.borderGroup.Height:Set(0)

    if layout == 'bottom' then
        DoBottomLayout()
    elseif layout == 'left' then
        DoLeftLayout()
    elseif layout == 'right' then
        DoRightLayout()
    end

    controls.bottomOS:DisableHitTest()
    controls.bottomLeftOS:DisableHitTest()
    controls.bottomRightOS:DisableHitTest()
    controls.leftOS:DisableHitTest()
    controls.rightOS:DisableHitTest()
    controls.topOS:DisableHitTest()
    controls.topLeftOS:DisableHitTest()
    controls.topRightOS:DisableHitTest()
    
end

function SetupBorderControl(parent)
    gameParent = parent
   
    SetLayout(UIUtil.currentLayout)
     
    return controls.controlClusterGroup, controls.statusClusterGroup, controls.mapGroup, controls.top
end

function GetMapGroup()
    return controls.mapGroup
end

function SplitMapGroup(splitState)
    if GetCurrentUIState() != "game" then return end
    if splitState then
        if not controls.mapGroupLeft then
            controls.mapGroupLeft = Group(gameParent, "mapGroupLeft")
        end
        controls.mapGroupLeft.Left:Set(controls.left.Right)
        controls.mapGroupLeft.Top:Set(controls.top.Bottom)
        controls.mapGroupLeft.Right:Set(function() return (controls.left.Right() - 3) + ((controls.right.Left() - controls.left.Right()) / 2) end)
        controls.mapGroupLeft.Bottom:Set(controls.bottom.Top)
        
        if not controls.mapGroupRight then
            controls.mapGroupRight = Group(gameParent, "mapGroupRight")
        end
        controls.mapGroupRight.Left:Set(function() return (controls.left.Right() + 3) + ((controls.right.Left() - controls.left.Right()) / 2) end)
        controls.mapGroupRight.Top:Set(controls.top.Bottom)
        controls.mapGroupRight.Right:Set(controls.right.Left)
        controls.mapGroupRight.Bottom:Set(controls.bottom.Top)
        
        if not controls.mapGroup then
            controls.mapGroup = Group(gameParent, "mapGroup")
        end
        controls.mapGroup.Left:Set(controls.mapGroupLeft.Left)
        controls.mapGroup.Top:Set(controls.top.Bottom)
        controls.mapGroup.Right:Set(controls.mapGroupRight.Right)
        controls.mapGroup.Bottom:Set(controls.bottom.Top)
        
        import('/lua/modules/ui/game/worldview.lua').CreateMainWorldView(gameParent, controls.mapGroupLeft, controls.mapGroupRight)
    else
        if not controls.mapGroup then
            controls.mapGroup = Group(gameParent, "mapGroup")
        end
        controls.mapGroup.Left:Set(controls.left.Right)
        controls.mapGroup.Top:Set(controls.top.Bottom)
        controls.mapGroup.Right:Set(controls.right.Left)
        controls.mapGroup.Bottom:Set(controls.bottom.Top)
        
        import('/lua/modules/ui/game/worldview.lua').CreateMainWorldView(gameParent, controls.mapGroup)
    end
end