--*****************************************************************************
--* File: lua/modules/ui/game/chat.lua
--* Author: Chris Blackwell
--* Summary: In game chat ui
--*
--* Copyright  :005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************


local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local EffectHelpers = import('/lua/modules/maui/effecthelpers.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Button = import('/lua/modules/maui/button.lua').Button
local Text = import('/lua/modules/maui/text.lua').Text
local Edit = import('/lua/modules/maui/edit.lua').Edit
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local OnlineProvider = import('/lua/modules/multiplayer/onlineprovider.lua')
local ItemList = import('/lua/modules/maui/itemlist.lua').ItemList
local Prefs = import('/lua/modules/ui/prefs.lua')
local tauntFile = import('/lua/modules/ui/game/taunt.lua')

local CHAT_INACTIVITY_TIMEOUT = 15  -- in seconds

local chatHistory = {}

local chatBottomLine = 1
local chatEdit = false
local chatDisplay = false
local chatInactivityTime = CHAT_INACTIVITY_TIMEOUT
local group = false
local parent = false

local topLeft = false
local topMiddle = false
local topRight = false
local middleLeft = false
local middleMiddle = false
local middleRight = false
local bottomLeft = false
local bottomMiddle = false
local bottomRight = false
local hideButton = false
local chatToAlliesButton = false
local chatToAllButton = false
local chatToGroup = "All"

local consoleFontName = UIUtil.fixedFont
local consoleFontSize = 12

local commandDeque = {}
local maxCommandDequeSize = 10
local currentCommandDequeIndex = 0
local currentText = ""
local consoleFontName = UIUtil.fixedFont
local consoleFontSize = 12

local chatStatus = false

chatDisplay = {}
    chatDisplay[1] = false
    chatDisplay[2] = false
    chatDisplay[3] = false
    chatDisplay[4] = false
    chatDisplay[5] = false
    chatDisplay[6] = false
    chatDisplay[7] = false
    chatDisplay[8] = false
    chatDisplay[9] = false
    chatDisplay[10] = false
    chatDisplay[11] = false

local function InsertCommand(text)
    table.insert(commandDeque, text)
    if table.getn(commandDeque) > maxCommandDequeSize then
        table.remove(commandDeque, 1)
    end
    currentCommandDequeIndex = table.getn(commandDeque)
end

function CreateBackground(group)

    if not topLeft then
        topLeft = Bitmap(group, UIUtil.UIFile('/game/chat_brd04/chat_brd_ul.dds'))
        topLeft:Hide()
    end
    topLeft.Top:Set(group.Top)
    topLeft.Left:Set(group.Left)
    topLeft:DisableHitTest()

    if not topRight then
        topRight = Bitmap(group, UIUtil.UIFile('/game/chat_brd04/chat_brd_ur.dds'))
        topRight:Hide()
    end
    topRight.Top:Set(group.Top)
    topRight.Right:Set(group.Right)
    topRight:DisableHitTest()

    if not topMiddle then
        topMiddle = Bitmap(group, UIUtil.UIFile('/game/chat_brd04/chat_brd_horz_um.dds'))
        topMiddle:Hide()
    end
    topMiddle.Top:Set(group.Top)
    topMiddle.Left:Set(function() return topLeft.Right() - 1 end)
    topMiddle.Right:Set(topRight.Left)
    topMiddle.Width:Set(function() return topMiddle.Right() - topMiddle.Left() end)
    topMiddle:DisableHitTest()

    if not bottomLeft then
        bottomLeft = Bitmap(group, UIUtil.UIFile('/game/chat_brd04/chat_brd_ll.dds'))
        bottomLeft:Hide()
    end
    bottomLeft.Bottom:Set(group.Bottom)
    bottomLeft.Left:Set(group.Left)
    bottomLeft:DisableHitTest()

    if not bottomRight then
        bottomRight = Bitmap(group, UIUtil.UIFile('/game/chat_brd04/chat_brd_lr.dds'))
        bottomRight:Hide()
    end
    bottomRight.Bottom:Set(group.Bottom)
    bottomRight.Right:Set(group.Right)
    bottomRight:DisableHitTest()

    if not bottomMiddle then
        bottomMiddle = Bitmap(group, UIUtil.UIFile('/game/chat_brd04/chat_brd_lm.dds'))
        bottomMiddle:Hide()
    end
    bottomMiddle.Bottom:Set(group.Bottom)
    bottomMiddle.Left:Set(function() return bottomLeft.Right() - 1 end)
    bottomMiddle.Right:Set(bottomRight.Left)
    bottomMiddle.Width:Set(function() return bottomMiddle.Right() - bottomMiddle.Left() end)
    bottomMiddle:DisableHitTest()

    if not middleLeft then
        middleLeft = Bitmap(group, UIUtil.UIFile('/game/chat_brd04/chat_brd_vert_l.dds'))
        middleLeft:Hide()
    end
    middleLeft.Top:Set(topLeft.Bottom)
    middleLeft.Bottom:Set(bottomLeft.Top)
    middleLeft.Left:Set(topLeft.Left)
    middleLeft.Height:Set(function() return middleLeft.Bottom() - middleLeft.Top() end)
    middleLeft:DisableHitTest()

    if not middleRight then
        middleRight = Bitmap(group, UIUtil.UIFile('/game/chat_brd04/chat_brd_vert_r.dds'))
        middleRight:Hide()
    end
    middleRight.Top:Set(topRight.Bottom)
    middleRight.Bottom:Set(bottomRight.Top)
    middleRight.Right:Set(group.Right)
    middleRight.Height:Set(function() return middleRight.Bottom() - middleRight.Top() end)
    middleRight:DisableHitTest()

    if not middleMiddle then
        middleMiddle = Bitmap(group, UIUtil.UIFile('/game/chat_brd04/chat_brd_m.dds'))
        middleMiddle:Hide()
    end
    middleMiddle.Top:Set(topMiddle.Bottom)
    middleMiddle.Bottom:Set(bottomMiddle.Top)
    middleMiddle.Left:Set(function() return middleLeft.Right() - 1 end)
    middleMiddle.Right:Set(middleRight.Left)
    middleMiddle.Height:Set(function() return middleMiddle.Bottom() - middleMiddle.Top() end)
    middleMiddle.Width:Set(function() return middleMiddle.Right() - middleMiddle.Left() end)
    middleMiddle:DisableHitTest()

    if not hideButton then
        hideButton = UIUtil.CreateButton(group,
            '/dialogs/close_btn/close_btn_up.dds',
            '/dialogs/close_btn/close_btn_down.dds',
            '/dialogs/close_btn/close_btn_over.dds',
            '/dialogs/close_btn/close_btn_dis.dds')
    end
    hideButton.Right:Set(function() return topRight.Right() - 10 end)
    hideButton.Top:Set(function() return topRight.Top() + 12 end)
    hideButton.OnClick = function(self, modifiers)
        group:Hide()
        TimeoutChatDisplay()
        chatEdit:AbandonFocus()
    end
    hideButton:Hide()
    hideButton.Depth:Set(1201)
    
    chatToAllButton = Button(group, 
            UIUtil.UIFile('/dialogs/diplomacy_btn/chat-all_btn_up.dds'), 
            UIUtil.UIFile('/dialogs/diplomacy_btn/chat-all_btn_down.dds'), 
            UIUtil.UIFile('/dialogs/diplomacy_btn/chat-all_btn_over.dds'), 
            UIUtil.UIFile('/dialogs/diplomacy_btn/chat-all_btn_dis.dds'))
    chatToAllButton.Top:Set(function() return topRight.Top() + 3 end)
    chatToAllButton.Right:Set(function() return hideButton.Left() - 5 end)
    chatToAllButton:Hide()
    chatToAllButton.Depth:Set(1201)
    chatToAllButton.OnClick = function(self, modifiers)
        chatToGroup = "All"
    end

    chatToAlliesButton = Button(group, 
            UIUtil.UIFile('/dialogs/diplomacy_btn/chat-allies_btn_up.dds'), 
            UIUtil.UIFile('/dialogs/diplomacy_btn/chat-allies_btn_down.dds'), 
            UIUtil.UIFile('/dialogs/diplomacy_btn/chat-allies_btn_over.dds'), 
            UIUtil.UIFile('/dialogs/diplomacy_btn/chat-allies_btn_dis.dds'))
    chatToAlliesButton.Top:Set(chatToAllButton.Top)
    chatToAlliesButton.Right:Set(function() return chatToAllButton.Left() - 5 end)
    chatToAlliesButton:Hide()
    chatToAlliesButton.Depth:Set(1201)
    chatToAlliesButton.OnClick = function(self, modifiers)
        chatToGroup = "Allies"
    end
end

function CreateChatDisplay(group)
    local leftOffset = 10
    local bottomOffset = 45
    local lastTop = false

    for i, v in chatDisplay do
        local depthCounter = 1100 + i
        if not chatDisplay[i] then
            chatDisplay[i] = UIUtil.CreateText(group, "", 14, UIUtil.bodyFont)
            chatDisplay[i]:Hide()
        end
        local index = i - 1
        if i == 1 then
            chatDisplay[i].Bottom:Set(function() return group.Bottom() - bottomOffset end)
        else
            chatDisplay[i].Bottom:Set(function() return chatDisplay[index].Top() - 5 end)
        end
        chatDisplay[i].Height:Set(10)
        chatDisplay[i].Left:Set(function() return group.Left() + leftOffset end)
        chatDisplay[i].Right:Set(function() return group.Right() - leftOffset end)
        chatDisplay[i]:SetDropShadow(true)
        chatDisplay[i]:DisableHitTest()
        chatDisplay[i].Depth:Set(depthCounter)

        chatDisplay[i].timeout = 0
    end

    if not chatStatus then
        chatStatus = UIUtil.CreateText(group, "Chat", 14, UIUtil.bodyFont)
    end
    chatStatus:Hide()
    chatStatus.Top:Set(function() return group.Top() + 6 end)
    chatStatus.Left:Set(function() return group.Left() + 15 end)
    chatStatus.Right:Set(group.Right)
    chatStatus.Height:Set(12)
    chatStatus:SetDropShadow(true)
    chatStatus.Depth:Set(1200)
    chatStatus:DisableHitTest()
end

function DoBottomLayout()
    if not group then
        group = Group(parent, "Chat Window")
    end
    group.Bottom:Set(function() return parent.Bottom() - 80 end)
    group.Left:Set(function() return parent.Left() + 20 end)
    group.Right:Set(function() return group.Left() + 400 end)
    group.Top:Set(function() return group.Bottom() - 240 end)
    group.Depth:Set(function() return parent.Depth() + 1000 end)
    group:SetRenderPass(UIUtil.UIRP_PostGlow)
    group:DisableHitTest()

    CreateBackground(group)

    if not chatEdit then
        chatEdit = Edit(group)
    end
    chatEdit.Height:Set(function() return chatEdit:GetFontHeight() end)
    chatEdit.Left:Set(function() return bottomLeft.Right() end)
    chatEdit.Right:Set(function() return bottomRight.Right() - 30 end)
    chatEdit.Width:Set(function() return chatEdit.Right() - chatEdit.Left() end)
    chatEdit.Bottom:Set(function() return group.Bottom() - 8 end)
    chatEdit.Depth:Set(function() return group.Depth() + 50 end)
    chatEdit:SetFont(UIUtil.bodyFont, 14)
    chatEdit:SetForegroundColor("ff00ff00")
    chatEdit:SetHighlightBackgroundColor(UIUtil.highlightColor)
    chatEdit:SetHighlightForegroundColor("blue")
    chatEdit:SetDropShadow(true)
    chatEdit:ShowBackground(false)
    chatEdit:AbandonFocus()

    CreateChatDisplay(group)
end

function DoLeftLayout()
    if not group then
        group = Group(parent, "Chat Window")
    end
    group.Bottom:Set(function() return parent.Bottom() - 80 end)
    group.Left:Set(function() return parent.Left() + 20 end)
    group.Right:Set(function() return group.Left() + 400 end)
    group.Top:Set(function() return group.Bottom() - 240 end)
    group.Depth:Set(function() return parent.Depth() + 1000 end)
    group:SetRenderPass(UIUtil.UIRP_PostGlow)
    group:DisableHitTest()

    CreateBackground(group)

    if not chatEdit then
        chatEdit = Edit(group)
    end
    chatEdit.Height:Set(function() return chatEdit:GetFontHeight() end)
    chatEdit.Left:Set(function() return bottomLeft.Right() end)
    chatEdit.Right:Set(function() return bottomRight.Right() - 30 end)
    chatEdit.Width:Set(function() return chatEdit.Right() - chatEdit.Left() end)
    chatEdit.Bottom:Set(function() return group.Bottom() - 8 end)
    chatEdit.Depth:Set(function() return group.Depth() + 50 end)
    chatEdit:SetFont(UIUtil.bodyFont, 14)
    chatEdit:SetForegroundColor("ff00ff00")
    chatEdit:SetHighlightBackgroundColor(UIUtil.highlightColor)
    chatEdit:SetHighlightForegroundColor("blue")
    chatEdit:SetDropShadow(true)
    chatEdit:ShowBackground(false)
    chatEdit:AbandonFocus()

    CreateChatDisplay(group)
end

function DoRightLayout()
    if not group then
        group = Group(parent, "Chat Window")
    end
    group.Bottom:Set(function() return parent.Bottom() - 80 end)
    group.Left:Set(function() return parent.Left() + 20 end)
    group.Right:Set(function() return group.Left() + 400 end)
    group.Top:Set(function() return group.Bottom() - 240 end)
    group.Depth:Set(function() return parent.Depth() + 1000 end)
    group:SetRenderPass(UIUtil.UIRP_PostGlow)
    group:DisableHitTest()

    CreateBackground(group)

    if not chatEdit then
        chatEdit = Edit(group)
    end
    chatEdit.Height:Set(function() return chatEdit:GetFontHeight() end)
    chatEdit.Left:Set(function() return bottomLeft.Right() end)
    chatEdit.Right:Set(function() return bottomRight.Right() - 30 end)
    chatEdit.Width:Set(function() return chatEdit.Right() - chatEdit.Left() end)
    chatEdit.Bottom:Set(function() return group.Bottom() - 8 end)
    chatEdit.Depth:Set(function() return group.Depth() + 50 end)
    chatEdit:SetFont(UIUtil.bodyFont, 14)
    chatEdit:SetForegroundColor("ff00ff00")
    chatEdit:SetHighlightBackgroundColor(UIUtil.highlightColor)
    chatEdit:SetHighlightForegroundColor("blue")
    chatEdit:SetDropShadow(true)
    chatEdit:ShowBackground(false)
    chatEdit:AbandonFocus()

    CreateChatDisplay(group)
end

function ActivateChat()
    if chatEdit then
        group:Show()
        local sound = Sound({Cue = "UI_Mail_Window_Open", Bank = "Interface",})
        PlaySound(sound)
        EffectHelpers.FadeIn(topLeft, .1)
        EffectHelpers.FadeIn(topMiddle, .1)
        EffectHelpers.FadeIn(topRight, .1)
        EffectHelpers.FadeIn(middleLeft, .1)
        EffectHelpers.FadeIn(middleMiddle, .1)
        EffectHelpers.FadeIn(middleRight, .1)
        EffectHelpers.FadeIn(bottomLeft, .1)
        EffectHelpers.FadeIn(bottomMiddle, .1)
        EffectHelpers.FadeIn(bottomRight, .1)
        chatInactivityTime = CHAT_INACTIVITY_TIMEOUT
        chatEdit:EnableInput()
        chatEdit:EnableHitTest()
        chatEdit:AcquireFocus()
    end
end

function ToggleChat()
    if chatEdit then
        if group:IsHidden() then
            local sound = Sound({Cue = "UI_Mail_Window_Open", Bank = "Interface",})
            PlaySound(sound)
            EffectHelpers.FadeIn(topLeft, .1)
            EffectHelpers.FadeIn(topMiddle, .1)
            EffectHelpers.FadeIn(topRight, .1)
            EffectHelpers.FadeIn(middleLeft, .1)
            EffectHelpers.FadeIn(middleMiddle, .1)
            EffectHelpers.FadeIn(middleRight, .1)
            EffectHelpers.FadeIn(bottomLeft, .1)
            EffectHelpers.FadeIn(bottomMiddle, .1)
            EffectHelpers.FadeIn(bottomRight, .1)
            ActivateChat()
        else
            local sound = Sound({Cue = "UI_Mail_Window_Close", Bank = "Interface",})
            PlaySound(sound)
            group:Hide()
            TimeoutChatDisplay()
            chatEdit:AbandonFocus()
            EffectHelpers.FadeOut(topLeft, .1)
            EffectHelpers.FadeOut(topMiddle, .1)
            EffectHelpers.FadeOut(topRight, .1)
            EffectHelpers.FadeOut(middleLeft, .1)
            EffectHelpers.FadeOut(middleMiddle, .1)
            EffectHelpers.FadeOut(middleRight, .1)
            EffectHelpers.FadeOut(bottomLeft, .1)
            EffectHelpers.FadeOut(bottomMiddle, .1)
            EffectHelpers.FadeOut(bottomRight, .1)
        end
    end
end

function UpdateChat(text)
    local textBoxWidth = chatDisplay[1].Right() - chatDisplay[1].Left()

    local tempTable = import('/lua/modules/maui/text.lua').WrapText(text, textBoxWidth,
    function(text)
        return chatDisplay[1]:GetStringAdvance(text)
    end)

    local tempLines = table.getn(tempTable)

    for i = 1, tempLines do
        table.insert(chatHistory, tempTable[i])
        chatBottomLine = chatBottomLine + 1
    end

    local i = 11
    while i > 1 do
        chatDisplay[i].timeout = chatDisplay[i-1].timeout
        i = i - 1
    end

    UpdateChatDisplay(tempLines)
end

function ReceiveChat(sender, msg)
    sender = sender or "nil sender"
    if type(msg) == 'string' then
        msg = { text = msg }
    elseif type(msg) != 'table' then
        msg = { text = repr(msg) }
    end
    if msg.tauntid then
        if Prefs.GetOption("mp_taunt_head_enabled") == 'true' then
            local taunt = tauntFile[msg.taunthead][msg.tauntid]
            if taunt and taunt['vid'] then
                import('/lua/modules/ui/game/missiontext.lua').PlayMFDMovie({'/movies/'..taunt['vid'], taunt['bank'], taunt['cue'],},
                                                                            {"["..sender.."] "..LOC(taunt['text'])},
                                                                            true)
            end
        end
    else
        local text = '['..sender
        if msg.to then
            text = text .. ' to ' .. msg.to
        end
        text = text .. '] ' .. msg.text
        UpdateChat(text)
    end
end


function ChatPageUp(mod)
    if group then
        if group:IsHidden() then
            ForkThread(
                function()
                    ActivateChat()
                end
            )
            return true
        end
        if chatBottomLine - mod > 0 then
            chatBottomLine = chatBottomLine - mod
        end
        UpdateChatDisplay()
    end
end

function ChatPageDown(mod)
    if group then
        if group:IsHidden() then
            ForkThread(
                function()
                    ActivateChat()
                end
            )
            return true
        elseif chatBottomLine == table.getn(chatHistory) + 1 then
            ForkThread(
                function()
                    TimeoutChatDisplay()
                    chatEdit:AbandonFocus()
                    chatInactivityTime = .1
                end
            )
            return true
        end
        if chatBottomLine + mod <= table.getn(chatHistory) then
            chatBottomLine = chatBottomLine + mod
        elseif chatBottomLine + mod > table.getn(chatHistory) then
            chatBottomLine = table.getn(chatHistory) + 1
        end
        UpdateChatDisplay()
    end
end

function SetLayout(layout)
    if layout == 'bottom' then
        DoBottomLayout()
    elseif layout == 'left' then
        DoLeftLayout()
    elseif layout == 'right' then
        DoRightLayout()
    end
end


# GetArmiesTable() returns something like this:
#{
#  armiesTable={
#    { authorizedCommandSources = { 2 }, ... },
#    { authorizedCommandSources = { 1 }, ... },
#    { authorizedCommandSources = { }, ... },
#    { authorizedCommandSources = { 1, 2 }, ... },
#  },
#  focusArmy=1,
#  numArmies=3
#}
#
# GetSessionClients() returns something like this:
# {
#   { authorizedCommandSources = { 1 }, ... },
#   { authorizedCommandSources = { 2 }, ... },
# }
#
# Other facts of note:
#  - No two clients can share a command source.
#  - A client can be authorized to switch between multiple command
#    sources (that nobody else can use -- see prior point).
#  - Armies can accept commands from multiple command sources.
#
function FindAlliedClients()
    local t = GetArmiesTable()
    local focus = t.focusArmy
    local result = {}
    if focus == -1 then
        for index,client in GetSessionClients() do
            if table.getn(client.authorizedCommandSources) == 0 then
                table.insert(result, index)
            end
        end
    else
        local srcs = {}
        for army,info in t.armiesTable do
            if IsAlly(focus, army) then
                for k,cmdsrc in info.authorizedCommandSources do
                    srcs[cmdsrc] = true
                end
            end
        end
        for index,client in GetSessionClients() do
            for k,cmdsrc in client.authorizedCommandSources do
                if srcs[cmdsrc] then
                    table.insert(result, index)
                    break
                end
            end
        end
    end
    return result
end


function SetupChatLayout(mapGroup)
    parent = mapGroup
    local conFuncsList = false
    local raiseConFuncList = true

    SetLayout(UIUtil.currentLayout)

    OnlineProvider.RegisterChatDisplay(false)

    chatEdit.OnNonTextKeyPressed = function(self, charcode)
        local function ChangeConFuncSelection(direction)
            local selection = math.max(math.min(conFuncsList:GetSelection() + direction, conFuncsList:GetItemCount() - 1), 0)
            conFuncsList:SetSelection(selection)
            raiseConFuncList = false
            chatEdit:SetText("/"..conFuncsList:GetItem(selection))
            raiseConFuncList = true
        end
        if charcode == UIUtil.VK_NEXT then
            if chatInactivityTime > 0 then
                ChatPageDown(10)
            end
            return true
        elseif charcode == UIUtil.VK_PRIOR then
            if chatInactivityTime > 0 then
                ChatPageUp(10)
            end
            return true
        elseif charcode == UIUtil.VK_UP then
            if conFuncsList then
                if conFuncsList:GetItemCount() > 1 then
                    ChangeConFuncSelection(-1)
                else
                    if string.sub(chatEdit:GetText(),2) == conFuncsList:GetItem(0) then
                        -- store off the current text if this is the first press
                        local commandDequeSize = table.getn(commandDeque)
                        if commandDequeSize > 0 then
                            conFuncsList:Destroy()
                            conFuncsList = false
                            if currentCommandDequeIndex == commandDequeSize then currentText = chatEdit:GetText() end
                            chatEdit:SetText(commandDeque[currentCommandDequeIndex])
                            currentCommandDequeIndex = currentCommandDequeIndex - 1
                            if currentCommandDequeIndex <= 0 then currentCommandDequeIndex = 1 end
                        end
                    else
                        chatEdit:SetText("/"..conFuncsList:GetItem(0))
                    end
                end
            else
                -- store off the current text if this is the first press
                local commandDequeSize = table.getn(commandDeque)
                if commandDequeSize > 0 then
                    if currentCommandDequeIndex == commandDequeSize then currentText = chatEdit:GetText() end
                    chatEdit:SetText(commandDeque[currentCommandDequeIndex])
                    currentCommandDequeIndex = currentCommandDequeIndex - 1
                    if currentCommandDequeIndex <= 0 then currentCommandDequeIndex = 1 end
                end
            end
        elseif charcode == UIUtil.VK_DOWN then
            if conFuncsList then
                if conFuncsList:GetItemCount() > 1 then
                    ChangeConFuncSelection(1)
                else
                    currentCommandDequeIndex = currentCommandDequeIndex + 1
                    if currentCommandDequeIndex > table.getn(commandDeque) then
                        currentCommandDequeIndex = table.getn(commandDeque)
                        chatEdit:SetText(currentText)
                        conFuncsList:Destroy()
                        conFuncsList = false
                    else
                        conFuncsList:Destroy()
                        conFuncsList = false
                        chatEdit:SetText(commandDeque[currentCommandDequeIndex])
                    end
                end
            else
                currentCommandDequeIndex = currentCommandDequeIndex + 1
                if currentCommandDequeIndex > table.getn(commandDeque) then
                    currentCommandDequeIndex = table.getn(commandDeque)
                    chatEdit:SetText(currentText)
                else
                    chatEdit:SetText(commandDeque[currentCommandDequeIndex])
                end
            end
        else
            LOG(charcode)
            return false
        end
    end

    chatEdit.OnCharPressed = function(self, charcode)
        if charcode == UIUtil.VK_TAB then
            if conFuncsList then
                chatEdit:SetText("/"..conFuncsList:GetItem(conFuncsList:GetSelection()))
                return true
            else
                return false
            end
        end
    end

    chatEdit.OnEnterPressed = function(self, text)
        if chatDisplay then
            --this next section just removes /commmands from broadcasting.
            if string.sub(text, 1, 1) != "/" then
                if text == "" then
                    ToggleChat()
                    TimeoutChatDisplay()
                    chatEdit:AbandonFocus()
                    chatInactivityTime = .1
                else
                    msg = { to = chatToGroup }
                    local tauntid = tonumber(text)
                    if tauntid == nil or tauntid > 99 then
                        OnlineProvider.SendMessage("To "..chatToGroup..": "..text)
                        msg.text = text
                    else
                        local taunthead = Prefs.GetOption("mp_taunt_head") or "Arnold"
                        OnlineProvider.SendMessage("TAUNT:"..string.format("%02d", tauntid)..":"..taunthead)
                        msg.tauntid = tauntid
                        msg.taunthead = taunthead
                    end
                    if chatToGroup == 'Allies' then
                        SessionSendChatMessage(FindAlliedClients(), msg)
                    else
                        SessionSendChatMessage(msg)
                    end
                    InsertCommand(text)
                end
            else
                OnlineProvider.SendMessage(text)
                ConExecuteSave(string.sub(text,2))
                InsertCommand(text)
                if conFuncsList then
                    conFuncsList:Destroy()
                    conFuncsList = false
                end
            end
        end
    end

    group.OnDestroy = function(self)
        OnlineProvider.UnregisterChat()
    end

    group:SetNeedsFrameUpdate(true)
    group.OnFrame = function(self, deltaTime)
        UpdateFrames(deltaTime)
    end

    chatEdit.OnTextChanged = function(self, newText, oldText)
        if newText != "" then
            chatInactivityTime = CHAT_INACTIVITY_TIMEOUT
            group:Show()
        end
        if string.sub(newText, 1, 1) == "/" then
            if raiseConFuncList then
                if conFuncsList then
                    conFuncsList:Destroy()
                    conFuncsList = false
                end
                local matches = ConTextMatches(string.sub(newText,2))
                local numMatches = table.getn(matches)
                if (numMatches > 0) then
                    conFuncsList = ItemList(chatEdit)
                    conFuncsList:SetFont(consoleFontName, consoleFontSize)
                    LayoutHelpers.Above(conFuncsList, chatEdit)
                    conFuncsList.Right:Set(function() return chatEdit.Right() - 32 end)
                    conFuncsList.Height:Set(function()
                        return math.min(middleMiddle.Height(), conFuncsList:GetRowHeight() * numMatches)
                    end)
                    for i,v in matches do
                        conFuncsList:AddItem(v)
                    end
                    UIUtil.CreateVertScrollbarFor(conFuncsList)
                    conFuncsList:SetSelection(conFuncsList:GetItemCount() - 1)
                    conFuncsList:ScrollToBottom()

                    conFuncsList.OnDoubleClick = function(self, row)
                        chatEdit:SetText(conFuncsList:GetItem(row))
                        chatEdit:AcquireFocus()
                    end
                end
             end
        end
    end
end

function UpdateChatDisplay(numLines)
    for r = 1, 11 do
        if chatHistory[chatBottomLine - r] then
            chatDisplay[r]:SetText(chatHistory[chatBottomLine - r])
        else
            chatDisplay[r]:SetText("")
        end
    end

    if numLines then
        for index = 1, numLines do
            chatDisplay[index]:Show()
            chatDisplay[index].timeout = CHAT_INACTIVITY_TIMEOUT
        end
    else
        group:Show()
        group:SetAlpha(1, true)
        chatInactivityTime = CHAT_INACTIVITY_TIMEOUT
    end

    local chatStatusTemp2 = chatBottomLine - 1
    local chatStatusTemp1 = ""
    if chatBottomLine > 11 then
        chatStatusTemp1 = chatBottomLine - 11
    else
        chatStatusTemp1 = 1
    end
    chatStatus:SetText("Chat ("..chatStatusTemp1.." - "..chatStatusTemp2.." of "..table.getn(chatHistory).." lines)")
end

function TimeoutChatDisplay()
    for i, v in chatDisplay do
        chatDisplay[i].timeout = 0
    end
end

function UpdateFrames(deltaTime)
    if group:IsHidden() then
        for i, v in chatDisplay do
            if chatDisplay[i].timeout > 0 then
                chatDisplay[i].timeout = chatDisplay[i].timeout - deltaTime
                chatDisplay[i]:Show()
            else
                chatDisplay[i]:Hide()
            end
        end
    else
        chatInactivityTime = chatInactivityTime - deltaTime
        if chatInactivityTime < 0 then
            group:Hide()
            TimeoutChatDisplay()
            chatEdit:AbandonFocus()
        end
    end
end