### Lists the textures and colors used to draw the command graph
### waypoints and order lines.
###
### If any value is not supplied for a particular type of order but
### there is an 'inherit_from' key, the value is inherited from the
### order named by the inherit_from key.  If there is no inherit_from
### key, the 'default' entry is used.
###
### Note: because of the way order lines are rendered, they can't use
### the texture batcher, so having different textures for order lines
### is very expensive.  Only do it if necessary.

CommandGraphParams = {

    default = {
        orderline_texture = '/textures/ui/common/game/orderline/orderline_generic.dds',
        orderline_uv_aspect_ratio = 0.5,
        orderline_anim_rate = 0.05,
        orderline_color = '5530e030',
        orderline_selected_color = 'aa30e030',
        orderline_highlight_color = 'ff30e030',
        orderline_glow = 0.0,
        orderline_selected_glow = 0.12,
        orderline_highlight_glow = 0.06,

        waypoint_color = '99999999',
        waypoint_selected_color = 'aaaaaaaa',
        waypoint_highlight_color = 'ffffffff',

        waypoint_scale = 1,
        waypoint_selected_scale = 1,
        waypoint_highlight_scale = 1,
    },

    UNITCOMMAND_Move = {
        orderline_color = '5500d7ff',
        orderline_selected_color = 'aa00d7ff',
        orderline_highlight_color = 'ff00d7ff',
        waypoint_texture = '/textures/ui/common/game/waypoints/move_btn_up.dds',
        arrowhead_texture = '/textures/ui/common/game/orderline/orderline_arrow.dds',
    },
    UNITCOMMAND_FormMove = { inherit_from = 'UNITCOMMAND_Move' },
    UNITCOMMAND_CoordinatedMove = { inherit_from = 'UNITCOMMAND_Move' },

    UNITCOMMAND_AggressiveMove = {
        orderline_color = '55ffd700',
        orderline_selected_color = 'aaffd700',
        orderline_highlight_color = 'ffffd700',
        waypoint_texture = '/textures/ui/common/game/waypoints/move_btn_up.dds',
        arrowhead_texture = '/textures/ui/common/game/orderline/orderline_arrow.dds',
    },
    UNITCOMMAND_FormAggressiveMove = { inherit_from = 'UNITCOMMAND_AggressiveMove' },    

    UNITCOMMAND_Patrol = {
        orderline_uv_aspect_ratio = .3,
        orderline_color = '5500d7ff',
        orderline_selected_color = 'aa00d7ff',
        orderline_highlight_color = 'ff00d7ff',
        waypoint_texture = '/textures/ui/common/game/waypoints/patrol_btn_up.dds',
        arrowhead_texture = '/textures/ui/common/game/orderline/orderline_arrow.dds',
    },
    UNITCOMMAND_FormPatrol = { inherit_from = 'UNITCOMMAND_Patrol' },

    UNITCOMMAND_Ferry = {
        inherit_from = 'UNITCOMMAND_Patrol',
        orderline_color = '557d60ce',
        orderline_selected_color = 'aa7d60ce',
        orderline_highlight_color = 'ff7d60ce',
    },

    UNITCOMMAND_Attack = {
        orderline_color = '55ff0000',
        orderline_selected_color = 'aaff0000',
        orderline_highlight_color = 'ffff0000',

        waypoint_texture = '/textures/ui/common/game/waypoints/attack_btn_up.dds',
        arrowhead_texture = '/textures/ui/common/game/orderline/orderline_arrow.dds',
    },
    UNITCOMMAND_Retaliate = {
        inherit_from = 'UNITCOMMAND_Attack',
    },

    UNITCOMMAND_FormAttack = { inherit_from = 'UNITCOMMAND_Attack' },
    UNITCOMMAND_Nuke = {
        inherit_from = 'UNITCOMMAND_Attack',
        orderline_color = '55ff8408',
        orderline_selected_color = 'aaff8408',
        orderline_highlight_color = 'ffff8408',
        waypoint_texture = '/textures/ui/common/game/waypoints/nuke_btn_up.dds',
    },

    UNITCOMMAND_OverCharge = {
        inherit_from = 'default_cons',
        orderline_color = '55ff00f0',
        orderline_selected_color = 'aaff00f0',
        orderline_highlight_color = 'ffff00f0',

        waypoint_texture = '/textures/ui/common/game/waypoints/attack_btn_up.dds',
    },

    ### Default Construction
    default_cons = {
        orderline_color = '55ffe155',
        orderline_selected_color = 'aaffe155',
        orderline_highlight_color = 'ffffe155',
    },

    UNITCOMMAND_BuildSilo = { inherit_from = 'default_cons' },
    UNITCOMMAND_BuildFactory = { inherit_from = 'default_cons' },
    UNITCOMMAND_BuildMobile = { inherit_from = 'default_cons' },
    UNITCOMMAND_BuildAssist = { inherit_from = 'default_cons' },

    UNITCOMMAND_Reclaim = {
        inherit_from = 'default_cons',
        waypoint_texture = '/textures/ui/common/game/waypoints/reclaim_btn_up.dds',
    },

    UNITCOMMAND_Repair = {
        inherit_from = 'default_cons',
        waypoint_texture = '/textures/ui/common/game/waypoints/repair_btn_up.dds',
    },

    UNITCOMMAND_Sacrifice = {
        inherit_from = 'default_cons',
        waypoint_texture = '/textures/ui/common/game/waypoints/repair_btn_up.dds',
    },

    UNITCOMMAND_Capture = {
        inherit_from = 'default_cons',
        waypoint_texture = '/textures/ui/common/game/waypoints/convert_btn_up.dds',
    },

    ### Default Transport
    default_trans = {
        orderline_color = '557d60ce',
        orderline_selected_color = 'aa7d60ce',
        orderline_highlight_color = 'ff7d60ce',
        waypoint_texture = '/textures/ui/common/game/waypoints/load_btn_up.dds',
    },

    UNITCOMMAND_TransportLoadUnits = { inherit_from = 'default_trans' },
    UNITCOMMAND_TransportReverseLoadUnits = { inherit_from = 'default_trans' },
    UNITCOMMAND_TransportUnloadUnits = {
        inherit_from = 'default_trans',
        waypoint_texture = '/textures/ui/common/game/waypoints/unload_btn_up.dds',
    },
    UNITCOMMAND_TransportUnloadSpecificUnits = {
        inherit_from = 'default_trans',
        waypoint_texture = '/textures/ui/common/game/waypoints/unload_btn_up.dds',
    },
    UNITCOMMAND_DetachFromTransport = {
        inherit_from = 'default_trans',
        waypoint_texture = '/textures/ui/common/game/waypoints/unload_btn_up.dds',
    },

    UNITCOMMAND_Teleport = {
        orderline_color = '557d60ce',
        orderline_selected_color = 'aa7d60ce',
        orderline_highlight_color = 'ff7d60ce',
        waypoint_texture = '/textures/ui/common/game/waypoints/patrol_btn_up.dds',
    },

    UNITCOMMAND_Guard = {
        orderline_color = '55989898',
        orderline_selected_color = 'aa989898',
        orderline_highlight_color = 'ff989898',
        waypoint_texture = '/textures/ui/common/game/waypoints/guard_btn_up.dds',
    },

    ### These are here if they need to be used in the future
    ### They are not displayed right now
    UNITCOMMAND_Stop = { inherit_from = 'default' },
    UNITCOMMAND_Land = { inherit_from = 'default' },
    UNITCOMMAND_Dive = { inherit_from = 'default' },
    UNITCOMMAND_Upgrade = { inherit_from = 'default' },
    UNITCOMMAND_Enhance = { inherit_from = 'default' },
    UNITCOMMAND_AssistCommander = { inherit_from = 'default' },
    UNITCOMMAND_KillSelf = { inherit_from = 'default' },
    UNITCOMMAND_DestroySelf = { inherit_from = 'default' },

}
