--*****************************************************************************
--* File: lua/modules/ui/game/commandmode.lua
--* Author: Chris Blackwell
--* Summary: Manages the current command mode, which determines what action
--* the mouse will take when next clicked in the world
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************
local Dragger = import('/lua/modules/maui/dragger.lua').Dragger
local Construction = import('/lua/modules/ui/game/construction.lua')

-- these are the strings which represent a command mode
commandModeTable = {
    "order",
    "build",
}

-- these strings come from the cpp code in UnitBP, don't change them please!
orderModeTable = {
    "RULEUCC_Nuke",
    "RULEUCC_Tactical",
    "RULEUCC_Reclaim",
    "RULEUCC_Move",
    "RULEUCC_Stop",
    "RULEUCC_Teleport",
    "RULEUCC_Capture",
    "RULEUCC_Attack",
    "RULEUCC_Transport",
    "RULEUCC_Repair",
    "RULEUCC_Patrol",
    "RULEUCC_Guard",
    "RULEUCC_CallTransport",
    "RULEUCC_Ferry",
    "RULEUCC_SiloBuildTactical",
    "RULEUCC_SiloBuildNuke",
    "RULEUCC_Sacrifice",
	"RULEUCC_Pause",
	"RULEUCC_Overcharge",
	"RULEUCC_Dive",
}

local commandMode = false
local modeData = false
local issuedOneCommand = false

local startBehaviors = {}
local endBehaviors = {}

function OnCommandModeBeat()
	if issuedOneCommand and not IsKeyDown('Shift') then
		EndCommandMode(true)
	end
end

import('/lua/modules/ui/game/gamemain.lua').AddBeatFunction(OnCommandModeBeat)

-- behaviors are functions that take a single string parameter, the commandMode (or false if none)
function AddStartBehavior(behavior)
    table.insert(startBehaviors, behavior)
end

function AddEndBehavior(behavior)
    table.insert(endBehaviors, behavior)
end

function StartCommandMode(newCommandMode, data)
	#LOG('startCommandMode: ', newCommandMode)
    if commandMode then
        EndCommandMode(false)
    end
    commandMode = newCommandMode
    modeData = data
    for i,v in startBehaviors do
        v(commandMode, modeData)
    end
end

function GetCommandMode()
    return {commandMode, modeData}
end

function EndCommandMode(clearModifier)
    #LOG('EndCommandMode ', clearModifier)
    for i,v in endBehaviors do
        v(commandMode, modeData)
    end

    commandMode = false
    modeData = false
	issuedOneCommand = false

	if clearModifier then
   	    Construction.ClearWhatIfModifier()
   	end
end

function OnCommandIssued(command)
    #LOG('Command Issued (mode=',commandMode,'): ',repr(command))
    if not command.Clear then
		issuedOneCommand = true
	else
		EndCommandMode(true)
	end
end
