--*****************************************************************************
--* File: lua/modules/ui/game/confirmunitxfer.lua
--* Author: Ted Snook
--* Summary: UI dialog for Transfering units
--*
--* Copyright  2006 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Button = import('/lua/modules/maui/button.lua').Button
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Text = import('/lua/modules/maui/text.lua').Text

-- creates the actual dialog
--- parent is the calling window
--- targetPlayer should be the player the units will be transferred to

function createUnitTransferDialog(targetPlayer, targetPlayerName)
    local layout = UIUtil.SkinnableFile('/dialogs/diplomacy-unit-transfer/diplomacy-unit-transfer_layout.lua')

    local worldView = import('/lua/modules/ui/game/worldview.lua').view
    dialog = Bitmap(worldView, UIUtil.SkinnableFile('/dialogs/diplomacy-unit-transfer/team-panel_bmp.png'))
    dialog:SetRenderPass(UIUtil.UIRP_PostGlow)  -- just in case our parent is the map
    dialog:SetName("Unit Transfer")
    LayoutHelpers.AtCenterIn(dialog, worldView)

    local dialogTitle = UIUtil.CreateText(dialog, "<LOC allyui_0001>Unit Transfer", 18, UIUtil.titleFont )
    dialogTitle:SetColor( UIUtil.dialogCaptionColor )
    LayoutHelpers.RelativeTo(dialogTitle, dialog, layout, "l_verify", "team-panel_bmp")

    local localizedText = LOCF( "<LOC unitxfer_0000>Give Selected Units to %s?", targetPlayerName )
    local message = UIUtil.CreateText(dialog, nil, 12, UIUtil.bodyFont )
    message:SetText(localizedText)
    LayoutHelpers.RelativeTo(message, dialog, layout, "l_unit bla bla", "team-panel_bmp")

    # overridden by caller
    dialog.OnOk = function()
        LOG('You should probably override OnOk()')
    end

    # overriden by caller
    dialog.OnCancel = function()
        LOG('You should probably override OnCancel()')
    end

    local okbutton = UIUtil.CreateDialogButtonStd(dialog, "/dialogs/standard_btn/standard", "<LOC _Ok>", 12)
    LayoutHelpers.RelativeTo(okbutton, dialog, layout, "l_standard_btn_up", "team-panel_bmp")
    okbutton.OnClick = function(self, modifiers)
        dialog.OnOk()
    end

    local cancelButton = UIUtil.CreateDialogButtonStd(dialog, "/dialogs/standard_btn/standard", "<LOC _Cancel>", 12)
    LayoutHelpers.RelativeTo(cancelButton, dialog, layout, "l_cancel_btn", "team-panel_bmp")
    cancelButton.OnClick = function(self, modifiers)
        dialog.OnCancel()
    end

    return dialog
end
