--*****************************************************************************
--* File: lua/modules/ui/game/diplomacy.lua
--* Summary: UI for the diplomacy control
--*
--* Copyright  2006 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local EffectHelpers = import('/lua/modules/maui/effecthelpers.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Text = import('/lua/modules/maui/text.lua').Text
local Checkbox = import('/lua/modules/maui/Checkbox.lua').Checkbox
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Button = import('/lua/modules/maui/button.lua').Button
local GameCommon = import('/lua/modules/ui/game/gamecommon.lua')
local Tooltip = import('/lua/modules/ui/game/tooltip.lua')
local Options = import('/lua/modules/ui/game/options.lua')

dialog = false

local uefTexture = UIUtil.UIFile('/widgets/faction-icons_bmp/uef_ico.dds')
local cybranTexture = UIUtil.UIFile('/widgets/faction-icons_bmp/cybran_ico.dds')
local aeonTexture = UIUtil.UIFile('/widgets/faction-icons_bmp/aeon_ico.dds')
local observerTexture = UIUtil.UIFile('/widgets/faction-icons_bmp/observer_ico.dds')
local PlayerLines = {}
local NeverAllyWith = {}
local lastFocus = -1
local lockTeams = false


# Process an alliance request (could be between any two players)
function OnAllianceQuery( queryData )

    # I only care if someone is asking me to ally.
    if queryData.To == GetFocusArmy() then

        # This query was intended for me. Show the accept/reject dialog if
        # the requester isn't in my ignore list
        local resultData = {
            Name = queryData.Name,
            To = queryData.From,
            From = queryData.To,
        }

        function SendResult(val)
            if val == 'never' then
                table.insert(NeverAllyWith,resultData.To)
            end
            resultData.ResultValue = val
            import('/lua/UserPlayerQuery.lua').SendResult(resultData)
        end

        for k,v in NeverAllyWith do
            if v == queryData.From then
                SendResult('never')
                return
            end
        end

        import('/lua/modules/ui/game/allianceOffer.lua').OfferAlliance(queryData,SendResult)
    end
end

# Process alliance request result (could be between any two players)
function OnQueryResult(resultData)
    # Only process this if the result was directed at me.
    if resultData.To == GetFocusArmy() then
        for k,v in PlayerLines do
            if v.armyIndex == resultData.From then
                v.check.OfferPending = false
            end
        end
    end
end

function UpdateState()
    local focus = GetFocusArmy()

    if lastFocus != focus then
        LayoutPlayerLines()
        lastFocus = focus
    end

    for index,playerLine in PlayerLines do
        if index == focus or focus == -1 then
            playerLine:SetParent(dialog.HiddenGroup)
            playerLine:Hide()
        else
            playerLine:SetParent(dialog)
            playerLine:SetAlpha(1.0,true)
            playerLine:Show()
            if IsAlly(focus,index) then
                playerLine.check:SetCheck(true,true)
                playerLine.unit:Enable()
                playerLine.resource:Enable()
            else
                playerLine.check:SetCheck(false,true)
                playerLine.unit:Disable()
                playerLine.resource:Disable()
            end

            if playerLine.check.OfferPending or lockTeams then
                playerLine.check:Disable()
            end
        end
    end

    if lockTeams then
        dialog.avCheck:SetCheck(true,true)
        dialog.avCheck:Disable()
        dialog.avTitle:Disable()
    end
end

function LayoutPlayerLines()
    local prevLine = nil
    local barOffset = 0
    local layoutFile = UIUtil.SkinnableFile('/dialogs/diplomacy/diplomacy_layout.lua')

    for index,v in PlayerLines do
        local playerLine = v
        if not prevLine then
            LayoutHelpers.RelativeTo(playerLine, dialog, layoutFile, "l_text-and icons", "diplomacy-panel_bmp")
            LayoutHelpers.RelativeTo(playerLine.bar, dialog, layoutFile, "diplomacy-bar02_bmp", "diplomacy-panel_bmp")

            LayoutHelpers.RelativeTo(playerLine.colorBG, dialog, layoutFile, "l_factional-icon", "diplomacy-panel_bmp")

            barOffset = playerLine.colorBG.Top() - playerLine.bar.Top()

            LayoutHelpers.FillParent(playerLine.icon, playerLine.colorBG)
            LayoutHelpers.RelativeTo(playerLine.name, dialog, layoutFile, "l_player-1", "diplomacy-panel_bmp")
            LayoutHelpers.RelativeTo(playerLine.check, dialog, layoutFile, "l_check-box", "diplomacy-panel_bmp")
            LayoutHelpers.RelativeTo(playerLine.unit, dialog, layoutFile, "l_give-unit_btn_up", "diplomacy-panel_bmp")
            LayoutHelpers.RelativeTo(playerLine.resource, dialog, layoutFile, "l_give-resource_btn _up", "diplomacy-panel_bmp")
        else
            LayoutHelpers.Below(playerLine, prevLine)
            LayoutHelpers.Below(playerLine.bar, prevLine.bar)

            playerLine.colorBG.Left:Set( prevLine.colorBG.Left )
            playerLine.colorBG.Top:Set( function() return playerLine.bar.Top() + barOffset end )

            playerLine.name.Left:Set( prevLine.name.Left )
            playerLine.name.Top:Set( playerLine.colorBG.Top )

            LayoutHelpers.FillParent(playerLine.icon, playerLine.colorBG)

            playerLine.check.Left:Set( prevLine.check.Left )
            playerLine.check.Top:Set( playerLine.colorBG.Top )

            playerLine.unit.Left:Set( prevLine.unit.Left )
            playerLine.unit.Top:Set( playerLine.colorBG.Top )

            playerLine.resource.Left:Set( prevLine.resource.Left )
            playerLine.resource.Top:Set( playerLine.colorBG.Top )
        end

        playerLine.colorBG.Depth:Set( function() return playerLine.bar.Depth() + 1 end )
        playerLine.icon.Depth:Set( function() return playerLine.colorBG.Depth() + 1 end )
        playerLine.name.Depth:Set( function() return playerLine.bar.Depth() + 1 end )

        if index != GetFocusArmy() then
            prevLine = playerLine
        end
    end
end

function CreatePlayerLines()
    local layoutFile = UIUtil.SkinnableFile('/dialogs/diplomacy/diplomacy_layout.lua')
    local layout = import(layoutFile()).layout
    local armies = GetArmiesTable().armiesTable

    local nonCivilian = {}
    for k,v in armies do
        if not v.civilian then
            table.insert(nonCivilian,v)
        end
    end

    for index, player in nonCivilian do
        local playerLine = Group(dialog, "Player Line: " .. player.name)
        playerLine.Width:Set( layout['l_text-and icons'].width )
        playerLine.Height:Set( layout['l_text-and icons'].height )

        playerLine.armyIndex = index

        playerLine.bar = Bitmap(playerLine, UIUtil.SkinnableFile('/dialogs/diplomacy/diplomacy-bar02_bmp.dds'))

        playerLine.colorBG = Bitmap(playerLine)
        playerLine.colorBG:SetSolidColor(player.color)
        playerLine.colorBG.Width:Set(20)
        playerLine.colorBG.Height:Set(15)

        playerLine.icon = Bitmap(playerLine.colorBG,UIUtil.UIFile(UIUtil.GetFactionIcon(player.faction)))

        playerLine.colorBG:DisableHitTest(true)

        playerLine.name = UIUtil.CreateText(playerLine, player.nickname, 12, UIUtil.bodyFont )

        playerLine.check = UIUtil.CreateCheckboxStd(playerLine,'/dialogs/check-box_btn/radio')
        playerLine.check.playerName = player.nickname
        playerLine.check.armyIndex = index
        playerLine.check.HandleEvent = function(self, event)
            if event.Type == 'MouseEnter' then
                Tooltip.CreateMouseoverDisplay(self, "dip_send_alliance", nil, true)
            elseif event.Type == 'MouseExit' then
                Tooltip.DestroyMouseoverDisplay()
            end
            Checkbox.HandleEvent(self, event)
        end

        playerLine.check.OnCheck = function(self, checked)
            if checked then
                self:Hide()
                local queryArgs = {}
                queryArgs.Name = "OfferAlliance"
                queryArgs.Text = LOCF( "<LOC dipui_0001>%s offers you an alliance.", armies[GetFocusArmy()].nickname )
                queryArgs.From = GetFocusArmy()
                queryArgs.To = self.armyIndex

                playerLine.check.OfferPending = true
                import('/lua/UserPlayerQuery.lua').Query( queryArgs, OnQueryResult )
            else
                SimCallback( { Func="BreakAlliance", Args = { From = GetFocusArmy(), To = self.armyIndex } } )
            end
        end

        playerLine.unit = UIUtil.CreateButtonStd(playerLine, '/dialogs/diplomacy_btn/give-unit')
        playerLine.unit.HandleEvent = function(self, event)
            if event.Type == 'MouseEnter' then
                Tooltip.CreateMouseoverDisplay(self, "Give Units")
            end
            if event.Type == 'MouseExit' then
                Tooltip.DestroyMouseoverDisplay()
            end
            Button.HandleEvent(self, event)
        end
        playerLine.unit.OnClick = function(self, modifiers)

            dialog:Hide()
            dialog.childDialog = import('/lua/modules/ui/game/confirmunitxfer.lua').createUnitTransferDialog(playerLine.armyIndex, playerLine.check.playerName)

            dialog.childDialog.OnOk = function()
                SimCallback( { Func="GiveUnitsToPlayer",
                               Args={ From=GetFocusArmy(), To=playerLine.armyIndex},
                              },
                              true
                            )

                dialog.childDialog:Destroy()
                dialog.childDialog = nil
                dialog:Show()
            end

            dialog.childDialog.OnCancel = function()
                dialog.childDialog:Destroy()
                dialog.childDialog = nil
                dialog:Show()
            end
        end

        playerLine.resource = UIUtil.CreateButtonStd(playerLine, '/dialogs/diplomacy_btn/give-resource')
        playerLine.resource.HandleEvent = function(self, event)
            if event.Type == 'MouseEnter' then
                Tooltip.CreateMouseoverDisplay(self, "Give Resources")
            end
            if event.Type == 'MouseExit' then
                Tooltip.DestroyMouseoverDisplay()
            end
            Button.HandleEvent(self, event)
        end
        playerLine.resource.OnClick = function(self, modifiers)
            dialog:Hide()
            dialog.childDialog = import('/lua/modules/ui/game/shareResources.lua').createShareResourcesDialog(playerLine.armyIndex, playerLine.check.playerName)

            dialog.childDialog.OnOk = function(mass,energy)
                SimCallback( { Func="GiveResourcesToPlayer",
                               Args={ From=GetFocusArmy(),
                                      To=playerLine.armyIndex,
                                      Mass=mass,
                                      Energy=energy,
                                    }
                              }
                           )

                dialog.childDialog:Destroy()
                dialog.childDialog = nil
                dialog:Show()
            end

            dialog.childDialog.OnCancel = function()
                dialog.childDialog:Destroy()
                dialog.childDialog = nil
                dialog:Show()
            end
        end

        table.insert(PlayerLines, playerLine)
    end

    LayoutPlayerLines()
end

function CreateDiplomacyControl()
    lastFocus = GetFocusArmy()

    local worldView = import('/lua/modules/ui/game/worldview.lua').view
    local layout = UIUtil.SkinnableFile('/dialogs/diplomacy/diplomacy_layout.lua')
    dialog = Bitmap(worldView, UIUtil.SkinnableFile('/dialogs/diplomacy/diplomacy-panel_bmp.dds'))
    dialog:SetRenderPass(UIUtil.UIRP_PostGlow)  -- just in case our parent is the map
    dialog:SetName("Diplomacy Window")
    LayoutHelpers.AtCenterIn(dialog, worldView)

    dialog.HiddenGroup = Group(worldView,"hidden player lines")
    dialog.HiddenGroup:Hide()

    dialog.dialogTitle = UIUtil.CreateText(dialog, "<LOC _Diplomacy>", 18, UIUtil.titleFont )
    dialog.dialogTitle:SetColor( UIUtil.dialogCaptionColor )
    LayoutHelpers.RelativeTo(dialog.dialogTitle, dialog, layout, "l_diplomacy", "diplomacy-panel_bmp")

    dialog.playerTitle = UIUtil.CreateText(dialog, "<LOC _Player>", 14, UIUtil.titleFont )
    dialog.playerTitle:SetColor( UIUtil.dialogColumnColor )
    LayoutHelpers.RelativeTo(dialog.playerTitle, dialog, layout, "l_player", "diplomacy-panel_bmp")

    dialog.teamTitle = UIUtil.CreateText(dialog, "<LOC _Team>", 14, UIUtil.titleFont)
    dialog.teamTitle:SetColor( UIUtil.dialogColumnColor )
    LayoutHelpers.RelativeTo(dialog.teamTitle, dialog, layout, "l_team", "diplomacy-panel_bmp")

    dialog.shareTitle = UIUtil.CreateText(dialog, "<LOC _Share>", 14, UIUtil.titleFont)
    dialog.shareTitle:SetColor( UIUtil.dialogColumnColor )
    LayoutHelpers.RelativeTo(dialog.shareTitle, dialog, layout, "l_share", "diplomacy-panel_bmp")

    dialog.resourceShareTitle = UIUtil.CreateText(dialog, "<LOC dipui_0002>Share Resources", 12, UIUtil.bodyFont)
    dialog.resourceShareTitle:SetColor( UIUtil.dialogColumnColor )
    LayoutHelpers.RelativeTo(dialog.resourceShareTitle, dialog, layout, "l_share-resources", "diplomacy-panel_bmp")

    dialog.setShareCheck = UIUtil.CreateCheckboxStd(dialog,'/dialogs/check-box_btn/radio')
    LayoutHelpers.RelativeTo(dialog.setShareCheck, dialog, layout, 'l_share-resources-check', 'diplomacy-panel_bmp')
    dialog.setShareCheck:SetCheck( GetResourceSharing() )
    dialog.setShareCheck.OnCheck = function(self,checked)
        SimCallback( {  Func = "SetResourceSharing",
                        Args = { Army = GetFocusArmy(),
                                 Value = checked,
                               }
                     }
                   )
    end

    dialog.avTitle = UIUtil.CreateText(dialog, "<LOC dipui_0000>Allied Victory", 12, UIUtil.bodyFont)
    dialog.avTitle:SetColor( UIUtil.dialogColumnColor )
    LayoutHelpers.RelativeTo(dialog.avTitle, dialog, layout, "l_allied-victory", "diplomacy-panel_bmp")

    dialog.avCheck = UIUtil.CreateCheckboxStd(dialog,'/dialogs/check-box_btn/radio')
    LayoutHelpers.RelativeTo(dialog.avCheck, dialog, layout, "l_checkbox", "diplomacy-panel_bmp")
    dialog.avCheck.OnCheck = function(self, checked)
        SimCallback( {  Func = "RequestAlliedVictory",
                        Args = { Army = GetFocusArmy(),
                                 Value = checked,
                               }
                     }
                   )
    end

    dialog.okButton = UIUtil.CreateDialogButtonStd(dialog, "/dialogs/standard_btn/standard", "<LOC _Ok>", 12)
    LayoutHelpers.RelativeTo(dialog.okButton, dialog, layout, "l_ok_btn_up", "diplomacy-panel_bmp")
    dialog.okButton.OnClick = function(self, modifiers)
        dialog:Hide()
    end

    CreatePlayerLines()

    dialog:Hide()
    dialog.OnHide = function(self,hidden)
        UpdateState()
    end


    # Start listening to alliance queries and results
    import('/lua/UserPlayerQuery.lua').AddQueryListener( "OfferAlliance", OnAllianceQuery )

    function OnAllianceResult( data )
        #LOG('ALLIANCE RESULT: ', repr(data))
        # Right now we don't keep track of alliances that don't involve us...
        # It sure would be nice to see who everyone is allied with. If we decide
        # to do so, this would be the place to listen in on changes.
    end

    import('/lua/UserPlayerQuery.lua').AddResultListener( "OfferAlliance", OnAllianceResult )

    ForkThread(
        function()
            while true do
                if dialog and not dialog:IsHidden() then
                    UpdateState()
                end
                WaitSeconds(0.5)
            end
        end
    )
end

function ToggleDialog()
    if dialog:IsHidden() then
        local sound = Sound({Cue = "UI_Diplomacy_Open", Bank = "Interface",})
        PlaySound(sound)
        EffectHelpers.FadeIn(dialog, .5)
        if not UIUtil.GetNetworkBool() then
            Options.PauseGame(true)
        end
    else
        local sound = Sound({Cue = "UI_Diplomacy_Close", Bank = "Interface",})
        PlaySound(sound)
        EffectHelpers.FadeOut(dialog, .5)
        if not UIUtil.GetNetworkBool() then
            Options.PauseGame(false)
        end
    end
end

function LockTeams()
    lockTeams = true
end

function CanShowAllianceOffer()
    return dialog and dialog:IsHidden() and not dialog.childDialog
end
