--*****************************************************************************
--* File: lua/modules/ui/game/economy.lua
--* Author: Chris Blackwell
--* Summary: Economy bar UI
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local StatusBar = import('/lua/modules/maui/statusbar.lua').StatusBar
local GameMain = import('/lua/modules/ui/game/gamemain.lua')
local Tooltip = import('/lua/modules/ui/game/tooltip.lua')
local Prefs = import('/lua/modules/ui/prefs.lua')
local Tooltip = import('/lua/modules/ui/game/tooltip.lua')

local massViewState = Prefs.GetFromCurrentProfile("massRateView") or 1
local energyViewState = Prefs.GetFromCurrentProfile("energyRateView") or 1
local UIState = true
local overallEfficiency = false
local hideWarnings = false
group = false

function Contract() 
    UIState = false
end

function Expand()
    UIState = true
end

function ToggleBuildEfficiency()
    if overallEfficiency:IsHidden() then
        overallEfficiency:Show()
    else
        overallEfficiency:Hide()
    end
end

function CreateRateBox(parent)
    local topLeft = Bitmap(parent, UIUtil.UIFile('/dialogs/drop-box_brd/drop-box_brd_ul.dds'))
    topLeft.Left:Set(parent.Left)
    topLeft.Bottom:Set(parent.Top)
    topLeft.Depth:Set(function() return parent.Depth() - 1 end)
    topLeft:DisableHitTest()
    
    local topRight = Bitmap(parent, UIUtil.UIFile('/dialogs/drop-box_brd/drop-box_brd_ur.dds'))
    topRight.Right:Set(parent.Right)
    topRight.Bottom:Set(parent.Top)
    topRight.Depth:Set(function() return parent.Depth() - 1 end)
    topRight:DisableHitTest()
    
    local bottomLeft = Bitmap(parent, UIUtil.UIFile('/dialogs/drop-box_brd/drop-box_brd_ll.dds'))
    bottomLeft.Left:Set(parent.Left)
    bottomLeft.Top:Set(parent.Bottom)
    bottomLeft.Depth:Set(function() return parent.Depth() - 1 end)
    bottomLeft:DisableHitTest()
    
    local bottomRight = Bitmap(parent, UIUtil.UIFile('/dialogs/drop-box_brd/drop-box_brd_lr.dds'))
    bottomRight.Right:Set(parent.Right)
    bottomRight.Top:Set(parent.Bottom)
    bottomRight.Depth:Set(function() return parent.Depth() - 1 end)
    bottomRight:DisableHitTest()
    
    local top = Bitmap(parent, UIUtil.UIFile('/dialogs/drop-box_brd/drop-box_brd_horz_um.dds'))
    top.Right:Set(topRight.Left)
    top.Left:Set(topLeft.Right)
    top.Bottom:Set(topLeft.Bottom)
    top.Depth:Set(function() return parent.Depth() - 1 end)
    top:DisableHitTest()
    
    local bottom = Bitmap(parent, UIUtil.UIFile('/dialogs/drop-box_brd/drop-box_brd_lm.dds'))
    bottom.Right:Set(bottomRight.Left)
    bottom.Left:Set(bottomLeft.Right)
    bottom.Top:Set(bottomLeft.Top)
    bottom.Depth:Set(function() return parent.Depth() - 1 end)
    bottom:DisableHitTest()
    
    local left = Bitmap(parent, UIUtil.UIFile('/dialogs/drop-box_brd/drop-box_brd_vert_l.dds'))
    left.Top:Set(topLeft.Bottom)
    left.Left:Set(topLeft.Left)
    left.Bottom:Set(bottomLeft.Top)
    left.Depth:Set(function() return parent.Depth() - 1 end)
    left:DisableHitTest()
    
    local right = Bitmap(parent, UIUtil.UIFile('/dialogs/drop-box_brd/drop-box_brd_vert_r.dds'))
    right.Top:Set(topRight.Bottom)
    right.Right:Set(topRight.Right)
    right.Bottom:Set(bottomRight.Top)
    right.Depth:Set(function() return parent.Depth() - 1 end)
    right:DisableHitTest()
end

function CreateEconomyBar(parent)
    group = Group(parent, "economyBarGroup")
    LayoutHelpers.FillParent(group, parent)
    
    
    -- mass setup
    local massBG = Bitmap(group, UIUtil.UIFile('/game/resources/mass-back_bmp.dds'))
    LayoutHelpers.AtLeftTopIn(massBG, group, -6, -1)

    overallEfficiency = UIUtil.CreateText(group, "100%", 20, UIUtil.titleFont)
    LayoutHelpers.AtHorizontalCenterIn(overallEfficiency, GetFrame(0))
    overallEfficiency.Top:Set(function() return massBG.Bottom() - 1 end)
    overallEfficiency.Depth:Set(100)
    overallEfficiency:Hide()
    overallEfficiency:DisableHitTest()
    
    local oeBG = Bitmap(overallEfficiency, UIUtil.UIFile('/game/build-efficiency/panel_bmp.dds'))
    LayoutHelpers.AtCenterIn(oeBG, overallEfficiency)
    oeBG.Depth:Set(function() return overallEfficiency.Depth() - 1 end)
    
    local massDropdown = Bitmap(massBG, UIUtil.UIFile('/game/resources/mass-advanced_bmp.dds'))
    LayoutHelpers.AtLeftTopIn(massDropdown, massBG, 327, 6)

    local massIncome = UIUtil.CreateText(massDropdown, "", 10, UIUtil.fixedFont)
    LayoutHelpers.AtHorizontalCenterIn(massIncome, massDropdown)
    LayoutHelpers.AtTopIn(massIncome, massDropdown, 5)
    massIncome:SetColor("green")

    local massExpense = UIUtil.CreateText(massDropdown, "", 10, UIUtil.fixedFont)
    LayoutHelpers.CenteredBelow(massExpense, massIncome, -1)
    massExpense:SetColor("red")

--[[
    local massEfficiency = UIUtil.CreateText(massDropdown, "", 14, UIUtil.fixedFont)
    massEfficiency.Right:Set(massDropdown.Right)
    massEfficiency.Top:Set(massExpense.Bottom)
--]]

    massIncome:DisableHitTest()
    massExpense:DisableHitTest()
--    massEfficiency:DisableHitTest()
    massDropdown:DisableHitTest()

    local massLabel = Bitmap(massBG, UIUtil.UIFile('/game/resources/mass_btn_up.dds'))
    LayoutHelpers.AtLeftTopIn(massLabel, massBG, 0, -5)
    massLabel.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "mass_button", .5, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
    end 

    local massStatus = StatusBar(massBG, 0, 100, false, false, UIUtil.UIFile('/game/resources/mass-bar-back_bmp.dds'), UIUtil.UIFile('/game/resources/mass-bar_bmp.dds'), false)
    LayoutHelpers.AtLeftTopIn(massStatus, massBG, 66, 13)
    massStatus:SetMinimumSlidePercentage(7/190)   --TODO some way to not hard code this! (it's the size of a tick in the status bar / size of the status bar)

    local massRateContainer = Group(massBG, "massRateContainter")
    massRateContainer.Left:Set(function() return massStatus.Right() + 2 end)
    massRateContainer.Top:Set(function() return massBG.Top() + 12 end)
    massRateContainer.Height:Set(20)
    massRateContainer.Right:Set(massDropdown.Left)
    
    CreateRateBox(massRateContainer)
    
    local massRate = UIUtil.CreateText(massBG, "", 20, UIUtil.bodyFont)
    LayoutHelpers.AtCenterIn(massRate, massRateContainer)
    massRate:DisableHitTest()
    
    local massWarning = UIUtil.CreateText(massBG, LOC("<LOC Econ_0000>Mass Needed"), 16, UIUtil.titleFont)
    massWarning.Depth:Set(100)
    LayoutHelpers.AtCenterIn(massWarning, massStatus, 2)
    massWarning:SetDropShadow(true)
    local massAscending = false
    local massMod = 1
    
    local massWarningBG = Bitmap(massWarning, UIUtil.UIFile('/game/orders/glow-02_bmp.dds'))
    massWarningBG.Depth:Set(function() return massWarning.Depth() - 1 end)
    LayoutHelpers.AtCenterIn(massWarningBG, massLabel)
    
    massWarning:Hide()
    
    local massCounter = 0
    local massFlashing = false
    massWarning.OnFrame = function(self, deltaTime)
        curAlpha = massWarning:GetAlpha()
        if massAscending then
            curAlpha = curAlpha + (deltaTime * massMod)
        else
            curAlpha = curAlpha - (deltaTime * massMod)
        end
        if curAlpha > 1 then
            massCounter = massCounter + 1
            massAscending = false
            curAlpha = 1
        end
        if curAlpha < 0 then
            massAscending = true
            curAlpha = 0
        end
        massWarning:SetAlpha(curAlpha)
        massWarningBG:SetAlpha(curAlpha*.4)
        if massCounter > 3 then
            massWarning:SetNeedsFrameUpdate(false)
            massWarningBG:SetAlpha(0)
            massCounter = 0
        end
    end
    
    massWarning:DisableHitTest(true)
    
    -- this exists only to act as a hit target for the dropdown data
    local massDropdownTarget = Group(massRate, "massDropdown")
    massDropdownTarget.Left:Set(massDropdown.Left)
    massDropdownTarget.Right:Set(massDropdown.Right) 
    massDropdownTarget.Top:Set(massBG.Top)
    massDropdownTarget.Bottom:Set(massBG.Bottom)
    
    local massStorage = UIUtil.CreateText(massStatus._bar, "", 10, UIUtil.bodyFont)
    massStorage.Left:Set(function() return massStatus.Left() end)
    massStorage.Top:Set(function() return massStatus.Bottom() end)
    
    local massMaxStorage = UIUtil.CreateText(massStatus._bar, "", 10, UIUtil.bodyFont)
    massMaxStorage.Right:Set(function() return massStatus.Right() end)
    massMaxStorage.Top:Set(function() return massStatus.Bottom() end)
    

    -- energy bar setup
    local energyBG = Bitmap(group, UIUtil.UIFile('/game/resources/energy-back_bmp.dds'))
    LayoutHelpers.AtRightIn(energyBG, group, -4)
    LayoutHelpers.AtTopIn(energyBG, group, -7)

    local energyDropdown = Bitmap(energyBG, UIUtil.UIFile('/game/resources/energy-advanced_bmp.dds'))
    LayoutHelpers.AtLeftTopIn(energyDropdown, energyBG, 20, 12)

    local energyIncome = UIUtil.CreateText(energyDropdown, "", 10, UIUtil.fixedFont)
    LayoutHelpers.AtHorizontalCenterIn(energyIncome, energyDropdown)
    LayoutHelpers.AtTopIn(energyIncome, energyDropdown, 5)
    energyIncome:SetColor("green")
   
    local energyExpense = UIUtil.CreateText(energyDropdown, "", 10, UIUtil.fixedFont)
    LayoutHelpers.CenteredBelow(energyExpense, energyIncome, -1)
    energyExpense:SetColor("red")

--[[
    local energyEfficiency = UIUtil.CreateText(energyDropdown, "", 14, UIUtil.fixedFont)
    energyEfficiency.Right:Set(energyDropdown.Right)
    energyEfficiency.Top:Set(energyExpense.Bottom)
--]]

    energyIncome:DisableHitTest()
    energyExpense:DisableHitTest()
--    energyEfficiency:DisableHitTest()
    energyDropdown:DisableHitTest()

    local energyLabel = Bitmap(energyBG, UIUtil.UIFile('/game/resources/energy_btn_up.dds'))
    LayoutHelpers.AtRightIn(energyLabel, energyBG, 5)
    LayoutHelpers.AtTopIn(energyLabel, energyBG, 3)
    energyLabel.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "energy_button", .5, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
    end 

    local energyStatus = StatusBar(energyBG, 0, 100, false, false, UIUtil.UIFile('/game/resources/energy-bar-back_bmp.dds'), UIUtil.UIFile('/game/resources/energy-bar_bmp.dds'), false)
    LayoutHelpers.AtLeftTopIn(energyStatus, energyBG, 139, 19)
    energyStatus:SetMinimumSlidePercentage(7/190)   --TODO some way to not hard code this! (it's the size of a tick in the status bar / size of the status bar)

    local energyStorage = UIUtil.CreateText(energyStatus._bar, "", 10, UIUtil.bodyFont)
    energyStorage.Left:Set(function() return energyStatus.Left() end)
    energyStorage.Top:Set(function() return energyStatus.Bottom() end)
    
    local energyMaxStorage = UIUtil.CreateText(energyStatus._bar, "", 10, UIUtil.bodyFont)
    energyMaxStorage.Right:Set(function() return energyStatus.Right() end)
    energyMaxStorage.Top:Set(function() return energyStatus.Bottom() end)

    local energyRateContainer = Group(energyBG, "energyRateContainer")
    energyRateContainer.Right:Set(function() return energyStatus.Left() - 5 end)
    energyRateContainer.Top:Set(function() return massBG.Top() + 12 end)
    energyRateContainer.Height:Set(20)
    energyRateContainer.Left:Set(function() return energyDropdown.Right() end)
    
    CreateRateBox(energyRateContainer)
    
    local energyRate = UIUtil.CreateText(energyBG, "", 20, UIUtil.bodyFont)
    LayoutHelpers.AtCenterIn(energyRate, energyRateContainer)
    energyRate:DisableHitTest()

    local energyWarning = UIUtil.CreateText(massBG, LOC("<LOC Econ_0001>Energy Needed"), 16, UIUtil.titleFont)
    energyWarning.Depth:Set(100)
    LayoutHelpers.AtCenterIn(energyWarning, energyStatus, 2)
    energyWarning:SetDropShadow(true)
    local energyAscending = false
    local energyMod = 1
    
    local energyWarningBG = Bitmap(energyWarning, UIUtil.UIFile('/game/orders/glow-02_bmp.dds'))
    energyWarningBG.Depth:Set(function() return energyWarning.Depth() - 1 end)
    LayoutHelpers.AtCenterIn(energyWarningBG, energyLabel)
    local energyCounter = 0
    local energyFlashing = false
    
    energyWarning:Hide()
    
    energyWarning.OnFrame = function(self, deltaTime)
        energyFlashing = true
        curAlpha = energyWarning:GetAlpha()
        if energyAscending then
            curAlpha = curAlpha + (deltaTime * energyMod)
        else
            curAlpha = curAlpha - (deltaTime * energyMod)
        end
        if curAlpha > 1 then
            energyCounter = energyCounter + 1
            energyAscending = false
            curAlpha = 1
        end
        if curAlpha < 0 then
            energyAscending = true
            curAlpha = 0
        end
        energyWarning:SetAlpha(curAlpha)
        energyWarningBG:SetAlpha(curAlpha*.4)
        if energyCounter > 3 then
            energyWarning:SetNeedsFrameUpdate(false)
            energyWarningBG:SetAlpha(0)
            energyCounter = 0
        end
    end
    
    energyWarning:DisableHitTest(true)
    
    -- this exists only to act as a hit target for the dropdown data
    local energyDropdownTarget = Group(energyRate, "energyDropdown")
    energyDropdownTarget.Right:Set(energyDropdown.Right)
    energyDropdownTarget.Left:Set(energyDropdown.Left)
    energyDropdownTarget.Top:Set(energyBG.Top)
    energyDropdownTarget.Bottom:Set(energyBG.Bottom)

    function _BeatFunction()
        local function CalculateEfficiency(income, expense)
            local efficiency = 0
            if income > expense then
                if income == 0 then
                    efficiency = false
                else
                    efficiency = (expense / income) * 100
                end
            elseif income < expense then
                if expense == 0 then
                    efficiency = false
                else
                    efficiency = (income / expense) * 100
                end
            end
            return efficiency
        end

        -- when negative, this is showing you how short you are, when positive shows how much extra you're making
        local function CalculateRate(income, requestedExpense, actualExpense, frequency)
            local retVal
            if actualExpense < requestedExpense then
                retVal = (actualExpense - requestedExpense) * frequency
            else
                retVal = (income - actualExpense) * frequency
            end
            return retVal
        end

        local econData = GetEconomyTotals()
        local simFrequency = GetSimTicksPerSecond()

        local massMaxStorageVal = econData["maxStorage"]["MASS"]
        local massStoredVal = econData["stored"]["MASS"]
        local massIncomeVal = econData["income"]["MASS"]
        local massLastRequestedVal = econData["lastUseRequested"]["MASS"]
        local massLastActualVal = econData["lastUseActual"]["MASS"]

        local massRequestedAvg = math.min(massLastRequestedVal * simFrequency, 99999)
        local massIncomeAvg = math.min(massIncomeVal * simFrequency, 99999)
        
        massStatus:SetRange(0, massMaxStorageVal)
        massStatus:SetValue(massStoredVal)
        massStorage:SetText(string.format("%d", massStoredVal))
        massMaxStorage:SetText(string.format("%d", massMaxStorageVal))
        
        massIncome:SetText(string.format("%d", math.floor( math.min(massIncomeVal * simFrequency, 99999) + 0.5 ) ))
        massExpense:SetText(string.format("%d", math.floor( math.min(massLastRequestedVal * simFrequency, 99999) + 0.5 ) ))
        
        local massRateVal = math.floor( math.min(CalculateRate(massIncomeVal, massLastActualVal, massLastRequestedVal, simFrequency), 99999) + 0.5 )
        massRateVal = math.floor( massRateVal + 0.5 )
        
        local massEffNumber = false
        if massRequestedAvg == 0 then
            massEffNumber = math.ceil(massIncomeAvg) * 100
        else
            massEffNumber = math.ceil((massIncomeAvg / massRequestedAvg) * 100)
        end
        if massViewState == 1 then
            massRate:SetText(string.format("%+d", massRateVal))
        elseif massViewState == 2 then
            massRate:SetText(string.format("%d%%", massEffNumber))
        #elseif massViewState == 3 then
        #    massRateVal = CalculateRate(massIncomeVal, massLastRequestedVal, massLastActualVal, simFrequency)
        #    massRate:SetText(string.format("%+d", massRateVal))
        end

        local massEfficiencyVal = CalculateEfficiency(massIncomeVal, massLastRequestedVal)

        local massRateColor
        if massRateVal < 0 then
            if massStoredVal > 0 then
                massRateColor = 'yellow'
            else
                massRateColor = 'red'
            end
        elseif massRateVal > 0 then
            if massStoredVal < massMaxStorageVal then
                massRateColor = 'green'
            else
                massRateColor = 'yellow'
            end
        else
            massRateColor = 'green'
        end
--        massEfficiency:SetColor(massRateColor)
        massRate:SetColor(massRateColor)

--        if massEfficiencyVal > 9999999 then
--            massEfficiency:SetText(">9999999%")
--        else
--            massEfficiency:SetText(string.format(" %8d%%", massEfficiencyVal))
--        end
        
        local energyMaxStorageVal = econData["maxStorage"]["ENERGY"]
        local energyStoredVal = econData["stored"]["ENERGY"]
        local energyIncomeVal = econData["income"]["ENERGY"]
        local energyLastRequestedVal = econData["lastUseRequested"]["ENERGY"]
        local energyLastActualVal = econData["lastUseActual"]["ENERGY"]

        local energyRequestedAvg = math.min(energyLastRequestedVal * simFrequency, 99999)
        local energyIncomeAvg = math.min(energyIncomeVal * simFrequency, 99999)
        
        energyStatus:SetRange(0, energyMaxStorageVal)
        energyStatus:SetValue(energyStoredVal)
        energyStorage:SetText(string.format("%d", energyStoredVal))
        energyMaxStorage:SetText(string.format("%d", energyMaxStorageVal))

        energyIncome:SetText(string.format("%d", math.floor( math.min(energyIncomeVal * simFrequency, 99999) + 0.5 ) ))
        
        energyExpense:SetText(string.format("%d", math.floor( math.min(energyLastRequestedVal * simFrequency, 99999) + 0.5 ) ))

        local energyRateVal = math.floor( math.min(CalculateRate(energyIncomeVal, energyLastActualVal, energyLastRequestedVal, simFrequency), 99999) + 0.5 )

        local energyEffNumber = false
        if energyRequestedAvg == 0 then
            energyEffNumber = math.floor(energyIncomeAvg) * 100
        else
            energyEffNumber = math.floor((energyIncomeAvg / energyRequestedAvg) * 100)
        end
        
        if energyViewState == 1 then
            energyRate:SetText(string.format("%+d", energyRateVal))
        elseif energyViewState == 2 then
            energyRate:SetText(string.format("%d%%", energyEffNumber))
        #elseif energyViewState == 3 then
        #    energyRateVal = CalculateRate(massIncomeVal, massLastRequestedVal, massLastActualVal, simFrequency)
        #    energyRate:SetText(string.format("%+d", energyRateVal))
        end
        local energyEfficiencyVal = CalculateEfficiency(energyIncomeVal, energyLastRequestedVal)
        
        local energyRateColor
        if energyRateVal < 0 then
            if energyStoredVal > 0 then
                energyRateColor = 'yellow'
            else
                energyRateColor = 'red'
            end
        elseif energyRateVal > 0 then
            if energyStoredVal < energyMaxStorageVal then
                energyRateColor = 'green'
            else
                energyRateColor = 'yellow'
            end
        else
            energyRateColor = 'green'
        end
        
        # Set the overall efficiency value:
        local tempMassEff = math.ceil((massIncomeAvg / massRequestedAvg) * 100)
        local tempEnergyEff = math.ceil((energyIncomeAvg / energyRequestedAvg) * 100)
        local tempOverallNum = math.min(math.min(tempMassEff, tempEnergyEff), 100)
        overallEfficiency:SetText(string.format("%d%%", tempOverallNum))
        if tempOverallNum < 25 then
            overallEfficiency:SetColor('red')
        elseif tempOverallNum <= 75 then
            overallEfficiency:SetColor('yellow')
        elseif tempOverallNum > 75 then
            overallEfficiency:SetColor('green')
        end
        
        if Prefs.GetOption('econ_warnings') and not hideWarnings then
            if UIState then
                if massStoredVal / massMaxStorageVal < .2 then
                    if not massFlashing then
                        if massEffNumber < 25 then
                            massWarning:SetNeedsFrameUpdate(true)
                            massFlashing = true
                            massWarning:Show()
                            massMod = 2
                            massWarning:SetColor('red')
                        elseif massEffNumber < 75 then
                            massWarning:SetNeedsFrameUpdate(true)
                            massFlashing = true
                            massWarning:Show()
                            massMod = 1
                            massWarning:SetColor('yellow')
                        elseif massEffNumber <= 100 then
                            massWarning:SetNeedsFrameUpdate(true)
                            massFlashing = true
                            massWarning:Show()
                            massMod = .5
                            massWarning:SetColor('green')
                        elseif massEffNumber > 100 then
                            massWarning:SetNeedsFrameUpdate(false)
                            massFlashing = false
                            massWarning:Hide()
                        end
                    end
                else
                    massWarning:SetNeedsFrameUpdate(false)
                    massFlashing = false
                    massWarning:Hide()
                end
                
                if energyStoredVal / energyMaxStorageVal < .2 then
                    if not energyFlashing then
                        if energyEffNumber < 25 then
                            energyWarning:SetNeedsFrameUpdate(true)
                            energyFlashing = true
                            energyWarning:Show()
                            energyMod = 2
                            energyWarning:SetColor('red')
                        elseif energyEffNumber < 75 then
                            energyWarning:SetNeedsFrameUpdate(true)
                            energyFlashing = true
                            energyMod = 1
                            energyWarning:Show()
                            energyWarning:SetColor('yellow')
                        elseif energyEffNumber <= 100 then
                            energyWarning:SetNeedsFrameUpdate(true)
                            energyFlashing = true
                            energyMod = .5
                            energyWarning:Show()
                            energyWarning:SetColor('green')
                        elseif energyEffNumber > 100 then
                            energyFlashing = false
                            energyWarning:SetNeedsFrameUpdate(false)
                            energyWarning:Hide()
                        end
                    end
                else
                    energyFlashing = false
                    energyWarning:SetNeedsFrameUpdate(false)
                    energyWarning:Hide()
                end
            else
                massWarning:Hide()
                energyWarning:Hide()
            end
        else
            massWarning:Hide()
            energyWarning:Hide()
        end
            
        
--        energyEfficiency:SetColor(energyRateColor)
        energyRate:SetColor(energyRateColor)

--        if energyEfficiencyVal > 9999999 then
--            energyEfficiency:SetText(">9999999%")
--        else
--            energyEfficiency:SetText(string.format(" %8d%%", energyEfficiencyVal))
--        end
    end

    GameMain.AddBeatFunction(_BeatFunction)

    group.OnDestroy = function(self)
        GameMain.RemoveBeatFunction(_BeatFunction)
    end
    
    local massDropdownVisible = Prefs.GetFromCurrentProfile("massDetailedView") or false
    local energyDropdownVisisble = Prefs.GetFromCurrentProfile("energyDetailedView") or false
    
    energyDropdown:Hide()
    massDropdown:Hide()
    
    if massDropdownVisible then
        massDropdown:Show()
    end
    
    if energyDropdownVisisble then
        energyDropdown:Show()
    end
    
    overallEfficiency.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "overall", .5, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
    end
    
    massStorage.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(massLabel, "storage", .5, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
    end
    
    massMaxStorage.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(massLabel, "storage", .5, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
    end
    
    massStatus.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(massLabel, "storage", .5, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
    end 
    
    energyStorage.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(energyLabel, "storage", .5, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
    end
    
    energyMaxStorage.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(energyLabel, "storage", .5, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
    end
    
    energyStatus.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(energyLabel, "storage", .5, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
    end
    
    massRateContainer.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(massLabel, "rate", .5, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        elseif event.Type == 'ButtonPress' or event.Type == 'ButtonDClick' then
            if massViewState >= 2 then
                massViewState = 1
            else
                massViewState = massViewState + 1
            end
            Prefs.SetToCurrentProfile("massRateView", massViewState)
        end
        Group.HandleEvent(self, event)
    end
    
    energyRateContainer.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(energyLabel, "rate", .5, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        elseif event.Type == 'ButtonPress' or event.Type == 'ButtonDClick' then
            if energyViewState >= 2 then
                energyViewState = 1
            else
                energyViewState = energyViewState + 1
            end
            Prefs.SetToCurrentProfile("energyRateView", energyViewState)
        end
        Group.HandleEvent(self, event)
    end
    
    massDropdownTarget.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(massLabel, "extended_display", .5, true)
            if not massDropdownVisible then
                massDropdown:Show()
            end
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
            if not massDropdownVisible then
                massDropdown:Hide()
            end
        elseif event.Type == 'ButtonPress' or event.Type == 'ButtonDClick' then
            massDropdownVisible = not massDropdownVisible
            massDropdown:SetHidden(not massDropdownVisible)
            Prefs.SetToCurrentProfile("massDetailedView", massDropdownVisible)
        end
    end

    energyDropdownTarget.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(energyLabel, "extended_display", .5, true)
            if not energyDropdownVisisble then
                energyDropdown:Show()
            end
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
            if not energyDropdownVisisble then
                energyDropdown:Hide()
            end
        elseif event.Type == 'ButtonPress' or event.Type == 'ButtonDClick' then
            energyDropdownVisisble = not energyDropdownVisisble
            energyDropdown:SetHidden(not energyDropdownVisisble)
            Prefs.SetToCurrentProfile("energyDetailedView", energyDropdownVisisble)
        end
    end

    return massBG, energyBG
end

function GetMassText()
    if massViewState == 1 then
        return "Mass Rate"
    elseif massViewState == 2 then
        return "Mass Efficiency"
    end
end

function EconWarningHide(bool)
    hideWarnings = bool
end