--*****************************************************************************
--* File: lua/modules/ui/game/enhancement.lua
--* Author: Ted Snook
--* Summary: UI for the Enhancement Window
--*
--* Copyright  2006 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Text = import('/lua/modules/maui/text.lua').Text
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Button = import('/lua/modules/maui/button.lua').Button

local dialog = false
local enhancementTable = {}
local parent = false

local mfdControl = false
local gameParent = false

local restrictedList = {}

local controls = {
    dialog = false,
    hideButton = false,
    unitImage = false,
    slotButtons = {},
}

function AddEnhancementEntry(entry)
    enhancementTable = entry
end

function RestrictList(table)
    restrictedList = table
end

function GetRestricted()
    return restrictedList
end

function GetEnhancements(entityID)
    if enhancementTable then
        for i, v in enhancementTable do
            if tostring(i) == tostring(entityID) then
                return enhancementTable[i]
            end
        end
    else
        return false
    end
end

function CreateElements()
    controls.dialog = Bitmap(parent, UIUtil.UIFile('/dialogs/enhance-panel/panel_bmp.dds'))
    controls.hideButton = Button(controls.dialog, 
        UIUtil.UIFile('/dialogs/close_btn/close_btn_up.dds'),
        UIUtil.UIFile('/dialogs/close_btn/close_btn_over.dds'),
        UIUtil.UIFile('/dialogs/close_btn/close_btn_down.dds'),
        UIUtil.UIFile('/dialogs/close_btn/close_btn_dis.dds'))
    controls.unitImage = Bitmap(controls.dialog, UIUtil.UIFile('/dialogs/enhancements/default_unit.dds'))
    controls.windowTitle = UIUtil.CreateText(controls.dialog, "<LOC ENHANCE_0000>Title Bar", 14, UIUtil.bodyFont)
end

function DoBottomLayout()
    controls.dialog.Bottom:Set(gameParent.Bottom)
    controls.dialog.Left:Set(mfdControl.Right)
end

function DoLeftLayout()
    controls.dialog.Bottom:Set(function() return gameParent.Bottom() - 60 end)
    controls.dialog.Left:Set(mfdControl.Right)
end

function DoRightLayout()
    controls.dialog.Bottom:Set(function() return gameParent.Bottom() - 60 end)
    controls.dialog.Left:Set(gameParent.Left)
end

function CommonLayout()
    controls.dialog.Depth:Set(1000)
    
    controls.hideButton.Right:Set(function() return controls.dialog.Right() - 13 end)
    controls.hideButton.Top:Set(function() return controls.dialog.Top() + 7 end)
    
    LayoutHelpers.AtCenterIn(controls.unitImage, controls.dialog, -10)
    controls.unitImage.Top:Set(function() return controls.dialog.Top() + 40 end)
    controls.unitImage:DisableHitTest()
    
    controls.windowTitle.Top:Set(function() return controls.dialog.Top() + 7 end)
    controls.windowTitle.Left:Set(function() return controls.dialog.Left() + 15 end)
end

function SetLayout(layoutVar)
    if layoutVar == 'bottom' then
        DoBottomLayout()
    elseif layoutVar == 'left' then
        DoLeftLayout()
    elseif layoutVar == 'right' then
        DoRightLayout()
    end
    CommonLayout()
end

function CreateDialog(layoutVar, inmfdControl, ingameParent, inMapGroup)
    parent = inMapGroup
    mfdControl = inmfdControl
    gameParent = ingameParent
    CreateElements()
    SetLayout(layoutVar)
            
    controls.hideButton.OnClick = function(self)
        controls.dialog:Hide()
        import('/lua/modules/ui/game/construction.lua').ToggleEnhancementWindow(false, controls.dialog)
        import('/lua/modules/ui/game/queue.lua').ToggleEnhancementWindow(false, controls.dialog)
    end
    controls.dialog:Hide() 
end

function ToggleDialog(enhancementTable, unitID, closeBool)
    if controls.dialog then 
        if closeBool or not controls.dialog:IsHidden() then
            controls.dialog:Hide() 
            import('/lua/modules/ui/game/construction.lua').ToggleEnhancementWindow(false, controls.dialog)
            import('/lua/modules/ui/game/queue.lua').ToggleEnhancementWindow(false, controls.dialog)
        else
            local tempbp = unitID:GetBlueprint()
            controls.windowTitle:SetText(LOC(tempbp.Description))
            for i, v in controls.slotButtons do
                if controls.slotButtons[i] then
                    controls.slotButtons[i]:Destroy()
                    controls.slotButtons[i] = false
                end
            end
                
            controls.dialog:Show() 
            import('/lua/modules/ui/game/construction.lua').ToggleEnhancementWindow(true, controls.dialog)
            import('/lua/modules/ui/game/queue.lua').ToggleEnhancementWindow(true, controls.dialog)
            local blueprintID = unitID:GetBlueprint().BlueprintId
            local entityID = unitID:GetEntityId()
            local blueprint = unitID:GetBlueprint()
            
            controls.unitImage:SetTexture(UIUtil.UIFile('/dialogs/enhance-panel/'..blueprintID..'_bmp.dds'))
            
            if enhancementTable.Slots then
                for i, v in enhancementTable.Slots do
                    if controls.slotButtons[i] then 
                        controls.slotButtons[i]:Destroy()
                        controls.slotButtons[i] = {}
                    else
                        controls.slotButtons[i] = {}
                    end
                    controls.slotButtons[i] = Button(controls.unitImage, GetSlotTextures(blueprintID, i))
                    local y = v['y']
                    local x = v['x']
                    local slotName = i
                    controls.slotButtons[i].SlotName = slotName
                    controls.slotButtons[i].UnitID = unitID
                    controls.slotButtons[i].Blueprint = blueprint
                    controls.slotButtons[i].EntityID = entityID
                    controls.slotButtons[i].Top:Set(function() return controls.unitImage.Top() + y end)
                    controls.slotButtons[i].Left:Set(function() return controls.unitImage.Left() + x end)
                    controls.slotButtons[i].OnClick = function(self)
                        import('/lua/modules/ui/game/construction.lua').EnhancementPopulate(slotName)
                    end
                    controls.slotButtons[i].HandleEvent = function(self, event)
                        if event.Type == 'MouseEnter' then
                            import('/lua/modules/ui/game/unitviewDetail.lua').ShowEnhancement(DisplayText(blueprintID, self.Blueprint, self.SlotName, self.EntityID, self.UnitID))
                        elseif event.Type == 'MouseExit' then
                            import('/lua/modules/ui/game/unitviewDetail.lua').Hide()
                        end
                        Button.HandleEvent(self, event)
                    end
                end
            else
                WARN("Enhancement Window: No slots defined in blueprint!")
            end
            
            if table.getsize(enhancementTable.Slots) == 1 then
                for i, v in controls.slotButtons do
                    if controls.slotButtons[i] then
                        controls.slotButtons[i]:Disable()
                    end
                end
                local tempSlotName = ""
                for i, v in enhancementTable.Slots do
                    tempSlotName = i
                end
                import('/lua/modules/ui/game/construction.lua').EnhancementPopulate(tempSlotName)
            end
        end
    end
end
        
function DisplayText(unitID, blueprint, slot, entityID, userunit)
    local curEnhString = "EMPTY"
    local curEnhancement = ""
    local iconID = ""
    local enhTable = GetEnhancements(entityID)
    if enhTable then
        for i, v in enhTable do
            if slot == i then
                curEnhancement = v
            end
        end
        for i, v in blueprint.Enhancements do   
            if i == curEnhancement then
                if v.Name then
                    curEnhString = LOC(v.Name)
                    iconID = v.Icon
                end
            end
        end
    end
    return {Slot = LOC(slot), Name = curEnhString}, unitID, iconID, userunit
end
        
function GetSlotTextures(unitID, slotName)
    local tempBmpname = ""
    if slotName == "Back" then
        tempBmpName = "b"
    elseif slotName == "LCH" then
        tempBmpName = "la"
    elseif slotName == "RCH" then
        tempBmpName = "ra"
    elseif slotName == "Shield" then
        tempBmpName = "ch"
    end
    
    local iconName = '/textures/ui/uef/dialogs/enhance-panel/' .. unitID .. '-' .. tempBmpName .. '_btn_up.dds'
    local upIconName = '/textures/ui/uef/dialogs/enhance-panel/' .. unitID .. '-' .. tempBmpName .. '_btn_sel.dds'
    local downIconName = '/textures/ui/uef/dialogs/enhance-panel/' .. unitID .. '-' .. tempBmpName .. '_btn_down.dds'
    local overIconName = '/textures/ui/uef/dialogs/enhance-panel/' .. unitID .. '-' .. tempBmpName .. '_btn_up.dds'
    
    if DiskGetFileInfo(iconName) == false then
        iconName = '/textures/ui/uef/dialogs/close_btn/close_btn_up.dds'
    end
    
    if DiskGetFileInfo(upIconName) == false then
        upIconName = iconName
    end

    if DiskGetFileInfo(downIconName) == false then
        downIconName = iconName
    end

    if DiskGetFileInfo(overIconName) == false then
        overIconName = iconName
    end
    
    return iconName, upIconName, downIconName, overIconName
end

function DestroyDialog()
    if controls.dialog then
        controls.dialog:Hide()
    end
end