--*****************************************************************************
--* File: lua/modules/ui/game/helptext.lua
--* Author: Ted Snook
--* Summary: Help Text Popup
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Text = import('/lua/modules/maui/text.lua').Text
local Button = import('/lua/modules/maui/button.lua').Button
local ItemList = import('/lua/modules/maui/itemlist.lua').ItemList
local Combo = import('/lua/modules/ui/controls/combo.lua').Combo

local keymap = import('/lua/keymap/defaultKeyMap.lua').defaultKeyMap
local keydesc = import('/lua/keymap/keydescriptions.lua').keyDescriptions

local panel = false

function CreateUI()
    if panel then return end
    panel = Bitmap(GetFrame(0), UIUtil.UIFile('/dialogs/options/panel-front-end_bmp.dds'))
    LayoutHelpers.AtCenterIn(panel, GetFrame(0))
    panel.Depth:Set(GetFrame(0):GetTopmostDepth() + 1)
    
    local worldCover = UIUtil.CreateWorldCover(panel)
    
    local title = UIUtil.CreateText(panel, LOC("<LOC key_binding_0000>Key Bindings"), 24)
    LayoutHelpers.AtHorizontalCenterIn(title, panel)
    LayoutHelpers.AtTopIn(title, panel, 30)
    
    local closeButton = UIUtil.CreateButtonStd(panel, "/widgets/small02", LOC("<LOC _Close>"), 12)
    LayoutHelpers.AtHorizontalCenterIn(closeButton, panel)
    LayoutHelpers.AtBottomIn(closeButton, panel, 40)
    closeButton.OnClick = function(self, modifiers)
        panel:Destroy()
        panel = false
    end
    
    local list = ItemList(panel)
    list:SetFont(UIUtil.bodyFont, 14)
    list:SetColors(UIUtil.fontColor, "00000000", UIUtil.fontColor, "00000000")
    list.Width:Set(487)
    list.Height:Set(307)
    LayoutHelpers.AtLeftTopIn(list, panel, 50, 78)
    UIUtil.CreateVertScrollbarFor(list)
    
    local keys = {}
    
    for i, v in keymap do
        if v.category then
            if not keys[v.category] then
                keys[v.category] = {}
            end
            table.insert(keys[v.category], {data = v, key = i})
        end
    end
    
    for i, v in keys do
        table.sort(v, function(val1, val2)
            if val1.data.order >= val2.data.order then
                return false
            else
                return true
            end
        end)
    end
    
    for i, v in keys do
        list:AddItem(LOC(i))
        for h, x in v do
            if x.data.stringkey and keydesc[x.data.stringkey] then
                list:AddItem(string.format("  %s", x.key))
                list:AddItem(LOCF("    %s", keydesc[x.data.stringkey]))
            end
        end
        list:AddItem("")
    end
end