--*****************************************************************************
--* File: lua/modules/ui/game/missiontext.lua
--* Author: Ted Snook
--* Summary: Mission text HUD
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Bitmap = import('/lua/modules/maui/Bitmap.lua').Bitmap
local ItemList = import('/lua/modules/maui/itemlist.lua').ItemList
local MultiLineText = import('/lua/modules/maui/multilinetext.lua').MultiLineText
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Prefs = import('/lua/modules/ui/prefs.lua')
local WrapText = import('/lua/modules/maui/text.lua').WrapText
local Movie = import('/lua/modules/maui/movie.lua').Movie
local GameCommon = import('/lua/modules/ui/game/gamecommon.lua')
local econ = import('/lua/modules/ui/game/economy.lua')

local MISSION_TEXT_TIMEOUT = 10
local missionTextInactivityTime = 11
local videoTextInactivityTime = 11
local resetEconWarnings = false
local movieText = ""
local currentlyPlaying = false
local textHistory = ""
local subtitleThread = false
local videoQueue = {}

controls = {
    infoBG = false,
    infoText = false,
    movieTextGroup = false,
    textCursor = false,
    movieGroup = false,
    movieText = {},
    movieGroupBG = false,
    movieTextGroupBGTop = false,
    movieTextGroupBGMiddle = false,
    movieTextGroupBGBottom = false,
}

function UpdateVideoText(string)
    currentlyShowingText = true
    movieText = LOC(string[1]) or ""
    for i=1, 20 do
        controls.movieText[i]:SetText("")
    end
    local textBoxWidth = controls.movieTextGroup.Right() - controls.movieTextGroup.Left()
    local tempTable = import('/lua/modules/maui/text.lua').WrapText(movieText, textBoxWidth,
    function(text)
        return controls.movieText[1]:GetStringAdvance(text)
    end)
    #videoTextInactivityTime = 0
    local tempBottom = controls.movieText[table.getn(tempTable)].Bottom or controls.movieTextGroupBGTop.Bottom
    controls.movieTextGroupBGBottom.Top:Set(tempBottom)
	  controls.movieGroup:Show()
	  local index = 1
    controls.movieText[1]:SetNeedsFrameUpdate(true)
    controls.movieText[1].OnFrame = function(self, deltaTime)
        if deltaTime then
            if tempTable[index] then
                controls.movieText[index]:StreamText(tempTable[index], 3)
            else
                controls.movieText[1]:SetNeedsFrameUpdate(false)
            end
            index = index + 1
        end
    end
end

local function CreateMovieGroup(parent)
    if not controls.movieGroup then
        controls.movieGroup = Group(parent,"MFD Movie Group")
    end
    controls.movieGroup.Height:Set(100)
	  controls.movieGroup.Width:Set(100)
	  controls.movieGroup.Left:Set(100)
	  controls.movieGroup.Top:Set(100)
    controls.movieGroup.OnHide = GameCommon.SupressShowingWhenRestoringUI

    if not controls.movieGroupBG then
        controls.movieGroupBG = Bitmap(controls.movieGroup, UIUtil.UIFile('/dialogs/transmission-incoming/panel_bmp_l.dds'))
    end
    LayoutHelpers.AtCenterIn(controls.movieGroupBG, controls.movieGroup)
    controls.movieGroupBG.Depth:Set(function() return controls.movieGroup.Depth() - 1 end)

    if not controls.movieTextGroup then
        controls.movieTextGroup = Group(controls.movieGroup)
    end
    controls.movieTextGroup.Left:Set(function() return controls.movieGroupBG.Right() + 20 end)
    controls.movieTextGroup.Top:Set(function() return controls.movieGroup.Top() + 15 end)
    controls.movieTextGroup.Width:Set(370)
    controls.movieTextGroup.Height:Set(150)
    #controls.movieTextGroup:SetNeedsFrameUpdate(true)
    #controls.movieTextGroup.OnFrame = function(self, deltaTime)
    #    if videoTextInactivityTime > MISSION_TEXT_TIMEOUT then
    #        controls.movieGroup:Hide()
    #    else
    #        videoTextInactivityTime = videoTextInactivityTime + deltaTime
    #        controls.movieGroup:Show()
    #    end
    #end

    if not controls.movieTextGroupBGTop then
        controls.movieTextGroupBGTop = Bitmap(controls.movieTextGroup, UIUtil.UIFile('/dialogs/transmission-incoming/panel_bmp_t.dds'))
    end
    controls.movieTextGroupBGTop.Bottom:Set(function() return controls.movieGroup.Top() + 23 end)
    controls.movieTextGroupBGTop.Left:Set(function() return controls.movieGroupBG.Right() end)
    controls.movieTextGroupBGTop.Depth:Set(function() return controls.movieTextGroup.Depth() - 1 end)

    if not controls.movieTextGroupBGBottom then
        controls.movieTextGroupBGBottom = Bitmap(controls.movieTextGroup, UIUtil.UIFile('/dialogs/transmission-incoming/panel_bmp_b.dds'))
    end
    controls.movieTextGroupBGBottom.Top:Set(controls.movieTextGroup.Bottom)
    controls.movieTextGroupBGBottom.Left:Set(controls.movieTextGroupBGTop.Left)
    controls.movieTextGroupBGBottom.Depth:Set(function() return controls.movieTextGroup.Depth() - 1 end)

    if not controls.movieTextGroupBGMiddle then
        controls.movieTextGroupBGMiddle = Bitmap(controls.movieTextGroup, UIUtil.UIFile('/dialogs/transmission-incoming/panel_bmp_m.dds'))
    end
    controls.movieTextGroupBGMiddle.Bottom:Set(controls.movieTextGroupBGBottom.Top)
    controls.movieTextGroupBGMiddle.Top:Set(controls.movieTextGroupBGTop.Bottom)
    controls.movieTextGroupBGMiddle.Depth:Set(function() return controls.movieTextGroup.Depth() - 1 end)
    controls.movieTextGroupBGMiddle:SetTiled(true)
    controls.movieTextGroupBGMiddle.Left:Set(controls.movieTextGroupBGTop.Left)

    local previousTextBox = false
    for i=1, 20 do
        if not controls.movieText[i] then
            controls.movieText[i] = UIUtil.CreateText(controls.movieTextGroup, "", 12, UIUtil.bodyFont)
        end
        if i == 1 then
            controls.movieText[i].Top:Set(controls.movieTextGroup.Top)
        else
            controls.movieText[i].Top:Set(previousTextBox.Bottom)
        end
        controls.movieText[i].Left:Set(controls.movieTextGroup.Left)
        previousTextBox = controls.movieText[i]
    end
    controls.movieGroup:DisableHitTest(true)
end

function CreateMissionText(parent)
    local worldView = import('/lua/modules/ui/game/borders.lua').GetMapGroup()

    if not controls.infoText then
        controls.infoText = UIUtil.CreateText(parent, "", 16, UIUtil.bodyFont)
    end
    LayoutHelpers.AtCenterIn(controls.infoText, worldView, -150)
    controls.infoText:SetNeedsFrameUpdate(true)
    controls.infoText.OnFrame = function(self, deltaTime)
        if missionTextInactivityTime > MISSION_TEXT_TIMEOUT then
            controls.infoText:Hide()
        else
            missionTextInactivityTime = missionTextInactivityTime + deltaTime
            controls.infoText:Show()
        end
    end

    if not controls.infoBG then
        controls.infoBG = Bitmap(controls.infoText, UIUtil.UIFile('/dialogs/mission/panel_bmp.dds'))
    end
    LayoutHelpers.AtCenterIn(controls.infoBG, controls.infoText)
    controls.infoBG.Depth:Set(function() return controls.infoText.Depth() - 1 end)


    CreateMovieGroup(parent)
    controls.movieGroup.Top:Set(function() return worldView.Top() + 30 end)
    controls.movieGroup.Left:Set(function() return worldView.Left() + 30 end)

    controls.infoText:DisableHitTest(true)
    controls.infoText:Hide()
    controls.movieGroup:Hide()
end

local currentMovie = false
local facMovie = false
function PlayMFDMovie(movieData, text, tauntBool)
    if not currentlyPlaying then
        #UpdateVideoText(text)
        currentlyPlaying = true
        if Prefs.GetOption('econ_warnings') then
            resetEconWarnings = true
            econ.EconWarningHide(true)
        end

        if currentMovie then
            currentMovie:Destroy()
        end
        controls.movieGroup:Show()
        controls.movieTextGroup:Hide()
        #controls.infoText:Show()

 		currentMovie = Movie(controls.movieGroup, movieData[1])
        currentMovie:DisableHitTest()
        LayoutHelpers.FillParentPreserveAspectRatio(currentMovie, controls.movieGroup)   -- note the movies now fill the parent

        if movieData[4] then
        	facMovie = Movie(controls.movieGroup, '/movies/comm_' .. movieData[4] .. '.sfd')
	        facMovie:DisableHitTest()
    	    LayoutHelpers.FillParentPreserveAspectRatio(facMovie, controls.movieGroup)
    	    currentMovie.OnLoaded = function(self) facMovie:Play() end
        	facMovie.OnFinished = function(self)
        		currentMovie:Play()
            	movieSound = PlaySound(Sound{ Bank=movieData[2], Cue=movieData[3] })
            	controls.movieTextGroup:Show()
            	UpdateVideoText(text)
        	end
    	else
    		LOG('ERROR: no faction parameter for this movie, not playing faction intro movie')
    		currentMovie:Play()
            movieSound = PlaySound(Sound{ Bank=movieData[2], Cue=movieData[3] })
            controls.movieTextGroup:Show()
            UpdateVideoText(text)
        end

        controls.movieGroup:SetNeedsFrameUpdate(true)
    	controls.movieGroup.OnFrame = function(self, elapsedTime)
        	if elapsedTime then
            	local entryData = {movieData[1], movieText, movieData[2], movieData[3]}
            	controls.movieGroup:SetNeedsFrameUpdate(false)
            	import('/lua/modules/ui/game/transmissionlog.lua').AddEntry(entryData)
        	end
    	end
        
        currentMovie.OnFinished = function(self)
            controls.movieGroup:Hide()
            if movieData[4] then
            	facMovie:Destroy()
            	facMovie = false
            end
            currentMovie:Destroy()
            currentMovie = false
            movieSound = false
            if resetEconWarnings then
                resetEconWarnings = false
                econ.EconWarningHide(false)
                currentlyPlaying = false
            end
            UpdateQueue()
            #videoTextInactivityTime = MISSION_TEXT_TIMEOUT + 1
        end
    else
        if not tauntBool then
            table.insert(videoQueue, {movieData[1], movieData[2], movieData[3], movieData[4], text})
        end
    end
end

function PauseTransmission()
    if currentMovie then
        currentMovie:Stop()
    end
end

function ResumeTransmission()
    if currentMovie then
        currentMovie:Play()
    end
end

function UpdateQueue()
    if table.getsize(videoQueue) > 0 then
        PlayMFDMovie({videoQueue[1][1], videoQueue[1][2], videoQueue[1][3], videoQueue[1][4]}, videoQueue[1][5])
        table.remove(videoQueue, 1)
    end
end

function UpdateMissionText(text)
    #local tempmissiontextvisible = Prefs.GetFromCurrentProfile("missiontextvisible")
    #if tempmissiontextvisible == nil then
    #    tempmissiontextvisible = true
    #end
    #
    #if tempmissiontextvisible then
    #    controls.infoText:Show()
    #    textHistory = table.concat(LOC_ALL(text), "\n")
    #    missionTextInactivityTime = 0
    #    local dispText = textHistory
    #    if string.len(textHistory) > 30 then
    #        dispText = string.sub(textHistory, 1, 30) .. "..."
    #    end
    #    controls.infoText:SetText(LOC(dispText))
    #else
    #    WARN("-----------------------------------------------------")
    #    WARN("----YOU HAVE MISSION TEXT DISABLED IN YOUR PROFILE---")
    #    WARN("-----------------------------------------------------")
    #end
end

function DisplaySubtitles(textControl,captions)
    subtitleThread = ForkThread(
        function()
            # Display subtitles
            local lastOff = 0
            for k,v in captions do
                WaitSeconds(v.offset)
                textControl:DeleteAllItems()
                locText = LOC(v.text)
                #LOG("Wrap: ",locText)
                local lines = WrapText(locText, textControl.Width(), function(text) return textControl:GetStringAdvance(text) end)
                for i,line in lines do
                    textControl:AddItem(line)
                end
                textControl:ScrollToBottom()
                lastOff = v.offset
            end
            subtitleThread = false
        end
    )
end

function PlayNIS(movieName,subtitleKey)
    LOG('MovieName = ',movieName)
    SessionRequestPause()
    ConExecute("ren_Oblivion")
    local parent = GetFrame(0)
    local nisBG = Bitmap(parent)
    nisBG:SetSolidColor('FF000000')
    LayoutHelpers.FillParent(nisBG, parent)
    nisBG.Depth:Set(99998)
    local nis = Movie(parent, movieName)
    nis.Depth:Set(99999)
    nis.Width:Set(1024)
    nis.Height:Set(576)
    LayoutHelpers.AtCenterIn(nis, parent)

    local textArea = ItemList(parent)
#    textArea.Height:Set( function() return (nis.Height() * 0.1181) - 2 end )
    textArea.Top:Set( function() return nis.Bottom() + 2 end )
    textArea.Width:Set( function() return nis.Width() / 2 end )
    textArea.Height:Set( function() return parent.Bottom() - textArea.Top() end )
    textArea.Left:Set( function() return parent.Left() + textArea.Width() / 2 end )
    textArea:SetFont(UIUtil.bodyFont, 15)
    textArea:SetColors(UIUtil.fontColor, "00000000", UIUtil.fontColor,  UIUtil.highlightColor)
    textArea.Depth:Set(100000)

    local strings = import('/tutorials/' .. subtitleKey .. '/' .. subtitleKey .. '.lua')[subtitleKey]
    DisplaySubtitles(textArea, strings.captions)

    nis:Play()
    nis.OnFinished = function(self)
        SessionResume()
        ConExecute("ren_Oblivion")
        if subtitleThread then
            KillThread(subtitleThread)
            subtitleThread = false
        end
        nisBG:Destroy()
        nis:Destroy()
        textArea:Destroy()
    end
    nis.HandleEvent = function(self, event)
        -- cancel movie playback on mouse click or key hit
        if event.Type == "ButtonPress" or event.Type == "KeyDown" then
            StopTutorialVO()
            if subtitleThread then
                KillThread(subtitleThread)
                subtitleThread = false
            end
            nis:Stop()
            SessionResume()
            ConExecute("ren_Oblivion")
            nisBG:Destroy()
            nis:Destroy()
            textArea:Destroy()
            return true
        end
    end
end

function PlayEndGameMovie(inFaction)
    SessionRequestPause()
    ConExecute("ren_Oblivion")
    local step = 1
    local faction = inFaction
    local movieName = GetFactionVideo(faction, step)
    local sound = Sound(GetFactionVO(faction, step))
    local parent = GetFrame(0)
    local nisBG = Bitmap(parent)
    nisBG:SetSolidColor('FF000000')
    LayoutHelpers.FillParent(nisBG, parent)
    nisBG.Depth:Set(99999)
    local outro = Movie(parent)
    outro:Set(movieName)
    outro.Depth:Set(100000)
    LayoutHelpers.FillParentPreserveAspectRatio(outro, parent)
    outro:Play()
    outro.Sound = PlaySound(sound)
    outro:AcquireKeyboardFocus()
    outro.OnFinished = function(self)
        outro:AbandonKeyboardFocus()
        StopSound(outro.Sound,true)
        SessionResume()
        ConExecute("ren_Oblivion")
        nisBG:Destroy()
        self:Destroy()
        return true
    end
    outro.HandleEvent = function(self, event)
        -- cancel movie playback on mouse click or key hit
        if event.Type == "ButtonPress" or event.Type == "KeyDown" then
            StopSound(outro.Sound,true)
            self:Stop()
            return true
        end
    end
    outro.OnStopped = function(self)
        outro:AbandonKeyboardFocus()
        StopSound(outro.Sound,true)
        SessionResume()
        ConExecute("ren_Oblivion")
        nisBG:Destroy()
        self:Destroy()
        return true
    end
end

function GetFactionVO(faction, step)
    local factionstr = GetFaction(faction)
    if step == 1 then
        return {Cue = "FMV_"..factionstr.."_Outro_1", Bank = "FMV",}
    elseif step == 2 then
        return {Cue = "FMV_"..factionstr.."_Credits", Bank = "FMV",}
    elseif step == 3 then
        return {Cue = "FMV_"..factionstr.."_Outro_2", Bank = "FMV",}
    end
end

function GetFactionVideo(faction, step)
    local factionstr = GetFaction(faction)
    if step == 1 then
        return "/movies/FMV_"..factionstr.."_Outro_1.sfd"
    elseif step == 2 then
        return "/movies/FMV_Credits.sfd"
    elseif step == 3 then
        return "/movies/FMV_"..factionstr.."_Outro_2.sfd"
    end
end

function GetFaction(faction)
    if faction == 1 then
        return "UEF"
    elseif faction == 2 then
        return "Aeon"
    elseif faction == 3 then
        return "Cybran"
    end
end