--*****************************************************************************
--* File: lua/modules/ui/game/multifunction.lua
--* Author: Chris Blackwell
--* Summary: UI for the multifunction display
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Checkbox = import('/lua/modules/maui/checkbox.lua').Checkbox
local Button = import('/lua/modules/maui/button.lua').Button
local Grid = import('/lua/modules/maui/grid.lua').Grid
local Text = import('/lua/modules/maui/text.lua').Text
local GameCommon = import('/lua/modules/ui/game/gamecommon.lua')
local Tooltip = import('/lua/modules/ui/game/tooltip.lua')
local econ = import("/lua/modules/ui/game/economy.lua")
local Prefs = import('/lua/modules/ui/prefs.lua')

local worldView = import('/lua/modules/ui/game/borders.lua').GetMapGroup()

local minimap = Prefs.GetOption('stratview') or false
local mouseoverDisplay = false

local buttonData = {
    [1] = {
        btnName = 'military',
        label = 'mfd_military',
        button = false,
        clickBehavior = function(self, checked)
            ConExecute('ui_RenderOverlayMilitary ' .. tostring(checked))
        end
    },
    [2] = {
        btnName = 'defenses',   --TODO get a button for this
        label = 'mfd_defense',
        button = false,
        clickBehavior = function(self, checked)
            ConExecute('ui_TeamColorMode tog')
        end
    },
    [3] = {
        btnName = 'economy',
        label = 'mfd_economy',
        button = true,
        clickBehavior = function(self, checked)
            toggleEconDisplay()
        end
    },
    [4] = {
        btnName = 'intelligence',
        label = 'mfd_intel',
        button = false,
        clickBehavior = function(self, checked)
            ConExecute('ui_RenderOverlayIntel ' .. tostring(checked))
        end
    },
    [5] = {
        btnName = 'control',
        label = 'mfd_strat_view',
        button = false,
        clickBehavior = function(self, checked)
            ToggleMinimap()
        end
    },
}

local NUM_MINIMAP_BUTTONS = 5

local controls = {
    controlClusterGroup = false,
    group = false,
    miniMap = false,
    bg = false,
    glow = false,
    displayGroup = false,
    infoText = false,
}
local econDisplayState = 0
function toggleEconDisplay()
    if econDisplayState == 0 then
        econDisplayState = 1
        econ.ToggleBuildEfficiency()
    elseif econDisplayState == 1 then
        econDisplayState = 2
        ConExecute('ui_RenderOverlayEconomy '.. tostring(true))
    else
        econDisplayState = 0
        ConExecute('ui_RenderOverlayEconomy '.. tostring(false))
        econ.ToggleBuildEfficiency()
    end
end

local minimapBtns = {}

local function CreateMFDCheckbox(name, behavior, label, isbutton)
    local centeringGroup = Group(controls.bg, "centeringGroup")
    centeringGroup.Width:Set(44)
    centeringGroup.Height:Set(33)
    if isbutton then
        centeringGroup.button = Button(centeringGroup,
            UIUtil.SkinnableFile('/game/mfd_btn/' .. name .. "_btn_dis.dds"),
            UIUtil.SkinnableFile('/game/mfd_btn/' .. name .. "_btn_down.dds"),
            UIUtil.SkinnableFile('/game/mfd_btn/' .. name .. "_btn_over.dds"),
            UIUtil.SkinnableFile('/game/mfd_btn/' .. name .. "_btn_dis.dds"),
            "UI_Action_MouseDown", "UI_Action_Rollover")
    else
        centeringGroup.button = Checkbox(centeringGroup,
            UIUtil.SkinnableFile('/game/mfd_btn/' .. name .. "_btn_dis.dds"),
            UIUtil.SkinnableFile('/game/mfd_btn/' .. name .. "_btn_up.dds"),
            UIUtil.SkinnableFile('/game/mfd_btn/' .. name .. "_btn_over.dds"),
            UIUtil.SkinnableFile('/game/mfd_btn/' .. name .. "_btn_over.dds"),
            UIUtil.SkinnableFile('/game/mfd_btn/' .. name .. "_btn_up.dds"),
            UIUtil.SkinnableFile('/game/mfd_btn/' .. name .. "_btn_down.dds"),
            "UI_Action_MouseDown", "UI_Action_Rollover")
    end

    centeringGroup.button.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, label, nil, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
        if isbutton then
            Button.HandleEvent(self, event)
        else
            Checkbox.HandleEvent(self, event)
        end
    end

    if isbutton then
        centeringGroup.button.OnClick = behavior
    else
        centeringGroup.button.OnCheck = behavior
    end
    LayoutHelpers.AtCenterIn(centeringGroup.button, centeringGroup)
    return centeringGroup
end

function DoBottomLayout()
    controls.group.Top:Set(controls.controlClusterGroup.Top)
    controls.group.Left:Set(controls.controlClusterGroup.Left)
    controls.group.Bottom:Set(controls.controlClusterGroup.Bottom)

    controls.bg:SetTexture(UIUtil.UIFile('/game/mfd_bmp/mfd_button_bmp.dds'))
    controls.bg.Left:Set(controls.group.Left)
    controls.bg.Top:Set(controls.group.Top)

    for i, v in minimapBtns do
        local tempLayoutName = "l_mfd_left_btn_" .. tostring(i)
        LayoutHelpers.RelativeTo(minimapBtns[i],
            controls.group,
            UIUtil.SkinnableFile('/game/mfd_bmp/mfd_bmp_layout.lua'),
            tempLayoutName,
            "mfd_button_bmp")
    end
    controls.miniMap.BG:SetTexture(UIUtil.UIFile('/game/mfd_bmp/mfd_bmp.dds'))
    controls.miniMap.BG.Left:Set(function() return controls.bg.Right() - 4 end)
    controls.miniMap.BG.Top:Set(controls.bg.Top)

    LayoutHelpers.AtCenterIn(controls.displayGroup, controls.miniMap.BG)
    controls.displayGroup.Height:Set(180)
    controls.displayGroup.Width:Set(180)

    if minimap then
        controls.group.Width:Set(function() return controls.miniMap.BG.Width() + controls.bg.Width() end)
    else
        controls.group.Width:Set(function() return controls.bg.Width() end)
    end
    LayoutHelpers.ResetRight(controls.group)
    LayoutHelpers.ResetHeight(controls.group)
end

function DoLeftLayout()
    controls.group.Top:Set(controls.controlClusterGroup.Top)
    controls.group.Left:Set(controls.controlClusterGroup.Left)
    controls.group.Right:Set(controls.controlClusterGroup.Right)

    controls.bg:SetTexture(UIUtil.UIFile('/game/mfd-vertical_bmp/mfd_button_bmp.dds'))
    controls.bg.Left:Set(controls.group.Left)
    controls.bg.Top:Set(controls.group.Top)

    for i, v in minimapBtns do
        local tempLayoutName = "l_mfd_left_btn_" .. tostring(i)
        LayoutHelpers.RelativeTo(minimapBtns[i],
            controls.group,
            UIUtil.SkinnableFile('/game/mfd-vertical_bmp/mfd-vertical_bmp_layout.lua'),
            tempLayoutName,
            "mfd_button_bmp")
    end
    controls.miniMap.BG:SetTexture(UIUtil.UIFile('/game/mfd-vertical_bmp/mfd_bmp.dds'))
    controls.miniMap.BG.Left:Set(function() return controls.bg.Left() - 0 end)
    controls.miniMap.BG.Top:Set(function() return controls.bg.Bottom() - 2 end)

    LayoutHelpers.AtCenterIn(controls.displayGroup, controls.miniMap.BG)
    controls.displayGroup.Height:Set(180)
    controls.displayGroup.Width:Set(180)

    if minimap then
        controls.group.Height:Set(function() return controls.miniMap.BG.Height() + controls.bg.Height() end)
    else
        controls.group.Height:Set(function() return controls.bg.Height() end)
    end
    LayoutHelpers.ResetBottom(controls.group)
    LayoutHelpers.ResetWidth(controls.group)
end

function Create()
    controls.group = Group(controls.controlClusterGroup, "MFD Group")
    controls.bg = Bitmap(controls.group)
    for i = 1, NUM_MINIMAP_BUTTONS do
        minimapBtns[i] = CreateMFDCheckbox(buttonData[i].btnName, buttonData[i].clickBehavior, buttonData[i].label, buttonData[i].button)
        minimapBtns[i].Depth:Set(function() return controls.bg.Depth() + 1 end)
    end
    controls.displayGroup = Group(controls.group,"MFD Display Group")
    controls.miniMap = import('/lua/modules/ui/controls/worldview.lua').WorldView(controls.displayGroup, 'MiniMap', 2, true)    -- depth value is above minimap
    controls.miniMap:SetName("Minimap")
    controls.miniMap:SetRenderPass(UIUtil.UIRP_UnderWorld) -- don't change this or the camera will lag one frame behind
    
    controls.miniMap.BG = Bitmap(controls.miniMap)
    controls.miniMap.BG.Depth:Set(function() return controls.miniMap.Depth() - 1 end) 
    
    controls.glow = Bitmap(controls.miniMap.BG)
    controls.glow:SetTexture(UIUtil.SkinnableFile('/game/mfd_bmp/mfd_glow.dds'))
    controls.glow:DisableHitTest()
    controls.glow.Depth:Set(function() return controls.miniMap.Depth() + 1 end)
    controls.glow:SetRenderPass(UIUtil.UIRP_PostGlow)
    LayoutHelpers.AtCenterIn(controls.glow, controls.miniMap)

    -- defer the camera reset one frame so a view is created, this causes a fully zoomed out map
    controls.miniMap:SetNeedsFrameUpdate(true)
    local frameCount = 0
    controls.miniMap.OnFrame = function(self, elapsedTime)
        if frameCount == 1 then
            controls.miniMap:SetNeedsFrameUpdate(false)
            controls.miniMap:CameraReset()
        end
        frameCount = frameCount + 1
    end
    LayoutHelpers.FillParent(controls.miniMap, controls.displayGroup)
    if minimap then 
        minimapBtns[5].button:SetCheck(true, true)
    else
        controls.displayGroup:Hide()
    end 
end

function SetLayout(layout)
    if layout == 'bottom' then
        DoBottomLayout()
    elseif layout == 'left' then
        DoLeftLayout()
    elseif layout == 'right' then
        DoLeftLayout()
    end
end

function SetupMultifunctionControl(parent)
    controls.controlClusterGroup = parent
    Create()
    SetLayout(UIUtil.currentLayout)
    return controls.group
end

function ToggleMilitary()
    minimapBtns[1].button:ToggleCheck()
end

function ToggleDefense()
    minimapBtns[2].button:ToggleCheck()
end

function ToggleEconomy()
    minimapBtns[3].button:ToggleCheck()
end

function ToggleIntel()
    minimapBtns[4].button:ToggleCheck()
end

function ToggleControl()
    minimapBtns[5].button:ToggleCheck()
end

function ToggleMinimap()
    minimap = not minimap
    if minimap then
        controls.displayGroup:Show()
        SetLayout(UIUtil.currentLayout)
    else
        controls.displayGroup:Hide()
        SetLayout(UIUtil.currentLayout)
    end
    Prefs.SetOption('stratview', minimap)
end

function DisableMiniMap()
    minimap = false
    controls.displayGroup:Hide()
    SetLayout(UIUtil.currentLayout)
    Prefs.SetOption('stratview', minimap)
end
