local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Button = import('/lua/modules/maui/button.lua').Button
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Grid = import('/lua/modules/maui/grid.lua').Grid

local savedParent = false
local DetailWindow = false

local WINDOW_WIDTH = 512
local WINDOW_HEIGHT = 440

function Create()
    DetailWindow = Group(savedParent)
    UIUtil.CreateBorder(DetailWindow,true)

    DetailWindow.Grid = Grid(DetailWindow, WINDOW_WIDTH - 48, 72 )
    DetailWindow.Grid.sb = UIUtil.CreateVertScrollbarFor(DetailWindow.Grid)
    DetailWindow.Widgets = {}

    DetailWindow.OkBtn = UIUtil.CreateButtonStd(DetailWindow, '/widgets/medium', "<LOC OK>", 16, -5)
    DetailWindow.OkBtn.OnClick = function(self, modifiers)
        ToggleDisplay()
    end
end

function CreateDetailWidget(parent,srcWidget)
    local w = Group(parent)
    w.ObjData = table.deepcopy(srcWidget.ObjData)

    local border,bg = UIUtil.CreateBorder(w,true)
    border:LayoutInsideControl(w)

    w.frameSm = Bitmap(w)
    w.frameSm:SetTexture(UIUtil.UIFile('/dialogs/objective-unit/panel-icon-sm_bmp.dds'))

    w.targetButton = Button(w.frameSm)
    w.targetButton:SetTexture(srcWidget.targetButton.mNormal)

    if srcWidget.actionButton then
        w.actionButton = Button(w.targetButton)
        w.actionButton:SetTexture(srcWidget.actionButton.mNormal)
    end

    w.title = UIUtil.CreateText(w, "", 12, UIUtil.titleFont)
    w.title:SetColor( UIUtil.dialogCaptionColor )
    w.title:SetDropShadow(true)

    w.progress = UIUtil.CreateText(w, "", 12, UIUtil.titleFont)
    w.progress:SetColor( UIUtil.dialogColumnColor )
    w.progress:SetDropShadow(true)

    w.elapsed = UIUtil.CreateText(w, "", 12, UIUtil.titleFont)
    w.elapsed:SetColor( UIUtil.dialogColumnColor )
    w.elapsed:SetDropShadow(true)

    if srcWidget.Status then
        w.Status = srcWidget.Status
    end

    w.StartTime = srcWidget.StartTime
    if srcWidget.EndTime then
        w.EndTime = srcWidget.EndTime
    end

    w:DisableHitTest(true)
    return w
end

function Refresh()

    Widgets = import('/lua/modules/ui/game/objectives2.lua').Widgets
    if not Widgets then
        return
    end

    DetailWindow.Widgets = {}
    DetailWindow.Grid:DeleteAndDestroyAll(true)
    DetailWindow.Grid:AppendCols(1, true)

    local index = 1
    for _,key in table.keys(Widgets) do
        local widget = Widgets[key]
        DetailWindow.Grid:AppendRows(1, true)
        local detail = CreateDetailWidget(DetailWindow.Grid, widget)
        detail.Width:Set( DetailWindow.Grid._itemWidth)
        detail.Height:Set( DetailWindow.Grid._itemHeight)
        DetailWindow.Grid:SetItem(detail, 1, index, true)
        table.insert( DetailWindow.Widgets, detail )
        index = index + 1
    end

    DetailWindow.Grid:EndBatch()
    SetLayout(UIUtil.currentLayout)
end

function SetLayout(layout)
    DetailWindow.Width:Set(WINDOW_WIDTH)
    DetailWindow.Height:Set(WINDOW_HEIGHT)
    LayoutHelpers.AtCenterIn(DetailWindow,savedParent,0,0)

    DetailWindow.Grid.Width:Set( WINDOW_WIDTH - 44 )
    DetailWindow.Grid.Height:Set( WINDOW_HEIGHT - 48 )
    LayoutHelpers.AtLeftTopIn( DetailWindow.Grid, DetailWindow, 8, 8 )

    for k,v in DetailWindow.Widgets do
        local widget = v

        widget.targetButton.Width:Set(48)
        widget.targetButton.Height:Set(48)
        LayoutHelpers.AtVerticalCenterIn( widget.targetButton, widget )
        widget.targetButton.Left:Set( function() return widget.Left() + 12 end )

        widget.frameSm.Left:Set( function() return widget.targetButton.Left() - 27 end )
        widget.frameSm.Top:Set( function() return widget.targetButton.Top() - 26 end )

        if widget.actionButton then
            LayoutHelpers.AtLeftTopIn( widget.actionButton, widget.targetButton, 0, 0 )
            widget.actionButton.Width:Set(24)
            widget.actionButton.Height:Set(24)
        end

        LayoutHelpers.RightOf( widget.title, widget.targetButton, 12 )
        widget.title.Width:Set( 96 )
        widget.title.Height:Set( 12 )
        if widget.ObjData.type == 'primary' then
            widget.title:SetText( LOC("<LOC objui_0000>Primary: ") .. LOC(widget.ObjData.title) )
        elseif widget.ObjData.type == 'secondary' then
            widget.title:SetText( LOC("<LOC objui_0001>Secondary: ") .. LOC(widget.ObjData.title) )
        else
            widget.title:SetText( LOC("<LOC objui_0002>Bonus: ") .. LOC(widget.ObjData.title) )
        end

        LayoutHelpers.Below( widget.progress, widget.title, 4 )
        widget.progress.Width:Set( 96 )
        widget.progress.Height:Set( 12 )
        if widget.Status then
            if widget.Status == 'complete' then
                widget.progress:SetText( LOC("<LOC objui_0003>Complete") )
                widget.progress:SetColor( "ff00d800" )
            else
                widget.progress:SetText( LOC("<LOC objui_0004>Failed") )
                widget.progress:SetColor( "ffd80000" )
            end
        else
            if not widget.ObjData.progress or widget.ObjData.progress == "" then
                widget.progress:SetText( LOC("<LOC objui_0005>Incomplete") )
            else
                widget.progress:SetText( LOC(widget.ObjData.progress) )
            end
        end

        LayoutHelpers.Below( widget.elapsed, widget.progress, 4 )
        widget.elapsed.Width:Set( 96 )
        widget.elapsed.Height:Set( 12 )
        if widget.EndTime then
            widget.elapsed:SetText( FormatTime(widget.EndTime - widget.StartTime) )
        else
            widget.elapsed:SetText( FormatTime(GetGameTimeSeconds() - widget.StartTime) )
        end
    end

    LayoutHelpers.AtBottomIn( DetailWindow.OkBtn, DetailWindow, 4 )
    LayoutHelpers.AtHorizontalCenterIn( DetailWindow.OkBtn, DetailWindow )
    DetailWindow.OkBtn.Depth:Set( function() return DetailWindow.Depth() + 10 end )
    DetailWindow.OkBtn:EnableHitTest()

    DetailWindow.Grid.sb:Show()
end

function SetupObjectiveDetail(parent)
    savedParent = parent
    Create()
    SetLayout(UIUtil.currentLayout)
    DetailWindow:Hide()
end

function ToggleDisplay()
    if not UIUtil.GetNetworkBool() then
        DetailWindow:SetHidden( not DetailWindow:IsHidden() )
        import('/lua/modules/ui/game/options.lua').PauseGame( not DetailWindow:IsHidden() )
        if not DetailWindow:IsHidden() then
            Refresh()
        end
    end
end
