
--*****************************************************************************
--* File: lua/modules/ui/game/pause.lua
--* Author: Ted Snook
--* Summary: Paused Game HUD
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Bitmap = import('/lua/modules/maui/Bitmap.lua').Bitmap
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Prefs = import('/lua/modules/ui/prefs.lua')

local parent = false
local hideBool = false

local controls = {
    bg = false,
    pausedText = false,
    pausesRemaining = false,
    pauseResumeButton = false,
    
}

function Expand()
    hideBool = false
end

function Contract()
    hideBool = true
end

function CreatePauseUI(inParent)
    parent = inParent
    if not controls.bg then
        controls.bg = Bitmap(parent, UIUtil.UIFile('/dialogs/pause/panel_bmp.dds'))
    end
    LayoutHelpers.AtHorizontalCenterIn(controls.bg, parent)
    controls.bg.Top:Set(function() return parent.Top() + 20 end)
    controls.bg.Depth:Set(20)
    
    if not controls.pausedText then
        controls.pausedText = UIUtil.CreateText(controls.bg, LOC("<LOC pause_screen_0000>Game Paused"), 18, UIUtil.bodyFont)
    end
    LayoutHelpers.AtHorizontalCenterIn(controls.pausedText, controls.bg)
    controls.pausedText.Top:Set(function() return controls.bg.Top() + 25 end)
    
    if not controls.pausesRemaining then
        controls.pausesRemaining = UIUtil.CreateText(controls.bg, LOC("<LOC pause_screen_0002>Timeouts Remaining"), 14, UIUtil.bodyFont)
    end
    LayoutHelpers.AtHorizontalCenterIn(controls.pausesRemaining, controls.bg)
    controls.pausesRemaining.Top:Set(function() return controls.pausedText.Bottom() + 13 end)
    
    if not controls.pauseResumeButton then
        controls.pauseResumeButton = UIUtil.CreateButtonStd(controls.bg, '/widgets/small', LOC("<LOC pause_screen_0003>Resume"), 12)
    end
    LayoutHelpers.AtHorizontalCenterIn(controls.pauseResumeButton, controls.bg)
    controls.pauseResumeButton.Top:Set(function() return controls.pausesRemaining.Bottom() + 7 end)
    controls.pauseResumeButton.OnClick = function(self, modifiers)
        SessionResume()
    end
    
    controls.bg:Hide()
end

function OnPause(owner, timeouts, isOwner)
    if not hideBool then
        if UIUtil.GetAnimationPrefs() then
            controls.bg:Show()
        else
            controls.bg:Show()
        end
        
        if owner then
            controls.pausedText:SetText(LOCF("<LOC pause_screen_0004>Game Paused: %s", SessionGetCommandSourceNames()[owner]))
        end
        if timeouts then
            if timeouts == -1 then
                controls.pausesRemaining:SetText(LOCF("<LOC pause_screen_0006>Infinite Timeouts", timeouts))
            else
                controls.pausesRemaining:SetText(LOCF("<LOC pause_screen_0005>Timeouts Remaining: %d", timeouts))
            end
        end
        
        if isOwner then
            controls.pauseResumeButton:Show()
        else
            controls.pauseResumeButton:Hide()
        end
    end
end

function OnResume()
    if UIUtil.GetAnimationPrefs() then
        controls.bg:Hide()
    else
        controls.bg:Hide()
    end
end        