--*****************************************************************************
--* File: lua/modules/ui/game/queue.lua
--* Author: Chris Blackwell / Ted Snook
--* Summary: Construction Queue UI
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Grid = import('/lua/modules/maui/grid.lua').Grid
local Checkbox = import('/lua/modules/maui/checkbox.lua').Checkbox
local Button = import('/lua/modules/maui/button.lua').Button
local GameCommon = import('/lua/modules/ui/game/gamecommon.lua')
local RadioGroup = import('/lua/modules/maui/mauiutil.lua').RadioGroup
local Tooltip = import('/lua/modules/ui/game/tooltip.lua')

local mfdControl = false
local ordersControl = false
local controlClusterGroup = false
local constructionControl = false
local currentSelection = false

local currentInfiniteQueueCheckStatus = false

local controls = {
    queueGroup = false,
    queueBGCapMin = false,
    queueBGMiddle = false,
    queueBGCapMax = false,
    queueGrid = false,
    scrollQueueFwdButton = false,
    scrollQueueBackButton = false,
    infiniteQueueToggle = false,
    pauseToggle = false,
}
    
local function CreateQueueGrid(parent)
    controls.queueGrid = Grid(parent, 62, 62)
    controls.queueGrid:SetName("Queue Grid")
end

function DoBottomLayout()
    if not controls.queueGroup then
        controls.queueGroup = Group(controlClusterGroup, "queueGroup")
    end
    controls.queueGroup.Height:Set(50)
    controls.queueGroup.Bottom:Set(controlClusterGroup.Bottom)
    controls.queueGroup.Left:Set(mfdControl.Right)
    controls.queueGroup.Right:Set(ordersControl.Left)
    
    -- set up queue
    if not controls.queueBGCapMin then
        controls.queueBGCapMin = Bitmap(controls.queueGroup)
    end
    controls.queueBGCapMin:SetTexture(UIUtil.SkinnableFile('/game/que_bmp/que-02_bmp_l.dds'))
    LayoutHelpers.AtBottomIn(controls.queueBGCapMin, controls.queueGroup, 3)
    LayoutHelpers.AtLeftIn(controls.queueBGCapMin, controls.queueGroup, -2)
    LayoutHelpers.ResetTop(controls.queueBGCapMin)
    LayoutHelpers.ResetRight(controls.queueBGCapMin)

    if not controls.queueBGCapMax then
        controls.queueBGCapMax = Bitmap(controls.queueGroup)
    end
    controls.queueBGCapMax:SetTexture(UIUtil.SkinnableFile('/game/que_bmp/que-02_bmp_r.dds'))
    LayoutHelpers.AtBottomIn(controls.queueBGCapMax, controls.queueGroup, 3)
    LayoutHelpers.AtRightIn(controls.queueBGCapMax, controls.queueGroup, 2)
    LayoutHelpers.ResetTop(controls.queueBGCapMax)
    LayoutHelpers.ResetLeft(controls.queueBGCapMax)

    if not controls.queueBGMiddle then
        controls.queueBGMiddle = Bitmap(controls.queueGroup)
    end
    controls.queueBGMiddle:SetTexture(UIUtil.SkinnableFile('/game/que_bmp/que-02_bmp_m.dds'))
    controls.queueBGMiddle.Left:Set(controls.queueBGCapMin.Right)
    controls.queueBGMiddle.Right:Set(controls.queueBGCapMax.Left)
    controls.queueBGMiddle.Top:Set(controls.queueBGCapMin.Top)
    LayoutHelpers.ResetWidth(controls.queueBGMiddle)
    controls.queueBGMiddle.Height:Set(controls.queueBGMiddle.BitmapHeight)
    LayoutHelpers.ResetBottom(controls.queueBGMiddle)
    controls.queueBGMiddle:SetTiled(true)

    if not controls.queueGrid then
        CreateQueueGrid(controls.queueBGMiddle)
    end

    if not controls.queueScrollBG then
        controls.queueScrollBG = Bitmap(controls.queueGroup)
    end
    controls.queueScrollBG.Depth:Set(function() return controls.queueBGCapMin.Depth() + 50 end)
    controls.queueScrollBG:SetTexture(UIUtil.SkinnableFile('/game/que_bmp/scroll-back_bmp.dds'))
    LayoutHelpers.RelativeTo(controls.queueScrollBG,
        controls.queueBGCapMin,
        UIUtil.SkinnableFile('/game/que_bmp/que_bmp_layout.lua'),
        "scroll-back_bmp",
        "que_bmp_l")


    -- scroll button behavior and look will change based on layout, so always create a new set of scroll buttons
    -- and get rid of any other queue scroll buttons
    if controls.scrollQueueBackButton then controls.scrollQueueBackButton:Destroy() end
    controls.scrollQueueBackButton = UIUtil.CreateButton(controls.queueBGCapMin
        , '/game/small-side_scr/arrow-left_scr_up.dds'
        , '/game/small-side_scr/arrow-left_scr_down.dds'
        , '/game/small-side_scr/arrow-left_scr_over.dds'
        , '/game/small-side_scr/arrow-left_scr_dis.dds')
    controls.scrollQueueBackButton.Depth:Set(function() return controls.queueScrollBG.Depth() + 1 end)
    LayoutHelpers.RelativeTo(controls.scrollQueueBackButton,
        controls.queueBGCapMin,
        UIUtil.SkinnableFile('/game/que_bmp/que_bmp_layout.lua'),
        "l_left_scroll",
        "que_bmp_l")
    controls.scrollQueueBackButton.OnClick = function(self)
        controls.queueGrid:ScrollLines("Horz", -1)
    end
    controls.scrollQueueBackButton:Disable()

    if controls.scrollQueueFwdButton then controls.scrollQueueFwdButton:Destroy() end
    controls.scrollQueueFwdButton = UIUtil.CreateButton(controls.queueBGCapMin
        , '/game/small-side_scr/arrow-right_scr_up.dds'
        , '/game/small-side_scr/arrow-right_scr_down.dds'
        , '/game/small-side_scr/arrow-right_scr_over.dds'
        , '/game/small-side_scr/arrow-right_scr_dis.dds')
    controls.scrollQueueFwdButton.Depth:Set(function() return controls.scrollQueueBackButton.Depth() + 1 end)
    LayoutHelpers.RelativeTo(controls.scrollQueueFwdButton,
        controls.queueBGCapMin,
        UIUtil.SkinnableFile('/game/que_bmp/que_bmp_layout.lua'),
        "l_right_scroll",
        "que_bmp_l")
    controls.scrollQueueFwdButton.OnClick = function(self)
        controls.queueGrid:ScrollLines("Horz", 1)
    end
    controls.scrollQueueFwdButton:Disable()

    if controls.infiniteQueueToggle then controls.infiniteQueueToggle:Destroy() end
    controls.infiniteQueueToggle = Checkbox(controls.queueBGCapMin,
        UIUtil.SkinnableFile('/game/infinite_btn/infinite_btn_up.dds'),
        UIUtil.SkinnableFile('/game/infinite_btn/infinite_btn_down.dds'),
        UIUtil.SkinnableFile('/game/infinite_btn/infinite_btn_over.dds'),
        UIUtil.SkinnableFile('/game/infinite_btn/infinite_btn_down.dds'),
        UIUtil.SkinnableFile('/game/infinite_btn/infinite_btn_dis.dds'),
        UIUtil.SkinnableFile('/game/infinite_btn/infinite_btn_dis.dds'))
    controls.infiniteQueueToggle.Top:Set(function() return controls.scrollQueueFwdButton.Bottom() - 5 end)
    -- place centered between queue scroll buttons
    controls.infiniteQueueToggle.Left:Set(function() return controls.queueBGCapMin.Left() + 2 end)
    controls.infiniteQueueToggle.Width:Set(30)
    controls.infiniteQueueToggle.OnClick = function(self)
        if currentInfiniteQueueCheckStatus then
            currentInfiniteQueueCheckStatus = false
        else
            currentInfiniteQueueCheckStatus = true
        end
        OnInfiniteQueueToggle(currentInfiniteQueueCheckStatus)
    end
    controls.infiniteQueueToggle.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "infinite_toggle")
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
        Checkbox.HandleEvent(self, event)
    end
    
    if controls.pauseToggle then controls.pauseToggle:Destroy() end
    controls.pauseToggle = Checkbox(controls.queueBGCapMin,
        UIUtil.SkinnableFile('/game/pause_btn/pause_btn_up.dds'),
        UIUtil.SkinnableFile('/game/pause_btn/pause_btn_down.dds'),
        UIUtil.SkinnableFile('/game/pause_btn/pause_btn_over.dds'),
        UIUtil.SkinnableFile('/game/pause_btn/pause_btn_down.dds'),
        UIUtil.SkinnableFile('/game/pause_btn/pause_btn_dis.dds'),
        UIUtil.SkinnableFile('/game/pause_btn/pause_btn_dis.dds'))
    -- place centered between queue scroll buttons
    LayoutHelpers.RightOf(controls.pauseToggle, controls.infiniteQueueToggle)
    controls.pauseToggle.Width:Set(30)
    controls.pauseToggle.OnCheck = function(self, checked)
        SetPaused(currentSelection, checked)
    end
    controls.pauseToggle.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "pause")
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
        Checkbox.HandleEvent(self, event)
    end

    controls.queueGrid.Left:Set(function() return controls.queueBGMiddle.Left() - 35 end)
    controls.queueGrid.Right:Set(function() return controls.queueBGCapMax.Right() - 20 end)
    controls.queueGrid.Height:Set(64)
    LayoutHelpers.AtVerticalCenterIn(controls.queueGrid, controls.queueBGMiddle, -30)
end

function DoLeftLayout()
    if not controls.queueGroup then
        controls.queueGroup = Group(controlClusterGroup, "queueGroup")
    end
    controls.queueGroup.Height:Set(55)
    controls.queueGroup.Bottom:Set(function() return controlClusterGroup.Bottom() + 5 end)
    controls.queueGroup.Left:Set(mfdControl.Left)
    controls.queueGroup.Right:Set(mfdControl.Right)
    
    -- set up queue
    if not controls.queueBGCapMin then
        controls.queueBGCapMin = Bitmap(controls.queueGroup)
    end
    controls.queueBGCapMin:SetTexture(UIUtil.SkinnableFile('/game/que_bmp/que-02_bmp_l.dds'))
    LayoutHelpers.AtBottomIn(controls.queueBGCapMin, controls.queueGroup, 3)
    LayoutHelpers.AtLeftIn(controls.queueBGCapMin, controls.queueGroup, -2)
    LayoutHelpers.ResetTop(controls.queueBGCapMin)
    LayoutHelpers.ResetRight(controls.queueBGCapMin)

    if not controls.queueBGCapMax then
        controls.queueBGCapMax = Bitmap(controls.queueGroup)
    end
    controls.queueBGCapMax:SetTexture(UIUtil.SkinnableFile('/game/que_bmp/que-02_bmp_r.dds'))
    LayoutHelpers.AtBottomIn(controls.queueBGCapMax, controls.queueGroup, 3)
    LayoutHelpers.AtRightIn(controls.queueBGCapMax, controls.queueGroup, 2)
    LayoutHelpers.ResetTop(controls.queueBGCapMax)
    LayoutHelpers.ResetLeft(controls.queueBGCapMax)

    if not controls.queueBGMiddle then
        controls.queueBGMiddle = Bitmap(controls.queueGroup)
    end
    controls.queueBGMiddle:SetTexture(UIUtil.SkinnableFile('/game/que_bmp/que-02_bmp_m.dds'))
    controls.queueBGMiddle.Left:Set(controls.queueBGCapMin.Right)
    controls.queueBGMiddle.Right:Set(controls.queueBGCapMax.Left)
    controls.queueBGMiddle.Top:Set(controls.queueBGCapMin.Top)
    LayoutHelpers.ResetWidth(controls.queueBGMiddle)
    controls.queueBGMiddle.Height:Set(controls.queueBGMiddle.BitmapHeight)
    LayoutHelpers.ResetBottom(controls.queueBGMiddle)
    controls.queueBGMiddle:SetTiled(true)

    if not controls.queueGrid then
        CreateQueueGrid(controls.queueBGMiddle)
    end

    if not controls.queueScrollBG then
        controls.queueScrollBG = Bitmap(controls.queueGroup)
    end
    controls.queueScrollBG.Depth:Set(function() return controls.queueBGCapMin.Depth() + 50 end)
    controls.queueScrollBG:SetTexture(UIUtil.SkinnableFile('/game/que_bmp/scroll-back_bmp.dds'))
    LayoutHelpers.RelativeTo(controls.queueScrollBG,
        controls.queueBGCapMin,
        UIUtil.SkinnableFile('/game/que_bmp/que_bmp_layout.lua'),
        "scroll-back_bmp",
        "que_bmp_l")


    -- scroll button behavior and look will change based on layout, so always create a new set of scroll buttons
    -- and get rid of any other queue scroll buttons
    if controls.scrollQueueBackButton then controls.scrollQueueBackButton:Destroy() end
    controls.scrollQueueBackButton = UIUtil.CreateButton(controls.queueBGCapMin
        , '/game/small-side_scr/arrow-left_scr_up.dds'
        , '/game/small-side_scr/arrow-left_scr_down.dds'
        , '/game/small-side_scr/arrow-left_scr_over.dds'
        , '/game/small-side_scr/arrow-left_scr_dis.dds')
    controls.scrollQueueBackButton.Depth:Set(function() return controls.queueScrollBG.Depth() + 1 end)
    LayoutHelpers.RelativeTo(controls.scrollQueueBackButton,
        controls.queueBGCapMin,
        UIUtil.SkinnableFile('/game/que_bmp/que_bmp_layout.lua'),
        "l_left_scroll",
        "que_bmp_l")
    controls.scrollQueueBackButton.OnClick = function(self)
        controls.queueGrid:ScrollLines("Horz", -1)
    end
    controls.scrollQueueBackButton:Disable()

    if controls.scrollQueueFwdButton then controls.scrollQueueFwdButton:Destroy() end
    controls.scrollQueueFwdButton = UIUtil.CreateButton(controls.queueBGCapMin
        , '/game/small-side_scr/arrow-right_scr_up.dds'
        , '/game/small-side_scr/arrow-right_scr_down.dds'
        , '/game/small-side_scr/arrow-right_scr_over.dds'
        , '/game/small-side_scr/arrow-right_scr_dis.dds')
    controls.scrollQueueFwdButton.Depth:Set(function() return controls.scrollQueueBackButton.Depth() + 1 end)
    LayoutHelpers.RelativeTo(controls.scrollQueueFwdButton,
        controls.queueBGCapMin,
        UIUtil.SkinnableFile('/game/que_bmp/que_bmp_layout.lua'),
        "l_right_scroll",
        "que_bmp_l")
    controls.scrollQueueFwdButton.OnClick = function(self)
        controls.queueGrid:ScrollLines("Horz", 1)
    end
    controls.scrollQueueFwdButton:Disable()

    if controls.infiniteQueueToggle then controls.infiniteQueueToggle:Destroy() end
    controls.infiniteQueueToggle = Checkbox(controls.queueBGCapMin,
        UIUtil.SkinnableFile('/game/infinite_btn/infinite_btn_up.dds'),
        UIUtil.SkinnableFile('/game/infinite_btn/infinite_btn_down.dds'),
        UIUtil.SkinnableFile('/game/infinite_btn/infinite_btn_over.dds'),
        UIUtil.SkinnableFile('/game/infinite_btn/infinite_btn_down.dds'),
        UIUtil.SkinnableFile('/game/infinite_btn/infinite_btn_dis.dds'),
        UIUtil.SkinnableFile('/game/infinite_btn/infinite_btn_dis.dds'))
    controls.infiniteQueueToggle.Top:Set(function() return controls.scrollQueueFwdButton.Bottom() - 5 end)
    -- place centered between queue scroll buttons
    controls.infiniteQueueToggle.Width:Set(30)
    controls.infiniteQueueToggle.Left:Set(function() return controls.scrollQueueBackButton.Left() + ((controls.scrollQueueBackButton.Width() + controls.scrollQueueFwdButton.Width() - controls.scrollQueueFwdButton.Width()) / 2) - 8 end)
    controls.infiniteQueueToggle.OnClick = function(self)
        if currentInfiniteQueueCheckStatus then
            currentInfiniteQueueCheckStatus = false
        else
            currentInfiniteQueueCheckStatus = true
        end
        OnInfiniteQueueToggle(currentInfiniteQueueCheckStatus)
    end
    
    if controls.pauseToggle then controls.pauseToggle:Destroy() end
    controls.pauseToggle = Checkbox(controls.queueBGCapMin,
        UIUtil.SkinnableFile('/game/pause_btn/pause_btn_up.dds'),
        UIUtil.SkinnableFile('/game/pause_btn/pause_btn_down.dds'),
        UIUtil.SkinnableFile('/game/pause_btn/pause_btn_over.dds'),
        UIUtil.SkinnableFile('/game/pause_btn/pause_btn_down.dds'),
        UIUtil.SkinnableFile('/game/pause_btn/pause_btn_dis.dds'),
        UIUtil.SkinnableFile('/game/pause_btn/pause_btn_dis.dds'))
    -- place centered between queue scroll buttons
    controls.pauseToggle.Width:Set(30)
    LayoutHelpers.RightOf(controls.pauseToggle, controls.infiniteQueueToggle)
    controls.pauseToggle.OnCheck = function(self, checked)
        SetPaused(currentSelection, checked)
    end
    controls.pauseToggle.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "pause")
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
        Checkbox.HandleEvent(self, event)
    end

    controls.queueGrid.Left:Set(function() return controls.queueBGMiddle.Left() - 35 end)
    controls.queueGrid.Right:Set(function() return controls.queueBGCapMax.Right() - 20 end)
    controls.queueGrid.Height:Set(64)
    LayoutHelpers.AtVerticalCenterIn(controls.queueGrid, controls.queueBGMiddle, -30)
end

function DoRightLayout()
    if not controls.queueGroup then
        controls.queueGroup = Group(controlClusterGroup, "queueGroup")
    end
    controls.queueGroup.Height:Set(60)
    controls.queueGroup.Bottom:Set(controlClusterGroup.Bottom)
    controls.queueGroup.Left:Set(mfdControl.Left)
    controls.queueGroup.Right:Set(mfdControl.Right)
    
    -- set up queue
    if not controls.queueBGCapMin then
        controls.queueBGCapMin = Bitmap(controls.queueGroup)
    end
    controls.queueBGCapMin:SetTexture(UIUtil.SkinnableFile('/game/que_bmp/que-02_bmp_l.dds'))
    LayoutHelpers.AtBottomIn(controls.queueBGCapMin, controls.queueGroup, 3)
    LayoutHelpers.AtLeftIn(controls.queueBGCapMin, controls.queueGroup, -2)
    LayoutHelpers.ResetTop(controls.queueBGCapMin)
    LayoutHelpers.ResetRight(controls.queueBGCapMin)

    if not controls.queueBGCapMax then
        controls.queueBGCapMax = Bitmap(controls.queueGroup)
    end
    controls.queueBGCapMax:SetTexture(UIUtil.SkinnableFile('/game/que_bmp/que-02_bmp_r.dds'))
    LayoutHelpers.AtBottomIn(controls.queueBGCapMax, controls.queueGroup, 3)
    LayoutHelpers.AtRightIn(controls.queueBGCapMax, controls.queueGroup, 2)
    LayoutHelpers.ResetTop(controls.queueBGCapMax)
    LayoutHelpers.ResetLeft(controls.queueBGCapMax)

    if not controls.queueBGMiddle then
        controls.queueBGMiddle = Bitmap(controls.queueGroup)
    end
    controls.queueBGMiddle:SetTexture(UIUtil.SkinnableFile('/game/que_bmp/que-02_bmp_m.dds'))
    controls.queueBGMiddle.Left:Set(controls.queueBGCapMin.Right)
    controls.queueBGMiddle.Right:Set(controls.queueBGCapMax.Left)
    controls.queueBGMiddle.Top:Set(controls.queueBGCapMin.Top)
    LayoutHelpers.ResetWidth(controls.queueBGMiddle)
    controls.queueBGMiddle.Height:Set(controls.queueBGMiddle.BitmapHeight)
    LayoutHelpers.ResetBottom(controls.queueBGMiddle)
    controls.queueBGMiddle:SetTiled(true)

    if not controls.queueGrid then
        CreateQueueGrid(controls.queueBGMiddle)
    end

    if not controls.queueScrollBG then
        controls.queueScrollBG = Bitmap(controls.queueGroup)
    end
    controls.queueScrollBG.Depth:Set(function() return controls.queueBGCapMin.Depth() + 50 end)
    controls.queueScrollBG:SetTexture(UIUtil.SkinnableFile('/game/que_bmp/scroll-back_bmp.dds'))
    LayoutHelpers.RelativeTo(controls.queueScrollBG,
        controls.queueBGCapMin,
        UIUtil.SkinnableFile('/game/que_bmp/que_bmp_layout.lua'),
        "scroll-back_bmp",
        "que_bmp_l")


    -- scroll button behavior and look will change based on layout, so always create a new set of scroll buttons
    -- and get rid of any other queue scroll buttons
    if controls.scrollQueueBackButton then controls.scrollQueueBackButton:Destroy() end
    controls.scrollQueueBackButton = UIUtil.CreateButton(controls.queueBGCapMin
        , '/game/small-side_scr/arrow-left_scr_up.dds'
        , '/game/small-side_scr/arrow-left_scr_down.dds'
        , '/game/small-side_scr/arrow-left_scr_over.dds'
        , '/game/small-side_scr/arrow-left_scr_dis.dds')
    controls.scrollQueueBackButton.Depth:Set(function() return controls.queueScrollBG.Depth() + 1 end)
    LayoutHelpers.RelativeTo(controls.scrollQueueBackButton,
        controls.queueBGCapMin,
        UIUtil.SkinnableFile('/game/que_bmp/que_bmp_layout.lua'),
        "l_left_scroll",
        "que_bmp_l")
    controls.scrollQueueBackButton.OnClick = function(self)
        controls.queueGrid:ScrollLines("Horz", -1)
    end
    controls.scrollQueueBackButton:Disable()

    if controls.scrollQueueFwdButton then controls.scrollQueueFwdButton:Destroy() end
    controls.scrollQueueFwdButton = UIUtil.CreateButton(controls.queueBGCapMin
        , '/game/small-side_scr/arrow-right_scr_up.dds'
        , '/game/small-side_scr/arrow-right_scr_down.dds'
        , '/game/small-side_scr/arrow-right_scr_over.dds'
        , '/game/small-side_scr/arrow-right_scr_dis.dds')
    controls.scrollQueueFwdButton.Depth:Set(function() return controls.scrollQueueBackButton.Depth() + 1 end)
    LayoutHelpers.RelativeTo(controls.scrollQueueFwdButton,
        controls.queueBGCapMin,
        UIUtil.SkinnableFile('/game/que_bmp/que_bmp_layout.lua'),
        "l_right_scroll",
        "que_bmp_l")
    controls.scrollQueueFwdButton.OnClick = function(self)
        controls.queueGrid:ScrollLines("Horz", 1)
    end
    controls.scrollQueueFwdButton:Disable()

    if controls.infiniteQueueToggle then controls.infiniteQueueToggle:Destroy() end
    controls.infiniteQueueToggle = Checkbox(controls.queueBGCapMin,
        UIUtil.SkinnableFile('/game/infinite_btn/infinite_btn_up.dds'),
        UIUtil.SkinnableFile('/game/infinite_btn/infinite_btn_down.dds'),
        UIUtil.SkinnableFile('/game/infinite_btn/infinite_btn_over.dds'),
        UIUtil.SkinnableFile('/game/infinite_btn/infinite_btn_down.dds'),
        UIUtil.SkinnableFile('/game/infinite_btn/infinite_btn_dis.dds'),
        UIUtil.SkinnableFile('/game/infinite_btn/infinite_btn_dis.dds'))
    controls.infiniteQueueToggle.Top:Set(function() return controls.scrollQueueFwdButton.Bottom() - 5 end)
    -- place centered between queue scroll buttons
    controls.infiniteQueueToggle.Width:Set(30)
    controls.infiniteQueueToggle.Left:Set(function() return controls.scrollQueueBackButton.Left() + ((controls.scrollQueueBackButton.Width() + controls.scrollQueueFwdButton.Width() - controls.scrollQueueFwdButton.Width()) / 2) - 8 end)
    controls.infiniteQueueToggle.OnClick = function(self)
        if currentInfiniteQueueCheckStatus then
            currentInfiniteQueueCheckStatus = false
        else
            currentInfiniteQueueCheckStatus = true
        end
        OnInfiniteQueueToggle(currentInfiniteQueueCheckStatus)
    end
    
    if controls.pauseToggle then controls.pauseToggle:Destroy() end
    controls.pauseToggle = Checkbox(controls.queueBGCapMin,
        UIUtil.SkinnableFile('/game/pause_btn/pause_btn_up.dds'),
        UIUtil.SkinnableFile('/game/pause_btn/pause_btn_down.dds'),
        UIUtil.SkinnableFile('/game/pause_btn/pause_btn_over.dds'),
        UIUtil.SkinnableFile('/game/pause_btn/pause_btn_down.dds'),
        UIUtil.SkinnableFile('/game/pause_btn/pause_btn_dis.dds'),
        UIUtil.SkinnableFile('/game/pause_btn/pause_btn_dis.dds'))
    -- place centered between queue scroll buttons
    controls.pauseToggle.Width:Set(30)
    LayoutHelpers.RightOf(controls.pauseToggle, controls.infiniteQueueToggle)
    controls.pauseToggle.OnCheck = function(self, checked)
        SetPaused(currentSelection, checked)
    end
    controls.pauseToggle.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "pause")
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
        Checkbox.HandleEvent(self, event)
    end

    controls.queueGrid.Left:Set(function() return controls.queueBGMiddle.Left() - 35 end)
    controls.queueGrid.Right:Set(function() return controls.queueBGCapMax.Right() - 20 end)
    controls.queueGrid.Height:Set(64)
    LayoutHelpers.AtVerticalCenterIn(controls.queueGrid, controls.queueBGMiddle, -30)
end

function SetLayout(layout)
    if layout == 'bottom' then
        DoBottomLayout()
    elseif layout == 'left' then
        DoLeftLayout()
    elseif layout == 'right' then
        DoRightLayout()
    end
end

function SetupQueueControl(parent, inMFDControl, inOrdersControl)
    mfdControl = inMFDControl
    ordersControl = inOrdersControl
    controlClusterGroup = parent

    SetLayout(UIUtil.currentLayout)
    return controls.queueGroup
end

function SetInfiniteQueueToggle(state)
    if state then
        controls.infiniteQueueToggle:Enable()
    else
        controls.infiniteQueueToggle:Disable()
    end
end

function SetInfinityCheck(state)       
    currentInfiniteQueueCheckStatus = state
    controls.infiniteQueueToggle:SetCheck(state)
end

-- command queue is a table of unit id's and how many to build, in sequence to be built
local currentCommandQueue = false

function CreateQueueIcon(parent)
    local bg = Bitmap(parent) -- TODO unit specific BG here
    bg:SetSolidColor('black')
    #bg.Width:Set(GameCommon.iconBmpWidth)
    #bg.Height:Set(GameCommon.iconBmpHeight)
    local icon = Bitmap(bg)
    LayoutHelpers.AtLeftTopIn(icon, bg)
    #LayoutHelpers.FillParent(icon, bg)
    local text = UIUtil.CreateText(icon, "", 18, UIUtil.bodyFont)
    LayoutHelpers.AtLeftTopIn(text, icon, 5, 5)
    text:SetDropShadow(true)
    text:SetColor('white')
    text:DisableHitTest()
    bg.text = text
    bg.unitIcon = icon
    bg:Hide()

    return bg
end

function ClearQueueGrid()
    ClearSessionExtraSelectList()
    controls.queueGrid:DeleteAndDestroyAll(true)
    controls.queueGrid:AppendRows(1, true)
    controls.queueGrid:EndBatch()
end

function PopulateQueueGrid(numIcons)
    if numIcons == 0 then
        controls.scrollQueueFwdButton:Disable()
        controls.scrollQueueBackButton:Disable()
        ClearQueueGrid()
        return
    end
    local cols, rows = controls.queueGrid:GetDimensions()
    if numIcons > cols then
        controls.queueGrid:AppendCols(numIcons - cols, true)
        for i = cols + 1, numIcons do
            controls.queueGrid:SetItem(CreateQueueIcon(controls.queueGrid), i, 1, true)
        end
    elseif numIcons < cols then
        for i = cols, numIcons + 1, -1 do
            controls.queueGrid:DestroyItem(i, 1, true)
            controls.queueGrid:DeleteCol(i, true)
        end
    end
end

function SetQueueGrid(queue, selection)
    local numIcons = table.getn(queue)
    
    PopulateQueueGrid(numIcons)
    
    for i = 1, numIcons do
        local icon = controls.queueGrid:GetItem(i, 1)
        if not icon then
            LOG("SetQueueGrid attempt to get item ", i, " but it's not there.")
            continue
        end
        local bpid = queue[i].id
        local blueprint = __blueprints[bpid]
        local iconName, upIconName, downIconName, overIconName = GameCommon.GetCachedUnitIconFileNames(blueprint)
        icon.unitIcon:SetTexture(upIconName)
        icon.text:SetText(tostring(queue[i].count))
        icon._blueprintId = bpid
        icon._index = i -- store array index for determining who was clicked on

        icon.HandleEvent = function(self, event)
            if event.Type == 'MouseEnter' then
                local blueprint = __blueprints[self._blueprintId]
                import('/lua/modules/ui/game/unitviewDetail.lua').Show(blueprint, currentSelection[1], self._blueprintId)
            elseif event.Type == 'MouseExit' then
                    import('/lua/modules/ui/game/unitviewDetail.lua').Hide()
            elseif event.Type == 'ButtonPress' or event.Type == 'ButtonDClick' then
                local count = 1
                if event.Modifiers.Ctrl == true and event.Modifiers.Shift == nil then
                    count = 50
                elseif event.Modifiers.Shift == true and event.Modifiers.Ctrl == nil then
                    count = 5
                elseif event.Modifiers.Shift == true and event.Modifiers.Ctrl == true then
                    count = 500
                end

                if event.Modifiers.Left then
                    IncreaseBuildCountInQueue(self._index, count)
                else
                    DecreaseBuildCountInQueue(self._index, count)
                end
            end
            Bitmap.HandleEvent(self, event)
        end

    end
    controls.queueGrid:EndBatch()
    local visCols, visRows = controls.queueGrid:GetVisible()
    if (visCols * visRows) >= numIcons then
        controls.scrollQueueFwdButton:Disable()
        controls.scrollQueueBackButton:Disable()
    else
        controls.scrollQueueFwdButton:Enable()
        controls.scrollQueueBackButton:Enable()
    end
end

function SetSelectionIcons(sortedUnitTable, sortedObjectTables)
    ClearQueueGrid()
    
    local numTypes = table.getn(sortedUnitTable)
    local numIcons = 0
    
    if numTypes then
        local iconBPID = nil
        local cols, rows = controls.queueGrid:GetDimensions()
        numIcons = table.getn(sortedUnitTable)
        
        PopulateQueueGrid(numIcons)
        
        local i = 1
        for index, v in sortedUnitTable do
            local icon = controls.queueGrid:GetItem(i, 1)
            if not icon then
                LOG("SetQueueGrid attempt to get item ", i, " but it's not there.")
            end
            local blueprint = __blueprints[index]
            local iconName, upIconName, downIconName, overIconName = GameCommon.GetCachedUnitIconFileNames(blueprint)
            icon.unitIcon:SetTexture(upIconName)
            icon.text:SetText(v)
            icon._unitType = index
            i = i + 1
            
            icon.HandleEvent = function(self, event)
                if event.Type == 'ButtonPress' or event.Type == 'ButtonDClick' then
                    if event.Modifiers.Left then
                        SelectUnits(sortedObjectTables[self._unitType])
                    else
                        local index = 1
                        local newUnitSelectTable = {}
                        for i, v in sortedObjectTables do
                            if i != self._unitType then
                                for y, z in v do
                                    newUnitSelectTable[index] = z
                                    index = index + 1
                                end
                            end
                        end
                        SelectUnits(newUnitSelectTable)
                    end
                end
                Bitmap.HandleEvent(self, event)
            end
        end
    end
        
    
    controls.queueGrid:EndBatch()
    local visCols, visRows = controls.queueGrid:GetVisible()
    if (visCols * visRows) >= numIcons then
        controls.scrollQueueFwdButton:Disable()
        controls.scrollQueueBackButton:Disable()
    else
        controls.scrollQueueFwdButton:Enable()
        controls.scrollQueueBackButton:Enable()
    end
end

function EnablePauseToggle()
    controls.pauseToggle:Enable()
end

-- called on new selection, determines what, if anything, goes in the order queue
function CheckForOrderQueue(selection, newSelection)
    if controls.pauseToggle then controls.pauseToggle:Disable() end
    currentSelection = newSelection
    if currentSelection[1] and EntityCategoryContains(categories.FACTORY, currentSelection[1]) then
        controls.infiniteQueueToggle:Enable()
    else
        controls.infiniteQueueToggle:Disable()
    end
    -- only show a queue if single item is selected
    if table.getn(selection) == 1 then
        currentCommandQueue = SetCurrentFactoryForQueueDisplay(selection[1])
        -- render the command queue
        if currentCommandQueue then
            SetQueueGrid(currentCommandQueue, selection)
        else
            ClearQueueGrid()
        end
    else
        ClearQueueGrid()
    end
end

function OnQueueChanged(newQueue)
    if newQueue then
        SetQueueGrid(newQueue)
    else
        ClearQueueGrid()
    end
end

function OnInfiniteQueueToggle(checked)
    controls.infiniteQueueToggle:SetCheck(checked)
    for i,v in currentSelection do
        if checked then
            v:ProcessInfo('SetRepeatQueue', 'true')
        else
            v:ProcessInfo('SetRepeatQueue', 'false')
        end
    end
end

function ToggleEnhancementWindow(state, enhancementWindow)
    if state then
        if UIUtil.currentLayout == 'bottom' then
            controls.queueBGCapMin.Left:Set(enhancementWindow.Right)
        end
    else
        if UIUtil.currentLayout == 'bottom' then
            LayoutHelpers.AtLeftIn(controls.queueBGCapMin, controls.constructionGroup, -2)
            LayoutHelpers.AtLeftIn(controls.queueBGCapMin, controls.queueGroup, -2)
        end
    end
end