--*****************************************************************************
--* File: lua/modules/ui/game/orders.lua
--* Author: Chris Blackwell
--* Summary: Unit orders UI
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Button = import('/lua/modules/maui/button.lua').Button
local Grid = import('/lua/modules/maui/grid.lua').Grid
local Checkbox = import('/lua/modules/maui/checkbox.lua').Checkbox
local GameCommon = import('/lua/modules/ui/game/gamecommon.lua')
local Button = import('/lua/modules/maui/button.lua').Button
local Tooltip = import('/lua/modules/ui/game/tooltip.lua')
local TooltipInfo = import('/lua/modules/ui/help/tooltips.lua')
local Prefs = import('/lua/modules/ui/prefs.lua')
local Combo = import('/lua/modules/ui/controls/combo.lua').Combo
local IntegerSlider = import('/lua/modules/maui/slider.lua').IntegerSlider
local Options = import('/lua/modules/ui/game/options.lua')

local controls =
{
    bg = false,
    pause = false,
    armycombo = false,
    speedSlider = false,
    currentSpeed = false,
}

-- positioning controls, don't belong to file
local controlClusterGroup = false
local mfdControl = false
local layoutVar = false
local glowThread = false


local function DoBottomLayout()
    if not controls.bg then
        controls.bg = Bitmap(controlClusterGroup)
    end
    controls.bg:SetTexture(UIUtil.SkinnableFile('/game/replay/panel_bmp.dds'))
    LayoutHelpers.AtRightIn(controls.bg, controlClusterGroup, 6)
    LayoutHelpers.AtTopIn(controls.bg, controlClusterGroup, -3)
    LayoutHelpers.ResetLeft(controls.bg)
    
    if not controls.armycombo then
        controls.armycombo = Combo(controls.bg)
    end
    controls.armycombo.Left:Set(function() return controls.bg.Left() + 10 end)
    controls.armycombo.Top:Set(function() return controls.bg.Top() + 10 end)
    controls.armycombo.Width:Set(140)
    controls.armycombo.Height:Set(20)
            
    if not controls.pause then
        controls.pause = Checkbox(controls.bg, 
            UIUtil.UIFile('/game/options_btn/pause_btn_up.dds'),
            UIUtil.UIFile('/game/options_btn/play_btn_up.dds'),
            UIUtil.UIFile('/game/options_btn/pause_btn_over.dds'),
            UIUtil.UIFile('/game/options_btn/play_btn_over.dds'),
            UIUtil.UIFile('/game/options_btn/pause_btn_dis.dds'),
            UIUtil.UIFile('/game/options_btn/play_btn_dis.dds'))
    end
    LayoutHelpers.RelativeTo(controls.pause, controls.bg, 
        UIUtil.SkinnableFile('/game/replay/replay_layout.lua'), 
        "l_pause_btn_up", "panel_bmp")
        
    if not controls.speedSlider then
        controls.speedSlider = IntegerSlider(controls.bg, false, -10, 10, 1, 
            UIUtil.SkinnableFile('/slider02/slider_btn_up.dds'), 
            UIUtil.SkinnableFile('/dialogs/options/slider-back_bmp.dds'))    
    end
    controls.speedSlider.Left:Set(function() return controls.bg.Left() + 10 end)
    controls.speedSlider.Right:Set(function() return controls.bg.Right() - 40 end)
    controls.speedSlider.Top:Set(function() return controls.bg.Bottom() - 40 end)
    controls.speedSlider._background.Left:Set(controls.speedSlider.Left)
    controls.speedSlider._background.Right:Set(controls.speedSlider.Right)
    controls.speedSlider._background.Top:Set(controls.speedSlider.Top)
    controls.speedSlider._background.Bottom:Set(controls.speedSlider.Bottom)
    
    if not controls.currentSpeed then
        controls.currentSpeed = UIUtil.CreateText(controls.speedSlider, "", 12)
    end
    LayoutHelpers.RightOf(controls.currentSpeed, controls.speedSlider)
end

local function DoLeftLayout()

    if not controls.bg then
        controls.bg = Bitmap(controlClusterGroup)
    end
    
    controls.bg:SetTexture(UIUtil.SkinnableFile('/game/replay-vertical/panel_bmp.dds'))
    LayoutHelpers.Below(controls.bg, mfdControl)
    LayoutHelpers.ResetRight(controls.bg)

    if not controls.armycombo then
        controls.armycombo = Combo(controls.bg)
    end
    controls.armycombo.Left:Set(function() return controls.bg.Left() + 15 end)
    controls.armycombo.Top:Set(function() return controls.bg.Top() + 11 end)
    controls.armycombo.Width:Set(140)
    controls.armycombo.Height:Set(20)
    
    if not controls.pause then
        controls.pause = Button(controls.bg, 
            UIUtil.UIFile('/game/options_btn/pause_btn_up.dds'),
            UIUtil.UIFile('/game/options_btn/play_btn_up.dds'),
            UIUtil.UIFile('/game/options_btn/pause_btn_over.dds'),
            UIUtil.UIFile('/game/options_btn/play_btn_over.dds'),
            UIUtil.UIFile('/game/options_btn/pause_btn_dis.dds'),
            UIUtil.UIFile('/game/options_btn/play_btn_dis.dds'))
    end
    LayoutHelpers.RelativeTo(controls.pause, controls.bg, 
        UIUtil.SkinnableFile('/game/replay-vertical/replay-vertical_layout.lua'), 
        "l_pause_btn_up", "panel_bmp")
        
    if not controls.speedSlider then
        controls.speedSlider = IntegerSlider(controls.bg, false, -10, 10, 1, 
            UIUtil.SkinnableFile('/slider02/slider_btn_up.dds'), 
            UIUtil.SkinnableFile('/dialogs/options/slider-back_bmp.dds'))    
    end
    controls.speedSlider.Left:Set(function() return controls.bg.Left() + 10 end)
    controls.speedSlider.Right:Set(function() return controls.bg.Right() - 40 end)
    controls.speedSlider.Top:Set(function() return controls.bg.Bottom() - 40 end)
    controls.speedSlider._background.Left:Set(controls.speedSlider.Left)
    controls.speedSlider._background.Right:Set(controls.speedSlider.Right)
    controls.speedSlider._background.Top:Set(controls.speedSlider.Top)
    controls.speedSlider._background.Bottom:Set(controls.speedSlider.Bottom)
    
    if not controls.currentSpeed then
        controls.currentSpeed = UIUtil.CreateText(controls.speedSlider, "", 12)
    end
    LayoutHelpers.RightOf(controls.currentSpeed, controls.speedSlider)
end

local function DoRightLayout()
    if not controls.bg then
        controls.bg = Bitmap(controlClusterGroup)
    end
    controls.bg:SetTexture(UIUtil.SkinnableFile('/game/replay-vertical/panel_bmp.dds'))
    LayoutHelpers.Below(controls.bg, mfdControl)
    LayoutHelpers.ResetRight(controls.bg)
    
    if not controls.armycombo then
        controls.armycombo = Combo(controls.bg)
    end
    controls.armycombo.Left:Set(function() return controls.bg.Left() + 15 end)
    controls.armycombo.Top:Set(function() return controls.bg.Top() + 11 end)
    controls.armycombo.Width:Set(140)
    controls.armycombo.Height:Set(20)
    
    if not controls.pause then
        controls.pause = Button(controls.bg, 
            UIUtil.UIFile('/game/options_btn/pause_btn_up.dds'),
            UIUtil.UIFile('/game/options_btn/play_btn_up.dds'),
            UIUtil.UIFile('/game/options_btn/pause_btn_over.dds'),
            UIUtil.UIFile('/game/options_btn/play_btn_over.dds'),
            UIUtil.UIFile('/game/options_btn/pause_btn_dis.dds'),
            UIUtil.UIFile('/game/options_btn/play_btn_dis.dds'))
    end
    LayoutHelpers.RelativeTo(controls.pause, controls.bg, 
        UIUtil.SkinnableFile('/game/replay-vertical/replay-vertical_layout.lua'), 
        "l_pause_btn_up", "panel_bmp")
        
    if not controls.speedSlider then
        controls.speedSlider = IntegerSlider(controls.bg, false, -10, 10, 1, 
            UIUtil.SkinnableFile('/slider02/slider_btn_up.dds'), 
            UIUtil.SkinnableFile('/dialogs/options/slider-back_bmp.dds'))    
    end
    controls.speedSlider.Left:Set(function() return controls.bg.Left() + 10 end)
    controls.speedSlider.Right:Set(function() return controls.bg.Right() - 40 end)
    controls.speedSlider.Top:Set(function() return controls.bg.Bottom() - 40 end)
    controls.speedSlider._background.Left:Set(controls.speedSlider.Left)
    controls.speedSlider._background.Right:Set(controls.speedSlider.Right)
    controls.speedSlider._background.Top:Set(controls.speedSlider.Top)
    controls.speedSlider._background.Bottom:Set(controls.speedSlider.Bottom)
    
    if not controls.currentSpeed then
        controls.currentSpeed = UIUtil.CreateText(controls.speedSlider, "", 12)
    end
    LayoutHelpers.RightOf(controls.currentSpeed, controls.speedSlider)
end

function SetLayout(layout)
    layoutVar = layout
    if layout == 'bottom' then
        DoBottomLayout()
    elseif layout == 'left' then
        DoLeftLayout()
    elseif layout == 'right' then
        DoRightLayout()
    end
    #Pausing during a replay currently causes a LOT of desync's.  Disable the button for now
    controls.pause:Disable()
    controls.pause.OnCheck = function(self, checked)
        Options.PauseGame(checked)
    end
    local itemArray = {}
    controls.armycombo.keyMap = {}
    local armyTable = GetArmiesTable()
    for index, val in armyTable.armiesTable do
        itemArray[index] = val.nickname
        controls.armycombo.keyMap[val.name] = index
    end
    local maxArmies = table.getsize(armyTable.armiesTable) - 1
    table.insert(itemArray, "Observer")
    local defValue = table.getsize(itemArray)
    controls.armycombo:AddItems(itemArray, defValue)
    controls.armycombo.OnClick = function(self, index, text)
        local armyNum = index - 1
        if armyNum > maxArmies then
            ConExecute('SetFocusArmy -1')
        else
            ConExecute('SetFocusArmy ' .. armyNum)
        end
    end
    controls.speedSlider.OnValueChanged = function(self, newValue)
        controls.currentSpeed:SetText(string.format("%+d", math.floor(tostring(newValue))))
    end
    
    controls.speedSlider.OnValueSet = function(self, newValue)
        ConExecute("WLD_GameSpeed " .. newValue)
    end
    
    -- set initial value
    controls.speedSlider:SetValue(0)
end

-- called from gamemain to create control
function SetupReplayControl(parent, mfd)
    controlClusterGroup = parent
    mfdControl = mfd

    SetLayout(UIUtil.currentLayout)

    return controls.bg
end
