--*****************************************************************************
--* File: lua/modules/ui/game/score.lua
--* Author: Chris Blackwell
--* Summary: In game score dialog
--*
--* Copyright  :005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

--[[
Armies table looks like this:
{
    numArmies (int),
    focusArmy (index),
    armiesTable (table) {
        [1] {
            name (string),
            nickname (string)
            faction (index),
            color (color),
            iconColor (color),
            showScore (bool),
        }
        etc... for all armies
    }
}
--]]

-- current score will contain the most recent score update from the sync
currentScores = false

local factionIcons = {
    "/widgets/faction-icons-alpha_bmp/uef_ico.dds",
    "/widgets/faction-icons-alpha_bmp/aeon_ico.dds",
    "/widgets/faction-icons-alpha_bmp/cybran_ico.dds"
}

function GetFactionIcon(factionIndex)
    return factionIcons[factionIndex + 1]
end

--if Prefs.GetOption('ui_animation') == 'off'


local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local GameMain = import('/lua/modules/ui/game/gamemain.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Text = import('/lua/modules/maui/text.lua').Text
local Grid = import('/lua/modules/maui/Grid.lua').Grid
local Prefs = import('/lua/modules/ui/prefs.lua')

local timeThread = false
local walltime = Prefs.GetFromCurrentProfile("scorewalltime") or false
local dialog = false
local overlay = false
local focusArmy = 0
local scoreGrid = false
local scoreFields = {}
local animationTable = {}
local bg = false
local bgGlow = false
local bgBrackets = false
local ThreadHandle = false

local overlay = {
    group = false,
    armyLines = {},
    time = false,
    units = false,
}

function CreateArmyLine(parent)
    local group = Group(parent)
    
    group.faction = Bitmap(group)    
    group.teamcolor = Bitmap(group)    
    group.playername = UIUtil.CreateText(group, "", 14, UIUtil.bodyFont)    
    group.score = UIUtil.CreateText(group, "", 14, UIUtil.bodyFont)    
    group.namebg = Bitmap(group)
    
    group.middlebg = Bitmap(group, UIUtil.UIFile('/dialogs/score-overlay/score_brd_m.dds'))
    group.rightbg = Bitmap(group, UIUtil.UIFile('/dialogs/score-overlay/score_brd_vert_r.dds'))
    group.leftbg = Bitmap(group, UIUtil.UIFile('/dialogs/score-overlay/score_brd_vert_l.dds'))
        
    return group
end

function LayoutLine(line)
    line.score.Right:Set(line.Right)
    
    line.playername.Right:Set(function() return line.score.Left() - 10 end)
    
    line.faction.Right:Set(function() return line.playername.Left() - 11 end)
    line.faction.Bottom:Set(function() return line.Bottom() - 3 end)
    line.faction.Height:Set(15)
    line.faction.Width:Set(15)
    LayoutHelpers.FillParent(line.teamcolor, line.faction)
    line.teamcolor.Depth:Set(function() return line.faction.Depth() - 1 end)
    
    LayoutHelpers.AtVerticalCenterIn(line.score, line)
    LayoutHelpers.AtVerticalCenterIn(line.playername, line)
    
    line.namebg.Left:Set(function() return line.playername.Left() - 3 end)
    line.namebg.Right:Set(function() return line.score.Right() + 3 end)
    line.namebg.Bottom:Set(line.playername.Bottom)
    line.namebg.Height:Set(function () return line.playername.Height() + 1 end)
    line.namebg.Depth:Set(function() return line.playername.Depth() - 1 end)
    
    line.middlebg.Left:Set(line.playername.Left)
    line.middlebg.Right:Set(line.namebg.Right)
    line.middlebg.Bottom:Set(line.Bottom)
    line.middlebg.Depth:Set(function() return line.namebg.Depth() - 1 end)
    
    line.rightbg.Left:Set(function() return line.middlebg.Right() - 3 end)
    line.rightbg.Bottom:Set(line.middlebg.Bottom)
    line.rightbg.Depth:Set(line.middlebg.Depth)
    
    line.leftbg.Right:Set(line.middlebg.Left)
    line.leftbg.Bottom:Set(line.middlebg.Bottom)
    line.leftbg.Depth:Set(line.middlebg.Depth)
end

function UpdateTime()
    while overlay.time do
        local time = os.date("*t")
        local ampm = ""
        if time.hour > 11 then
            ampm = "<LOC _PM>PM"
        else
            ampm = "<LOC _AM>AM"
        end
        overlay.time:SetText(LOCF("%02d:%02d:%02d %s", math.mod(time.hour, 12), time.min, time.sec, ampm))
        WaitSeconds(1)
    end
end

function CreateScoreOverlay()
    local worldView = import('/lua/modules/ui/game/borders.lua').GetMapGroup()
    
    local armiesInfo = GetArmiesTable()
    
    overlay.group = Group(worldView, "scoreOverlayGroup")
    overlay.group:SetRenderPass(UIUtil.UIRP_PostGlow)
    overlay.group.Top:Set(worldView.Top)
    overlay.group.Left:Set(worldView.Left)
    overlay.group.Right:Set(function() return worldView.Right() - 16 end)
    overlay.group.Bottom:Set(function() return worldView.Bottom() - 25 end)
    
    local counter = 1
    for i, v in armiesInfo.armiesTable do
        if v.showScore and not v.civilian then
            overlay.armyLines[counter] = CreateArmyLine(overlay.group)
            counter = counter + 1
        end
    end
    
    local index = 0
    for i, v in overlay.armyLines do
        index = i
        if index == 1 then
            v.Bottom:Set(overlay.group.Bottom)
        else
            v.Bottom:Set(overlay.armyLines[index-1].Top)
        end
        v.Right:Set(overlay.group.Right)
        v.Height:Set(v.middlebg.Height)
        v.Left:Set(v.faction.Left)
        LayoutLine(v)
    end
    
    
    overlay.unitsBG = Bitmap(overlay.group, UIUtil.UIFile('/dialogs/time-units-tabs/panel-units_bmp.dds'))
    LayoutHelpers.AtHorizontalCenterIn(overlay.unitsBG, GetFrame(0), 146)
    LayoutHelpers.AtTopIn(overlay.unitsBG, worldView, 1)
    
    overlay.unitsTitle = UIUtil.CreateText(overlay.group, "<LOC score_overlay_0000>Units", 14)
    LayoutHelpers.AtLeftTopIn(overlay.unitsTitle, overlay.unitsBG, 25, 15)
    
    overlay.units = UIUtil.CreateText(overlay.group, "", 12, UIUtil.bodyFont)
    LayoutHelpers.AtCenterIn(overlay.units, overlay.unitsBG, 0, 25)
    
    overlay.timeBG = Bitmap(worldView, UIUtil.UIFile('/dialogs/time-units-tabs/panel-time_bmp.dds'))
    LayoutHelpers.AtHorizontalCenterIn(overlay.timeBG, GetFrame(0), -146)
    LayoutHelpers.AtTopIn(overlay.timeBG, worldView, 1)
    overlay.timeBG:SetRenderPass(UIUtil.UIRP_PostGlow)
    overlay.timeBG.Depth:Set(100)
    overlay.timeBG.HandleEvent = function(self, event)
        if event.Type == 'ButtonPress' or event.Type == 'ButtonDClick' then
            walltime = not walltime
            Prefs.SetToCurrentProfile("scorewalltime", walltime)
            if walltime then    
                timeThread = ForkThread(UpdateTime)
            else
                KillThread(timeThread)
            end
        end
    end
    
    overlay.timeTitle = UIUtil.CreateText(overlay.timeBG, "<LOC score_overlay_0001>Time", 14)
    LayoutHelpers.AtLeftTopIn(overlay.timeTitle, overlay.timeBG, 25, 15)
    overlay.timeTitle:DisableHitTest()
    
    overlay.time = UIUtil.CreateText(overlay.timeBG, "", 12, UIUtil.bodyFont)
    LayoutHelpers.AtCenterIn(overlay.time, overlay.timeBG, 0, 25)
    overlay.time:DisableHitTest()
    
    overlay.bottomRight = Bitmap(overlay.group, UIUtil.UIFile('/dialogs/score-overlay/score_brd_lr.dds'))
    overlay.bottomRight.Top:Set(overlay.armyLines[1].Bottom)
    overlay.bottomRight.Right:Set(function() return overlay.armyLines[1].rightbg.Right() + 1 end)
    
    overlay.bottomLeft = Bitmap(overlay.group, UIUtil.UIFile('/dialogs/score-overlay/score_brd_ll.dds'))
    overlay.bottomLeft.Top:Set(overlay.armyLines[1].Bottom)
    overlay.bottomLeft.Left:Set(function() return overlay.armyLines[1].leftbg.Left() - 1 end)
    
    overlay.bottomMiddle = Bitmap(overlay.group, UIUtil.UIFile('/dialogs/score-overlay/score_brd_lm.dds'))
    overlay.bottomMiddle.Top:Set(overlay.bottomRight.Top)
    overlay.bottomMiddle.Right:Set(overlay.bottomRight.Left)
    overlay.bottomMiddle.Left:Set(function() return overlay.bottomLeft.Right() - 1 end)
    
    overlay.topRight = Bitmap(overlay.group, UIUtil.UIFile('/dialogs/score-overlay/score_brd_ur.dds'))
    overlay.topRight.Bottom:Set(function() return overlay.armyLines[index].Top() end)
    overlay.topRight.Right:Set(function() return overlay.armyLines[1].rightbg.Right() end)
    
    overlay.topLeft = Bitmap(overlay.group, UIUtil.UIFile('/dialogs/score-overlay/score_brd_ul.dds'))
    overlay.topLeft.Bottom:Set(overlay.topRight.Bottom)
    overlay.topLeft.Left:Set(function() return overlay.armyLines[1].leftbg.Left() end)
    
    overlay.topMiddle = Bitmap(overlay.group, UIUtil.UIFile('/dialogs/score-overlay/score_brd_horz_um.dds'))
    overlay.topMiddle.Bottom:Set(function() return overlay.armyLines[index].Top() end)
    overlay.topMiddle.Left:Set(overlay.topLeft.Right)
    overlay.topMiddle.Right:Set(overlay.topRight.Left)
    overlay.topMiddle.Height:Set(function() return overlay.topMiddle.BitmapHeight() + 2 end)
    
    overlay.group:DisableHitTest(true)
    overlay.group:Hide()
    overlay.timeBG:Hide()
    overlay.group:SetNeedsFrameUpdate(true)
    overlay.group.OnFrame = function(self, deltatime)
        if Prefs.GetFromCurrentProfile("scoreoverlay") != nil and Prefs.GetFromCurrentProfile("scoreoverlay") != false then
            overlay.group:Show()
            overlay.timeBG:Show()
        end
        overlay.group:SetNeedsFrameUpdate(false)
    end
    
    GameMain.AddBeatFunction(_BeatFunction)
    
    if walltime then    
        timeThread = ForkThread(UpdateTime)
    end
    
    overlay.group.OnDestroy = function(self)
        GameMain.RemoveBeatFunction(_BeatFunction)
    end
end


function _BeatFunction()
    if not walltime then
        overlay.time:SetText(GetGameTime())
    end
    local armiesInfo = GetArmiesTable()
    if armiesInfo then
        focusArmy = armiesInfo.focusArmy
        local sortedFocus = 0
        local sortedTable = {}
        if currentScores != nil then
            local index = 1
            for i, v in armiesInfo.armiesTable do
                if v.showScore and not v.civilian then
                    sortedTable[index] = v
                    sortedTable[index].score = currentScores[i].general.score or 0
                    if focusArmy == index then
                        if currentScores[i].general.currentunits.count and currentScores[i].general.currentcap.count then
                            overlay.units:SetText(string.format("%d/%d", 
                                currentScores[i].general.currentunits.count, 
                                currentScores[i].general.currentcap.count))
                        end
                        sortedTable[index].focus = true
                    else
                        sortedTable[index].focus = false
                    end
                    index = index + 1
                end
            end
            table.sort(sortedTable, function(table1, table2)
                if table1.score >= table2.score then
                    return false
                else
                    return true
                end
            end)
        end
        for i, v in sortedTable do
            local index = i
            SetBackgroundColor(overlay.armyLines[index], v.focus)
            overlay.armyLines[index].playername:SetText(v.nickname)
            overlay.armyLines[index].score:SetText(v.score)
            overlay.armyLines[index].faction:SetTexture(UIUtil.UIFile(UIUtil.GetFactionIcon(v.faction)))
            overlay.armyLines[index].teamcolor:SetSolidColor(v.color)
        end
        local maxPlayerWidth = 0
        local index = 0
        for i, v in overlay.armyLines do
            index = i
            if v.playername.Width() > maxPlayerWidth then
                maxPlayerWidth = v.playername.Width()
            end
        end
        for i, v in overlay.armyLines do
            v.playername.Width:Set(maxPlayerWidth)
            v.playername.Right:Set(function() return overlay.armyLines[index].score.Left() - 10 end)
        end
    end
end



local unitColor = 'red'

function SetBackgroundColor(control, focus)
    if focus then
        control.playername:SetColor("FF000000")
        control.score:SetColor("FF000000")
        control.namebg:SetSolidColor("70FFFFFF")
    else
        control.playername:SetColor("FFFFFFFF")
        control.score:SetColor("FFFFFFFF")
        control.namebg:SetSolidColor("70000000")
    end
end

function ToggleScoreControl()
    if overlay.group:IsHidden() then
        Prefs.SetToCurrentProfile("scoreoverlay", true)
        overlay.group:Show()
        overlay.timeBG:Show()
        local sound = Sound({Cue = "UI_Score_Window_Open", Bank = "Interface",})
        PlaySound(sound)
    else
        Prefs.SetToCurrentProfile("scoreoverlay", false)
        local sound = Sound({Cue = "UI_Score_Window_Close", Bank = "Interface",})
        PlaySound(sound)
        overlay.timeBG:Hide()
        overlay.group:Hide()
    end
end