--*****************************************************************************
--* File: lua/modules/ui/game/shareResources.lua
--* Summary: UI for Sharing (and giving) Resources
--*
--* Copyright  2006 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Checkbox = import('/lua/modules/maui/Checkbox.lua').Checkbox
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Slider = import('/lua/modules/maui/Slider.lua').Slider
local StatusBar = import('/lua/modules/maui/statusbar.lua').StatusBar
local Edit = import('/lua/modules/maui/Edit.lua').Edit


local function SetupEditField(control)
    control:SetForegroundColor(UIUtil.fontColor)
    control:SetBackgroundColor("FF569FFF")
    control:SetHighlightForegroundColor(UIUtil.highlightColor)
    control:SetHighlightBackgroundColor("880085EF")
    control:SetFont(UIUtil.bodyFont, 20)
    control.Height:Set(function() return control:GetFontHeight() end)
    return control
end

local radWidth, radHeight = GetTextureDimensions(UIUtil.UIFile('/widgets/large-h_scr/bar-mid_scr_up.dds'))
local lradWidth, lradHeight = GetTextureDimensions(UIUtil.UIFile('/widgets/large-h_scr/bar-left_scr_up.dds'))
local rradWidth, rradHeight = GetTextureDimensions(UIUtil.UIFile('/widgets/large-h_scr/bar-right_scr_up.dds'))

-- creates the actual dialog
--- parent is the calling window
function createShareResourcesDialog(targetPlayer, targetPlayerName)

    local massVal = 0
    local energyVal = 0

    local worldView = import('/lua/modules/ui/game/worldview.lua').view
    local dialog = Bitmap(worldView, UIUtil.UIFile('/dialogs/diplomacy-resources_options/diplomacy-panel_bmp.dds'))
    dialog:SetRenderPass(UIUtil.UIRP_PostGlow)  -- just in case our parent is the map
    dialog:SetName("Share Resources Window")
    LayoutHelpers.AtCenterIn(dialog, worldView)

    local chatTitle = UIUtil.CreateText(dialog, LOC("<LOC SHARERES_0000>Send Resources to ") .. targetPlayerName, 16, UIUtil.bodyFont)
    LayoutHelpers.AtLeftIn(chatTitle, dialog, 30)
    LayoutHelpers.AtTopIn(chatTitle, dialog, 25)

    local massIcon = Bitmap(dialog, UIUtil.UIFile('/dialogs/diplomacy-resources_options/mass_btn_up.dds'))
    massIcon.Left:Set(function() return dialog.Left() + 27 end)
    massIcon.Top:Set(function() return dialog.Top() + 60 end)

    local massInput = Edit(dialog)
    massInput.Top:Set(function() return massIcon.Top() + 12 end)
    massInput.Right:Set(function() return dialog.Right() - 40 end)
    massInput.Height:Set(20)
    massInput.Width:Set(80)
    SetupEditField(massInput)
    massInput:SetFont(UIUtil.bodyFont, 12)
    massInput:ShowBackground(false)
    massInput:SetText( massVal )

    massInput.OnEnterPressed = function(self, newText, oldText)
        massSlider:SetValue( newText )
    end

    local massStatus = StatusBar(dialog, 0, 100, false, false,
        UIUtil.UIFile('/dialogs/diplomacy-resources_options/mass-bar-back_bmp.dds'),
        UIUtil.UIFile('/dialogs/diplomacy-resources_options/mass-bar_bmp.dds'), false)

    local massSlider = Slider(dialog, false, 0, 100,
        UIUtil.UIFile('/dialogs/slider_btn/mass-bar-edge_btn_up.dds'))
    massSlider.Top:Set(function() return massIcon.Top() + 25 end)
    massSlider.Left:Set(function() return massIcon.Right() - 1 end)
    massSlider.Right:Set(function() return massInput.Left() - 10 end)
    massSlider:SetValue( massVal )
    massSlider.OnValueChanged = function(self, newValue)
        massInput:ClearText()
        massInput:SetText(math.floor(newValue) .. "%")
        massStatus:SetValue(math.floor(newValue))
    end

    massStatus.Top:Set(function() return massIcon.Top() + 12 end)
    massStatus.Left:Set(function() return massIcon.Right()  end)
    massStatus.Right:Set(function() return massInput.Left() - 12 end)
    massStatus.Depth:Set(function() return massSlider.Depth() - 1 end)
    massStatus:SetMinimumSlidePercentage(7/260)
    massStatus:SetRange(0, 100)
    massStatus:SetValue( massVal )

    local energyIcon = Bitmap(dialog, UIUtil.UIFile('/dialogs/diplomacy-resources_options/energy_btn_up.dds'))
    energyIcon.Left:Set(function() return dialog.Left() + 27 end)
    energyIcon.Top:Set(function() return dialog.Top() + 140 end)

    local energyInput = Edit(dialog)
    energyInput.Top:Set(function() return energyIcon.Top() + 12 end)
    energyInput.Right:Set(function() return dialog.Right() - 40 end)
    energyInput.Height:Set(20)
    energyInput.Width:Set(80)
    SetupEditField(energyInput)
    energyInput:SetFont(UIUtil.bodyFont, 12)
    energyInput:ShowBackground(false)
    energyInput:SetText( energyVal )
    energyInput.OnEnterPressed = function(self, newText, oldText)
        energySlider:SetValue(newText)
    end

    local energyStatus = StatusBar(dialog, 0, 100, false, false,
        UIUtil.UIFile('/dialogs/diplomacy-resources_options/energy-bar-back_bmp.dds'),
        UIUtil.UIFile('/dialogs/diplomacy-resources_options/energy-bar_bmp.dds'), false)

    local energySlider = Slider(dialog, false, 0, 100,
        UIUtil.UIFile('/dialogs/slider_btn/energy-bar-edge_btn_up.dds'))
    energySlider.Top:Set(function() return energyIcon.Top() + 25 end)
    energySlider.Left:Set(function() return energyIcon.Right() - 1 end)
    energySlider.Right:Set(function() return energyInput.Left() - 10 end)
    energySlider:SetValue( energyVal )
    energySlider.OnValueChanged = function(self, newValue)
        energyInput:ClearText()
        energyInput:SetText(math.floor(newValue) .. "%")
        energyStatus:SetValue(math.floor(newValue))
    end

    energyStatus.Top:Set(function() return energyIcon.Top() + 12 end)
    energyStatus.Left:Set(function() return energyIcon.Right()  end)
    energyStatus.Right:Set(function() return energyInput.Left() - 12 end)
    energyStatus.Depth:Set(function() return energySlider.Depth() - 1 end)
    energyStatus:SetMinimumSlidePercentage(7/260)
    energyStatus:SetRange(0, 100)
    energyStatus:SetValue( energyVal )

    # overriden by caller
    dialog.OnOk = function(mass,energy)
        LOG('You should override OnOk() if you want Share/Give Resources to do anything.')
    end

    # overriden by caller
    dialog.OnCancel = function()
        LOG('You should override OnCancel()')
    end

    local cancelButton = UIUtil.CreateButtonStd(dialog, "/dialogs/standard_btn/standard", "<LOC _Cancel>", 12)
    LayoutHelpers.AtBottomIn(cancelButton, dialog, 20)
    LayoutHelpers.AtRightIn(cancelButton, dialog, 24)
    cancelButton.OnClick = function(self, modifiers)
        dialog.OnCancel()
    end

    local okButton = UIUtil.CreateButtonStd(dialog, "/dialogs/standard_btn/standard", "<LOC _OK>", 12)
    LayoutHelpers.LeftOf(okButton, cancelButton, 3)
    okButton.OnClick = function(self, modifiers)
        dialog.OnOk( massSlider:GetValue() / 100.0, energySlider:GetValue() / 100.0 )
    end

    return dialog
end