--*****************************************************************************
--* File: lua/modules/ui/game/transmissionlog.lua
--* Author: Ted Snook
--* Summary: Transmission log
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Button = import('/lua/modules/maui/button.lua').Button
local ItemList = import('/lua/modules/maui/itemlist.lua').ItemList
local Movie = import('/lua/modules/maui/movie.lua').Movie
local Options = import('/lua/modules/ui/game/options.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')

local controls = {
    bg = false,
    logEntries = {},
}

local transmissionHistory = {}
local transmissionIndex = 0
local displayIndex = 1
function CreateTransmissionLog(parent)
    if not controls.bg then
        controls.bg = Bitmap(parent, UIUtil.UIFile('/dialogs/transmission/transmission-panel_bmp.dds'))
    end
    LayoutHelpers.AtCenterIn(controls.bg, parent, -30)
    controls.bg.Depth:Set(100)
    controls.bg:Hide()

    if not controls.title then
        controls.title = UIUtil.CreateText(controls.bg, LOC("<LOC trans_log_0000>Transmission Log"), 16)
    end
    LayoutHelpers.AtLeftTopIn(controls.title, controls.bg, 30, 22)

    if not controls.logindex then
        controls.logindex = UIUtil.CreateText(controls.bg, LOC("<LOC trans_log_0001>No entries"), 14)
    end
    LayoutHelpers.AtLeftTopIn(controls.logindex, controls.bg, 30, 46)

    if not controls.hidebutton then
        controls.hidebutton = UIUtil.CreateButton(controls.bg,
            '/dialogs/close_btn/close_btn_up.dds',
            '/dialogs/close_btn/close_btn_down.dds',
            '/dialogs/close_btn/close_btn_over.dds',
            '/dialogs/close_btn/close_btn_dis.dds')
    end
    controls.hidebutton.Top:Set(function() return controls.bg.Top() + 45 end)
    controls.hidebutton.Right:Set(function() return controls.bg.Right() - 25 end)
    controls.hidebutton.OnClick = function(self, modifiers)
        ToggleTransmissionLog()
    end
    controls.hidebutton.Depth:Set(function() return controls.bg.Depth() + 5 end)

    if not controls.pageup then
        controls.pageup = UIUtil.CreateButton(controls.bg
            , '/game/small-vert_scr/arrow-up_scr_up.dds'
            , '/game/small-vert_scr/arrow-up_scr_down.dds'
            , '/game/small-vert_scr/arrow-up_scr_over.dds'
            , '/game/small-vert_scr/arrow-up_scr_dis.dds')
    end
    controls.pageup.Top:Set(function() return controls.bg.Top() + 80 end)
    controls.pageup.Right:Set(function() return controls.bg.Right() - 25 end)
    controls.pageup.Depth:Set(function() return controls.bg.Depth() + 5 end)
    controls.pageup.OnClick = function(self, modifiers)
        if displayIndex > 1 then
            displayIndex = displayIndex - 1
        end
        UpdateTransmissionLog()
    end

    if not controls.pagedown then
        controls.pagedown = UIUtil.CreateButton(controls.bg
            , '/game/small-vert_scr/arrow-down_scr_up.dds'
            , '/game/small-vert_scr/arrow-down_scr_down.dds'
            , '/game/small-vert_scr/arrow-down_scr_over.dds'
            , '/game/small-vert_scr/arrow-down_scr_dis.dds')
    end
    controls.pagedown.Top:Set(controls.pageup.Bottom)
    controls.pagedown.Right:Set(function() return controls.bg.Right() - 25 end)
    controls.pagedown.Depth:Set(function() return controls.bg.Depth() + 5 end)
    controls.pagedown.OnClick = function(self, modifiers)
        if displayIndex < (table.getn(transmissionHistory) - 2) then
            displayIndex = displayIndex + 1
        end
        UpdateTransmissionLog()
    end

    local prevEntry = false
    for i=1, 3 do
        local index = i
        if not controls.logEntries[index] then
            controls.logEntries[index] = Group(controls.bg)
        end

        if index == 1 then
            controls.logEntries[index].Top:Set(function() return controls.bg.Top() + 80 end)
        else
            controls.logEntries[index].Top:Set(function() return controls.logEntries[index-1].Bottom() + 21 end)
        end
        controls.logEntries[index].Left:Set(function() return controls.bg.Left() + 30 end)
        controls.logEntries[index].Right:Set(function() return controls.bg.Right() - 90 end)
        controls.logEntries[index].Height:Set(100)

        if not controls.logEntries[index].videoContainer then
            controls.logEntries[index].videoContainer = Group(controls.logEntries[i])
        end
        controls.logEntries[index].videoContainer.Height:Set(76)
        controls.logEntries[index].videoContainer.Width:Set(76)
        controls.logEntries[index].videoContainer.Top:Set(controls.logEntries[index].Top)
        controls.logEntries[index].videoContainer.Left:Set(function() return controls.logEntries[index].Left() + 5 end)

        #if not controls.logEntries[index].currentMovie then
            #controls.logEntries[index].currentMovie = Movie(controls.logEntries[index], '/movies/gpglogo.sfd')
        #end
        if controls.logEntries[index].currentMovie then
            controls.logEntries[index].currentMovie:DisableHitTest()
            LayoutHelpers.FillParent(controls.logEntries[index].currentMovie, controls.logEntries[index].videoContainer)   -- note the movies now fill the parent
        end

        controls.logEntries[index].cue = ""
        controls.logEntries[index].bank = ""
        controls.logEntries[index].sound = false

        if not controls.logEntries[index].playButton then
            controls.logEntries[index].playButton = Button(controls.logEntries[i],
                UIUtil.UIFile('/dialogs/movie-control/nav-play_btn_up.dds'),
                UIUtil.UIFile('/dialogs/movie-control/nav-play_btn_down.dds'),
                UIUtil.UIFile('/dialogs/movie-control/nav-play_btn_over.dds'),
                UIUtil.UIFile('/dialogs/movie-control/nav-play_btn_dis.dds'))
        end
        controls.logEntries[index].playButton.Bottom:Set(function() return controls.logEntries[index].Bottom() + 10 end)
        controls.logEntries[index].playButton.Left:Set(function() return controls.logEntries[index].Left() + 7 end)
        controls.logEntries[index].playButton.Index = index
        controls.logEntries[index].playButton.OnClick = function(self, modifiers)
            for x = 1, 3 do
                local tempindex = x
                if tempindex != self.Index then
                    if controls.logEntries[tempindex].currentMovie then
                        controls.logEntries[tempindex].currentMovie:Stop()
                    end
                    if controls.logEntries[tempindex].sound then
                        StopSound(controls.logEntries[tempindex].sound,true)
                    end
                end
            end
            if controls.logEntries[index].currentMovie then
                controls.logEntries[index].currentMovie:Set(transmissionHistory[displayIndex + (index-1)][1])
                controls.logEntries[index].currentMovie:Play()
            end
            if controls.logEntries[index].sound then
                StopSound(controls.logEntries[index].sound,true)
            end
            local sound = Sound({Cue = controls.logEntries[index].cue, Bank = controls.logEntries[index].bank,})
            controls.logEntries[index].sound = PlaySound(sound)
        end

        if not controls.logEntries[index].stopButton then
            controls.logEntries[index].stopButton = UIUtil.CreateButton(controls.logEntries[i],
                '/dialogs/movie-control/nav-pause_btn_up.dds',
                '/dialogs/movie-control/nav-pause_btn_down.dds',
                '/dialogs/movie-control/nav-pause_btn_over.dds',
                '/dialogs/movie-control/nav-pause_btn_dis.dds')
        end
        controls.logEntries[index].stopButton.Bottom:Set(controls.logEntries[index].playButton.Bottom)
        controls.logEntries[index].stopButton.Left:Set(controls.logEntries[index].playButton.Right)
        controls.logEntries[index].stopButton.OnClick = function(self, modifiers)
            if controls.logEntries[index].currentMovie then
                controls.logEntries[index].currentMovie:Stop()
            end
            StopSound(controls.logEntries[index].sound,true)
        end

        if not controls.logEntries[index].text then
            controls.logEntries[index].text = ItemList(controls.logEntries[i])
        end
        controls.logEntries[index].text.Top:Set(controls.logEntries[index].videoContainer.Top)
        controls.logEntries[index].text.Left:Set(function() return controls.logEntries[index].videoContainer.Right() + 10 end)
        controls.logEntries[index].text.Right:Set(controls.logEntries[index].Right)
        controls.logEntries[index].text.Bottom:Set(controls.logEntries[index].Bottom)
        controls.logEntries[index].text:SetColors(UIUtil.consoleFGColor(), UIUtil.consoleTextBGColor(), UIUtil.consoleFGColor(), UIUtil.consoleTextBGColor()) -- we don't really want selection here so don't differentiate colors
        controls.logEntries[index].text:SetFont(UIUtil.bodyFont, 12)

        controls.logEntries[index].VideoName = ""

        controls.logEntries[index].scrollbar = UIUtil.CreateVertScrollbarFor(controls.logEntries[i].text)

        controls.logEntries[index]:Hide()
        prevEntry = controls.logEntries[index]
    end
end

function ToggleTransmissionLog()
    if controls.bg:IsHidden() then
        controls.bg:Show()
        UpdateTransmissionLog(true)
        if not UIUtil.GetNetworkBool() then
            Options.PauseGame(true)
        end
    else
        for x = 1, 3 do
            local tempindex = x
            if controls.logEntries[tempindex].currentMovie then
                controls.logEntries[tempindex].currentMovie:Stop()
            end
            if controls.logEntries[tempindex].sound then
                StopSound(controls.logEntries[tempindex].sound,true)
            end
        end
        controls.bg:Hide()
        if not UIUtil.GetNetworkBool() then
            Options.PauseGame(false)
        end
    end
end

function AddEntry(entryData)
    table.insert(transmissionHistory, 1, entryData)
    UpdateTransmissionLog()
end

function UpdateTransmissionLog(loadMovies)
    controls.logindex:SetText(LOCF("<LOC trans_log_0002>%d - %d of %d entries", displayIndex, math.min(displayIndex + 2, table.getsize(transmissionHistory)), table.getsize(transmissionHistory)))
    for i = 1, 3 do
        local trans = transmissionHistory[displayIndex + (i-1)]
        if trans[1] then
            if not controls.bg:IsHidden() then
                controls.logEntries[i]:Show()
            end
            controls.logEntries[i].text:DeleteAllItems()
            local textBoxWidth = controls.logEntries[i].text.Right() - controls.logEntries[i].text.Left()
            local tempTable = import('/lua/modules/maui/text.lua').WrapText(trans[2], textBoxWidth,
            function(text)
                return controls.logEntries[i].text:GetStringAdvance(text)
            end)

            local tempLines = table.getn(tempTable)
            if tempLines < 6 then
                controls.logEntries[i].scrollbar:Hide()
            else
                if not controls.bg:IsHidden() then
                    controls.logEntries[i].scrollbar:Show()
                end
            end

            for x = 1, tempLines do
                controls.logEntries[i].text:AddItem(tempTable[x])
            end
            if loadMovies then
                if controls.logEntries[i].currentMovie then
                    controls.logEntries[i].currentMovie:Set(trans[1])
                end
            end
            controls.logEntries[i].cue = trans[4]
            controls.logEntries[i].bank = trans[3]
        else
            controls.logEntries[i]:Hide()
        end
    end
end