--*****************************************************************************
--* File: lua/modules/ui/game/unitview.lua
--* Author: Chris Blackwell
--* Summary: Rollover unit view control
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local GameCommon = import('/lua/modules/ui/game/gamecommon.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local StatusBar = import('/lua/modules/maui/statusbar.lua').StatusBar

local rolloverInfo = false
local consTrue = false
local focusBool = false

local showBool = true

local numRoDisplayFields = 3

local controls = {
    roInfoLine = {},
    roInfoLineBG = {},
    roInfoLine2 = {},
    parent = false,
    rolloverGroup = false,
    roBG = false,
    roIcon = false,
    roStratIcon = false,
    roVeteranIcons = {},
    roDisplay = {},
    roVeteranIconParent = false,
    roTextGroup = false,
    roTextGroupBG = false,
    roName = false,
    roId = false,
    roHealth = false,
    roHealthBar = false,
    roFocusArrow = false,
    roFocusName = false,
    roFocusIcon = false,
    roFocusHealthBar = false,
    roFocusHealthPercent = false,
    roTechLevelPips = {},
}

function Contract()
    showBool = false
end

function Expand()
    showBool = true
end

function SetZoomAlpha()
    local view = GetCamera('WorldCamera')
    
    local minZoom = view:GetMinZoom()
    local maxZoom = view:GetMaxZoom()
    local curZoom = view:GetZoom()
    
    if (curZoom / (maxZoom - minZoom)) > .9 then
        controls.rolloverGroup:SetAlpha(.5, true)
    else
        controls.rolloverGroup:SetAlpha(1, true)
    end
end

function ShowROBox(state, focusBool, inDisplayIndex)
    if(state) then
        controls.rolloverGroup:Show()
        controls.rolloverGroup:SetAlpha(1, true)
    else
        controls.rolloverGroup:Hide()
        consTrue = false
    end
    if focusBool then
        controls.roFocusBG:Show()
        controls.roFocusHealthBar:Show()
        controls.roFocusIcon:Show()
        controls.roFocusName:Show()
        controls.roFocusHealthPercent:Show()
    else
        controls.roFocusHealthBar:Hide()
        controls.roFocusBG:Hide()
        controls.roFocusIcon:Hide()
        controls.roFocusHealthPercent:Hide()
        controls.roFocusIcon:SetSolidColor("00000000")
        controls.roFocusName:Hide()
    end
end

function NetResources(produced, requested)
    local netReturn = produced - requested
    if(netReturn >= 0) then
        return "+" .. tostring(netReturn)
    else
        return netReturn
    end
end

function UpdateVetIcons(level)
    for i = 1, 5 do
        controls.roVeteranIcons[i]:Hide()
    end
    for i = 1, level do
        controls.roVeteranIcons[i]:Show()
    end
end

function FormatTime(seconds)
    local tempSeconds = math.floor(seconds)
    local tempMinutes = 00

    tempMinutes = math.floor(tempSeconds / 60)
    tempSeconds = tempSeconds - (tempMinutes * 60)
    if(tempMinutes < 10) then
        tempMinutes = "0" .. tostring(tempMinutes)
    end
    if(tempSeconds < 10) then
        tempSeconds = "0" .. tostring(tempSeconds)
    end
    local tempTime = tostring(tempMinutes) .. ":" .. tostring(tempSeconds)
    return tempTime
end

function UpdateROInfo(bp, constructionBool, buildingUnit, enhancementBool)
    local vetLevel = 0
    focusBool = false
    if bp then
        local displayIndex = 1
        if enhancementBool then
            controls.roName:SetText(bp.Name)
            controls.roIcon:SetTexture(UIUtil.UIFile('/icons/units/default_icon.dds'))
            controls.roStratIcon:Hide()
            controls.roName:SetColor("FFFF9E06")
            controls.roId:SetColor("FF06ECEC")
            controls.roHealth:SetText("")
            controls.roHealthBar:SetValue(0)
            controls.roHealthBar._bar:SetTexture(UIUtil.UIFile('/game/unit_bmp/bar-02_bmp.dds'))
            displayIndex = SetDisplayLine(displayIndex, controls.roDisplay['mass'], tostring(math.ceil(bp.BuildCostMass)), 'FF00FF00', 2, false)
            displayIndex = SetDisplayLine(displayIndex, controls.roDisplay['energy'], tostring(math.ceil(bp.BuildCostEnergy)), 'FF00FF00', 3, false)
            displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[1], FormatTime(bp.BuildTime), 'FF00FF00', 5, false)
            UpdateVetIcons(0)
            consTrue = true
            ShowROBox(true, focusBool, displayIndex)
        else
            local techLevel = 0
            for i, v in bp.Categories do
                if v == "TECH1" then
                    techLevel = 1
                end
                if v == "TECH2" then
                    techLevel = 2
                end
                if v == "TECH3" then
                    techLevel = 3
                end
            end
            for i = 1, 3 do
                if i <= techLevel then
                    controls.roTechLevelPips[i]:SetTexture(UIUtil.UIFile('/game/icon_tech-level/tech-level_solid.dds'))
                else
                    controls.roTechLevelPips[i]:SetTexture(UIUtil.UIFile('/game/icon_tech-level/tech-level_open.dds'))
                end
            end
            local iconName = GameCommon.GetCachedUnitIconFileNames(bp)
            controls.roIcon:SetTexture(iconName)
            if (bp.StrategicIconName) then
                controls.roStratIcon:SetTexture("/textures/ui/common/game/strategicicons/" .. string.lower(bp.StrategicIconName) .. "_rest.dds")
                controls.roStratIcon:Show()
            end
            
            local civilian = false
            for i, v in bp.Categories do
                if v == 'CIVILIAN' then
                    civilian = true
                end
            end

            if rolloverInfo.customName then
                controls.roName:SetText(rolloverInfo.customName)
            else
                if civilian then
                    if bp.General.UnitName then
                        controls.roName:SetText(LOCF("%s", bp.General.UnitName))
                    end
                else
                    if techLevel == 0 then
                        controls.roName:SetText(LOCF("%s", bp.Description))
                    else
                        if bp.General.UnitName then
                            controls.roName:SetText(LOCF("%s: Tech %d %s", bp.General.UnitName, techLevel, bp.Description))
                        else
                            controls.roName:SetText(LOCF("Tech %d %s", techLevel, bp.Description))
                        end
                    end
                end
            end

            controls.roName:SetColor("FFFF9E06")
            controls.roId:SetColor("FF06ECEC")

            if(constructionBool) then
                controls.roHealth:SetText(string.format("%d/%d", bp.Defense.MaxHealth, bp.Defense.MaxHealth))
                controls.roHealth:SetColor("FF02f3e2")
                if controls.roHealthBar._rangeMax != bp.Defense.MaxHealth then
                    controls.roHealthBar:SetRange(0, bp.Defense.MaxHealth)
                end
                controls.roHealthBar:SetValue(bp.Defense.MaxHealth)
            else
                controls.roHealth:SetText(string.format("%d/%d", rolloverInfo.health, rolloverInfo.maxHealth))
                controls.roHealth:SetColor("FFF0F0F0")
                if controls.roHealthBar._rangeMax != rolloverInfo.maxHealth then
                    controls.roHealthBar:SetRange(0, rolloverInfo.maxHealth)
                end
                if rolloverInfo.health / rolloverInfo.maxHealth > .75 then
                    controls.roHealthBar._bar:SetTexture(UIUtil.UIFile('/game/unit_bmp/bar-02_bmp.dds'))
                elseif rolloverInfo.health / rolloverInfo.maxHealth > .25 then
                    controls.roHealthBar._bar:SetTexture(UIUtil.UIFile('/game/unit_bmp/bar-01_bmp.dds'))
                elseif rolloverInfo.health / rolloverInfo.maxHealth > 0 then
                    controls.roHealthBar._bar:SetTexture(UIUtil.UIFile('/game/unit_bmp/bar-03_bmp.dds'))
                end
                controls.roHealthBar:SetValue(rolloverInfo.health)
            end
            if(constructionBool) then
            else
                if rolloverInfo.nukeSiloMaxStorageCount then
                    if( rolloverInfo.nukeSiloMaxStorageCount > 0 ) then
                        if( rolloverInfo.nukeSiloBuildCount > 0 ) then
                            displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], tostring(rolloverInfo.nukeSiloStorageCount) .. " / " .. tostring(rolloverInfo.nukeSiloMaxStorageCount) .. " (+" .. tostring(rolloverInfo.nukeSiloBuildCount) .. ")", 'FF9eff19', 1, false)
                        else
                            displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], tostring(rolloverInfo.nukeSiloStorageCount) .. " / " .. tostring(rolloverInfo.nukeSiloMaxStorageCount), 'FF9eff19', 1, false)
                        end
                    end
                end
                if rolloverInfo.tacticalSiloMaxStorageCount then
                    if( rolloverInfo.tacticalSiloMaxStorageCount > 0 ) then
                        if( rolloverInfo.tacticalSiloBuildCount > 0 ) then
                            displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], tostring(rolloverInfo.tacticalSiloStorageCount) .. " / " .. tostring(rolloverInfo.tacticalSiloMaxStorageCount) .. " (+" .. tostring(rolloverInfo.tacticalSiloBuildCount) .. ")", 'FF9eff19', 7, false)
                        else
                            displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], tostring(rolloverInfo.tacticalSiloStorageCount) .. " / " .. tostring(rolloverInfo.tacticalSiloMaxStorageCount), 'FF9eff19', 7, false)
                        end
                    end
                end
            end
            if(constructionBool) then
                consTrue = true
                if(buildingUnit) then
                    local timeCost, energyCost, massCost = import('/lua/modules/game.lua').GetConstructEconomyModel(buildingUnit, bp.Economy)
                    if(massCost) then
                        SetDisplayLine(displayIndex, controls.roDisplay['mass'], math.ceil(massCost), "FF00FF00", 2, false)
                    end
                    if(energyCost) then
                        SetDisplayLine(displayIndex, controls.roDisplay['energy'], math.ceil(energyCost), "FF00FF00", 3, false)
                    end
                    if(timeCost) then
                        displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], FormatTime(timeCost), "FF00FF00", 5, false)
                    end
                end
            else
                if(rolloverInfo.massProduced and rolloverInfo.massConsumed ) then
                    SetDisplayLine(displayIndex, controls.roDisplay['mass'], NetResources(rolloverInfo.massProduced, rolloverInfo.massConsumed), 'FF00FF00', 2, false)
                end
                if(rolloverInfo.energyProduced and rolloverInfo.energyConsumed) then
                    SetDisplayLine(displayIndex, controls.roDisplay['energy'], NetResources(rolloverInfo.energyProduced, rolloverInfo.energyConsumed), 'FF00FF00', 3, false)
                end
                if(rolloverInfo.kills != 0 and rolloverInfo.kills != nil) then
                    local vetLevel5 = 1000
                    local vetLevel4 = 750
                    local vetLevel3 = 500
                    local vetLevel2 = 250
                    local vetLevel1 = 100
                    if bp.Veteran then
                        vetLevel5 = bp.Veteran['Level5']
                        vetLevel4 = bp.Veteran['Level4']
                        vetLevel3 = bp.Veteran['Level3']
                        vetLevel2 = bp.Veteran['Level2']
                        vetLevel1 = bp.Veteran['Level1']
                    end
                     
                    displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], rolloverInfo.kills, 'FFFF0000', 4, false)
                    if(rolloverInfo.kills >= vetLevel5) then
                        vetLevel = 5
                    elseif(rolloverInfo.kills >= vetLevel4) then
                        vetLevel = 4
                    elseif(rolloverInfo.kills >= vetLevel3) then
                        vetLevel = 3
                    elseif(rolloverInfo.kills >= vetLevel2) then
                        vetLevel = 2
                    elseif(rolloverInfo.kills >= vetLevel1) then
                        vetLevel = 1
                    else
                        vetLevel = 0
                    end
                end
            end
            if rolloverInfo.focus then
                focusBP = __blueprints[rolloverInfo.focus.blueprintId]
                if focusBP.General.UnitName != nil then
                    controls.roFocusName:SetText(LOC(focusBP.General.UnitName))
                else
                    controls.roFocusName:SetText(LOC(focusBP.Description))
                end
                local iconFocusName = GameCommon.GetCachedUnitIconFileNames(focusBP)
                controls.roFocusIcon:SetTexture(iconFocusName)
                if controls.roFocusHealthBar._rangeMax != rolloverInfo.focus.maxHealth then
                    controls.roFocusHealthBar:SetRange(0, rolloverInfo.focus.maxHealth)
                end
                controls.roFocusHealthBar:SetValue(rolloverInfo.focus.health)
                if rolloverInfo.focus.health / rolloverInfo.focus.maxHealth > .75 then
                    controls.roFocusHealthBar._bar:SetTexture(UIUtil.UIFile('/game/unit_bmp/bar-02_bmp.dds'))
                elseif rolloverInfo.focus.health / rolloverInfo.focus.maxHealth > .25 then
                    controls.roFocusHealthBar._bar:SetTexture(UIUtil.UIFile('/game/unit_bmp/bar-01_bmp.dds'))
                elseif rolloverInfo.focus.health / rolloverInfo.focus.maxHealth > 0 then
                    controls.roFocusHealthBar._bar:SetTexture(UIUtil.UIFile('/game/unit_bmp/bar-03_bmp.dds'))
                end
                controls.roFocusHealthPercent:SetText(math.ceil(100 * (rolloverInfo.focus.health / rolloverInfo.focus.maxHealth)) .. "%")
                controls.roFocusHealthPercent:SetColor("FFF0F0F0")
                focusBool = true
            else
                focusBool = false
            end
            ShowROBox(true, focusBool, displayIndex)
            #SetZoomAlpha()
            UpdateVetIcons(vetLevel)
            while displayIndex <= numRoDisplayFields do
                displayIndex = SetDisplayLine(displayIndex, controls.roDisplay[displayIndex], '', 'FFFF0000', 6, true)
            end
        end
    else
        if(consTrue) then
            ShowROBox(true, focusBool)
            UpdateVetIcons(vetLevel)
        else
            ShowROBox(false, focusBool)
        end
    end
end

function SetDisplayLine(displayIndex, control, text, color, iconType, hidden)
    control.text:SetText(text)
    local textColor = color
    if tonumber(string.sub(text, 1)) then
        if tonumber(string.sub(text, 1)) >= 0 then
            textColor = 'FF00FF00'
        else
            textColor = 'FFFF0000'
        end
    end
    control.text:SetColor(textColor)
    if iconType == 1 then
        control.icon:SetTexture(UIUtil.UIFile('/game/unit-over/icon-nuke_bmp.dds'))
    elseif iconType == 2 then
        control.icon:SetTexture(UIUtil.UIFile('/game/unit-over/icon-mass_bmp.dds'))
    elseif iconType == 3 then
        control.icon:SetTexture(UIUtil.UIFile('/game/unit-over/icon-energy_bmp.dds'))
    elseif iconType == 4 then
        control.icon:SetTexture(UIUtil.UIFile('/game/unit-over/icon-skull_bmp.dds'))
    elseif iconType == 5 then
        control.icon:SetTexture(UIUtil.UIFile('/game/unit-over/icon-clock_bmp.dds'))
    elseif iconType == 6 then
        control.icon:SetSolidColor('00000000')
    elseif iconType == 7 then
        control.icon:SetTexture(UIUtil.UIFile('/game/unit_view_icons/tactical.dds'))
    end
    if hidden then
        control.text:Hide()
        control.icon:Hide()
    else
        control.text:Show()
        control.icon:Show()
    end
    local tempIndex = displayIndex + 1
    return tempIndex
end

function DoBottomLayout()
    if not controls.rolloverGroup then
        controls.rolloverGroup = Group(controls.parent, "Unit Rollover Group")
    end
    controls.rolloverGroup.Bottom:Set(function() return controls.parent.Top() + 2 end)
    controls.rolloverGroup.Top:Set(function() return controls.rolloverGroup.Bottom() - 50 end)
    controls.rolloverGroup.Left:Set(function() return controls.parent.Left() + 2 end)
    controls.rolloverGroup.Depth:Set(function() return controls.parent.Depth() + 50 end)

    if not controls.roBG then
        controls.roBG = Bitmap(controls.rolloverGroup, UIUtil.UIFile('/game/unit-over/unit-over_bmp.dds'))
    end
    controls.roBG.Bottom:Set(controls.rolloverGroup.Bottom)
    controls.roBG.Left:Set(controls.rolloverGroup.Left)
    
    if not controls.roBG2 then
        controls.roBG2 = Bitmap(controls.rolloverGroup, UIUtil.UIFile('/game/border_bottom/back_brd_bl02.dds'))
    end
    controls.roBG2.Bottom:Set(function() return controls.rolloverGroup.Bottom() - 2 end)
    controls.roBG2.Left:Set(function() return controls.rolloverGroup.Left() - 1 end)
    controls.roBG2.Depth:Set(function() return controls.roBG.Depth() - 1 end)

    if not controls.roIcon then
        controls.roIcon = Bitmap(controls.rolloverGroup)
    end
    controls.roIcon.Top:Set(function() return controls.rolloverGroup.Top() - 3 end)
    controls.roIcon.Left:Set(function() return controls.rolloverGroup.Left() + 4 end)
    controls.roIcon.Height:Set(46)
    controls.roIcon.Width:Set(46)
    
    if not controls.roStratIcon then
        controls.roStratIcon = Bitmap(controls.rolloverGroup)
    end
    controls.roStratIcon.Top:Set(function() return controls.roIcon.Top() + 12 end)
    controls.roStratIcon.Left:Set(function() return controls.roIcon.Right() + 10 end)

    local previousPip = false
    for i = 1, 3 do
        if not controls.roTechLevelPips[i] then
            controls.roTechLevelPips[i] = Bitmap(controls.rolloverGroup)
        end
        if i == 1 then
            controls.roTechLevelPips[i].Left:Set(function() return controls.roStratIcon.Right() + 4 end)
        else
            controls.roTechLevelPips[i].Left:Set(previousPip.Right)
        end
        controls.roTechLevelPips[i].Top:Set(function() return controls.roIcon.Top() + 17 end)
        controls.roTechLevelPips[i].Depth:Set(function() return controls.roIcon.Depth() + 1 end)
        previousPip = controls.roTechLevelPips[i]
    end

    if not controls.roHealthBar then
        controls.roHealthBar = StatusBar(controls.rolloverGroup, 0, 1, false, false, UIUtil.UIFile('/game/unit-over/unit-over_bmp.dds'), UIUtil.UIFile('/game/unit-over/bar01_bmp.dds'), true, "Unit RO Health Status Bar")
    end
    controls.roHealthBar.Left:Set(function() return controls.roIcon.Right() + 10 end)
    controls.roHealthBar.Right:Set(function() return controls.roHealthBar.Left() + 75 end)
    controls.roHealthBar.Bottom:Set(function() return controls.rolloverGroup.Bottom() - 25 end)
    controls.roHealthBar.Top:Set(function() return controls.roHealthBar.Bottom() - 4 end)
    controls.roHealthBar.Height:Set(function() return controls.roHealthBar.Bottom() - controls.roHealthBar.Top() end)
    controls.roHealthBar.Width:Set(function() return controls.roHealthBar.Right() - controls.roHealthBar.Left() end)

    if not controls.roName then
        controls.roName = UIUtil.CreateText(controls.rolloverGroup, "", 12, UIUtil.bodyFont)
    end
    controls.roName.Top:Set(function() return controls.roIcon.Top() + 3 end)
    controls.roName.Left:Set(function() return controls.roIcon.Right() + 65 end)
    controls.roName:SetDropShadow(true)
    
    local prevVetIcon = false
    for i=1, 5 do
        local index = i
        if not controls.roVeteranIcons[i] then
            controls.roVeteranIcons[index] = Bitmap(controls.rolloverGroup)
        end
        if index == 1 then
            controls.roVeteranIcons[index].Left:Set(controls.roStratIcon.Left)
        else
            LayoutHelpers.RightOf(controls.roVeteranIcons[index], controls.roVeteranIcons[index-1], -3)
        end
        controls.roVeteranIcons[index].Bottom:Set(function() return controls.roStratIcon.Top() end)
        controls.roVeteranIcons[index]:SetTexture(UIUtil.UIFile('/game/veteran-logo_bmp/uef-veteran_bmp.dds'))
        controls.roVeteranIcons[index]:Hide()
        prevVetIcon = controls.roVeteranIcons[index]
    end
    
    if not controls.roId then
        controls.roId = UIUtil.CreateText(controls.rolloverGroup, "", 12, UIUtil.bodyFont)
    end
    controls.roId.Top:Set(function() return controls.roName.Bottom() + 12 end)
    LayoutHelpers.AtHorizontalCenterIn(controls.roId, controls.roHealthBar)
    controls.roId:SetDropShadow(true)


    if not controls.roHealth then
        controls.roHealth = UIUtil.CreateText(controls.roHealthBar, "", 10, UIUtil.bodyFont)
    end
    LayoutHelpers.AtHorizontalCenterIn(controls.roHealth, controls.roHealthBar)
    controls.roHealth.Top:Set(function() return controls.roHealthBar.Bottom() + 4 end)
    controls.roHealth.Depth:Set(function() return controls.roHealthBar.Depth() + 10 end)
    controls.roHealth:SetDropShadow(true)

    if not controls.roDisplay['mass'] then
        controls.roDisplay['mass'] = {}
        controls.roDisplay['mass'].text = UIUtil.CreateText(controls.rolloverGroup, "", 12, UIUtil.bodyFont)
        controls.roDisplay['mass'].icon = Bitmap(controls.rolloverGroup)
    end
    controls.roDisplay['mass'].icon.Left:Set(function() return controls.roHealthBar.Right() + 10 end)
    controls.roDisplay['mass'].icon.Top:Set(function() return controls.roHealthBar.Top() end)
    controls.roDisplay['mass'].text.Left:Set(function() return controls.roDisplay['mass'].icon.Right()  end)
    controls.roDisplay['mass'].text.Top:Set(function() return controls.roDisplay['mass'].icon.Top() + 1 end)

    if not controls.roDisplay['energy'] then
        controls.roDisplay['energy'] = {}
        controls.roDisplay['energy'].text = UIUtil.CreateText(controls.rolloverGroup, "", 12, UIUtil.bodyFont)
        controls.roDisplay['energy'].icon = Bitmap(controls.rolloverGroup)
    end
    controls.roDisplay['energy'].icon.Left:Set(function() return controls.roDisplay['mass'].icon.Right() + 43 end)
    controls.roDisplay['energy'].icon.Top:Set(controls.roDisplay['mass'].icon.Top)
    controls.roDisplay['energy'].text.Left:Set(function() return controls.roDisplay['energy'].icon.Right()  end)
    controls.roDisplay['energy'].text.Top:Set(function() return controls.roDisplay['energy'].icon.Top() + 1 end)

    for i=1, numRoDisplayFields do
        if not controls.roDisplay[i] then
            controls.roDisplay[i] = {}
            controls.roDisplay[i].text = UIUtil.CreateText(controls.rolloverGroup, "", 12, UIUtil.bodyFont)
            controls.roDisplay[i].icon = Bitmap(controls.rolloverGroup)
        end
        if i == 1 then
            controls.roDisplay[1].icon.Left:Set(function() return controls.roDisplay['energy'].icon.Right() + 43 end)
            LayoutHelpers.AtVerticalCenterIn(controls.roDisplay[1].icon, controls.roDisplay['mass'].icon)
            #controls.roDisplay[1].icon.Top:Set(function() return controls.roDisplay['mass'].icon.Top()  end)
            controls.roDisplay[1].text.Left:Set(function() return controls.roDisplay[1].icon.Right() + 5 end)
            controls.roDisplay[1].text.Top:Set(function() return controls.roName.Bottom() + 8 end)
        else
            local prevControl = controls.roDisplay[i - 1].text
            controls.roDisplay[i].icon.Left:Set(function() return prevControl.Right() + 15 end)
            LayoutHelpers.AtVerticalCenterIn(controls.roDisplay[i].icon, controls.roDisplay[1].icon)
            controls.roDisplay[i].text.Left:Set(function() return prevControl.Right() + 35 end)
            controls.roDisplay[i].text.Top:Set(controls.roDisplay[1].text.Top)
        end
    end

    if not controls.roFocusBG then
        controls.roFocusBG = Bitmap(controls.rolloverGroup, UIUtil.UIFile('/game/unit-over/unit-over-build_bmp.dds'))
    end
    controls.roFocusBG.Top:Set(function() return controls.roBG.Top() + 6 end)
    controls.roFocusBG.Left:Set(function() return controls.roBG.Right() - 44 end)

    if not controls.roFocusIcon then
        controls.roFocusIcon = Bitmap(controls.rolloverGroup)
    end
    controls.roFocusIcon.Top:Set(controls.roIcon.Top)
    controls.roFocusIcon.Left:Set(function() return controls.roFocusBG.Left() + 40 end)
    controls.roFocusIcon.Height:Set(46)
    controls.roFocusIcon.Width:Set(46)
    controls.roFocusIcon.Depth:Set(function() return controls.roFocusBG.Depth() + 1 end)
    
    if not controls.roFocusHealthPercent then
        controls.roFocusHealthPercent = UIUtil.CreateText(controls.rolloverGroup, "", 14, UIUtil.bodyFont)
    end
    controls.roFocusHealthPercent.Bottom:Set(function() return controls.roFocusIcon.Bottom() - 5 end)
    controls.roFocusHealthPercent.Right:Set(function() return controls.roFocusIcon.Right() + 40 end)
    controls.roFocusHealthPercent:SetDropShadow(true)
    controls.roFocusHealthPercent.Depth:Set(function() return controls.roFocusBG.Depth() + 1 end)
    
    if not controls.roFocusHealthBar then
        controls.roFocusHealthBar = StatusBar(controls.rolloverGroup, 0, 1, false, false, UIUtil.UIFile('/game/unit-over/unit-over_bmp.dds'), UIUtil.UIFile('/game/unit-over/bar01_bmp.dds'), true, "Unit RO Health Status Bar")
    end
    controls.roFocusHealthBar.Left:Set(function() return controls.roFocusHealthPercent.Right() + 10 end)
    controls.roFocusHealthBar.Right:Set(function() return controls.roFocusHealthBar.Left() + 184 end)
    controls.roFocusHealthBar.Bottom:Set(function() return controls.roFocusHealthPercent.Bottom() - 5 end)
    controls.roFocusHealthBar.Top:Set(function() return controls.roFocusHealthBar.Bottom() - 5 end)
    controls.roFocusHealthBar.Height:Set(function() return controls.roFocusHealthBar.Bottom() - controls.roFocusHealthBar.Top() end)
    controls.roFocusHealthBar.Width:Set(function() return controls.roFocusHealthBar.Right() - controls.roFocusHealthBar.Left() end)
    controls.roFocusHealthBar.Depth:Set(function() return controls.roFocusBG.Depth() + 1 end)
    controls.roFocusHealthBar._bar.Depth:Set(function() return controls.roFocusBG.Depth() + 2 end)

    if not controls.roFocusName then
        controls.roFocusName = UIUtil.CreateText(controls.rolloverGroup, "", 14, UIUtil.bodyFont)
    end
    controls.roFocusName.Top:Set(function() return controls.roIcon.Top() + 7 end)
    controls.roFocusName.Left:Set(function() return controls.roFocusIcon.Right() + 15 end)
    controls.roFocusName:SetDropShadow(true)
    controls.roFocusName.Depth:Set(function() return controls.roFocusBG.Depth() + 1 end)

    controls.rolloverGroup:DisableHitTest(true)
end

function DoLeftLayout()
    if not controls.rolloverGroup then
        controls.rolloverGroup = Group(controls.parent, "Unit Rollover Group")
    end
    controls.rolloverGroup.Width:Set(500)
    controls.rolloverGroup.Bottom:Set(controls.parent.Bottom)
    controls.rolloverGroup.Top:Set(function() return controls.rolloverGroup.Bottom() - 50 end)
    controls.rolloverGroup.Left:Set(function() return controls.parent.Left() + 250 end)
    controls.rolloverGroup.Depth:Set(function() return controls.parent.Depth() + 50 end)

    if not controls.roBG then
        controls.roBG = Bitmap(controls.rolloverGroup, UIUtil.UIFile('/game/unit-over/unit-over_bmp.dds'))
    end
    controls.roBG.Bottom:Set(controls.rolloverGroup.Bottom)
    controls.roBG.Left:Set(function() return controls.rolloverGroup.Left() - 5 end)

    if not controls.roBG2 then
        controls.roBG2 = Bitmap(controls.rolloverGroup, UIUtil.UIFile('/game/border_bottom/back_brd_bl02.dds'))
    end
    controls.roBG2.Bottom:Set(function() return controls.rolloverGroup.Bottom() end)
    controls.roBG2.Left:Set(function() return controls.rolloverGroup.Left() - 9 end)
    controls.roBG2.Depth:Set(function() return controls.roBG.Depth() - 1 end)

    if not controls.roIcon then
        controls.roIcon = Bitmap(controls.rolloverGroup)
    end
    controls.roIcon.Top:Set(function() return controls.rolloverGroup.Top() - 3 end)
    controls.roIcon.Left:Set(function() return controls.rolloverGroup.Left() + 4 end)
    controls.roIcon.Height:Set(46)
    controls.roIcon.Width:Set(46)
    
    if not controls.roStratIcon then
        controls.roStratIcon = Bitmap(controls.rolloverGroup)
    end
    controls.roStratIcon.Top:Set(function() return controls.roIcon.Top() + 12 end)
    controls.roStratIcon.Left:Set(function() return controls.roIcon.Right() + 10 end)

    local previousPip = false
    for i = 1, 3 do
        if not controls.roTechLevelPips[i] then
            controls.roTechLevelPips[i] = Bitmap(controls.rolloverGroup)
        end
        if i == 1 then
            controls.roTechLevelPips[i].Left:Set(function() return controls.roStratIcon.Right() + 4 end)
        else
            controls.roTechLevelPips[i].Left:Set(previousPip.Right)
        end
        controls.roTechLevelPips[i].Top:Set(function() return controls.roIcon.Top() + 17 end)
        controls.roTechLevelPips[i].Depth:Set(function() return controls.roIcon.Depth() + 1 end)
        previousPip = controls.roTechLevelPips[i]
    end

    if not controls.roHealthBar then
        controls.roHealthBar = StatusBar(controls.rolloverGroup, 0, 1, false, false, UIUtil.UIFile('/game/unit-over/unit-over_bmp.dds'), UIUtil.UIFile('/game/unit-over/bar01_bmp.dds'), true, "Unit RO Health Status Bar")
    end
    controls.roHealthBar.Left:Set(function() return controls.roIcon.Right() + 10 end)
    controls.roHealthBar.Right:Set(function() return controls.roHealthBar.Left() + 75 end)
    controls.roHealthBar.Bottom:Set(function() return controls.rolloverGroup.Bottom() - 25 end)
    controls.roHealthBar.Top:Set(function() return controls.roHealthBar.Bottom() - 4 end)
    controls.roHealthBar.Height:Set(function() return controls.roHealthBar.Bottom() - controls.roHealthBar.Top() end)
    controls.roHealthBar.Width:Set(function() return controls.roHealthBar.Right() - controls.roHealthBar.Left() end)

    if not controls.roName then
        controls.roName = UIUtil.CreateText(controls.rolloverGroup, "", 12, UIUtil.bodyFont)
    end
    controls.roName.Top:Set(function() return controls.roIcon.Top() + 3 end)
    controls.roName.Left:Set(function() return controls.roIcon.Right() + 65 end)
    controls.roName:SetDropShadow(true)
    
    local prevVetIcon = false
    for i=1, 5 do
        if not controls.roVeteranIcons[i] then
            controls.roVeteranIcons[i] = Bitmap(controls.rolloverGroup)
        end
        if i == 1 then
            controls.roVeteranIcons[i].Left:Set(controls.roStratIcon.Left)
        else
            controls.roVeteranIcons[i].Left:Set(function() return prevVetIcon.Right() - 3 end)
        end
        controls.roVeteranIcons[i].Bottom:Set(function() return controls.roStratIcon.Top() end)
        controls.roVeteranIcons[i]:SetTexture(UIUtil.UIFile('/game/veteran-logo_bmp/uef-veteran_bmp.dds'))
        controls.roVeteranIcons[i]:Hide()
        prevVetIcon = controls.roVeteranIcons[i]
    end
    
    if not controls.roId then
        controls.roId = UIUtil.CreateText(controls.rolloverGroup, "", 12, UIUtil.bodyFont)
    end
    controls.roId.Top:Set(function() return controls.roName.Bottom() + 12 end)
    LayoutHelpers.AtHorizontalCenterIn(controls.roId, controls.roHealthBar)
    controls.roId:SetDropShadow(true)


    if not controls.roHealth then
        controls.roHealth = UIUtil.CreateText(controls.roHealthBar, "", 10, UIUtil.bodyFont)
    end
    LayoutHelpers.AtHorizontalCenterIn(controls.roHealth, controls.roHealthBar)
    controls.roHealth.Top:Set(function() return controls.roHealthBar.Bottom() + 4 end)
    controls.roHealth.Depth:Set(function() return controls.roHealthBar.Depth() + 10 end)
    controls.roHealth:SetDropShadow(true)

    if not controls.roDisplay['mass'] then
        controls.roDisplay['mass'] = {}
        controls.roDisplay['mass'].text = UIUtil.CreateText(controls.rolloverGroup, "", 12, UIUtil.bodyFont)
        controls.roDisplay['mass'].icon = Bitmap(controls.rolloverGroup)
    end
    controls.roDisplay['mass'].icon.Left:Set(function() return controls.roHealthBar.Right() + 10 end)
    controls.roDisplay['mass'].icon.Top:Set(function() return controls.roHealthBar.Top() end)
    controls.roDisplay['mass'].text.Left:Set(function() return controls.roDisplay['mass'].icon.Right()  end)
    controls.roDisplay['mass'].text.Top:Set(function() return controls.roDisplay['mass'].icon.Top() + 1 end)

    if not controls.roDisplay['energy'] then
        controls.roDisplay['energy'] = {}
        controls.roDisplay['energy'].text = UIUtil.CreateText(controls.rolloverGroup, "", 12, UIUtil.bodyFont)
        controls.roDisplay['energy'].icon = Bitmap(controls.rolloverGroup)
    end
    controls.roDisplay['energy'].icon.Left:Set(function() return controls.roDisplay['mass'].icon.Right() + 43 end)
    controls.roDisplay['energy'].icon.Top:Set(controls.roDisplay['mass'].icon.Top)
    controls.roDisplay['energy'].text.Left:Set(function() return controls.roDisplay['energy'].icon.Right()  end)
    controls.roDisplay['energy'].text.Top:Set(function() return controls.roDisplay['energy'].icon.Top() + 1 end)

    for i=1, numRoDisplayFields do
        if not controls.roDisplay[i] then
            controls.roDisplay[i] = {}
            controls.roDisplay[i].text = UIUtil.CreateText(controls.rolloverGroup, "", 12, UIUtil.bodyFont)
            controls.roDisplay[i].icon = Bitmap(controls.rolloverGroup)
        end
        if i == 1 then
            controls.roDisplay[1].icon.Left:Set(function() return controls.roDisplay['energy'].icon.Right() + 43 end)
            controls.roDisplay[1].icon.Top:Set(controls.roDisplay['mass'].icon.Top)
            controls.roDisplay[1].text.Left:Set(function() return controls.roDisplay[1].icon.Right() + 5 end)
            controls.roDisplay[1].text.Top:Set(function() return controls.roName.Bottom() + 8 end)
        else
            local prevControl = controls.roDisplay[i - 1].text
            controls.roDisplay[i].icon.Left:Set(function() return prevControl.Right() + 15 end)
            controls.roDisplay[i].icon.Bottom:Set(controls.roDisplay[1].text.Bottom)
            controls.roDisplay[i].text.Left:Set(function() return prevControl.Right() + 35 end)
            controls.roDisplay[i].text.Bottom:Set(controls.roDisplay[1].text.Bottom)
        end
    end

    if not controls.roFocusBG then
        controls.roFocusBG = Bitmap(controls.rolloverGroup, UIUtil.UIFile('/game/unit-over/unit-over-build_bmp.dds'))
    end
    controls.roFocusBG.Top:Set(function() return controls.roBG.Top() + 6 end)
    controls.roFocusBG.Left:Set(function() return controls.roBG.Right() - 44 end)

    if not controls.roFocusIcon then
        controls.roFocusIcon = Bitmap(controls.rolloverGroup)
    end
    controls.roFocusIcon.Top:Set(controls.roIcon.Top)
    controls.roFocusIcon.Left:Set(function() return controls.roFocusBG.Left() + 40 end)
    controls.roFocusIcon.Height:Set(46)
    controls.roFocusIcon.Width:Set(46)
    controls.roFocusIcon.Depth:Set(function() return controls.roFocusBG.Depth() + 1 end)
    
    if not controls.roFocusHealthPercent then
        controls.roFocusHealthPercent = UIUtil.CreateText(controls.rolloverGroup, "", 14, UIUtil.bodyFont)
    end
    controls.roFocusHealthPercent.Bottom:Set(function() return controls.roFocusIcon.Bottom() - 5 end)
    controls.roFocusHealthPercent.Right:Set(function() return controls.roFocusIcon.Right() + 40 end)
    controls.roFocusHealthPercent:SetDropShadow(true)
    controls.roFocusHealthPercent.Depth:Set(function() return controls.roFocusBG.Depth() + 1 end)
    
    if not controls.roFocusHealthBar then
        controls.roFocusHealthBar = StatusBar(controls.rolloverGroup, 0, 1, false, false, UIUtil.UIFile('/game/unit-over/unit-over_bmp.dds'), UIUtil.UIFile('/game/unit-over/bar01_bmp.dds'), true, "Unit RO Health Status Bar")
    end
    controls.roFocusHealthBar.Left:Set(function() return controls.roFocusHealthPercent.Right() + 10 end)
    controls.roFocusHealthBar.Right:Set(function() return controls.roFocusHealthBar.Left() + 184 end)
    controls.roFocusHealthBar.Bottom:Set(function() return controls.roFocusHealthPercent.Bottom() - 5 end)
    controls.roFocusHealthBar.Top:Set(function() return controls.roFocusHealthBar.Bottom() - 5 end)
    controls.roFocusHealthBar.Height:Set(function() return controls.roFocusHealthBar.Bottom() - controls.roFocusHealthBar.Top() end)
    controls.roFocusHealthBar.Width:Set(function() return controls.roFocusHealthBar.Right() - controls.roFocusHealthBar.Left() end)
    controls.roFocusHealthBar.Depth:Set(function() return controls.roFocusBG.Depth() + 1 end)
    controls.roFocusHealthBar._bar.Depth:Set(function() return controls.roFocusBG.Depth() + 2 end)

    if not controls.roFocusName then
        controls.roFocusName = UIUtil.CreateText(controls.rolloverGroup, "", 14, UIUtil.bodyFont)
    end
    controls.roFocusName.Top:Set(function() return controls.roIcon.Top() + 7 end)
    controls.roFocusName.Left:Set(function() return controls.roFocusIcon.Right() + 15 end)
    controls.roFocusName:SetDropShadow(true)
    controls.roFocusName.Depth:Set(function() return controls.roFocusBG.Depth() + 1 end)

    controls.rolloverGroup:DisableHitTest(true)
end

function DoRightLayout()
    local worldView = import('/lua/modules/ui/game/borders.lua').GetMapGroup()
    if not controls.rolloverGroup then
        controls.rolloverGroup = Group(controls.parent, "Unit Rollover Group")
    end
    controls.rolloverGroup.Width:Set(500)
    controls.rolloverGroup.Bottom:Set(worldView.Bottom)
    controls.rolloverGroup.Top:Set(function() return controls.rolloverGroup.Bottom() - 50 end)
    controls.rolloverGroup.Left:Set(function() return worldView.Left() + 5 end)
    controls.rolloverGroup.Depth:Set(function() return controls.parent.Depth() + 50 end)

    if not controls.roBG then
        controls.roBG = Bitmap(controls.rolloverGroup, UIUtil.UIFile('/game/unit-over/unit-over_bmp.dds'))
    end
    controls.roBG.Bottom:Set(controls.rolloverGroup.Bottom)
    controls.roBG.Left:Set(function() return controls.rolloverGroup.Left() - 5 end)

    if not controls.roBG2 then
        controls.roBG2 = Bitmap(controls.rolloverGroup, UIUtil.UIFile('/game/border_bottom/back_brd_bl02.dds'))
    end
    controls.roBG2.Bottom:Set(function() return controls.rolloverGroup.Bottom() + 2 end)
    controls.roBG2.Left:Set(function() return controls.rolloverGroup.Left() - 6 end)
    controls.roBG2.Depth:Set(function() return controls.roBG.Depth() - 1 end)

    if not controls.roIcon then
        controls.roIcon = Bitmap(controls.rolloverGroup)
    end
    controls.roIcon.Top:Set(function() return controls.rolloverGroup.Top() - 3 end)
    controls.roIcon.Left:Set(function() return controls.rolloverGroup.Left() + 4 end)
    controls.roIcon.Height:Set(46)
    controls.roIcon.Width:Set(46)
    
    if not controls.roStratIcon then
        controls.roStratIcon = Bitmap(controls.rolloverGroup)
    end
    controls.roStratIcon.Top:Set(function() return controls.roIcon.Top() + 12 end)
    controls.roStratIcon.Left:Set(function() return controls.roIcon.Right() + 10 end)

    local previousPip = false
    for i = 1, 3 do
        if not controls.roTechLevelPips[i] then
            controls.roTechLevelPips[i] = Bitmap(controls.rolloverGroup)
        end
        if i == 1 then
            controls.roTechLevelPips[i].Left:Set(function() return controls.roStratIcon.Right() + 4 end)
        else
            controls.roTechLevelPips[i].Left:Set(previousPip.Right)
        end
        controls.roTechLevelPips[i].Top:Set(function() return controls.roIcon.Top() + 17 end)
        controls.roTechLevelPips[i].Depth:Set(function() return controls.roIcon.Depth() + 1 end)
        previousPip = controls.roTechLevelPips[i]
    end

    if not controls.roHealthBar then
        controls.roHealthBar = StatusBar(controls.rolloverGroup, 0, 1, false, false, UIUtil.UIFile('/game/unit-over/unit-over_bmp.dds'), UIUtil.UIFile('/game/unit-over/bar01_bmp.dds'), true, "Unit RO Health Status Bar")
    end
    controls.roHealthBar.Left:Set(function() return controls.roIcon.Right() + 10 end)
    controls.roHealthBar.Right:Set(function() return controls.roHealthBar.Left() + 75 end)
    controls.roHealthBar.Bottom:Set(function() return controls.rolloverGroup.Bottom() - 25 end)
    controls.roHealthBar.Top:Set(function() return controls.roHealthBar.Bottom() - 4 end)
    controls.roHealthBar.Height:Set(function() return controls.roHealthBar.Bottom() - controls.roHealthBar.Top() end)
    controls.roHealthBar.Width:Set(function() return controls.roHealthBar.Right() - controls.roHealthBar.Left() end)

    if not controls.roName then
        controls.roName = UIUtil.CreateText(controls.rolloverGroup, "", 12, UIUtil.bodyFont)
    end
    controls.roName.Top:Set(function() return controls.roIcon.Top() + 3 end)
    controls.roName.Left:Set(function() return controls.roIcon.Right() + 65 end)
    controls.roName:SetDropShadow(true)
    
    local prevVetIcon = false
    for i=1, 5 do
        if not controls.roVeteranIcons[i] then
            controls.roVeteranIcons[i] = Bitmap(controls.rolloverGroup)
        end
        if i == 1 then
            controls.roVeteranIcons[i].Left:Set(controls.roStratIcon.Left)
        else
            controls.roVeteranIcons[i].Left:Set(function() return prevVetIcon.Right() - 3 end)
        end
        controls.roVeteranIcons[i].Bottom:Set(function() return controls.roStratIcon.Top() end)
        controls.roVeteranIcons[i]:SetTexture(UIUtil.UIFile('/game/veteran-logo_bmp/uef-veteran_bmp.dds'))
        controls.roVeteranIcons[i]:Hide()
        prevVetIcon = controls.roVeteranIcons[i]
    end
    
    if not controls.roId then
        controls.roId = UIUtil.CreateText(controls.rolloverGroup, "", 12, UIUtil.bodyFont)
    end
    controls.roId.Top:Set(function() return controls.roName.Bottom() + 12 end)
    LayoutHelpers.AtHorizontalCenterIn(controls.roId, controls.roHealthBar)
    controls.roId:SetDropShadow(true)


    if not controls.roHealth then
        controls.roHealth = UIUtil.CreateText(controls.roHealthBar, "", 10, UIUtil.bodyFont)
    end
    LayoutHelpers.AtHorizontalCenterIn(controls.roHealth, controls.roHealthBar)
    controls.roHealth.Top:Set(function() return controls.roHealthBar.Bottom() + 4 end)
    controls.roHealth.Depth:Set(function() return controls.roHealthBar.Depth() + 10 end)
    controls.roHealth:SetDropShadow(true)

    if not controls.roDisplay['mass'] then
        controls.roDisplay['mass'] = {}
        controls.roDisplay['mass'].text = UIUtil.CreateText(controls.rolloverGroup, "", 12, UIUtil.bodyFont)
        controls.roDisplay['mass'].icon = Bitmap(controls.rolloverGroup)
    end
    controls.roDisplay['mass'].icon.Left:Set(function() return controls.roHealthBar.Right() + 10 end)
    controls.roDisplay['mass'].icon.Top:Set(function() return controls.roHealthBar.Top() end)
    controls.roDisplay['mass'].text.Left:Set(function() return controls.roDisplay['mass'].icon.Right()  end)
    controls.roDisplay['mass'].text.Top:Set(function() return controls.roDisplay['mass'].icon.Top() + 1 end)

    if not controls.roDisplay['energy'] then
        controls.roDisplay['energy'] = {}
        controls.roDisplay['energy'].text = UIUtil.CreateText(controls.rolloverGroup, "", 12, UIUtil.bodyFont)
        controls.roDisplay['energy'].icon = Bitmap(controls.rolloverGroup)
    end
    controls.roDisplay['energy'].icon.Left:Set(function() return controls.roDisplay['mass'].icon.Right() + 43 end)
    controls.roDisplay['energy'].icon.Top:Set(controls.roDisplay['mass'].icon.Top)
    controls.roDisplay['energy'].text.Left:Set(function() return controls.roDisplay['energy'].icon.Right()  end)
    controls.roDisplay['energy'].text.Top:Set(function() return controls.roDisplay['energy'].icon.Top() + 1 end)

    for i=1, numRoDisplayFields do
        if not controls.roDisplay[i] then
            controls.roDisplay[i] = {}
            controls.roDisplay[i].text = UIUtil.CreateText(controls.rolloverGroup, "", 12, UIUtil.bodyFont)
            controls.roDisplay[i].icon = Bitmap(controls.rolloverGroup)
        end
        if i == 1 then
            controls.roDisplay[1].icon.Left:Set(function() return controls.roDisplay['energy'].icon.Right() + 43 end)
            controls.roDisplay[1].icon.Top:Set(controls.roDisplay['mass'].icon.Top)
            controls.roDisplay[1].text.Left:Set(function() return controls.roDisplay[1].icon.Right() + 5 end)
            controls.roDisplay[1].text.Top:Set(function() return controls.roName.Bottom() + 8 end)
        else
            local prevControl = controls.roDisplay[i - 1].text
            controls.roDisplay[i].icon.Left:Set(function() return prevControl.Right() + 15 end)
            controls.roDisplay[i].icon.Bottom:Set(controls.roDisplay[1].text.Bottom)
            controls.roDisplay[i].text.Left:Set(function() return prevControl.Right() + 35 end)
            controls.roDisplay[i].text.Bottom:Set(controls.roDisplay[1].text.Bottom)
        end
    end

    if not controls.roFocusBG then
        controls.roFocusBG = Bitmap(controls.rolloverGroup, UIUtil.UIFile('/game/unit-over/unit-over-build_bmp.dds'))
    end
    controls.roFocusBG.Top:Set(function() return controls.roBG.Top() + 6 end)
    controls.roFocusBG.Left:Set(function() return controls.roBG.Right() - 44 end)

    if not controls.roFocusIcon then
        controls.roFocusIcon = Bitmap(controls.rolloverGroup)
    end
    controls.roFocusIcon.Top:Set(controls.roIcon.Top)
    controls.roFocusIcon.Left:Set(function() return controls.roFocusBG.Left() + 40 end)
    controls.roFocusIcon.Height:Set(46)
    controls.roFocusIcon.Width:Set(46)
    controls.roFocusIcon.Depth:Set(function() return controls.roFocusBG.Depth() + 1 end)
    
    if not controls.roFocusHealthPercent then
        controls.roFocusHealthPercent = UIUtil.CreateText(controls.rolloverGroup, "", 14, UIUtil.bodyFont)
    end
    controls.roFocusHealthPercent.Bottom:Set(function() return controls.roFocusIcon.Bottom() - 5 end)
    controls.roFocusHealthPercent.Right:Set(function() return controls.roFocusIcon.Right() + 40 end)
    controls.roFocusHealthPercent:SetDropShadow(true)
    controls.roFocusHealthPercent.Depth:Set(function() return controls.roFocusBG.Depth() + 1 end)
    
    if not controls.roFocusHealthBar then
        controls.roFocusHealthBar = StatusBar(controls.rolloverGroup, 0, 1, false, false, UIUtil.UIFile('/game/unit-over/unit-over_bmp.dds'), UIUtil.UIFile('/game/unit-over/bar01_bmp.dds'), true, "Unit RO Health Status Bar")
    end
    controls.roFocusHealthBar.Left:Set(function() return controls.roFocusHealthPercent.Right() + 10 end)
    controls.roFocusHealthBar.Right:Set(function() return controls.roFocusHealthBar.Left() + 184 end)
    controls.roFocusHealthBar.Bottom:Set(function() return controls.roFocusHealthPercent.Bottom() - 5 end)
    controls.roFocusHealthBar.Top:Set(function() return controls.roFocusHealthBar.Bottom() - 5 end)
    controls.roFocusHealthBar.Height:Set(function() return controls.roFocusHealthBar.Bottom() - controls.roFocusHealthBar.Top() end)
    controls.roFocusHealthBar.Width:Set(function() return controls.roFocusHealthBar.Right() - controls.roFocusHealthBar.Left() end)
    controls.roFocusHealthBar.Depth:Set(function() return controls.roFocusBG.Depth() + 1 end)
    controls.roFocusHealthBar._bar.Depth:Set(function() return controls.roFocusBG.Depth() + 2 end)

    if not controls.roFocusName then
        controls.roFocusName = UIUtil.CreateText(controls.rolloverGroup, "", 14, UIUtil.bodyFont)
    end
    controls.roFocusName.Top:Set(function() return controls.roIcon.Top() + 7 end)
    controls.roFocusName.Left:Set(function() return controls.roFocusIcon.Right() + 15 end)
    controls.roFocusName:SetDropShadow(true)
    controls.roFocusName.Depth:Set(function() return controls.roFocusBG.Depth() + 1 end)

    controls.rolloverGroup:DisableHitTest(true)
end

function SetLayout(layout)
    if layout == 'bottom' then
        DoBottomLayout()
    elseif layout == 'left' then
        DoLeftLayout()
    elseif layout == 'right' then
        DoRightLayout()
    end
    controls.roBG:DisableHitTest(true)
    controls.rolloverGroup:DisableHitTest(true)
    if controls.rolloverGroup then
        -- get frame updates so we can check what's under the cursor
        controls.rolloverGroup:DisableHitTest(true)
        controls.rolloverGroup:SetNeedsFrameUpdate(true)
        controls.rolloverGroup.OnFrame = function(self, elapsedTime)
            -- supress rollover if UI hidden
            if import('/lua/modules/ui/game/gamemain.lua').gameUIHidden then return end
            rolloverInfo = GetRolloverInfo()
            local bp = nil
            if rolloverInfo then
                controls.rolloverGroup:SetAlpha(1, true)
                bp = __blueprints[rolloverInfo.blueprintId]
            else
                local cam = GetCamera('WorldCamera')
                local camMax = cam:GetMaxZoom()
                local curZoom = cam:GetZoom() / camMax
                if curZoom > .25 then
                    controls.rolloverGroup:SetAlpha(1.1 - curZoom, true)
                else
                    controls.rolloverGroup:SetAlpha(1, true)
                end
            end
            if showBool then
                UpdateROInfo(bp, false)
            end
        end
    end

end

function SetupUnitViewLayout(orderControl)
    controls.parent = orderControl
    SetLayout(UIUtil.currentLayout)
end