
local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local GameCommon = import('/lua/modules/ui/game/gamecommon.lua')
local ItemList = import('/lua/modules/maui/itemlist.lua').ItemList
local Prefs = import('/lua/modules/ui/prefs.lua')
local UnitDescriptions = import('/lua/modules/ui/help/unitdescription.lua').Description

local View = nil
local ViewState = "full"

# Temp layout sizes
local GEN_FULL_WIDTH = 300
local GEN_FULL_HEIGHT = 280
local GEN_MINI_WIDTH = 300
local GEN_MINI_HEIGHT = 100

# Function to calculate 3 columns of resource groups
local widthFunc = function() return (View.BG.Right() - View.UnitName.Left()) / 2 end

function GetTechLevelString(bp)
    if EntityCategoryContains(categories.TECH1, bp.BlueprintId) then
        return 1
    elseif EntityCategoryContains(categories.TECH2, bp.BlueprintId) then
        return 2
    elseif EntityCategoryContains(categories.TECH3, bp.BlueprintId) then
        return 3
    elseif EntityCategoryContains(categories.EXPERIMENTAL, bp.BlueprintId) then
        return 4
    else
        return 3
    end
end

function FormatTime(seconds)
    local tempSeconds = math.floor(seconds)
    local tempMinutes = 00

    tempMinutes = math.floor(tempSeconds / 60)
    tempSeconds = tempSeconds - (tempMinutes * 60)
    if(tempMinutes < 10) then
        tempMinutes = "0" .. tostring(tempMinutes)
    end
    if(tempSeconds < 10) then
        tempSeconds = "0" .. tostring(tempSeconds)
    end
    local tempTime = tostring(tempMinutes) .. ":" .. tostring(tempSeconds)
    return tempTime
end

function CreateResourceGroup(parent, groupLabel)
    local group = Group(parent)

    # Group label
    group.Label = UIUtil.CreateText( group, groupLabel, 12, "Arial Bold" )
    group.Label:SetColor("FFBEBEBE")

    # Energy Icon
    group.EnergyIcon = Bitmap(group)
    group.EnergyIcon:SetTexture(UIUtil.UIFile('/game/unit-over/icon-energy_bmp.dds'))

    # Energy Value
    group.EnergyValue = UIUtil.CreateText( group, "0", 12, UIUtil.bodyFont )
    group.EnergyValue:SetColor("FF00F000")

    # Mass Icon
    group.MassIcon = Bitmap(group)
    group.MassIcon:SetTexture(UIUtil.UIFile('/game/unit-over/icon-mass_bmp.dds'))

    # Mass Value
    group.MassValue = UIUtil.CreateText( group, "0", 12, UIUtil.bodyFont )
    group.MassValue:SetColor("FF00F000")

    return group
end

# A 'Stat Group' is an icon or text label with a value on the right
# e.g.  Health 3000
#       Shield 6000
#       <icon> 24
function CreateStatGroup(parent, labelIcon)
    local group = Group(parent)
    
    group.Label = Bitmap(group)
    group.Label:SetTexture(labelIcon)

    group.Value = UIUtil.CreateText( group, "", 12, UIUtil.bodyFont )
    group.Value:SetColor("FF00F000")

    return group
end

function CreateTextbox(parent, label)
    local group = Group(parent)
    
    group.Label = UIUtil.CreateText( group, LOC(label), 12, "Arial Bold" )
    group.Label:SetColor("FFBEBEBE")

    group.Value = {}
    for i = 1, 5 do
        local index = i
        group.Value[index] = UIUtil.CreateText( group, "Line: "..index, 12, UIUtil.bodyFont)
    end
    
    return group
end

function Create(parent)
    # Main window
    View = Group(parent)
    View.BG = Bitmap(View)
    View.BG:SetSolidColor("80606000")

    # Main window background
    local border,bg = UIUtil.CreateBorder(View,true)
    View.Border = border

    # Unit icon
    if false then
        View.UnitIcon = Bitmap(View)
    end
    
    # Unit Name
    View.UnitName = UIUtil.CreateText( View.BG, "", 18, UIUtil.bodyFont )
    View.UnitName:SetColor("FFFF9E06")

    # Unit Description
    View.UnitShortDesc = UIUtil.CreateText( View.BG, "", 12, UIUtil.bodyFont )
    View.UnitShortDesc:SetColor("FFFF9E06")

    # Separator
    View.Separator = Bitmap(View.BG)
    View.Separator:SetSolidColor("FF000000")

    # Cost
    View.BuildCostGroup = CreateResourceGroup(View.BG, "<LOC uvd_0000>Build Cost (Rate)")

    # Upkeep
    View.UpkeepGroup = CreateResourceGroup(View.BG, "<LOC uvd_0002>Yield")
    
    # Description
    View.Description = CreateTextbox(View.BG, "<LOC uvd_0003>Description")
    
    # Ability List
    View.AbilityList = CreateTextbox(View.BG, "<LOC uvd_0004>Notes")

    # Health stat
    View.HealthStat = CreateStatGroup( View.BG, UIUtil.UIFile('/game/unit_view_icons/redcross.dds') )
    # Time stat
    View.TimeStat = CreateStatGroup( View.BG, UIUtil.UIFile('/game/unit-over/icon-clock_bmp.dds') )

end

function GetAbilityList(bp)
    local abilitiesList = {}

    return abilitiesList
end
    
function CheckFormat()
    if ViewState != Prefs.GetOption('uvd_format') then
        LayoutCommon()
    end
    if ViewState == "off" then
        return false
    else
        return true
    end
end
    
function ShowView(showUpKeep, enhancement, showecon)
    import('/lua/modules/ui/game/unitview.lua').ShowROBox(false, false)
    View:Show()
    if enhancement or not showUpKeep then
        View.UpkeepGroup:Hide()
    end
    if not showecon then
        View.BuildCostGroup:Hide()
        View.UpkeepGroup:Hide()
        View.TimeStat:Hide()
        View.HealthStat:Hide()
    end
    if enhancement then
        View.HealthStat:Hide()
        View.AbilityList:Hide()
    end
    if ViewState == "limited" then
        View.Description:Hide()
        View.AbilityList:Hide()
    end
end
    
function ShowEnhancement(bp, bpID, iconID, userUnit)
    if CheckFormat() then
        # Name / Description
        if false then
            View.UnitIcon:SetTexture(UIUtil.UIFile('/icons/units/default_icon.dds'))    
        end
        if bp.Name != nil then
            View.UnitName:SetText(LOC(bp.Name))
        else
            View.UnitName:SetText("")
        end
        View.UnitShortDesc:SetText(LOC(bp.Slot))
        
        
        local showecon = true
        local showUpKeep = false
        local time, energy, mass
        if bp.Icon != nil then
            time, energy, mass = import('/lua/modules/game.lua').GetConstructEconomyModel(userUnit, bp)
            showUpKeep = DisplayResources(bp, time, energy, mass)
            View.TimeStat.Value:SetText(string.format("%s", FormatTime(time)))
        else
            showecon = false
        end
        
        local tempDescID = bpID.."-"..iconID
        if UnitDescriptions[tempDescID] then
            local tempDesc = LOC(UnitDescriptions[tempDescID])
            local lines = import('/lua/modules/maui/text.lua').WrapText(tempDesc, View.Description.Width(),
                    function(text) return View.Description.Value[1]:GetStringAdvance(text) end
                )
            for i=1,5 do
                local index = i
                if lines[index] then
                    View.Description.Value[index]:SetText(lines[index])
                else
                    View.Description.Value[index]:SetText("")
                end
            end
        else
            for i=1,5 do
                local index = i
                View.Description.Value[index]:SetText("")
            end
            View.Description:Hide()
        end
    
        ShowView(showUpKeep, true, showecon)
        if time == 0 and energy == 0 and mass == 0 then
            View.BuildCostGroup:Hide()
            View.TimeStat:Hide()
        end
    else
        View:Hide()
    end
end
    
function WrapAndPlaceText(text, control)
    local lines = import('/lua/modules/maui/text.lua').WrapText(text, control.Width(),
            function(text) return control.Value[1]:GetStringAdvance(text) end
        )
    for i=1,5 do
        local index = i
        if lines[index] then
            control.Value[index]:SetText(lines[index])
        else
            control.Value[index]:SetText("")
        end
    end
end
    
function Show(bp, buildingUnit, bpID)
    if CheckFormat() then
        # Name / Description
        if false then
            local foo, iconName = GameCommon.GetCachedUnitIconFileNames(bp)
            if iconName then
                View.UnitIcon:SetTexture(iconName)
            else
                View.UnitIcon:SetTexture(UIUtil.UIFile('/icons/units/default_icon.dds'))    
            end
        end
        if bp.General.UnitName != nil then
            View.UnitName:SetText(LOC(bp.General.UnitName))
        else
            View.UnitName:SetText(LOC(bp.Description))
        end
        View.UnitShortDesc:SetText(LOCF("Tech %d %s", GetTechLevelString(bp), bp.Description))
                
        local showecon = true
        local showUpKeep = false
        if buildingUnit != nil then
            local time, energy, mass = import('/lua/modules/game.lua').GetConstructEconomyModel(buildingUnit, bp.Economy)
            
            showUpKeep = DisplayResources(bp, time, energy, mass)
            View.TimeStat.Value:SetText(string.format("%s", FormatTime(time)))
        else
            showecon = false
        end
            
        
        # Health stat
        View.HealthStat.Value:SetText(string.format("%d", bp.Defense.MaxHealth))
    
        if UnitDescriptions[bpID] then
            WrapAndPlaceText(LOC(UnitDescriptions[bpID]), View.Description)
        else
            View.Description:Hide()
            for i=1,5 do
                local index = i
                View.Description.Value[index]:SetText("")
            end
        end
        
        if bp.Display.Abilities != nil then
            for i=1, 5 do   
                local index = i
                if bp.Display.Abilities[index] then
                    View.AbilityList.Value[index]:SetText(LOC(bp.Display.Abilities[index]))
                else
                    View.AbilityList.Value[index]:SetText("")
                end
            end
        else
            for i=1, 5 do   
                local index = i
                if index == 1 then
                    View.AbilityList.Value[index]:SetText(LOC("<LOC uvd_0005>No Abilities"))
                else
                    View.AbilityList.Value[index]:SetText("")
                end
            end
        end
    
        ShowView(showUpKeep, false, showecon)
    else
        View:Hide()
    end
end

function DisplayResources(bp, time, energy, mass)
    # Cost Group
    if time > 0 then
        local consumeEnergy = -energy / time
        local consumeMass = -mass / time
        View.BuildCostGroup.EnergyValue:SetText( string.format("%d (%d)",-energy,consumeEnergy) )
        View.BuildCostGroup.MassValue:SetText( string.format("%d (%d)",-mass,consumeMass) )
        
        View.BuildCostGroup.EnergyValue:SetColor( "FFF00000" )
        View.BuildCostGroup.MassValue:SetColor( "FFF00000" )
    end

    # Upkeep Group
    local upkeepEnergy = GetYield(bp.Economy.MaintenanceConsumptionPerSecondEnergy, bp.Economy.ProductionPerSecondEnergy)
    local upkeepMass = GetYield(bp.Economy.MaintenanceConsumptionPerSecondMass, bp.Economy.ProductionPerSecondMass)
    local showUpkeep = false
    if upkeepEnergy != 0 or upkeepMass != 0 then
        View.UpkeepGroup.Label:SetText(LOC("<LOC uvd_0002>Yield"))
        View.UpkeepGroup.EnergyValue:SetText( string.format("%d",upkeepEnergy) )
        View.UpkeepGroup.MassValue:SetText( string.format("%d",upkeepMass) )
        if upkeepEnergy >= 0 then
            View.UpkeepGroup.EnergyValue:SetColor( "FF00F000" )
        else
            View.UpkeepGroup.EnergyValue:SetColor( "FFF00000" )
        end
    
        if upkeepMass >= 0 then
            View.UpkeepGroup.MassValue:SetColor( "FF00F000" )
        else
            View.UpkeepGroup.MassValue:SetColor( "FFF00000" )
        end
        showUpkeep = true
    elseif bp.Economy.StorageEnergy != 0 or bp.Economy.StorageMass != 0 then
        View.UpkeepGroup.Label:SetText(LOC("<LOC uvd_0006>Storage"))
        local upkeepEnergy = bp.Economy.StorageEnergy or 0
        local upkeepMass = bp.Economy.StorageMass or 0
        View.UpkeepGroup.EnergyValue:SetText( string.format("%d",upkeepEnergy) )
        View.UpkeepGroup.MassValue:SetText( string.format("%d",upkeepMass) )
        View.UpkeepGroup.EnergyValue:SetColor( "FF00F000" )
        View.UpkeepGroup.MassValue:SetColor( "FF00F000" )
        showUpkeep = true
    end
    
    return showUpkeep 
end

function GetYield(consumption, production)
    if consumption then
        return -consumption
    elseif production then
        return production
    else
        return 0
    end
end

function Hide()
    View:Hide()
end

function LayoutResourceGroup(group)
    
    LayoutHelpers.AtTopIn( group.Label, group )
    LayoutHelpers.AtHorizontalCenterIn( group.Label, group )

    LayoutHelpers.Below( group.EnergyIcon, group.Label )
    group.EnergyIcon.Left:Set( function() return group.Left() - 4 end )

    LayoutHelpers.RightOf( group.EnergyValue, group.EnergyIcon )
    group.EnergyValue.Top:Set( function() return group.EnergyIcon.Top() + 1 end )

    LayoutHelpers.RightOf( group.MassValue, group.MassIcon )
    group.MassValue.Right:Set( function() return group.Label.Right() end )
    group.MassValue.Top:Set( function() return group.MassIcon.Top() + 1 end )

    LayoutHelpers.Below( group.MassIcon, group.EnergyIcon )
end

function LayoutStatGroup(group)
    group.Label.Right:Set(group.Right)
    group.Label.Top:Set(group.Top)
    group.Value.Right:Set(function() return group.Label.Left() - 4 end)
    group.Value.Top:Set(group.Top)
end  

function LayoutTextbox(group)
    group.Label.Top:Set(group.Top)
    group.Label.Left:Set(group.Left)
    
    for i = 1, 5 do
        local index = i
        if index == 1 then
            group.Value[1].Top:Set(group.Label.Bottom)
            group.Value[1].Left:Set(group.Label.Left)
        else
            group.Value[index].Top:Set(group.Value[index-1].Bottom)
            group.Value[index].Left:Set(group.Value[1].Left)
        end
    end
end

function LayoutCommon()
    ViewState = Prefs.GetOption('uvd_format')
    
    # Main window background
    LayoutHelpers.FillParent( View.BG, View )

    # Main window border
    View.Border:LayoutAroundControl(View)

    # Unit Name
    LayoutHelpers.AtLeftTopIn( View.UnitName, View.BG, 4, 4 )
    View.UnitName.Width:Set( function() return View.BG.Right() - View.UnitName.Left() end )

    # Health Stat
    View.HealthStat.Right:Set(function() return View.BG.Right() - 4 end)
    View.HealthStat.Top:Set(View.UnitName.Top)
    View.HealthStat.Width:Set(1)
    View.HealthStat.Height:Set(View.HealthStat.Label.Height)
    LayoutStatGroup( View.HealthStat )
    
    # Shield stat
    LayoutHelpers.Below( View.TimeStat, View.HealthStat, 4 )
    View.TimeStat.Width:Set(1)
    View.TimeStat.Height:Set(View.TimeStat.Label.Height)
    LayoutStatGroup( View.TimeStat )
 
    # Unit Description
    LayoutHelpers.Below( View.UnitShortDesc, View.UnitName )
    View.UnitShortDesc.Width:Set( View.BG.Width )
    
    # Separator
    LayoutHelpers.Below( View.Separator, View.UnitShortDesc )
    View.Separator.Width:Set( View.UnitName.Width )
    
    # Build Resource Group
    LayoutHelpers.Below( View.BuildCostGroup, View.Separator )
    View.BuildCostGroup.Width:Set( widthFunc )
    LayoutResourceGroup( View.BuildCostGroup )
    View.BuildCostGroup.Bottom:Set( function() return View.BuildCostGroup.MassValue.Bottom() + 1 end )

    # Upkeep Resource Group
    LayoutHelpers.RightOf( View.UpkeepGroup, View.BuildCostGroup )
    View.UpkeepGroup.Width:Set( widthFunc )
    View.UpkeepGroup.Bottom:Set( View.BuildCostGroup.Bottom )
    LayoutResourceGroup( View.UpkeepGroup )
    
    # Description
    LayoutHelpers.Below(View.Description, View.BuildCostGroup, 4 )
    View.Description.Width:Set(function() return View.HealthStat.Right() - View.UnitName.Left() end)
    LayoutTextbox(View.Description)
    View.Description.Bottom:Set(View.Description.Value[5].Bottom)
    
    # Ability List
    LayoutHelpers.Below( View.AbilityList, View.Description, 4 )
    View.AbilityList.Width:Set(View.Description.Width)
    LayoutTextbox(View.AbilityList)
    View.AbilityList.Bottom:Set(View.AbilityList.Value[5].Bottom)
    
    if ViewState == "full" then
        View.Width:Set( GEN_FULL_WIDTH )
        View.Height:Set( GEN_FULL_HEIGHT )
    elseif ViewState == "limited" then
        View.Width:Set( GEN_MINI_WIDTH )
        View.Height:Set( GEN_MINI_HEIGHT )
    end
end


function DoBottomLayout(relativeToControl)
    LayoutHelpers.Reset(View)
    local parent = View:GetParent()
    #View.Left:Set( relativeToControl.Left )
    LayoutHelpers.AtHorizontalCenterIn(View, relativeToControl)
    View.Bottom:Set( function() return relativeToControl.Top() - 14 end )
    View.Width:Set( GEN_FULL_WIDTH )
    View.Height:Set( GEN_FULL_HEIGHT )

    LayoutCommon()
end

function DoLeftLayout(relativeToControl)
    LayoutHelpers.Reset(View)
    local parent = View:GetParent()
    View.Left:Set( function() return relativeToControl.Right() + 12 end )
    View.Top:Set( relativeToControl.Top )
    View.Width:Set( GEN_FULL_WIDTH )
    View.Height:Set( GEN_FULL_HEIGHT )

    LayoutCommon()
end

function DoRightLayout(relativeToControl)
    LayoutHelpers.Reset(View)
    local parent = View:GetParent()
    View.Left:Set( function() return relativeToControl.Left() - View.Width() - 12 end )
    View.Top:Set( relativeToControl.Top )
    View.Width:Set( GEN_FULL_WIDTH )
    View.Height:Set( GEN_FULL_HEIGHT )

    LayoutCommon()
end

function SetLayout(layout,relativeToControl)
    if not View then
        return
    end

    if layout == 'bottom' then
        DoBottomLayout(relativeToControl)
    elseif layout == 'left' then
        DoLeftLayout(relativeToControl)
    elseif layout == 'right' then
        DoRightLayout(relativeToControl)
    end
end

function SetupUnitViewLayout(parent,relativeToControl)
    if View then
        View:Destroy()
        View = nil
    end
    Create(parent)
    SetLayout(UIUtil.currentLayout,relativeToControl)
    View:Hide()
end
