--*****************************************************************************
--* File: lua/modules/ui/game/worldscroll.lua
--* Author: Chris Blackwell
--* Summary: Scroll controls for edge of world
--*
--* Copyright  2006 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local Group = import('/lua/modules/maui/group.lua').Group

maxPanSpeed = 50
scrollBorderSize = 1

function CreateWorldScrollers()
    local mainFrame = GetFrame(0)

    -- note that the left and right groups control diagonal scrolling as well
    local leftGroup = Group(mainFrame)
    leftGroup.Depth:Set(50000)    -- set really high so it's above game
    leftGroup.Left:Set(mainFrame.Left)
    leftGroup.Right:Set(function() return leftGroup.Left() + scrollBorderSize end)
    leftGroup.Top:Set(mainFrame.Top)
    leftGroup.Bottom:Set(mainFrame.Bottom)

    local leftMouseX = 0
    local leftMouseY = 0
    leftGroup.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            self:SetNeedsFrameUpdate(true)
        elseif event.Type == 'MouseExit' then
            self:SetNeedsFrameUpdate(false)
        elseif event.Type == 'MouseMotion' then
            leftMouseX = event.MouseX
            leftMouseY = event.MouseY
        end
        return false
    end

    leftGroup.OnFrame = function(self, elapsedTime)
        -- maybe this should be exponential?
        local xPan = (1.0 - ((leftMouseX - self.Left()) / self.Width())) * maxPanSpeed
        local adjustedMouseY = leftMouseY - self.Top()
        local yPan = 0
        -- width in these calculations is not a bug, it's to get the border size, not the height!
        if adjustedMouseY <= self.Top() + scrollBorderSize then
            yPan = (1.0 - ((adjustedMouseY - self.Top()) / self.Width())) * maxPanSpeed
        elseif adjustedMouseY >= self.Bottom() - scrollBorderSize then
            yPan = -(1.0 - ((self.Bottom() - adjustedMouseY) / self.Width())) * maxPanSpeed
        end
        PanMainView(xPan, yPan)
    end

    local rightGroup = Group(mainFrame, "wsRightGroup")
    rightGroup.Depth:Set(50000)    -- set really high so it's above game
    rightGroup.Right:Set(mainFrame.Right)
    rightGroup.Left:Set(function() return rightGroup.Right() - scrollBorderSize end)
    rightGroup.Top:Set(mainFrame.Top)
    rightGroup.Bottom:Set(mainFrame.Bottom)

    local rightMouseX = 0
    local rightMouseY = 0
    rightGroup.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            self:SetNeedsFrameUpdate(true)
        elseif event.Type == 'MouseExit' then
            self:SetNeedsFrameUpdate(false)
        elseif event.Type == 'MouseMotion' then
            rightMouseX = event.MouseX
            rightMouseY = event.MouseY
        end
        return false
    end

    rightGroup.OnFrame = function(self, elapsedTime)
        local xPan = ((1 + rightMouseX - self.Left()) / self.Width()) * maxPanSpeed
        local adjustedMouseY = rightMouseY - self.Top()
        local yPan = 0
        -- width in these calculations is not a bug, it's to get the border size, not the height!
        if adjustedMouseY <= self.Top() + scrollBorderSize then
            yPan = (1.0 - ((adjustedMouseY - self.Top()) / self.Width())) * maxPanSpeed
        elseif adjustedMouseY >= self.Bottom() - scrollBorderSize then
            yPan = -(1.0 - ((self.Bottom() - adjustedMouseY) / self.Width())) * maxPanSpeed
        end
        PanMainView(-xPan, yPan)
    end

    local topGroup = Group(mainFrame, "wsTopGroup")
    topGroup.Depth:Set(50000)    -- set really high so it's above game
    topGroup.Top:Set(mainFrame.Top)
    topGroup.Bottom:Set(function() return topGroup.Top() + scrollBorderSize end)
    topGroup.Left:Set(leftGroup.Right)
    topGroup.Right:Set(rightGroup.Left)

    local topMouseY = 0
    topGroup.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            self:SetNeedsFrameUpdate(true)
        elseif event.Type == 'MouseExit' then
            self:SetNeedsFrameUpdate(false)
        elseif event.Type == 'MouseMotion' then
            topMouseY = event.MouseY
        end
        return false
    end

    topGroup.OnFrame = function(self, elapsedTime)
        local pan = (1.0 - ((topMouseY - self.Top()) / self.Height())) * maxPanSpeed
        PanMainView(0, pan)
    end

    local bottomGroup = Group(mainFrame, "wsBottomGroup")
    bottomGroup.Depth:Set(50000)    -- set really high so it's above game
    bottomGroup.Bottom:Set(mainFrame.Bottom)
    bottomGroup.Top:Set(function() return bottomGroup.Bottom() - scrollBorderSize end)
    bottomGroup.Left:Set(leftGroup.Right)
    bottomGroup.Right:Set(rightGroup.Left)

    local bottomMouseY = 0
    bottomGroup.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            self:SetNeedsFrameUpdate(true)
        elseif event.Type == 'MouseExit' then
            self:SetNeedsFrameUpdate(false)
        elseif event.Type == 'MouseMotion' then
            bottomMouseY = event.MouseY
        end
        return false
    end

    bottomGroup.OnFrame = function(self, elapsedTime)
        local pan = ((1 + bottomMouseY - self.Top()) / self.Height()) * maxPanSpeed
        PanMainView(0, -pan)
    end
end
