--*****************************************************************************
--* File: lua/modules/ui/game/wolrdview.lua
--* Author: Chris Blackwell
--* Summary: UI to manage the games main world view
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Group = import('/lua/modules/maui/group.lua').Group

view = false
viewLeft = false
viewRight = false
secondaryView = false
tertiaryView = false
local parentForFrame = false

function CreateMainWorldView(parent, mapGroup, mapGroupRight)    
    if viewLeft then    
        viewLeft:Destroy()
        viewLeft = false  
    end
    if viewRight then
        viewRight:Destroy()
        viewRight = false
    end
    if view then
        view:Destroy()
        view = false
    end
    if mapGroupRight then
        parentForFrame = parent
        viewLeft = import('/lua/modules/ui/controls/worldview.lua').WorldView(mapGroup, 'WorldCamera', 1, false) -- depth value should be below minimap
        viewLeft:SetRenderPass(UIUtil.UIRP_UnderWorld) -- don't change this or the camera will lag one frame behind
        LayoutHelpers.FillParent(viewLeft, mapGroup)
        viewLeft:UseCameraPanKeys(true)
        
        viewRight = import('/lua/modules/ui/controls/worldview.lua').WorldView(mapGroupRight, 'WorldCamera2', 1, false) -- depth value should be below minimap
        viewRight:SetRenderPass(UIUtil.UIRP_UnderWorld) -- don't change this or the camera will lag one frame behind
        LayoutHelpers.FillParent(viewRight, mapGroupRight)
        viewRight:UseCameraPanKeys(false)
        
        view = Group(viewLeft)
        view.Left:Set(viewLeft.Left)
        view.Top:Set(viewLeft.Top)
        view.Bottom:Set(viewLeft.Bottom)
        view.Right:Set(viewRight.Right)
        view:DisableHitTest()
    else
        parentForFrame = parent
        viewLeft = import('/lua/modules/ui/controls/worldview.lua').WorldView(mapGroup, 'WorldCamera', 1, false) -- depth value should be below minimap
        viewLeft:SetRenderPass(UIUtil.UIRP_UnderWorld) -- don't change this or the camera will lag one frame behind
        LayoutHelpers.FillParent(viewLeft, mapGroup)
        viewLeft:UseCameraPanKeys(true)
        
        view = Group(viewLeft)
        view:DisableHitTest()
        LayoutHelpers.FillParent(view, viewLeft)
    end
end

-- these two functions will cause the world view to fill the screen or go back to its original settings
local origLeft = false
local origRight = false
local origTop = false
local origBottom = false

function Expand()
    if viewRight then
        origRightLeft = viewRight.Left.compute
        origRightRight = viewRight.Right.compute
        origRightTop = viewRight.Top.compute
        origRightBottom = viewRight.Bottom.compute
        origLeft = viewLeft.Left.compute
        origRight = viewLeft.Right.compute
        origTop = viewLeft.Top.compute
        origBottom = viewLeft.Bottom.compute
        
        local gameViewTemp = GetFrame(parentForFrame:GetRootFrame():GetTargetHead())
        
        viewLeft.Top:Set(gameViewTemp.Top)
        viewLeft.Left:Set(gameViewTemp.Left)
        viewLeft.Right:Set(function() return (gameViewTemp.Left() - 2) + ((gameViewTemp.Right() - gameViewTemp.Left()) / 2) end)
        viewLeft.Bottom:Set(gameViewTemp.Bottom)
        
        viewRight.Top:Set(gameViewTemp.Top)
        viewRight.Left:Set(function() return (gameViewTemp.Left() + 2) + ((gameViewTemp.Right() - gameViewTemp.Left()) / 2) end)
        viewRight.Right:Set(gameViewTemp.Right)
        viewRight.Bottom:Set(gameViewTemp.Bottom)
    else
        origLeft = viewLeft.Left.compute
        origRight = viewLeft.Right.compute
        origTop = viewLeft.Top.compute
        origBottom = viewLeft.Bottom.compute
        LayoutHelpers.FillParent(viewLeft, GetFrame(parentForFrame:GetRootFrame():GetTargetHead()))
    end
end

function Contract()
    if viewRight then        
        viewRight.Left:Set(origRightLeft)
        viewRight.Right:Set(origRightRight)
        viewRight.Top:Set(origRightTop)
        viewRight.Bottom:Set(origRightBottom)
    end
    
    viewLeft.Left:Set(origLeft)
    viewLeft.Right:Set(origRight)
    viewLeft.Top:Set(origTop)
    viewLeft.Bottom:Set(origBottom)
end

function Destroy()
    if viewLeft then
        viewLeft:Destroy()
    end
    if viewRight then
        viewRight:Destroy()
    end
end