--*****************************************************************************
--* File: lua/modules/ui/lobby/directconnect.lua
--* Author: Chris Blackwell
--* Summary: Allow direct connection to a game given an IP, allows you to connect
--* to games not on your subnet (ie over the internet)
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local MenuCommon = import('/lua/modules/ui/menus/menucommon.lua')
local Prefs = import('/lua/modules/ui/prefs.lua')
local Checkbox = import('/lua/modules/maui/checkbox.lua').Checkbox
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Edit = import('/lua/modules/maui/edit.lua').Edit

function CreateEditField(parent)
    local control = Edit(parent)
    control:SetForegroundColor(UIUtil.fontColor)
    control:SetHighlightForegroundColor(UIUtil.highlightColor)
    control:SetHighlightBackgroundColor("880085EF")
    control.Height:Set(function() return control:GetFontHeight() end)
    control.Width:Set(250)
    control:SetFont(UIUtil.bodyFont, 16)
    return control
end

function CreateUI(playerName)
    playerName = playerName or Prefs.GetFromCurrentProfile('NetName') or Prefs.GetFromCurrentProfile('Name')

    local parent = UIUtil.CreateScreenGroup(GetFrame(0), "Direct Connect ScreenGroup")

    local background = MenuCommon.SetupBackground(GetFrame(0))

    local exitButton = MenuCommon.CreateExitMenuButton(parent, background, "<LOC _Back>")

    local panel = Bitmap(parent, UIUtil.SkinnableFile('/lobby/gamecreate/panel_bmp.dds'))
    LayoutHelpers.AtCenterIn(panel, parent)
    
    local panelTitle = UIUtil.CreateText(panel, "<LOC DIRCON_0000>Direct IP Connection", 22)
    LayoutHelpers.AtHorizontalCenterIn(panelTitle, panel)
    panelTitle.Top:Set(function() return panel.Top() + 42 end)

    local ipaddressEdit = CreateEditField(panel)
    LayoutHelpers.AtHorizontalCenterIn(ipaddressEdit, panel)
    ipaddressEdit.Width:Set(340)
    LayoutHelpers.AtTopIn(ipaddressEdit, panel, 114)

    ipaddressEdit:SetText(Prefs.GetFromCurrentProfile('last_dc_ipaddress') or "")

    local ipaddressLabel = UIUtil.CreateText(panel, "<LOC DIRCON_0001>IP Address/Hostname", 14)
    LayoutHelpers.Above(ipaddressLabel, ipaddressEdit)

    local portEdit = CreateEditField(panel)
    LayoutHelpers.Below(portEdit, ipaddressEdit, 36)
    portEdit.Width:Set(ipaddressEdit.Width)
    portEdit:SetText(Prefs.GetFromCurrentProfile('last_dc_port') or "")

    local portLabel = UIUtil.CreateText(panel, "Port", 14)
    LayoutHelpers.Above(portLabel, portEdit)

    local joinButton = UIUtil.CreateButtonStd(panel, '/widgets/small', "<LOC _Join>", 16)
    LayoutHelpers.AtHorizontalCenterIn(joinButton, panel)
    LayoutHelpers.AtBottomIn(joinButton, panel, 50)

    local observerLabel = UIUtil.CreateText(panel, "As Observer", 14)
    LayoutHelpers.CenteredBelow(observerLabel, portEdit, 5)

    local observer = UIUtil.CreateCheckboxStd(panel, '/dialogs/check-box_btn/radio')
    LayoutHelpers.CenteredRightOf(observer, observerLabel, 5)

    exitButton.OnClick = function(self)
        parent:Destroy()
        import('/lua/modules/ui/menus/multiplayer.lua').CreateUI()
    end

    -- ascii values of 0-9
    local portValidSet = {
        [48] = true,
        [49] = true,
        [50] = true,
        [51] = true,
        [52] = true,
        [53] = true,
        [54] = true,
        [55] = true,
        [56] = true,
        [57] = true,
        [46] = true,
    }

    portEdit.OnCharPressed = function(self, charcode)
        if portValidSet[charcode] then
            return false
        else
            return true
        end
    end

    joinButton.OnClick = function(self, modifiers)
        local lobby = import('/lua/modules/ui/lobby/lobby.lua')

        local ipaddress = ipaddressEdit:GetText()
        local port = portEdit:GetText()
        local isobserver = observer:IsChecked()

        Prefs.SetToCurrentProfile('last_dc_ipaddress', ipaddress)
        Prefs.SetToCurrentProfile('last_dc_port', port)

        lobby.CreateLobby("UDP", 0, playerName, nil, nil)
        parent:Destroy()
        lobby.JoinGame(ipaddress .. ":" .. port, isobserver)
    end

end

function CanQuickDirectConnect()
    local playerName = Prefs.GetFromCurrentProfile('NetName') or Prefs.GetFromCurrentProfile('Name')
    local ipaddress = Prefs.GetFromCurrentProfile('last_dc_ipaddress')
    local port = Prefs.GetFromCurrentProfile('last_dc_port')
    return (playerName != nil) and (ipaddress != nil) and (port != nil)
end

function QuickDirectConnect()
    local playerName = Prefs.GetFromCurrentProfile('NetName') or Prefs.GetFromCurrentProfile('Name')
    local ipaddress = Prefs.GetFromCurrentProfile('last_dc_ipaddress')
    local port = Prefs.GetFromCurrentProfile('last_dc_port')
    local lobby = import('/lua/modules/ui/lobby/lobby.lua')
    lobby.CreateLobby("UDP", 0, playerName, nil, nil)
    lobby.JoinGame(ipaddress .. ":" .. port, false)
end