--*****************************************************************************
--* File: lua/modules/ui/lobby/gamecreate.lua
--* Author: Chris Blackwell
--* Summary: game creation UI
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Checkbox = import('/lua/modules/maui/checkbox.lua').Checkbox
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Edit = import('/lua/modules/maui/edit.lua').Edit
local MenuCommon = import('/lua/modules/ui/menus/menucommon.lua')
local Prefs = import('/lua/modules/ui/prefs.lua')

local unselectedCheckboxFile = UIUtil.SkinnableFile('/widgets/rad_un.dds')
local selectedCheckboxFile = UIUtil.SkinnableFile('/widgets/rad_sel.dds')

local errorDialog = false
local defaultPort = 16010   -- this is the default port

function CreateEditField(parent)
    local control = Edit(parent)
    control:SetForegroundColor(UIUtil.fontColor)
    control:SetHighlightForegroundColor(UIUtil.highlightColor)
    control:SetHighlightBackgroundColor("880085EF")
    control.Height:Set(function() return control:GetFontHeight() end)
    control.Width:Set(250)
    control:SetFont(UIUtil.bodyFont, 16)
    return control
end

function CreateUI(playerName)
    playerName = playerName or Prefs.GetFromCurrentProfile('NetName') or Prefs.GetFromCurrentProfile('Name')

-- control layout
    local parent = UIUtil.CreateScreenGroup(GetFrame(0), "Game Create ScreenGroup")

    local background = MenuCommon.SetupBackground(GetFrame(0))

    local exitButton = MenuCommon.CreateExitMenuButton(parent, background, "<LOC _Back>")

    local panel = Bitmap(parent, UIUtil.SkinnableFile('/lobby/gamecreate/panel_bmp.dds'))
    LayoutHelpers.AtCenterIn(panel, parent)
    
    local panelTitle = UIUtil.CreateText(panel, "<LOC _Create_LAN_Game>", 22)
    LayoutHelpers.AtHorizontalCenterIn(panelTitle, panel)
    panelTitle.Top:Set(function() return panel.Top() + 42 end)

    local gameNameEdit = CreateEditField(panel)
    gameNameEdit:SetText(Prefs.GetFromCurrentProfile('last_game_name') or "")
    gameNameEdit.Width:Set(340)
    LayoutHelpers.AtHorizontalCenterIn(gameNameEdit, panel)
    LayoutHelpers.AtTopIn(gameNameEdit, panel, 114)

    local gameNameLabel = UIUtil.CreateText(panel, "<LOC _Game_Name>", 14)
    LayoutHelpers.Above(gameNameLabel, gameNameEdit, 5)
    
    local portEdit = CreateEditField(panel)
    portEdit.Width:Set(gameNameEdit.Width)
    LayoutHelpers.AtHorizontalCenterIn(portEdit, panel)
    LayoutHelpers.Below(portEdit, gameNameEdit, 36)

    local portLabel = UIUtil.CreateText(panel, "<LOC _Port>", 14)
    LayoutHelpers.Above(portLabel, portEdit, 5)

    local autoPort = UIUtil.CreateCheckboxStd(panel, '/dialogs/check-box_btn/radio')
    autoPort.Right:Set(portEdit.Right)
    autoPort.Bottom:Set(function() return portEdit.Top() - 5 end)
    autoPort.OnCheck = function(self, checked)
        if checked then
            portEdit:Disable()
            portEdit:SetText("Auto")
        else
            portEdit:Enable()
            portEdit:SetText(Prefs.GetFromCurrentProfile('LastPort') or defaultPort)
        end
    end
    autoPort:SetCheck(true)
    
    local autoPortLabel = UIUtil.CreateText(panel, "Auto Port", 14)
    autoPortLabel.Right:Set(autoPort.Left)
    autoPortLabel.Bottom:Set(autoPort.Bottom)

    -- ascii values of 0-9
    local portValidSet = {
        [48] = true,
        [49] = true,
        [50] = true,
        [51] = true,
        [52] = true,
        [53] = true,
        [54] = true,
        [55] = true,
        [56] = true,
        [57] = true,
        [46] = true,
    }

    portEdit.OnCharPressed = function(self, charcode)
        if portValidSet[charcode] then
            return false
        else
            return true
        end
    end

    local continueButton = UIUtil.CreateButtonStd(panel, '/widgets/small', "<LOC GAMECREATE_0001>Create", 16)
    LayoutHelpers.AtHorizontalCenterIn(continueButton, panel)
    LayoutHelpers.AtBottomIn(continueButton, panel, 50)

--  control behvaior
    exitButton.OnClick = function(self)
        parent:Destroy()
        import('/lua/modules/ui/lobby/gameselect.lua').CreateUI()
    end

    continueButton.OnClick = function(self)
        local gameName = gameNameEdit:GetText()
        if (not gameName) or (gameName == "") then
            if errorDialog then errorDialog:Destroy() end
            errorDialog = UIUtil.ShowInfoDialog(parent, "<LOC GAMECREATE_0000>Please choose a game name", "OK")
            return
        end

        local port = tonumber(portEdit:GetText()) or 0  -- default of port 0 will cause engine to choose
        Prefs.SetToCurrentProfile('LastPort', port)

        -- modify this if you want "TCP" or "None"
        -- no longer user selectable
        local protocol = "UDP"

        local function StartLobby(scenarioFileName)
            local lobby = import('/lua/modules/ui/lobby/lobby.lua')
            lobby.CreateLobby(protocol, port, playerName, nil, nil)
            Prefs.SetToCurrentProfile('last_game_name', gameName)
            lobby.HostGame(gameName, scenarioFileName, false)
        end

        local lastScenario = Prefs.GetFromCurrentProfile('LastScenario')
        if lastScenario then
            parent:Destroy()
            StartLobby(lastScenario)
        else
            parent:Destroy()
            local mapSelectDialog
            local function SelectBehavior(selectedScenario)
                Prefs.SetToCurrentProfile('LastScenario', selectedScenario.file)
                mapSelectDialog:Destroy()
                StartLobby(selectedScenario.file)
            end
            local function ExitBehavior()
                mapSelectDialog:Destroy()
                ExitLobby()
                CreateUI(playerName)
            end
            mapSelectDialog = import('/lua/modules/ui/dialogs/mapselect.lua').CreateDialog(SelectBehavior, ExitBehavior)
        end
    end
end

function CanQuickHost()
    local lastScenario = Prefs.GetFromCurrentProfile('LastScenario')
    local lastPort = Prefs.GetFromCurrentProfile('LastPort')
    local playerName = Prefs.GetFromCurrentProfile('NetName') or Prefs.GetFromCurrentProfile('Name')
    local lastGameName = Prefs.GetFromCurrentProfile('last_game_name')

    return lastScenario != nil and lastPort != nil and playerName != nil and lastGameName != nil
end

function QuickHost()
    local lastScenario = Prefs.GetFromCurrentProfile('LastScenario')
    local lastPort = Prefs.GetFromCurrentProfile('LastPort')
    local playerName = Prefs.GetFromCurrentProfile('NetName') or Prefs.GetFromCurrentProfile('Name')
    local lastGameName = Prefs.GetFromCurrentProfile('last_game_name')

    local lobby = import('/lua/modules/ui/lobby/lobby.lua')
    lobby.CreateLobby("UDP", lastPort, playerName, nil, nil)
    lobby.HostGame(lastGameName, lastScenario, false)
end