--*****************************************************************************
--* File: lua/modules/ui/lobby/gameselect.lua
--* Author: Chris Blackwell
--* Summary: Game selection UI
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Text = import('/lua/modules/maui/text.lua').Text
local ItemList = import('/lua/modules/maui/itemlist.lua').ItemList
local Edit = import('/lua/modules/maui/edit.lua').Edit
local Button = import('/lua/modules/maui/button.lua').Button
local Scrollbar = import('/lua/modules/maui/scrollbar.lua').Scrollbar
local MenuCommon = import('/lua/modules/ui/menus/menucommon.lua')
local MultiLineText = import('/lua/modules/maui/multilinetext.lua').MultiLineText
local Prefs = import('/lua/modules/ui/prefs.lua')
local Tooltip = import('/lua/modules/ui/game/tooltip.lua')

local errorDialog = false

function GetHostString(hostInfo)
    return hostInfo.GameName .. " (" .. tostring(hostInfo.PlayerCount) .. " " .. LOC("<LOC GAMSEL_0000>player(s)") .. ")"
end

function CreateUI()
    local discovery = import('/lua/modules/ui/lobby/lobbyComm.lua').CreateDiscoveryService()

-- control layout
    local parent = UIUtil.CreateScreenGroup(GetFrame(0), "Game Select ScreenGroup")

    -- don't parent background to screen group so it doesn't get destroyed until we leave the menus
    local background = MenuCommon.SetupBackground(GetFrame(0))

    local panel = Bitmap(parent, UIUtil.SkinnableFile('/lobby/gameselect02/panel_bmp.dds'))
    LayoutHelpers.AtCenterIn(panel, parent)

    local OFFSET = 20    
    
    local exitButton = UIUtil.CreateButtonStd(panel, '/widgets/small', "<LOC _Back>", 14)
    LayoutHelpers.AtLeftTopIn(exitButton, panel, 330, 480)
    
    local exitButton = UIUtil.CreateButtonStd(panel, '/widgets/small', "<LOC _Back>", 14)
    LayoutHelpers.AtLeftTopIn(exitButton, panel, 200, 480)
    
    local joinButton = UIUtil.CreateButtonStd(panel, '/widgets/small', "<LOC _Join>", 14)
    LayoutHelpers.RightOf(joinButton, exitButton, OFFSET)
    
    local observeButton = UIUtil.CreateButtonStd(panel, '/widgets/small', "<LOC _Observe>", 14)
    LayoutHelpers.RightOf(observeButton, joinButton, OFFSET)
    
    local createButton = UIUtil.CreateButtonStd(panel, '/widgets/small', "<LOC _Host>", 14)
    LayoutHelpers.RightOf(createButton, observeButton, OFFSET)

    #local coopCampaignButton = UIUtil.CreateButtonStd(panel, '/widgets/medium', "<LOC GAMESEL_0009>Co-op Campaign", 16)
    #LayoutHelpers.Below(coopCampaignButton, gpgnetButton)
    #coopCampaignButton:Disable()

    local nameEdit = Edit(panel)
    LayoutHelpers.AtLeftIn(nameEdit, panel, 56)
    LayoutHelpers.AtTopIn(nameEdit, panel, 102)
    nameEdit.Width:Set(365)
    nameEdit.Height:Set(16)
    nameEdit:SetFont(UIUtil.bodyFont, 12)
    nameEdit:SetForegroundColor(UIUtil.fontColor)
    nameEdit:SetText(Prefs.GetFromCurrentProfile('NetName') or Prefs.GetFromCurrentProfile('Name'))
    nameEdit.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "mpselect_name", nil, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
        Edit.HandleEvent(self, event)
    end
    
    local title = UIUtil.CreateText(panel, "<LOC GAMESEL_0000>Select LAN Game", 24)
    LayoutHelpers.AtLeftIn(title, panel, 35)
    LayoutHelpers.AtTopIn(title, panel, 27)

    local nameLabel = UIUtil.CreateText(panel, "<LOC NICKNAME>Nickname", 14)
    LayoutHelpers.Above(nameLabel, nameEdit, 3)

    local serverInfo = MultiLineText(panel, UIUtil.bodyFont, 12, UIUtil.fontColor)
    serverInfo.Width:Set(262)
    serverInfo.Height:Set(215)
    LayoutHelpers.AtLeftIn(serverInfo, panel, 460)
    LayoutHelpers.AtTopIn(serverInfo, panel, 150)
    serverInfo.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "mpselect_serverinfo", nil, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
        MultiLineText.HandleEvent(self, event)
    end

    local serverInfoLabel = UIUtil.CreateText(panel, "<LOC GAMESEL_0001>Server Information", 14)
    LayoutHelpers.Above(serverInfoLabel, serverInfo, 5)
    serverInfoLabel.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "mpselect_serverlist", nil, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
        Text.HandleEvent(self, event)
    end

    local serverList = ItemList(panel)
    serverList.Configs = {}
    serverList:SetFont(UIUtil.bodyFont, 16)
    serverList:SetColors(UIUtil.fontColor, "00000000", UIUtil.fontColor,  UIUtil.highlightColor)
    LayoutHelpers.AtLeftIn(serverList, panel, 55)
    LayoutHelpers.AtTopIn(serverList, panel, 155)
    serverList.Width:Set(357)
    serverList.Height:Set(275)

    UIUtil.CreateVertScrollbarFor(serverList)

    local serverListLabel = UIUtil.CreateText(panel, "<LOC GAMESEL_0002>Server List", 14)
    LayoutHelpers.Above(serverListLabel, serverList, 5)

--  control behvaior
    exitButton.OnClick = function(self)
        parent:Destroy()
        discovery:Destroy()
        discovery = false
        import('/lua/modules/ui/menus/multiplayer.lua').CreateUI()
    end
    exitButton.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "mpselect_exit", nil, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
        Button.HandleEvent(self, event)
    end

    createButton.OnClick = function(self)
        local name = nameEdit:GetText()
        if name == "" then
            if errorDialog then errorDialog:Destroy() end
            errorDialog = UIUtil.ShowInfoDialog(parent, "<LOC GAMESEL_0003>Please fill in your nickname", "OK")
        else
            Prefs.SetToCurrentProfile('NetName', name)
            parent:Destroy()
            discovery:Destroy()
            discovery = false
            import('/lua/modules/ui/lobby/gamecreate.lua').CreateUI(name)
        end
    end
    createButton.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "mpselect_create", nil, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
        Button.HandleEvent(self, event)
    end

    local function JoinOrObserve(asObserver)
        local curServerIndex = serverList:GetSelection()
        if curServerIndex >= 0 then
            local name = nameEdit:GetText()
            if name == "" then
                if errorDialog then errorDialog:Destroy() end
                errorDialog = UIUtil.ShowInfoDialog(parent, "<LOC GAMESEL_0004>Please fill in your nickname", "<LOC _OK>")
            else
                Prefs.SetToCurrentProfile('NetName', name)
                local hostInfo = serverList.Configs[curServerIndex]
                local lobby = import('/lua/modules/ui/lobby/lobby.lua')
                lobby.CreateLobby(hostInfo.Protocol, 0, name, nil, nil)
                parent:Destroy()
                discovery:Destroy()
                discovery = false
                LOG('Joining ', repr(hostInfo))
                lobby.JoinGame(hostInfo.Address, asObserver)
            end
        else
            if errorDialog then errorDialog:Destroy() end
            errorDialog = UIUtil.ShowInfoDialog(parent, "<LOC GAMESEL_0006>Please select a server", "<LOC _OK>")
        end
    end


    joinButton.OnClick = function(self) JoinOrObserve(false) end
    joinButton.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "mpselect_join", nil, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
        Button.HandleEvent(self, event)
    end
    observeButton.OnClick = function(self) JoinOrObserve(true) end
    observeButton.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "mpselect_observe", nil, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
        Button.HandleEvent(self, event)
    end

    function Update(hostInfo)
        local serverInfoString
        if hostInfo then

            local obsStr
            if hostInfo.Options.AllowObservers then
                observeButton:Enable()
                obsStr = LOC("<LOC _Yes>")
            else
                observeButton:Disable()
                obsStr = LOC("<LOC _No>")
            end

            serverInfoString =
                LOC("<LOC _Name>") .. " : " .. hostInfo.GameName .. "\n" ..
                LOC("<LOC _Players>") .. " : " .. tostring(hostInfo.PlayerCount) .. "\n" ..
                LOC("<LOC _Address>") .. " : " .. hostInfo.Address .. "\n" ..
                LOC("<LOC _Protocol>") .. " : " .. hostInfo.Protocol .. "\n" ..
                LOC("<LOC GAMESEL_0010>Observers Allowed") .. " : " .. obsStr
        else
            serverInfoString = "<LOC GAMESEL_0007>No Information"
        end
        serverInfo:SetText(serverInfoString)
    end

    serverList.OnClick = function(self, row)
        serverList:SetSelection(row)
        local hostInfo = serverList.Configs[row]
        Update(hostInfo)
    end

    serverList.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "mpselect_serverlist", nil, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
        ItemList.HandleEvent(self, event)
    end

-- discovery behaviors
    discovery.RemoveGame = function(self,index)
        serverList:DeleteItem(index)
    end

    discovery.GameFound = function(self,index,gameConfig)
        serverList:AddItem(GetHostString(gameConfig))
        serverList.Configs[index] = gameConfig
    end

    discovery.GameUpdated = function(self,index,gameConfig)
        serverList:ModifyItem(index,GetHostString(gameConfig))
        serverList.Configs[index] = gameConfig
        if serverList:GetSelection() == index then
            Update(serverList.Configs[index])
        end
    end
end
