--*****************************************************************************
--* File: lua/modules/ui/menus/eula.lua
--* Author: Ted Snook
--* Summary: Front end display of the EULA
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Button = import('/lua/modules/maui/button.lua').Button
local Group = import('/lua/modules/maui/group.lua').Group
local Text = import('/lua/modules/maui/text.lua').Text
local ItemList = import('/lua/modules/maui/itemlist.lua').ItemList
local MenuCommon = import('/lua/modules/ui/menus/menucommon.lua')

function CreateEULA()
    local parent = Group(GetFrame(0))
    LayoutHelpers.FillParent(parent, GetFrame(0))
    parent.Depth:Set(GetFrame(0):GetTopmostDepth() + 10)
    
    local worldCover = UIUtil.CreateWorldCover(parent)
    
    local exitButton = MenuCommon.CreateExitMenuButton(parent, worldCover, "<LOC _Back>")
    exitButton.OnClick = function(self)
        parent:Destroy()
    end
    
    local bg = Bitmap(parent, UIUtil.UIFile('/game/generic_brd/generic_brd_m.dds'))
    bg.Height:Set(500)
    bg.Width:Set(600)
    LayoutHelpers.AtCenterIn(bg, parent)
    
    local bgTopLeft = Bitmap(bg, UIUtil.UIFile('/game/generic_brd/generic_brd_ul.dds'))
    bgTopLeft.Bottom:Set(bg.Top)
    bgTopLeft.Right:Set(bg.Left)
    
    local bgTopRight = Bitmap(bg, UIUtil.UIFile('/game/generic_brd/generic_brd_ur.dds'))
    bgTopRight.Bottom:Set(bg.Top)
    bgTopRight.Left:Set(bg.Right)
    
    local bgBottomLeft = Bitmap(bg, UIUtil.UIFile('/game/generic_brd/generic_brd_ll.dds'))
    bgBottomLeft.Top:Set(bg.Bottom)
    bgBottomLeft.Right:Set(bg.Left)
    
    local bgBottomRight = Bitmap(bg, UIUtil.UIFile('/game/generic_brd/generic_brd_lr.dds'))
    bgBottomRight.Top:Set(bg.Bottom)
    bgBottomRight.Left:Set(bg.Right)
    
    local bgTopMid = Bitmap(bg, UIUtil.UIFile('/game/generic_brd/generic_brd_horz_um.dds'))
    bgTopMid.Bottom:Set(bg.Top)
    bgTopMid.Left:Set(bgTopLeft.Right)
    bgTopMid.Right:Set(bgTopRight.Left)

    local bgBottomMid = Bitmap(bg, UIUtil.UIFile('/game/generic_brd/generic_brd_horz_lm.dds'))
    bgBottomMid.Top:Set(bg.Bottom)
    bgBottomMid.Left:Set(bgBottomLeft.Right)
    bgBottomMid.Right:Set(bgBottomRight.Left)
    
    local bgLeftMid = Bitmap(bg, UIUtil.UIFile('/game/generic_brd/generic_brd_vert_l.dds'))
    bgLeftMid.Bottom:Set(bgBottomLeft.Top)
    bgLeftMid.Top:Set(bgTopLeft.Bottom)
    bgLeftMid.Right:Set(bgTopLeft.Right)

    local bgRightMid = Bitmap(bg, UIUtil.UIFile('/game/generic_brd/generic_brd_vert_r.dds'))
    bgRightMid.Bottom:Set(bgBottomRight.Top)
    bgRightMid.Top:Set(bgTopRight.Bottom)
    bgRightMid.Left:Set(bgTopRight.Left)
       
    local title = UIUtil.CreateText(bg, "<LOC EULA_TITLE>End User License Agreement", 20, UIUtil.bodyFont)
    LayoutHelpers.AtHorizontalCenterIn(title, bg)
    title.Bottom:Set(function() return bg.Top() - 5 end)
    
    local tempText = UIUtil.CreateText(bg, "", 12, UIUtil.bodyFont)
    tempText.Left:Set(0)
    tempText.Top:Set(0)
    
    local eulaBody = ItemList(bg)
    LayoutHelpers.FillParent(eulaBody, bg)
    eulaBody:SetColors(UIUtil.consoleFGColor(), UIUtil.consoleTextBGColor(), UIUtil.consoleFGColor(), UIUtil.consoleTextBGColor()) -- we don't really want selection here so don't differentiate colors
    eulaBody:SetFont(UIUtil.bodyFont, 12)
    UIUtil.CreateVertScrollbarFor(eulaBody)
    
    local eulaText = import('/lua/modules/ui/help/eula.lua').EULA
    local textBoxWidth = eulaBody.Width()

    local tempTable = import('/lua/modules/maui/text.lua').WrapText(LOC(eulaText), textBoxWidth,
    function(text)
        return tempText:GetStringAdvance(text)
    end)
    
    for i, v in tempTable do
        eulaBody:AddItem(v)
    end
end