--*****************************************************************************
--* File: lua/modules/ui/menus/main.lua
--* Author: Chris Blackwell
--* Summary: create and control pre-game splash screens
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local MenuCommon = import('/lua/modules/ui/menus/menucommon.lua')
local MultiLineText = import('/lua/modules/maui/multilinetext.lua').MultiLineText
local Button = import('/lua/modules/maui/button.lua').Button
local Group = import('/lua/modules/maui/group.lua').Group
local Prefs = import('/lua/modules/ui/prefs.lua')
local Tooltip = import('/lua/modules/ui/game/tooltip.lua')
local MapUtil = import('/lua/modules/ui/maputil.lua')
local TooltipInfo = import('/lua/modules/ui/help/tooltips.lua')

local mapErrorDialog = false

local TOOLTIP_DELAY = 1

function CreateQuickBG(parent, topControl, bottomControl)
    local topbg = Bitmap(parent, UIUtil.UIFile('/menus/main02/quick-panel_t.dds'))
    LayoutHelpers.AtHorizontalCenterIn(topbg, topControl)
    LayoutHelpers.AtTopIn(topbg, topControl, -5)
    
    topbg.Bot = Bitmap(topbg, UIUtil.UIFile('/menus/main02/quick-panel-b.dds'))
    LayoutHelpers.AtHorizontalCenterIn(topbg.Bot, topControl)
    LayoutHelpers.AtBottomIn(topbg.Bot, bottomControl)
    
    topbg.Mid = Bitmap(topbg, UIUtil.UIFile('/menus/main02/quick-panel-m.dds'))
    LayoutHelpers.AtHorizontalCenterIn(topbg.Mid, topControl)
    topbg.Mid.Top:Set(topbg.Bottom)
    topbg.Mid.Bottom:Set(topbg.Bot.Top)
    
    return topbg
end

function CreateUI()
    MenuCommon.StartMenuAmbientSounds()

-- control layout
    local parent = UIUtil.CreateScreenGroup(GetFrame(0), "Main Menu ScreenGroup")

    -- don't parent background to screen group so it doesn't get destroyed until we leave the menus
    local background = MenuCommon.SetupBackground(GetFrame(0), '/menus02/borde02-l_bmp.dds')
    
    local legalText = MultiLineText(background, UIUtil.bodyFont, 9, UIUtil.fontColor)
    LayoutHelpers.AtBottomIn(legalText, parent)
    LayoutHelpers.AtLeftIn(legalText, parent, 10)
    legalText.Width:Set(570)
    legalText.Height:Set(48)
    legalText:SetDropShadow(true)
    legalText:SetText(LOC(import('/lua/modules/ui/help/eula.lua').LEGAL_TEXT))
    
    local logo = Bitmap(parent, UIUtil.UIFile('/menus/main02/sc-logo-lg_bmp.dds'))
    logo.Left:Set(background.Left)
    logo.Top:Set(background.Top)
    
    if UIUtil.BETA_FLAG then
        local logostamp = Bitmap(parent, UIUtil.UIFile('/menus/main02/multiplayer_beta.dds'))
        LayoutHelpers.AtHorizontalCenterIn(logostamp, logo)
        logostamp.Top:Set(function() return logo.Top() + 30 end)
    end

    local titlebg = Bitmap(parent, UIUtil.UIFile('/menus/main02/panel-top_bmp.dds'))
    LayoutHelpers.AtRightTopIn(titlebg, parent, 0, 3)
    
    local titletext = UIUtil.CreateText(titlebg, "<LOC _Main_Menu>Main Menu", 26)
    LayoutHelpers.AtCenterIn(titletext, titlebg, 5)
    
    -- profile button is always at the center, but will be relative to the top of the frame
    local profileButton = MenuCommon.CreateProfileButton(titlebg)
    LayoutHelpers.CenteredBelow(profileButton, titlebg, -20)
    

    -- this should be shown if there are no profiles
    if not GetPreference("profile.current") then
        profileButton:OnClick()
    end
    
    local campaignGroup = Group(parent)
    local multiplayerGroup = Group(parent)
    local skirmishGroup = Group(parent)
    local replayGroup = Group(parent)
    
    local campaignButton = UIUtil.CreateButtonStd(campaignGroup, '/menus/main02/large', "<LOC _Campaign>", 22, 0, 0, "UI_Menu_MouseDown", "UI_Opt_Affirm_Over")
    local multiPlayerButton = UIUtil.CreateButtonStd(multiplayerGroup, '/menus/main02/large', "<LOC _Multiplayer>", 22, 0, 0, "UI_Menu_MouseDown", "UI_Opt_Affirm_Over")
    local skirmishButton = UIUtil.CreateButtonStd(skirmishGroup, '/menus/main02/large', "<LOC _Skirmish>", 22, 0, 0, "UI_Menu_MouseDown", "UI_Opt_Affirm_Over")
    local replayButton = UIUtil.CreateButtonStd(replayGroup, '/menus/main02/large', "<LOC _Replay>", 22, 0, 0, "UI_Menu_MouseDown", "UI_Opt_Affirm_Over")
    local tutorialButton = UIUtil.CreateButtonStd(parent, '/menus/main02/large', "<LOC _Tutorial>Tutorial", 22, 0, 0, "UI_Menu_MouseDown", "UI_Opt_Affirm_Over")
    local optionsButton = UIUtil.CreateButtonStd(parent, '/menus/main02/large', "<LOC _Options>", 22, 0, 0, "UI_Menu_MouseDown", "UI_Opt_Affirm_Over")
    local modButton = UIUtil.CreateButtonStd(parent, '/menus/main02/large', "<LOC _Mod_Manager>", 22, 0, 0, "UI_Menu_MouseDown", "UI_Opt_Affirm_Over")
    local exitButton = UIUtil.CreateButtonStd(parent, '/menus/main02/large', "<LOC _Exit>", 22, 0, 0, "UI_Menu_MouseDown", "UI_Opt_Affirm_Over")
    
    campaignGroup.Top:Set(campaignButton.Top)
    campaignGroup.Left:Set(campaignButton.Left)
    campaignGroup.Right:Set(campaignButton.Right)
    campaignGroup.Bottom:Set(campaignButton.Bottom)
    
    multiplayerGroup.Top:Set(multiPlayerButton.Top)
    multiplayerGroup.Left:Set(multiPlayerButton.Left)
    multiplayerGroup.Right:Set(multiPlayerButton.Right)
    
    skirmishGroup.Top:Set(skirmishButton.Top)
    skirmishGroup.Left:Set(skirmishButton.Left)
    skirmishGroup.Right:Set(skirmishButton.Right)
    skirmishGroup.Bottom:Set(skirmishButton.Bottom)
    
    replayGroup.Top:Set(replayButton.Top)
    replayGroup.Left:Set(replayButton.Left)
    replayGroup.Right:Set(replayButton.Right)
    replayGroup.Bottom:Set(replayButton.Bottom)
    
    campaignButton:UseAlphaHitTest(false)
    multiPlayerButton:UseAlphaHitTest(false)
    skirmishButton:UseAlphaHitTest(false)
    replayButton:UseAlphaHitTest(false)
    tutorialButton:UseAlphaHitTest(false)
    optionsButton:UseAlphaHitTest(false)
    modButton:UseAlphaHitTest(false)
    
    --TODO: Add logic to enable continue button
--[[
    if import('/lua/modules/ui/campaign/campaignmanager.lua').CanQuickContinue() then
        local quickCampaignContinue = UIUtil.CreateButtonStd(campaignGroup, '/menus/main02/thin', "<LOC MAINMENU_0000>Continue Campaign", 12, 0, 0, "UI_Menu_MouseDown", "UI_Menu_Rollover")
        LayoutHelpers.CenteredBelow(quickCampaignContinue, campaignButton, 0)
        quickCampaignContinue.Depth:Set(function() return campaignGroup.Depth() + 5 end)
        local quickcampBG = CreateQuickBG(campaignGroup, quickCampaignContinue, quickCampaignContinue)
        quickCampaignContinue.OnClick = function(self, modifiers)
            parent:Destroy()
            MenuCommon.MenuCleanup()
            import('/lua/modules/ui/campaign/campaignmanager.lua').QuickContinue()
        end
        Tooltip.AddButtonTooltip(quickCampaignContinue, "mainmenu_quickcampaign", TOOLTIP_DELAY)
        campaignGroup.Bottom:Set(quickCampaignContinue.Bottom)
    end
--]]
    
    local quickLANHost = UIUtil.CreateButtonStd(multiplayerGroup, '/menus/main02/thin', "<LOC MAINMENU_0001>LAN Host", 12, 0, 0, "UI_Menu_MouseDown", "UI_Menu_Rollover")
    local quickIPConnect = UIUtil.CreateButtonStd(multiplayerGroup, '/menus/main02/thin', "<LOC MAINMENU_0002>IP Connect", 12, 0, 0, "UI_Menu_MouseDown", "UI_Menu_Rollover")
    local quickGPGNet = UIUtil.CreateButtonStd(multiplayerGroup, '/menus/main02/thin', "<LOC MAINMENU_0003>GPGNet", 12, 0, 0, "UI_Menu_MouseDown", "UI_Menu_Rollover")
    local quickmpBG = CreateQuickBG(multiplayerGroup, quickLANHost, quickGPGNet)
    quickLANHost.Depth:Set(function() return multiplayerGroup.Depth() + 5 end)
    quickIPConnect.Depth:Set(function() return multiplayerGroup.Depth() + 5 end)
    quickGPGNet.Depth:Set(function() return multiplayerGroup.Depth() + 5 end)
    LayoutHelpers.CenteredBelow(quickLANHost, multiPlayerButton, 0)
    LayoutHelpers.CenteredBelow(quickIPConnect, quickLANHost, 0)
    LayoutHelpers.CenteredBelow(quickGPGNet, quickIPConnect, 0)
    
    if import('/lua/modules/ui/lobby/gamecreate.lua').CanQuickHost() then
        quickLANHost:Enable()
    else
        quickLANHost:Disable()
    end
    quickLANHost.OnClick = function(self, modifiers)
        parent:Destroy()
        MenuCommon.MenuCleanup()
        import('/lua/modules/ui/lobby/gamecreate.lua').QuickHost()
    end
    Tooltip.AddButtonTooltip(quickLANHost, "mainmenu_quicklanhost", TOOLTIP_DELAY)

    if import('/lua/modules/ui/lobby/directconnect.lua').CanQuickDirectConnect() then
        quickIPConnect:Enable()
    else
        quickIPConnect:Disable()
    end
    
    quickIPConnect.OnClick = function(self, modifiers)
        parent:Destroy()
        MenuCommon.MenuCleanup()
        import('/lua/modules/ui/lobby/directconnect.lua').QuickDirectConnect()
    end
    Tooltip.AddButtonTooltip(quickIPConnect, "mainmenu_quickipconnect", TOOLTIP_DELAY)
    
    quickGPGNet.OnClick = function(self, modifiers)
        LaunchGPGNet()
        MenuCommon.MenuCleanup()
        parent:Destroy()
        ExitApplication()
    end
    Tooltip.AddButtonTooltip(quickGPGNet, "mainmenu_quickgpgnet", TOOLTIP_DELAY)
    multiplayerGroup.Bottom:Set(quickGPGNet.Bottom)

    local filename = Prefs.GetFromCurrentProfile("LastSaveFile")
    if filename then
        local quickSkirmishLoad = UIUtil.CreateButtonStd(skirmishGroup, '/menus/main02/thin', "<LOC MAINMENU_0004>Quick Load", 12, 0, 0, "UI_Opt_Menu_Top_Click", "UI_Opt_Menu_Top_Over")
        LayoutHelpers.CenteredBelow(quickSkirmishLoad, skirmishButton, 0)
        local quickskirmishBG = CreateQuickBG(skirmishGroup, quickSkirmishLoad, quickSkirmishLoad)
        quickSkirmishLoad.Depth:Set(function() return skirmishGroup.Depth() + 5 end)
        quickSkirmishLoad.OnClick = function(self, modifiers) 
            if not LoadSavedGame(GetSpecialFilePath(filename, "SaveGame")) then
                UIUtil.ShowInfoDialog(parent, LOCF("<LOC MAINMENU_0006>Couldn't open savegame %s", filename), "<LOC _Ok>")
            end
        end
        Tooltip.AddButtonTooltip(quickSkirmishLoad, "mainmenu_quickskirmishload", TOOLTIP_DELAY)
        skirmishGroup.Bottom:Set(quickSkirmishLoad.Bottom)
    end
        
    local currentReplays = GetSpecialFiles("Replay")
    local lastGameFound = false
    for index, file in currentReplays.files do
        if file == "LastGame" then
            lastGameFound = true
        end
    end
    if lastGameFound then
        local quickReplay = UIUtil.CreateButtonStd(replayGroup, '/menus/main02/thin', "<LOC MAINMENU_0005>Quick Replay", 12, 0, 0, "UI_Menu_MouseDown", "UI_Menu_Rollover")
        LayoutHelpers.CenteredBelow(quickReplay, replayButton, 0)
        quickReplay.Depth:Set(function() return replayGroup.Depth() + 5 end)
        local quickreplayBG = CreateQuickBG(replayGroup, quickReplay, quickReplay)
        quickReplay.OnClick = function(self, modifiers)
            if not LaunchReplaySession(GetSpecialFilePath("LastGame", "Replay")) then
                UIUtil.ShowInfoDialog(parent, "<LOC MAINMENU_0008>Couldn't play replay, none found", "<LOC _Ok>")
            end
        end
        Tooltip.AddButtonTooltip(quickReplay, "mainmenu_quickreplay", TOOLTIP_DELAY)
        replayGroup.Bottom:Set(quickReplay.Bottom)
    end

    LayoutHelpers.CenteredBelow(campaignButton, profileButton, 0)
    LayoutHelpers.CenteredBelow(multiPlayerButton, campaignGroup)
    LayoutHelpers.CenteredBelow(skirmishButton, multiplayerGroup)
    LayoutHelpers.CenteredBelow(replayButton, skirmishGroup)
    LayoutHelpers.CenteredBelow(tutorialButton, replayGroup)
    LayoutHelpers.CenteredBelow(optionsButton, tutorialButton)
    LayoutHelpers.CenteredBelow(modButton, optionsButton)
    LayoutHelpers.CenteredBelow(exitButton, modButton)

    local bottomcap = Bitmap(parent, UIUtil.UIFile('/menus/main02/panel-bottom_bmp.dds'))
    LayoutHelpers.CenteredBelow(bottomcap, exitButton, -20)
    
    local versionText = UIUtil.CreateText(background, "<LOC MAINMENU_????>Version : " .. GetVersion(), 15)
    LayoutHelpers.AtLeftTopIn(versionText, background, 5, 5)
    
--  control behvaior
    exitButton.OnClick = function(self, modifiers)
        UIUtil.QuickDialog(GetFrame(0), "Are you sure you'd like to exit?", 
            "Yes", function() 
                MenuCommon.MenuCleanup()
                parent:Destroy()
                ExitApplication()
                end, "No")
    end
    Tooltip.AddButtonTooltip(exitButton, "mainmenu_exit", TOOLTIP_DELAY)

    modButton.OnClick = function(self, modifiers)
        import('/lua/modules/ui/dialogs/modmanager.lua').CreateDialog(parent, false)
    end
    
    Tooltip.AddButtonTooltip(modButton, "mainmenu_mod", TOOLTIP_DELAY)
    
    campaignButton.OnClick = function(self, modifiers)
        parent:Destroy()
        MenuCommon.MenuCleanup()
        import('/lua/modules/ui/campaign/selectcampaign.lua').CreateUI()
    end
    Tooltip.AddButtonTooltip(campaignButton, "mainmenu_campaign", TOOLTIP_DELAY)

    multiPlayerButton.OnClick = function(self, modifiers)
        parent:Destroy()
        MenuCommon.MenuCleanup()
        import('/lua/modules/ui/menus/multiplayer.lua').CreateUI()
    end
    Tooltip.AddButtonTooltip(multiPlayerButton, "mainmenu_mp", TOOLTIP_DELAY)

    skirmishButton.OnClick = function(self, modifiers)
        parent:Destroy()

        local function StartLobby(scenarioFileName)
            local playerName = Prefs.GetCurrentProfile().Name or "Unknown"
            local lobby = import('/lua/modules/ui/lobby/lobby.lua')
            lobby.CreateLobby('None', 0, playerName, nil, nil)
            lobby.HostGame(playerName .. "'s Skirmish", scenarioFileName, true)
        end

        local lastScenario = Prefs.GetFromCurrentProfile('LastScenario')
        if lastScenario then
            StartLobby(lastScenario)
        else
            local mapSelectDialog
            local function SelectBehavior(selectedScenario)
                Prefs.SetToCurrentProfile('LastScenario', selectedScenario.file)
                mapSelectDialog:Destroy()
                StartLobby(selectedScenario.file)
            end
            local function ExitBehavior()
                mapSelectDialog:Destroy()
                CreateUI()
            end
            mapSelectDialog = import('/lua/modules/ui/dialogs/mapselect.lua').CreateDialog(SelectBehavior, ExitBehavior)
        end
    end
    Tooltip.AddButtonTooltip(skirmishButton, "mainmenu_skirmish", TOOLTIP_DELAY)

    replayButton.OnClick = function(self, modifiers)
        parent:Destroy()
        import('/lua/modules/ui/dialogs/replay.lua').CreateDialog(nil, function()
            CreateUI()
        end)
    end
    Tooltip.AddButtonTooltip(replayButton, "mainmenu_replay", TOOLTIP_DELAY)
    
    tutorialButton.OnClick = function(self, modifiers)
        parent:Destroy()
        MenuCommon.MenuCleanup()
        LoadMap(import('/lua/modules/ui/maputil.lua').LoadScenario('/maps/SCCA_Tutorial/SCCA_Tutorial_scenario.lua').file, 0, 2)
    end
    Tooltip.AddButtonTooltip(tutorialButton, "mainmenu_tutorial", TOOLTIP_DELAY)
    
    optionsButton.OnClick = function(self, modifiers)
        parent:Destroy()
        import('/lua/modules/ui/dialogs/options.lua').CreateDialog(nil, function() CreateUI() end)
    end
    Tooltip.AddButtonTooltip(optionsButton, "mainmenu_options", TOOLTIP_DELAY)
    
    
    if UIUtil.BETA_FLAG then
        campaignButton:Disable()
        tutorialButton:Disable()
    end
end