--*****************************************************************************
--* File: lua/modules/ui/menus/multiplayer.lua
--* Author: Chris Blackwell / Ted Snook
--* Summary: create and control pre-game splash screens
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local MenuCommon = import('/lua/modules/ui/menus/menucommon.lua')
local MultiLineText = import('/lua/modules/maui/multilinetext.lua').MultiLineText
local Button = import('/lua/modules/maui/button.lua').Button
local Group = import('/lua/modules/maui/group.lua').Group
local Prefs = import('/lua/modules/ui/prefs.lua')
local Tooltip = import('/lua/modules/ui/game/tooltip.lua')
local MapUtil = import('/lua/modules/ui/maputil.lua')
local TooltipInfo = import('/lua/modules/ui/help/tooltips.lua')

function CreateUI()
    local parent = UIUtil.CreateScreenGroup(GetFrame(0), "Main Menu ScreenGroup")
    
    local panel = Bitmap(parent, UIUtil.UIFile('/lobby/multiplayer-select/panel02_bmp.dds'))
    LayoutHelpers.AtCenterIn(panel, parent)
    panel.Depth:Set(function() return parent.Depth() + 1000 end)
    
    local background = MenuCommon.SetupBackground(GetFrame(0))
    
    local title = UIUtil.CreateText(panel, LOC("<LOC _Multiplayer>"), 24)
    LayoutHelpers.AtHorizontalCenterIn(title, panel)
    LayoutHelpers.AtTopIn(title, panel, 40)
    
    local gpgnetButton = UIUtil.CreateButtonStd(panel, '/menus/main02/large', "<LOC _GPG_Net>", 18)
    LayoutHelpers.AtHorizontalCenterIn(gpgnetButton, panel)
    LayoutHelpers.AtTopIn(gpgnetButton, panel, 115)
    
    local lanButton = UIUtil.CreateButtonStd(panel, '/menus/main02/large', "<LOC GAMESEL_0011>LAN", 18)
    LayoutHelpers.Below(lanButton, gpgnetButton, 20)
    
    local directConnectButton = UIUtil.CreateButtonStd(panel, '/menus/main02/large', "<LOC GAMESEL_0008>Direct Connect", 18)
    LayoutHelpers.Below(directConnectButton, lanButton, 20)
    
    local cancelButton = UIUtil.CreateButtonStd(panel, '/widgets/small02', "<LOC _Back>", 14)
    LayoutHelpers.CenteredBelow(cancelButton, directConnectButton, 60)
    
    gpgnetButton:UseAlphaHitTest(false)
    lanButton:UseAlphaHitTest(false)
    directConnectButton:UseAlphaHitTest(false)
    
    gpgnetButton.OnClick = function(self)
		    LaunchGPGNet()
        MenuCommon.MenuCleanup()
        parent:Destroy()
        ExitApplication()
    end
    gpgnetButton.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "mpselect_gpgnet", nil, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
        Button.HandleEvent(self, event)
    end
    
    lanButton.OnClick = function(self)
        parent:Destroy()
        import('/lua/modules/ui/lobby/gameselect.lua').CreateUI()
    end
    lanButton.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "mpselect_lan", nil, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
        Button.HandleEvent(self, event)
    end
    
    directConnectButton.OnClick = function(self)
        local name = Prefs.GetFromCurrentProfile('Name')
        parent:Destroy()
        import('/lua/modules/ui/lobby/directconnect.lua').CreateUI(name)
    end
    directConnectButton.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "mpselect_connect", nil, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
        Button.HandleEvent(self, event)
    end
    
    cancelButton.OnClick = function(self)
        panel:Destroy()
        import('/lua/modules/ui/menus/main.lua').CreateUI()
    end
end