--*****************************************************************************
--* File: lua/modules/ui/splash/splash.lua
--* Author: Chris Blackwell
--* Summary: create and control pre-game splash screens
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Movie = import('/lua/modules/maui/movie.lua').Movie

function CreateUI()
    if GetPreference("movie.nologo") then
        EngineStartFrontEndUI()
        return
    end

    local parent = UIUtil.CreateScreenGroup(GetFrame(0), "Splash ScreenGroup")
    parent:AcquireKeyboardFocus()

    local movie = Movie(parent)
    LayoutHelpers.FillParentPreserveAspectRatio(movie, parent)
    movie:DisableHitTest()    -- get clicks to parent group

    local currentMovie = 1
    movie:Set("/movies/thqlogo.sfd")
    movie:Play()

    local function LeaveSplashScreen()
        parent:AbandonKeyboardFocus()
        movie:Stop()
        parent:Destroy()
        EngineStartFrontEndUI()
    end

    parent.HandleEvent = function(self, event)
        -- cancel movie playback on mouse click or key hit
        if event.Type == "ButtonPress" or event.Type == "KeyDown" then
            LeaveSplashScreen()
            
            return true
        end
    end

    movie.OnFinished = function(self)
        if currentMovie == 1 then
            currentMovie = 2
            movie:Set("/movies/gpglogo.sfd")
            movie:Play()
        elseif currentMovie == 2 then
            currentMovie = 3
            movie:Set("/movies/partnerlogo.sfd")
            movie:Play()
        elseif currentMovie == 3 then
            LeaveSplashScreen()
        end
    end
end
