#****************************************************************************
#**
#**  File     :  /cdimage/lua/modules/unpacker.lua
#**  Author(s):  John Comes
#**
#**  Summary  :  Utility functions for scripts.
#**
#**  Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************



#THIS CLASS IS USED FOR WEAPON UNPACKING IN CASES OF UNITS LIKE AEON UNITS.
WeaponUnpacker = Class(MultiEvent) {


    __init = function(self, unit, spec)
        MultiEvent.__init(self)
        self.OpenCount = 0
        self.UnpackerAnimManip = CreateAnimator(unit)
        self.UnpackAnimName = spec.UnpackingAnimName or unit:GetBlueprint().Display.AnimationOpen
        self.UnpackAnimRate = spec.UnpackingAnimRate
        self.RepackDelay = spec.RepackingDelay
        self.unit = unit
        ChangeState(self, self.ClosedIdleState)
    end,

    Destroy = function(self)
        self.UnpackerAnimManip:Destroy()
        ChangeState(self, self.DestroyedState)
    end,

    AddOpenRequest = function(self)
        self.OpenCount = self.OpenCount + 1
        #LOG('*DEBUG: ++openReq: ',self.OpenCount)
        self:OnStateEnter()
    end,

    RemoveOpenRequest = function(self)
        self.OpenCount = self.OpenCount - 1
        #LOG('*DEBUG: --openReq: ',self.OpenCount)
        self:OnStateEnter()
    end,

    #
    # The default for all states to ignore OpenRequested() and CloseRequested()
    #
    OpenRequested = function() end,
    CloseRequested = function() end,

    #
    # When entering any state, re-check OpenCount
    #
    OnStateEnter = function(self)
        #LOG('*DEBUG: WeaponUnpacker OnStateEnter. OpenCnt: ',self.OpenCount)
        if self.OpenCount > 0 then
            self:OpenRequested()
        else
            self:CloseRequested()
        end
    end,

    ClosedIdleState = State {
        Main = function(self)
            #LOG('*DEBUG: ClosedIdleState')
            if self.OpenCount > 0 then
                ChangeState(self, self.UnpackingState)
            end
        end,

        OpenRequested = function(self)
            ChangeState(self, self.UnpackingState)
        end,
    },

    OpenIdleState = State {
        OpenRequested = function(self)
            ChangeState(self, self.OpenActiveState)
        end,

        Main = function(self)
            #LOG('*DEBUG: OpenIdleState')
            #self:EventReset()
            WaitSeconds(self.RepackDelay)
            ChangeState(self, self.PackingState)
        end,
    },

    OpenActiveState = State {
        CloseRequested = function(self)
            ChangeState(self, self.OpenIdleState)
        end,
        Main = function(self)
            #LOG('*DEBUG: OpenActiveState')
        end,
    },

    PackingState = State {
        Main = function(self)
            #LOG('*DEBUG: PackingState')
            self.UnpackerAnimManip:SetRate(-self.UnpackAnimRate)

            # Play the "pack" sound
            self.unit:PlayUnitSound('Pack')

            self:EventReset()
            WaitFor(self.UnpackerAnimManip)
            self:EventSet()
            #self.UnpackerAnimManip:SetAnim(nil)
            ChangeState(self, self.ClosedIdleState)
        end,
    },

    UnpackingState = State {
        Main = function(self)
            #LOG('*DEBUG: UNPACKING STATE FOR ', repr(self))
            self.UnpackerAnimManip:PlayAnim(self.UnpackAnimName)
            #LOG('*DEBUG: PLAYED ANIMATION - ', repr(self.UnpackAnimName))
            self.UnpackerAnimManip:SetRate(self.UnpackAnimRate or 1)
            #LOG('*DEBUG: WAITING FOR UNPACKER ANIMATION')

            # Play the "unpack" sound
            self.unit:PlayUnitSound('Unpack')

            #LOG('*DEBUG: Waiting for unpack animation...')
            self:EventReset()
            WaitFor(self.UnpackerAnimManip)
            self:EventSet()
            #LOG('*DEBUG: ... unpack anim finished. Going to OpenActiveState')
            ChangeState(self, self.OpenActiveState)
        end,

        RemoveOpenRequest = function(self)
            self.OpenCount = self.OpenCount - 1
            #LOG('*DEBUG: --openReq: ',self.OpenCount)
            if self.OpenCount == 0 then
                ChangeState(self, self.PackingState)
            else
                self:OnStateEnter()
            end
        end,
    },

    DestroyedState = State {
    }
}
