

CheckBrainForDefeat = function(brain, categoryCheck)
    local defeated = false
    
    if brain:GetCurrentUnits(categoryCheck) == 0 then

        defeated = true

        # Ok you don't have any units required to stay alive, but
        # you're not defeated if you and an alive ally are both
        # requesting allied victory.
        if brain.RequestingAlliedVictory then
            for k,v in ArmyBrains do
                if v.RequestingAlliedVictory and IsAlly(brain:GetArmyIndex(),v:GetArmyIndex()) and v:GetCurrentUnits(categoryCheck) > 0 then
                    defeated = false
                    break
                end
            end
        end
    end

    return defeated
end

CheckVictory = function(scenarioInfo)
    local categoryCheck = nil

    if scenarioInfo.Options.Victory == 'demoralization' then
        # You're dead if you have no commanders
        categoryCheck = categories.COMMAND
    elseif scenarioInfo.Options.Victory == 'domination' then
        # You're dead if all structures and engineers are destroyed
        categoryCheck = categories.STRUCTURE + categories.ENGINEER
    elseif scenarioInfo.Options.Victory == 'eradication' then
        # You're dead if you have no units
        categoryCheck = categories.ALLUNITS
    else
        # no victory condition
        return
    end

    while true do

        # Look for newly defeated brains and tell them they're dead
        for index,brain in ArmyBrains do
            if not brain:IsDefeated() and not ArmyIsCivilian(brain:GetArmyIndex()) then
                if CheckBrainForDefeat(brain,categoryCheck) then
                    brain:OnDefeat()
                end
            end
        end

        # See if there's a victor
        local winners = {}
        for index,brain in ArmyBrains do
            if not ArmyIsCivilian(brain:GetArmyIndex()) and not brain:IsDefeated() then
                if ( table.getn(winners) > 0
                     and not IsAlly(winners[1]:GetArmyIndex(),brain:GetArmyIndex())) then

                    # more than 1 enemy alive, no winner
                    winners = {}
                    break
                else
                    table.insert(winners,brain)
                end
            end
        end

        local winnerCount = table.getn(winners)
        if winnerCount > 0 then

            local gameOver = true

            if winnerCount > 1 then
                # We have more than one potential winners. If they all have
                # allied victory set, they all win. Otherwise we play on.
                for k,v in winners do
                    if not v.RequestingAlliedVictory then
                        gameOver = false
                        break
                    end
                end
            end

            if gameOver then
                WaitSeconds(20.0) # Give time to watch commander death
                for k,winner in winners do                    
                    # double check to make sure the winner is still alive and not defeated
                    # this is to prevent a cheap shot to kill opponent that will result in own death
                    if CheckBrainForDefeat(winner,categoryCheck) then
                        winner:OnDefeat()
                    else
                        winner:OnVictory()
                    end
                end
                EndGame(scenarioInfo)
                return # exit loop
            end
        end

        WaitSeconds(3.0)
    end
end

EndGame = function(scenarioInfo)
    LOG('Game Over')
end
