#===================================================================================
# This module (scenarioEnvironment.lua) is used during sim init to load the
# save file and script file. It is accessible as the global "ScenarioInfo.Env",
# or just by importing it.
#
# The save and script files are loaded directly into this module's environment,
# so by defining functions like "OnPopulate" and "OnStart" they can override
# the default behavior.
#
# Typically the save file defines a single table, 'Scenario', containing all of
# the save data; the script file uses that table to actually populate the world
# with stuff.
#===================================================================================

function OnPopulate(scen)
    import('/lua/ScenarioUtilities.lua').InitializeArmies()
end

function OnStart(scen)
end

function OnCtrlF5()
    ArmyBrains[1]:OnCreateAI()
    import('/lua/modules/utilities.lua').UserConRequest( "SetFocusArmy -1" )
end

function OnCtrlF4()
    if not ScenarioInfo.DEBUG then
        ScenarioInfo.DEBUG = {}
        if not ScenarioInfo.DEBUG.MarkersTable then
            ScenarioInfo.DEBUG.MarkersTable = { 'Defensive Point', 'Expansion Area', 'Combat Zone', 'Protected Experimental Construction', 'Rally Point', 'ALL OFF' }
            ScenarioInfo.DEBUG.MarkerDebugUnits = {}
        end
        ScenarioInfo.DEBUG.CurrMarkerType = table.getn( ScenarioInfo.DEBUG.MarkersTable )
    end
    ScenarioInfo.DEBUG.CurrMarkerType = ScenarioInfo.DEBUG.CurrMarkerType + 1
    if ScenarioInfo.DEBUG.CurrMarkerType > table.getn( ScenarioInfo.DEBUG.MarkersTable ) then
        ScenarioInfo.DEBUG.CurrMarkerType = 1
    end
    if ScenarioInfo.DEBUG.MarkerDebugUnits then
        for k,v in ScenarioInfo.DEBUG.MarkerDebugUnits do
            v:Destroy()
        end
    end
    print( 'DEBUG MARKERS - ' .. ScenarioInfo.DEBUG.MarkersTable[ScenarioInfo.DEBUG.CurrMarkerType] )
    if ScenarioInfo.DEBUG.CurrMarkerType ~= table.getn( ScenarioInfo.DEBUG.MarkersTable ) then
        local points = import('/lua/modules/ai/aiutilities.lua').AIGetMarkerLocations(ArmyBrains[1], ScenarioInfo.DEBUG.MarkersTable[ScenarioInfo.DEBUG.CurrMarkerType])
        for k,v in points do
            local unit = CreateUnitHPR(
                'uab5102',
                'ARMY_1',
                v.Position[1], v.Position[2], v.Position[3], 
                0, 0, 0
            )
            table.insert( ScenarioInfo.DEBUG.MarkerDebugUnits, unit )
        end
    end
end
