-- This file defines skin directories as well as certain skin settings like fonts and colors
-- Each skin is a table entry in the skins table
-- it contains the following fields:
--  default: The skin to look at if we can't find a needed component in the current skin. Note that this cascades,
--           so if the component isn't in the default skin, it will check the defaults default, etc, until there are no more skins to check
--  texturesPath: where the textures (and currently layout files) are found in this skin
--  buttonFont: default font used for button faces
--  bodyFont: font used for all other text
--  fixedFont: font used for fixed width characters
--  fontColor: common font color
--  highlightColor: text highlight color
--  disabledColor: text disabled color
--  panelColor: default color when drawing a panel


skins = {
    uef = {
        default = "default",
        texturesPath = "/textures/ui/uef",
        buttonFont = "Zeroes Three",
        bodyFont = "Arial",
        fixedFont = "Arial",
        titleFont = "Zeroes Three",
        fontColor = "FFbadbdb",
        bodyColor = "FF6FAFAF",
        dialogCaptionColor = "FFbadbdb",
        dialogColumnColor = "FF6FAFAF",
        dialogButtonColor = "FF69AB4D",
        dialogButtonFont = "Zeroes Two",
        highlightColor = "FF7FA8A4",
        disabledColor = "FF93B0B9",
        layouts = {
            "bottom",
            "left",
            "right"
        },
    },

    #cybran = {
    #    default = "uef",    -- change this to default when we get all the cybran art
    #    texturesPath = "/textures/ui/cybran",
    #    buttonFont = "Wintermute",
    #    bodyFont = "Arial",
    #    fixedFont = "Arial",
    #    titleFont = "Wintermute",
    #    fontColor = "FFFF9900",
    #    bodyColor = "FF6FAFAF",
    #    dialogCaptionColor = "FFbadbdb",
    #    dialogColumnColor = "FF6FAFAF",
    #    dialogButtonColor = "FF69AB4D",
    #    dialogButtonFont = "Gunship",
    #    highlightColor = "FFbEFF4F",
    #    disabledColor = "FF93B0B9",
    #},

    #aeon = {
    #    default = "uef",    -- change this to default when we get all the cybran art
    #    texturesPath = "/textures/ui/aeon",
    #    buttonFont = "Butterbelly",
    #    bodyFont = "Arial",
    #    fixedFont = "Arial",
    #    titleFont = "Butterbelly",
    #    fontColor = "FF6ED660",
    #    bodyColor = "FF6FAFAF",
    #    dialogCaptionColor = "FFbadbdb",
    #    dialogColumnColor = "FF6FAFAF",
    #    dialogButtonColor = "FF69AB4D",
    #    dialogButtonFont = "Gunship",
    #    highlightColor = "FFbEFF4F",
    #    disabledColor = "FF93B0B9",
    #},

    -- warning, do not change the name of this skin descriptor or things will break!
    default = {
        -- note, no default value here, so the "default chain" will get broken at this point, and you'll get file not found
        texturesPath = "/textures/ui/common",
        buttonFont = "Bank Gothic Medium BT",
        bodyFont = "Arial",
        fixedFont = "Andale Mono",
        titleFont = "Bank Gothic Medium BT",
        fontColor = "FF6ED660",
        fontOverColor = "FFFFFFFF",
        fontDownColor = "FF1a3c28",
        bodyColor = "FF6FAFAF",
        dialogCaptionColor = "FFbadbdb",
        dialogColumnColor = "FF6FAFAF",
        dialogButtonColor = "FF69AB4D",
        dialogButtonFont = "Gunship",
        highlightColor = "FFbEFF4F",
        disabledColor = "FF93B0B9",
        panelColor = "FF4A5D6B",
        transparentPanelColor = "AA4A5D6B",
        consoleBGColor = "BB008080",
        consoleFGColor = "white",
        consoleTextBGColor = "80000000",
        layouts = {
            "bottom",
        },
        -- cursor format is: texture name, hotspotx, hotspoty, [optional] num frames, [optional] fps
        cursors = {
            RULEUCC_Attack = {'/textures/ui/common/game/cursors/attack.dds', 15, 15, 8, 12},
            RULEUCC_CallTransport = {'/textures/ui/common/game/cursors/load.dds', 15, 19},
            RULEUCC_Capture = {'/textures/ui/common/game/cursors/capture.dds', 15, 15},
            RULEUCC_Ferry = {'/textures/ui/common/game/cursors/ferry.dds', 15, 15},
            RULEUCC_Guard = {'/textures/ui/common/game/cursors/guard.dds', 15, 15},
            RULEUCC_Move = {'/textures/ui/common/game/cursors/move.dds', 15, 15},
            RULEUCC_Nuke = {'/textures/ui/common/game/cursors/launch.dds', 15, 15},
            RULEUCC_Tactical = {'/textures/ui/common/game/cursors/launch.dds', 15, 15},
            RULEUCC_Overcharge = {'/textures/ui/common/game/cursors/attack.dds', 15, 15},
            RULEUCC_Patrol = {'/textures/ui/common/game/cursors/patrol.dds', 15, 15},
            RULEUCC_Reclaim = {'/textures/ui/common/game/cursors/reclaim.dds', 15, 15},
            RULEUCC_Repair = {'/textures/ui/common/game/cursors/repair.dds', 15, 15},
            RULEUCC_Sacrifice = {'/textures/ui/common/game/cursors/sacrifice.dds', 15, 15},
            RULEUCC_Transport = {'/textures/ui/common/game/cursors/unload.dds', 15, 3},
            RULEUCC_Teleport = {'/textures/ui/common/game/cursors/transport.dds', 15, 15},
            COORDINATED_ATTACK = {'/textures/ui/common/game/cursors/attack_coordinated.dds', 15, 15},
            BUILD = {'/textures/ui/common/game/cursors/selectable.dds', 2, 2},
            HOVERCOMMAND = {'/textures/ui/common/game/cursors/waypoint-hover.dds', 7, 7},
            DRAGCOMMAND = {'/textures/ui/common/game/cursors/waypoint-drag.dds', 7, 7},
            MOVE2PATROLCOMMAND = {'/textures/ui/common/game/cursors/patrol.dds', 15, 15},
            DEFAULT = {'/textures/ui/common/game/cursors/selectable.dds', 2, 2},
        },
    },
}