# Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#
# Configuration file to globally control how Lua behaves


# Make sure __scenario_filename is set to something (the engine sets it but
# scmviewer and editor don't).
__scenario_filename = __scenario_filename or ''

#====================================================================================
# Disable the LuaPlus bit where you can add attributes to nil, numbers, and strings.
#------------------------------------------------------------------------------------
local function metacleanup(obj)
    local name = type(obj)
    mmt = {
        __newindex = function(table,key,value)
            error(string.format("Attempt to set attribute '%s' on %s", tostring(key), name), 2)
        end,
        #__index = function(table,key)
        #    error(string.format("Attempt to get attribute '%s' on %s", tostring(key), name), 2)
        #end
    }
    setmetatable(getmetatable(obj), mmt)
end

metacleanup(nil)
metacleanup(0)
metacleanup('')


#====================================================================================
# Set up a metatable for coroutines (a.k.a. threads)
#------------------------------------------------------------------------------------
local thread_mt = {Destroy = KillThread}
thread_mt.__index = thread_mt
function thread_mt.__newindex(table,key,value)
    error('Attempt to set an attribute on a thread',2)
end
local function dummy() end
setmetatable(getmetatable(coroutine.create(dummy)),thread_mt)


#====================================================================================
# If sim random exists use it, otherwise just use math.random
#------------------------------------------------------------------------------------
if Random then
    math.random = Random
else
    math.randomseed(os.time())
end


#====================================================================================
# Give globals an __index() with an error function. This causes an error message
# when a nonexistent global is accessed, instead of just quietly returning nil.
#------------------------------------------------------------------------------------
local globalsmeta = {
    __index = function(table, key)
        error("access to nonexistent global variable "..repr(key),2)
    end
}
setmetatable(_G, globalsmeta)


#====================================================================================
# Check if an item is callable, ie not a variable. Returns the callable item,
# otherwise returns nil
#------------------------------------------------------------------------------------
function iscallable(f)
    local tt = type(f)
    if tt == 'function' or tt == 'cfunction' then
        return f
    end
    if tt == 'table' and getmetatable(f).__call then
        return f
    end
end
